//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "common.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif



//  Constants
//
INT nNUM_CUSTOM_COLORS = 16;
CONST CHAR szCUSTOM_COLORS[] = "CustomColors";


//  Global variables
//
#if _DEBUG_MODE
//INT _nTraceLevel = 1;		// defined in COMMONXX.CXX
CString sFilter1;
CString sFilter2;
CString sFilter3;
CString sFilter4;
CString sFilter5;
#endif


//// move this into a class derived from CWinApp
//BOOL bPrintFile = FALSE;
CString printFile;
//Cmap<PCSZ, INT> mapOption;

#if TRUE
CStringArray asNull;
CString sNull;
CWinApp* pWinApp = NULL;
#endif

//  Local functions
//
#if !defined(_M_I86LM) && !defined(_WIN32)
VOID ParseCmdLineAux (CONST CString FAR& line);
#endif
VOID ParseCmdLineAux (CONST CStringx& line);



//-----------------------------------------------------------------------------
//  CStringx Implementation
//-----------------------------------------------------------------------------

CStringx::CStringx (VOID) :
    CString()
{
}

CStringx::CStringx (const CString& stringSrc) :
    CString(stringSrc)
{
}

CStringx::CStringx (char ch, int nRepeat) :
    CString(ch, nRepeat)
{
}

CStringx::CStringx (const char* psz) :
    CString(psz)
{
}

CStringx::CStringx (const char* pch, int nLength) :
    CString(pch, nLength)
{
}

//  Since assignment operators are not inherited, the ones from CString were
//  cloned. Thus make sure these stay in sync with the originals.
//
const CStringx& CStringx::operator =(const CStringx& stringSrc)
{
    return ((CStringx&)CString::operator=(*(CString*)&stringSrc));
//  AssignCopy(stringSrc.m_nDataLength, stringSrc.m_pchData);
//  return *this;
}

const CStringx& CStringx::operator =(const CString& stringSrc)
{
    return ((CStringx&)CString::operator=(stringSrc));
//  AssignCopy(stringSrc.m_nDataLength, stringSrc.m_pchData);
//  return *this;
}

const CStringx& CStringx::operator =(const char* psz)
{
    return ((CStringx&)CString::operator=(psz));
//  AssignCopy(SafeStrlen(psz), psz);
//  return *this;
}

const CStringx& CStringx::operator =(char ch)
{
    return ((CStringx&)CString::operator=(ch));
//  ASSERT(!_AfxIsDBCSLeadByte(ch));    // can't set single lead byte
//  AssignCopy(1, &ch);
//  return *this;
}

//  FirstNonblank - returns position of first nonblank or length of string
//
INT CStringx::FirstNonblank (VOID)
{
    for (INT nFirst = 0; nFirst < GetLength(); nFirst++)  {
        if (!IsSpace(GetAt(nFirst))) {
            break;
            }
        }

    return (nFirst);
}

//  LastNonblank - returns position of last nonblank or -1
//
INT CStringx::LastNonblank (VOID)
{
    for (INT nLast = GetLength() - 1; nLast >= 0; nLast--)  {
        if (!IsSpace(GetAt(nLast))) {
            break;
            }
        }

    return (nLast);
}

//  TrimLeading - remove trailing spaces
//
CStringx CStringx::TrimLeading (VOID)
{
    INT nNewLen = GetLength() - FirstNonblank();
    return (Right(nNewLen));
}

//  TrimTrailing - remove trailing spaces
//
CStringx CStringx::TrimTrailing (VOID)
{
    INT nNewLen = LastNonblank() + 1;
    return (Left(nNewLen));
}

//  Trim - remove leading and trailing spaces
//
CStringx CStringx::Trim (VOID)
{
    INT nStart = FirstNonblank();
    INT nEnd = LastNonblank();
    INT nNewLen = (nStart > nEnd) ? 0 : nEnd - nStart + 1;

    return (Mid(nStart, nNewLen));
}


CStringx CStringx::RemoveComment (CHAR commentChar)
{
    INT nLen = GetLength();
    INT nPos = Find(commentChar);
    
    if (nPos != -1) {
        nLen = nPos;
        }

    return (Left(nLen));
}

INT CStringx::FindAt (INT nPos, CHAR ch)
{
    INT nLoc = -1;
    for (; nPos < GetLength(); nPos++) {
        if (GetAt(nPos) == ch) {
            nLoc = nPos;
            break;
            }
        }
    
    return (nLoc);
}


INT CStringx::FindAt (INT nPos, CONST CStringx& chars)
{
    INT nLoc = -1;
    for (; nPos < GetLength(); nPos++) {
//// use FindOneOf???
        if (chars.Find(GetAt(nPos)) != -1) {
            nLoc = nPos;
            break;
            }
        }
    
    return (nLoc);
}

INT CStringx::SkipDelims (INT nPos, CONST CStringx& delims)
{
    INT nNewPos = nPos;
    while ((nNewPos < GetLength()) && (delims.Find(GetAt(nNewPos)) != -1)) {
        nNewPos++;
        }

    DEBUG_OUT(TL_GARRULOUS, ("%d <= CStringx::SkipDelims(%d, '%s') on '%s'\n", 
                nNewPos, nPos, F1(delims), F2(*this)));
    return (nNewPos);
}


//-----------------------------------------------------------------------------
//  Miscellaneous functions
//-----------------------------------------------------------------------------


INT StringToInt (CONST CString& text, INT& nPos)
{
    ASSERT(nPos <= text.GetLength());
    SkipSpaces(text, nPos);
    PCSTR pText = text;
    PSZ pPos = (PSZ)pText + nPos;
    LONG lNum = strtol(pPos, &pPos, 10);
    nPos = pPos - pText;

    return (ToInt(lNum));
}


// ReportAssertion: obsolete assertion handling function
//
VOID ReportAssertion (PCSZ pszExpr, PCSZ pszFile, INT nLine)
{
    CHAR szMessage[512];
    sprintf(szMessage, "Assertion failed: %s (%s: %d)", pszExpr, pszFile, nLine);
    DisplayError(szMessage);
}


// DisplayError: obsolete error message function
//
BOOL DisplayError (PCSZ pszMessage)
{
    ASSERT(pszMessage);
#if !defined(QUICK_WIN)
    AfxMessageBox(pszMessage);
#else
    fprintf(stderr, "%s", pszMessage);
#endif
    return (TRUE);
}


// ParseCmdLineAux: helper function for ParseCmdLine that does all the work
//
// TODO: return the part not processed
//
VOID ParseCmdLineAux (CONST CStringx& line)
{
	DEBUG_OUT(TL_VERBOSE, ("ParseCmdLineAux(line='%s')\n", F(line)));
    INT i = 0;  
    while (i < line.GetLength()) {
        SkipSpaces(line, i);
        if (i == line.GetLength()) {
            break;
            }
        if (IsDelim(line[i++])) {
            CHAR cOption = line[i++];
            switch (ToUpper(cOption)) {
                case 'P': {
//// change " \t" references to whitespace constant
                    CString file = ExtractToken(line, i, " \t");
                    SetPrintFile(file);
                    if (GetPrintFile() == "") {
                        DisplayError("Filename required with print command option (/P)");
                        }
                    break;
                    }
#if _DEBUG_MODE
                case 'T': {
                    _nTraceLevel = StringToInt(line, i);
                    break;
                    }
#endif
                default:
                    break;
                }
            }
        }
}

#if !defined(_M_I86LM) && !defined(_WIN32)
VOID ParseCmdLineAux (CONST CString FAR& line)
{
	DEBUG_OUT(TL_VERBOSE, ("ParseCmdLineAux((FAR)line='%s')\n", F(line)));
    CString sLine(line);
    ParseCmdLineAux(sLine);
}
#endif


VOID ParseCmdLine (LPCSTR lpArgs)
{
	DEBUG_OUT(TL_VERBOSE, ("ParseCmdLine('%s')\n", F(lpArgs)));
    CString sArgs(lpArgs);
    ParseCmdLineAux (sArgs);
}

VOID ParseCmdLine (CONST CString& line)
{
	DEBUG_OUT(TL_VERBOSE, ("ParseCmdLine(line='%s')\n", F(line)));
    ParseCmdLineAux (line);
}

VOID ParseCmdLine (PSZ apszArgv[], INT nArgc)
{
	DEBUG_OUT(TL_VERBOSE, ("ParseCmdLine(%lx, %d)\n", apszArgv, nArgc));
    CString line = "";
    for (INT i = 1; i < nArgc; i++) {
        line += apszArgv[i];
        }
        
    DEBUG_OUT(TL_DETAILED, ("ParseCmdLine: [%s] {%s} len=%d\n", F1(apszArgv[0]), F2(line), nArgc));
    ParseCmdLineAux(line);
}


// TokenizeText: split the text into tokens based on the character delimiters in delims
// The resulting tokens are added to the tokens string array
//
VOID TokenizeText (CStringArray& tokens, CONST CStringx& txt, CONST CStringx& delims)
{
    DEBUG_OUT(TL_GARRULOUS, ("TokenizeText (_, '%s', '%s')\n", F1(txt), F2(delims)));
//// remove CONST from reference parameter for txt?
    CStringx& text = (CStringx&)txt;

    for (INT nPos = 0; nPos < text.GetLength(); ) {
        CString token = ExtractToken(text, nPos, delims);
        if (token == "") {
			if (nPos < text.GetLength()) {
				DEBUG_OUTF(TL_DETAILED, ("TokenizeText: remainder not tokenized (pos %d of %s)\n",
            		   nPos, F(txt)));
				}
            break;
            }
        tokens.Add(token);
        }
}


CString JoinTokens (CONST CStringArray& tokens, CHAR cSeparator /* = cCOMMA */)
{
    CString sList;
    for (INT i = 0; i < tokens.GetSize(); i++) {
        if (i > 0) {
            sList += cSeparator;
            }
        sList += tokens[i];
        }

    return (sList);
}

CString FilterControlChars (PCSZ pszText)
{
    CString sFilteredText;

    ASSERT(pszText);
    while (*pszText) {
        if (IsPrintable(*pszText)) {
            sFilteredText += *pszText;
            }
        else {
            CHAR szHex[10];
            PrintToBuffer(szHex, "(0x%0.2x)", *pszText);
            sFilteredText += szHex;
            }
        pszText++;
        }
    
    return (sFilteredText);
}


//  Extract the next token from the string starting at the given position.
//  If a quote character is encountered, then a multi-word token is returned,
//  provided that the closing quote is also found. Therefore the quote
//  cannot be used as one of the delimiters (a function parameter).
//

CString ExtractToken (CONST CStringx& txt, INT& nPos, CONST CString& delims)
{
//// remove CONST from reference parameter for txt?
    INT nStartPos = nPos;
    ASSERT(delims.Find("\"") == -1);
    CStringx& text = (CStringx&)txt;
    nPos = text.SkipDelims(nStartPos, delims);
    INT nEnd = -1;
    CStringx token = "";
//// this routine needs major surgery
    if (nPos < text.GetLength()) {
        if (text[nPos] == '\"') {
            nPos++;
            nEnd = text.FindAt(nPos, '\"');
            if (nEnd == -1) {
                nEnd = nPos;
                }
            INT nLen = nEnd - nPos;
            token = text.Mid(nPos, nLen);
            nPos = nEnd + 1;
            }
        else {
            nEnd = text.FindAt(nPos + 1, delims);
            if (nEnd == -1) {
                nEnd = 1 + text.GetLength();
                }
            INT nLen = nEnd - nPos;
            token = text.Mid(nPos, nLen);
            nPos = nEnd;
            }
        }
    DEBUG_OUT(TL_GARRULOUS, ("'%s' <= ExtractToken('%s', %d, '%s')\n", 
              F1(token), F2(text), nStartPos, F3(delims)));
    
    return (token);
}

//// reconcile w/ ExtractToken
CString ExtractHead (CStringx& phrase)
{
    INT nEndPos = phrase.FindAt(1, " -");
    INT nLen = (nEndPos == -1) ? phrase.GetLength() : nEndPos;
    ASSERT((nLen > 0) && (nLen <= phrase.GetLength()));
        
    return (phrase.Left(nLen));
}


PSZ AccessString (CString* pstr, INT nLen)
{
    PSZ pszStr = NULL;
    
    if (pstr) {
        pszStr = pstr->GetBuffer(nLen);
        }
    
    return (pszStr);
}

VOID ReleaseString (CString* pstr, INT nLen /* = -1*/)
{
    if (pstr) {
        pstr->ReleaseBuffer(nLen);
        }
}

VOID SplitPath (CONST CString& filename, CString* pdrive,
                CString* pdir, CString* pfname, CString* pext)
{
	DEBUG_OUT(TL_VERBOSE, ("SplitPath(%s, %x, %x, %x, %x)\n", 
			  F1(filename), pdrive, pdir, pfname, pext));
    PSZ pszDrive = AccessString(pdrive, _MAX_DRIVE);
    PSZ pszDir = AccessString(pdir, _MAX_DIR);
    PSZ pszFname = AccessString(pfname, _MAX_FNAME);
    PSZ pszExt = AccessString(pext, _MAX_EXT);

    _splitpath(filename, pszDrive, pszDir, pszFname, pszExt);

    ReleaseString(pdrive);
    ReleaseString(pdir);
    ReleaseString(pfname);
    ReleaseString(pext);
}


CString MakePath (PCSZ pszDrive, PCSZ pszDir, PCSZ pszFname, PCSZ pszExt)
{
	DEBUG_OUT(TL_VERBOSE, ("MakePath(%s, %s, %s, %s)\n", 
			  F1(pszDrive), F2(pszDir), F3(pszFname), F4(pszExt)));
	CString sNewPath;
	PSZ pszNewPath = sNewPath.GetBuffer(MAX_PATH);

    _makepath(pszNewPath, pszDrive, pszDir, pszFname, pszExt);
	sNewPath.ReleaseBuffer();
	DEBUG_OUT(TL_VERBOSE, ("MakePath => %s\n", F1(sNewPath)));

	return (sNewPath);
}


CString ExtractFileBase (CONST CString& filename)
{
	DEBUG_OUT(TL_VERBOSE, ("ExtractFileBase(%s)\n", 
			  F1(filename)));
    CString fname;
    SplitPath(filename, NULL, NULL, &fname, NULL);
        
    return (fname);
}


CString ReplaceExtension (PCSZ pszFileName, PCSZ pszExt)
{
	DEBUG_OUT(TL_VERBOSE, ("ReplaceExtension(%s, %s)\n", 
			  F1(pszFileName), F2(pszExt)));
    CString sDir;
    CString sDrive;
    CString sName;
    SplitPath(pszFileName, &sDrive, &sDir, &sName, NULL);
    CString sNewFile = MakePath(sDrive, sDir, sName, pszExt);
        
    return (sNewFile);
}


CString Blanks (INT nLen)
{
    CONST INT nMAX_BLANKS = 256;
    STATIC CString blanks (' ', nMAX_BLANKS);
    ASSERT(nLen <= nMAX_BLANKS);
    return (blanks.Left(nLen));
}

#if !defined(QUICK_WIN)
// TODO: replace w/ AfxGetMainWnd()
HWND GetMainHwnd (VOID)
{
    HWND hMainWnd = NULL;

    CWnd* pMainWnd = AfxGetApp()->m_pMainWnd;
    if (pMainWnd) {
        hMainWnd = pMainWnd->m_hWnd;
        ASSERT(::IsWindow(hMainWnd));
        }
        
    return (hMainWnd);
}
#endif


#if !defined(QUICK_WIN)
CString GetTempPath (VOID)
{
    LPSTR lpszTempPath = NULL;
    CONST CHAR szTEMP[] = "TEMP=";

#if defined(_WIN32)
    LPSTR lpszEnv = ::GetEnvironmentStrings();
#else
    LPSTR lpszEnv = ::GetDOSEnvironment();
#endif
    while (*lpszEnv != '\0') {
        lpszEnv += (1 + lstrlen(lpszEnv));
        if (!StrNCmpI(szTEMP, lpszEnv, STR_LEN(szTEMP))) {
            lpszTempPath = lpszEnv + STR_LEN(szTEMP);
            break;
            }
        }
    
    return (lpszTempPath);
}

#if FALSE
VOID Pause (DWORD dwMilliSecs)
{
    ::WaitMessage();
    DWORD dwStart = ::GetTickCount();
    TRACE("In Pause: %ld\n", dwStart);
    do {
        if (IsAltPressed()) {
            ::WaitMessage();
            }
        } while ((::GetTickCount() - dwStart) < dwMilliSecs);
    TRACE("Out of Pause: %ld (%ld)\n", GetTickCount(), GetTickCount() - dwStart);
}
#endif

#endif	// !defined(QUICK_WIN)

#if FALSE
LONG AvailMem1 (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("AvailMem1\n"));
//  CONST INT nBLOCK_SIZE = 4096;
    CONST INT nBLOCK_SIZE = SHRT_MAX;       // CByteArray restriction
    CONST INT nMAX_BLOCKS = 16000;          // CObArray restriction
    CObArray blocks;
    
    for (INT i = 0; i < nMAX_BLOCKS; i++) {
        if ((i % 10) == 0) {
            DEBUG_OUT(TL_VERBOSE, ("Trying to allocate block %d\n", i));
            }
        else {
            DEBUG_OUT(TL_GARRULOUS, ("Trying to allocate block %d\n", i));
            }
        CByteArray* pblock = NULL;
        TRY {
            pblock = new CByteArray();
            pblock->SetSize(nBLOCK_SIZE);
            }
        CATCH (CMemoryException, pExc) {
            if (pblock) {
                delete pblock;
                pblock = NULL;
                }
            }
        END_CATCH
        if (!pblock) {
            break;
            }
        blocks.Add(pblock);
        }
        
    for (i = 0; i < blocks.GetSize(); i++) {
        delete (CByteArray*)blocks[i];
        }

    CONST LONG lBYTES_PER_BLOCK = (LONG)nBLOCK_SIZE + 1 + sizeof(CByteArray) + sizeof(CByteArray*);
    LONG lFree = lBYTES_PER_BLOCK * blocks.GetSize() + sizeof(CObArray);

    return (lFree);
}

LONG AvailMem2 (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("AvailMem2\n"));
    CONST WORD wSEGMENT_SIZE = WORD_MAX;
    CONST WORD wBLOCK_SIZE = wSEGMENT_SIZE/2;
//  CONST WORD wMAX_BLOCKS = wSEGMENT_SIZE/sizeof(PSZ);
    CONST WORD wMAX_BLOCKS = wBLOCK_SIZE/sizeof(PSZ);

    WORD wNumBlocks = 0;
    PSZ* pBlocks = new PSZ [wMAX_BLOCKS];
    
    for (WORD i = 0; i < wMAX_BLOCKS; i++) {
        if ((i % 10) == 0) {
            DEBUG_OUT(TL_VERBOSE, ("Trying to allocate block %d\n", i));
            }
        else {
            DEBUG_OUT(TL_GARRULOUS, ("Trying to allocate block %u\n", i));
            }
        TRY {
            pBlocks[i] = new CHAR [wBLOCK_SIZE];
            }
        CATCH (CMemoryException, pExc) {
            pBlocks[i] = NULL;
            }
        END_CATCH
        if (!pBlocks[i]) {
            wNumBlocks = i;
            break;
            }
        }
        
    for (i = 0; i < wNumBlocks; i++) {
        delete pBlocks[i];
        }
    delete pBlocks;

    LONG lFree = ((LONG)wNumBlocks * wBLOCK_SIZE) + wSEGMENT_SIZE;

    return (lFree);
}

LONG AvailMem (VOID)
{
#if !defined(QUICK_WIN)
    return (IsControlPressed() ? AvailMem1() : AvailMem2());
#else
    return (AvailMem2());
#endif
}
#endif

CString FormatDate (CTime& date, BOOL bEuro /* = FALSE*/)
{
    STATIC PSZ aszMonths[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                              "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"};
    CHAR szText[20];
    if (date.GetLocalTm(NULL) == NULL) {
        if (bEuro) {
            strcpy(szText, "1 Jan 1900");
            }
        else {
            strcpy(szText, "1/1/1900");
            }
        }
    else {
        if (bEuro) {
            sprintf(szText, "%d %s %d",
                    date.GetDay(), aszMonths[date.GetMonth() - 1], date.GetYear());
            }
        else {
            sprintf(szText, "%d/%d/%d",
                    date.GetMonth(), date.GetDay(), date.GetYear());
            }
        }

    return (szText);
}

CString FormatDate (BOOL bEuro /* = FALSE*/)
{
    CTime today = CTime::GetCurrentTime();
    return (FormatDate(today, bEuro));
}

CString FormatTime (CTime& date)
{
    CHAR szText[20];
    if (date.GetLocalTm(NULL) == NULL) {
        strcpy(szText, "00:00:00");
        }
    else {
        sprintf(szText, "%02d:%02d:%02d",
                date.GetHour(), date.GetMinute(), date.GetSecond());
        }

    return (szText);
}

CString FormatTime (VOID)
{
    CTime now = CTime::GetCurrentTime();
    return (FormatTime(now));
}

CString FormatDateTime (CTime& date, BOOL bEuro /* = FALSE*/)
{
    CString datetime = FormatDate(date, bEuro) + " " + FormatTime(date);
    return (datetime);
}

CString FormatDateTime (BOOL bEuro /* = FALSE*/)
{
    CTime time = CTime::GetCurrentTime();;
    return (FormatDateTime(time, bEuro));
}


//	make this independent of MFC's measely 512 byte limit

CString FilterText (PCSZ pszInText, INT nMaxLen /* = nMAX_FILTER_ARG */)
{
	CString sText;

	PCSZ pszText = pszInText ? pszInText : szNULL_PTR;
	while (*pszText && (sText.GetLength() < nMaxLen)) {
		if (MyIsPrintable(*pszText)) {
			sText += *pszText;
			}
		else {
			CHAR szHexChar[10];
			wsprintf(szHexChar, ".(%x)", *pszText);
			sText + szHexChar;
			}
		pszText++;
		}
	DEBUG_OUT(TL_GARRULOUS + 1, ("FilterText() => '%.*s'\n", nMaxLen, (PCSZ)sText));

	return (sText);
}


// Registry/INI functions. Use SetRegistryKey to have these go to the registry instead
// of an INI file.
//

VOID WriteAppOption (PCSZ pszOption, INT nValue, PCSZ pszSection /* = szOPTIONS */)
{
	BOOL bWriteOK = MyAfxGetApp()->WriteProfileInt(pszSection, pszOption, nValue);
	ASSERT(bWriteOK);
}


VOID WriteAppOption (PCSZ pszOption, PCSZ pszValue, PCSZ pszSection /* = szOPTIONS */)
{
	BOOL bWriteOK = MyAfxGetApp()->WriteProfileString(pszSection, pszOption, pszValue);
	ASSERT(bWriteOK);
}


INT GetAppOption (PCSZ pszOption, INT nDefault, PCSZ pszSection /* = szOPTIONS */)
{
	// use mapOption
    INT nOption = MyAfxGetApp()->GetProfileInt(pszSection, pszOption, nDefault);
	if (nOption == nDefault) {
		WriteAppOption(pszOption, nDefault, pszSection);
	}
	DEBUG_OUT(TL_VERBOSE, ("GetAppOption('%s', %d) => %d\n", pszOption, nDefault, nOption));

	return (nOption);
}


CString GetAppOption (PCSZ pszOption, PCSZ pszDefault, PCSZ pszSection /* = szOPTIONS */)
{
    CString sOption = MyAfxGetApp()->GetProfileString(pszSection, pszOption, pszDefault);
	if (sOption == pszDefault) {
		WriteAppOption(pszOption, pszDefault, pszSection);
	}
	DEBUG_OUT(TL_VERBOSE, ("GetAppOption('%s', '%s') => '%s'\n", pszOption, pszDefault, F(sOption)));

	return (sOption);
}


VOID WriteAppBinaryOption (PCSZ pszOption, PVOID pValue, DWORD dwSize, PCSZ pszSection /* = szOPTIONS */)
{
	DEBUG_OUT(TL_VERBOSE, ("WriteAppBinaryOption(%s, %x, %lu, %s)\n",
			  F(pszOption), pValue, dwSize, F(pszSection)));

	BOOL bWriteOK = MyAfxGetApp()->WriteProfileBinary(pszSection, pszOption, (PBYTE)pValue, dwSize);
	ASSERT(bWriteOK);
}


BOOL GetAppBinaryOption (PCSZ pszOption, PVOID pValue, DWORD dwSize, PCSZ pszSection /* = szOPTIONS */)
{
	DEBUG_OUT(TL_VERBOSE, ("GetAppBinaryOption(%s, %x, %lu, %s)\n",
			  F(pszOption), pValue, dwSize, F(pszSection)));

	PBYTE pBytes = NULL;
	UINT uActualBytes;
    BOOL bOK = MyAfxGetApp()->GetProfileBinary(pszSection, pszOption, &pBytes, &uActualBytes);
	bOK = bOK && (dwSize == (DWORD)uActualBytes);
	if (bOK) {
		CopyMemory(pValue, pBytes, dwSize);
	}
	if (pBytes) {
		delete pBytes;
	}

	return (bOK);
}


COLORREF GetColorOption (COLORREF colorDefault, CWnd* pWnd /* = NULL*/)
{											   
	DEBUG_OUT(TL_VERBOSE, ("GetColorOption(%08x, 0x%x)\n", colorDefault, pWnd));

	//CColorDialog dialogColor(colorDefault, CC_FULLOPEN, pWnd);
	CColorDialog dialogColor(colorDefault, 0, pWnd);
	COLORREF colorSelected = colorDefault;

	// Get the custom colors from the registry
	CDWordArray adwCustomColors;
	adwCustomColors.SetSize(nNUM_CUSTOM_COLORS);
	PDWORD pData = adwCustomColors.GetData();
	DWORD dwColorsSize = adwCustomColors.GetSize() * sizeof(adwCustomColors[0]);
	BOOL bOK = GetAppBinaryOption(szCUSTOM_COLORS, pData, dwColorsSize);
	if (bOK) {
		dialogColor.m_cc.lpCustColors = pData;
	}

	dialogColor.m_cc.rgbResult = colorDefault;
	if (dialogColor.DoModal() == IDOK) {
		//COLORREF* pCustomColors = dialogColor.GetSavedCustomColors();
		colorSelected = dialogColor.GetColor();

		// Save the custom colors in the registry
		//WriteAppBinaryOption(szCUSTOM_COLORS, (PVOID)pCustomColors, dwColorsSize);
		WriteAppBinaryOption(szCUSTOM_COLORS, (PVOID)pData, dwColorsSize);
	}

	return (colorSelected);
}


BOOL FileExists (PCSZ pszFile)
{
	OFSTRUCT ofs;
	HFILE hFile = OpenFile(pszFile, &ofs, OF_EXIST);
	BOOL bExists = (hFile != HFILE_ERROR);
	DEBUG_OUT(TL_VERBOSE, ("FileExists(%s) => %d\n", pszFile, bExists));

	return (bExists);
}


CString ExtractFileDir (PCSZ pszFileName)
{
	DEBUG_OUT(TL_VERBOSE, ("ExtractFileBase(%s)\n", F1(pszFileName)));
	CString sDrive;
    CString sBareDir;
	CString sDir;
    SplitPath(pszFileName, &sDrive, &sBareDir, NULL, NULL);
	sDir = MakePath(sDrive, sBareDir, NULL, NULL);
	DEBUG_OUT(TL_VERBOSE, ("ExtractFileDir =>\n", F1(sDir)));
        
    return (sDir);
}


CString GetProgramDir (VOID)
{
	CHAR szModuleFileName[MAX_PATH] = { cNULL };
	GetModuleFileName(NULL, szModuleFileName, STR_LEN(szModuleFileName));
	CString sDir = ExtractFileDir(szModuleFileName);
	DEBUG_OUT(TL_VERBOSE, ("GetProgramDir() => %s\n", F1(sDir)));

	return (sDir);
}


VOID AddPathDelim (CString& sFile)
{
	DEBUG_OUT(TL_GARRULOUS, ("AddPathDelim(cs='%s')\n", F1(sFile)));
	PSZ pszFile = sFile.GetBuffer(MAX_PATH);
	AddPathDelim(pszFile);
	sFile.ReleaseBuffer();
}


// ParseOption: parse an optional setting spec (eg, the "+" or "-" in "/P+", "/P-", "/P")
// NOTE: The position must be pointing at the switch indicator and option char.
//
INT ParseOption (CONST CStringx& line, INT& pos, BOOL bDefault /* = TRUE */)
{
	DEBUG_OUT(TL_GARRULOUS, ("ParseOption('%s', %d, %d)\n", 
			  F((PCSZ)line), &pos, bDefault));
	BOOL bOptionSet = bDefault;

	if (pos < line.GetLength()) {
		if (line[pos] == '-') {
			bOptionSet = FALSE;
			pos++;
		}
		else if (line[pos] == '+') {
			bOptionSet = TRUE;
			pos++;
		}
	}
	DEBUG_OUT(TL_GARRULOUS, ("ParseOption => %d\n", bOptionSet));
	
	return (bOptionSet);
}
