//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#if !defined(_CORE_H)
#define _CORE_H


// No support from nmake/vc, so hardcode DEBUG_REL or not
#define _DEBUG_REL

//  This header file is used by both common & commonxx.

// TODO: define _DEBUG if _DEBUGREL defined???
// NOTE: if makes sense to require checks for both, but this can be cumbersome
//		 plue defining it automatically might lead to trouble w/ MFC
#if defined(_DEBUG) || defined(_DEBUG_REL)
#define _DEBUGGING		1
#define _DEBUG_MODE		1
#else
#define _DEBUGGING		0
#define _DEBUG_MODE		0
#endif
//#define _DEBUG_MODE		(defined(_DEBUG) || defined(_DEBUG_REL))

// Include the main windows header
#define WIN32_EXTRA_LEAN
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

// Include Windows Crackers unless this is an MFC app
#if !defined(_NON_STD_AFX_H)
#	include <windowsx.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stddef.h>
#include <ctype.h>
#include <time.h> 
#include <limits.h>
#include <share.h>
#include <direct.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <math.h>

#if defined(WIN32) && !defined(_WIN32)
#define _WIN32
#endif

#if defined(_WIN32) && !defined(WIN32)
#define WIN32
#endif

//  Define the types that "should" have been defined in Win16
//

#if !defined(_WIN32)
#   define CONST    const
#   define STATIC   static
#   define EXTERN   extern
#   define VOID     void
#define INLINE              inline
#define STATIC              static
//#define BYTE unsigned char
    //typedef int     INT;
    //typedef char    CHAR;
    //typedef double  DOUBLE;
    //typedef float   FLOAT;

    typedef int                 INT;
    typedef char                CHAR;
    typedef double              DOUBLE;
    typedef float               FLOAT;
    typedef unsigned long       DWORD;
    typedef unsigned int        UINT;

#   if defined(QUICK_WIN)
        typedef CHAR*           PSZ;
        typedef VOID __far*     LPVOID;
#   endif
    typedef WORD __far*     LPWORD;
    typedef LPSTR __far*    LPLPSTR;
    typedef CHAR*           PCHAR;
    typedef CHAR*           PSZ;
    typedef CONST CHAR*     PCSTR;
    typedef CONST CHAR*     PCSZ;
    typedef BYTE*           PBYT;

#       if !defined(INVALID_HANDLE_VALUE)
#               define INVALID_HANDLE_VALUE 0
#       endif

#else
    typedef double              DOUBLE;
    typedef float               FLOAT;
    typedef WORD *      LPWORD;
    typedef LPSTR *     LPLPSTR;
    typedef CHAR*           PSZ;
    typedef CONST CHAR*     PCSZ;
    typedef BYTE*           PBYT;
#endif

#if !defined(QUICK_WIN)
    typedef WNDPROC*    PWNDPROC;
#endif

#define WORD_MAX    (USHRT_MAX)
#define DWORD_MAX   (ULONG_MAX)


//-----------------------------------------------------------------------------
//  Declarations (with overrides)
//-----------------------------------------------------------------------------

#if !defined(WIN32)
#include <toolhelp.h>
//#define WNDCALLBACK         __export CALLBACK
//#define CHAR                char     
//#define INT                 int
//#define DWORD               unsigned long
//#define UINT                unsigned int
#define APIENTRY            PASCAL
//#define WINAPI                FAR PASCAL
//#define WNDPROC             FARPROC
#define CONST               const
#define WINAPIV             FAR CDECL
//#define MAKEWORD(lo, hi)    (WORD)((lo) | ((hi) << 8))
#define EXPORT              _export
#define BASEDCS             __based(__segname("_CODE"))

//  redefine Windows stupid definitions for PBYTE, PLONG, etc to eradicate NEAR usage
//
//  typesafe redefine of PBYTE
typedef unsigned char       BYT;
typedef BYT *               PBYT;
typedef CONST BYT *         PCBYT;
#define PBYTE               PBYT        //  hides Windows definition of PBYTE
#define PCBYTE              PCBYT
//  typesafe redefine of PLONG
typedef long                LNG;
typedef LNG *               PLNG;
typedef CONST BYT *         PCLNG;
#define PLONG               PLNG        // hides Windows definition of PLONG
#define PCLONG              PCLNG

#else

#define WNDCALLBACK         CALLBACK
#undef EXPORT
#define EXPORT              __declspec(dllexport)
#define BASEDCS             //declspec(based(segname("code")))

#endif   // !defined(WIN32)

#if !defined(EXTERN)
#define EXTERN              extern
#endif

#if !defined(EXTERN_C)
#define EXTERN_C            EXTERN "C"
#endif

#if !defined(DLLEXPORT)
#define DLLEXPORT           EXPORT WINAPI
#endif

#if defined(__cplusplus)
#   define CONSTANT         const
#   define INLINE           inline
#else
#   define INLINE           __inline
#   define CONSTANT         static const
#endif

#if !defined(STATIC)
#define STATIC              static
#endif

#define NUM_ELEMENTS(array) (sizeof(array) / sizeof(array[0]))
#define NUM_CHARS(string)   (NUM_ELEMENTS(string))
#define STR_LEN(string)     (NUM_CHARS(string) - 1)
typedef LPSTR FAR*          LPLPSTR;
typedef CHAR *              PSZ;
typedef PSZ *               PPSZ;
typedef CONST CHAR *        PCSZ;
typedef PCSZ *              PPCSZ;
typedef VOID *              PVOID;
typedef CONST VOID *        PCVOID;
//typedef FILE*             PFILE;
typedef HFILE               PFILE;


//#if defined(__cplusplus)

//  Common constants
//
CONSTANT CHAR cBACK_SLASH =		'\\';
CONSTANT CHAR cCOMMA =			',';
CONSTANT CHAR cDOUBLE_QUOTE =	'\"';
CONSTANT CHAR cSINGLE_QUOTE =	'\'';
CONSTANT CHAR cNULL =			'\0';
CONSTANT CHAR cSPACE =			' ';
CONSTANT CHAR cTAB =			'\t';
CONSTANT CHAR cDOT =			'.';
CONSTANT CHAR cRETURN =			'\r';
CONSTANT CHAR cNEW_LINE =		'\n';
CONSTANT CHAR cDASH =			'-';
CONSTANT CHAR cSEMI_COLON =		';';
CONSTANT CHAR cLEFT_PAREN =		'(';
CONSTANT CHAR cRIGHT_PAREN =	')';
CONSTANT CHAR cLEFT_BRACE =		'{';
CONSTANT CHAR cRIGHT_BRACE =	'}';
CONSTANT CHAR cPERIOD =			'.';

CONSTANT CHAR szNEW_LINE[] =	"\n";
CONSTANT CHAR szNEWLINE[] =		"\n";
CONSTANT CHAR szNULL[] =		"";
CONSTANT CHAR szNULL_STR[] =	"";
CONSTANT CHAR szNULL_PTR[] =	"NULL";
CONSTANT CHAR szTEMP[] =		"TEMP";
CONSTANT CHAR szCRLF[] =		"\r\n";
CONSTANT CHAR szTAB[] =			"\t";
CONSTANT CHAR szLEFT_PAREN[] =	"(";
CONSTANT CHAR szRIGHT_PAREN[] =	")";

CONSTANT INT nINVALID_INDEX =	-1;


//  Helper macros
//

//#define NUM_ELEMENTS(array) (sizeof(array) / sizeof(array[0]))
//#define STR_LEN(string)     (NUM_ELEMENTS(string) - 1)

// NOTE: this dump context is not defined in MFC release build
//		 so _DEBUG_MODE is too general
#if defined(_DEBUG)
CONSTANT CHAR szEQUALS[] =     " = ";
CONSTANT CHAR szTERM[] =       "; ";
#define DUMPLN()        dc << szNEW_LINE;
#define DUMP_VAR(var)   dc << #var << szEQUALS << var << szTERM
#define DUMPLN_VAR(var) DUMP_VAR(var) << szNEW_LINE;
#define DUMP_OBJ(obj)   dc << #obj << szEQUALS << &obj << szTERM
#define DUMPLN_OBJ(obj) DUMP_OBJ(obj) << szNEW_LINE;
#endif

//#endif  // defined(__cplusplus)


#undef ASSERT
#if _DEBUG_MODE

EXTERN_C VOID _ReportAssertion (PCSZ pszExpr, PCSZ pszFile, INT nLine);
#define ASSERT(exp) \
    if (!(exp)) { \
        _ReportAssertion(#exp, __FILE__, __LINE__); \
        }
#else

#define ASSERT(exp) \


#endif

//  Wrapper macros
//

#define PrintToBuffer   sprintf
#define vPrintToBuffer  vsprintf


//  Miscellaneous numeric operations
//

#define NORMALIZE(value, divisor)   (((value) + ((divisor) - 1)) & (0x0FFFFFFF & ~((divisor) - 1)))

#include "core.inl"
#include "gnu_gpl.h"

#endif  // !defined(_CORE_H)
