//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "commonxx.h"

//  Private Constants
//
CONST CHAR szCOMMON[] =     "COMMON";
CONST INT nMAX_PAGES = 8192;
CONST INT nPAGE_SIZE = 4096;
#if _DEBUG_MODE
CONST CHAR szDEBUG[] =          "DEBUG";
CONST CHAR szTRACE_LEVEL[] =    "TraceLevel";
CONST CHAR szTRACE_TO_LOG[] =   "TraceToLog";
CONST CHAR szUSE_STDERR[] =		"UseStderr";
CONST CHAR szSHOW_THREAD_ID[] =	"IncludeThreadID";
CONST CHAR BASEDCS szDEBUG_OUT_FAILED[] =   "Unable to output debugging text";
CONST CHAR BASEDCS szBAD_FORMATTING[] =     "The programmer's an egghead";
#endif


#if defined(_WIN32)
CONST INT nTRACE_BUFFER_MAX = 2048;
CONST INT nTEXT_BUFFER_MAX = 4092;
#else
CONST INT nTRACE_BUFFER_MAX = 256;
CONST INT nTEXT_BUFFER_MAX = 512;
#endif

#define UNNAMED_APP		"Unnamed App"
CONST CHAR szUNNAMED_APP[] = UNNAMED_APP;

//  Global variables
//  
INT _nNumClients = 0;
CHAR _szAppName[MAX_PATH] = UNNAMED_APP;
HWND _hAppWnd = NULL;
BOOL _bAppActive = FALSE;
CLIENT_INFO _aci[nMAX_CLIENTS];
BOOL _bPauseMsgLoop = FALSE;


//  Support for debugging
#if _DEBUG_MODE
BOOL _bDebugging = TRUE;
INT _nTraceLevel = TL_ALWAYS;
PFILE _hTrace = NULL;                       // file for diagnostics
CHAR _szDebugText[nTRACE_BUFFER_MAX];
BOOL bUseStderr = FALSE;					// output to stderr
BOOL _bShowThreadID = FALSE;
#endif

//  Local functions
//
INT GetFreeAppPos (VOID);
INT FindAppByHwnd (HWND hWnd);
INT FindAppByInst (HINSTANCE hInst);
INT FindAppByTask (HTASK hTask);
INT GetCurrentClient (VOID);


#if _DEBUG_MODE

CHAR pszAppIniFile[MAX_PATH] = "mydebug.ini";

BOOL GetDebugOption (PCSZ pszOption, INT nDefault)
{
    INT nOption = GetPrivateProfileInt(szDEBUG, pszOption, nDefault, pszAppIniFile);
	if (nOption == nDefault) {
		CHAR szValue[20];
		wsprintf(szValue, "%d", nDefault);
		BOOL bWriteOK = WritePrivateProfileString(szDEBUG, pszOption, szValue, pszAppIniFile);
		ASSERT(bWriteOK);
	}
	DEBUG_OUT(TL_VERBOSE, ("GetDebugOption('%s', %d) => %d\n", pszOption, nDefault, nOption));

	return (nOption);
}


#else

BOOL GetDebugOption (PCSZ pszOption, INT nDefault)
{
	return (nDefault);
}

#endif

//-----------------------------------------------------------------------------
//  Debugging support
//-----------------------------------------------------------------------------

#if _DEBUG_MODE

BOOL InitDebugging (PCSZ pszIniFile)
{
#if FALSE
    INT _nTraceLevel = GetPrivateProfileInt(szDEBUG, szTRACE_LEVEL, TL_ALWAYS, pszIniFile);
    SetTraceLevel(_nTraceLevel);
    BOOL bTraceToLog = GetPrivateProfileInt(szDEBUG, szTRACE_TO_LOG, FALSE, pszIniFile);
    bUseStderr = GetPrivateProfileInt(szDEBUG, szUSE_STDERR, TRUE, pszIniFile);
#endif
	StringCopy(pszAppIniFile, pszIniFile);
    INT _nTraceLevel = GetDebugOption(szTRACE_LEVEL, TL_ALWAYS);
    SetTraceLevel(_nTraceLevel);
    BOOL bTraceToLog = GetDebugOption(szTRACE_TO_LOG, FALSE);
    bUseStderr = GetDebugOption(szUSE_STDERR, TRUE);
	_bShowThreadID = GetDebugOption(szSHOW_THREAD_ID, TRUE);;

    //  SetTraceToLog(bTraceToLog);

    //  Create a log file for the trace, usually <temp dir>\APPNAME.LOG. If the normal
	//	log file cannot be created, <temp dir>\APPNAME##.LOG will be attempted.
    if (bTraceToLog) {
        CHAR szTempDir[PATH_MAX];
        CHAR szTempFile[PATH_MAX];
		CHAR szFileSuffix[10] = { cNULL };
        GetTempDir(szTempDir, STR_LEN(szTempDir));
		_hTrace = NULL;
#if TRUE
		// If the app name is not yet specified, use the module name
		PCSZ pszAppName = GetAppName();
		if (StringCompare(pszAppName, szUNNAMED_APP) == 0) {
			CHAR szModuleFileBase[MAX_PATH] = { cNULL };
			CHAR szModuleFileName[MAX_PATH] = { cNULL };
			GetModuleFileName(NULL, szModuleFileName, STR_LEN(szModuleFileName));

			_splitpath(szModuleFileName, NULL, NULL, szModuleFileBase, NULL);
			SetAppName(szModuleFileBase);
			}
#endif
		for (INT i = 0; (i < 100) && (_hTrace == NULL); i++) {
        	//wsprintf(szTempFile, "%s\\%s.LOG", szTempDir, GetAppName());
        	wsprintf(szTempFile, "%s%s%s.LOG", szTempDir, GetAppName(), szFileSuffix);
        	_hTrace = OpenTextFile(szTempFile, "w", _hTrace, NULL, FALSE);
			wsprintf(szFileSuffix, "%d", (i + 1));
			}
        ASSERT(_hTrace);
        }
    DEBUG_OUT(TL_VERBOSE, ("%s:InitDebugging(%s)\n", GetAppName(), pszIniFile));
	DEBUG_OUT(TL_VERBOSE, ("TraceLevel = %d\n", _nTraceLevel));
	DEBUG_OUT(TL_VERBOSE, ("TraceToLog = %d\n", bTraceToLog));
	DEBUG_OUT(TL_VERBOSE, ("ShowThreadID = %d\n", _bShowThreadID));

    return (TRUE);
}

BOOL EndDebugging (VOID)
{
    DEBUG_OUTF(TL_VERBOSE, ("%s:EndDebugging()\n", GetAppName()));
    CloseTextFile(_hTrace, NULL);
    return (TRUE);
}

//  Format and output debugging text
//
//  NOTE:   there should be no reliance on the data segment
//
BOOL OutputDebugText (PCSZ pszFormat, ...)
{
    static va_list vaArgs;
    va_start(vaArgs, pszFormat);
    CHAR szOut[nTRACE_BUFFER_MAX];                // must not be static or global
    //INT nLen = wvsprintf(szOut, pszFormat, vaArgs);
	INT nLen = vsprintf(szOut, pszFormat, vaArgs);
    if (nLen > NUM_CHARS(szOut)) {
		INT nPos = GetCurrentClient();
		HWND hWnd = (nPos != nINVALID_INDEX) ? GetClientAppWnd(nPos) : NULL;
        MessageBox(NULL, szDEBUG_OUT_FAILED, szBAD_FORMATTING, MB_OK);
        }
    else {
        OutputDebugString(szOut);
        }
	if (bUseStderr) {
		fputs(szOut, stderr);
		}
    va_end(vaArgs);
        
    return (TRUE);
}

VOID _ReportAssertion (PCSZ pszExpr, PCSZ pszFile, INT nLine)
{
    DEBUG_OUTF(TL_VERBOSE, ("ReportAssertion(%s,%s,%d)\n", pszExpr, pszFile, nLine));
	STATIC BOOL bIgnore = FALSE;

	if (!bIgnore) {
		CHAR szMessage[nTRACE_BUFFER_MAX];
		wsprintf(szMessage, "Assertion failed: %s (%s: %d)", pszExpr, pszFile, nLine);
		INT nResp = HandleError(szCOMMON, szMessage, 0, NULL);
#if FALSE
		if (nResp == IDCANCEL) {
			DEBUG_OUTF(TL_ALWAYS, ("Bye bye ...\n"));
	#if defined(_WIN32)
	        ExitProcess(666);
	#else
		    TerminateApp(NULL, NO_UAE_BOX);
	#endif
			}
#endif

		if (nResp == IDABORT) {
			DEBUG_OUTF(TL_ALWAYS, ("Bye bye ...\n"));
#if defined(_WIN32)
	        ExitProcess(666);
#else
		    TerminateApp(NULL, NO_UAE_BOX);
#endif
			}
		else if (nResp == IDIGNORE) {
			bIgnore = TRUE;
			}
		}
}


VOID DebugOut (PCSZ pszFormat, ...)
{
#if defined(WIN32)
	if (_bShowThreadID) {
		sprintf(_szDebugText, "[%x] ", GetCurrentThreadId());
		OutputDebugText("%s", _szDebugText);
		if (_hTrace) {
			WriteText(_hTrace, _szDebugText);
			}
		}
#endif

    // format the trace text
    va_list vaArgs;
    va_start (vaArgs, pszFormat);
    //INT nLen = wvsprintf(_szDebugText, pszFormat, vaArgs);
	//	TODO: think of any valid reasons not to use C Runtime version,
	//		  which is much more flexible (eg, "%.*s", nLen, pszLongString)
	INT nLen = vsprintf(_szDebugText, pszFormat, vaArgs);
    Assert(nLen < STR_LEN(_szDebugText));
    va_end (vaArgs);    

    // display it
    OutputDebugText("%s", _szDebugText);
    if (_hTrace) {
        WriteText(_hTrace, _szDebugText);
        }

	// Pause for effect (if control down & scroll lock on)
	if (IsControlPressed() && IsKeyToggled(VK_SCROLL)) {
		Pause(100);
		}
}


#if FALSE

VOID _ReportAssertion (PCSZ pszExpr, PCSZ pszFile, INT nLine)
{
    CHAR szMsg[nTRACE_BUFFER_MAX];
    sprintf(szMsg, "\nAssertion failed: %s (%s: %d)\n", pszExpr, pszFile, nLine);
    DebugOut("%s", szMsg);
    fprintf(stderr, "%s", szMsg);
    fprintf(stderr, "%s", "Press c to continue, anything else to quit\n");
    INT c = getchar();
    if (toupper(c) != 'C') {
        exit(1);
        }
}

#endif  // FALSE


VOID _ReportError (PCSZ pszExpr, PCSZ pszFile, INT nLine)
{
    CHAR szMsg[nTRACE_BUFFER_MAX];
    sprintf(szMsg, "\nUnrecoverable internal error; contact TPO (%s; %d)\n", 
            pszExpr, pszFile, nLine);
    DebugOut("%s", szMsg);
    fprintf(stderr, "%s", szMsg);
#if defined(QUICK_WIN)
    _wsetexit(_WINEXITPERSIST);
#endif
    exit(1);
}


#endif  // _DEBUG_MODE

//-----------------------------------------------------------------------------
//  Exported functions
//-----------------------------------------------------------------------------

BOOL InitCommon (HINSTANCE hInst, HWND hWnd, PCSZ pszAppName)
{
    DEBUG_OUTF(TL_ALWAYS, ("InitCommon(%x, %x, %s)\n", hInst, hWnd, pszAppName));
    SetAppName(pszAppName);

    //  Make a bogus client entry for the (server) app to itself. 
    BOOL bOK = RegisterClientApp(hInst, hWnd, pszAppName);
#if !defined(WIN32)
	// Clear the task handle so that searches won't get this instead of the 
	// real active client.
    INT nPos = FindAppByInst(hInst);
    if (nPos != nINVALID_INDEX) {
        SetClientAppTask(nPos, NULL);
        }
#endif

#if _DEBUG_MODE
	CHAR szModuleFileName[MAX_PATH] = { cNULL };
	GetModuleFileName(hInst, szModuleFileName, STR_LEN(szModuleFileName));
	DEBUG_OUT(TL_VERBOSE, ("Module Filename = %s\n", szModuleFileName));
#endif

    return (bOK);
}


// The following functions implement basic client application tracking that is a 
// carry out from 16-bit Windows.
//


BOOL RegisterAppWindow (HINSTANCE hInst, HWND hWnd)
{
    DEBUG_OUTF(TL_VERBOSE, ("RegisterAppWindow(%x, %x)\n", hInst, hWnd));
    ASSERT(hWnd);

    BOOL bOK = (hWnd != NULL);
    INT nPos = FindAppByInst(hInst);
	ASSERT(nPos != nINVALID_INDEX);
    if (nPos != nINVALID_INDEX) {
	    SetClientAppWnd(nPos, hWnd);
		}

    return (bOK);
}


INT GetFreeAppPos (VOID)
{
    INT nPos = nINVALID_INDEX;

    for (INT i = 0; i < nMAX_CLIENTS; i++) {
        if (!IsClientAppActive(i)) {
            nPos = i;
            break;
            } 
        }
    return (nPos);
}

INT FindAppByHwnd (HWND hWnd)
{
    DEBUG_OUTF(TL_VERBOSE, ("FindAppByHwnd(%x)\n", hWnd));

    INT nPos = nINVALID_INDEX;
    for (INT i = 0; i < nMAX_CLIENTS; i++) {
        if (GetClientAppWnd(i) == hWnd) {
            nPos = i;
            break;
            } 
        }
    return (nPos);
}

INT FindAppByInst (HINSTANCE hInst)
{
    DEBUG_OUTF(TL_VERBOSE, ("FindAppByInst(%x)\n", hInst));

    INT nPos = nINVALID_INDEX;
    for (INT i = 0; i < nMAX_CLIENTS; i++) {
        if (GetClientAppInst(i) == hInst) {
            nPos = i;
            break;
            } 
        }
    return (nPos);
}

INT FindAppByTask (HTASK hTask)
{
    DEBUG_OUTF(TL_VERBOSE, ("FindAppByTask(%x)\n", hTask));
    INT nPos = nINVALID_INDEX;
    for (INT i = 0; i < nMAX_CLIENTS; i++) {
        if (GetClientAppTask(i) == hTask) {
            nPos = i;
            break;
            } 
        }
    return (nPos);
}

INT GetCurrentClient (VOID)
{
    DEBUG_OUTF(TL_VERBOSE, ("GetCurrentClient()\n"));
    HTASK hTask = GetCurrentTask();
    INT nPos = FindAppByTask(hTask);
    if (nPos == nINVALID_INDEX) {
        DEBUG_OUTF(TL_ALWAYS, ("***** Unable to find client app for current task (%x) *****\n", hTask));
        }

    return (nPos);
}

BOOL RegisterClientApp (HINSTANCE hInst, HWND hWnd, PCSZ pszName)
{
    DEBUG_OUTF(TL_VERBOSE, ("RegisterClientApp(%x, %x, %s)\n", hInst, hWnd, pszName));
    ASSERT(hInst);
    ASSERT(pszName);

    BOOL bOK = FALSE;
	ASSERT(GetNumClients() < nMAX_CLIENTS);
    if (GetNumClients() < nMAX_CLIENTS) {
        INT nNumClients = 1 + GetNumClients();
        SetNumClients(nNumClients);
        INT nPos = GetFreeAppPos();
        bOK = ((hInst != NULL) && (hWnd != NULL));
        SetClientAppInst(nPos, hInst);
        SetClientAppTask(nPos, GetCurrentTask());
        SetClientAppWnd(nPos, hWnd);
        SetClientAppName(nPos, pszName);
        SetClientAppActive(nPos, bOK);
        }

    return (bOK);
}


// Move all inactive entries to the end of the list
//
VOID PruneAppList (VOID)
{
    DEBUG_OUTF(TL_DETAILED, ("PruneAppList()\n"));
	INT nNumClients = GetNumClients();		// num active or inactive
    for (INT i = 0; i < (nNumClients - 1); i++) {
        if (IsClientAppActive(i) == FALSE) {
		    for (INT j = (i + 1); j < nNumClients; j++) {
				if (IsClientAppActive(j) == TRUE) {
					_aci[i] = _aci[j];
					ZeroMemory(&_aci[j], sizeof(_aci[j]));
					nNumClients--;
				}
            } 
        }
	}
	SetNumClients(nNumClients);
}


BOOL UnregisterClientApp (HINSTANCE hInst)
{
    DEBUG_OUTF(TL_VERBOSE, ("UnregisterClientApp(%x)\n", hInst));
    BOOL bOK = FALSE;
    INT nPos = FindAppByInst(hInst);
    if (nPos != nINVALID_INDEX) {
        SetClientAppActive(nPos, FALSE);
        bOK = TRUE;
        }
#if TRUE
	if (GetNumClients() == nMAX_CLIENTS) {
		PruneAppList();
	}
#endif
    return (bOK);
}

BOOL InitAppShutdown (VOID)
{
    DEBUG_OUTF(TL_VERBOSE, ("InitAppShutdown()\n")); 

    BOOL bOK = FALSE;
    //HTASK hTask = GetCurrentTask();
    //INT nPos = FindAppByTask(hTask);
    INT nPos = GetCurrentClient();
    //ASSERT(nPos != nINVALID_INDEX);
    if (nPos != nINVALID_INDEX) {
        return (FALSE);
        }

    if (IsClientAppActive(nPos)) {
        HWND hWnd = GetClientAppWnd(nPos);
        ASSERT(hWnd);
        if (hWnd && IsWindow(hWnd)) {
            SendMessage(hWnd, WM_CLOSE, 0, 0);
            }
        SetClientAppActive(nPos, FALSE);
        bOK = TRUE;
        }

    return (bOK);
}


BOOL ParseString (PCSZ lpszText, PSZ lpszBuffer, INT nBufSize, PPCSZ ppszEnd)
{
    DEBUG_OUTF(TL_VERBOSE, ("ParseString(%s, %08lx, %d, %08lx)\n", 
                            lpszText, lpszBuffer, nBufSize, ppszEnd));
    lpszText = SkipSpaces(lpszText);
    CHAR cDelim = cSPACE;
    if (*lpszText == cDOUBLE_QUOTE) {
        cDelim = cDOUBLE_QUOTE;
        lpszText++;
        }
    INT nSize = 1;
    while (*lpszText && (*lpszText != cDelim) && (nSize < nBufSize)) {
        *lpszBuffer++ = *lpszText++;
        nSize++;
        }
    *lpszBuffer++ = cNULL;

    BOOL bOK = (*lpszText == cDelim) || ((cDelim == cSPACE) && (*lpszText == cNULL));
    if (bOK && (cDelim == cDOUBLE_QUOTE)) {
        lpszText++;
        }
    WinAssert(ppszEnd);
    if (ppszEnd) {
        *ppszEnd = lpszText;
        }
    return (bOK);
}



//  Gets current directory, returned in a static buffer
//
PCSZ GetCurrentDir (VOID)
{
    STATIC CHAR szCurrentDir[MAX_PATH];
    
    MyGetCurrentDirectory(szCurrentDir, STR_LEN(szCurrentDir));
    DEBUG_OUTF(TL_VERBOSE, ("GetCurrentDir() => '%s'\n", szCurrentDir));

    return (szCurrentDir);
}


//  Get Temp directory, always with a terminating backslash.
//
BOOL GetTempDir (PSZ pszTempDir, INT nMaxLen)
{
    pszTempDir[0] = cNULL;
	INT nLen = GetTempPath(nMaxLen, pszTempDir);

	if (pszTempDir[0] == cNULL) {
    	PCSZ pszTemp = GetEnvironmentVar(szTEMP);

    	//  Determine the temporary directoy, using Windows directory if not found
    	if (pszTemp) {
        	nLen = wsprintf(pszTempDir, "%s", pszTemp);
        	}
    	else {
        	nLen = GetWindowsDirectory(pszTempDir, nMaxLen);
        	}
		}
    AddPathDelim(pszTempDir, nLen);
    DEBUG_OUTF(TL_VERBOSE, ("GetTempDir(%lx, %d) => '%s'\n", pszTempDir, nMaxLen, pszTempDir));

    return (TRUE);
}


//-----------------------------------------------------------------------------
// Stuff from the Dos/QuickWin version of common
//

//  OpenFile    Creates, opens, reopens, or deletes a file 
//  _lclose     Closes an open file 
//  _lcreat     Creates or opens a file 
//  _llseek     Repositions the file pointer 
//  _lopen      Opens an existing file 
//  _lread      Reads data from a file 
//  _lwrite     Writes data to a file 

//  Turn string-base STDIO fopen mode spec into Windows OpenFile mode code.
//
UINT WinModeFromStdioMode (PCSZ pszStdioMode)
{
    DEBUG_OUTF(TL_VERBOSE, ("WinModeFromStdioMode(%s)\n", pszStdioMode));
    UINT uMode = OF_SHARE_COMPAT;
    if (StringEquals(pszStdioMode, "r")
        || StringEquals(pszStdioMode, "rb")) {
        uMode |= OF_READ;
        }
    else if (StringEquals(pszStdioMode, "w")
             || StringEquals(pszStdioMode, "wb")) {
        uMode |= OF_WRITE | OF_CREATE;
        }
    else {
        DEBUG_OUTF(TL_BASIC, ("Unsupported STDIO file mode '%s'\n", pszStdioMode));
        }

    return (uMode);
}

PFILE MyOpenFile (PCSZ pszFileName, PCSZ pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure)
{
    DEBUG_OUTF(TL_VERBOSE, ("MyOpenFile(%s, %s, %x, %x, %d)\n",
                            pszFileName, pszFileMode, hFile, hDefaultFile, bExitOnFailure));

    if (hFile) {
        MyCloseFile(hFile, hDefaultFile);
        }
    UINT uMode = WinModeFromStdioMode(pszFileMode);
    UINT uOldErrorMode = SetErrorMode(SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
    OFSTRUCT ofs;
    hFile = OpenFile(pszFileName, &ofs, uMode);
    if (hFile == HFILE_ERROR) {
        if (bExitOnFailure) {
            ReportAppErrorFmt("Error opening the file '%s'", pszFileName);
            }
        else {
            DEBUG_OUTF(TL_BASIC, ("Error opening the file '%s'\n", pszFileName));
            }
        hFile = NULL;
        }
    SetErrorMode(uOldErrorMode);
    DEBUG_OUTF(TL_VERBOSE, ("%x <= MyOpenFile()\n", hFile));

    return (hFile);
}

PFILE OpenTextFile (PCSZ pszFileName, PCSZ pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure)
{
    DEBUG_OUTF(TL_VERBOSE, ("OpenTextFile(%s, %s, %x, %x, %d)\n",
                            pszFileName, pszFileMode, hFile, hDefaultFile, bExitOnFailure));
    
    return (MyOpenFile(pszFileName, pszFileMode, hFile, hDefaultFile, bExitOnFailure));
}

PFILE OpenDirFile (PCSZ pszDir, PCSZ pszFileName, PCSZ pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure)
{
    DEBUG_OUTF(TL_VERBOSE, ("OpenDirFile(%s, %s, %s, %x, %x, %d)\n",
                            pszDir, pszFileName, pszFileMode, hFile, hDefaultFile, bExitOnFailure));
    CHAR szFile[PATH_MAX];
    wsprintf(szFile, "%s\\%s", pszDir, pszFileName);
    
    return (MyOpenFile(szFile, pszFileMode, hFile, hDefaultFile, bExitOnFailure));
}

PFILE MyCloseFile (PFILE hFile,  PFILE hDefaultFile)
{
    DEBUG_OUTF(TL_VERBOSE, ("MyCloseFile(%x, %x)\n", hFile, hDefaultFile));
    if (hFile != hDefaultFile) {
        _lclose(hFile);
        hFile = hDefaultFile;
        }

    return (hFile);
}

PFILE CloseTextFile (PFILE hFile,  PFILE hDefaultFile)
{
    DEBUG_OUTF(TL_VERBOSE, ("CloseTextFile(%x, %x)\n", hFile, hDefaultFile));

    return (MyCloseFile(hFile, hDefaultFile));
}

BOOL RemoveDirFile (PCSZ pszDir, PCSZ pszFileName)
{
    DEBUG_OUTF(TL_VERBOSE, ("RemoveDirFile(%s, %s)\n", pszDir, pszFileName));
    CHAR szFile[PATH_MAX];
    wsprintf(szFile, "%s\\%s", pszDir, pszFileName);
    INT nResult = remove(szFile);

    return (nResult == 0);
}

UINT MyReadFile (PFILE hFile, PVOID pData, UINT uBytesToRead)
{
    ASSERT(hFile);

    UINT uBytesRead = _lread(hFile,         // file handle
                             pData,         // address of buffer for read data
                             uBytesToRead); // length of data buffer
    if (uBytesRead == HFILE_ERROR) {
        DEBUG_OUTF(TL_BASIC, ("Error reading file %x\n", hFile));
        uBytesRead = 0;
        }
    DEBUG_OUTF(TL_GARRULOUS, ("%u <= MyReadFile(%x, %lx, %u)\n", 
                              uBytesRead, hFile, pData, uBytesToRead));
    return (uBytesRead);
}

INT ReadData (PVOID pData, INT nSize, INT nToRead, PFILE hFile)
{
    DEBUG_OUTF(TL_GARRULOUS, ("ReadData(%lx, %d, %d, %x)\n", 
                            pData, nSize, nToRead, hFile));
    UINT uBytesToRead = (UINT)nSize * nToRead;
//  Assert((nBytesRead == nBytesToRead) || feof(hFile));
//  BOOL bOK = ((nRead == nToRead) || feof(hFile));
    UINT uBytesRead = MyReadFile(hFile, pData, uBytesToRead);
    INT nNumRead = (LONG)uBytesRead / nSize;
    ASSERT(nNumRead >= 0);
    DEBUG_OUTF(TL_GARRULOUS, ("%d <= ReadData()\n", 
                              nNumRead));

    return (nNumRead);
}

CONST CHAR szCR_NL[] = "\r\n";

INT GetString (PFILE hFile, PSZ pszBuffer, INT nMaxLen)
{
    DEBUG_OUTF(TL_GARRULOUS, ("GetString(%x, %lx, %d)\n", 
                            hFile, pszBuffer, nMaxLen));
    INT nLen = 0;

    //  Read the maximum amount of bytes
    LONG lStartPos = GetFilePos(hFile);
    pszBuffer[nMaxLen] = cNULL;
    UINT uSize = MyReadFile(hFile, pszBuffer, nMaxLen);
    if (uSize > 0) {
        //  Find where the string ends
        nLen = uSize;
        PCSZ pszEnd = FindString(pszBuffer, szCR_NL);
        if (pszEnd) {
            nLen = pszEnd - pszBuffer;
            pszBuffer[nLen] = cNULL;
            
            //  Move the file pointer back to just past the CR/NL
            INT nExtra = (INT)uSize - nLen - STR_LEN(szCR_NL);
            ASSERT(nExtra >= 0);
//          INT nExtra = nMaxLen - nLen - STR_LEN(szCR_NL);
            SetFilePos(hFile, -nExtra, SEEK_CUR);
            }

        LONG lEndPos = GetFilePos(hFile);
        LONG lDiff = lEndPos - lStartPos;
        ASSERT((lDiff == nLen) 
                || (pszEnd && (lDiff == (nLen + (INT)STR_LEN(szCR_NL)))));
        }
    DEBUG_OUTF(TL_GARRULOUS, ("%d <= GetString(): '%s'\n", 
                              nLen, pszBuffer));

    return (nLen);
}


BOOL MyWriteFile (PFILE hFile, PCVOID pData, UINT uLen)
{
#if _DEBUG_MODE
	if (hFile != _hTrace) {
    	DEBUG_OUTF(TL_GARRULOUS, ("MyWriteFile(%lx, %lx, %u)\n", 
               	   hFile, pData, uLen));
		}
#endif
    UINT nNumWritten = _lwrite(hFile, (PCSZ)pData, uLen);
    BOOL bOK = (nNumWritten == uLen);
    return (bOK);
}

inline PCSZ FindChar (PCSZ pszSource, CHAR cTarget)     { return (strchr(pszSource, cTarget)); }

//  Write text to the file, filtering ansi new lines (\n) into
//  carraige return/line feed sequences (\r\n).
//
BOOL WriteText (PFILE hFile, PCSZ pszText)
{
#if _DEBUG_MODE
	if (hFile != _hTrace) {
	    DEBUG_OUTF(TL_GARRULOUS, ("WriteText(%x, %s)\n", 
                   hFile, pszText));
		}
#endif
    PCSZ pszPos = pszText;
    while (*pszPos) {
        PCSZ pszEnd = FindChar(pszPos, cNEW_LINE);
        INT nLen = (pszEnd ? pszEnd - pszPos : StringLength(pszPos));
        MyWriteFile(hFile, pszPos, nLen);
        pszPos += nLen;         //  skip past the text
        if (pszEnd) {
            MyWriteFile(hFile, szCR_NL, STR_LEN(szCR_NL));
            pszPos += 1;        //  skip past new line
            }
        }
    
    return (TRUE);
}


BOOL WriteTextFmt (PFILE hFile, PCSZ pszFormat, ...)
{
#if _DEBUG_MODE
	if (hFile != _hTrace) {
	    DEBUG_OUTF(TL_GARRULOUS, ("WriteTextFmt(%x, %s, ...)\n", 
               	   hFile, pszFormat));
		}
#endif
    va_list vaArgs;
    va_start (vaArgs, pszFormat);
    CHAR szText[nTEXT_BUFFER_MAX];
    INT nLen = wvsprintf(szText, pszFormat, vaArgs);
    ASSERT(nLen <= STR_LEN(szText));
    WriteText(hFile, szText);
    
    va_end (vaArgs);
    
    return (TRUE);
}


LONG GetFilePos (PFILE hFile)
{
    ASSERT(hFile != HFILE_ERROR);
    LONG lPos = _llseek(hFile, 0, SEEK_CUR);
    ASSERT(lPos != HFILE_ERROR);
    DEBUG_OUTF(TL_GARRULOUS, ("%lx <= GetFilePos(%x)\n", lPos, hFile));
    
    return (lPos);
}


BOOL SetFilePos (PFILE hFile, LONG lOffset, INT nFromWhere)
{
    DEBUG_OUTF(TL_GARRULOUS, ("SetFilePos(%x, %lx (%ld), %d)\n", 
                            hFile, lOffset, lOffset, nFromWhere));
    ASSERT(hFile != HFILE_ERROR);
    LONG lNewPos = _llseek(hFile, lOffset, nFromWhere);
    ASSERT(lNewPos != HFILE_ERROR);
    BOOL bOK = (lNewPos != HFILE_ERROR);
    
    return (bOK);
}


LONG MyGetFileSize (PFILE hFile)
{
    DEBUG_OUTF(TL_GARRULOUS, ("MyGetFileSize(%x)\n", 
                              hFile));
    ASSERT(hFile != HFILE_ERROR);
    LONG lPos = GetFilePos(hFile);
    SetFilePos(hFile, (LONG) 0, SEEK_END);
    LONG lNumChars = GetFilePos(hFile);
    SetFilePos(hFile, lPos, SEEK_SET);
    
    return (lNumChars);
}


LONG OldGetFileSize (PCSZ pszFileName)
{
    DEBUG_OUTF(TL_GARRULOUS, ("OldGetFileSize(%s)\n", pszFileName));

    LONG lSize = -1;
    struct _stat stInfo;
    INT nResult = _stat(pszFileName, &stInfo);
    if (nResult == 0) {
        lSize = stInfo.st_size;
        }

    return (lSize);
}


#if FALSE
BOOL DisplayAvailMem (VOID)
{
    DEBUG_OUTF(TL_VERBOSE, ("DisplayAvailMem(%lx)\n"));
    WORD wListSize = (WORD)nMAX_PAGES * sizeof(PSZ);
    PPSZ apsz = (PPSZ)malloc(wListSize);
    if (!apsz) {
        DebugOut("Unable to allocate 64K buffer for DisplayAvailMem\n");
        return (FALSE);
        }
        
    INT i;
    INT nNumPages = 0;

    for (i = 0; i < nMAX_PAGES; i++) {
        apsz[i] = (PSZ)malloc(nPAGE_SIZE);
        if (!apsz[i]) {
            nNumPages = i;
            break;
            }
        }

    for (i = 0; i < nNumPages; i++) {
        free(apsz[i]);
        }
    free(apsz);

    LONG lFree = ((LONG)nNumPages * nPAGE_SIZE) + sizeof(apsz);
    DebugOut("Roughly, %l bytes available\n", lFree);
    
    return (TRUE);
}
#endif


INT StringToCode (PCODEC pCodeTable, INT nSize, PCSZ pszString)
{
    DEBUG_OUTF(TL_VERBOSE, ("StringToCode(%lx, %d, %s)\n",
                            pCodeTable, nSize, pszString));
    INT nCode = pCodeTable[0].nCode;
    for (INT i = 1; i < nSize; i++) {
        if (StringEquals(pCodeTable[i].pszString, pszString)) {
            nCode = pCodeTable[i].nCode;
            break;
            }
        }
    
    return (nCode);
}


PCSZ CodeToString (PCODEC pCodeTable, INT nSize, INT nCode)
{
    DEBUG_OUTF(TL_VERBOSE, ("CodeToString(%lx, %d, %d)\n",
                            pCodeTable, nSize, nCode));
    PCSZ pszString = pCodeTable[0].pszString;
    for (INT i = 1; i < nSize; i++) {
        if (pCodeTable[i].nCode == nCode) {
            pszString = pCodeTable[i].pszString;
            break;
            }
        }
    
    return (pszString);
}


//  Handle application/library error
//  TODO: define version w/ variable number of parameters
INT HandleError (LPCSTR pszSource, LPCSTR pszTask, DWORD dwCode, LPCSTR pszError)
{
    DEBUG_OUTF(TL_VERBOSE, ("HandleError(%s,%s,%lu,%s)\n",
                           pszSource, pszTask, dwCode, PtrText(pszError)));
    CONST CHAR szERROR_FMT1[] = "%s error %lu (%s): %s";
    CONST CHAR szERROR_FMT2[] = "%s error %lu: %s";
    CHAR szMessage[nTRACE_BUFFER_MAX];
    BOOL bOK = FALSE;
    if (pszError) {
        wsprintf(szMessage, szERROR_FMT1, pszSource, dwCode, pszError, pszTask);
        }
    else {
        wsprintf(szMessage, szERROR_FMT2, pszSource, dwCode, pszTask);
        }
    INT nPos = GetCurrentClient();
    PCSZ pszApp = (nPos != nINVALID_INDEX) ? GetClientAppName(nPos) : "Unnamed App";
    INT nResp = ReportAppError(szMessage, pszApp);

    return (nResp);
}


// ReportAppError: Generic routine for handling most error messages, including assertions
//
// TODO: add an optional paramater to override the use of Abort/Retry/Ignore
//
INT ReportAppError (PCSZ lpMessage, PCSZ pszExeName)
{
    DEBUG_OUTF(TL_BASIC, ("ReportAppError(%s,%s)\n", lpMessage, pszExeName));
#if TRUE
    INT nPos = GetCurrentClient();
	HWND hWnd = (nPos != nINVALID_INDEX) ? GetClientAppWnd(nPos) : NULL;
    INT nResult = MessageBox(hWnd, lpMessage, pszExeName, MB_ICONSTOP | MB_TASKMODAL | MB_ABORTRETRYIGNORE);
#else
    INT nResult = MessageBox(NULL, lpMessage, pszExeName, MB_ICONSTOP | MB_TASKMODAL | MB_ABORTRETRYIGNORE);
#endif
    DEBUG_OUTF(TL_BASIC, ("ReportAppError:nResult = %d\n", nResult));
    //BOOL bOK = (nResult==IDOK);

	//	If the user has lost patience, exit quickly
	if (nResult == IDABORT) {
	    ////  Initiate app shutdown if user was dissed
		//InitAppShutdown();
        ExitProcess(666);
        }

	else if (nResult == IDRETRY) {
		//	If they want to try debugging the app, break into the debugger
		IF_DEBUG(
			BOOL bEnterDebugger = GetDebugOption("EnterDebuggerOnReties", FALSE);
			if (bEnterDebugger) {
				DebugBreak();
				}
			);
		}

    return (nResult);
}


// ReportAppErrorFmt: A printf-like version ReportAppError
//
// TODO: add a way to override the use of Abort/Retry/Ignore
//		 have a version that accepts a resource ID for the format string
//
INT ReportAppErrorFmt (PCSZ pszMessageFmt, ...)
{
    DEBUG_OUTF(TL_BASIC, ("ReportAppErrorFmt(%s,...)\n", pszMessageFmt));
    // format the error text
    CHAR szErrorText[nTRACE_BUFFER_MAX];
    va_list vaArgs;
    va_start (vaArgs, pszMessageFmt);
    INT nLen = vsprintf(szErrorText, pszMessageFmt, vaArgs);
    Assert(nLen < STR_LEN(szErrorText));
    va_end (vaArgs);    

    // display it
    DEBUG_OUTF(TL_BASIC, ("Error: %s\n", szErrorText));
    INT nPos = GetCurrentClient();
    PCSZ pszAppName = GetClientAppName(nPos);
    pszAppName = pszAppName ? pszAppName : "Unnamed App";
    INT nResp = ReportAppError(szErrorText, pszAppName);
    
    return (nResp);
}


BOOL Pause (DWORD dwNumMilliSecs)
{
    DEBUG_OUTF(TL_VERBOSE, ("Pause(%lu)\n", dwNumMilliSecs));

    DWORD dwStart = GetTickCount();
    while ((GetTickCount() - dwStart) < dwNumMilliSecs) {

#if FALSE
        //  Be polite and force a yield
        WaitMessage();
#endif
        MSG msg;
        if (PeekMessage(&msg,NULL,NULL,NULL,PM_REMOVE)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }
        }
    return (TRUE);
}


VOID PurgeMessageQueue (HWND hWnd)
{
    DEBUG_OUTF(TL_VERBOSE, ("PurgeMessageQueue(%x)\n", hWnd));

    MSG msg;
    WinAssert(hWnd);
    DEBUG_OUTF(TL_DETAILED, ("purging the message queue for window %x", hWnd));
    while (PeekMessage(&msg, hWnd, 0, 0, PM_REMOVE)) {
        DEBUG_OUTF(TL_VERBOSE, ("removed message %u", msg.message));
        //  Null body
        }
}


//-----------------------------------------------------------------------------
//  Memory Allocation
//-----------------------------------------------------------------------------

//  TODO: add memory validation

CONST CHAR szNOT_ENOUGH_MEMORY_FMT[] =  "%s: Not enough memory, please free some up pronto!";
CONST CHAR szRESOURCE_PROBLEM[] =   "Resource problem";
CHAR szText[nTEXT_BUFFER_MAX];

//  Allocate memory, returning only when sufficient memory exists
//
PVOID Alloc (DWORD dwSize)
{
    DEBUG_OUTF(TL_GARRULOUS, ("Alloc(%lu)\n", dwSize));
    PVOID pMem = NULL;

    do {
        //pMem = (PVOID)GlobalAllocPtr(GPTR, dwSize);
        pMem = (PVOID)GlobalAllocPtr(GHND, dwSize);
        ASSERT(pMem);
        if (!pMem) {
            PauseMessageLoop(TRUE);
            DEBUG_OUTF(TL_BASIC, ("Alloc: Unable to allocate buffer of size %lu\n", dwSize));
            INT nPos = GetCurrentClient();
            HWND hWnd = (nPos != nINVALID_INDEX) ? GetClientAppWnd(nPos) : NULL;
            PCSZ pszApp = (nPos != nINVALID_INDEX) ? GetClientAppName(nPos) : "Unnamed App";
            wsprintf(szText, szNOT_ENOUGH_MEMORY_FMT, pszApp);
            MessageBox(hWnd, szText, szRESOURCE_PROBLEM, MB_OK | MB_TASKMODAL);
            PauseMessageLoop(FALSE);
            }
        } while (!pMem);

    return (pMem);
}

PVOID Free (PVOID pMem)
{
    DEBUG_OUTF(TL_GARRULOUS, ("Free(%lx)\n", pMem));
    if (pMem) {
        GlobalFreePtr(pMem);
        }

    return (NULL);
}

BOOL HasTrailingPathDelim (PCSZ pszDir)
{
    DEBUG_OUTF(TL_GARRULOUS, ("HasTrailingPathDelim(%s)\n", PtrText(pszDir)));
    ASSERT(pszDir);
    INT nLen = StringLength(pszDir);
    BOOL bOK = ((nLen > 0) && (pszDir[nLen - 1] == cBACK_SLASH));
    return (bOK);
}

//  Enforce a trailing back slash for the directory spec.
//  Assumes text buffer is sufficient for one more character.
//
VOID AddPathDelim (PSZ pszDir, INT nLen)
{
    DEBUG_OUTF(TL_GARRULOUS, ("AddPathDelim(%s, %d)\n", PtrText(pszDir), nLen));
    ASSERT(pszDir);

    if (nLen == -1) {
        nLen = StringLength(pszDir);
        }
    BOOL bAddIt = TRUE;
    if ((nLen > 0) && (pszDir[nLen - 1] == cBACK_SLASH)) {
        bAddIt = FALSE;
        }
    if (bAddIt) {
        pszDir[nLen] = cBACK_SLASH;
        nLen++;
        pszDir[nLen] = cNULL;
        }
    ASSERT(HasTrailingPathDelim(pszDir));
}


BOOL MyGetCurrentDirectory (PSZ pszCurrentDir, INT nMaxLen)
{
    DEBUG_OUTF(TL_GARRULOUS, ("MyGetCurrentDirectory(%lx, %d)\n", pszCurrentDir, nMaxLen));
#if FALSE
	// This is causing exceptions under NT (? library conflict)
    PCSZ pszDir = _getcwd(pszCurrentDir, nMaxLen);
#endif
	DWORD dwLen = GetCurrentDirectory(nMaxLen, pszCurrentDir);
	BOOL bOK = (dwLen > 0);

    return (bOK);
}


// FindStringIgnoreCase: Find the position of the target string in the source without
// regard for case. This uses the brute-force approach.
//
INT FindStringIgnoreCase (PCSZ pszSource, PCSZ pszTarget)
{
	DEBUG_OUT(TL_GARRULOUS, ("FindStringIgnoreCase('%s', '%s')\n", pszSource, pszTarget));

	PCSZ pszSourceAnchor = pszSource;
	BOOL bFound = FALSE;
	while (*pszSourceAnchor && !bFound) {
		PCSZ pszSourcePos = pszSourceAnchor;
		PCSZ pszTargetPos = pszTarget;
		bFound = TRUE;
		while (*pszTargetPos) {
			if (ToUpper(*pszSourcePos) != ToUpper(*pszTargetPos)) {
				bFound = FALSE;
				break;
				}
			pszSourcePos++;
			pszTargetPos++;
			}
		if (!bFound) {
			pszSourceAnchor++;
			}
		}
	INT nPos = bFound ? (pszSourceAnchor - pszSource) : nINVALID_INDEX;

	return (nPos);
}


// CreateDummyWindow: create a dummy window (static text control used)
//
HWND CreateDummyWindow (VOID)
{
	CONST CHAR szSTATIC[] = "STATIC";
	CONST CHAR szDUMMY[] = "I'm a dummy, and I'm damn proud of it!";

    // Create the dummy window
	HINSTANCE hInstance = GetClientAppInst(GetCurrentClient());
    HWND hWnd = CreateWindow (szSTATIC,			// name of window class to use
						 szDUMMY,				// name for the window
						 WS_DISABLED,			// window style
						 CW_USEDEFAULT,			// use default x position
						 CW_USEDEFAULT,			// use default y position
						 CW_USEDEFAULT,			// use default width
						 CW_USEDEFAULT,			// use default height
						 NULL,					// no parent or owner
						 NULL,					// no menu
						 hInstance,				// instance handle of application
						 NULL);					// no window creation data

	return (hWnd);
}
