//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#if !defined(_COMMONXX_H)
#define _COMMONXX_H

#if !defined(ALL_WARNINGS)
#pragma warning (disable : 4001)	// nonstandard extension 'single line comment' was used
#pragma warning (disable : 4201)	// nonstandard extension used : nameless struct/union
#pragma warning (disable : 4214)	// nonstandard extension used : bit field types other than int
#pragma warning (disable : 4505)	// removed unreferenced local function
#pragma warning (disable : 4514)	// unreferenced inline function has been removed
#endif

#include "core.h"


//  Constants
//
CONSTANT CHAR  szMYDEBUG_INI[] =       "MYDEBUG.INI";

CONSTANT INT nMAX_CLIENTS = 25;
//#endif

// Old global constants
//#define TRUE      (1)
//#define FALSE         (0)
#define SPACE       (' ')
#define NULL_CHAR   ('\0')
#define PATH_MAX    (256)

#if defined(__cplusplus)
extern "C" {
#endif

#if defined(_WIN32)
//	TODO: rework hTask references to be a generic type
#undef HTASK
#undef GetCurrentTask
#define HTASK			HANDLE
#define GetCurrentTask	(HTASK)GetCurrentProcessId
#endif

//	Structure for maintaining info about client apps of a DLL
//  TODO: make this dynamic in case the DLL becomes frequently used
//

typedef struct {
    CHAR szClientAppName[MAX_PATH];
    HINSTANCE hClientAppInst;
    HTASK hClientAppTask;
    HWND hClientAppWnd;
    BOOL bAppActive;
    } CLIENT_INFO, *PCLIENT_INFO;

//  Global variables (don't use directly; use inline functions defined below)
//
//#if defined(__cplusplus)
EXTERN INT _nNumClients;
//EXTERN CLIENT_INFO _aci[nMAX_CLIENTS];
EXTERN CLIENT_INFO _aci[];
//#endif
EXTERN CHAR _szAppName[MAX_PATH]; 
EXTERN HWND _hAppWnd;
EXTERN BOOL _bAppActive;
EXTERN BOOL _bPauseMsgLoop;


// Old global macros
#define EQUAL_STR(STR1, STR2)   (!strcmp((STR1), (STR2)))
#define CHAR_LOC(STR1, CHAR1)   (strchr((STR1), (CHAR1)))
#define STR_LOC(STR1, STR2)     (strstr((STR1), (STR2)))
#define NUM_ELEMENTS(array)     (sizeof(array) / sizeof(array[0]))
#define NUM_CHARS(string)       (NUM_ELEMENTS(string))
#define STRING_SIZE(string)     (NUM_CHARS(string) - 1)
#define MAX_CHARS(string)       (STRING_SIZE(string))

//-----------------------------------------------------------------------------
//  Debugging support
//-----------------------------------------------------------------------------

#if _DEBUG_MODE
typedef enum {TL_NONE, TL_ALWAYS, TL_ERROR=TL_ALWAYS, TL_BASIC, TL_DETAILED, TL_VERBOSE, TL_GARRULOUS} TRACE_LEVEL;
#endif

//  Global variables for debugging
//
#if _DEBUG_MODE
EXTERN INT _nTraceLevel;
EXTERN PFILE hTrace;                    // file for diagnostics
EXTERN BOOL bDebugging;
#endif

//  Exported debugging functions
//
BOOL    InitDebugging       (PCSZ pszIniFile);
BOOL    EndDebugging        (VOID);
BOOL    OutputDebugText     (PCSZ pszFormat, ...);
VOID    DebugOut            (PCSZ pszFormat, ...);
BOOL	GetDebugOption		(PCSZ pszOption, INT nDefault);
VOID    _ReportAssertion     (PCSZ pszExpr, PCSZ pszFile, INT nLine);
#if FALSE
VOID    _ReportAssertion    (PCSZ pszExpr, PCSZ pszFile, INT nLine);
#endif
VOID    _ReportError        (PCSZ pszExpr, PCSZ pszFile, INT nLine);

//  Macros for debugging
//
#if _DEBUG_MODE

#undef ASSERT
#define ASSERT(exp) \
    if (!(exp)) { \
        _ReportAssertion(#exp, __FILE__, __LINE__); \
        }

#if !defined(AssertError)
#define AssertError() 	_ReportAssertion("Unexpected Error", __FILE__, __LINE__);
#endif

#if !defined(WinAssert)
#define WinAssert   ASSERT
#endif
#if !defined(Assert)
#define Assert      ASSERT
#endif

#else

#define ASSERT(exp)
#define AssertError()
#define WinAssert(exp)
#define Assert(exp)

#endif // _DEBUG_MODE


#define ASSERT_ERROR()		AssertError()

#define	INIT_COMMON(hInst, hWnd, pszName)		InitCommon(hInst, hWnd, pszName)
#define	SET_APP_WINDOW(hInst, hWnd)				RegisterAppWindow(hInst, hWnd)



#if _DEBUG_MODE

#define INIT_DEBUGGING()        								\
			InitDebugging(szMYDEBUG_INI)
#define INIT_DEBUGGING_(ini_file)        						\
			InitDebugging(ini_file)
#define END_DEBUGGING()         								\
			EndDebugging()
#define DEBUG_OUT(nLevel, params) 								\
            if (_nTraceLevel >= nLevel) { DebugOut params; }
#define OUTPUT_DEBUG_TEXT(params) 								\
            OutputDebugText params
//DEBUG_OUT(TL_VERBOSE, ("()\n", ));

		// Be careful with embedded comma's & blocks
#define IF_DEBUG(whatever)				whatever
#define DEBUG_BLOCK(nLevel, whatever) 							\
            if (_nTraceLevel >= nLevel) { whatever; }

#else

#define INIT_DEBUGGING()
#define INIT_DEBUGGING_(file)
#define END_DEBUGGING()
#define DEBUG_OUT(nLevel, params)
#define OUTPUT_DEBUG_TEXT(params)
#define IF_DEBUG(whatever)
#define DEBUG_BLOCK(nLevel, whatever)

#endif // _DEBUG_MODE



#define DEBUG_OUTF  DEBUG_OUT

#if defined(_DEBUG)
#define OldAssert(exp)  										\
    if (!(exp)) { 												\
        _ReportAssertion(#exp, __FILE__, __LINE__); 			\
        }
#else
#define OldAssert(exp)
#endif

//  what to do if all else fails
//
#define OldFatalAssert(exp) \
    if (!(exp)) { \
        _ReportError(#exp, __FILE__, __LINE__); \
        }

//-----------------------------------------------------------------------------
//  Miscellaenous
//-----------------------------------------------------------------------------

//  conversion table
//
typedef struct {
    INT nCode;
    PCSZ pszString;
    } CODEC, *PCODEC;
    
//  Exported functions
//
BOOL    InitCommon          (HINSTANCE hInst, HWND hWnd, PCSZ pszAppName);
BOOL	RegisterAppWindow	(HINSTANCE hInst, HWND hWnd);
BOOL    RegisterClientApp   (HINSTANCE hInst, HWND hWnd, PCSZ pszName);
BOOL    UnregisterClientApp (HINSTANCE hInst);
BOOL    InitAppShutdown     (VOID);
BOOL    ParseString         (PCSZ lpszText, PSZ lpszBuffer, INT nBufSize, PPCSZ ppszEnd);
INT     ReportAppError      (PCSZ lpMessage, PCSZ pszExeName);
INT     ReportAppErrorFmt   (PCSZ pszMessageFmt, ...);
INT     HandleError         (LPCSTR, LPCSTR, DWORD, LPCSTR);

PCSZ    GetCurrentDir (VOID);
BOOL    GetTempDir (PSZ pszTempDir, INT nMaxLen);

PFILE   MyOpenFile (PCSZ pszFileName, PCSZ pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure);
PFILE   OpenTextFile (PCSZ pszFileName, PCSZ pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure);
PFILE   OpenDirFile (PCSZ pszDir, PCSZ pszFileName, PCSZ pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure);
PFILE   MyCloseFile (PFILE hFile,  PFILE hDefaultFile);
PFILE   CloseTextFile (PFILE hFile,  PFILE hDefaultFile);
BOOL    RemoveDirFile (PCSZ pszDir, PCSZ pszFileName);
UINT    MyReadFile (PFILE hFile, PVOID pData, UINT uBytesToRead);
INT     ReadData (PVOID pData, INT nSize, INT nNumToRead, PFILE hFile);
INT     GetString (PFILE hFile, PSZ pszBuffer, INT nMaxLen);
BOOL    MyWriteFile (PFILE hFile, PCVOID pData, UINT uLen);
BOOL    WriteText (PFILE hFile, PCSZ pszText);
BOOL    WriteTextFmt (PFILE hFile, PCSZ pszFormat, ...);
//BOOL  WriteText (PFILE hFile, PCSZ pszFormat, ...);
LONG    GetFilePos (PFILE hFile);
BOOL    SetFilePos (PFILE hFile, LONG lOffset, INT nFromWhere);
LONG    MyGetFileSize (PFILE hFile);
LONG    OldGetFileSize (PCSZ pszFileName);

//BOOL    DisplayAvailMem (VOID);
INT     StringToCode (PCODEC pCodeTable, INT nSize, PCSZ pszString);
PCSZ        CodeToString (PCODEC pCodeTable, INT nSize, INT nCode);
BOOL    Pause               (DWORD);
VOID    PurgeMessageQueue (HWND hWnd);
PVOID   Alloc (DWORD dwSize);
PVOID   Free (PVOID pMem);
BOOL HasTrailingPathDelim (PCSZ pszDir);
VOID AddPathDelim (PSZ pszDir, INT nLen);
BOOL MyGetCurrentDirectory (PSZ pszCurrentDir, INT nMaxLen);
INT FindStringIgnoreCase (PCSZ pszSource, PCSZ pszTarget);
HWND CreateDummyWindow (VOID);


//  Macro wrappers for exported functions (specifically memory allocation)
//
#define ALLOC_ARRAY(count, type)    (type*)Alloc((count) * sizeof(type))
#define ALLOC_STRING(length)        ALLOC_ARRAY(1 + (length), CHAR)
#define ALLOC(type)                 ALLOC_ARRAY(1, type)
#define FREE_MEM(ptr, type)         (type*)Free(ptr)
#define FREE(ptr)                   FREE_MEM(ptr, VOID)
#define FREE_ARRAY(ptr)             FREE(ptr)
#define FREE_STRING(ptr)            FREE_ARRAY(ptr)


#if defined(__cplusplus)
    }
#endif


#if _DEBUG_MODE

//#if defined(__cplusplus)
INLINE INT GetTraceLevel (VOID)         { return (_nTraceLevel); }
INLINE VOID SetTraceLevel (INT nLevel)  { _nTraceLevel = nLevel; }
//#endif

#endif  // defined(_DEBUG)


//#include "common.inl"
#include "commonxx.inl"

#endif  // !defined(_COMMONXX_H)
