//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#if !defined(_CORE_INL)
#define _CORE_INL


//	First, some C++ specific stuff
//

#if defined(__cplusplus)

#undef min
#undef max
INLINE INT min(INT n, INT m)		{ return ( (n < m) ? n : m ); }
INLINE INT max(INT n, INT m)		{ return ( (n > m) ? n : m ); }
INLINE UINT min(UINT n, UINT m)		{ return ( (n < m) ? n : m ); }
INLINE UINT max(UINT n, UINT m)		{ return ( (n > m) ? n : m ); }

INLINE INT minmax (INT nValue, INT nMin, INT nMax)
{
	return (max(nMin, 
			    min(nMax, nValue))); 
}

#if !defined(_WIN32)
INLINE LONG min(LONG n, LONG m)		{ return ( (n < m) ? n : m ); }
INLINE LONG max(LONG n, LONG m)		{ return ( (n > m) ? n : m ); }


INLINE BOOL IsKeyPressed (int nKey, BOOL bAsync)
{
    INT nState = bAsync ? GetAsyncKeyState(nKey) : GetKeyState(nKey);
    BOOL bPressed = (nState < 0);
    return (bPressed);
}

INLINE BOOL IsShiftKeyPressed (BOOL bAsync)
{
    return (IsKeyPressed(VK_SHIFT, bAsync));
}

INLINE BOOL IsControlKeyPressed (BOOL bAsync)
{
    return (IsKeyPressed(VK_CONTROL, bAsync));
}

INLINE BOOL IsAltKeyPressed (BOOL bAsync)
{
    return (IsKeyPressed(VK_MENU, bAsync));
}

#endif


//  "Type-safe" conversions
//

#if defined(_WIN32)
#define ToInt(x)	(INT)x
#else
INLINE INT ToInt (LONG lValue)
{
    ASSERT((lValue >= INT_MIN) && (lValue <= INT_MAX));
    return ((INT)lValue);
}

INLINE INT ToInt (WORD wNum)
{
    ASSERT(wNum <= INT_MAX);
    return ((INT)wNum);
}
#endif

INLINE INT LongToInt (LONG lValue)
{
    //TRACE("LongToInt is obsolete; use ToInt instead\n");
    return (ToInt(lValue));
}

INLINE INT DwordToInt (DWORD dwValue)
{
    ASSERT(dwValue <= INT_MAX);
    return ((INT)dwValue);
}

INLINE LONG ToLong (INT nValue)
{
    return ((LONG)nValue);
}

INLINE LONG IntToLong (INT nValue)
{
    //TRACE("IntToLong is obsolete; use ToLong instead\n");
    return (ToLong(nValue));
}

INLINE DWORD IntToDword (INT nValue)
{
    ASSERT(nValue >= 0);
    return ((DWORD)nValue);
}

INLINE DWORD LongToDword (LONG lValue)
{
    ASSERT(lValue >= 0);
    return ((DWORD)lValue);
}

INLINE INT ToUInt (LONG lNum)
{
    ASSERT((0 <= lNum) && (lNum <= UINT_MAX));
    return ((UINT)lNum);
}

INLINE WORD ToWord (INT nNum)
{
    ASSERT((0 <= nNum) && (nNum <= WORD_MAX));
    return ((WORD)nNum);
}

INLINE CHAR ToChar (INT nValue)
{
    //ASSERT((nValue >= CHAR_MIN) && (nValue <= CHAR_MAX));
    return ((CHAR)nValue);
}

INLINE CHAR ToChar (BYTE by)
{
    return ((CHAR)by);
}

#else

#define ToChar(xyz)		(CHAR)(xyz)

#endif  // defined(__cplusplus)


//	The remaining must be valid for normal C, so INLINE must resolve to '__inline'
//	instead of 'inline' and CONST to 'static const' instead of 'const'. In addition,
//	there must be no overloading and no reference parameters.
//


//  Wrapper inlines
//

INLINE BOOL IsSpace (CHAR c)			{ return (isspace(c)); }
INLINE BOOL MyIsSpace (CHAR c)			
				{ return ((c == ' ') || (c == '\r') || (c == '\n') || (c == '\t')); }
INLINE BOOL IsPrint (CHAR c)			{ return (isprint(c)); }
INLINE BOOL IsPrintable (CHAR c)		{ return (IsPrint(c)); }
INLINE BOOL MyIsPrintable (CHAR c)		{ return ((c >= ' ') && (c <= '~')); }
INLINE BOOL IsControlChar (CHAR c)		{ return (iscntrl(c)); }
INLINE BOOL IsDigit (CHAR ch)			{ return (isdigit(ch)); }
INLINE BOOL IsAphabetic (INT nChar)		{ return (isalpha(nChar)); }
INLINE CHAR ToUpper (CHAR c)			{ return (ToChar(toupper(c))); }
INLINE CHAR ToLower (CHAR c)			{ return (ToChar(tolower(c))); }

#if !defined(WIN32)

INLINE INT StrCmp (PCSZ psz1, PCSZ psz2) { return (_fstrcmp(psz1, psz2)); }

INLINE INT StrNCmpI (LPCSTR str1, LPCSTR str2, INT nLen) { return (_fstrnicmp(str1, str2, nLen)); }

INLINE VOID ZeroMem (LPVOID lpData, INT nLen)	{ _fmemset(lpData, 0, nLen); }

#else

INLINE INT StrCmp (PCSZ psz1, PCSZ psz2) { return (strcmp(psz1, psz2)); }

INLINE INT StrNCmpI (PCSZ psz1, PCSZ psz2, INT nLen) { return (strnicmp(psz1, psz2, nLen)); }

INLINE VOID ZeroMem (LPVOID lpData, INT nLen)	{ ZeroMemory(lpData, nLen); }

#endif



#if !defined(QUICK_WIN)
INLINE BOOL IsKeyPressed (INT nKey)
{
    return (GetKeyState(nKey) < 0);
}

INLINE BOOL IsKeyToggled (INT nKey)
{
    return ((GetKeyState(nKey) & 0x1) == 0x1);
}

INLINE BOOL IsAltPressed (VOID)     { return (IsKeyPressed(VK_MENU)); }
INLINE BOOL IsShiftPressed (VOID)   { return (IsKeyPressed(VK_SHIFT)); }
INLINE BOOL IsControlPressed (VOID) { return (IsKeyPressed(VK_CONTROL)); }
#endif


//-----------------------------------------------------------------------------
//  Inline Functions (C++ linkage)
//-----------------------------------------------------------------------------

//  API wrappers
//

INLINE INT StringLength (PCSZ pszText)      { return (lstrlen(pszText)); }

INLINE PSZ StringCopy (PSZ pszDest, PCSZ pszSource)
{
    return (lstrcpy(pszDest, pszSource));
}

INLINE PSZ StringAppend (PSZ pszSource, PCSZ pszDest)
{
    return (lstrcat(pszSource, pszDest));
}

INLINE INT StringCompare (PCSZ pszSource, PCSZ pszDest)
{
    return (lstrcmp(pszSource, pszDest));
}

INLINE BOOL StringEquals (PCSZ pszSource, PCSZ pszDest)
{
    return (StringCompare(pszSource, pszDest) == 0);
}

INLINE LONG StringToLong (PCSZ pszSource)
{
    return (strtol(pszSource,       // String to convert
                   NULL,            // Pointer to character that stops scan
                   0));             // Number base to use (0 to infer from text)
}

INLINE PCSZ FindString (PCSZ pszSource, PCSZ pszTarget)
{
    return (strstr(pszSource, pszTarget));
}

INLINE PSZ StringCopyN (PSZ pszDest, PCSZ pszSource, INT nLen)
{
    lstrcpyn(pszDest, pszSource, nLen + 1);
    ASSERT((StringLength(pszDest) == StringLength(pszSource))
           || (StringLength(pszDest) == nLen));

    return (pszDest);
}

INLINE PCSZ GetEnvironmentVar (PCSZ pszVar) { return (getenv(pszVar)); }

//  Basic numeric functions
//

INLINE unsigned Abs (int nNum)
{
    return (nNum > 0 ? nNum : -nNum);
}

//  Basic string functions
//

INLINE PCSZ SkipSpaces (PCSZ pszText)
{
    while (isspace(*pszText)) {
        pszText++;
        };
    return (pszText);
}

INLINE VOID MakeUpper (PSZ psz)
{
    while (*psz) {
        *psz = ToUpper(*psz);
        psz++;
        }
}

INLINE PCSZ PtrText (PCSZ pszText)
{
    return (pszText ? pszText : szNULL_PTR);
}


INLINE BOOL ValidHandle (HANDLE h)
{
	return ((h != NULL) && (h != INVALID_HANDLE_VALUE));
}

#endif  // !defined(_CORE_INL)
