//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "common.h"
#include "editx.h"

//IMPLEMENT_DYNAMIC(CEditx, CEdit)
IMPLEMENT_DYNCREATE(CEditx, CEdit)

#if defined(_DEBUG)
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


//-----------------------------------------------------------------------------
//	Support for extended notification message headers
//-----------------------------------------------------------------------------

//	MFC's nortification message header is extended to allow addition information
//	to be passed back to the parent.
//
//	NOTE: the extended structure must contains MFC's structure first, so that
//		  the offsets of the three fields overlap.
//
//	typedef struct tagNMHDR {
//		HWND hwndFrom;					// child window sending message
//		UINT idFrom;					// control id of the child w/in parent
//		UINT code;						// notication code
//		} NMHDR;
//
//	typedef struct tagNMHDREX {
//		NMHDR nmhdr;					// MFC's notification messager header
//		WPARAM wParam;					// generic parameter number 1
//		LPARAM lParam;					// generic parameter number 2
//		CHAR szSignature[8];			// signature for sanity check ("NMHDREX")
//		} NMHDREX;

//	Fill in an extended notification header structure, adding a signature
//	for sanity checking.
//
VOID FillNMHDREX (NMHDREX* pNMHDREX, HWND hWnd, UINT uIdFrom, UINT uCode, 
				  WPARAM wParam, LPARAM lParam)
{
	ASSERT(pNMHDREX);
	IF_DEBUG(INT nZero = 0;);
	ASSERT(nZero + offsetof(NMHDR, hwndFrom) == offsetof(NMHDREX, nmhdr.hwndFrom));
	ASSERT(nZero + offsetof(NMHDR, idFrom) == offsetof(NMHDREX, nmhdr.idFrom));
	ASSERT(nZero + offsetof(NMHDR, code) == offsetof(NMHDREX, nmhdr.code));

	if (pNMHDREX) {
		pNMHDREX->nmhdr.hwndFrom = hWnd;
		pNMHDREX->nmhdr.idFrom = uIdFrom;
		pNMHDREX->nmhdr.code = uCode;
		pNMHDREX->wParam = wParam;
		pNMHDREX->lParam = lParam;
		StringCopy(pNMHDREX->szSignature, szNMHDREX);
		}
}

//	Send a notification to the parent that some event occurred. Note that  the
//	notification structure is valid until after the SendMessage, which returns
//	only after the receiver processes the message. This is not so with PostMessage
//	which returns immediately.

VOID SendNotificationToParent (CWnd* pWnd, UINT uCode, WPARAM wParam, LPARAM lParam)
{
	ASSERT(pWnd);
	if (pWnd) {
		CWnd* pParentWnd = pWnd->GetParent();
		ASSERT(pParentWnd);
		if (pParentWnd) {
			UINT uIdFrom = pWnd->GetDlgCtrlID();
			NMHDREX nmhdrex;
			FillNMHDREX(&nmhdrex, pWnd->m_hWnd, uIdFrom, uCode, wParam, lParam);
			pParentWnd->SendMessage(WM_NOTIFY, uIdFrom, (LPARAM)&nmhdrex);
			}
		}
}


//-----------------------------------------------------------------------------
// CEditx
//-----------------------------------------------------------------------------

CEditx::CEditx (void)
	: CEdit (),
	m_pfont (NULL)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::CEditx()\n"));
	m_bNotifyOfKeystrokes = FALSE;
	m_bNotifyOfClicks = FALSE;
}

BOOL CEditx::Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::Create(%x, (%d, %d, %d, %d), %x, %d)\n",
			  dwStyle, rect.left, rect.top, rect.right, rect.bottom, pParentWnd, nID));
	return (CEdit::Create(dwStyle, rect, pParentWnd, nID));
//	return (CEdit::Create(dwStyle | CS_DBLCLKS, rect, pParentWnd, nID));
}

CEditx::~CEditx (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::~CEditx()\n"));
	DestroyWindow();
}

BEGIN_MESSAGE_MAP(CEditx, CEdit)
	//{{AFX_MSG_MAP(CEditx)
	ON_WM_LBUTTONDBLCLK()
	ON_WM_CLOSE()
	ON_WM_DESTROY()
	ON_WM_KEYDOWN()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CEditx::AllocFont (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::AllocFont()\n"));
	ReleaseFont();
	m_pfont = new CFont();
}

void CEditx::ReleaseFont (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::ReleaseFont()\n"));
	if (m_pfont) {
		CWindowDC wdc(this);
		CFont *pSystemFont = (CFont*)wdc.SelectStockObject(SYSTEM_FONT);
		SetFont(pSystemFont, FALSE);
		delete m_pfont;
		m_pfont = NULL;
		}
}


void CEditx::CreateFont (INT nSize, CStringx facename)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::CreateFont(%d, %s)\n", nSize, F1(facename)));
//	CWindowDC wdc(this);
//	INT nHeight = -MulDiv(nSize, GetDeviceCaps(wdc.m_hDC, LOGPIXELSY), 72);
	CWindowDC dc(this);
	INT nHeight = -MulDiv(nSize, dc.GetDeviceCaps(LOGPIXELSY), 72);
	//	BOOL CreateFont( int nHeight, int nWidth, int nEscapement, int nOrientation, int nWeight, BYTE 
	//	bItalic, BYTE bUnderline, BYTE cStrikeOut, BYTE nCharSet, BYTE nOutPrecision, BYTE 
	//	nClipPrecision, BYTE nQuality, BYTE nPitchAndFamily, LPCSTR lpszFacename);
	AllocFont();
	m_pfont->CreateFont(nHeight, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, facename);
    SetFont(m_pfont, TRUE);
}

void CEditx::CreateFontIndirect (const LOGFONT FAR* lpLogFont)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::CreateFontIndirect(%x)\n", lpLogFont));
	AllocFont();
	m_pfont->CreateFontIndirect(lpLogFont);
    SetFont(m_pfont, TRUE);
}

void CEditx::ChangeFont (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::ChangeFont()\n"));
	LOGFONT lf;
	ZeroMem(&lf, sizeof(lf));
	if (m_pfont) {
		m_pfont->GetObject(sizeof(LOGFONT), &lf);
		}
	else {
		strcpy(lf.lfFaceName, "System");
		lf.lfHeight = 10;
		}

	CFontDialog cfd (&lf, CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS);
	cfd.DoModal();
	CreateFontIndirect(&lf);
}
                           
void CEditx::InvokeChangeFont (CWnd* pWnd)
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::InvokeChangeFont(%x)\n", pWnd));
	if (!pWnd->IsKindOf(RUNTIME_CLASS(CEditx))) {
		pWnd->MessageBox("Incorrect window type for change font");
		return;
		}
	CEditx* pedit = (CEditx*)pWnd;
	pedit->ChangeFont();
}

PWNDPROC CEditx::GetSuperWndProcAddr (void)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CEditx::GetSuperWndProcAddr()\n"));
	static WNDPROC NEAR pfnSuper;
	return &pfnSuper;
}

void CEditx::OnLButtonDblClk (UINT nFlags, CPoint point)
{
	DEBUG_OUT(TL_GARRULOUS, ("CEditx::OnLButtonDblClk(%x, (%d,%d))\n",
			  nFlags, point.x, point.y));
	CEdit::OnLButtonDblClk(nFlags, point);
	CWnd* pParentWnd = GetParent();
	if (pParentWnd) {
#if FALSE
//		pParentWnd->OnLButtonDblClk(nFlags, point);
#if defined(_WIN32)
		::SendMessage(pParentWnd->m_hWnd, WM_LBUTTONDBLCLK, MAKELPARAM(nFlags, point.x), point.y);
#else
		::SendMessage(pParentWnd->m_hWnd, WM_LBUTTONDBLCLK, nFlags, MAKELPARAM(point.x, point.y));
#endif
#endif
		//	Tell the parent window what happened, if they are interested
		if (m_bNotifyOfClicks) {
			SendNotificationToParent(this, WM_LBUTTONDBLCLK, MAKELPARAM(nFlags, point.x), point.y);
			}
		}
}

void CEditx::OnClose() 
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::OnClose()\n"));
	CEdit::OnClose();
}

void CEditx::OnDestroy() 
{
	DEBUG_OUT(TL_VERBOSE, ("CEditx::OnDestroy()\n"));
	if (IsFontSet()) {
		ReleaseFont();
		}

	CEdit::OnDestroy();
}


void CEditx::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CEditx::OnKeyDown(%x, %d, %x)\n",
			  nChar, nRepCnt, nFlags));
	//	Tell the parent window what happened, if they are interested
	if (m_bNotifyOfKeystrokes) {
		SendNotificationToParent(this, WM_KEYDOWN, nChar, 0L);
		}
	
	CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CEditx::OnLButtonDown(UINT nFlags, CPoint point) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CEditx::OnLButtonDown(%x, (%d,%d))\n",
			  nFlags, point.x, point.y));
	
	CEdit::OnLButtonDown(nFlags, point);
	SetFocus();
	CWnd* pParentWnd = GetParent();
	if (pParentWnd) {
		//	Tell the parent window what happened, if they are interested
		if (m_bNotifyOfClicks) {
			SendNotificationToParent(this, WM_LBUTTONDOWN, MAKELPARAM(nFlags, point.x), point.y);
			}
		}
}
