//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

//	Portions of this module are adapted from Dave Edson's article/sample
//	from MSJ (Aug 1994: "Chigaco's Interface Gadgets Part II: Toolbars,
//	Status Bars, and the RichEdit Control".
//

#include "common.h"
#include "rtfedit.h"

IMPLEMENT_DYNCREATE(CRtfEdit, CEditx)

#if defined(_DEBUG)
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


// for common.inl
#define SIZE_FMT											\
			"(%d, %d)"
#define	SIZE_FIELDS(size)									\
			size.cx, size.cy
#define DUMP_SIZE(level, prefix, size)						\
			DEBUG_OUT(level, (prefix SIZE_FMT "\n",			\
					  SIZE_FIELDS(size)));

#define POINT_FMT											\
			"(%d, %d)"
#define	POINT_FIELDS(point)									\
			point.x, point.y
#define DUMP_POINT(level, prefix, point)					\
			DEBUG_OUT(level, (prefix POINT_FMT "\n",		\
					  POINT_FIELDS(point)));

//	Prototypes for local functions
//	
DWORD CALLBACK GlobalSetTextCallback (DWORD dwCookie, PBYTE pbBuff, LONG cb, PLONG pcb);


// Globals
BOOL bUsePlainEditBox = FALSE;

//-----------------------------------------------------------------------------
// CRtfEdit
//
// TODO: derive CRtfEdit from CRichEditCtrl
//-----------------------------------------------------------------------------

HINSTANCE CRtfEdit::m_hRTFLib = NULL;
PCSZ CRtfEdit::m_pszEditControl = pszRICHEDIT;


CRtfEdit::CRtfEdit (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::CRtfEdit()\n"));
	LoadRtfLibrary();
	m_nPos = nINVALID_INDEX;
	m_psInText = NULL;
}

CONST CHAR szBRAIN_DEAD[] = "Unable to load RICHED32.DLL (Error Code %ld).\nEdit boxes will be braindead.";

VOID CRtfEdit::LoadRtfLibrary (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::LoadRtfLibrary()\n"));

#if _DEBUG_MODE
	bUsePlainEditBox = GetDebugOption("UsePlainEditBox", FALSE);
	if (bUsePlainEditBox) {
		m_pszEditControl = pszEDIT;
		return;
		}
#endif

	//	Load in the DLL for the RichEdit control
	//	NOTE: this might be put into COMCTL32 like the other new controls
	if (!m_hRTFLib) {
		m_hRTFLib = LoadLibrary("RICHED32.DLL");
	    if (!m_hRTFLib) {
    	    CString sErr;
        	//sErr.Format("Unable to load RICHED32.DLL (Error Code %ld). NOTE: Edit boxes will be braindead.", GetLastError());
        	//AfxMessageBox(sErr, MB_OK);
			ReportAppErrorFmt(szBRAIN_DEAD, GetLastError());
			m_pszEditControl = pszEDIT;
    		}
		}

}


//	Create the window for the RTF control. If the RTF control DLL couldn't be loaded,
//	this will use and edit control instead.
//
BOOL CRtfEdit::Create(DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::Create(%x, (%d, %d, %d, %d), %x, %d)\n",
			  dwStyle, rect.left, rect.top, rect.right, rect.bottom, pParentWnd, nID));
	ASSERT(m_pszEditControl != NULL);
	//return CWnd::Create(m_pszEditControl, NULL, dwStyle, rect, pParentWnd, nID);
	CWnd* pWnd = this;
	return pWnd->Create(m_pszEditControl, NULL, dwStyle, rect, pParentWnd, nID);
}


CRtfEdit::~CRtfEdit (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::~CRtfEdit()\n"));
	DestroyWindow();
}


BEGIN_MESSAGE_MAP(CRtfEdit, CEditx)
	//{{AFX_MSG_MAP(CRtfEdit)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


PWNDPROC CRtfEdit::GetSuperWndProcAddr (void)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CRtfEdit::GetSuperWndProcAddr()\n"));
	static WNDPROC NEAR pfnSuper;
	return &pfnSuper;
}


int CRtfEdit::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::OnCreate(%x)\n", lpCreateStruct));
	if (CEditx::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	return 0;
}


DWORD CRtfEdit::SetTextCallback (DWORD dwCookie, PBYTE pbBuff, LONG cb, PLONG pcb)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::SetTextCallback(%x, %x, %d, %x)\n", 
	  		  dwCookie, pbBuff, cb, pcb));
	DWORD dwResult = 0;					// assume success

	//	Transfer the next chunk of the buffer
	INT nRem = m_psInText->GetLength() - m_nPos;
	INT nChunkSize = min(cb, nRem);
	PCSZ pszTextStart = (PCSZ)(*m_psInText) + m_nPos;
	CopyMemory(pbBuff, pszTextStart, nChunkSize);

	//	Indicate bytes copied & revise the position
	*pcb = nChunkSize;
	m_nPos += nChunkSize;

	return (dwResult);
}


BOOL CRtfEdit::SetText (CONST CStringx& sText)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::SetText(%x)\n", 
  		      sText));

#if _DEBUG_MODE
	if (bUsePlainEditBox) {
		SetWindowText(sText);
		return (TRUE);
		}
#endif

	//	Initialize the buffer tracking
	m_psInText = &sText;
	m_nPos = 0;

	//	Setup the edit stream descriptor to read from CString variable (baroqueness)
  	EDITSTREAM es;
	es.dwCookie = (DWORD)this;
  	es.dwError = 0;
  	es.pfnCallback = GlobalSetTextCallback;

    //	Start the streaming
    SendMessage(EM_STREAMIN, SF_RTF, (LPARAM)&es);

	return (TRUE);
}


void LogfontToCF (LOGFONT& lf, CHARFORMAT& cf)
{
    // Set the mask to tell the RTF Edit control to pay attention to
    // the font formatting bits.
    cf.dwMask = CFM_BOLD | CFM_FACE | CFM_ITALIC | CFM_OFFSET | CFM_SIZE | CFM_UNDERLINE;
    cf.yHeight = lf.lfHeight;
    cf.dwEffects = 0;
    if (lf.lfWeight == FW_BOLD) {
        cf.dwEffects |= CFE_BOLD;
		}
    if (lf.lfItalic) {
        cf.dwEffects |= CFE_ITALIC;
		}
    if (lf.lfUnderline) {
        cf.dwEffects |= CFE_UNDERLINE;
		}

    cf.bPitchAndFamily = lf.lfPitchAndFamily;
    lstrcpy(cf.szFaceName, lf.lfFaceName);
} 


void CRtfEdit::ChangeFont (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::ChangeFont()\n"));

	CEditx::ChangeFont();
	CFont* pFont = GetFont();
	ASSERT(pFont);

	if (pFont) {
		SetRedraw(FALSE);

		//	Get the font from the base class
		LOGFONT lf;
		ZeroMem(&lf, sizeof(lf));
		pFont->GetObject(sizeof(LOGFONT), &lf);

		//	Set the selection to the entire buffer, saving current selection
		INT nStart;
		INT nEnd;
		GetSel(nStart, nEnd);
		//SetSel(0, -1, TRUE);	// entire selection; don't scroll into view
		StupidSetSel(0, -1);

		//	Convert to RtfEdit's stupidly different CHARFORMAT structure
		//	(ok, so it allows color, but ...)
	    CHARFORMAT cf;
    	cf.cbSize = sizeof(cf);
	    SendMessage(EM_GETCHARFORMAT, TRUE, (LPARAM)&cf);
		LogfontToCF(lf, cf);

        // Set the new font for entire buffer
        SendMessage(EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);

		//	Restore the selection
		//SetSel(nStart, nEnd, TRUE);
		StupidSetSel(nStart, nEnd);
		SetRedraw(TRUE);
		}
}


void CRtfEdit::ChangeBackgroundColor (COLORREF color)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::ChangeBackgroundColor(%08x)\n", color));

	SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)color);
}


void CRtfEdit::ChangeTextColor (COLORREF color)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::ChangeTextColor(%08x)\n", color));

	SetRedraw(FALSE);

	//	Set the selection to the entire buffer, saving current selection
	INT nStart;
	INT nEnd;
	GetSel(nStart, nEnd);
	//SetSel(0, -1, TRUE);	// entire selection; don't scroll into view
	StupidSetSel(0, -1);

	//	Fill in the RtfEdit's CHARFORMAT structure to specify the text color
	CHARFORMAT cf;
    cf.cbSize = sizeof(cf);
	cf.dwMask = CFM_COLOR;
	cf.crTextColor = color;
	cf.dwEffects = 0;

    // Set the new font for entire buffer
    SendMessage(EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);

	//	Restore the selection
	//SetSel(nStart, nEnd, TRUE);
	StupidSetSel(nStart, nEnd);
	SetRedraw(TRUE);
}


//	Convert a rectange in logical coordinates into twips.
//
CRect ToTwips (CDC* pDC, CRect& rectLogical)
{
	DEBUG_OUTF(TL_VERBOSE, ("ToTwips(%x, %x)\n", pDC, rectLogical));
	ASSERT(pDC);
	//CDC dc;
	//dc.CreateCompatibleDC(pDC);

	//	First convert the coordinates into device coordinates
	CRect rectTwips = rectLogical;
	pDC->LPtoDP(rectTwips);

	//	Convert the coordinates into twips (by setting the logical coordinate
	//	system temporarily into twips.)
	INT nOldMapMode = pDC->SetMapMode(MM_TWIPS);
	pDC->DPtoLP(rectTwips);

	//	Restore the orginal mapping mode (eg, MM_LOENGLISH)
	pDC->SetMapMode(nOldMapMode);

	return (rectTwips);
}


//	Draw the RTF control in the given DC. This can be used just to measure the
//	requires size by setting the display parameter to FALSE.
//
LRESULT CRtfEdit::SplatTextToDC (CDC* pDC, CRect& rDrawTo, CRect& rPage, BOOL bDisplay)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::SplatTextToDC(%x, %x, %x, %d)\n",
							pDC, &rDrawTo, &rPage, bDisplay));
	DUMP_RECT(TL_VERBOSE, "\trDrawTo = ", rDrawTo, "\n");
	DUMP_RECT(TL_VERBOSE, "\trPage = ", rPage, "\n");
	ASSERT(pDC);

#if FALSE
	//	Convert from logical coordinates into twips (1/20th of a point: ~1440/in)
	CRect rRtfDrawTo = ToTwips(pDC, rDrawTo);
	CRect rRtfPage = ToTwips(pDC, rPage);
	DUMP_RECT(TL_VERBOSE, "\trRtfDrawTo = ", rRtfDrawTo, "\n");
	DUMP_RECT(TL_VERBOSE, "\trRtfPage = ", rRtfPage, "\n");
#endif

	//	Set up the descriptor the RtfEdit control uses for its drawing task
	FORMATRANGE fr;
	fr.hdc = pDC->m_hDC;				// DC for rendering
	fr.hdcTarget = pDC->m_hAttribDC;	// DC for measuring
	fr.rc = rDrawTo;  					// area on device to be drawn (in twips)
	fr.rcPage = rPage;					// size of the page (in twips)
	fr.chrg.cpMin = 0;					// draw entire control
	fr.chrg.cpMax = -1;
#if _DEBUG_MODE
	DEBUG_OUT(TL_VERBOSE, ("fr = {%x, %x, %x, %x, %d, %d}\n",
			  fr.hdc, fr.hdcTarget, &fr.rc, &fr.rcPage, fr.chrg.cpMin, fr.chrg.cpMax)); 
	DEBUG_OUT(TL_VERBOSE, ("DC mapping mode is %d\n", pDC->GetMapMode()));
	CSize sizeVpExtent = pDC->GetViewportExt();
	DUMP_SIZE(TL_VERBOSE, "DC viewpoint extent is ", sizeVpExtent);
	CSize sizeWinExtent = pDC->GetWindowExt();
	DUMP_SIZE(TL_VERBOSE, "DC window extent is ", sizeWinExtent);
	CPoint pointWinOrg = pDC->GetWindowOrg();
	DUMP_POINT(TL_VERBOSE, "DC window origin is ", pointWinOrg);
	LOGFONT lf;
	ZeroMem(&lf, sizeof(lf));
	CFont* pFont = pDC->GetCurrentFont();
	pFont->GetObject(sizeof(LOGFONT), &lf);
	DEBUG_OUT(TL_VERBOSE, ("DC font size is %d\n", lf.lfHeight));
#endif

	//	Perform the drawing
	LRESULT lResult = 0;
	SendMessage(EM_FORMATRANGE, FALSE, NULL); 			// clear out cache
	lResult = SendMessage(EM_FORMATRANGE, bDisplay, (LPARAM)&fr);	// start it up
	SendMessage(EM_FORMATRANGE, FALSE, NULL);			// shut it down

	if (!bDisplay) {
		rDrawTo = fr.rc;
		rPage = fr.rcPage;
		DUMP_RECT(TL_VERBOSE, "\trDrawTo => ", rDrawTo, "\n");
		DUMP_RECT(TL_VERBOSE, "\trPage => ", rPage, "\n");
		}

	return (lResult);
}


// StupidSetSel: set the selection in the RTF edit control

VOID CRtfEdit::StupidSetSel (INT nStartChar, INT nEndChar)
{
	DEBUG_OUT(TL_VERBOSE, ("CRtfEdit::StupidSetSel(%d %d)\n", nStartChar, nEndChar));
	CHARRANGE cr;
	cr.cpMin = nStartChar;
	cr.cpMax = nEndChar;
	SendMessage(EM_EXSETSEL, 0, (LPARAM)&cr);
}


//-----------------------------------------------------------------------------
// Local functions
//-----------------------------------------------------------------------------

DWORD CALLBACK GlobalSetTextCallback (DWORD dwCookie, PBYTE pbBuff, LONG cb, PLONG pcb)
{
	DEBUG_OUT(TL_VERBOSE, ("GlobalSetTextCallback(%x, %x, %d, %x)\n", 
	  		  dwCookie, pbBuff, cb, pcb));
	DWORD dwResult = (DWORD)-1;

	CRtfEdit* pRtfEdit = (CRtfEdit*)dwCookie;
	ASSERT(pRtfEdit && pRtfEdit->IsKindOf(RUNTIME_CLASS(CRtfEdit)));
	if (pRtfEdit) {
		dwResult = pRtfEdit->SetTextCallback(dwCookie, pbBuff, cb, pcb);
		}

	return (dwResult);
}
