// gridview.cpp : implementation file
//

#include "stdafx.h"
//#include "common.h"
#include "cword.h"
#include "editx.h"
#define SKIP_LOOKUP_DETAILS
#include "lookitup.h"
#include "lookup.h"
#include "gridview.h"

IMPLEMENT_DYNCREATE(CGridView, CFormView)

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


CONST COLORREF clrLIGHT_GRAY = RGB(192, 192, 192);
CONST COLORREF clrGRAY = RGB(128, 128, 128);
//CONST COLORREF clrLIGHT_GRAY = RGB(192, 192, 192);
CONST COLORREF clrBLACK = RGB(0, 0, 0);
CONST COLORREF clrPALE_GREEN = RGB(159, 225, 196);

// for common
inline BOOL IsAphabetic (INT nChar)		{ return (isalpha(nChar)); }

#define RECT_FMT											\
			"(%d, %d, %d, %d)"
#define	RECT_FIELDS(rect)									\
			rect.left, rect.top, rect.right, rect.bottom
#define DUMP_RECT(level, prefix, rect, suffix)				\
			DEBUG_OUT(level, (prefix RECT_FMT suffix,		\
					  RECT_FIELDS(rect)));


/////////////////////////////////////////////////////////////////////////////
// CGridView

CONST INT nBUFLEN = SHRT_MAX - 1;

CGridView::CGridView()
    : NUM_ROWS(15),
      NUM_COLS(15),
      CFormView(CGridView::IDD)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::CGridView()\n"));
    //{{AFX_DATA_INIT(CGridView)
    m_szChar = "";
    //}}AFX_DATA_INIT

    // allocate the crossword structure
    m_pcRow = new CByteArray[NUM_ROWS];

    // initialize internal representation of CrossWord to all blanks
    for (INT i = 0; i < NUM_ROWS; i++) {
        m_pcRow[i].SetAtGrow(NUM_ROWS, NULL);
        for (INT j = 0; j < NUM_COLS; j++) {
            m_pcRow[i].SetAt(j, '?');
            }
        }

#if TRUE
    m_nCellHeight = 25; // 1/4 inch
    m_nCellWidth = 25;
#endif
    m_nRow = 0;
    m_nCol = 0;

    //  specify the mouse cursor to replace the hourglass during lookups
    lu.SetWaitCursor(IDC_COFFEE);

	//	have the cell character box reflect keystrokes
	m_cwordChar.NotifyOfKeystrokes();
}

CGridView::~CGridView()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::~CGridView()\n"));
    delete [] m_pcRow;
}


//	Update the crossword grid and cell contents.
//
void CGridView::OnDraw (CDC* pDC)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnDraw(%x)\n", pDC));
    INT i;
    INT j;

	//	Set the mapping mode for 0.01" for each logical point
    pDC->SetMapMode(MM_LOENGLISH);

	//	Check out the invalid rectange
#if defined(_DEBUG)
	CRect rectClip;
	INT nClipType = pDC->GetClipBox(&rectClip); // COMPLEXREGION
	DEBUG_OUTF(TL_VERBOSE, ("clip rect (type %d) ", nClipType));
	DUMP_RECT(TL_VERBOSE, "in LP: ", rectClip, "; ");
	pDC->LPtoDP(&rectClip);
	DUMP_RECT(TL_VERBOSE, "in DP: ", rectClip, "\n");
#endif

    //  Create the letter font, matching the grid cell height to the character cell height
	//	TODO: cache the font
    CFont letter_font;
    letter_font.CreateFont(-8 * m_nCellHeight/10, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial");
    CFont* pOldFont = pDC->SelectObject(&letter_font);

    //  Draw a highlight rectangle for the current cell
	CBrush brushHighlight (clrPALE_GREEN);
    CBrush* pOldBrush = pDC->SelectObject(&brushHighlight);
    pDC->Rectangle(GetCurrentCellRect());
    pDC->SelectObject(pOldBrush);

    //  Draw the crossword grid lines
    for (i = 0; i <= NUM_ROWS; i++) {
        pDC->MoveTo(0, -i * m_nCellHeight);
        pDC->LineTo(NUM_COLS * m_nCellWidth, -i * m_nCellHeight);
        for (j = 0; j <= NUM_COLS; j++) {
            pDC->MoveTo(j * m_nCellWidth, 0);
            pDC->LineTo(j * m_nCellWidth, -NUM_ROWS * m_nCellHeight);
            }
        }
            
    //  Create black blush for filling a blackened cell
    CBrush brushBlackenedCell(clrBLACK);
    pOldBrush = pDC->SelectObject(&brushBlackenedCell);

	//	Make sure the text background is not rendered
	pDC->SetBkMode(TRANSPARENT);

    //  Fill in the cells that the user typed letters into.
    //  And, copy the bitmap over each of the "blackened" squares
    for (i = 0; i < NUM_ROWS; i++) {
        for (j = 0; j < NUM_COLS; j++) {
            CRect rect = GetCellRect(i, j);
			InflateRect(rect, 0, -1);
            if (m_pcRow[i][j] == '*') {
                DEBUG_OUT(TL_VERBOSE, ("blackening cell [%d,%d] (h=%d)", 
                          i, j, m_nCellHeight));
				DUMP_RECT(TL_VERBOSE, " at ", rect, "\n");
                pDC->Rectangle(rect);
                }
            else if (m_pcRow[i][j] != '?') {
                DEBUG_OUT(TL_VERBOSE, ("setting letter %c in cell [%d,%d]", 
                            m_pcRow[i][j], i, j));
				DUMP_RECT(TL_VERBOSE, " at ", rect, "\n");
                CStringx letter (m_pcRow[i][j]);
                pDC->DrawText(letter, -1, rect, DT_CENTER | DT_VCENTER | DT_EXTERNALLEADING);
                }
            }
        }

    //  restore the old GDI resources
    pDC->SelectObject(pOldFont);
    pDC->SelectObject(pOldBrush);

	//	Let the base class do the dialog-control updates    
    CFormView::OnDraw(pDC);
}


void CGridView::OnInitialUpdate()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnInitialUpdate()\n"));
    CFormView::OnInitialUpdate();       // tie into Document

    //  Create a small font for definitions
    //m_def.CreateFont(-8, "Arial");
	m_def.CreateFont(8, "Arial");

    m_clue.SetWindowText("");
    SetScrollSizes(MM_LOENGLISH, CSize(0,0));

    CWnd* pGridFrame = GetDlgItem(IDC_GRID_FRAME);
    ASSERT(pGridFrame);
    CRect rectFrame;
    if (pGridFrame) {
        pGridFrame->GetClientRect(rectFrame);
#if TRUE
        m_nCellHeight = (rectFrame.bottom + NUM_ROWS - 1) / NUM_ROWS;
        m_nCellWidth = (rectFrame.right + NUM_COLS - 1) / NUM_COLS;
        DEBUG_OUT(TL_VERBOSE, ("rectGridFrame: (%d, %d, %d, %d); cell dimension: %dx%d\n",
              rectFrame.left, rectFrame.top, rectFrame.right, rectFrame.bottom,
              m_nCellWidth, m_nCellHeight));
#endif
        }
}

void CGridView::DoDataExchange(CDataExchange* pDX)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::DoDataExchange(%x)\n", pDX));
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CGridView)
    DDX_Control(pDX, IDC_SUGGESTIONS, m_suggestions);
    DDX_Control(pDX, IDC_WORDS, m_words);
    DDX_Control(pDX, IDC_CLUE, m_clue);
    DDX_Control(pDX, IDC_DEF, m_def);
    DDX_Text(pDX, IDC_CWORD_CHAR, m_szChar);
    DDV_MaxChars(pDX, m_szChar, 1);
    DDX_Text(pDX, IDC_CLUE, m_szClue);
    DDX_Control(pDX, IDC_CWORD_CHAR, m_cwordChar);
    //}}AFX_DATA_MAP
}

                    
BEGIN_MESSAGE_MAP(CGridView, CFormView)
    //{{AFX_MSG_MAP(CGridView)
    ON_EN_CHANGE(IDC_CWORD_CHAR, OnChangeCwordChar)
    ON_COMMAND(ID_HELP_SUGGESTHORIZONTAL, OnSuggestHorizontal)
    ON_COMMAND(ID_HELP_SUGGESTVERTICAL, OnSuggestVertical)
    ON_COMMAND(ID_HELP_SYNONYMS, OnHelpSynonyms)
    ON_COMMAND(ID_HELP_REFERENCES, OnHelpReferences)
    ON_COMMAND(ID_HELP_DEFINITION, OnHelpDefinition)
    ON_COMMAND(ID_MISC_CHANGEFONT, OnMiscChangefont)
    ON_WM_LBUTTONDBLCLK()
    ON_WM_LBUTTONDOWN()
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_WM_CTLCOLOR()
	ON_NOTIFY(WM_KEYDOWN, IDC_CWORD_CHAR, OnNotifyKeydown)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGridView message handlers


//	Convert a point in device coordinates to row/col coordinates.
//	Returns TRUE if the point is within the grid. Modifies the
//	reference parameters for the row and column regardless.
//
BOOL CGridView::ConvertPoint (CPoint& point, INT& nRow, INT& nCol)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::ConvertPoint((%d,%d), %x, %x)\n",
							 point.x, point.y, &nRow, &nCol));
    CClientDC cdc(this);
    cdc.SetMapMode(MM_LOENGLISH);
    cdc.DPtoLP(&point);
    
    nRow = -point.y / m_nCellHeight;
    nCol = point.x / m_nCellWidth;
    
    BOOL bOK = (nRow >= 0) && (nRow < NUM_ROWS) && (nCol >= 0) && (nCol < NUM_COLS);
	DEBUG_OUT(TL_GARRULOUS, ("%d <= ConvertPoint: (%d,%d)\n", 
			  bOK, point.x, point.y));
    
    return (bOK);
}


void CGridView::OnLButtonDown(UINT nFlags, CPoint point)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnLButtonDown(%x, (%d, %d))\n", nFlags, point.x, point.y));
    INT nCol, nRow;
    BOOL bOK = ConvertPoint(point, nRow, nCol);

    //  process the mouse click if within the puzzle   
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnLButtonDown (%d, %d) to (%d, %d)\n", point.x, point.y, nRow, nCol));
    if (bOK) {
        m_cwordChar.SetFocus();
        ChangeCell(nRow, nCol);
        }

    //  otherwise let the framework handle it
    else {
        CFormView::OnLButtonDown(nFlags, point);
        }
}

void CGridView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnLButtonDblClk(%x, (%d, %d))\n", nFlags, point.x, point.y));
    INT nCol, nRow;
    BOOL bOK = ConvertPoint(point, nRow, nCol);

    //  process the double click if within the puzzle   
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnLButtonDblClk (%d, %d)\n", nRow, nCol));
    if (bOK) {
        //  toggle the darkened status of the cell
        if (m_pcRow[nRow][nCol] == '*') {
            m_pcRow[nRow].SetAt(nCol,'?');
            }
        else {
            m_pcRow[nRow].SetAt(nCol,'*');
            }
        ChangeCell(nRow, nCol);
        }

    //  otherwise let the framework handle it
    else {
        CFormView::OnLButtonDblClk(nFlags, point);
        }
}

//	Get the logical coordinates for the specified cell.
//
CRect CGridView::GetCellRect (INT nRow, INT nCol)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CGridView::GetCellRect(%d, %d)", nRow, nCol));

    CRect rect (nCol * m_nCellWidth, -nRow * m_nCellHeight,
                (nCol + 1) * m_nCellWidth, -(nRow + 1) * m_nCellHeight);
	DUMP_RECT(TL_GARRULOUS, " => ", rect, "\n");

    return (rect);
}


//	Get the coordinates for the current cell.
//
CRect CGridView::GetCurrentCellRect (void)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CGridView::GetCurrentCellRect()\n"));
    return (GetCellRect(m_nRow, m_nCol));
}


//	Add the area occupied by the current cell to the update region.
//
void CGridView::InvalidateCell (void)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::InvalidateCell()\n"));
    CClientDC dc(this);
    dc.SetMapMode(MM_LOENGLISH);
    CRect rect = GetCurrentCellRect();
    dc.LPtoDP(rect);
    InvalidateRect(rect);
#if defined(_DEBUG)
    if (IsControlPressed()) {
		if (IsAltPressed()) {
			InvalidateRect(NULL);
			}
        UpdateWindow();
        }
#endif
}

//	Set the current cell to the new coordinates, invalidating both the new
//	and old cell so the highlight is updated.
//
void CGridView::ChangeCell (INT nRow, INT nCol)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::ChangeCell(%d, %d)\n", nRow, nCol));
	ASSERT(nRow < NUM_ROWS);
	ASSERT(nCol < NUM_COLS);
    InvalidateCell();
    m_nRow = nRow;
    m_nCol = nCol;
    InvalidateCell();
}

//	Get the new character entered in the (invisible) character edit box and
//	put it into the cell at the current position. Then advance the cell in
//	the current direction.
//
void CGridView::OnChangeCwordChar()
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnChangeCwordChar()\n"));
    UpdateData(TRUE);
    if (m_szChar != "") {
        m_pcRow[m_nRow].SetAt(m_nCol, m_szChar[0]);
        m_szChar = "";
        UpdateData(FALSE);
        InvalidateCell();
#if TRUE
		INT nNewRow = m_nRow;
		INT nNewCol = min(m_nCol + 1, NUM_COLS - 1);
		ChangeCell(nNewRow, nNewCol);
#endif
        }
}

#if FALSE
void CGridView::OnGotfocusDialog1(UINT, INT, CWnd*, LPVOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnGotfocusDialog1()\n"));
    m_cwordChar.SetFocus();
}
#endif


void CGridView::OnSuggestHorizontal()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnSuggestHorizontal()\n"));
    GetHorizontalPattern();
    ShowSuggestions();
}


void CGridView::OnSuggestVertical()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnSuggestVertical()\n"));
    GetVerticalPattern();
    ShowSuggestions();
}


void CGridView::GetHorizontalPattern (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::GetHorizontalPattern()\n"));
    // determine the extent of the current word pattern
    INT nPos;
    
    lu.pattern = "";
    for (nPos = m_nCol - 1; nPos >= 0; nPos--) {
        if (m_pcRow[m_nRow].GetAt(nPos) == '*') {
            break;
            }
        lu.pattern += m_pcRow[m_nRow].GetAt(nPos);
        }
    lu.pattern.MakeReverse();
    for (nPos = m_nCol; nPos < NUM_COLS; nPos++) {
        if (m_pcRow[m_nRow].GetAt(nPos) == '*') {
            break;
            }
        lu.pattern += m_pcRow[m_nRow].GetAt(nPos);
        }
}


void CGridView::GetVerticalPattern (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::GetVerticalPattern()\n"));
    // determine the extent of the current word pattern
    INT nPos;

    lu.pattern = "";
    for (nPos = m_nRow - 1; nPos >= 0; nPos--) {
        if (m_pcRow[nPos].GetAt(m_nCol) == '*') {
            break;
            }
        lu.pattern += m_pcRow[nPos].GetAt(m_nCol);
        }
    lu.pattern.MakeReverse();
    for (nPos = m_nRow; nPos < NUM_ROWS; nPos++) {
        if (m_pcRow[nPos].GetAt(m_nCol) == '*') {
            break;
            }
        lu.pattern += m_pcRow[nPos].GetAt(m_nCol);
        }
}


void CGridView::ShowSuggestions (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::ShowSuggestions()\n"));
    // look up the word pattern
    lu.pattern.MakeUpper();
    UpdateData(TRUE);
    lu.DeriveSuggestions(m_szClue);
    m_suggestions.SetWindowText("Suggestions for " + lu.pattern);
    m_suggestions.ShowWindow(SW_SHOW);
    m_words.SetWindowText(lu.suggestions);
    m_words.ShowWindow(SW_SHOW);
}


void CGridView::OnHelpSynonyms (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpSynonyms()\n"));
    UpdateData(TRUE);
    lu.GetSynonyms(m_szClue);
    m_suggestions.SetWindowText("Synonyms for '" + m_szClue + "' ");
    m_suggestions.ShowWindow(SW_SHOW);
    m_words.SetWindowText(lu.buffer);
    m_words.ShowWindow(SW_SHOW);
}

void CGridView::OnHelpDefinition (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpDefinition()\n"));
    UpdateData(TRUE);
    lu.GetDefinition(m_szClue);
    m_def.SetWindowText(lu.buffer);
    m_def.ShowWindow(SW_SHOW);
}

void CGridView::OnHelpReferences (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpReferences()\n"));
    UpdateData(TRUE);
    lu.GetReferences(m_szClue);
    m_suggestions.SetWindowText("Cross References for '" + m_szClue + "' ");
    m_suggestions.ShowWindow(SW_SHOW);
    m_words.SetWindowText(lu.buffer);
    m_words.ShowWindow(SW_SHOW);
}

void CGridView::OnMiscChangefont()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnMiscChangefont()\n"));
    CWnd* pwnd = GetFocus();
    if (!pwnd) {
        MessageBox("No window has the focus");
        return;
        }
    CEditx::InvokeChangeFont(pwnd);
}


void CGridView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnKeyDown(%x, %d, %x)\n",
			  nChar, nRepCnt, nFlags));
	INT nNewRow = m_nRow;
	INT nNewCol = m_nCol;

	switch (nChar) {
    	case VK_UP:
			nNewRow = max(0, m_nRow - 1);
			break;
        case VK_DOWN:
			nNewRow = min(m_nRow + 1, NUM_ROWS - 1);
           	break;
       	case VK_LEFT:
			nNewCol = max(0, m_nCol - 1);
           	break;
       	case VK_RIGHT:
			nNewCol = min(m_nCol + 1, NUM_COLS - 1);
           	break;
       	}
	if ((nNewRow != m_nRow) || (nNewCol != m_nCol)) {
		ChangeCell(nNewRow, nNewCol);
		}
	
	CFormView::OnKeyDown(nChar, nRepCnt, nFlags);
}


void CGridView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnChar(%x, %d, %x)\n",
			  nChar, nRepCnt, nFlags));
	if (IsAphabetic(nChar)) {
		m_szChar = (CHAR)nChar;
		OnChangeCwordChar();
		}
	
	CFormView::OnChar(nChar, nRepCnt, nFlags);
}


HBRUSH CGridView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnCtlColor(%x, %x, %x)\n", 
			  pDC, pWnd, nCtlColor));
	//	Set the background for the edit box to gray (instead of white)
	if (nCtlColor == CTLCOLOR_EDIT) {
		pDC->SetBkColor(clrLIGHT_GRAY);
		HBRUSH hBrush = (HBRUSH)::GetStockObject(LTGRAY_BRUSH);
		return (hBrush);
		}
	
	return CFormView::OnCtlColor(pDC, pWnd, nCtlColor);
}

VOID CGridView::OnNotifyKeydown (NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnNotifyKeydown(%x, %x)\n", pNMHDR, pResult));
	NMHDREX* pNMHDREX = (NMHDREX*)pNMHDR;
	ASSERT(IsValidNMHDREX(pNMHDREX));
	OnKeyDown(pNMHDREX->wParam, 1, 0);
	*pResult = 1;						// standard MFC notification processed indicator
}
