// GridInfer.cpp : Defines the class behaviors for the application.
//
// This is an adaptation of DIBLOOK from the Microsoft MFC SDK:
//
//		This is a part of the Microsoft Foundation Classes C++ library.
//		Copyright (C) 1992 Microsoft Corporation
//		All rights reserved.
//
//		This source code is only intended as a supplement to the
//		Microsoft Foundation Classes Reference and Microsoft
//		QuickHelp and/or WinHelp documentation provided with the library.
//		See these sources for detailed information regarding the
//		Microsoft Foundation Classes product.
//

#include "stdafx.h"
#include "GridInfer.h"

#if TRUE
#include "xword.h"
#endif

#include "mainfrm.h"
#include "dibdoc.h"
#include "dibview.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGridInferApp

BEGIN_MESSAGE_MAP(CGridInferApp, CWinApp)
	//{{AFX_MSG_MAP(CGridInferApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	//ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGridInferApp construction
// Place all significant initialization in InitInstance

CGridInferApp::CGridInferApp()
{
	m_bBatchMode = FALSE;
	m_bLaunchedEmbedded = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CGridInferApp object

CGridInferApp NEAR theApp;

/////////////////////////////////////////////////////////////////////////////
// CGridInferApp initialization

BOOL CGridInferApp::InitInstance()
{
	// Standard initialization
	//  (if you are not using these features and wish to reduce the size
	// of your final executable, you should remove the following initialization

	Enable3dControls();     // Use 3d controls in dialogs
	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register document templates which serve as connection between
	//  documents and views.  Views are contained in the specified view

	AddDocTemplate(new CMultiDocTemplate(IDR_DIBTYPE,
			RUNTIME_CLASS(CDibDoc),
			RUNTIME_CLASS(CMDIChildWnd),        // standard MDI child frame
			RUNTIME_CLASS(CDibView)));

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

#if TRUE
	InitCommon(m_hInstance, m_pMainWnd->m_hWnd, "GridInfer");
	INIT_DEBUGGING_("GridInfer.ini");
	DEBUG_OUT(TL_VERBOSE, ("CGridInferApp::InitInstance()\n"));
#endif

	pMainFrame->ShowWindow(m_bBatchMode ? SW_SHOWMINIMIZED : m_nCmdShow);
	pMainFrame->UpdateWindow();

	// enable file manager drag/drop and DDE Execute open
	m_pMainWnd->DragAcceptFiles();
	EnableShellOpen();
	RegisterShellFileTypes();

	CString sRemCommandLine = ParseCmdLine();

	// simple command line parsing
	if (m_sFile.IsEmpty())
	{
		// create a new (empty) document
		// OnFileNew();
	}
	else if (m_bLaunchedEmbedded)
	{
		// program launched embedded - wait for DDE or OLE open
	}
	else
	{
		// open an existing document
		OpenDocumentFile(m_sFile);
		if (m_bBatchMode) {
			m_pMainWnd->PostMessage(WM_COMMAND, ID_APP_EXIT);
		}
	}

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg() : CDialog(CAboutDlg::IDD)
		{
			//{{AFX_DATA_INIT(CAboutDlg)
			//}}AFX_DATA_INIT
		}

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
		enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

// Implementation
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CGridInferApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CGridInferApp commands

void CGridInferApp::OnFileOpen() 
{
	// TODO: Add your command handler code here
	
}


CString CGridInferApp::ParseCmdLine (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridInferApp::ParseCmdLine()\n"));
    INT i = 0;  
	CString sRemainder;

	CONST CStringx& line = m_lpCmdLine;
	::ParseCmdLine(line);
    while (i < line.GetLength()) {
        SkipSpaces(line, i);
		INT nStartPos = i;
        if (i == line.GetLength()) {
            break;
            }
        if (IsCommandSwitchDelim(line[i])) {
			i++;
            CHAR cOption = line[i++];
            switch (ToUpper(cOption)) {
                case 'B': {
                    m_bBatchMode = ParseOption(line, i);
                    break;
                    }
                case 'E': {
                    m_bLaunchedEmbedded = ParseOption(line, i);
                    break;
                    }
                default:
					i -= 2;
					sRemainder += ExtractToken(line, i, " \t");
					sRemainder += cSPACE;
                    break;
                }
            }
		else if (m_sFile.IsEmpty()) {
			m_sFile = ExtractToken(line, i, " \t");
		}
		else {
			sRemainder += ExtractToken(line, i, " \t");
			sRemainder += cSPACE;
		}
		if (nStartPos == i) {
			ASSERT_ERROR();
			i++;
		}
	}
	DEBUG_OUT(TL_VERBOSE, ("CGridInferApp::ParseCmdLine => '%s'\n", F(sRemainder)));

	return (sRemainder);
}
