#if !defined(_IDXINFO_H)
#define _IDXINFO_H

#ifdef __cplusplus
extern "C" {
#endif

//	Constants regarding IDX files
//
CONST INT 	nBLOCK_SIZE = 			1024;
CONST WORD  wGROUP_SIZE =			32768;
CONST INT 	nBLOCKS_PER_GROUP = 	wGROUP_SIZE / nBLOCK_SIZE;
CONST INT 	nMIN_ENTRY = 			12;					//	0x0C
CONST LONG 	lMEG = 					(LONG)1024 * 1024;
CONST CHAR	szNA[] =				"N/A";
CONST CHAR	szUNKOWN_REFS[] =		"UNKNOWN REFS";
//CONST LONG 	lMAX_DATA_OFFSET = 		16 * lMEG;

//	Constants regarding BNX file (Block Index)
//
CONST INT nMAX_GROUPS = 512;

typedef struct tagIndex *PINDEX;
typedef struct tagIndex {
    PCSZ pszPhrase;                     // phrase or word in entry
    } INDEX;

typedef struct {
	CHAR szSource[10];					// name of the reference source
	PFILE hIdxFile;						// index to be indexed
	PFILE hDatFile;						// actual data file (ASCII text)
	PFILE hIndex;						// block index file (BNX)
    PFILE hAsciiIndex;                  // ascii version of the index  
    LONG lStartOffset;					// offset to start scanning IDX
    LONG lEndOffset;					// offset to end scanning IDX
    LONG lMaxDataOffset;				// largest data offset permissible in IDX
    BOOL bAsciiIndex;                   // produce ASCII index file (ANX)
    BOOL bDebugMode;                    // run in debugging mode
    BYTE abyResidual[sizeof(LONG)];		// portion of xref at end of block
    INT nBytesLeft;						// number of residual bytes
    PSTR pszLastPhrase;					// the phrase being continued
    LONG lBlockNum;						// current block number
#if FALSE
    LONG lPrevBlock;					// previous block in index chain
    LONG lNextBlock;					// next block in index chain
#endif
    } IDXFILE_INFO, *PIDXFILE_INFO;


//	Block entry data structures
//
CONST BYTE	POS_MASK =		0x80;

CONST BYTE	ENTRY_HEAD =	0x02;
CONST BYTE  DEF_WORD =		0x03;

CONST BYTE	MULTIPLE_REFS =	0x01;
CONST BYTE	UNKNOWN_REFS =	0x02;
CONST BYTE	SINGLE_REF =	0x03;
CONST BYTE	QUOT_REF =		0x07;

#pragma pack(1)
typedef struct {
	LONG lCode1;
	LONG lNextBlock;
	LONG lPrevBlock;
	WORD wCode1;
	WORD wCode2;
	WORD wCode3;
	} BLOCK_HDR, *PBLOCK_HDR;

typedef struct {
	BYTE bySize;						// number of bytes in entry (exluding size)
	BYTE byRefType;						// 0x01 for Name + Refs;  0x03 for Ref + Name
	BYTE byEntryPos;					// position in composite entry (|| 0x80 if last)
	BYTE byCode2;
	BYTE byEntryType;					// 0x02 entry head; 0x03 word in entry
	BYTE byCode3;
	BYTE byNull;						// prefix null to facilitate searching?
	} ENTRY_HDR, *PENTRY_HDR;
#pragma pack()


typedef struct {
	PENTRY_HDR pHdr;
	WORD wSize;
	PCSZ pszPhrase;
	INT nPhraseLen;						// note that -1 is used for NULLs
	INT nBaseSize;
	PBYTE pFirstRef;
	PLONG plRefs;
	INT nNumRefs;
	BOOL bContinuation;
	PBYTE pNext;
	} ENTRY_INFO, *PENTRY_INFO;

inline BOOL CloseCount (LONG lCount, LONG lPrevCount)
{
	LONG lDiff = (lCount - lPrevCount);
	return ((lDiff > 0) && (lDiff < 10));
}

BOOL IsIndexBlock	(PIDXFILE_INFO pxi, PBLOCK_HDR pBlockHdr);
BOOL InterpretEntry (PBYTE pStart, PENTRY_INFO pEntry, PIDXFILE_INFO pxi);
BOOL InterpretRefs 	(PBYTE pStart, PENTRY_INFO pEntry, PIDXFILE_INFO pxi);

#ifdef __cplusplus
}
#endif

#endif // !defined(_IDX_INFO_H)
