//	Copyleft 1993-1995 Tom O'Hara		A simple library of common stuff
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#if !defined(_COMMON_H)
#define _COMMON_H

#if defined(__cplusplus)
#include "NonStdAfx.h"
#endif

#include "core.h"


#if defined(__cplusplus)

// Type definitions
//
typedef CArray<INT, INT>	CIntArray;


// Constants
//
CONST INT nMAX_FILTER_TEXT = 512;
CONST INT nMAX_FILTER_ARG = 256;
CONST CHAR szOPTIONS[] = "Options";
// For use with UpdateData()
CONST BOOL bSAVE_AND_VALIDATE = TRUE;
CONST BOOL bINITIALIZE = !bSAVE_AND_VALIDATE;

//  Global variables
//
EXTERN CStringArray asNull;
EXTERN CString sNull;
EXTERN CWinApp* pWinApp;

////    Change this to an Application Class (eg, CWinAppX)
////
EXTERN CString printFile;
INLINE VOID SetPrintFile (CString& file)    { printFile = file; }
INLINE CString& GetPrintFile (VOID)         { return (printFile); }


//  String extension
//
class CStringx : public CString
{
public:
// Constructors
    CStringx();
    CStringx(const CString& stringSrc);
    CStringx(char ch, int nRepeat = 1);
    CStringx(const char* psz);
    CStringx(const char* pch, int nLength);
    //  operations
public:
    INT FirstNonblank (VOID);
    INT LastNonblank (VOID);
    CStringx TrimLeading (VOID);
    CStringx TrimTrailing (VOID);
    CStringx Trim (VOID);
    CStringx RemoveComment (CHAR commentChar);
    INT SkipDelims (INT nPos, CONST CStringx& chars);
    INT FindAt (INT nPos, CHAR ch);
    INT FindAt (INT nPos, CONST CStringx& chars);
public:
    // overloaded assignment
    const CStringx& operator=(const CString& stringSrc);
    const CStringx& operator=(const CStringx& stringSrc);
    const CStringx& operator=(char ch);
    const CStringx& operator=(const char* psz);
    operator CString (VOID) { return (*(CString*)this); }
};

#if FALSE
class CStringxArray : public CObjArray
{
public:
    CStringxArray();
    VOID ~CStringxArray();
    CStringx operator[] (INT i) { 
}
#endif

//  Miscellaneous functions
//
VOID ReportAssertion (PCSZ pszText, PCSZ pszFile, INT nLine);
BOOL DisplayError (PCSZ pszMessage);
INT StringToInt (CONST CString& text, INT& nPos);
VOID ParseCmdLine (LPCSTR lpArgs);
VOID ParseCmdLine (CONST CString& line);
VOID ParseCmdLine (PSZ apszArgv[], INT nArgc);
VOID TokenizeText (CStringArray& tokens, CONST CStringx& text, CONST CStringx& delims);
CString JoinTokens (CONST CStringArray& tokens, CHAR cSeparator = cCOMMA);
CString FilterControlChars (PCSZ pszText);
CString ExtractToken (CONST CStringx& text, INT& pos, CONST CString& delims);
CString ExtractHead (CStringx& phrase);
PSZ AccessString (CString* pstr, INT nLen);
VOID ReleaseString (CString* pstr, INT nLen = -1);
VOID SplitPath (CONST CString& filename, CString* pdrive, CString* pdir, CString* pfname, CString* pext);
CString MakePath (PCSZ pszDrive, PCSZ pszDir, PCSZ pszFname, PCSZ pszExt);
CString ExtractFileBase (CONST CString& filename);
CString ReplaceExtension (PCSZ pszFileName, PCSZ pszExt);
CString Blanks (INT nLen);
#if !defined(QUICK_WIN)
HWND GetMainHwnd (VOID);
CString GetTempPath (VOID);
//VOID Pause (DWORD dwMilliSecs);
#endif
//LONG AvailMem (VOID);
CString FormatDate (CTime& date, BOOL bEuro = FALSE);
CString FormatDate (BOOL bEuro = FALSE);
CString FormatTime (CTime& date);
CString FormatTime (VOID);
CString FormatDateTime (CTime& date, BOOL bEuro = FALSE);
CString FormatDateTime (BOOL bEuro = FALSE);
CString FilterText (PCSZ pszInText, INT nMaxLen = nMAX_FILTER_ARG);
VOID WriteAppOption (PCSZ pszOption, INT nValue, PCSZ pszSection = szOPTIONS);
VOID WriteAppOption (PCSZ pszOption, PCSZ pszValue, PCSZ pszSection = szOPTIONS);
INT GetAppOption (PCSZ pszOption, INT nDefault, PCSZ pszSection = szOPTIONS);
CString GetAppOption (PCSZ pszOption, PCSZ pszDefault, PCSZ pszSection = szOPTIONS);
VOID WriteAppBinaryOption (PCSZ pszOption, PVOID pValue, DWORD dwSize, PCSZ pszSection = szOPTIONS);
BOOL GetAppBinaryOption (PCSZ pszOption, PVOID pValue, DWORD dwSize, PCSZ pszSection = szOPTIONS);
COLORREF GetColorOption (COLORREF colorDefault, CWnd* pWnd = NULL);
BOOL FileExists (PCSZ pszFile);
CString ExtractFileDir (PCSZ pszFileName);
CString GetProgramDir (VOID);
VOID AddPathDelim (CString& sFile);
INT ParseOption (CONST CStringx& line, INT& pos, BOOL bDefault = TRUE);


//  Miscellaneous numeric operations
//

#define NORMALIZE(value, divisor)   (((value) + ((divisor) - 1)) & (0x0FFFFFFF & ~((divisor) - 1)))

//  Other stuff
//
#define MAKE_CLASS_CAST(class_name) \
    inline class_name* To##class_name (CObject* pObject) \
    {   class_name* pClassObject = (class_name*)pObject; \
        if (pClassObject) { \
            ASSERT(pClassObject->IsKindOf(RUNTIME_CLASS(class_name))); \
            ASSERT_VALID(pClassObject); \
            } \
        return (pClassObject); }

// TODO: define an inline function that'll work with template classes (if possible)
#define VALID_INDEX(nPos, array)	((nPos >=0) && (nPos < array.GetSize()))


// Macros for dynamically loading DLL's and accessing specific functions
// For each library there will be a vaiable of type HMODULE h<libname>.
// For each function there will be a pointer to the function p<function>
// of type PFN_<function>.
//
// NOTES: typedef's must be defined for each function (PFN_<function_name>)
//				typedef BOOL (* PFN_fubar) (INT nParam);
//
// TODO: put in commonxx.h
//		 use inlines to simplify part of the macro definitions
//
#define DEFINE_LIBRARY(library)												\
	HMODULE h##library = NULL;

#define DEFINE_FUNCTION(function)											\
	PFN_##function p##function = NULL;

#define LOAD_LIBRARY(library)												\
	h##library = LoadLibrary(#library ".dll");								\
	if (!h##library) {														\
		DEBUG_OUT(TL_ERROR, ("*** Unable to load " #library ".dll\n"));		\
		throw 666;															\
		}

#define LOAD_LIBRARY_(library, alt_file_name)								\
	h##library = LoadLibrary(#library ".dll");								\
	if (!h##library) {														\
		h##library = LoadLibrary(alt_file_name);											\
		}																	\
	if (!h##library) {														\
		DEBUG_OUT(TL_ERROR, ("*** Unable to load " #library ".dll\n"));		\
		throw 666;															\
		}

#define UNLOAD_LIBRARY(library)												\
	if (h##library) {														\
		FreeLibrary(h##library);											\
		h##library = NULL;													\
		}

#define LOAD_FUNCTION(library, function)									\
	p##function = (PFN_##function)GetProcAddress(h##library, #function);	\
	if (!p##function) {														\
		DEBUG_OUT(TL_ERROR, ("*** Unable to load " #function "\n"));		\
		throw 666;															\
		}


//  C-style linkage stuff
//

#if defined(__cplusplus)
extern "C" {
#endif

//  Globals
//
#if _DEBUG_MODE
//EXTERN INT _nTraceLevel;	// defined in COMMONXX.X
EXTERN CString sFilter1;
EXTERN CString sFilter2;
EXTERN CString sFilter3;
EXTERN CString sFilter4;
EXTERN CString sFilter5;
#endif

#if defined(__cplusplus)
    }
#endif

//  Debugging support
//
#if _DEBUG_MODE
//EXTERN INT _nTraceLevel;
#define TRACEX(nLevel, nParams)         if (_nTraceLevel >= nLevel) { TRACE nParams; }
//#define AFX_DUMPX(nLevel, Variable)       if (nTraceLevel >= nLevel) AFX_DUMP1(#Variable " = ", Variable)
#define AFX_DUMPX(nLevel, Variable)     if (_nTraceLevel >= nLevel) afxDump << Variable << szNEW_LINE
#else
#define TRACEX(nLevel, nParams)
#define AFX_DUMPX(nLevel, nParams)
#endif

//  Macros for debugging
//
#if _DEBUG_MODE

#define ASSERTX(exp) \
    if (!(exp)) { \
        ReportAssertion(#exp, __FILE__, __LINE__); \
        }
#define AssertError() ReportAssertion("Unexpected Error", __FILE__, __LINE__);
#define WinAssert   ASSERTX
#define Assert      ASSERTX

//	TODO: elimiate the use of global variables (not thread-safe)
#define Fn(n, text)		(PCSZ)(sFilter##n = FilterText(text))
#define F(text)			Fn(1, text)
#define F1(text)		Fn(1, text)
#define F2(text)		Fn(2, text)
#define F3(text)		Fn(3, text)
#define F4(text)		Fn(4, text)
#define F5(text)		Fn(5, text)

#define RECT_FMT											\
			"(%d, %d, %d, %d)"
#define	RECT_FIELDS(rect)									\
			rect.left, rect.top, rect.right, rect.bottom
#define DUMP_RECT(level, prefix, rect, suffix)				\
			DEBUG_OUT(level, (prefix RECT_FMT suffix,		\
					  RECT_FIELDS(rect)));
#define DUMP_RECT_(level, prefix, rect)						\
			DEBUG_OUT(level, (prefix RECT_FMT "\n",			\
					  RECT_FIELDS(rect)));

#else

#define RECT_FMT
#define	RECT_FIELDS(rect)
#define DUMP_RECT(level, prefix, rect, suffix)
#define DUMP_RECT_(level, prefix, rect)
#define ASSERTX(exp)
#define AssertError()
#define WinAssert(exp)
#define Assert(exp)
#define Fn(n, text)
#define F(text)
#define F1(text)
#define F2(text)
#define F3(text)
#define F4(text)
#define F5(text)

#endif

#include "common.inl"
#include "commonxx.h"
#include "editx.h"
#include "rtfedit.h"

#endif  // defined(__cplusplus)

#endif  // !defined(_COMMON_H)
