//	Copyleft 1993-1996 Tom O'Hara		Lookup source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// lookup.h : header file
//

#if defined(HIDE_LOOKUP_DETAILS)
#define DLL_CLASS_EXPORT
#else
#define DLL_CLASS_EXPORT	__declspec(dllexport)
#endif
#if !defined(DLL_EXPORT)
#define DLL_EXPORT			__declspec(dllexport)
#endif


//	Constants
//
CONST UINT uNULL_COMMAND = 0;
CONST CHAR cENTRY_SEP = ';';

// Stuff for the SGML to RTF conversion
// NOTE: base careful with spaces used as RTF command terminators vs as plain text 
//       extra spacing is desirable after italics due to overhang
CONST CHAR szRTF_HEADER[] =     "{\\rtf1\\ansi\\deff0\\deftab720"
								"{\\fonttbl{\\f0\\fnil MS Sans Serif;}"
									  "{\\f1\\fnil\\fcharset2 Symbol;}"
									  "{\\f2\\fswiss\\fprq2 System;}}"
									"{\\colortbl\\red0\\green0\\blue0;}"
									"\\deflang1033\\pard\\plain"
									"\\f0\\fs20 ";
CONST CHAR szRTF_HANGING_INDENT[] = "\\tx180\\li180\\fi-180";   // 1/8" hanging indent
CONST CHAR szRTF_TRAILER[] =			"}";
CONST CHAR szRTF_START_BOLD[] =         "\\b ";
CONST CHAR szRTF_END_BOLD[] =           "\\b0 ";
CONST CHAR szRTF_START_ITALICS[] =      "\\i ";
CONST CHAR szRTF_END_ITALICS[] =        "\\i0 ";
CONST CHAR szRTF_NEW_PARA[] =           "\\par ";
CONST CHAR szRTF_TAB[] =                "\\tab ";
CONST CHAR cSGML_TAG_START =			'<';
CONST CHAR cSGML_TAG_END =              '>';

CONST CHAR szUSE_WORDNET[] =		"UseWordNet";
CONST CHAR szWORDNET_DIR[] =		"WordNetDirectory";
CONST CHAR szDEF_WORDNET_DIR[] =	"C:\\WordNet";
CONST CHAR szWORDNET_TEST_FILE[] =	"NOUN.DAT";

CONST CHAR szUSE_BOOKSHELF[] =		"UseBookshelf";
CONST CHAR szBOOKSHELF_DIR[] =		"BookshelfDirectory";
CONST CHAR szDEF_BOOKSHELF_DIR[] =	"D:\\Books";
CONST CHAR szBOOKSHELF_FILE[] =		"BookshelfFile";
CONST CHAR szDEF_BOOKSHELF_FILE[] =	"Bshelf95.m14";

CONST CHAR szUSE_TRL[] =			"UseTrl";
CONST CHAR szTRL_DATA_DIR[] =		"TrlDataDirectory";
CONST CHAR szDEF_TRL_DATA_DIR[] =	"D:\\Data";
CONST CHAR szTRL_INDEX_DIR[] =		"TrlIndexDirectory";
CONST CHAR szDEF_TRL_INDEX_DIR[] =	"C:\\CWord";
CONST CHAR szTRL_TEST_FILE[] =		"DICT.DAT";

#if !defined(_TRL_LOOKUP_H)
#define PLOOKUP_INFO	PVOID
#endif

#if !defined(_MV_LOOKUP_H)
#define PMVINFO			PVOID
#define PMVLOOKUP		PVOID
#endif

class CMvLookup;
class CMvInfo;

/////////////////////////////////////////////////////////////////////////////
// CLookup
//
// Class for encapsulating lookup information. This works as a dispatcher for
// coordinating the lookup's with various sources, such as WordNet, Bookshelf,
// and Toolworks Reference Library (to name them all).
//
// NOTE: Since this is multithreaded, special attention must be given to 
//	     passing C++ objects between thread, especially CWnd-derived classes
//		 or classes with embedded CWnd objects (such as CMvInfo). In particular,
//		 the MFC document suggests using window handles instead of CWnd* pointers.
//

class DLL_CLASS_EXPORT CLookup : public CObject
{
	DECLARE_DYNAMIC(CLookup);
public:
    CLookup(CWnd* pClient = NULL);
    virtual ~CLookup();

    static BOOL m_bUseWordNet;          // should the WordNet database be used?
    static BOOL m_bUseTRL;              // use the Toolwork's Reference Library?
	static BOOL m_bUseBookshelf;		// use Microsoft Bookself?
    static PLOOKUP_INFO m_pTrl;			// TRL lookup descriptor
	//static CMvInfo* m_pMvInfo;			// class for MediaView title info
	//static CMvLookup* m_pMvLookup;		// class for MediaView lookup's
	static PMVINFO m_pMvInfo;			// handle for MediaView title info
	static PMVLOOKUP m_pMvLookup;		// handle for MediaView lookup's
	static CWinThread* m_pInitThread;	// the thread for initializing the data sources
	static CWnd* m_pStaticParentWnd;	// parent window for view-independent hidden controls
	static DWORD m_dwTimeOut;			// time-out for initialization & lookup

public:
    // CONST INT nBUFLEN;				// obsolete limit
    UINT m_nTimerID;
    CStringx buffer;					// buffer for lookup results
    CStringx pattern;					// pattern for suggestions (eg, L??K)
    CStringx suggestions;				// suggestions derived form lookups
	// TODO: make this a static member (ie, just one instance/dialog for whole application)
	//CMvInfo* m_pMvInfo;					// class for MediaView lookup's
	//CMvLookup* m_pMvLookup;				// class for MediaView lookup's

private:
    //UINT m_nCursorID;					// cursor resource ID (for long lookups)
    //CStringx lookup_file;
    //CStringx results_file;
    //CStringx lookup_dir;
//	Implementation of client interface

	//CWnd* m_pClientWnd;				// client window for the lookup operation
	HWND m_hClientWnd;					// client window to receive notification

	//UINT uNotificationCommandID;		// command sent back to client when done
//	Implementation of interface for multithreading
private:
	CWinThread* m_pLookupThread;		// the thread for performing lookups
	LOOKUP_TYPE m_eLookupType;			// current type of lookup (eg, synonym)
	CStringx m_sSource;					// main source for the lookup operation
	CStringx m_sAltSource;				// alternative source for the lookup operation
	CStringx m_sClue;					// clue being analyzed
	CStringx m_sPhrase;					// phrase being looked up (eg, "by the way")
	BOOL m_bFormat;						// format the lookup results into RTF
	UINT m_uNotificationID;				// command ID for client notification
	friend UINT DoLookupProc (CLookup* pLookup);
	friend UINT DoDefLookupProc (CLookup* pLookup);
	friend UINT DoSuggestionsProc (CLookup* pLookup);

    // operations
public:
    //BOOL LoadLookup (void);
	static BOOL InitLookup (CWnd* pWnd, BOOL bReinit = FALSE);
	static BOOL InitLookupAux (BOOL bReinit = FALSE);
	// static BOOL ReinitLookup (VOID)			{ return (InitLookupAux(TRUE)); }
	static BOOL ReinitLookup (VOID);
	static VOID CleanupLookup (VOID);
    static void LoadProfileSettings (void);
	static VOID InitBookshelf (BOOL bReinit = FALSE);
	static VOID CleanupBookshelf (VOID);
	static BOOL ResolveMediaViewFunctions (VOID);
	static BOOL CheckBookshelfTitle (PCSZ pszDir, CString& sTitle);
	static VOID InitTrl (BOOL bReinit = FALSE);
	static VOID CleanupTrl (VOID);
	static BOOL ResolveTrlFunctions (VOID);
	static VOID InitWordNet (BOOL bReinit = FALSE);
	static VOID CleanupWordNet (VOID);
	static BOOL ResolveWordNetFunctions (VOID);

	// the basic interface
	BOOL InitSimpleLookup (BOOL bUseTRL, BOOL bUseWordNet, BOOL bUseBookshelf);
	VOID CleanupSimpleLookup (VOID);
    CString GetSynonymsText (PCSZ pszText, BOOL bFormat = FALSE)
				{ GetSynonyms(pszText, 0, bFormat);
				  return (buffer); }
	CString GetDefinitionText (PCSZ pszText, BOOL bFormat = FALSE)
				{ m_sPhrase = pszText;
				  DoDefinitionLookup(bFormat);
				  return (buffer); }
    CString GetReferencesText (PCSZ pszText, BOOL bFormat = FALSE)
				{ GetReferences(pszText, 0, bFormat);
				  return (buffer); }
    CString GetDescendantsText (PCSZ pszText, BOOL bFormat = FALSE)
				{ GetDescendants(pszText, 0, bFormat);
				  return (buffer); }

	// the complicated interface
    void DeriveSuggestions (const CStringx& clue, UINT uCommandID = 0);
    CStringx MatchPhrases (const CStringx& buffer, const CStringx& pattern, CONST CHAR cDelim);
	CStringx MatchPhrases (const CStringx& buffer, const CStringx& pattern, CONST CStringx& delims);
    BOOL Match (const CStringx& buffer, const CStringx& pattern);

    //void SetWaitCursor (UINT nCursorID)     { m_nCursorID = nCursorID; }
    void GetSynonyms (const CStringx& phrase, UINT uCommandID = 0, BOOL bFormat = TRUE);
    void GetDefinition (const CStringx& phrase, UINT uCommandID = 0, BOOL bFormat = TRUE);
    void GetReferences (const CStringx& phrase, UINT uCommandID = 0, BOOL bFormat = TRUE);
    void GetDescendants (const CStringx& phrase, UINT uCommandID = 0, BOOL bFormat = TRUE);
	VOID AbortLookup (VOID);
public:
    //CStringx& GetLookupDirName (void)   { return (lookup_dir); }
    //CStringx& GetLookupFileName (void)  { return (lookup_file); }
    //CStringx& GetResultsFileName (void) { return (results_file); }
private:
    void LookupClue (const CStringx& source, LOOKUP_TYPE eType, const CStringx& phrase, BOOL bFormat);
	void StartLookup (const CStringx& sSource, LOOKUP_TYPE eType, const CStringx& phrase, UINT uCommandID, BOOL bFormat);
	void StartDefLookup (const CStringx& phrase, UINT uCommandID, BOOL bFormat);
	void StartSuggestions (const CStringx& sClue, UINT uCommandID);
	void DoDefinitionLookup (BOOL bFormat = TRUE);
	void DoSuggestions (VOID);
	VOID SendNotification (UINT uCommandID);
};


// TODO: define C-Wrappers for CLookup & its member functions
//
