// MvLookup.h : main header file for the MVLOOKUP DLL
//

#if !defined(_MVLOOKUP_H)
#define _MVLOOKUP_H


#if defined(HIDE_MVLOOKUP_DETAILS)
#define DLL_CLASS_EXPORT
#else
#define DLL_CLASS_EXPORT	__declspec(dllexport)
#endif
#if !defined(DLL_EXPORT)
#define DLL_EXPORT			__declspec(dllexport)
#endif


#if defined(__cplusplus)


#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols


// Constants
//
CONST CHAR cMV_ENTRY_SEP = ';';


/////////////////////////////////////////////////////////////////////////////
// CMvLookupDll
// See MvLookup.cpp for the implementation of this class
//

class CMvLookupDll : public CWinApp
{
public:
	CMvLookupDll();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMvLookupDll)
	public:
	virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CMvLookupDll)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


class CMediaView;


//-----------------------------------------------------------------------------
// CMvInfo
// 
// Class for accessing MediaView. 
//
// OpenTitle: Open a MediaView title
//
// OpenTitle: Close the MediaView title
//
// OpenWordWheel: Open the MediaView wordwheel with the given name
// Wordwheels are used for doing keyword lookup
//
// OpenTopicList: Open the MediaView topic list with the given name
// Topic lists are used for doing full-text search (for cross-references)
//
// UseSgml: Indicate whether the results should be formatted with (pseudo) SGML
//

class DLL_CLASS_EXPORT CMvInfo : public CObject
{
	DECLARE_DYNCREATE(CMvInfo);
private:
	typedef enum { e_None, e_LookupPhrase, e_LookupReferences, e_OpenTitle, 
				   e_CloseTitle, e_OpenWordWheel, e_OpenTopicList, 
				   e_Exit, m_eUnknown } LOOKUP_ACTION;
public:
	CMvInfo (VOID);
	~CMvInfo (VOID);
	BOOL Init();

	// Basic access functions
	BOOL OpenTitle (PCSZ pszTitle);
	BOOL CloseTitle (VOID);
	BOOL OpenWordWheel (PCSZ pszName);
	BOOL OpenTopicList (PCSZ pszName);
	CString LookupPhrase (PCSZ pszPhrase, BOOL bDefinition = FALSE);
	CString LookupDefinitionPhrase (PCSZ pszPhrase)
				{ return (LookupPhrase(pszPhrase, TRUE)); }
	CString LookupCrossReferences (PCSZ pszPhrase);

	// Multithreaded access functions
	BOOL MtOpenTitle (PCSZ pszTitle);
	BOOL MtCloseTitle (VOID);
	BOOL MtOpenWordWheel (PCSZ pszName);
	BOOL MtOpenTopicList (PCSZ pszName);
	CString MtLookupPhrase (PCSZ pszPhrase, BOOL bDefinition = FALSE);
	CString MtLookupDefinitionPhrase (PCSZ pszPhrase)
				{ return (MtLookupPhrase(pszPhrase, TRUE)); }
	CString MtLookupCrossReferences  (PCSZ pszReferences);

	// Support for multithreading
	VOID LookupWorker (VOID);
	BOOL AcquireLock (VOID);
	BOOL ReleaseLock (VOID);

	// Miscellaneous
	BOOL IsValid (VOID);
public:
	BOOL m_bAddSgml;					// true if SGML-formatting is desired
	BOOL m_bDefinition;					// true if looking up definition text
	CString m_sPhrase;					// phrase to lookup in the MV database
	CString m_sResults;					// results from the lookup
	CString m_sTitle;					// file name of the MediaView title
	CString m_sWordWheel;				// name of word wheel (eg, Dictionary)
	CString m_sTopicList;				// name of topic list (eg, Thesaurus)
	BOOL m_bOK;							// boolean status of last command
	HWND m_hWndParent;					// handle of parent window (for dialog)
private:
	LOOKUP_ACTION m_eLookupAction;		// lookup action to be performed
	CMediaView* m_pMediaView;			// MediaView wrapper class
	CCriticalSection m_csMediaView;		// critical section for synchronization
	CEvent m_eventLookup;				// event for lookup coordination
	CEvent m_eventResults;				// event for signalling results ready
	CWinThread* m_pLookupThread;		// the thread for performing lookups
};

MAKE_CLASS_CAST(CMvInfo);


//-----------------------------------------------------------------------------
// CMvLookup
// 
// Class for doing MediaView lookups (designed particularly for Bookshelf)
// This must be used in conjunction with CMvInfo. These are separated for
// both resource efficiency and for multithreaded concerns.
//
// LookupPhrase: Lookup the topics in the database given a phrase keyword
//
// LookupCrossReferences: Lookup the topics that contain a given phrase
//
// UseSgml: Indicate whether the results should be formatted with (pseudo) SGML
//

class DLL_CLASS_EXPORT CMvLookup : public CObject
{
	DECLARE_DYNAMIC(CMvLookup);
public:
	CMvLookup (CMvInfo* pMvInfo);
	~CMvLookup (VOID);

	CString LookupPhrase (PCSZ pszPhrase, BOOL bDefinition = FALSE);
	CString LookupDefinitionPhrase (PCSZ pszPhrase)
				{ return (LookupPhrase(pszPhrase, TRUE)); }
	CString LookupCrossReferences (PCSZ pszPhrase);
	CString MtLookupPhrase (PCSZ pszPhrase, BOOL bDefinition = FALSE)
				{ return (LookupPhrase(pszPhrase, bDefinition)); }
	CString MtLookupDefinitionPhrase (PCSZ pszPhrase)
				{ return (LookupPhrase(pszPhrase, TRUE)); }
	CString MtLookupCrossReferences  (PCSZ pszReferences)
				{ return (LookupPhrase(pszReferences)); }
	VOID UseSgml (BOOL bUseSgml = TRUE);
	BOOL IsValid (VOID);

private:
	CMvInfo* m_pMvInfo;					// class for MediaView title info
};

MAKE_CLASS_CAST(CMvLookup);

/////////////////////////////////////////////////////////////////////////////

typedef struct _MvInfo {
	CMvInfo* pMvInfo;
	CString sResult;
	} MVINFO, *PMVINFO;

typedef struct _MvLookuo {
	CMvLookup* pMvLookup;
	CString sResult;
	} MVLOOKUP, *PMVLOOKUP;


#else


	typedef PVOID PMVINFO;
	typedef PVOID PMVLOOKUP;


#endif	// defined(__cplusplus)



#if defined(__cplusplus)

EXTERN_C {

	// C-wrappers for CMvInfo
	//
	PMVINFO DLL_EXPORT MviCreate (VOID);
	PMVINFO DLL_EXPORT MviDelete (PMVINFO pMvi);
	BOOL DLL_EXPORT MviOpenTitle (PMVINFO pMvi, PCSZ pszTitle);
	BOOL DLL_EXPORT MviCloseTitle (PMVINFO pMvi);
	BOOL DLL_EXPORT MviOpenWordWheel (PMVINFO pMvi, PCSZ pszName);
	BOOL DLL_EXPORT MviOpenTopicList (PMVINFO pMvi, PCSZ pszName);
	PCSZ DLL_EXPORT MviLookupPhrase (PMVINFO pMvi, PCSZ pszPhrase, BOOL bDefinition);
	PCSZ DLL_EXPORT MviLookupDefinitionPhrase (PMVINFO pMvi, PCSZ pszPhrase);
	PCSZ DLL_EXPORT MviLookupCrossReferences (PMVINFO pMvi, PCSZ pszPhrase);
	// Multithreaded access functions
	BOOL DLL_EXPORT MviMtOpenTitle (PMVINFO pMvi, PCSZ pszTitle);
	BOOL DLL_EXPORT MviMtCloseTitle (PMVINFO pMvi);
	BOOL DLL_EXPORT MviMtOpenWordWheel (PMVINFO pMvi, PCSZ pszName);
	BOOL DLL_EXPORT MviMtOpenTopicList (PMVINFO pMvi, PCSZ pszName);
	PCSZ DLL_EXPORT MviMtLookupPhrase (PMVINFO pMvi, PCSZ pszPhrase, BOOL bDefinition);
	PCSZ DLL_EXPORT MviMtLookupDefinitionPhrase (PMVINFO pMvi, PCSZ pszPhrase);
	PCSZ DLL_EXPORT MviMtLookupCrossReferences  (PMVINFO pMvi, PCSZ pszReferences);

	// C-wrappers for CMvLookup
	//
	PMVLOOKUP DLL_EXPORT MvlCreate (PMVINFO pMvi);
	PMVLOOKUP DLL_EXPORT MvlDelete (PMVLOOKUP pMvl);
	PCSZ DLL_EXPORT MvlLookupPhrase (PMVLOOKUP pMvl, PCSZ pszPhrase, BOOL bDefinition);
	PCSZ DLL_EXPORT MvlLookupDefinitionPhrase (PMVLOOKUP pMvl, PCSZ pszPhrase);
	PCSZ DLL_EXPORT MvlLookupCrossReferences (PMVLOOKUP pMvl, PCSZ pszPhrase);
	VOID DLL_EXPORT MvlUseSgml (PMVLOOKUP pMvl, BOOL bUseSgml);
	BOOL DLL_EXPORT MvlIsValid (PMVLOOKUP pMvl);
	// Multithreaded access functions
	PCSZ DLL_EXPORT MvlMtLookupPhrase (PMVLOOKUP pMvl, PCSZ pszPhrase, BOOL bDefinition);
	PCSZ DLL_EXPORT MvlMtLookupDefinitionPhrase (PMVLOOKUP pMvl, PCSZ pszPhrase);
	PCSZ DLL_EXPORT MvlMtLookupCrossReferences  (PMVLOOKUP pMvl, PCSZ pszReferences);

	}

#endif // defined(__cplusplus)



#endif // !defined(_MVLOOKUP_H)

