//	Copyleft 1993-1996 Tom O'Hara		TrlLookup source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#if !defined(_IDXINFO_H)
#define _IDXINFO_H

#ifdef __cplusplus
extern "C" {
#endif

//	Constants regarding IDX files
//
CONST INT 	nBLOCK_SIZE = 			1024;
CONST WORD  wGROUP_SIZE =			32768;
CONST INT 	nBLOCKS_PER_GROUP = 	wGROUP_SIZE / nBLOCK_SIZE;
CONST INT 	nMIN_ENTRY = 			12;					//	0x0C
CONST LONG 	lMEG = 					(LONG)1024 * 1024;
CONST CHAR	szNA[] =				"N/A";
CONST CHAR	szUNKOWN_REFS[] =		"UNKNOWN REFS";
//CONST LONG 	lMAX_DATA_OFFSET = 		16 * lMEG;

//	Constants regarding BNX file (Block Index)
//
CONST INT nMAX_GROUPS = 512;

typedef struct tagIndex {
    PCSZ pszPhrase;                     // phrase or word in entry
    } INDEX, *PINDEX;

typedef struct {
	CHAR szSource[10];					// name of the reference source
	PFILE hIdxFile;						// index to be indexed
	PFILE hDatFile;						// actual data file (ASCII text)
	PFILE hIndex;						// block index file (BNX)
    PFILE hAsciiIndex;                  // ascii version of the index  
    LONG lStartOffset;					// offset to start scanning IDX
    LONG lEndOffset;					// offset to end scanning IDX
    LONG lMaxDataOffset;				// largest data offset permissible in IDX
    BOOL bAsciiIndex;                   // produce ASCII index file (ANX)
    BOOL bDebugMode;                    // run in debugging mode
    BYTE abyResidual[sizeof(LONG)];		// portion of xref at end of block
    INT nBytesLeft;						// number of residual bytes
    PCSZ pszLastPhrase;					// the phrase being continued
    LONG lBlockNum;						// current block number
#if FALSE
    LONG lPrevBlock;					// previous block in index chain
    LONG lNextBlock;					// next block in index chain
#endif
    } IDXFILE_INFO, *PIDXFILE_INFO;


//	Constants used in IDX blocks
//
CONST BYTE	POS_MASK =		0x80;

CONST BYTE	ENTRY_HEAD =	0x02;
CONST BYTE  DEF_WORD =		0x03;

CONST BYTE	MULTIPLE_REFS =	0x01;
CONST BYTE	UNKNOWN_REFS =	0x02;
CONST BYTE	SINGLE_REF =	0x03;
CONST BYTE	QUOT_REF =		0x07;


//	TRL descriptors used in the IDX file
//
#pragma pack(1)
typedef struct {
	LONG lCode1;						// misc long-sized code (0x0 if index)
	LONG lNextBlock;					// offset of next block
	LONG lPrevBlock;					// offset of previous block
	WORD wCode1;						// unknown word-sized code
	WORD wCode2;						// another unknown word-sized code
	WORD wCode3;						// yet another unknown word-sized code
	} BLOCK_HDR, *PBLOCK_HDR;

typedef struct {
	BYTE bySize;						// number of bytes in entry (exluding size)
	BYTE byRefType;						// 0x01 for Name + Refs;  0x03 for Ref + Name
	BYTE byEntryPos;					// position in composite entry (or 0x80 if last)
	BYTE byCode2;						// unknown byte-sized code
	BYTE byEntryType;					// 0x02 entry head; 0x03 word in entry
	BYTE byCode3;						// another unknown byte-sized code
	BYTE byNull;						// prefix null to facilitate searching?
	} ENTRY_HDR, *PENTRY_HDR;
#pragma pack()


//	My entry descriptor for an index entry residing in memory. This contains the
//	result of the index entry interpretation in which offsets are resolved into 
//	pointers and the important fields delimited.
//
typedef struct {
	PENTRY_HDR pHdr;					// ptr to header block
	WORD wSize;							// size of the index entry (not this struct)
	PCSZ pszPhrase;						// phrase being indexed
	INT nPhraseLen;						// note that -1 is used for NULLs
	INT nBaseSize;						// size of entry w/o references (header + phrase)
	PBYTE pFirstRef;					// ptr to first reference
	PLONG plRefs;						// ptr to the list of reference offsets (into DAT file)
	INT nNumRefs;						// the number of references
	BOOL bContinuation;					// is this entry a continuation of previous? (eg, many many refs)
	PBYTE pNext;						// ptr to next header block
	} ENTRY_INFO, *PENTRY_INFO;


//	Decide whether the count fields taken from index blocks are sufficiently close to
//	consider the blocks related.
//
inline BOOL CloseCount (LONG lCount, LONG lPrevCount)
{
	LONG lDiff = (lCount - lPrevCount);
	return ((lDiff > 0) && (lDiff < 10));
}

BOOL IsIndexBlock	(PIDXFILE_INFO pxi, PBLOCK_HDR pBlockHdr);
BOOL InterpretEntry (PBYTE pStart, PENTRY_INFO pEntry, PIDXFILE_INFO pxi);
BOOL InterpretEntryRefs 	(PBYTE pStart, PENTRY_INFO pEntry, PIDXFILE_INFO pxi);

#ifdef __cplusplus
}
#endif

#endif // !defined(_IDX_INFO_H)
