#if !defined(_COMMON_INL)
#define _COMMON_INL

//-----------------------------------------------------------------------------
//  C++ Specific

#if defined(__cplusplus)

//  Windows API helper functions
//

INLINE BOOL IsPressed (INT nKey)
{
    return (GetAsyncKeyState(nKey) < 0);
}


//
//  Type conversion wrappers
//

INLINE DWORD LongToDword (LONG lNum)
{
    Assert(lNum >= 0);
    return ((DWORD)lNum);
}

INLINE INT LongToInt (LONG lNum)
{
    Assert((lNum >= INT_MIN) && (lNum <= INT_MAX));
    return ((INT)lNum);
}

INLINE INT DwordToInt (DWORD dwNum)
{
    Assert(dwNum <= INT_MAX);
    return ((INT)dwNum);
}

INLINE LONG DwordToLong (DWORD dwNum)
{
    Assert((dwNum <= LONG_MAX));
    return ((LONG)dwNum);
}

INLINE UINT DwordToUint (DWORD dwNum)
{
    Assert(dwNum <= UINT_MAX);
    return ((UINT)dwNum);
}

INLINE CHAR IntToChar (INT nChar)
{
    Assert(((nChar >= CHAR_MIN) && (nChar <= CHAR_MAX)) || (nChar == EOF));
    return ((CHAR)nChar);
}

//
//  Wrappers for string functions
//

CONST INT nINVALID_INDEX = -1;

INLINE BOOL IsAlpha (CHAR c)        { return (isalpha(c)); }
INLINE BOOL IsControl (CHAR c)      { return (iscntrl(c)); }
INLINE BOOL IsSpace (CHAR c)        { return (isspace(c) != 0); }
INLINE BOOL IsDigit (CHAR c)        { return (isdigit(c)); }
INLINE BOOL IsAlphaNumeric (CHAR c) { return (isalnum(c)); }
INLINE BOOL IsPrintable (CHAR c)    { return (isprint(c)); }
INLINE CHAR ToUpper (CHAR c)        { return (IntToChar(toupper(c))); }
INLINE CHAR ToLower (CHAR c)        { return (IntToChar(tolower(c))); }
INLINE BOOL IsHexChar (CHAR c)      { return (isxdigit(c)); }

INLINE BOOL IsOptionChar (CHAR c)   { return ((c == cDASH) || (c == cSLASH)); }
INLINE BOOL IsTokenChar (CHAR c)    { return (!IsSpace(c) && (c != cCOMMA)); }

INLINE BOOL IsQuoteChar (CHAR c)
{
    return ((c == cDOUBLE_QUOTE) || (c == cSINGLE_QUOTE));
}

INLINE INT StringCompare (PCSZ psz1, PCSZ psz2)
{
    return (lstrcmp(psz1, psz2));
}

INLINE PSZ StringCopy (PSZ pszDestination, PCSZ pszSource)
{
    return (lstrcpy(pszDestination, pszSource));
}

INLINE PSZ StringAppend (PSZ pszDestination, PCSZ pszSource)
{
    return (lstrcat(pszDestination, pszSource));
}

INLINE INT StringCompareNoCase (PCSZ psz1, PCSZ psz2)
{
    return (lstrcmpi(psz1, psz2));
}

INLINE INT StringLength (PCSZ psz)
{
    return (lstrlen(psz));
}


// atoi & strtol should work w/ DBCS since '0' to '9' can't be lead bytes
// But once the code is completely 32-bits, these should be replaced w/
// runtime functions from Win32.

INLINE INT StringToInt (PCSZ psz)
{
    return (atoi(psz));
}

INLINE LONG StringToLong (PCSZ psz, PPCSZ ppszEnd, INT nBase)
{
    PPSZ ppszEndOfNum = (PPSZ)ppszEnd;
    return (strtol(psz, ppszEndOfNum, nBase));
}

INLINE DWORD HexStringToDword (PCSZ psz)
{
    PSZ pszEnd;
    return (LongToDword(strtol(psz, &pszEnd, 16)));
}

INLINE PSZ StringCopyN (PSZ pszDestination, PCSZ pszSource, INT nLen)
{
#if defined(DBCS)
    PSZ pszBuffer = DbcsStringCopyN(pszDestination, pszSource, nLen);
#else
	//	TODO: resolve problem when compiled along w/ MFC
    PSZ pszBuffer = lstrcpyn(pszDestination, pszSource, 1 + nLen);
    pszBuffer[nLen] = cNULL;
    Assert(pszBuffer);
#endif
    Assert((StringLength(pszDestination) == nLen)
        || (StringLength(pszDestination) == StringLength(pszSource)));

    return (pszBuffer);
}


INLINE INT CharPos (PCSZ pszSource, CHAR cTarget)
{
    INT nPos = nINVALID_INDEX;

#if defined(DBCS)
    nPos = DbcsCharPos(pszSource, cTarget);
#else
    PCSZ pszPos = strchr(pszSource, cTarget);
    if (pszPos)
    {
        nPos = pszPos - pszSource;
    }
#endif

    return (nPos);
}


INLINE PCSZ CharPosPtr (PCSZ pszSource, CHAR cTarget)
{
	INT nPos = CharPos(pszSource, cTarget);
	PCSZ pszPos = (nPos != nINVALID_INDEX) ? (pszSource + nPos) : NULL;

	return (pszPos);
}


INLINE INT StringPos (PCSZ pszSource, PCSZ pszTarget)
{
    INT nPos = nINVALID_INDEX;

#if defined(DBCS)
    nPos = DbcsStringPos(pszSource, pszTarget);
#else
    PCSZ pszPos = strstr(pszSource, pszTarget);
    if (pszPos)
    {
        nPos = pszPos - pszSource;
    }
#endif

    return (nPos);
}


INLINE PCSZ StringPosPtr (PCSZ pszSource, PCSZ pszTarget)
{
	INT nPos = StringPos(pszSource, pszTarget);
	PCSZ pszPos = (nPos != nINVALID_INDEX) ? (pszSource + nPos) : NULL;

	return (pszPos);
}


INLINE PCSZ PcszAdvance (PCSZ pszSource)
{
#if defined(DBCS)
    return AnsiNext(pszSource);
#else
    return (pszSource + 1);
#endif
}

INLINE PSZ PszAdvance (PSZ pszSource)
{
#if defined(DBCS)
    return AnsiNext(pszSource);
#else
    return (pszSource + 1);
#endif
}

INLINE PCSZ SkipSpaces (PCSZ pszSource)
{
    while (IsSpace(*pszSource))
    {
        pszSource = PcszAdvance(pszSource);
    }

    return (pszSource);
}


INLINE PCSZ SkipOptionalComma (PCSZ pszSource)
{
    pszSource = SkipSpaces(pszSource);
    if (*pszSource == cCOMMA)
    {
        pszSource = PcszAdvance(pszSource);
    }

    return (pszSource);
}


INLINE INT nStringCopy (PSZ pszDestination, PCSZ pszSource)
{
    INT nLen;
    Assert(pszSource);
    Assert(pszDestination);

    nLen = StringLength(pszSource);
    StringCopyN(pszDestination, pszSource, nLen);
    Assert(StringLength(pszDestination) == nLen);

    return (nLen);
}


INLINE PCSZ FilterText (PCSZ pszText)
{
    return (pszText ? pszText : szNULL);
}


INLINE BOOL ValidHandle (HANDLE handle)
{
    return ((handle != NULL) && (handle != INVALID_HANDLE_VALUE));
}


//  Free a global memory handle, returning NULL.
//  Usage: gh = FreeMemoryHandle(gh);
//
INLINE HGLOBAL FreeMemoryHandle (HGLOBAL gh)
{
    if (gh != NULL)
        {
        GlobalUnlock(gh);
        GlobalFree(gh);
        }

    return (NULL);
}


INLINE VOID ForceRepaint (HWND hWnd)
{
    if (hWnd)
        {
        InvalidateRect(hWnd, NULL, TRUE);
        UpdateWindow(hWnd);
        }
}


INLINE VOID MakeUpper (PSZ pszText)
{
    AnsiUpper(pszText);
}


INLINE VOID MakeUpperN (PSZ pszText, INT nLen)
{
#if defined(WIN32)
    CharUpperBuff(pszText, nLen);
#else
    AnsiUpperBuff(pszText, nLen);
#endif
}


INLINE BOOL NotEmpty (PCSZ pszText)
{
    BOOL bNotEmpty = pszText ? (pszText[0] != cNULL) : FALSE;
    Assert(pszText);

    return (bNotEmpty);
}


INLINE BOOL StringFill (PSZ pszText, INT nLen, CHAR cFill)
{
    memset(pszText, cFill, nLen);
    pszText[nLen] = cNULL;

    return (TRUE);
}


INLINE VOID MemoryCopy (PVOID pDest, PVOID pSrc, UINT uCount)
{
    Assert(uCount > 0);
    //DEBUG_OUTF(TL_GARRULOUS, ("MemoryCopy(%lx,%lx,%u)\n", pDest, pSrc, uCount));
    memmove(pDest, pSrc, uCount);
}

//
//  Simple API wrappers
//

INLINE BOOL FileDoesExist (PCSZ pszFile)
{
    OFSTRUCT ofs;
    return (FileExists(pszFile, &ofs));
}


/*---------------------------------------------------------------------------

    CopyCharAdvance

    Helper inline function for copying characters, with conditional support
    for DBCS characters. The current source character is copied over and
    both the source and destination pointers are advanced.

    NOTE:   All parameters are references.

------------------------------------------------------------------- a-tomo -*/

INLINE VOID CopyCharAdvance (PSZ& pszBuffer, PCSZ& pszSource, INT& nMax)
{
#if defined(DBCS)
    //  If the current character is a lead byte, copy it and advance pointers
    if (IsDBCSLeadByte(*pszSource))
    {
       *pszBuffer++ = *pszSource++;
       nMax--;
    }
#endif
    //  Copy the current character and advance the pointers
    *pszBuffer++ = *pszSource++;
    nMax--;
}


INLINE HFILE ToHFILE (HANDLE hFile)
{
    return ((HFILE)hFile);
}

#endif	//	defined(__cplusplus)

#endif  //  !defined(_COMMON_INL)
