// lookup.cpp : implementation file
//

#include "stdafx.h"
//#define USE_TASK
//#include "common.h"
//#include "task.h"
#include "TrlLookup.h"
#include "lookup.h"

//  WordNet-specific includes
extern "C" {
    #include "wn.h"
    #include "wnconsts.h"
    #include "wnglobals.h"
    }

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW

CLookup::CLookup() :
    nBUFLEN(SHRT_MAX - 1),
    buffer("")
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CLookup()\n"));
    m_nCursorID = 0;
    //m_nTimerID = ::SetTimer(NULL, 1, 1000, NULL);   // 1sec timer for LookUp synchronization
    LoadProfileSettings();
    
    //  Initialize the DLL for accessing the Software Toolwork's Reference Library
    if (m_bUseTRL) {
        HINSTANCE hInst = AfxGetApp()->m_hInstance;
        HWND hMainWnd = AfxGetApp()->m_pMainWnd ? AfxGetApp()->m_pMainWnd->m_hWnd : NULL;
        m_pLI = ::InitLookup(hInst, hMainWnd, "Lookup");
        ASSERT(m_pLI);
        m_bUseTRL = (m_pLI != NULL);
        }

    //  Initialize WordNet support
    //m_bUseWordNet = TRUE;
#if FALSE    
    if (m_bUseWordNet && wninit()) {
        DEBUG_OUT(TL_BASIC, ("wn: error - cannot open WordNet database\n"));
        m_bUseWordNet = FALSE;
        }
#endif
}

CLookup::~CLookup()
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::~Lookup()\n"));
    //  Shutdown the TRL search engine
    //::KillTimer(NULL, m_nTimerID);
    if (m_bUseTRL) {
        ASSERT(m_pLI);
        HINSTANCE hInst = AfxGetApp()->m_hInstance;
        ::EndLookup(hInst, m_pLI);
        m_pLI = NULL;
        }

    //  Shutdown the WordNet search engine
    if (m_bUseWordNet) {
        //  nothing to do
        }
}

void CLookup::DeriveSuggestions (CStringx& clue)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DeriveSuggestions('%s')\n", F(clue)));
    CStringx phrase = clue.Trim();

    CStringArray words;
    TokenizeText(words, phrase, ", ;.");
    suggestions = "";
    for (INT i = 0; i < words.GetSize(); i++) {
        CStringx word (words[i]);
        GetSynonyms(word, FALSE);
        suggestions += MatchPhrases(buffer, pattern, ';');
        GetDefinition(word, FALSE);
        suggestions += MatchPhrases(buffer, pattern, " ;");
        GetReferences(word, FALSE);
        suggestions += MatchPhrases(buffer, pattern, ';');
        }
}


inline CString SkipSpaces (CONST CString& text)
{
	INT i = 0;
	SkipSpaces(text, i);
	CString sRest = text.Mid(i);
	return (sRest);
}


CStringx CLookup::MatchPhrases (CStringx& buffer, CStringx& pattern, CONST CHAR cDelim)
{
	DEBUG_OUT(TL_GARRULOUS, ("CLookup::MatchPhrases(%x, %s, %c)\n", 
			  buffer, F1(pattern), cDelim));
    CStringx delims (cDelim);
	return (MatchPhrases (buffer, pattern, delims));
}

CStringx CLookup::MatchPhrases (CStringx& buffer, CStringx& pattern, CONST CStringx& delims)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::MatchPhrases(%lx, '%s', '%s')\n", &buffer, F(pattern), delims));
    CStringx result = "";
//  INT nEnd;

    CStringArray phrases;
    TokenizeText(phrases, buffer, delims);
    for (INT i = 0; i < phrases.GetSize(); i++) {
		CStringx phrase (SkipSpaces(phrases[i]));
#if TRUE
		//	The cross-references have leading & trailing spaces which must be ignored
        phrase = phrase.Trim();
#endif
        if (Match(phrase, pattern)) {
//// convert result list to a string array
            result += phrase + "; ";
            }
        }
//  PurgeStringArray(phrases);

    return (result);
}

//  Match the mixed-case text against the pattern (uppercase assumed)
//
BOOL CLookup::Match (CStringx& text, CStringx& pattern)
{
	DEBUG_OUT(TL_GARRULOUS, ("CLookup::Match('%s', '%s')\n", F1(text), F2(pattern)));
    if (text.GetLength() != pattern.GetLength()) {
        return (FALSE);
        }

    for (INT nPos = 0; nPos < pattern.GetLength(); nPos++) {
        if ((ToUpper(text[nPos]) != pattern[nPos]) && (pattern[nPos] != '?')) {
            return (FALSE);
            }
        }

    return (TRUE);
}


INLINE BOOL MyIsPrintable (CHAR c)  { return ((c >= ' ') && (c <= '~')); }
INLINE BOOL MyIsSpace (CHAR c)		{ return ((c == ' ') || (c == '\r') || (c == '\n') || (c == '\t')); }
CONST CHAR cDOT = '.';

CONST CHAR cWIDE_DASH = 	(CHAR)0xC4;
CONST BYTE byWIDE_DASH = 	0xC4;
CONST CHAR cRETURN =		'\r';
CONST CHAR cNEW_LINE =		'\n';
CONST CHAR cDASH =			'-';

CONST CHAR szRTF_HEADER[] = 	"{\\rtf1\\ansi\\deff0\\deftab720"
							    	"{\\fonttbl{\\f0\\fnil MS Sans Serif;}"
							    		  "{\\f1\\fnil\\fcharset2 Symbol;}"
							    		  "{\\f2\\fswiss\\fprq2 System;}}"
									"{\\colortbl\\red0\\green0\\blue0;}"
									"\\deflang1033\\pard\\plain"
									"\\f0\\fs20"
									"\\tx360\\li360\\fi-360";	// 1/4" hanging indent
CONST CHAR szRTF_TRAILER[] = 	"}";
CONST CHAR szRTF_START_BOLD[] =		"\\b ";
CONST CHAR szRTF_END_BOLD[] =		"\\b0 ";
CONST CHAR szRTF_START_ITALICS[] =	"\\i ";
CONST CHAR szRTF_END_ITALICS[] =	"\\i0 ";
CONST CHAR szRTF_NEW_PARA[] =		"\\par ";
CONST CHAR szRTF_TAB[] =			"\\tab ";

CONST CHAR cSGML_TAG_START =	'<';
CONST CHAR cSGML_TAG_END =		'>';


INLINE BOOL IsSgmlTag (const CStringx& sText)
{
	BOOL bIsTag = FALSE;
	INT nLen = sText.GetLength();
	if (nLen == 0) {
		bIsTag = FALSE;
		}
	else if ((sText[0] == cSGML_TAG_START) && (sText[nLen - 1] == cSGML_TAG_END))
		{
		bIsTag = TRUE;
		}

	return (bIsTag);
}

CONST CHAR cSEMI_COLON = ';';

//	Helper function for ConvertLookupSgmlToRtf. SGML tags are converted
//	into their RTF representations.
//
VOID ProcessSgmlTag (CString& sLookupText, CONST CString& sTag, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_GARRULOUS, ("ProcessLinePrefixStuff(%x, %s, %d)\n",
							 sLookupText, F(sTag), eType));
	if (sTag == SGML_START_TAG(HEADWORD)) {
		sLookupText += szRTF_START_BOLD;
		}
	else if (sTag == SGML_END_TAG(HEADWORD)) {
		sLookupText += szRTF_END_BOLD;
		}
	else if (sTag == SGML_START_TAG(BOLD)) {
		sLookupText += szRTF_START_BOLD;
		}
	else if (sTag == SGML_END_TAG(BOLD)) {
		sLookupText += szRTF_END_BOLD;
		}
	else if (sTag == SGML_START_TAG(ITALICS)) {
		sLookupText += szRTF_START_ITALICS;
		}
	else if (sTag == SGML_END_TAG(ITALICS)) {
		sLookupText += szRTF_END_ITALICS;
		}
	else if (sTag == SGML_START_TAG(PARA)) {
		if (eType != eDEFINITION) {
			sLookupText += cSEMI_COLON;
			}
		}
	else {
		//	Ignore other SGML tags
		}
}


//	Another helper function for ConvertLookupSgmlToRtf. This one
//	checks for numeric sense delimiter (eg, '3.'), which are made
//	boldface and part of speech labels (eg, 'adv.'), which are
//	made boldface and italicized.
//
VOID ProcessLinePrefixStuff (CStringx& sLookupText, CONST CStringx& sLine, CStringx& sRemLine, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_GARRULOUS, ("ProcessLinePrefixStuff(%x, %s, %x, %d)\n",
						 	 sLookupText, F(sLine), sRemLine, eType));
	INT nDotOffset = sLine.Find(cDOT);
	INT nRemPos = 0;
	if (nDotOffset != nINVALID_INDEX) {
		CONST CHAR szTYPES[] = "adj adv n vi vt interj";
		Assert(nDotOffset != 0);
		CStringx sToken(sLine, nDotOffset);
		// TODO: have an inplace TrimLeading() to avoid confusion?
		CStringx sTrimToken = sToken.TrimLeading();
		if (IsNumeric(sTrimToken)) {
			sLookupText += cSPACE;
			sLookupText += szRTF_START_BOLD;
			sLookupText += sTrimToken + cDOT;
			sLookupText += szRTF_END_BOLD;
			//sLookupText += szRTF_TAB;
			nRemPos += 1 + sToken.GetLength();
			}
		else if (Pos(szTYPES, sTrimToken + cSPACE) != nINVALID_INDEX) {
			sLookupText += cSPACE;
			sLookupText += szRTF_START_BOLD;
			sLookupText += szRTF_START_ITALICS;
			sLookupText += sTrimToken + cDOT;
			sLookupText += szRTF_END_ITALICS;
			sLookupText += szRTF_END_BOLD;
			nRemPos += 1 + sToken.GetLength();
			}
		}

	sRemLine += ((PCSZ)sLine + nRemPos);
}


//	Convert the SGML lookup buffer into RTF. SGML tags are replaced
//	with the appropriate RTF format codes, and extraneous spacing is 
//	ignored. In addition, sense numbers and parts of speach are 
//	highlighted.
//
CString ConvertLookupSgmlToRtf (PCSZ pszText, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_VERBOSE, ("ConvertLookupSgmlToRtf('%s', %d)\n", F(pszText), eType));
	CStringx sLookupText (szRTF_HEADER);
	CStringx sCRNL ("\r\n");
	CStringx sText (pszText);
	INT i = 0;

	//	Read the Lookup results buffer line by line, checking for
	//	SGML tags (each is on a separate line).
	BOOL bNeedsSpace = FALSE;
	while (i < sText.GetLength()) {
		BOOL bFirstLine = (i == 0);
		CStringx sLine = ExtractToken(sText, i, sCRNL);
		sLine = sLine.TrimLeading();
		if (IsSgmlTag(sLine)) {
			ProcessSgmlTag(sLookupText, sLine, eType);
			}
		else {
			if (bNeedsSpace) {
				sLookupText += cSPACE;
				}
			CStringx sRemLine;
			ProcessLinePrefixStuff(sLookupText, sLine, sRemLine, eType);
			sRemLine.Trim();
			sLookupText += sRemLine;
			bNeedsSpace = (sRemLine.GetLength() > 0);	// for next line
			}
		}
	
	sLookupText += szRTF_TRAILER;

	return (sLookupText);
}


//	Convert the SGML lookup buffer into straight text. SGML tags are ignored.
//
CString ConvertLookupSgmlToText (PCSZ pszText, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_VERBOSE, ("ConvertLookupSgmlToText('%s', %d)\n", F(pszText), eType));
	CStringx sLookupText;
	CStringx sCRNL ("\r\n");
	CStringx sText (pszText);
	INT i = 0;

	//	Read the Lookup results buffer line by line, checking for
	//	SGML tags (each is on a separate line).
	BOOL bNeedsSpace = FALSE;
	while (i < sText.GetLength()) {
		BOOL bFirstLine = (i == 0);
		CStringx sLine = ExtractToken(sText, i, sCRNL);
		sLine = sLine.TrimLeading();
		if (!IsSgmlTag(sLine)) {
			sLookupText += sLine;
			}
		else if (sLine == SGML_START_TAG(PARA)) {
			if (eType != eDEFINITION) {
				sLookupText += cSEMI_COLON;
				}
			}
		}
	
	return (sLookupText);
}


//	Filter out garbage from the TRL lookup. In addition, 'format' the text into
//	lines.
//
CString WayUncoolFilterLookupGarbage (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("WayUncoolFilterLookupGarbage(%x)\n", pszText));
    CString sText;

	CHAR cLast = cSPACE;	// pretend last charcater was a space (to ignore leading spaces)
    while (*pszText) {
		CHAR cCurrent = *pszText;
		if (MyIsSpace(cCurrent)) {
			//	See if a newline sequence (\r\n) is acceptable.
			//		new sense number (3.) or new sense type (adj.)
			if (cCurrent == cRETURN) {
				INT nDotOffset = Pos(pszText, cDOT);
				if (nDotOffset != nINVALID_INDEX) {
					CONST CHAR szTYPES[] = "adj adv n vi vt interj";
					Assert(nDotOffset != 0);
					CStringx sToken(pszText, nDotOffset);
					// TODO: have an inplace TrimLeading() to avoid confusion?
					CStringx sTrimToken = sToken.TrimLeading();
					if (IsNumeric(sTrimToken)) {
						sText += szRTF_NEW_PARA;
						sText += szRTF_START_BOLD;
						sText += sTrimToken + cDOT;
						sText += szRTF_END_BOLD;
						sText += szRTF_TAB;
						pszText += sToken.GetLength();
						cCurrent = cSPACE;
						}
					else if (Pos(szTYPES, sTrimToken + ' ') != nINVALID_INDEX) {
						sText += szRTF_NEW_PARA;
						sText += szRTF_START_BOLD;
						sText += szRTF_START_ITALICS;
						sText += sTrimToken + cDOT;
						sText += szRTF_END_ITALICS;
						sText += szRTF_END_BOLD;
						pszText += sToken.GetLength();
						cCurrent = cDOT;
						}
					else {
						sText += cSPACE;
						cCurrent = cSPACE;		// make it a true space
						}
					}
				}
			else if (cCurrent == cNEW_LINE) {
				cCurrent = cSPACE;				// make it a true space
				// skip
				}
			else if (cLast != cCurrent) {
				sText += cCurrent;
				}
			}
		else if (cCurrent == cDASH) {
			//	Convert a double dash sequence into a CR/NL sequence
			if (cLast != cRETURN) {
				cCurrent = cRETURN;
				}
			else {
				cCurrent = cNEW_LINE;
				}
			sText += cCurrent;
			}
        else if (!MyIsPrintable(cCurrent)) {
			//	Convert a double wide-dash sequence into a CR/NL sequence
			if ((BYTE)cCurrent == byWIDE_DASH) {
				if (cLast != cRETURN) {
					cCurrent = cRETURN;
					}
				else {
					cCurrent = cNEW_LINE;
					}
				sText += cCurrent;
				}
			else {
           		//bSkip = TRUE;				// skip nonprinting characters
				}
			}
		else {
			sText += cCurrent;
			}
		cLast = cCurrent;
        pszText++;			// TODO: allow for DBCS
        }

    return (sText);
}


CString FilterWordNetGarbage (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterWordNetGarbage(%x)\n", pszText));
    CString sText;

    while (*pszText) {
        CHAR cNext = *pszText;
        if (cNext == '\n') {
            sText += "\r\n";
            }
        else if (MyIsPrintable(cNext)) {
            sText += cNext;
            }
        pszText++;
        }

    return (sText);
}


void CLookup::LookupClue (CStringx sSource, CStringx type, CStringx phrase, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LookupClue(%s, %s, %s, %d)\n", 
			  F1(sSource), F2(type), F3(phrase), bFormat));
    //  change the mouse cursor to the supplied cursor or the hourglass
    HCURSOR hOldCursor = NULL;
    if (m_nCursorID == 0) {
        AfxGetApp()->BeginWaitCursor();
        }
    else {
        hOldCursor = ::SetCursor(::LoadCursor(AfxGetInstanceHandle(), 
                                              MAKEINTRESOURCE(m_nCursorID)));
        }
    CONST INT nTIME_OUT = 300;
    
    //LoadLookup();
    buffer.Empty();


    LOOKUP_TYPE eType = 
        (type.CompareNoCase("Definition") == 0) ? eDEFINITION
        : (type.CompareNoCase("Synonyms") == 0) ? eSYNONYMS
        :                                         eCROSS_REFS;

    //  Look up the phrase
    if (m_bUseTRL) {
        BOOL bOK = ::LookupPhrase(m_pLI, phrase, sSource, eType);
    
        // Copy the results
        if (bOK) {
            PCSZ pszResults = GetResultsPtr(m_pLI);
            ASSERT(pszResults);
            //buffer += ::FilterLookupGarbage(pszResults);
			buffer +=  (bFormat
						? ConvertLookupSgmlToRtf(pszResults, eType)
						: ConvertLookupSgmlToText(pszResults, eType));
            }
        }

    //  Add info from a WordNet search
    //CONST INT nNUM_PARTS = 4;
    if (m_bUseWordNet && (eType == eSYNONYMS)) {
        if (!wninit()) {
            CString sPhrase = phrase;
            DEBUG_OUT(TL_VERBOSE, ("Initiating WordNet search for %s\n", (PCSZ)sPhrase));
            for (INT nPartOfSearch = NOUN; nPartOfSearch < ADJSAT; nPartOfSearch++) {
                PCSZ pszResults;
                PSZ pszPhrase = (PSZ)(PCSZ)sPhrase;
                pszResults = findtheinfo(pszPhrase, nPartOfSearch, SIMPTR, ALLSENSES);
                ASSERT(pszResults);
                buffer += ::FilterWordNetGarbage(pszResults);
                pszResults = findtheinfo(pszPhrase, nPartOfSearch, HYPERPTR, ALLSENSES);
                ASSERT(pszResults);
                buffer += ::FilterWordNetGarbage(pszResults);
                }
            wnclose();
            }
        else {
            DEBUG_OUT(TL_BASIC, ("wn: error - cannot open WordNet database\n"));
            }
        }

    DEBUG_OUT(TL_VERBOSE, ("Lookup(%s, %s, %s)", 
               F1(sSource), F2(type), F3(phrase)));
    DEBUG_OUT(TL_VERBOSE, (" => %s\n", F(buffer)));
    //AFX_DUMPX(5, buffer);

    //  restore the mouse cursor
    if (m_nCursorID == 0) {
        AfxGetApp()->EndWaitCursor();
        }
    else {
        ::SetCursor(hOldCursor);
        }
}


void CLookup::GetSynonyms (CStringx phrase, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetSynonyms(%s, %d)\n", F(phrase), bFormat));
    LookupClue("THES2", "Synonyms", phrase, bFormat);
}


//	Get the definition for the phrase from the TRL reference. This
//	first checks looks for the first occurrence of a multiple entry
//	phrase (eg, 'cant' has three separate entries) by explicitly
//	specifying the occurence indicator ('<word>{1}' as in 'cant{1}').
//	If this search fails, the word is searched without the indicator.
//	Then if this also fails, the searches are repeated using an
//	alternative index source.
//
//	NOTE: TRL has teo main  indexes for the dictionary (two others
//		are not relevant). One index provides for words used in multiple
//		entries (eg, 'acid test', 'intelligence test', etc.). Another
//		goes directly to a word.
//
void CLookup::GetDefinition (CStringx sPhrase, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetDefinition(%s, %d)\n", F(sPhrase), bFormat));
	CONST CHAR szSOURCE[] = "DICT4";
	CONST CHAR szALT_SOURCE[] = "DICT3";
    CStringx sSource (szSOURCE);
	CStringx sAltSource (szALT_SOURCE);
    if (IsAltPressed()) {
        sSource = szALT_SOURCE;
		sAltSource = szSOURCE;
        }

    //  First try with the sense indicator (eg, TEST{1})
    CString sPhrase1 = sPhrase + "{1}";
    LookupClue(sSource, "Definition", sPhrase1);

	//	If no luck, try with the unadorned word (eg, TEST)
    if (buffer.LastNonblank() == -1) {
	    LookupClue(sSource, "Definition", sPhrase);
		}

	//	If still nothing returned, try the alternative dictionary index
    if (buffer.LastNonblank() == -1) {
        DEBUG_OUT(TL_VERBOSE, ("Retrying alternate definition lookup (%s) using sense indicator: %s\n", 
        		  F1(sAltSource), F2(sPhrase1)));
        LookupClue(sAltSource, "Definition", sPhrase1);
        }
    if (buffer.LastNonblank() == -1) {
        DEBUG_OUT(TL_VERBOSE, ("Retrying alternate lookup (%s): %s\n", 
        		  F1(sAltSource), F2(sPhrase)));
        LookupClue(sAltSource, "Definition", sPhrase);
        }
}


void CLookup::GetReferences (CStringx phrase, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetReferences(%s, %d)\n", F(phrase), bFormat));
    LookupClue("DICT1", "Cross References", phrase, bFormat);
}


BOOL CLookup::LoadLookup (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LoadLookup()\n"));
#if FALSE
    CStringx lookup = "LOOKUP.EXE";
    CStringx path = GetLookupDirName() + "\\";
    CStringx arguments = "/tLOOKUP.TXT /b /&" + path + " "
                        + "/iDICT1 /iDICT3 /iDICT4 /iTHES2";
    
    HMODULE hLookupMod = ::GetModuleHandle(lookup);
    if (!hLookupMod) {
        CStringx CommandLine = path + lookup + " " + arguments;
//// this seems to make the application loose focus
        hLookupMod = (HMODULE)::WinExec(CommandLine, SW_MINIMIZE);
        SetActiveWindow(GetMainHwnd());
        if (hLookupMod < HINSTANCE_ERROR) {
            AfxMessageBox("Unable to load lookup program");
            return (FALSE);
            }
        }

    return (TRUE);
#endif
    return (FALSE);
}


void CLookup::LoadProfileSettings (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LoadProfileSettings()\n"));
//  CString GetProfileString( LPCSTR lpszSection, LPCSTR lpszEntry, LPCSTRlpszDefault = NULL );
    lookup_dir = AfxGetApp()->GetProfileString("Program_Dirs", "Lookup", "\\CWORD");
    CString temp_path = GetTempPath() + "\\";
    lookup_file = temp_path + AfxGetApp()->GetProfileString("Lookup_Interface", "Lookup_File", "LOOKUP.TXT");
    results_file = temp_path + AfxGetApp()->GetProfileString("Lookup_Interface", "Results_File", "RESULTS.TXT");
    m_bUseWordNet = AfxGetApp()->GetProfileInt("Options", "UseWordNet", TRUE);
    m_bUseTRL = AfxGetApp()->GetProfileInt("Options", "UseTRL", TRUE);
}
