//	Copyleft 1993-1996 Tom O'Hara		Lookup source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// lookup.cpp : implementation file
//

#include "stdafx.h"

#include "TrlLookup.h"
#include "MvInfo.h"

#include "resource.h"
#include "CommonLookup.h"
#include "LookupOptions.h"
#include "LookupMain.h"
#include "LookupDll.h"

//  WordNet-specific includes
//	TODO: wrap these in a separate header (eg, WnLookup.h) to avoid C/C++ conflicts
extern "C" {
    #include "wn.h"
    #include "wnconsts.h"
    #include "wnglobals.h"
    }

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW


// more stuff for common.inl

INLINE CHAR FirstChar (CONST CString& s)
{ 
	return (s.IsEmpty() ? cNULL : s[0]);
}

INLINE CHAR LastChar (CONST CString& s)
{ 
	return (s.IsEmpty() ? cNULL : s[s.GetLength() - 1]);
}

INLINE BOOL IsPunct (CHAR c)			{ return (ispunct(c)); }



// TODO: put this in MvLookup.h
typedef PMVINFO (*PFN_MviCreate) (VOID);
typedef PMVINFO (*PFN_MviDelete) (PMVINFO pMvi);
typedef BOOL (*PFN_MviOpenTitle) (PMVINFO pMvi, PCSZ pszTitle);
typedef BOOL (*PFN_MviCloseTitle) (PMVINFO pMvi);
typedef BOOL (*PFN_MviOpenWordWheel) (PMVINFO pMvi, PCSZ pszName);
typedef BOOL (*PFN_MviOpenTopicList) (PMVINFO pMvi, PCSZ pszName);
typedef PCSZ (*PFN_MviLookupPhrase) (PMVINFO pMvi, PCSZ pszPhrase, BOOL bDefinition);
typedef PCSZ (*PFN_MviLookupDefinitionPhrase) (PMVINFO pMvi, PCSZ pszPhrase);
typedef PCSZ (*PFN_MviLookupCrossReferences) (PMVINFO pMvi, PCSZ pszPhrase);
typedef BOOL (*PFN_MviMtOpenTitle) (PMVINFO pMvi, PCSZ pszTitle);
typedef BOOL (*PFN_MviMtCloseTitle) (PMVINFO pMvi);
typedef BOOL (*PFN_MviMtOpenWordWheel) (PMVINFO pMvi, PCSZ pszName);
typedef BOOL (*PFN_MviMtOpenTopicList) (PMVINFO pMvi, PCSZ pszName);
typedef PCSZ (*PFN_MviMtLookupPhrase) (PMVINFO pMvi, PCSZ pszPhrase, BOOL bDefinition);
typedef PCSZ (*PFN_MviMtLookupDefinitionPhrase) (PMVINFO pMvi, PCSZ pszPhrase);
typedef PCSZ (*PFN_MviMtLookupCrossReferences)  (PMVINFO pMvi, PCSZ pszReferences);
typedef PMVLOOKUP (*PFN_MvlCreate) (PMVINFO pMvi);
typedef PMVLOOKUP (*PFN_MvlDelete) (PMVLOOKUP pMvl);
typedef PCSZ (*PFN_MvlLookupPhrase) (PMVLOOKUP pMvl, PCSZ pszPhrase, BOOL bDefinition);
typedef PCSZ (*PFN_MvlLookupDefinitionPhrase) (PMVLOOKUP pMvl, PCSZ pszPhrase);
typedef PCSZ (*PFN_MvlLookupCrossReferences) (PMVLOOKUP pMvl, PCSZ pszPhrase);
typedef VOID (*PFN_MvlUseSgml) (PMVLOOKUP pMvl, BOOL bUseSgml);
typedef BOOL (*PFN_MvlIsValid) (PMVLOOKUP pMvl);
typedef PCSZ (*PFN_MvlMtLookupPhrase) (PMVLOOKUP pMvl, PCSZ pszPhrase, BOOL bDefinition);
typedef PCSZ (*PFN_MvlMtLookupDefinitionPhrase) (PMVLOOKUP pMvl, PCSZ pszPhrase);
typedef PCSZ (*PFN_MvlMtLookupCrossReferences)  (PMVLOOKUP pMvl, PCSZ pszReferences);

// TODO: put this in WnLookup.h
typedef INT (*PFN_wninit) (VOID);
typedef INT (*PFN_wnclose) (VOID);
typedef PCSZ (*PFN_findtheinfo) (PCSZ, INT, INT, INT);

// TODO: put in TrlLoookup.h
typedef PLOOKUP_INFO (* PFN_InitLookup)      
		(HINSTANCE hInst, HWND hWndFrame, PCSZ pszClientAppName, PCSZ pszDataDir, PCSZ pszIndexDir);
typedef BOOL (* PFN_EndLookup)       (HINSTANCE hInst, PLOOKUP_INFO pLI);
typedef BOOL (* PFN_AddSource)       (PLOOKUP_INFO pLI, PCSZ pszSource);
typedef BOOL (* PFN_LookupPhrase)    (PLOOKUP_INFO pLI, PCSZ pszPhrase, PCSZ pszSource, LOOKUP_TYPE eType);
typedef PCSZ (* PFN_GetResultsPtr)   (PLOOKUP_INFO pLI);


//      Constants
//
//CONST CHAR cWIDE_DASH =       (CHAR)0xC4;
CONST BYTE byWIDE_DASH =        0xC4;
CONST CHAR szENTRY_SEP[] =      "; ";
CONST CHAR szTHESAURUS[] =		"Roget's Thesaurus";
CONST CHAR szDICTIONARY[] =		"Dictionary";

CONST CHAR szBOOKSHELF_NOT_FOUND[] = "Unable to locate Bookshelf file: %s\nReconfigure if moved.";
CONST CHAR szINVALID_LOOKUP_DLL[] =  "Missing or invalid DLL (%s).\n%s access will be disabled";
CONST CHAR szMV_LOOKUP[] =	"MvLookup.dll";
CONST CHAR szBOOKSHELF[] =	"Bookshelf";
CONST CHAR szWN_LOOKUP[] =	"WnLookup.dll";
CONST CHAR szWORDNET[] =	"WordNet";
CONST CHAR szTRL_LOOKUP[] =	"TrlLookup.dll";
CONST CHAR szTRL[] =		"Toolworks Reference Library";

// Globals
//
BOOL bUseThreads = FALSE;
// MediaView access
DEFINE_LIBRARY(MvLookup);
DEFINE_FUNCTION(MviCreate);
DEFINE_FUNCTION(MviDelete);
DEFINE_FUNCTION(MviOpenTitle);
DEFINE_FUNCTION(MviCloseTitle);
DEFINE_FUNCTION(MviOpenWordWheel);
DEFINE_FUNCTION(MviOpenTopicList);
DEFINE_FUNCTION(MviLookupPhrase);
DEFINE_FUNCTION(MviLookupDefinitionPhrase);
DEFINE_FUNCTION(MviLookupCrossReferences);
DEFINE_FUNCTION(MviMtOpenTitle);
DEFINE_FUNCTION(MviMtCloseTitle);
DEFINE_FUNCTION(MviMtOpenWordWheel);
DEFINE_FUNCTION(MviMtOpenTopicList);
DEFINE_FUNCTION(MviMtLookupPhrase);
DEFINE_FUNCTION(MviMtLookupDefinitionPhrase);
DEFINE_FUNCTION(MviMtLookupCrossReferences);
DEFINE_FUNCTION(MvlCreate);
DEFINE_FUNCTION(MvlDelete);
DEFINE_FUNCTION(MvlLookupPhrase);
DEFINE_FUNCTION(MvlLookupDefinitionPhrase);
DEFINE_FUNCTION(MvlLookupCrossReferences);
DEFINE_FUNCTION(MvlUseSgml);
DEFINE_FUNCTION(MvlIsValid);
DEFINE_FUNCTION(MvlMtLookupPhrase);
DEFINE_FUNCTION(MvlMtLookupDefinitionPhrase);
DEFINE_FUNCTION(MvlMtLookupCrossReferences);
// WordNet access
DEFINE_LIBRARY(WnLookup);
DEFINE_FUNCTION(wninit);
DEFINE_FUNCTION(wnclose);
DEFINE_FUNCTION(findtheinfo);
// TRL access
DEFINE_LIBRARY(TrlLookup);
DEFINE_FUNCTION(InitLookup);
DEFINE_FUNCTION(EndLookup);
DEFINE_FUNCTION(AddSource);
DEFINE_FUNCTION(LookupPhrase);
DEFINE_FUNCTION(GetResultsPtr);

//      Local functions
//
STATIC UINT DoInitProc (PVOID pData);
STATIC UINT DoLookupProc (CLookup* pLookup);
STATIC UINT DoDefLookupProc (CLookup* pLookup);
STATIC UINT DoSuggestionsProc (CLookup* pLookup);


#if FALSE
void EXPORT CALLBACK TimerProc(
	HWND hWnd,              // handle of CWnd that called SetTimer
	UINT nMsg,              // WM_TIMER
	UINT nIDEvent,  // timer identification
	DWORD dwTime    // system time
);
#endif


BOOL PostMessageToFrame(UINT message, WPARAM wParam = 0, LPARAM lParam = 0)
{
	CWnd* pFrame = AfxGetApp()->m_pMainWnd;
	BOOL bOK = pFrame->PostMessage(message, wParam, lParam);

	return (bOK);
}

CONST CHAR szSTOP_WORDS[] = ";a;also;an;and;any;are;as;at;but;by;does;of;or;for;from;into;
							"is;not;one;some;the;that;these;they;"
                            "this;through;time;to;up;used;very;what;which;who;with;";

// Determine if the word is a common english function word or other type of "stop-list" word.
//
BOOL StopWord (CONST CString& sWord)
{
	BOOL bStopWord = FALSE;

	INT nPos = FindStringIgnoreCase(szSTOP_WORDS, sWord);
	if (nPos != nINVALID_INDEX) {
		bStopWord = ((nPos > 0) && (szSTOP_WORDS[nPos - 1] == cSEMI_COLON) 
					  		    && (szSTOP_WORDS[nPos + sWord.GetLength()] == cSEMI_COLON));
		}

	return (bStopWord);
}


// Insert the word into the list to maintain nonduplicated sort-order 
// NOTE: Stop words are omitted.
//
void InsertUnique (CStringArray& saList, CONST CString& sItem)
{
	// Ignore common function words, etc
	if (StopWord(sItem)) {
		return;
		}

	// Find the position at which to insert the word
	INT nNewPos = saList.GetSize();
    for (INT i = 0; i < saList.GetSize(); i++) {
		INT nCompare = sItem.CompareNoCase(saList[i]);
		if (nCompare == 0) {
			nNewPos == nINVALID_INDEX;
			break;
			}
		else if (nCompare < 0) {
			nNewPos = i;
			break;
			}
		}

	// Add the new word (unless if a duplicate)
	if (nNewPos != nINVALID_INDEX) {
		saList.InsertAt(nNewPos, sItem);
		}
}


//-----------------------------------------------------------------------------
// CLookup class implementation
//
// Mondo dispatch/consolation class for doing lookup's with various sources
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC(CLookup, CObject);

BOOL CLookup::m_bUseTRL = FALSE;
BOOL CLookup::m_bUseWordNet = FALSE;
BOOL CLookup::m_bUseBookshelf = FALSE;
//CMvInfo* CLookup::m_pMvInfo = NULL;
PMVINFO CLookup::m_pMvInfo = NULL;
//CMvLookup* CLookup::m_pMvLookup = NULL;
PMVLOOKUP CLookup::m_pMvLookup = NULL;
PLOOKUP_INFO CLookup::m_pTrl = NULL;
CWinThread* CLookup::m_pInitThread = NULL;
CWnd* CLookup::m_pStaticParentWnd = NULL;
DWORD CLookup::m_dwTimeOut = 0;

CLookup::CLookup(CWnd* pClient /* = NULL */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CLookup()\n"));

	// Pointer initialization; be careful w/ these
	m_hClientWnd = pClient ? pClient->m_hWnd : NULL;
	m_pLookupThread = NULL;

	// Make sure the initialization thread has completed.
	// Shut down the app if it still doesn't terminate after
	// a two minute time out (with user permission of course)
	CWaitCursor waitCursor;
	BOOL bDone = FALSE;
#if TRUE
	bDone = (bUseThreads == FALSE) || (m_pInitThread == NULL);
#endif
	while (!bDone) {
		//ASSERT(m_pInitThread && ValidHandle(m_pInitThread->m_hThread));
		DWORD dwEvent = WaitForSingleObject(m_pInitThread->m_hThread, m_dwTimeOut * 1000);
		bDone = TRUE;
		if (dwEvent == WAIT_TIMEOUT) {
			CONST CHAR szINIT_TIMEOUT[] = "Timeout during reference source initialization.\nPlease check the cofiguration";
			if (IDRETRY != ReportAppErrorFmt(szINIT_TIMEOUT)) {
				bDone = FALSE;
				}
			}
		else if (dwEvent == WAIT_FAILED) {
			DEBUG_OUT(TL_BASIC, ("Error %lu in WaitForSingleObject\n", GetLastError()));
			}
		}
#if TRUE
	m_pInitThread = NULL;
#endif

#if FALSE
	m_pMvLookup = NULL;
	if (m_bUseBookshelf) {
		ASSERT(m_pMvInfo);
		m_pMvLookup = new CMvLookup(m_pMvInfo);
		}
#endif
}


CLookup::~CLookup()
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::~Lookup()\n"));

	if (m_pLookupThread) {
		AbortLookup();
		}

#if FALSE
	if (m_pMvLookup) {
		delete m_pMvLookup;
		m_pMvLookup = NULL;
		}
#endif
}


// InitLookup: Initialize the static portions of the class. This is done
// via a separate thread due to the quirky MvInfo interface and for
// efficiency (from the user's point of view)
//
BOOL CLookup::InitLookup (CWnd* pWnd, BOOL bReinit /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitLookup(%x)\n", pWnd));
	BOOL bOK = FALSE;

	m_pStaticParentWnd = pWnd;
	bUseThreads = GetDebugOption("Multithreaded", TRUE);
	if (bUseThreads) {
		m_pInitThread = AfxBeginThread((AFX_THREADPROC)::DoInitProc, &bReinit, THREAD_PRIORITY_HIGHEST);
		bOK = (m_pInitThread != NULL);
		}
	else {
		bOK = DoInitProc(&bReinit);
		}

	return (bOK);
}


BOOL CLookup::ReinitLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::ReinitLookup()\n"));

	return (InitLookup(m_pStaticParentWnd, TRUE));
}


// InitLookupAux: Initialize the information sources that will be used
//
// NOTE: This is done in a separate thread from the main one so no CWnd 
//		 based objects should be saved  (due to thread-local storage HWND maps). 
//		 This is due to quirks with using MediaView from separate threads. 
//
// TODO: straighten out thread usage
//
BOOL CLookup::InitLookupAux (BOOL bReinit /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitLookupAux(%d)\n", bReinit));

    LoadProfileSettings();
    
    // Initialize the DLL for accessing the Software Toolwork's Reference Library
    if (m_bUseTRL) {
		InitTrl(bReinit);
		}

    // Initialize WordNet support
    if (m_bUseWordNet) {
		InitWordNet(bReinit);
		}

	// Initialize the MediaView support DLL for access to Bookshelf
	if (m_bUseBookshelf) {         // make sure instance exists???
		InitBookshelf(bReinit);
		}

	return (TRUE);
}


void CLookup::CleanupLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupLookup()\n"));

    //  Shutdown the TRL search engine
    if (m_bUseTRL) {
		CleanupTrl();
		}

    //  Shutdown the WordNet search engine
    if (m_bUseWordNet) {
		CleanupWordNet();
		}

	// Shutdown the MediaView search engine
	if (m_bUseBookshelf) {
		CleanupBookshelf();
		}
}


CONST CHAR szNO_REFERENCE_SOURCES[] = "There are no active reference sources";

void CLookup::LoadProfileSettings (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LoadProfileSettings()\n"));

    m_bUseWordNet = GetAppOption(szUSE_WORDNET, FALSE);
    m_bUseTRL = GetAppOption(szUSE_TRL, FALSE);
    m_bUseBookshelf = GetAppOption(szUSE_BOOKSHELF, FALSE);
	m_dwTimeOut = GetAppOption("TimeOut", 120);		// default to 120 seconds

#if TRUE
	if (!m_bUseWordNet && !m_bUseTRL && !m_bUseBookshelf) {
		ReportAppErrorFmt(szNO_REFERENCE_SOURCES);
		}
#endif
}


// InitSimpleLookup: Initialize for a basic lookup (eg, no multithreading)
//
BOOL CLookup::InitSimpleLookup (BOOL bUseTRL, BOOL bUseWordNet, BOOL bUseBookshelf)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitSimpleLookup(%d, %d, %d)\n", 
			  bUseTRL, bUseWordNet, bUseBookshelf));
	m_bUseTRL = bUseTRL;
	m_bUseWordNet = bUseWordNet;
	m_bUseBookshelf = bUseBookshelf;

    // Initialize the DLL for accessing the Software Toolwork's Reference Library
    if (m_bUseTRL) {
		InitTrl();
		}

    // Initialize WordNet support
    if (m_bUseWordNet) {
		InitWordNet();
		}

	// Initialize the MediaView support DLL for access to Bookshelf
	if (m_bUseBookshelf) {		// make sure instance exists???
		InitBookshelf();
		}

	return (TRUE);
}


// CleanupSimpleLookup: Cleanup after a basic lookup
//
void CLookup::CleanupSimpleLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupSimpleLookup()\n"));
	CLookup::CleanupLookup();
}


VOID CLookup::InitBookshelf (BOOL bReinit /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitBookshelf(%d)\n", bReinit));

	// First make sure an instance doesn't already exist
	if (bReinit) {
		CleanupBookshelf();
		}

	// Load the MediaLookup wrapper DLL (MvLookup)
	// TODO: verify this in the Lookup options dialog handler
	BOOL bOK = ResolveMediaViewFunctions(); 
	if (!bOK) {
		m_bUseBookshelf = FALSE;
		return;
		}

#if FALSE
	// NOTE: The existing instance is kept if reinitialing since there might be 
	//		 active MvLookup classes refererring to it
	// TODO: cleanup the CMvInfo/CMvLookup interface
#endif

	//m_pMvInfo = new CMvInfo();
	m_pMvInfo = pMviCreate();
	ASSERT(m_pMvInfo);

	// Get the file name of the title (default to Bookshelf)
	//CString sBookshelfDir = GetAppOption(szBOOKSHELF_DIR, szDEF_BOOKSHELF_DIR);
	//AddPathDelim(sBookshelfDir);
	//CString sFile = GetAppOption(szBOOKSHELF_FILE, szDEF_BOOKSHELF_FILE);
	//CString sTitle = sBookshelfDir + sFile;
	CString sTitle;
	CString sBookshelfDir = GetAppOption(szBOOKSHELF_DIR, szDEF_BOOKSHELF_DIR);
	bOK = CheckBookshelfTitle(sBookshelfDir, sTitle);
	if (!bOK || !m_pMvInfo) {
		m_bUseBookshelf = FALSE;
		CleanupBookshelf();
		}
	else {
		//m_pMvInfo->MtOpenTitle(sTitle);
		pMviMtOpenTitle(m_pMvInfo, sTitle);
		//m_pMvInfo->MtOpenWordWheel(szDICTIONARY);
		pMviMtOpenWordWheel(m_pMvInfo, szDICTIONARY);
		//m_pMvInfo->MtOpenTopicList(szDICTIONARY);
		pMviMtOpenTopicList(m_pMvInfo, szDICTIONARY);
		ASSERT(m_pMvInfo);
		ASSERT(!m_pMvLookup);
		//m_pMvLookup = new CMvLookup(m_pMvInfo);
		m_pMvLookup = pMvlCreate(m_pMvInfo);
		}
}
		

VOID CLookup::CleanupBookshelf (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupBookshelf()\n"));

	if (m_pMvInfo) {
		//delete m_pMvInfo;
		pMviDelete(m_pMvInfo);
		m_pMvInfo = NULL;
		}
	if (m_pMvLookup) {
		//delete m_pMvLookup;
		pMvlDelete(m_pMvLookup);
		m_pMvLookup = NULL;
		}
	UNLOAD_LIBRARY(MvLookup);
}


BOOL CLookup::ResolveMediaViewFunctions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::ResolveMediaViewFunctions()\n"));
	BOOL bOK = FALSE;

	while (!bOK) {
		try {
			LOAD_LIBRARY(MvLookup);
			LOAD_FUNCTION(MvLookup, MviCreate);
			LOAD_FUNCTION(MvLookup, MviDelete);
			LOAD_FUNCTION(MvLookup, MviOpenTitle);
			LOAD_FUNCTION(MvLookup, MviCloseTitle);
			LOAD_FUNCTION(MvLookup, MviOpenWordWheel);
			LOAD_FUNCTION(MvLookup, MviOpenTopicList);
			LOAD_FUNCTION(MvLookup, MviLookupPhrase);
			LOAD_FUNCTION(MvLookup, MviLookupDefinitionPhrase);
			LOAD_FUNCTION(MvLookup, MviLookupCrossReferences);
			LOAD_FUNCTION(MvLookup, MviMtOpenTitle);
			LOAD_FUNCTION(MvLookup, MviMtCloseTitle);
			LOAD_FUNCTION(MvLookup, MviMtOpenWordWheel);
			LOAD_FUNCTION(MvLookup, MviMtOpenTopicList);
			LOAD_FUNCTION(MvLookup, MviMtLookupPhrase);
			LOAD_FUNCTION(MvLookup, MviMtLookupDefinitionPhrase);
			LOAD_FUNCTION(MvLookup, MviMtLookupCrossReferences);
			LOAD_FUNCTION(MvLookup, MvlCreate);
			LOAD_FUNCTION(MvLookup, MvlDelete);
			LOAD_FUNCTION(MvLookup, MvlLookupPhrase);
			LOAD_FUNCTION(MvLookup, MvlLookupDefinitionPhrase);
			LOAD_FUNCTION(MvLookup, MvlLookupCrossReferences);
			LOAD_FUNCTION(MvLookup, MvlUseSgml);
			LOAD_FUNCTION(MvLookup, MvlIsValid);
			LOAD_FUNCTION(MvLookup, MvlMtLookupPhrase);
			LOAD_FUNCTION(MvLookup, MvlMtLookupDefinitionPhrase);
			LOAD_FUNCTION(MvLookup, MvlMtLookupCrossReferences);
			bOK = TRUE;
			}
		catch (...) {
			DEBUG_OUT(TL_ERROR, ("Trouble loading MvLookup!\n"));
			}
		if (!bOK) {
			UNLOAD_LIBRARY(MvLookup);
			INT nResponse = ReportAppErrorFmt(szINVALID_LOOKUP_DLL, szMV_LOOKUP, szBOOKSHELF);
			if (nResponse == IDIGNORE) {
				break;
				}
			}
		}

	return (bOK);
}


BOOL CLookup::CheckBookshelfTitle (PCSZ pszDir, CString& sTitle)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CheckBookshelfTitle(%s, %x)\n", 
			  F1(pszDir), sTitle));
	BOOL bOK = FALSE;

	// Determine the full pathname for the title
	sTitle = pszDir;
	AddPathDelim(sTitle);
	CString sFile = GetAppOption(szBOOKSHELF_FILE, szDEF_BOOKSHELF_FILE);
	sTitle += sFile;

	// See if it exists, giving the user a chance to insert the CD, etc.
	BOOL bKeepTrying = TRUE;
	while (bKeepTrying) {
		if (FileExists(sTitle)) {
			bOK = TRUE;
			break;
			}
		INT nResponse = ReportAppErrorFmt(szBOOKSHELF_NOT_FOUND, (PCSZ)sTitle);
		bKeepTrying = (nResponse == IDRETRY);
		}

	return (bOK);
}


VOID CLookup::InitWordNet (BOOL bReinit /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitWordNet(%d)\n", bReinit));
	UNREFERENCED_PARAMETER(bReinit);
	m_bUseWordNet = FALSE;

	ResolveWordNetFunctions();
	if (!pwninit()) {
		m_bUseWordNet = TRUE;
		}
	else {
		m_bUseWordNet = FALSE;
		DEBUG_OUT(TL_BASIC, ("error in wninit: cannot open WordNet database\n"));
		}
}


VOID CLookup::CleanupWordNet (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupWordNet()\n"));

	if (hWnLookup) {
		pwnclose();
		}
	UNLOAD_LIBRARY(WnLookup);
}


BOOL CLookup::ResolveWordNetFunctions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::ResolveWordNetFunctions()\n"));
	BOOL bOK = FALSE;

	while (!bOK) {
		try {
			LOAD_LIBRARY(WnLookup);
			LOAD_FUNCTION(WnLookup, wninit);
			LOAD_FUNCTION(WnLookup, wnclose);
			LOAD_FUNCTION(WnLookup, findtheinfo);
			bOK = TRUE;
			}
		catch (...) {
			DEBUG_OUT(TL_ERROR, ("Trouble loading WnLookup!\n"));
			}
		if (!bOK) {
			UNLOAD_LIBRARY(WnLookup);
			INT nResponse = ReportAppErrorFmt(szINVALID_LOOKUP_DLL, szWN_LOOKUP, szWORDNET);
			if (nResponse == IDIGNORE) {
				break;
				}
			}
		}

	return (bOK);
}


VOID CLookup::InitTrl (BOOL bReinit /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitTrl(%d)\n", bReinit));
	UNREFERENCED_PARAMETER(bReinit);

	// First make sure an instance doesn't already exist
	CleanupTrl();

	// Dynamically link to the DLL
	if (!ResolveTrlFunctions()) {
		m_bUseTRL = FALSE;
		return;
		}

	// Get the user options for the TRL data directory & (BNX) index driectory
	HINSTANCE hInst = AfxGetApp()->m_hInstance;
	HWND hMainWnd = AfxGetApp()->m_pMainWnd ? AfxGetApp()->m_pMainWnd->m_hWnd : NULL;
	CString sDataDirectory(GetAppOption(szTRL_DATA_DIR, szDEF_TRL_DATA_DIR));
	CString sIndexDirectory(GetAppOption(szTRL_INDEX_DIR, GetProgramDir()));
	m_pTrl = pInitLookup(hInst, hMainWnd, "Lookup", sDataDirectory, sIndexDirectory);
	ASSERT(m_pTrl);
	m_bUseTRL = (m_pTrl != NULL);
}


VOID CLookup::CleanupTrl (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupTrl()\n"));

	if (m_pTrl) {
		HINSTANCE hInst = AfxGetApp()->m_hInstance;
		pEndLookup(hInst, m_pTrl);
		m_pTrl = NULL;
		}
	UNLOAD_LIBRARY(TrlLookup);
}


BOOL CLookup::ResolveTrlFunctions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::ResolveTrlFunctions()\n"));
	BOOL bOK = FALSE;

	while (!bOK) {
		try {
			LOAD_LIBRARY_(TrlLookup, "TRLLOO~1.DLL");
			LOAD_FUNCTION(TrlLookup, InitLookup);
			LOAD_FUNCTION(TrlLookup, EndLookup);
			LOAD_FUNCTION(TrlLookup, AddSource);
			LOAD_FUNCTION(TrlLookup, LookupPhrase);
			LOAD_FUNCTION(TrlLookup, GetResultsPtr);
			bOK = TRUE;
			}
		catch (...) {
			DEBUG_OUT(TL_ERROR, ("Trouble loading WnLookup!\n"));
			}
		if (!bOK) {
			UNLOAD_LIBRARY(TrlLookup);
			INT nResponse = ReportAppErrorFmt(szINVALID_LOOKUP_DLL, szTRL_LOOKUP, szTRL);
			if (nResponse == IDIGNORE) {
				break;
				}
			}
		}

	return (bOK);
}


//      Terminate the lookup thread Arnold-style
//      TODO: use events to signal thread to end itself
//
VOID CLookup::AbortLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::AbortLookup()\n"));

	if (m_pLookupThread) {
		ASSERT(ValidHandle(m_pLookupThread->m_hThread));
		TerminateThread(m_pLookupThread->m_hThread, 666);
		m_pLookupThread = NULL;
		}
}


VOID CLookup::MatchPhrases (CStringArray& saResult, const CStringx& buffer, const CStringx& pattern, CONST CHAR cDelim)
{
	DEBUG_OUT(TL_GARRULOUS, ("CLookup::MatchPhrases(%x, %s, %c)\n", 
			  buffer, F1(pattern), cDelim));
    CStringx delims (cDelim);
	MatchPhrases (saResult, buffer, pattern, delims);
}


VOID CLookup::MatchPhrases (CStringArray& saResult, const CStringx& buffer, const CStringx& pattern, CONST CStringx& delims)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::MatchPhrases(%lx, %lx, '%s', '%s')\n", 
						   &saResult, &buffer, F(pattern), delims));
    CStringArray phrases;
    TokenizeText(phrases, buffer, delims);
    for (INT i = 0; i < phrases.GetSize(); i++) {
		CStringx phrase (SkipSpaces(phrases[i]));
		// The cross-references have leading & trailing spaces which must be ignored
		phrase = phrase.Trim();
		if (Match(phrase, pattern)
			&& !IsPunct(FirstChar(phrase)) && !IsPunct(LastChar(phrase))) {
			InsertUnique(saResult, phrase);
			}
		}

}


//  Match the mixed-case text against the pattern (uppercase assumed)
//
BOOL CLookup::Match (const CStringx& text, const CStringx& pattern)
{
	DEBUG_OUT(TL_GARRULOUS, ("CLookup::Match('%s', '%s')\n", F1(text), F2(pattern)));
    if (text.GetLength() != pattern.GetLength()) {
	return (FALSE);
	}

    for (INT nPos = 0; nPos < pattern.GetLength(); nPos++) {
		if ((ToUpper(text[nPos]) != pattern[nPos]) && (pattern[nPos] != '?')) {
			return (FALSE);
			}
		}

    return (TRUE);
}



//      Is the string an SGML tag (eg, a start tag <headword> or an endtag </headword>).
//
INLINE BOOL IsSgmlTag (const CStringx& sText)
{
	BOOL bIsTag = FALSE;
	INT nLen = sText.GetLength();
	if (nLen == 0) {
		bIsTag = FALSE;
		}
	else if ((sText[0] == cSGML_TAG_START) && (sText[nLen - 1] == cSGML_TAG_END))
		{
		bIsTag = TRUE;
		}

	return (bIsTag);
}


//      Helper function for ConvertLookupSgmlToRtf. SGML tags are converted
//      into their RTF representations.
//
VOID ProcessSgmlTag (CString& sLookupText, CONST CString& sTag, LOOKUP_TYPE eType, INT& nExtraSpaces)
{
	DEBUG_OUT(TL_GARRULOUS, ("ProcessSgmlTag(%x, %s, %d, %x)\n",
							 sLookupText, F(sTag), eType, &nExtraSpaces));
	if (sTag == SGML_START_TAG(HEADWORD)) {
		if (eType == eDEFINITION) {
			if (sLookupText.Find(szRTF_START_BOLD) != nINVALID_INDEX) {
				sLookupText += szRTF_NEW_PARA;
				}
			sLookupText += szRTF_START_BOLD;
			}
		}
	else if (sTag == SGML_END_TAG(HEADWORD)) {
		if (eType == eDEFINITION) {
			sLookupText += szRTF_END_BOLD;
			}
		else {
			sLookupText += szENTRY_SEP;
			}
		}
	else if (sTag == SGML_START_TAG(BOLD)) {
		sLookupText += szRTF_START_BOLD;
		}
	else if (sTag == SGML_END_TAG(BOLD)) {
		sLookupText += szRTF_END_BOLD;
		}
	else if (sTag == SGML_START_TAG(ITALICS)) {
		sLookupText += szRTF_START_ITALICS;
		}
	else if (sTag == SGML_END_TAG(ITALICS)) {
		sLookupText += szRTF_END_ITALICS;
		nExtraSpaces += 2;
		}
	else if (sTag == SGML_START_TAG(PARA)) {
		if (eType == eSYNONYMS) {
			sLookupText += szRTF_NEW_PARA;
			}
		else if (eType != eDEFINITION) {
			sLookupText += cSEMI_COLON;
			}
		}
	else if (sTag == SGML_START_TAG(PART_OF_SPEECH)) {
		sLookupText += cSPACE;
		sLookupText += cSPACE;
		sLookupText += szRTF_START_BOLD;
		sLookupText += szRTF_START_ITALICS;
		}
	else if (sTag == SGML_END_TAG(PART_OF_SPEECH)) {
		sLookupText += szRTF_END_ITALICS;
		sLookupText += szRTF_END_BOLD;
		nExtraSpaces += 2;
		}
	else {
		//      Ignore other SGML tags
		}
}


//      Another helper function for ConvertLookupSgmlToRtf. This one
//      checks for numeric sense delimiter (eg, '3.'), which are made
//      boldface and part of speech labels (eg, 'adv.'), which are
//      made boldface and italicized.
//
VOID ProcessLinePrefixStuff (CStringx& sLookupText, CONST CStringx& sLine, CStringx& sRemLine, 
							 LOOKUP_TYPE eType, BOOL& nExtraSpaces)
{
	DEBUG_OUT(TL_GARRULOUS, ("ProcessLinePrefixStuff(%x, %s, %x, %d, %x)\n",
							 sLookupText, F(sLine), sRemLine, eType, &nExtraSpaces));
	UNREFERENCED_PARAMETER(eType);

	INT nDotOffset = sLine.Find(cDOT);
	INT nRemPos = 0;
	if (nDotOffset != nINVALID_INDEX) {
		CONST CHAR szTYPES[] = "adj adv n vi vt interj "; // space delimits entries
		Assert(nDotOffset != 0);
		CStringx sToken(sLine, nDotOffset);
		// TODO: have an inplace TrimLeading() to avoid confusion?
		CStringx sTrimToken = sToken.TrimLeading();
		if (IsNumeric(sTrimToken)) {
			sLookupText += cSPACE;
			sLookupText += cSPACE;
			sLookupText += szRTF_START_BOLD;
			sLookupText += sTrimToken + cDOT;
			sLookupText += szRTF_END_BOLD;
			//sLookupText += szRTF_TAB;
			nRemPos += 1 + sToken.GetLength();
			}
		else if (Pos(szTYPES, sTrimToken + cSPACE) != nINVALID_INDEX) {
			sLookupText += cSPACE;
			sLookupText += cSPACE;
			sLookupText += szRTF_START_BOLD;
			sLookupText += szRTF_START_ITALICS;
			sLookupText += sTrimToken + cDOT;
			sLookupText += szRTF_END_ITALICS;
			sLookupText += szRTF_END_BOLD;
			nExtraSpaces += 2;
			nRemPos += 1 + sToken.GetLength();
			}
		}

	sRemLine += ((PCSZ)sLine + nRemPos);
}


// ToRtfPlainText: Convert the ascii text to RTF plain text. Characters treated
// special in RTF (eg, '{') must be preceded by a backslash ('\').
//
CString ToRtfPlainText (CStringx& sText)
{
	CString sPlainText;
	CONST CHAR szRTF_SPECIAL_CHARS[] = "\\{}";
	CONST CHAR cRTF_ESCAPE = '\\';

	for (INT nPos = 0; nPos < sText.GetLength(); nPos++) {
		if (Pos(szRTF_SPECIAL_CHARS, sText[nPos]) != nINVALID_INDEX) {
			sPlainText += cRTF_ESCAPE;
			}
		sPlainText += sText[nPos];
		}
	DEBUG_OUT(TL_GARRULOUS, ("ToRtfPlainText('%s')\n=> '%s'\n", F1(sText), F2(sPlainText)));

	return (sPlainText);
}


//      Convert the SGML lookup buffer into RTF. SGML tags are replaced
//      with the appropriate RTF format codes, and extraneous spacing is 
//      ignored. In addition, sense numbers and parts of speach are 
//      highlighted.
//
CString ConvertLookupSgmlToRtf (PCSZ pszText, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_VERBOSE, ("ConvertLookupSgmlToRtf('%s', %d)\n", F(pszText), eType));
	CONST CHAR szTEXT_DELIMITERS[] = ":,.";
#if FALSE // the caller adds this since several calls might be used
	CStringx sLookupText (szRTF_HEADER);
#else
	CStringx sLookupText;
#endif
	CStringx sCRNL ("\r\n");
	CStringx sText (pszText);
	INT i = 0;

	if (eType == eDEFINITION) {
		sLookupText += szRTF_HANGING_INDENT;            // small hanging indentation (1/8")
		}

	// Read the Lookup results buffer line by line, checking for SGML tags, which
	// are placed on separate lines. An extra space will be inserted whenever
	// there is a soft line break and after formatting italicized text.
	BOOL nExtraSpaces = 0;
	while (i < sText.GetLength()) {
		BOOL bFirstLine = (i == 0);
		CStringx sLine = ExtractToken(sText, i, sCRNL);
		sLine = sLine.TrimLeading();
		if (IsSgmlTag(sLine)) {
			ProcessSgmlTag(sLookupText, sLine, eType, nExtraSpaces);
			}
		else {
			CStringx sRemLine;
			ProcessLinePrefixStuff(sLookupText, sLine, sRemLine, eType, nExtraSpaces);
			sRemLine = sRemLine.Trim();
			if (!sRemLine.IsEmpty()) {
				if (Pos(szTEXT_DELIMITERS, sRemLine[0]) != nINVALID_INDEX) {
					nExtraSpaces = 0;
					}
				nExtraSpaces = min(2, nExtraSpaces);
				while (nExtraSpaces > 0) {
					sLookupText += cSPACE;
					nExtraSpaces--;
					}
				ASSERT(!IsSpace(sRemLine[0]));
				sLookupText += ToRtfPlainText(sRemLine);
				nExtraSpaces = 1;
				}
			}
		}

#if FALSE // the caller adds this since several calls might be used
	sLookupText += szRTF_TRAILER;
#endif

	return (sLookupText);
}


//      Convert the SGML lookup buffer into straight text. SGML tags are ignored.
//
CString ConvertLookupSgmlToText (PCSZ pszText, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_VERBOSE, ("ConvertLookupSgmlToText('%s', %d)\n", F(pszText), eType));
	CStringx sLookupText;
	CStringx sCRNL ("\r\n");
	CStringx sText (pszText);
	INT i = 0;

	//      Read the Lookup results buffer line by line, checking for
	//      SGML tags (each is on a separate line).
	BOOL bNeedsSpace = FALSE;
	while (i < sText.GetLength()) {
		BOOL bFirstLine = (i == 0);
		CStringx sLine = ExtractToken(sText, i, sCRNL);
		sLine = sLine.TrimLeading();
		if (!IsSgmlTag(sLine)) {
			sLookupText += sLine;
			}
		else if ((sLine == SGML_START_TAG(PARA))
				 || (sLine == SGML_END_TAG(HEADWORD))) {
			if (eType == eDEFINITION) {
				sLookupText += cSPACE;
				}
			else {
				sLookupText += cSEMI_COLON;
				}
			}
		}
	
	return (sLookupText);
}


//      Filter out garbage from the TRL lookup. In addition, 'format' the text into
//      lines.
//
CString WayUncoolFilterLookupGarbage (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("WayUncoolFilterLookupGarbage(%x)\n", pszText));
    CString sText;

	CHAR cLast = cSPACE;    // pretend last charcater was a space (to ignore leading spaces)
    while (*pszText) {
		CHAR cCurrent = *pszText;
		if (MyIsSpace(cCurrent)) {
			//      See if a newline sequence (\r\n) is acceptable.
			//              new sense number (3.) or new sense type (adj.)
			if (cCurrent == cRETURN) {
				INT nDotOffset = Pos(pszText, cDOT);
				if (nDotOffset != nINVALID_INDEX) {
					CONST CHAR szTYPES[] = "adj adv n vi vt interj";
					Assert(nDotOffset != 0);
					CStringx sToken(pszText, nDotOffset);
					// TODO: have an inplace TrimLeading() to avoid confusion?
					CStringx sTrimToken = sToken.TrimLeading();
					if (IsNumeric(sTrimToken)) {
						sText += szRTF_NEW_PARA;
						sText += szRTF_START_BOLD;
						sText += sTrimToken + cDOT;
						sText += szRTF_END_BOLD;
						sText += szRTF_TAB;
						pszText += sToken.GetLength();
						cCurrent = cSPACE;
						}
					else if (Pos(szTYPES, sTrimToken + ' ') != nINVALID_INDEX) {
						sText += szRTF_NEW_PARA;
						sText += szRTF_START_BOLD;
						sText += szRTF_START_ITALICS;
						sText += sTrimToken + cDOT;
						sText += szRTF_END_ITALICS;
						sText += szRTF_END_BOLD;
						pszText += sToken.GetLength();
						cCurrent = cDOT;
						}
					else {
						sText += cSPACE;
						cCurrent = cSPACE;              // make it a true space
						}
					}
				}
			else if (cCurrent == cNEW_LINE) {
				cCurrent = cSPACE;                              // make it a true space
				// skip
				}
			else if (cLast != cCurrent) {
				sText += cCurrent;
				}
			}
		else if (cCurrent == cDASH) {
			//      Convert a double dash sequence into a CR/NL sequence
			if (cLast != cRETURN) {
				cCurrent = cRETURN;
				}
			else {
				cCurrent = cNEW_LINE;
				}
			sText += cCurrent;
			}
	else if (!MyIsPrintable(cCurrent)) {
			//      Convert a double wide-dash sequence into a CR/NL sequence
			if ((BYTE)cCurrent == byWIDE_DASH) {
				if (cLast != cRETURN) {
					cCurrent = cRETURN;
					}
				else {
					cCurrent = cNEW_LINE;
					}
				sText += cCurrent;
				}
			else {
			//bSkip = TRUE;                         // skip nonprinting characters
				}
			}
		else {
			sText += cCurrent;
			}
		cLast = cCurrent;
	pszText++;                      // TODO: allow for DBCS
	}

    return (sText);
}


CString FilterWordNetGarbage (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterWordNetGarbage(%x)\n", pszText));
    CString sText;

    while (*pszText) {
	CHAR cNext = *pszText;
	if (cNext == '\n') {
	    sText += "\r\n";
	    }
	else if (MyIsPrintable(cNext)) {
	    sText += cNext;
	    }
	pszText++;
	}

    return (sText);
}


CString GetWordNetHeads (PCSZ pszPhrase, PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("GetWordNetHeads(%x)\n", pszText));
	CString sHeads;
	CStringArray lines;
	TokenizeText(lines, pszText, "\r\n");

	for (INT i = 0; i < lines.GetSize(); i++) {
		CONST CHAR szSENSE[] = "Sense";

		// Check for the start of a new sense group
		if (Pos(lines[i], szSENSE) == 0) {
			// Skip over the sense indicator line ("Sense 2")
			i++;

			// Isolate the list of head words (sans the definition)
			//    "cad, bounder, ... heel -- (someone who is morally reprehesible)"
			INT nEnd = Pos(lines[i], cDASH);
			if (nEnd == nINVALID_INDEX) {
				nEnd = lines[i].GetLength();
			}
			CStringx sText (lines[i], nEnd);

			// Extract the individual head words
			INT nPos = 0;
			while (nPos < sText.GetLength()) {
				CString sHead = ExtractToken(sText, nPos, ",");
				sHead.TrimLeft();
				sHead.TrimRight();
				// Include the head if not the source phrase
				if (sHead.CompareNoCase(pszPhrase) != 0) {
					// TODO: eliminate duplicates
					sHeads += sHead;
					sHeads += szENTRY_SEP;
				}
			}
		}
	}

    return (sHeads);
}


CString GetWordNetRelations (PCSZ pszPhrase, PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("GetWordNetRelations(%x)\n", pszText));
	UNREFERENCED_PARAMETER(pszPhrase);
	CString sRelations;

	// Split the buffer into lines
	// TODO: eliminate extra buffer since the tree can be quite large
	CStringArray lines;
	TokenizeText(lines, pszText, "\r\n");

	for (INT i = 0; i < lines.GetSize(); i++) {
		CONST CHAR szARROW[] = " => ";

		// Check for the start of a new sense group
		INT nPos = Pos(lines[i], szARROW);
		if (nPos != nINVALID_INDEX) {
			// Isolate the list of relations words (sans the definition)
			//    " => location -- (a point or extent in space)"
			INT nEnd = Pos(lines[i], cDASH);
			if (nEnd == nINVALID_INDEX) {
				nEnd = lines[i].GetLength();
			}
			CStringx sText (lines[i], nEnd);

			// Extract the individual head words
			nPos += STR_LEN(szARROW);
			while (nPos < sText.GetLength()) {
				CString sRelation = ExtractToken(sText, nPos, ",");
				sRelation.TrimLeft();
				sRelation.TrimRight();
				sRelations += sRelation;
				sRelations += szENTRY_SEP;
				}
			}
		}

    return (sRelations);
}


// FilterBookshelfTextEntry: Filter the plain text returned from a Bookshelf
// lookup (via MediaView)
//
// TODO:  convert this into SGML to take advantage of SgmlToRtf routines
//
CString FilterBookshelfTextEntry (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterBookshelfTextEntry(%x)\n", pszText));
    CString sText;

	INT nPos = 0;

	// Skip over the header text
	CString sJunk;
	do {
		sJunk.Empty();
		sJunk = ExtractToken(pszText, nPos, "[]");
		sJunk += ExtractToken(pszText, nPos, "<>");
		DEBUG_OUT(TL_VERBOSE, ("Ignoring entry header: %s\n", F(sJunk)));
	} while (!sJunk.IsEmpty());

	// Get the main text
	while (pszText[nPos]) {
		CString sLine = ExtractToken(pszText, nPos, "\r\n");
		sText += sLine;
	}

    return (sText);
}


CString FilterBookshelfSynsEntry (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterBookshelfSynsEntry(%x)\n", pszText));
    CString sText;

	// Get the main text
	INT nPos = 0;
	while (pszText[nPos]) {
		if (pszText[nPos] == cCOMMA) {
			sText += cMV_ENTRY_SEP;
			}
		else {
			sText += pszText[nPos];
			}
		nPos++;
	}

    return (sText);
}


CString FilterBookshelfWordList (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterBookshelfWordList(%x)\n", pszText));
    CString sText;

	// Get the main text
	INT nPos = 0;
	while (pszText[nPos]) {
		// Extract the next phrase from the text
		SkipSpaces(pszText, nPos);
		CStringx sPhrase = ExtractToken(pszText, nPos, cMV_ENTRY_SEP);
		INT nStart = 0;
		SkipSpaces(sPhrase, nStart);
		if (nStart == sPhrase.GetLength()) {
			break;
		}

		// Ignore the category number and the sense indicator
		ASSERT(IsDigit(sPhrase[nStart]));
		nStart++;
		CONST CHAR cLEFT_PAREN = '(';
		INT nEnd = sPhrase.FindAt(nStart, cLEFT_PAREN);
		INT nLen = ((nEnd == nINVALID_INDEX) 
						? (sPhrase.GetLength() - nStart + 1)
						: (nEnd - nStart));
		sPhrase = sPhrase.Mid(nStart, nLen);
		sText += sPhrase;
		sText += szENTRY_SEP;
	}

    return (sText);
}


//      Workhorse routine for doing actual lookups. For each active information source, a
//      request is submitted. Afterwards, the results are optionally converted into RTF from
//      the pseudo-SGML format of the raw lookup results.
//
void CLookup::LookupClue (const CStringx& sSource, LOOKUP_TYPE eType, const CStringx& phrase, BOOL bFormat)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LookupClue(%s, %d, %s, %d)\n", 
			  F1(sSource), eType, F2(phrase), bFormat));
	CString sSgmlBuffer;
    
    //  Look up the phrase
    if (m_bUseTRL
		&& ((eType == eDEFINITION) || (eType == eSYNONYMS) || (eType == eCROSS_REFS))) {
		BOOL bOK = pLookupPhrase(m_pTrl, phrase, sSource, eType);
    
		// Copy the results
		if (bOK) {
			PCSZ pszResults = pGetResultsPtr(m_pTrl);
			ASSERT(pszResults);
			sSgmlBuffer += pszResults;
			}
	}

    //  Add info from a WordNet search
	//      TODO: add an intermediate layer to format the WordNet results into SGML
    if (m_bUseWordNet && (eType == eSYNONYMS)) {
		CString sWordNet;
		CString sPhrase = phrase;
		DEBUG_OUT(TL_VERBOSE, ("Initiating WordNet search for %s\n", (PCSZ)sPhrase));
		for (INT nPartOfSearch = NOUN; nPartOfSearch < ADJSAT; nPartOfSearch++) {
			PCSZ pszResults;
			PSZ pszPhrase = (PSZ)(PCSZ)sPhrase;
			pszResults = pfindtheinfo(pszPhrase, nPartOfSearch, SIMPTR, ALLSENSES);
			ASSERT(pszResults);
			sWordNet += GetWordNetHeads(pszPhrase, pszResults);
			}
		// TODO: add optional RTF formatting
		sSgmlBuffer += sWordNet;
		}

	// Check for WordNet hyponym tree
	// TODO: reduce redundancy with synonym check
    if (m_bUseWordNet && (eType == eDESCENDANTS)) {
		CString sWordNet;
		CString sPhrase = phrase;
		DEBUG_OUT(TL_VERBOSE, ("Initiating WordNet hyponym search for %s\n", (PCSZ)sPhrase));
		for (INT nPartOfSearch = NOUN; nPartOfSearch < ADJSAT; nPartOfSearch++) {
			PCSZ pszResults;
			PSZ pszPhrase = (PSZ)(PCSZ)sPhrase;
			pszResults = pfindtheinfo(pszPhrase, nPartOfSearch, -HYPOPTR, ALLSENSES);
			ASSERT(pszResults);
			sWordNet += GetWordNetRelations(pszPhrase, pszResults);
			}
		// TODO: add optional RTF formatting
		sSgmlBuffer += sWordNet;
    }


	if (m_bUseBookshelf && (eType == eDEFINITION)) {
		ASSERT(m_pMvInfo && m_pMvLookup);
		pMviMtOpenWordWheel(m_pMvInfo, szDICTIONARY);
		CString sBookshelfInfo = pMvlLookupDefinitionPhrase(m_pMvLookup, phrase);
		sSgmlBuffer += sBookshelfInfo;
	}

	if (m_bUseBookshelf && (eType == eSYNONYMS)) {
		ASSERT(m_pMvInfo && m_pMvLookup);
		pMviMtOpenWordWheel(m_pMvInfo, szTHESAURUS);
		CString sBookshelfInfo = pMvlLookupPhrase(m_pMvLookup, phrase, FALSE);
		sBookshelfInfo = FilterBookshelfSynsEntry(sBookshelfInfo);
		sSgmlBuffer += sBookshelfInfo;
	}

	if (m_bUseBookshelf && (eType == eCROSS_REFS)) {
		ASSERT(m_pMvInfo && m_pMvLookup);
		pMviMtOpenWordWheel(m_pMvInfo, szDICTIONARY);
		CString sBookshelfInfo = pMvlLookupCrossReferences(m_pMvLookup, phrase);
		sSgmlBuffer += sBookshelfInfo;
	}

	// Convert the SGML results into RTF or plain text
    if (bFormat) {
	    buffer = szRTF_HEADER;
		buffer += ConvertLookupSgmlToRtf(sSgmlBuffer, eType);
	    buffer += szRTF_TRAILER;
		}
	else {
		buffer = ConvertLookupSgmlToText(sSgmlBuffer, eType);
		}

    DEBUG_OUT(TL_VERBOSE, ("Lookup(%s, %d, %s)", F1(sSource), eType, F2(phrase)));
    DEBUG_OUT(TL_VERBOSE, (" => %s\n", F(buffer)));
}


void CLookup::StartLookup (const CStringx& sSource, LOOKUP_TYPE eType, const CStringx& phrase, UINT uCommandID, BOOL bFormat)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::StartLookup(%s, %d, %s, %d, %d)\n", 
			  F1(sSource), eType, F2(phrase), uCommandID, bFormat));
	//BOOL bUseThread = !IsKeyToggled(VK_SCROLL);
	if (bUseThreads) {
		m_sSource = sSource;
		m_eLookupType = eType;
		m_sPhrase = phrase;
		m_bFormat = bFormat;
		m_uNotificationID = uCommandID;
		if (m_pLookupThread == NULL) {
			m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoLookupProc, this);
			}
		}
	else {
	    LookupClue(sSource, eType, phrase, bFormat);
		SendNotification(uCommandID);
		}
}


void CLookup::StartDefLookup (const CStringx& phrase, UINT uCommandID, BOOL bFormat)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::StartDefLookup(%s, %d, %d)\n", 
			  F1(phrase), uCommandID, bFormat));
	m_sPhrase = phrase;
	m_bFormat = bFormat;
	m_uNotificationID = uCommandID;

	//BOOL bUseThreads = GetDebugOption("Multithreaded", TRUE);
	if (bUseThreads) {
		if (m_pLookupThread == NULL) {
			m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoDefLookupProc, this);
			}
		}
	else {
		DoDefinitionLookup(bFormat);
	    //LookupClue(sSource, eType, phrase, uCommandID, bFormat);
		SendNotification(uCommandID);
		}
}


void CLookup::StartSuggestions (const CStringx& sClue, UINT uCommandID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::StartSuggestions(%s, %d)\n", 
			  F1(sClue), uCommandID));
	m_sClue = sClue;
	m_uNotificationID = uCommandID;

	//BOOL bUseThread = !IsKeyToggled(VK_SCROLL);
	if (bUseThreads) {
		if (m_pLookupThread == NULL) {
			m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoSuggestionsProc, this);
			}
		}
	else {
		DoSuggestions();
		SendNotification(uCommandID);
		}
}


void CLookup::GetSynonyms (const CStringx& phrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetSynonyms(%s, %d, %d)\n", 
			  F(phrase), uCommandID, bFormat));
    StartLookup("THES2", eSYNONYMS, phrase, uCommandID, bFormat);
}


void CLookup::GetReferences (const CStringx& phrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetReferences(%s, %d, %d)\n", 
			  F(phrase), uCommandID, bFormat));
    StartLookup("DICT1", eCROSS_REFS, phrase, uCommandID, bFormat);
}

void CLookup::GetDescendants (const CStringx& phrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetReferences(%s, %d, %d)\n", 
			  F(phrase), uCommandID, bFormat));
    StartLookup("N/A", eDESCENDANTS, phrase, uCommandID, bFormat);
}


// NoDefinition: Determine if the buffer has no definition in it.
// If there is no formatting then the buffer should be blank if there is
// no definition. Otherwise, there should be no headword formatting (boldface).
//
// TODO: fix the lookup to not return a valid rtf buffer upon failuure
//
inline BOOL HasNoDefinition (CStringx& sBuffer, BOOL bFormat)
{
	BOOL bHasNoDefinition = FALSE;

	if ((sBuffer.LastNonblank() == -1)
		|| (bFormat && (sBuffer.Find(szRTF_START_BOLD) == nINVALID_INDEX))) {
		bHasNoDefinition = TRUE;
		}

	return (bHasNoDefinition);
}


// Get the definition for the phrase from the TRL reference. This
// first checks looks for the first occurrence of a multiple entry
// phrase (eg, 'cant' has three separate entries) by explicitly
// specifying the occurence indicator ('<word>{1}' as in 'cant{1}').
// If this search fails, the word is searched without the indicator.
// Then if this also fails, the searches are repeated using an
// alternative index source.
//
// NOTE: TRL has two main indexes for the dictionary (two others
//		 are not relevant). One index provides for words used in multiple
//		 entries (eg, 'acid test', 'intelligence test', etc.). Another
//		 goes directly to a word.
//
// TODO: decouple the TRL dependencies from this routines since Bookshelf and
//       WordNet will also be used for definition text
//
void CLookup::DoDefinitionLookup (BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DoDefinitionLookup()\n"));
	CONST CHAR szSOURCE[] = "DICT4";
	CONST CHAR szALT_SOURCE[] = "DICT3";
    CString sSource = szSOURCE;
	CString sAltSource = szALT_SOURCE;
#if defined(_DEBUG)
    if (IsAltPressed()) {
		CString sSource = szALT_SOURCE;
		CString sAltSource = szSOURCE;
	}
#endif

    //  First try with the sense indicator (eg, TEST{1})
    CString sPhrase1 = m_sPhrase + "{1}";
    LookupClue(sSource, eDEFINITION, m_sPhrase, bFormat);

	// If no luck, try with the unadorned word (eg, TEST)
	// TODO: account for multiple knowledge sources
    if (HasNoDefinition(buffer, bFormat)) {
	    LookupClue(sSource, eDEFINITION, m_sPhrase, bFormat);
		}

	// If still nothing returned, try the alternative dictionary index
    if (HasNoDefinition(buffer, bFormat)) {
		DEBUG_OUT(TL_VERBOSE, ("Retrying alternate definition lookup (%s) using sense indicator: %s\n", 
				  F1(sAltSource), F2(sPhrase1)));
		LookupClue(sAltSource, eDEFINITION, sPhrase1, bFormat);
	}
    if (HasNoDefinition(buffer, bFormat)) {
		DEBUG_OUT(TL_VERBOSE, ("Retrying alternate lookup (%s): %s\n", 
				  F1(sAltSource), F2(m_sPhrase)));
		LookupClue(sAltSource, eDEFINITION, m_sPhrase, bFormat);
	}
}


void CLookup::GetDefinition (const CStringx& sPhrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetDefinition(%s, %d, %d)\n", 
			  F(sPhrase), uCommandID, bFormat));
	StartDefLookup(sPhrase, uCommandID, bFormat);
}


void CLookup::DeriveSuggestions (const CStringx& clue, UINT uCommandID /* = 0 */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DeriveSuggestions('%s', %d)\n", 
			  F(clue), uCommandID));
	StartSuggestions(clue, uCommandID);
}



void CLookup::DoSuggestions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DoSuggestions()\n"));
    CStringArray words;
	CStringArray result;
    CStringx phrase = m_sClue.Trim();

	// Look up each word individually, saving unique occurrences in a sorted list
    TokenizeText(words, phrase, ", ;.");
    for (INT i = 0; i < words.GetSize(); i++) {
		CStringx word (words[i]);
		m_sPhrase = word;
	    LookupClue("THES2", eSYNONYMS, m_sPhrase, FALSE);
		// InsertUnique(result, MatchPhrases(buffer, pattern, ';'));
		MatchPhrases(result, buffer, pattern, ';');
		DoDefinitionLookup(FALSE);
		// InsertUnique(result, MatchPhrases(buffer, pattern, " ;"));
		MatchPhrases(result, buffer, pattern, " ;");
	    LookupClue("DICT2", eCROSS_REFS, m_sPhrase, FALSE);
		// InsertUnique(result, MatchPhrases(buffer, pattern, ';'));
		MatchPhrases(result, buffer, pattern, ';');
		if (IsControlPressed()) {
			LookupClue("N/A", eDESCENDANTS, m_sPhrase, FALSE);
			// InsertUnique(result, MatchPhrases(buffer, pattern, ';'));
			MatchPhrases(result, buffer, pattern, ';');
		}
	}

	// Create the final list
    suggestions = "";
    for (i = 0; i < result.GetSize(); i++) {
		suggestions += result[i] + szENTRY_SEP;
		}

	DEBUG_OUT(TL_VERBOSE, ("DoSuggestions() => %s\n", F(suggestions)));
}


#if FALSE
BOOL CLookup::LoadLookup (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LoadLookup()\n"));
#if FALSE
    CStringx lookup = "LOOKUP.EXE";
    CStringx path = GetLookupDirName() + "\\";
    CStringx arguments = "/tLOOKUP.TXT /b /&" + path + " "
			+ "/iDICT1 /iDICT3 /iDICT4 /iTHES2";
    
    HMODULE hLookupMod = ::GetModuleHandle(lookup);
    if (!hLookupMod) {
	CStringx CommandLine = path + lookup + " " + arguments;
//// this seems to make the application lose focus
	hLookupMod = (HMODULE)::WinExec(CommandLine, SW_MINIMIZE);
	SetActiveWindow(GetMainHwnd());
	if (hLookupMod < HINSTANCE_ERROR) {
	    AfxMessageBox("Unable to load lookup program");
	    return (FALSE);
	    }
	}

    return (TRUE);
#endif
    return (FALSE);
}
#endif


//      Send a notification to the client that the lookup results are ready
//
void CLookup::SendNotification (UINT uCommandID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::SendNotification(%u)\n", uCommandID));

	CWnd* pClientWnd = CWnd::FromHandle(m_hClientWnd);
	ASSERT((!pClientWnd) || pClientWnd->IsKindOf(RUNTIME_CLASS(CWnd)));
	if (pClientWnd 
			&& pClientWnd->IsKindOf(RUNTIME_CLASS(CWnd))
			&& (uCommandID != 0)) {
		pClientWnd->PostMessage(WM_COMMAND, uCommandID, 0);
		}
}


//-----------------------------------------------------------------------------
//      Other (helper) functions
//-----------------------------------------------------------------------------


// Thread procedure for doing static initialization of the info sources
//
STATIC UINT DoInitProc (PVOID pData)
{
	DEBUG_OUT(TL_VERBOSE, ("DoInitProc(%x)\n", pData));

	PBOOL pReinit = (PBOOL)pData;
	BOOL bInit = pReinit ? *pReinit : FALSE;
	BOOL bOK = CLookup::InitLookupAux(bInit);

	return (bOK);
}


// Thread procedure for looking up phrases.
//
STATIC UINT DoLookupProc (CLookup* pLookup)
{
	DEBUG_OUT(TL_VERBOSE, ("DoLookupProc(%x)\n", pLookup));
	BOOL bOK = FALSE;

	ASSERT(pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup)));
	if (pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup))) {
		pLookup->LookupClue(pLookup->m_sSource,             // reference source (TRL-specific)
							pLookup->m_eLookupType,         // type of lookup (eg, cross refs)
							pLookup->m_sPhrase,             // phrase to lookup
							pLookup->m_bFormat);            // format results using RTF
		pLookup->SendNotification(pLookup->m_uNotificationID);
		pLookup->m_pLookupThread = NULL;
		bOK = TRUE;
		}

	return (bOK);
}


// Thread procedure for handling special case of definition lookup
//
STATIC UINT DoDefLookupProc (CLookup* pLookup)
{
	DEBUG_OUT(TL_VERBOSE, ("DoLookupProc(%x)\n", pLookup));
	BOOL bOK = FALSE;

	ASSERT(pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup)));
	if (pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup))) {
		// TODO: handle unformatted lookup
		pLookup->DoDefinitionLookup();
		pLookup->SendNotification(pLookup->m_uNotificationID);
		pLookup->m_pLookupThread = NULL;
		bOK = TRUE;
		}

	return (bOK);
}


//      Thread procedure for handling special case of deriving suggestions
//
STATIC UINT DoSuggestionsProc (CLookup* pLookup)
{
	DEBUG_OUT(TL_VERBOSE, ("DoSuggestionsProc(%x)\n", pLookup));
	BOOL bOK = FALSE;

	ASSERT(pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup)));
	if (pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup))) {
		pLookup->DoSuggestions();
		pLookup->SendNotification(pLookup->m_uNotificationID);
		pLookup->m_pLookupThread = NULL;
		bOK = TRUE;
		}

	return (bOK);
}


//      Unused timer procedure
//
#if FALSE
void EXPORT CALLBACK TimerProc (HWND hWnd, UINT nMsg, UINT nIDEvent, DWORD dwTime)
{
	// TODO: map hWnd into lookup instance via list of lookup pClientWnd's
	// NOTE: perhaps its better for the client to maintain the cursor since they
	//       might want to have several lookup's going on simulataneously
	//       (eg, for the poor fellow with the dual pentium and numerous GB drives)
}
#endif
