//	Copyleft 1993-1996 Tom O'Hara		Lookup source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// LookupOptions.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CommonLookup.h"
#include "LookupOptions.h"
#include "LookupMain.h"
#include "LookupDll.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// CBookshelfOptions property page


IMPLEMENT_DYNCREATE(CBookshelfOptions, CPropertyPage)

CBookshelfOptions::CBookshelfOptions() : CPropertyPage(CBookshelfOptions::IDD)
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::CBookshelfOptions()\n"));

	m_psp.hInstance = GetDllInstance();
	m_bSettingsChanged = FALSE;
	//{{AFX_DATA_INIT(CBookshelfOptions)
	m_sBookshelfDir = GetAppOption(szBOOKSHELF_DIR, szDEF_BOOKSHELF_DIR);
	m_bUseBookshelf = GetAppOption(szUSE_BOOKSHELF, FALSE);
	//}}AFX_DATA_INIT
}


CBookshelfOptions::~CBookshelfOptions()
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::~CBookshelfOptions()\n"));
}


VOID CBookshelfOptions::PrepareMisc (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::PrepareMisc()\n"));
	m_psp.hInstance = GetDllInstance();
}


void CBookshelfOptions::DoDataExchange(CDataExchange* pDX)
{
	DEBUG_OUT(TL_GARRULOUS, ("CBookshelfOptions::DoDataExchange(%x)\n", pDX));

	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBookshelfOptions)
	DDX_Control(pDX, IDC_BROWSE, m_btnBrowseDir);
	DDX_Text(pDX, IDC_BOOKSHELF_DATA_DIRECTORY, m_sBookshelfDir);
	DDX_Check(pDX, IDC_USE_BOOKSHELF, m_bUseBookshelf);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBookshelfOptions, CPropertyPage)
	//{{AFX_MSG_MAP(CBookshelfOptions)
	ON_EN_CHANGE(IDC_BOOKSHELF_DATA_DIRECTORY, OnChangeBookshelfDataDirectory)
	ON_BN_CLICKED(IDC_USE_BOOKSHELF, OnUseBookshelf)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBookshelfOptions message handlers

void CBookshelfOptions::OnOK (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::OnOK()\n"));

	ApplySettings();	
	CPropertyPage::OnOK();
}


BOOL CBookshelfOptions::OnApply() 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::OnApply()\n"));
	
	BOOL bOK = ApplySettings();
	if (bOK) {
		bOK = CPropertyPage::OnApply();
		}

	return (bOK);
}


BOOL CBookshelfOptions::ApplySettings (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::ApplySettings()\n"));

	UpdateData(bSAVE_AND_VALIDATE);
	BOOL bOK = TRUE;
	if (m_bUseBookshelf) {
		CString sTitle;
		// Give the user some warnings if the support DLL or title are invalid
		bOK = bOK && CLookup::ResolveMediaViewFunctions();
		bOK = bOK && CLookup::CheckBookshelfTitle(m_sBookshelfDir, sTitle);
		}
	if (bOK) {
		WriteAppOption(szUSE_BOOKSHELF, m_bUseBookshelf);
		WriteAppOption(szBOOKSHELF_DIR, m_sBookshelfDir);
		}

	return (bOK);
}


void CBookshelfOptions::OnChangeBookshelfDataDirectory() 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::OnChangeBookshelfDataDirectory::OnOK()\n"));
	
	m_bSettingsChanged = TRUE;
}

void CBookshelfOptions::OnUseBookshelf() 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::OnUseBookshelf()\n"));
	
	m_bSettingsChanged = TRUE;
}


void CBookshelfOptions::OnBrowse() 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::OnBrowse()\n"));
	CONST CHAR szM14[] = "m14";

	UpdateData(bSAVE_AND_VALIDATE);
	CFileDialog dlg(TRUE, szM14, szDEF_BOOKSHELF_FILE);
	dlg.m_ofn.lpstrInitialDir = m_sBookshelfDir;
	dlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;

	if (dlg.DoModal() == IDOK) {
		//m_sTitle = dlg.GetPathName();
		m_sBookshelfDir = ExtractFileDir(dlg.GetPathName());
		DEBUG_OUT(TL_VERBOSE, ("m_sBookshelfDir = %s\n", (PCSZ)m_sBookshelfDir));
		UpdateData(bINITIALIZE);
		m_bSettingsChanged = TRUE;
	}
}


/////////////////////////////////////////////////////////////////////////////
// CTrlOptions property page


IMPLEMENT_DYNCREATE(CTrlOptions, CPropertyPage)

CTrlOptions::CTrlOptions() : CPropertyPage(CTrlOptions::IDD)
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::CTrlOptions()\n"));

	m_psp.hInstance = GetDllInstance();
	m_bSettingsChanged = FALSE;
	//{{AFX_DATA_INIT(CTrlOptions)
	m_sTrlDir = GetAppOption(szTRL_DATA_DIR, szDEF_TRL_DATA_DIR);
	m_bUseTrl = GetAppOption(szUSE_TRL, FALSE);
	//}}AFX_DATA_INIT
}


CTrlOptions::~CTrlOptions()
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::~CTrlOptions()\n"));
}


VOID CTrlOptions::PrepareMisc (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::PrepareMisc()\n"));
	m_psp.hInstance = GetDllInstance();
}


void CTrlOptions::DoDataExchange(CDataExchange* pDX)
{
	DEBUG_OUT(TL_GARRULOUS, ("CTrlOptions::DoDataExchange(%x)\n", pDX));

	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTrlOptions)
	DDX_Control(pDX, IDC_BROWSE, m_btnBrowseDir);
	DDX_Text(pDX, IDC_TRL_DATA_DIRECTORY, m_sTrlDir);
	DDX_Check(pDX, IDC_USE_TRL, m_bUseTrl);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTrlOptions, CPropertyPage)
	//{{AFX_MSG_MAP(CTrlOptions)
	ON_EN_CHANGE(IDC_TRL_DATA_DIRECTORY, OnChangeTrlDataDirectory)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(IDC_USE_TRL, OnUseTrl)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrlOptions message handlers

BOOL CTrlOptions::OnApply() 
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::OnApply()\n"));
	
	ApplySettings();	
	return CPropertyPage::OnApply();
}

void CTrlOptions::OnOK() 
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::OnOK()\n"));
	
	ApplySettings();	
	CPropertyPage::OnOK();
}


void CTrlOptions::ApplySettings (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::ApplySettings()\n"));

	WriteAppOption(szUSE_TRL, m_bUseTrl);
	WriteAppOption(szTRL_DATA_DIR, m_sTrlDir);
}


void CTrlOptions::OnChangeTrlDataDirectory() 
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::OnChangeTrlDataDirectory()\n"));
	
	m_bSettingsChanged = TRUE;
}

void CTrlOptions::OnBrowse() 
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::OnBrowse()\n"));
	CONST CHAR szDAT[] = "DAT";

	UpdateData(bSAVE_AND_VALIDATE);
	CFileDialog dlg(TRUE, szDAT, szTRL_TEST_FILE);
	dlg.m_ofn.lpstrInitialDir = m_sTrlDir;
	dlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;

	if (dlg.DoModal() == IDOK) {
		//m_sTitle = dlg.GetPathName();
		m_sTrlDir = ExtractFileDir(dlg.GetPathName());
		DEBUG_OUT(TL_VERBOSE, ("m_sTrlDir = %s\n", (PCSZ)m_sTrlDir));
		m_bSettingsChanged = TRUE;
		UpdateData(bINITIALIZE);
	}
}

void CTrlOptions::OnUseTrl() 
{
	DEBUG_OUT(TL_VERBOSE, ("CTrlOptions::OnUseTrl()\n"));
	
	m_bSettingsChanged = TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CWordNetOptions property page

IMPLEMENT_DYNCREATE(CWordNetOptions, CPropertyPage)

CWordNetOptions::CWordNetOptions() : CPropertyPage(CWordNetOptions::IDD)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::CWordNetOptions()\n"));

	m_psp.hInstance = GetDllInstance();
	m_bSettingsChanged = FALSE;
	ASSERT((DWORD)m_psp.hInstance != 0x0400000);
	//{{AFX_DATA_INIT(CWordNetOptions)
	m_sWordNetDir = GetAppOption(szWORDNET_DIR, szDEF_WORDNET_DIR);
	m_bUseWordNet = GetAppOption(szUSE_WORDNET, FALSE);
	m_bConsultWnsearchdir = FALSE;
	//}}AFX_DATA_INIT
}


CWordNetOptions::~CWordNetOptions()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::~CWordNetOptions()\n"));
}


VOID CWordNetOptions::PrepareMisc (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::PrepareMisc()\n"));
	m_psp.hInstance = GetDllInstance();
}


void CWordNetOptions::DoDataExchange(CDataExchange* pDX)
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordNetOptions::DoDataExchange(%x)\n", pDX));

	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWordNetOptions)
	DDX_Control(pDX, IDC_BROWSE, m_btnBrowseDir);
	DDX_Check(pDX, IDC_USE_WORDNET, m_bUseWordNet);
	DDX_Text(pDX, IDC_WORDNET_DATA_DIRECTORY, m_sWordNetDir);
	DDX_Check(pDX, IDC_CONSULT_WNSEARCHDIR, m_bConsultWnsearchdir);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWordNetOptions, CPropertyPage)
	//{{AFX_MSG_MAP(CWordNetOptions)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(IDC_CONSULT_WNSEARCHDIR, OnConsultWnsearchdir)
	ON_BN_CLICKED(IDC_USE_WORDNET, OnUseWordnet)
	ON_EN_CHANGE(IDC_WORDNET_DATA_DIRECTORY, OnChangeWordnetDataDirectory)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWordNetOptions message handlers


BOOL CWordNetOptions::OnApply() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::OnApply()\n"));
	
	ApplySettings();	
	return CPropertyPage::OnApply();
}

void CWordNetOptions::OnOK() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::OnOK()\n"));
	
	ApplySettings();	
	CPropertyPage::OnOK();
}


void CWordNetOptions::ApplySettings (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CBookshelfOptions::ApplySettings()\n"));

	WriteAppOption(szUSE_WORDNET, m_bUseWordNet);
	WriteAppOption(szWORDNET_DIR, m_sWordNetDir);
}

void CWordNetOptions::OnBrowse() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::OnBrowse()\n"));
	CONST CHAR szDAT[] = "DAT";

	UpdateData(bSAVE_AND_VALIDATE);
	CFileDialog dlg(TRUE, szDAT, szWORDNET_TEST_FILE);
	dlg.m_ofn.lpstrInitialDir = m_sWordNetDir;
	dlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;

	if (dlg.DoModal() == IDOK) {
		//m_sTitle = dlg.GetPathName();
		m_sWordNetDir = ExtractFileDir(dlg.GetPathName());
		DEBUG_OUT(TL_VERBOSE, ("m_sWordNetDir = %s\n", (PCSZ)m_sWordNetDir));
		m_bSettingsChanged = TRUE;
		UpdateData(bINITIALIZE);
	}
}

void CWordNetOptions::OnConsultWnsearchdir() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::OnConsultWnsearchdir()\n"));
	
	m_bSettingsChanged = TRUE;
}

void CWordNetOptions::OnUseWordnet() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::OnUseWordnet()\n"));
	
	m_bSettingsChanged = TRUE;
}

void CWordNetOptions::OnChangeWordnetDataDirectory() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordNetOptions::OnChangeWordnetDataDirectory()\n"));
	
	m_bSettingsChanged = TRUE;
}
