// lookup.cpp : implementation file
//

#include "stdafx.h"
//#define USE_TASK
//#include "common.h"
//#include "task.h"
#include "TrlLookup.h"
#include "lookup.h"

//  WordNet-specific includes
//	TODO: wrap these in a separate header (eg, WnLookup.h) to avoid C/C++ conflicts
extern "C" {
    #include "wn.h"
    #include "wnconsts.h"
    #include "wnglobals.h"
    }

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW


// for common.inl
INLINE BOOL MyIsSpace (CHAR c)          { return ((c == ' ') || (c == '\r') || (c == '\n') || (c == '\t')); }

INLINE CString SkipSpaces (CONST CString& text)
{
	INT i = 0;
	SkipSpaces(text, i);
	CString sRest = text.Mid(i);
	return (sRest);
}

// for common.cpp
//

STATIC BOOL FileExists (PCSZ pszFile)
{
	OFSTRUCT ofs;
	HFILE hFile = OpenFile(pszFile, &ofs, OF_EXIST);
	BOOL bExists = (hFile != HFILE_ERROR);
	DEBUG_OUT(TL_VERBOSE, ("FileExists(%s) => %d\n", pszFile, bExists));

	return (bExists);
}


STATIC CString ExtractFileDir (PCSZ pszFileName)
{
	DEBUG_OUT(TL_VERBOSE, ("ExtractFileBase(%s)\n", 
			  F1(sFileName)));
    CString sDir;
    SplitPath(pszFileName, NULL, &sDir, NULL, NULL);
        
    return (sDir);
}


STATIC CString GetProgramDir (VOID)
{
	CHAR szModuleFileName[MAX_PATH] = { cNULL };
	GetModuleFileName(NULL, szModuleFileName, STR_LEN(szModuleFileName));
	CString sDir = ExtractFileDir(szModuleFileName);

	return (sDir);
}


//      Constants
//
CONST CHAR cDOT = '.';
//CONST CHAR cWIDE_DASH =       (CHAR)0xC4;
CONST BYTE byWIDE_DASH =        0xC4;
CONST CHAR cRETURN =            '\r';
CONST CHAR cNEW_LINE =          '\n';
CONST CHAR cDASH =                      '-';
CONST CHAR cSEMI_COLON =        ';';
CONST CHAR szENTRY_SEP[] =      "; ";
CONST CHAR szTHESAURUS[] = "Roget's Thesaurus";
CONST CHAR szDICTIONARY[] = "Dictionary";

// Globals
//
BOOL bUseThreads = FALSE;

//      Local functions
//
STATIC UINT DoInitProc (PVOID pData);
STATIC UINT DoLookupProc (CLookup* pLookup);
STATIC UINT DoDefLookupProc (CLookup* pLookup);
STATIC UINT DoSuggestionsProc (CLookup* pLookup);


#if FALSE
void EXPORT CALLBACK TimerProc(
	HWND hWnd,              // handle of CWnd that called SetTimer
	UINT nMsg,              // WM_TIMER
	UINT nIDEvent,  // timer identification
	DWORD dwTime    // system time
);
#endif


BOOL PostMessageToFrame(UINT message, WPARAM wParam = 0, LPARAM lParam = 0)
{
	CWnd* pFrame = AfxGetApp()->m_pMainWnd;
	BOOL bOK = pFrame->PostMessage(message, wParam, lParam);

	return (bOK);
}


//-----------------------------------------------------------------------------
// CLookup class implementation
//
// Mondo dispatch/consolation class for doing lookup's with various sources
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC(CLookup, CObject);

BOOL CLookup::m_bUseTRL = FALSE;
BOOL CLookup::m_bUseWordNet = FALSE;
BOOL CLookup::m_bUseBookshelf = FALSE;
CMvInfo* CLookup::m_pMvInfo = NULL;
PLOOKUP_INFO CLookup::m_pLI = NULL;
CWinThread* CLookup::m_pInitThread = NULL;
CWnd* CLookup::m_pStaticParentWnd = NULL;
DWORD CLookup::m_dwTimeOut = 0;

CLookup::CLookup(CWnd* pClient /* = NULL */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CLookup()\n"));

	// Pointer initialization; be careful w/ these
	m_hClientWnd = pClient ? pClient->m_hWnd : NULL;
	m_pLookupThread = NULL;

	// Make sure the initialization thread has completed.
	// Shut down the app if it still doesn't terminate after
	// a two minute time out (with user permission of course)
	CWaitCursor waitCursor;
	BOOL bDone = FALSE;
#if TRUE
	bDone = (bUseThreads == FALSE) || (m_pInitThread == NULL);
#endif
	while (!bDone) {
		ASSERT(m_pInitThread && ValidHandle(m_pInitThread->m_hThread));
		DWORD dwEvent = WaitForSingleObject(m_pInitThread->m_hThread, m_dwTimeOut * 1000);
		bDone = TRUE;
		if (dwEvent == WAIT_TIMEOUT) {
			CONST CHAR szINIT_TIMEOUT[] = "Timeout during reference source initialization.\nPlease check the cofiguration";
			if (IDCANCEL == AfxMessageBox(szINIT_TIMEOUT, MB_RETRYCANCEL | MB_ICONSTOP)) {
				PostMessageToFrame(ID_APP_EXIT);
				}
			else {
				bDone = FALSE;
				}
			}
		else if (dwEvent == WAIT_FAILED) {
			DEBUG_OUT(TL_BASIC, ("Error %lu in WaitForSingleObject\n", GetLastError()));
			}
		}
#if TRUE
	m_pInitThread = NULL;
#endif

	m_pMvLookup = NULL;
	if (m_bUseBookshelf) {
		ASSERT(m_pMvInfo);
		m_pMvLookup = new CMvLookup(m_pMvInfo);
		}
}


CLookup::~CLookup()
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::~Lookup()\n"));

	if (m_pLookupThread) {
		AbortLookup();
		}

	if (m_pMvLookup) {
		delete m_pMvLookup;
		m_pMvLookup = NULL;
		}
}


VOID CLookup::InitBookshelf (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitBookshelf()\n"));
	m_bUseBookshelf = FALSE;

	// First make sure an instance doesn't already exist
	CleanupBookshelf();

	// Get the file name of the title (default to Bookshelf)
	CString sTitle = GetAppOption("BookshelfFile", szNULL);
#if defined(_DEBUG)
	if (sTitle.IsEmpty()) {
		// TODO: search (cdrom/network) drive(s) for this
		sTitle = "g:\\books\\Bshelf95.m14";
	}
#endif

	if (FileExists(sTitle)) {
		//m_pMvInfo = new CMvInfo(AfxGetApp()->m_pMainWnd, sTitle);
		m_pMvInfo = new CMvInfo();
		ASSERT(m_pMvInfo);

		if (m_pMvInfo) {
			m_pMvInfo->MtOpenTitle(sTitle);
			m_pMvInfo->MtOpenWordWheel(szDICTIONARY);
			m_pMvInfo->MtOpenTopicList(szDICTIONARY);
			m_bUseBookshelf = TRUE;
		}
	}
}
		

VOID CLookup::CleanupBookshelf (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupBookshelf()\n"));

	if (m_pMvInfo) {
		delete m_pMvInfo;
		m_pMvInfo = NULL;
	}
}


//      Terminate the lookup thread Arnold-style
//      TODO: use events to signal thread to end itself
//
VOID CLookup::AbortLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::~Lookup()\n"));

	if (m_pLookupThread) {
		ASSERT(ValidHandle(m_pLookupThread->m_hThread));
		TerminateThread(m_pLookupThread->m_hThread, 666);
		m_pLookupThread = NULL;
		}
}


CStringx CLookup::MatchPhrases (const CStringx& buffer, const CStringx& pattern, CONST CHAR cDelim)
{
	DEBUG_OUT(TL_GARRULOUS, ("CLookup::MatchPhrases(%x, %s, %c)\n", 
			  buffer, F1(pattern), cDelim));
    CStringx delims (cDelim);
	return (MatchPhrases (buffer, pattern, delims));
}


CStringx CLookup::MatchPhrases (const CStringx& buffer, const CStringx& pattern, CONST CStringx& delims)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::MatchPhrases(%lx, '%s', '%s')\n", &buffer, F(pattern), delims));
    CStringx result = "";
//  INT nEnd;

    CStringArray phrases;
    TokenizeText(phrases, buffer, delims);
    for (INT i = 0; i < phrases.GetSize(); i++) {
		CStringx phrase (SkipSpaces(phrases[i]));
#if TRUE
		//      The cross-references have leading & trailing spaces which must be ignored
	phrase = phrase.Trim();
#endif
	if (Match(phrase, pattern)) {
//// convert result list to a string array
	    result += phrase + szENTRY_SEP;
	    }
	}
//  PurgeStringArray(phrases);

    return (result);
}


//  Match the mixed-case text against the pattern (uppercase assumed)
//
BOOL CLookup::Match (const CStringx& text, const CStringx& pattern)
{
	DEBUG_OUT(TL_GARRULOUS, ("CLookup::Match('%s', '%s')\n", F1(text), F2(pattern)));
    if (text.GetLength() != pattern.GetLength()) {
	return (FALSE);
	}

    for (INT nPos = 0; nPos < pattern.GetLength(); nPos++) {
		if ((ToUpper(text[nPos]) != pattern[nPos]) && (pattern[nPos] != '?')) {
			return (FALSE);
			}
		}

    return (TRUE);
}



//      Is the string an SGML tag (eg, a start tag <headword> or an endtag </headword>).
//
INLINE BOOL IsSgmlTag (const CStringx& sText)
{
	BOOL bIsTag = FALSE;
	INT nLen = sText.GetLength();
	if (nLen == 0) {
		bIsTag = FALSE;
		}
	else if ((sText[0] == cSGML_TAG_START) && (sText[nLen - 1] == cSGML_TAG_END))
		{
		bIsTag = TRUE;
		}

	return (bIsTag);
}


//      Helper function for ConvertLookupSgmlToRtf. SGML tags are converted
//      into their RTF representations.
//
VOID ProcessSgmlTag (CString& sLookupText, CONST CString& sTag, LOOKUP_TYPE eType, INT& nExtraSpaces)
{
	DEBUG_OUT(TL_GARRULOUS, ("ProcessSgmlTag(%x, %s, %d, %x)\n",
							 sLookupText, F(sTag), eType, &nExtraSpaces));
	if (sTag == SGML_START_TAG(HEADWORD)) {
		if (eType == eDEFINITION) {
			if (sLookupText.Find(szRTF_START_BOLD) != nINVALID_INDEX) {
				sLookupText += szRTF_NEW_PARA;
				}
			sLookupText += szRTF_START_BOLD;
			}
		}
	else if (sTag == SGML_END_TAG(HEADWORD)) {
		if (eType == eDEFINITION) {
			sLookupText += szRTF_END_BOLD;
			}
		}
	else if (sTag == SGML_START_TAG(BOLD)) {
		sLookupText += szRTF_START_BOLD;
		}
	else if (sTag == SGML_END_TAG(BOLD)) {
		sLookupText += szRTF_END_BOLD;
		}
	else if (sTag == SGML_START_TAG(ITALICS)) {
		sLookupText += szRTF_START_ITALICS;
		}
	else if (sTag == SGML_END_TAG(ITALICS)) {
		sLookupText += szRTF_END_ITALICS;
		nExtraSpaces += 2;
		}
	else if (sTag == SGML_START_TAG(PARA)) {
		if (eType != eDEFINITION) {
			sLookupText += cSEMI_COLON;
			}
		}
	else if (sTag == SGML_START_TAG(PART_OF_SPEECH)) {
		sLookupText += cSPACE;
		sLookupText += cSPACE;
		sLookupText += szRTF_START_BOLD;
		sLookupText += szRTF_START_ITALICS;
		}
	else if (sTag == SGML_END_TAG(PART_OF_SPEECH)) {
		sLookupText += szRTF_END_ITALICS;
		sLookupText += szRTF_END_BOLD;
		nExtraSpaces += 2;
		}
	else {
		//      Ignore other SGML tags
		}
}


//      Another helper function for ConvertLookupSgmlToRtf. This one
//      checks for numeric sense delimiter (eg, '3.'), which are made
//      boldface and part of speech labels (eg, 'adv.'), which are
//      made boldface and italicized.
//
VOID ProcessLinePrefixStuff (CStringx& sLookupText, CONST CStringx& sLine, CStringx& sRemLine, 
							 LOOKUP_TYPE eType, BOOL& nExtraSpaces)
{
	DEBUG_OUT(TL_GARRULOUS, ("ProcessLinePrefixStuff(%x, %s, %x, %d, %x)\n",
							 sLookupText, F(sLine), sRemLine, eType, &nExtraSpaces));
	INT nDotOffset = sLine.Find(cDOT);
	INT nRemPos = 0;
	if (nDotOffset != nINVALID_INDEX) {
		CONST CHAR szTYPES[] = "adj adv n vi vt interj "; // space delimits entries
		Assert(nDotOffset != 0);
		CStringx sToken(sLine, nDotOffset);
		// TODO: have an inplace TrimLeading() to avoid confusion?
		CStringx sTrimToken = sToken.TrimLeading();
		if (IsNumeric(sTrimToken)) {
			sLookupText += cSPACE;
			sLookupText += cSPACE;
			sLookupText += szRTF_START_BOLD;
			sLookupText += sTrimToken + cDOT;
			sLookupText += szRTF_END_BOLD;
			//sLookupText += szRTF_TAB;
			nRemPos += 1 + sToken.GetLength();
			}
		else if (Pos(szTYPES, sTrimToken + cSPACE) != nINVALID_INDEX) {
			sLookupText += cSPACE;
			sLookupText += cSPACE;
			sLookupText += szRTF_START_BOLD;
			sLookupText += szRTF_START_ITALICS;
			sLookupText += sTrimToken + cDOT;
			sLookupText += szRTF_END_ITALICS;
			sLookupText += szRTF_END_BOLD;
			nExtraSpaces += 2;
			nRemPos += 1 + sToken.GetLength();
			}
		}

	sRemLine += ((PCSZ)sLine + nRemPos);
}


// ToRtfPlainText: Convert the ascii text to RTF plain text. Characters treated
// special in RTF (eg, '{') must be preceded by a backslash ('\').
//
CString ToRtfPlainText (CStringx& sText)
{
	CString sPlainText;
	CONST CHAR szRTF_SPECIAL_CHARS[] = "\\{}";
	CONST CHAR cRTF_ESCAPE = '\\';

	for (INT nPos = 0; nPos < sText.GetLength(); nPos++) {
		if (Pos(szRTF_SPECIAL_CHARS, sText[nPos]) != nINVALID_INDEX) {
			sPlainText += cRTF_ESCAPE;
			}
		sPlainText += sText[nPos];
		}
	DEBUG_OUT(TL_GARRULOUS, ("ToRtfPlainText('%s')\n=> '%s'\n", F1(sText), F2(sPlainText)));

	return (sPlainText);
}


//      Convert the SGML lookup buffer into RTF. SGML tags are replaced
//      with the appropriate RTF format codes, and extraneous spacing is 
//      ignored. In addition, sense numbers and parts of speach are 
//      highlighted.
//
CString ConvertLookupSgmlToRtf (PCSZ pszText, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_VERBOSE, ("ConvertLookupSgmlToRtf('%s', %d)\n", F(pszText), eType));
	CONST CHAR szTEXT_DELIMITERS[] = ":,.";
#if FALSE // the caller adds this since several calls might be used
	CStringx sLookupText (szRTF_HEADER);
#else
	CStringx sLookupText;
#endif
	CStringx sCRNL ("\r\n");
	CStringx sText (pszText);
	INT i = 0;

	if (eType == eDEFINITION) {
		sLookupText += szRTF_HANGING_INDENT;            // small hanging indentation (1/8")
		}

	// Read the Lookup results buffer line by line, checking for SGML tags, which
	// are placed on separate lines. An extra space will be inserted whenever
	// there is a soft line break and after formatting italicized text.
	BOOL nExtraSpaces = 0;
	while (i < sText.GetLength()) {
		BOOL bFirstLine = (i == 0);
		CStringx sLine = ExtractToken(sText, i, sCRNL);
		sLine = sLine.TrimLeading();
		if (IsSgmlTag(sLine)) {
			ProcessSgmlTag(sLookupText, sLine, eType, nExtraSpaces);
			}
		else {
			CStringx sRemLine;
			ProcessLinePrefixStuff(sLookupText, sLine, sRemLine, eType, nExtraSpaces);
			sRemLine = sRemLine.Trim();
			if (!sRemLine.IsEmpty()) {
				if (Pos(szTEXT_DELIMITERS, sRemLine[0]) != nINVALID_INDEX) {
					nExtraSpaces = 0;
					}
				nExtraSpaces = min(2, nExtraSpaces);
				while (nExtraSpaces > 0) {
					sLookupText += cSPACE;
					nExtraSpaces--;
					}
				ASSERT(!IsSpace(sRemLine[0]));
				sLookupText += ToRtfPlainText(sRemLine);
				nExtraSpaces = 1;
				}
			}
		}

#if FALSE // the caller adds this since several calls might be used
	sLookupText += szRTF_TRAILER;
#endif

	return (sLookupText);
}


//      Convert the SGML lookup buffer into straight text. SGML tags are ignored.
//
CString ConvertLookupSgmlToText (PCSZ pszText, LOOKUP_TYPE eType)
{
	DEBUG_OUT(TL_VERBOSE, ("ConvertLookupSgmlToText('%s', %d)\n", F(pszText), eType));
	CStringx sLookupText;
	CStringx sCRNL ("\r\n");
	CStringx sText (pszText);
	INT i = 0;

	//      Read the Lookup results buffer line by line, checking for
	//      SGML tags (each is on a separate line).
	BOOL bNeedsSpace = FALSE;
	while (i < sText.GetLength()) {
		BOOL bFirstLine = (i == 0);
		CStringx sLine = ExtractToken(sText, i, sCRNL);
		sLine = sLine.TrimLeading();
		if (!IsSgmlTag(sLine)) {
			sLookupText += sLine;
			}
		else if ((sLine == SGML_START_TAG(PARA))
				 || (sLine == SGML_END_TAG(HEADWORD))) {
			if (eType != eDEFINITION) {
				sLookupText += cSEMI_COLON;
				}
			}
		}
	
	return (sLookupText);
}


//      Filter out garbage from the TRL lookup. In addition, 'format' the text into
//      lines.
//
CString WayUncoolFilterLookupGarbage (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("WayUncoolFilterLookupGarbage(%x)\n", pszText));
    CString sText;

	CHAR cLast = cSPACE;    // pretend last charcater was a space (to ignore leading spaces)
    while (*pszText) {
		CHAR cCurrent = *pszText;
		if (MyIsSpace(cCurrent)) {
			//      See if a newline sequence (\r\n) is acceptable.
			//              new sense number (3.) or new sense type (adj.)
			if (cCurrent == cRETURN) {
				INT nDotOffset = Pos(pszText, cDOT);
				if (nDotOffset != nINVALID_INDEX) {
					CONST CHAR szTYPES[] = "adj adv n vi vt interj";
					Assert(nDotOffset != 0);
					CStringx sToken(pszText, nDotOffset);
					// TODO: have an inplace TrimLeading() to avoid confusion?
					CStringx sTrimToken = sToken.TrimLeading();
					if (IsNumeric(sTrimToken)) {
						sText += szRTF_NEW_PARA;
						sText += szRTF_START_BOLD;
						sText += sTrimToken + cDOT;
						sText += szRTF_END_BOLD;
						sText += szRTF_TAB;
						pszText += sToken.GetLength();
						cCurrent = cSPACE;
						}
					else if (Pos(szTYPES, sTrimToken + ' ') != nINVALID_INDEX) {
						sText += szRTF_NEW_PARA;
						sText += szRTF_START_BOLD;
						sText += szRTF_START_ITALICS;
						sText += sTrimToken + cDOT;
						sText += szRTF_END_ITALICS;
						sText += szRTF_END_BOLD;
						pszText += sToken.GetLength();
						cCurrent = cDOT;
						}
					else {
						sText += cSPACE;
						cCurrent = cSPACE;              // make it a true space
						}
					}
				}
			else if (cCurrent == cNEW_LINE) {
				cCurrent = cSPACE;                              // make it a true space
				// skip
				}
			else if (cLast != cCurrent) {
				sText += cCurrent;
				}
			}
		else if (cCurrent == cDASH) {
			//      Convert a double dash sequence into a CR/NL sequence
			if (cLast != cRETURN) {
				cCurrent = cRETURN;
				}
			else {
				cCurrent = cNEW_LINE;
				}
			sText += cCurrent;
			}
	else if (!MyIsPrintable(cCurrent)) {
			//      Convert a double wide-dash sequence into a CR/NL sequence
			if ((BYTE)cCurrent == byWIDE_DASH) {
				if (cLast != cRETURN) {
					cCurrent = cRETURN;
					}
				else {
					cCurrent = cNEW_LINE;
					}
				sText += cCurrent;
				}
			else {
			//bSkip = TRUE;                         // skip nonprinting characters
				}
			}
		else {
			sText += cCurrent;
			}
		cLast = cCurrent;
	pszText++;                      // TODO: allow for DBCS
	}

    return (sText);
}


CString FilterWordNetGarbage (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterWordNetGarbage(%x)\n", pszText));
    CString sText;

    while (*pszText) {
	CHAR cNext = *pszText;
	if (cNext == '\n') {
	    sText += "\r\n";
	    }
	else if (MyIsPrintable(cNext)) {
	    sText += cNext;
	    }
	pszText++;
	}

    return (sText);
}


CString GetWordNetHeads (PCSZ pszPhrase, PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("GetWordNetHeads(%x)\n", pszText));
	CString sHeads;
	CStringArray lines;
	TokenizeText(lines, pszText, "\r\n");

	for (INT i = 0; i < lines.GetSize(); i++) {
		CONST CHAR szSENSE[] = "Sense";

		// Check for the start of a new sense group
		if (Pos(lines[i], szSENSE) == 0) {
			// Skip over the sense indicator line ("Sense 2")
			i++;

			// Isolate the list of head words (sans the definition)
			//    "cad, bounder, ... heel -- (someone who is morally reprehesible)"
			INT nEnd = Pos(lines[i], cDASH);
			if (nEnd == nINVALID_INDEX) {
				nEnd = lines[i].GetLength();
			}
			CStringx sText (lines[i], nEnd);

			// Extract the individual head words
			INT nPos = 0;
			while (nPos < sText.GetLength()) {
				CString sHead = ExtractToken(sText, nPos, ",");
				sHead.TrimLeft();
				sHead.TrimRight();
				// Include the head if not the source phrase
				if (sHead.CompareNoCase(pszPhrase) != 0) {
					// TODO: eliminate duplicates
					sHeads += sHead;
					sHeads += szENTRY_SEP;
				}
			}
		}
	}

    return (sHeads);
}


CString GetWordNetRelations (PCSZ pszPhrase, PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("GetWordNetRelations(%x)\n", pszText));
	UNREFERENCED_PARAMETER(pszPhrase);
	CString sRelations;

	// Split the buffer into lines
	// TODO: eliminate extra buffer since the tree can be quite large
	CStringArray lines;
	TokenizeText(lines, pszText, "\r\n");

	for (INT i = 0; i < lines.GetSize(); i++) {
		CONST CHAR szARROW[] = " => ";

		// Check for the start of a new sense group
		INT nPos = Pos(lines[i], szARROW);
		if (nPos != nINVALID_INDEX) {
			// Isolate the list of relations words (sans the definition)
			//    " => location -- (a point or extent in space)"
			INT nEnd = Pos(lines[i], cDASH);
			if (nEnd == nINVALID_INDEX) {
				nEnd = lines[i].GetLength();
			}
			CStringx sText (lines[i], nEnd);

			// Extract the individual head words
			nPos += STR_LEN(szARROW);
			while (nPos < sText.GetLength()) {
				CString sRelation = ExtractToken(sText, nPos, ",");
				sRelation.TrimLeft();
				sRelation.TrimRight();
				sRelations += sRelation;
				sRelations += szENTRY_SEP;
				}
			}
		}

    return (sRelations);
}


// FilterBookshelfTextEntry: Filter the plain text returned from a Bookshelf
// lookup (via MediaView)
//
// TODO:  convert this into SGML to take advantage of SgmlToRtf routines
//
CString FilterBookshelfTextEntry (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterBookshelfTextEntry(%x)\n", pszText));
    CString sText;

	INT nPos = 0;

	// Skip over the header text
	CString sJunk;
	do {
		sJunk.Empty();
		sJunk = ExtractToken(pszText, nPos, "[]");
		sJunk += ExtractToken(pszText, nPos, "<>");
		DEBUG_OUT(TL_VERBOSE, ("Ignoring entry header: %s\n", F(sJunk)));
	} while (!sJunk.IsEmpty());

	// Get the main text
	while (pszText[nPos]) {
		CString sLine = ExtractToken(pszText, nPos, "\r\n");
		sText += sLine;
	}

    return (sText);
}


CString FilterBookshelfSynsEntry (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterBookshelfSynsEntry(%x)\n", pszText));
    CString sText;

	// Get the main text
	INT nPos = 0;
	while (pszText[nPos]) {
		if (pszText[nPos] == cCOMMA) {
			sText += cMV_ENTRY_SEP;
			}
		else {
			sText += pszText[nPos];
			}
		nPos++;
	}

    return (sText);
}


CString FilterBookshelfWordList (PCSZ pszText)
{
	DEBUG_OUT(TL_VERBOSE, ("FilterBookshelfWordList(%x)\n", pszText));
    CString sText;

	// Get the main text
	INT nPos = 0;
	while (pszText[nPos]) {
		// Extract the next phrase from the text
		SkipSpaces(pszText, nPos);
		CStringx sPhrase = ExtractToken(pszText, nPos, cMV_ENTRY_SEP);
		INT nStart = 0;
		SkipSpaces(sPhrase, nStart);
		if (nStart == sPhrase.GetLength()) {
			break;
		}

		// Ignore the category number and the sense indicator
		ASSERT(IsDigit(sPhrase[nStart]));
		nStart++;
		CONST CHAR cLEFT_PAREN = '(';
		INT nEnd = sPhrase.FindAt(nStart, cLEFT_PAREN);
		INT nLen = ((nEnd == nINVALID_INDEX) 
						? (sPhrase.GetLength() - nStart + 1)
						: (nEnd - nStart));
		sPhrase = sPhrase.Mid(nStart, nLen);
		sText += sPhrase;
		sText += szENTRY_SEP;
	}

    return (sText);
}


//      Workhorse routine for doing actual lookups. For each active information source, a
//      request is submitted. Afterwards, the results are optionally converted into RTF from
//      the pseudo-SGML format of the raw lookup results.
//
void CLookup::LookupClue (const CStringx& sSource, LOOKUP_TYPE eType, const CStringx& phrase, BOOL bFormat)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LookupClue(%s, %d, %s, %d)\n", 
			  F1(sSource), eType, F2(phrase), bFormat));
    //  change the mouse cursor to the supplied cursor or the hourglass
    HCURSOR hOldCursor = NULL;
#if FALSE
	//      OBSOLETE: the client now maintains the cursor (see comments in stub for timer proc below)
    if (m_nCursorID == 0) {
		AfxGetApp()->BeginWaitCursor();
	}
    else {
		hOldCursor = ::SetCursor(::LoadCursor(AfxGetInstanceHandle(), 
					      MAKEINTRESOURCE(m_nCursorID)));
	}
    CONST INT nTIME_OUT = 300;

    LOOKUP_TYPE eType = 
		(type.CompareNoCase("Definition") == 0) ? eDEFINITION
		: (type.CompareNoCase("Synonyms") == 0) ? eSYNONYMS
		: (type.CompareNoCase("CrossReferences") == 0) ? eCROSS_REFS
			: (type.CompareNoCase("Descendants") == 0) ? eDESCENDANTS
		:                                         eNONE;
#endif
    
    //LoadLookup();
    buffer.Empty();
    if (bFormat) {
	    buffer += szRTF_HEADER;
    }

    //  Look up the phrase
    if (m_bUseTRL
		&& ((eType == eDEFINITION) || (eType == eSYNONYMS) || (eType == eCROSS_REFS))) {
		BOOL bOK = ::LookupPhrase(m_pLI, phrase, sSource, eType);
    
		// Copy the results
		if (bOK) {
			PCSZ pszResults = GetResultsPtr(m_pLI);
			ASSERT(pszResults);
			//buffer += ::FilterLookupGarbage(pszResults);
			buffer += (bFormat
				? ConvertLookupSgmlToRtf(pszResults, eType)
				: ConvertLookupSgmlToText(pszResults, eType));
			}
	}

    //  Add info from a WordNet search
	//      TODO: add an intermediate layer to format the WordNet results into SGML
    if (m_bUseWordNet && (eType == eSYNONYMS)) {
		CString sWordNet;
		if (!wninit()) {
			CString sPhrase = phrase;
			DEBUG_OUT(TL_VERBOSE, ("Initiating WordNet search for %s\n", (PCSZ)sPhrase));
			for (INT nPartOfSearch = NOUN; nPartOfSearch < ADJSAT; nPartOfSearch++) {
				PCSZ pszResults;
				PSZ pszPhrase = (PSZ)(PCSZ)sPhrase;
				pszResults = findtheinfo(pszPhrase, nPartOfSearch, SIMPTR, ALLSENSES);
				ASSERT(pszResults);
				sWordNet += GetWordNetHeads(pszPhrase, pszResults);
#if FALSE
				sWordNet += ::FilterWordNetGarbage(pszResults);
				pszResults = findtheinfo(pszPhrase, nPartOfSearch, HYPERPTR, ALLSENSES);
				ASSERT(pszResults);
				sWordNet += ::FilterWordNetGarbage(pszResults);
#endif
				}
			wnclose();
			}
		else {
			DEBUG_OUT(TL_BASIC, ("wn: error - cannot open WordNet database\n"));
		}
		// TODO: add optional RTF formatting
		buffer +=  sWordNet;
		}

	// Check for WordNet hyponym tree
	// TODO: reduce redundancy with synonym check
    if (m_bUseWordNet && (eType == eDESCENDANTS)) {
		CString sWordNet;
		if (!wninit()) {
		    CString sPhrase = phrase;
			DEBUG_OUT(TL_VERBOSE, ("Initiating WordNet hyponym search for %s\n", (PCSZ)sPhrase));
			for (INT nPartOfSearch = NOUN; nPartOfSearch < ADJSAT; nPartOfSearch++) {
				PCSZ pszResults;
				PSZ pszPhrase = (PSZ)(PCSZ)sPhrase;
				pszResults = findtheinfo(pszPhrase, nPartOfSearch, -HYPOPTR, ALLSENSES);
				ASSERT(pszResults);
				sWordNet += GetWordNetRelations(pszPhrase, pszResults);
			}
		wnclose();
		}
		else {
			DEBUG_OUT(TL_BASIC, ("wn: error - cannot open WordNet database\n"));
		}
		// TODO: add optional RTF formatting
		buffer += sWordNet;
    }


	if (m_bUseBookshelf && (eType == eDEFINITION)) {
		m_pMvInfo->MtOpenWordWheel(szDICTIONARY);
		CString sBookshelfInfo = m_pMvLookup->LookupDefinitionPhrase(phrase);
		//sBookshelfInfo = FilterBookshelfTextEntry(sBookshelfInfo);
		buffer += (bFormat
				? ConvertLookupSgmlToRtf(sBookshelfInfo, eType)
				: ConvertLookupSgmlToText(sBookshelfInfo, eType));
	}

	if (m_bUseBookshelf && (eType == eSYNONYMS)) {
		m_pMvInfo->MtOpenWordWheel(szTHESAURUS);
		CString sBookshelfInfo = m_pMvLookup->LookupPhrase(phrase);
		sBookshelfInfo = FilterBookshelfSynsEntry(sBookshelfInfo);
		buffer += (bFormat
				? ConvertLookupSgmlToRtf(sBookshelfInfo, eType)
				: ConvertLookupSgmlToText(sBookshelfInfo, eType));
	}

	if (m_bUseBookshelf && (eType == eCROSS_REFS)) {
		m_pMvInfo->MtOpenTopicList(szDICTIONARY);
		CString sBookshelfInfo = m_pMvLookup->LookupCrossReferences(phrase);
		//buffer += FilterBookshelfWordList(sBookshelf);
		buffer += (bFormat
				? ConvertLookupSgmlToRtf(sBookshelfInfo, eType)
				: ConvertLookupSgmlToText(sBookshelfInfo, eType));
	}

    if (bFormat) {
	    buffer += szRTF_TRAILER;
    }

    DEBUG_OUT(TL_VERBOSE, ("Lookup(%s, %d, %s)", 
			F1(sSource), eType, F2(phrase)));
    DEBUG_OUT(TL_VERBOSE, (" => %s\n", F(buffer)));
    //AFX_DUMPX(5, buffer);

#if FALSE
    //  OBSOLETE: restore the mouse cursor
    if (m_nCursorID == 0) {
		AfxGetApp()->EndWaitCursor();
	}
    else {
		::SetCursor(hOldCursor);
	}
#endif
}


void CLookup::StartLookup (const CStringx& sSource, LOOKUP_TYPE eType, const CStringx& phrase, UINT uCommandID, BOOL bFormat)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::StartLookup(%s, %d, %s, %d, %d)\n", 
			  F1(sSource), eType, F2(phrase), uCommandID, bFormat));
	//BOOL bUseThread = !IsKeyToggled(VK_SCROLL);
	if (bUseThreads) {
		m_sSource = sSource;
		m_eLookupType = eType;
		m_sPhrase = phrase;
		m_bFormat = bFormat;
		m_uNotificationID = uCommandID;
		if (m_pLookupThread == NULL) {
			m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoLookupProc, this);
			}
		}
	else {
	    LookupClue(sSource, eType, phrase, bFormat);
		SendNotification(uCommandID);
		}
}


void CLookup::StartDefLookup (const CStringx& phrase, UINT uCommandID, BOOL bFormat)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::StartDefLookup(%s, %d, %d)\n", 
			  F1(phrase), uCommandID, bFormat));
	m_sPhrase = phrase;
	m_bFormat = bFormat;
	m_uNotificationID = uCommandID;

	//BOOL bUseThreads = GetDebugOption("Multithreaded", TRUE);
	if (bUseThreads) {
		if (m_pLookupThread == NULL) {
			m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoDefLookupProc, this);
			}
		}
	else {
		DoDefinitionLookup();
	    //LookupClue(sSource, eType, phrase, uCommandID, bFormat);
		SendNotification(uCommandID);
		}
}


void CLookup::StartSuggestions (const CStringx& sClue, UINT uCommandID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::StartSuggestions(%s, %d)\n", 
			  F1(sClue), uCommandID));
	m_sClue = sClue;
	m_uNotificationID = uCommandID;

	//BOOL bUseThread = !IsKeyToggled(VK_SCROLL);
	if (bUseThreads) {
		if (m_pLookupThread == NULL) {
			m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoSuggestionsProc, this);
			}
		}
	else {
		DoSuggestions();
		SendNotification(uCommandID);
		}
}


void CLookup::GetSynonyms (const CStringx& phrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetSynonyms(%s, %d, %d)\n", 
			  F(phrase), uCommandID, bFormat));
    StartLookup("THES2", eSYNONYMS, phrase, uCommandID, bFormat);
}


void CLookup::GetReferences (const CStringx& phrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetReferences(%s, %d, %d)\n", 
			  F(phrase), uCommandID, bFormat));
    StartLookup("DICT1", eCROSS_REFS, phrase, uCommandID, bFormat);
}

void CLookup::GetDescendants (const CStringx& phrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetReferences(%s, %d, %d)\n", 
			  F(phrase), uCommandID, bFormat));
    StartLookup("N/A", eDESCENDANTS, phrase, uCommandID, bFormat);
}


// NoDefinition: Determine if the buffer has no definition in it.
// If there is no formatting then the buffer should be blank if there is
// no definition. Otherwise, there should be no headword formatting (boldface).
//
// TODO: fix the lookup to not return a valid rtf buffer upon failuure
//
inline BOOL HasNoDefinition (CStringx& sBuffer, BOOL bFormat)
{
	BOOL bHasNoDefinition = FALSE;

	if ((sBuffer.LastNonblank() == -1)
		|| (bFormat && (sBuffer.Find(szRTF_START_BOLD) == nINVALID_INDEX))) {
		bHasNoDefinition = TRUE;
		}

	return (bHasNoDefinition);
}


// Get the definition for the phrase from the TRL reference. This
// first checks looks for the first occurrence of a multiple entry
// phrase (eg, 'cant' has three separate entries) by explicitly
// specifying the occurence indicator ('<word>{1}' as in 'cant{1}').
// If this search fails, the word is searched without the indicator.
// Then if this also fails, the searches are repeated using an
// alternative index source.
//
// NOTE: TRL has two main indexes for the dictionary (two others
//		 are not relevant). One index provides for words used in multiple
//		 entries (eg, 'acid test', 'intelligence test', etc.). Another
//		 goes directly to a word.
//
// TODO: decouple the TRL dependencies from this routines since Bookshelf and
//       WordNet will also be used for definition text
//
void CLookup::DoDefinitionLookup (BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DoDefinitionLookup()\n"));
	CONST CHAR szSOURCE[] = "DICT4";
	CONST CHAR szALT_SOURCE[] = "DICT3";
    CString sSource = szSOURCE;
	CString sAltSource = szALT_SOURCE;
#if defined(_DEBUG)
    if (IsAltPressed()) {
		CString sSource = szALT_SOURCE;
		CString sAltSource = szSOURCE;
	}
#endif

    //  First try with the sense indicator (eg, TEST{1})
    CString sPhrase1 = m_sPhrase + "{1}";
    LookupClue(sSource, eDEFINITION, m_sPhrase, bFormat);

	// If no luck, try with the unadorned word (eg, TEST)
    if (HasNoDefinition(buffer, bFormat)) {
	    LookupClue(sSource, eDEFINITION, m_sPhrase, bFormat);
		}

	// If still nothing returned, try the alternative dictionary index
    if (HasNoDefinition(buffer, bFormat)) {
		DEBUG_OUT(TL_VERBOSE, ("Retrying alternate definition lookup (%s) using sense indicator: %s\n", 
				  F1(sAltSource), F2(sPhrase1)));
		LookupClue(sAltSource, eDEFINITION, sPhrase1, bFormat);
	}
    if (HasNoDefinition(buffer, bFormat)) {
		DEBUG_OUT(TL_VERBOSE, ("Retrying alternate lookup (%s): %s\n", 
				  F1(sAltSource), F2(m_sPhrase)));
		LookupClue(sAltSource, eDEFINITION, m_sPhrase, bFormat);
	}
}


void CLookup::GetDefinition (const CStringx& sPhrase, UINT uCommandID /* = 0 */, BOOL bFormat /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::GetDefinition(%s, %d, %d)\n", 
			  F(sPhrase), uCommandID, bFormat));
	StartDefLookup(sPhrase, uCommandID, bFormat);
}


void CLookup::DeriveSuggestions (const CStringx& clue, UINT uCommandID /* = 0 */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DeriveSuggestions('%s', %d)\n", 
			  F(clue), uCommandID));
	StartSuggestions(clue, uCommandID);
}


void CLookup::DoSuggestions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DoSuggestions()\n"));
    CStringx phrase = m_sClue.Trim();

    CStringArray words;
    TokenizeText(words, phrase, ", ;.");
    suggestions = "";
    for (INT i = 0; i < words.GetSize(); i++) {
		CStringx word (words[i]);
		m_sPhrase = word;
		//GetSynonyms(word, uNULL_COMMAND, FALSE);
	    LookupClue("THES2", eSYNONYMS, m_sPhrase, FALSE);
		suggestions += MatchPhrases(buffer, pattern, ';');
		//GetDefinition(word, uNULL_COMMAND, FALSE);
		DoDefinitionLookup(FALSE);
		suggestions += MatchPhrases(buffer, pattern, " ;");
		//GetReferences(word, uNULL_COMMAND, FALSE);
	    LookupClue("DICT2", eCROSS_REFS, m_sPhrase, FALSE);
		suggestions += MatchPhrases(buffer, pattern, ';');
		if (IsControlPressed()) {
			LookupClue("N/A", eDESCENDANTS, m_sPhrase, FALSE);
			suggestions += MatchPhrases(buffer, pattern, ';');
		}
	}
	DEBUG_OUT(TL_VERBOSE, ("DoSuggestions() => %s\n", F(suggestions)));
}


#if FALSE
BOOL CLookup::LoadLookup (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LoadLookup()\n"));
#if FALSE
    CStringx lookup = "LOOKUP.EXE";
    CStringx path = GetLookupDirName() + "\\";
    CStringx arguments = "/tLOOKUP.TXT /b /&" + path + " "
			+ "/iDICT1 /iDICT3 /iDICT4 /iTHES2";
    
    HMODULE hLookupMod = ::GetModuleHandle(lookup);
    if (!hLookupMod) {
	CStringx CommandLine = path + lookup + " " + arguments;
//// this seems to make the application lose focus
	hLookupMod = (HMODULE)::WinExec(CommandLine, SW_MINIMIZE);
	SetActiveWindow(GetMainHwnd());
	if (hLookupMod < HINSTANCE_ERROR) {
	    AfxMessageBox("Unable to load lookup program");
	    return (FALSE);
	    }
	}

    return (TRUE);
#endif
    return (FALSE);
}
#endif


// InitLookup: Initialize the static portions of the class. This is done
// via a separate thread due to the quirky MvInfo interface and for
// efficiency (from the user's point of view)
//
void CLookup::InitLookup (CWnd* pWnd)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitLookup(%x)\n", pWnd));

	m_pStaticParentWnd = pWnd;
	bUseThreads = GetDebugOption("Multithreaded", TRUE);
	if (bUseThreads) {
		m_pInitThread = AfxBeginThread((AFX_THREADPROC)::DoInitProc, NULL);
		}
	else {
		DoInitProc(NULL);
		}
}


// InitLookupAux: Initialize the information sources that will be used
//
// NOTE: This is done in a separate thread from the main one so no CWnd
//		 based objects should be saved (due to thread-local storage HWND
//		 maps)
//
BOOL CLookup::InitLookupAux (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::InitLookupAux()\n"));

    LoadProfileSettings();
    
    //  Initialize the DLL for accessing the Software Toolwork's Reference Library
    if (m_bUseTRL) {
		HINSTANCE hInst = AfxGetApp()->m_hInstance;
		HWND hMainWnd = AfxGetApp()->m_pMainWnd ? AfxGetApp()->m_pMainWnd->m_hWnd : NULL;
		CString sDataDirectory(GetAppOption("TrlDataDirectory", "D:\\TRL"));
		CString sIndexDirectory(GetAppOption("TrlIndexDirectory", GetProgramDir()));
		m_pLI = ::InitLookup(hInst, hMainWnd, "Lookup", sDataDirectory, sIndexDirectory);
		ASSERT(m_pLI);
		m_bUseTRL = (m_pLI != NULL);
	}

    //  Initialize WordNet support
    //m_bUseWordNet = TRUE;
#if TRUE    
    if (m_bUseWordNet && wninit()) {
		DEBUG_OUT(TL_BASIC, ("wn: error - cannot open WordNet database\n"));
		m_bUseWordNet = FALSE;
	}
#endif


	if (m_bUseBookshelf) {
		InitBookshelf();
	}

	return (TRUE);
}


void CLookup::CleanupLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::CleanupLookup()\n"));

    //  Shutdown the TRL search engine
    if (m_bUseTRL && m_pLI) {
		ASSERT(m_pLI);
		HINSTANCE hInst = AfxGetApp()->m_hInstance;
		::EndLookup(hInst, m_pLI);
		m_pLI = NULL;
	}

    //  Shutdown the WordNet search engine
    if (m_bUseWordNet) {
		//  nothing to do
	}

	if (m_bUseBookshelf) {
		CleanupBookshelf();
	}
}


void CLookup::LoadProfileSettings (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::LoadProfileSettings()\n"));
//  CString GetProfileString( LPCSTR lpszSection, LPCSTR lpszEntry, LPCSTRlpszDefault = NULL );
#if FALSE
    lookup_dir = AfxGetApp()->GetProfileString("Program_Dirs", "Lookup", "\\CWORD");
    CString temp_path = GetTempPath() + "\\";
    lookup_file = temp_path + AfxGetApp()->GetProfileString("Lookup_Interface", "Lookup_File", "LOOKUP.TXT");
    results_file = temp_path + AfxGetApp()->GetProfileString("Lookup_Interface", "Results_File", "RESULTS.TXT");
//#else
    m_bUseWordNet = AfxGetApp()->GetProfileInt("Options", "UseWordNet", TRUE);
    m_bUseTRL = AfxGetApp()->GetProfileInt("Options", "UseTRL", TRUE);
    m_bUseBookShelf = AfxGetApp()->GetProfileInt("Options", "UseBookShelf", FALSE);
#endif
    m_bUseWordNet = GetAppOption("UseWordNet", TRUE);
    m_bUseTRL = GetAppOption("UseTRL", TRUE);
    m_bUseBookshelf = GetAppOption("UseBookShelf", TRUE);
	m_dwTimeOut = GetAppOption("TimeOut", 120);		// default to 120 seconds
}


//      Send a notification to the client that the lookup results are ready
//
void CLookup::SendNotification (UINT uCommandID)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::SendNotification(%u)\n", uCommandID));

	CWnd* pClientWnd = CWnd::FromHandle(m_hClientWnd);
	ASSERT((!pClientWnd) || pClientWnd->IsKindOf(RUNTIME_CLASS(CWnd)));
	if (pClientWnd 
			&& pClientWnd->IsKindOf(RUNTIME_CLASS(CWnd))
			&& (uCommandID != 0)) {
		pClientWnd->PostMessage(WM_COMMAND, uCommandID, 0);
		}
}


//-----------------------------------------------------------------------------
//      Other (helper) functions
//-----------------------------------------------------------------------------


// Thread procedure for doing static initialization of the info sources
//
STATIC UINT DoInitProc (PVOID pData)
{
	DEBUG_OUT(TL_VERBOSE, ("DoInitProc(%x)\n", pData));

	BOOL bOK = CLookup::InitLookupAux();

	return (bOK);
}


// Thread procedure for looking up phrases.
//
STATIC UINT DoLookupProc (CLookup* pLookup)
{
	DEBUG_OUT(TL_VERBOSE, ("DoLookupProc(%x)\n", pLookup));
	BOOL bOK = FALSE;

	ASSERT(pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup)));
	if (pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup))) {
		pLookup->LookupClue(pLookup->m_sSource,             // reference source (TRL-specific)
							pLookup->m_eLookupType,         // type of lookup (eg, cross refs)
							pLookup->m_sPhrase,             // phrase to lookup
							pLookup->m_bFormat);            // format results using RTF
		pLookup->SendNotification(pLookup->m_uNotificationID);
		pLookup->m_pLookupThread = NULL;
		bOK = TRUE;
		}

	return (bOK);
}


// Thread procedure for handling special case of definition lookup
//
STATIC UINT DoDefLookupProc (CLookup* pLookup)
{
	DEBUG_OUT(TL_VERBOSE, ("DoLookupProc(%x)\n", pLookup));
	BOOL bOK = FALSE;

	ASSERT(pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup)));
	if (pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup))) {
		pLookup->DoDefinitionLookup();
		pLookup->SendNotification(pLookup->m_uNotificationID);
		pLookup->m_pLookupThread = NULL;
		bOK = TRUE;
		}

	return (bOK);
}


//      Thread procedure for handling special case of deriving suggestions
//
STATIC UINT DoSuggestionsProc (CLookup* pLookup)
{
	DEBUG_OUT(TL_VERBOSE, ("DoSuggestionsProc(%x)\n", pLookup));
	BOOL bOK = FALSE;

	ASSERT(pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup)));
	if (pLookup && pLookup->IsKindOf(RUNTIME_CLASS(CLookup))) {
		pLookup->DoSuggestions();
		pLookup->SendNotification(pLookup->m_uNotificationID);
		pLookup->m_pLookupThread = NULL;
		bOK = TRUE;
		}

	return (bOK);
}


//      Unused timer procedure
//
#if FALSE
void EXPORT CALLBACK TimerProc (HWND hWnd, UINT nMsg, UINT nIDEvent, DWORD dwTime)
{
	// TODO: map hWnd into lookup instance via list of lookup pClientWnd's
	// NOTE: perhaps its better for the client to maintain the cursor since they
	//       might want to have several lookup's going on simulataneously
	//       (eg, for the poor fellow with the dual pentium and numerous GB drives)
}
#endif
