// MediaView.cpp : implementation file
//
// Based on player sample from mv14 sdk
//

#include "stdafx.h"
#include "MvLookup.h"
#include "MediaView.h"
#include "CommonLookup.h"

IMPLEMENT_DYNAMIC(CKeyInfo, CObject);
IMPLEMENT_DYNAMIC(CMediaView, CObject);

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Constants
//
CONST CHAR szSENSE_FMT[] = "{%d}";
CONST INT nMAX_NAME = 512;


// Parts of speech labels (in a string format suitable for a resource)
// NOTE: the list must be ordered so that the most specific entries
// come first (that is, "plural noun" before "noun")
//
CONST CHAR szPartsOfSpeech[] = 
	"adjective\t"
	"adverb\t"
	"conjunction\t"
	"definite article\t"
	"indefinite article\t"
	"interjection\t"
	"plural noun\t"
	"preposition\t"
	"pronoun\t"
	"singular noun\t"
	"verb, intransitive\t"
	"verb, transitive\t"
	"noun\t"
	"verb\t"
	;


CKeyInfo::CKeyInfo (CONST CString& sExternalName, CONST CString& sInternalName, CONST CString& sInfo)
{
	DEBUG_OUT(TL_GARRULOUS, ("CKeyInfo::CKeyInfo(%s, %s, %s): this=%x\n", 
			 (PCSZ)sExternalName, (PCSZ)sInternalName, (PCSZ)sInfo, this));

	m_sExternalName = sExternalName;
	m_sInternalName = sInternalName;
	m_sInfo = sInfo;
}


CKeyInfo::CKeyInfo (VOID)
{
	DEBUG_OUT(TL_GARRULOUS, ("CKeyInfo::CKeyInfo(): this=%x\n"));
}


CKeyInfo::~CKeyInfo (VOID)
{
	DEBUG_OUT(TL_GARRULOUS, ("CKeyInfo::~CKeyInfo(): this=%x\n", this));
}


CONST CKeyInfo& CKeyInfo::operator= (CONST CKeyInfo& other)
{
	DEBUG_OUT(TL_GARRULOUS + 3, ("CKeyInfo::operator=(%x): this=%x\n", &other, this));


	m_sExternalName = other.m_sExternalName;
	m_sInternalName = other.m_sInternalName;
	m_sInfo = other.m_sInfo;

	return (*this);
}


/////////////////////////////////////////////////////////////////////////////
// CMediaView class

CMediaView::CMediaView (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CMediaView()\n"));

	m_hWndScrollable = NULL;
	m_hWndNonScrollable = NULL;
	m_hTitle = NULL;
	m_hTopicList = NULL;
	m_hWordWheel = NULL;
	m_pMvNonScrollable = NULL;
	m_pMvScrollable = NULL;

	// Fill in the parts of speech array
	TokenizeText(m_asPartOfSpeech, szPartsOfSpeech, szTAB);

	// Initialize access to MediaView and set up the MediaView objects
	InitMediaView();
}


CMediaView::~CMediaView (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::~CMediaView()\n"));

	CleanupMediaView();
}



// Init MediaView access
//
VOID CMediaView::InitMediaView (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::InitMediaView()\n"));

	// Set the instance handle
	::MVSetInstance(AfxGetInstanceHandle());

	m_hWndNonScrollable = CreateDummyWindow();
	m_hWndScrollable = CreateDummyWindow();

	// Create MediaView objects for rendering the scrollable and nonscrollable 
	// parts of the topic.
	BOOL bAssocOK;
	m_pMvNonScrollable = lpMVNew(NULL, &m_err);
	bAssocOK = fMVSetWindow(m_pMvNonScrollable, m_hWndNonScrollable, &m_err);
	ASSERT(bAssocOK);
	m_pMvScrollable = lpMVNew(NULL, &m_err);
	bAssocOK = fMVSetWindow(m_pMvScrollable, m_hWndScrollable, &m_err);
	ASSERT(bAssocOK);

	// Open the title
	if (!m_sFile.IsEmpty()) {
		OpenTitle();
	}
}
	

// Clean up MediaView stuff
//
VOID CMediaView::CleanupMediaView (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CleanupMediaView()\n"));

	CloseTitle();

	// delete the MediaView objects
	if (m_pMvNonScrollable) {
		MVDelete(m_pMvNonScrollable);
		m_pMvNonScrollable = NULL;
		}
	if (m_pMvScrollable) {
		MVDelete(m_pMvScrollable);
		m_pMvScrollable = NULL;
		}

	// delete the dummy windows
	if (m_hWndNonScrollable) {
		::DestroyWindow(m_hWndNonScrollable);
		m_hWndNonScrollable = NULL;
		}
	if (m_hWndScrollable) {
		::DestroyWindow(m_hWndScrollable);
		m_hWndScrollable = NULL;
		}

	::MVTerminate(AfxGetInstanceHandle());
}


BOOL CMediaView::OpenTitle (CONST CString& sTitle)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::OpenTitle(%s)\n", F(sTitle)));
	m_sFile = sTitle;

	// Close the existing title, if any
	OpenTitle();

	return (m_hTitle != NULL);
}


VOID CMediaView::OpenTitle (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::OpenTitle()\n"));

	// Close the existing title, if any
	CloseTitle();

	// Open the title file
	// TODO: add an interrupt call back
	m_hTitle = hMVTitleOpen(m_sFile);
	if (m_hTitle == NULL) {
		CString sError;
		sError.Format("Unable to open the title:\n\t'%s'", (PCSZ)m_sFile);
		AfxMessageBox(sError);
		return;
	}

	// Get the title name and use it in the app title
	PSZ pszName = m_sTitleName.GetBuffer(nMAX_NAME);
	lMVTitleGetInfo(m_hTitle, TTLINF_TITLE, nMAX_NAME, (LONG)pszName);
	m_sTitleName.ReleaseBuffer();

	// Associate the title with the MediaView object
	fMVSetTitle(m_pMvNonScrollable, m_hTitle, m_sTitleName, &m_err);
	fMVSetTitle(m_pMvScrollable, m_hTitle, m_sTitleName, &m_err);

	// Extract the searchable group titles from the title and add to the list
	// Based on MV_LoadGroups in MediaView sample
	LRESULT lRes = 0;
	INT nGroup;
	DEBUG_OUT(TL_VERBOSE, ("Title Groups:\n"))
	for (nGroup = 0; lRes != -1; nGroup++) {
		CStringx sGroupInfo;
		PSZ pszGroup = sGroupInfo.GetBuffer(nMAX_NAME);
		lRes = ::lMVTitleGetInfo(m_hTitle, TTLINF_GROUPS, MAKELONG(nMAX_NAME, nGroup),
								(LONG)pszGroup);
		sGroupInfo.ReleaseBuffer();
		if (lRes != -1) {
			DEBUG_OUT(TL_VERBOSE, ("Group %d: %s\n", nGroup, (PCSZ)sGroupInfo))
			// Extract the group name & title ('name.grp "title"')
			INT nPos = 0;
			CString sInternalName = ExtractToken(sGroupInfo, nPos, '.');
			nPos = sGroupInfo.FindAt(nPos, '\"');
			CString sExternalName = ExtractToken(sGroupInfo, nPos, ";");

			// Associate the group name with the title
			if (!sExternalName.IsEmpty()) {
				CKeyInfo key_info (sExternalName, sInternalName, sGroupInfo);
				m_mapTopicGroups.SetAt(sExternalName, key_info);
			}
		}
	}

	// Get the keyword groups
	DEBUG_OUT(TL_VERBOSE, ("Keyword Groups:\n"))
	CHAR cGroup = '?';
	for (nGroup = 0; cGroup != '\0'; nGroup++) {
		CStringx sGroupInfo;
		PSZ pszGroup = sGroupInfo.GetBuffer(nMAX_NAME);
		cGroup = (CHAR)::lMVTitleGetInfo(m_hTitle, TTLINF_KEYINDEX, 
										 MAKELONG(nMAX_NAME, nGroup), (LONG)pszGroup);
		sGroupInfo.ReleaseBuffer();
		if (lRes != 0) {
			DEBUG_OUT(TL_VERBOSE, ("Group %d: %s\n", nGroup, (PCSZ)sGroupInfo));
			CString& sExternalName = sGroupInfo;
			CString sInternalName;
			sInternalName.Format("|%c", cGroup);
			CKeyInfo key_info (sExternalName, sInternalName, sGroupInfo);
			m_mapKeywordGroups.SetAt(sExternalName, key_info);
		}
	}
}


BOOL CMediaView::CloseTitle (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CloseTitle()\n"));
	BOOL bOK = FALSE;

	if (m_hTitle != NULL) {
		::MVTitleClose(m_hTitle);
		m_hTitle = NULL;
		bOK = TRUE;
	}
	CloseWordWheel();
	CloseTopicList();
	m_mapKeywordGroups.RemoveAll();
	m_mapTopicGroups.RemoveAll();

	return (bOK);
}

	
BOOL CMediaView::OpenWordWheel (PCSZ pszWordWheel)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::OpenWordWheel(%s)\n", F(pszWordWheel)));
	ASSERT(m_hTitle);
	CloseWordWheel();

	// Lookup Wordwheel name ("|<name_char>")
	CKeyInfo key_info;
	BOOL bOK = m_mapKeywordGroups.Lookup(pszWordWheel, key_info);

	// Open the wordwheel (which has the keyword list)
	if (bOK) {
		CString& sWordWheel = key_info.m_sInternalName;
		m_hWordWheel = ::hMVWordWheelOpenTitle(m_hTitle, sWordWheel);
		DEBUG_OUT(TL_VERBOSE, ("%x <= hMVWordWheelOpenTitle(%x, %s): name=%s\n", 
				  m_hWordWheel, m_hTitle, F(key_info.m_sInternalName)));
		if (m_hWordWheel == NULL) {
			DEBUG_OUT(TL_DETAILED, ("WordWheel open failure for '%s' ('%s')\n", 
					  F1(pszWordWheel), F2(sWordWheel)));
			bOK = FALSE;
			}
		}
	else {
		DEBUG_OUT(TL_DETAILED, ("Unable to find wordwheel key info for '%s'\n", 
				  F1(pszWordWheel)));
		}

	return (bOK);
}


VOID CMediaView::CloseWordWheel (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CloseWordWheel()\n"));

	if (m_hWordWheel != NULL) {
		::MVWordWheelClose(m_hWordWheel);
		m_hWordWheel = NULL;
	}
}


BOOL CMediaView::OpenTopicList (PCSZ pszTopic)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::OpenTopicList(%s)\n", F(pszTopic)));
	ASSERT(m_hTitle);
	CloseTopicList();

	// Lookup internal name for the topic list
	CKeyInfo key_info;
	BOOL bOK = m_mapTopicGroups.Lookup(pszTopic, key_info);
	CString& sWordWheel = key_info.m_sInternalName;

	// Open the topic list
	if (bOK) {
		CString& sGroupName = key_info.m_sInternalName;
		m_hTopicList = ::hMVTopicListLoad(m_hTitle, sGroupName);
		DEBUG_OUT(TL_VERBOSE, ("%x <= hMVTopicListLoad(%x, %s)\n", 
				  m_hTopicList, m_hTitle, F(sGroupName)));
		if (m_hTopicList == NULL) {
			DEBUG_OUT(TL_DETAILED, ("TopicList open failure for '%s' ('%s')\n", 
					  F1(pszTopic), F2(sGroupName)));
			bOK = FALSE;
			}
		}
	else {
		DEBUG_OUT(TL_DETAILED, ("Unable to find topic list key info for '%s'\n", 
				  F1(pszTopic)));
		}

	return (bOK);
}


VOID CMediaView::CloseTopicList (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CloseTopicList()\n"));

	if (m_hTopicList != NULL) {
		::MVTopicListDestroy(m_hTopicList);
		m_hTopicList = NULL;
	}
}


// PlainTextToSgml: Filter the plain text, converting special characters.
//
// TODO: use SGML entities
//
CString PlainTextToSgml (PCSZ pszText)
{
	DEBUG_OUT(TL_GARRULOUS, ("PlainTextToSgml(%s)\n", F(pszText)));
	CString sSgmlText;

	while (*pszText) {
		CHAR cNext = *pszText;
		switch (cNext) {
			case '':
				sSgmlText += cDASH;
				sSgmlText += cDASH;
				break;
			case '':
			case '':
				sSgmlText += cDOUBLE_QUOTE;
				break;
			default:
				sSgmlText += cNext;
				break;
			}
		pszText++;
		}
	
	return (sSgmlText);
}


// TopicTextToLookupSgml: Convert the MediaView topic text into lookup SGML.
// This will consist of the topic embedded within <hp1>...</hp1> tags
// plus ad hoc indicators.
//
//
CString CMediaView::TopicTextToLookupSgml (CString& sTopicHeader, CString& sTopicBody)
{
	DEBUG_OUT(TL_GARRULOUS, ("CMediaView::TopicTextToLookupSgml(%s, %s)\n", 
			  F1(sTopicHeader), F2(sTopicBody)));
	CString sResult;
	INT nPos;

	// If there is no topic header, just return the body without tags
	sTopicHeader.TrimLeft();
	sTopicHeader.TrimRight();
	if (sTopicHeader.GetLength() == 0) {
		DEBUG_OUT(TL_BASIC, ("WARNING: [TopicTextToLookupSgml] empty topic line (nonscrolling region)\n"));
		return (sTopicBody);
		}

	// Add the topic text in bold italics
	sResult += szSGML_START_BOLD;
	sResult += szSGML_START_ITALICS;
	nPos = 0;
	while (nPos < sTopicHeader.GetLength()) {
		CStringx sLine = ExtractToken(sTopicHeader, nPos, szCRLF);
		sLine.TrimLeft();
		sLine.TrimRight();

		// Add the line text
		if (!sLine.IsEmpty()) {
			sResult += PlainTextToSgml(sTopicHeader);
			sResult += szSGML_START_PARA;
			}
		}
	sResult += szSGML_END_ITALICS;
	sResult += szSGML_END_BOLD;
	sResult += szSGML_START_PARA;

	// Format the body with ad hoc formatting
	// TODO: key this off of the wordwheel name (eg, dictionary vs thesaurus)
	CStringx sBody = sTopicBody;
	sBody.TrimLeft();

	nPos = 0;
	while (nPos < sBody.GetLength()) {
		CStringx sLine = ExtractToken(sBody, nPos, szCRLF);

		// Add the line text
		sResult += PlainTextToSgml(sLine);
		sResult += szCRLF;		// what's this for (readability?)
		}

	return (sResult);
}


// DefinitionTextToLookupSgml: Convert the MediaView definition text into lookup 
// SGML. This will consist of the topic embedded within <head>...</head> tags
// plus ad hoc indicators (eg, <hp1>Pathology</hp1>).
//
// Each SGML tag is placed on a separate line to facilate parsing, so the
// result will look something like the following(1):
//
//		<head>
//		monster
//		</head>
//		monster (mnster) noun
//		1.	a. An imaginary or legendary creature, such as a centaur or Harpy, 
//		that combines parts from various animal or human forms. b. A creature 
//		having a strange or frightening appearance.
//		2.	An animal, a plant, or other organism having structural defects 
//		or deformities. 3.	Pathology. A fetus or an infant that is grotesquely 
//		abnormal and usually not viable.4.	A very large animal, plant, or object.
//		5.	One who inspires horror or disgust: a monster of selfishness.
//		[Middle English monstre, from Old French, from Latin monstrum, portent, 
//		monster, from monre, to warn.]
//
// (1) The American Heritage Dictionary of the English Language
//
// NOTE: the parameters may be modified
//
// TODO: translate SGML special characters into character entities (eg, &lt.)
//		 tag the pronunciation, part of speech, and etymology
//		 don't modify the parameters
//
CString CMediaView::DefinitionTextToLookupSgml (CString& sTopicHeader, CString& sTopicBody)
{
	DEBUG_OUT(TL_GARRULOUS, ("CMediaView::DefinitionTextToLookupSgml(%s, %s)\n", 
			  F1(sTopicHeader), F2(sTopicBody)));
	CString sResult;

	// If there is no topic header, just return the body without tags
	sTopicHeader.TrimLeft();
	sTopicHeader.TrimRight();
	if (sTopicHeader.GetLength() == 0) {
		DEBUG_OUT(TL_BASIC, ("WARNING: [DefinitionTextToLookupSgml] empty topic line (nonscrolling region)\n"));
		return (sTopicBody);
		}

	// Since the nonscrollable region contains the topic name, format
	// this within SGML <head> tags:
	//		<head>monster</head>
	//

	// Add the starting tag
	sResult += szSGML_START_HEAD;

	// Isolate the name from the optional the sense indicator (eg, "scrap2")
	// Format the sense with braces instead of parens (eg, "{2}")
	// This makes sure the number isn't part of the entry (? "H2O")
	// TODO: determine if any entries really end in numbers
	CString sName;
	CString sSense;
	INT nLast = sTopicHeader.GetLength();
	if (IsDigit(sTopicHeader[nLast - 1])
			&& (sTopicHeader.GetLength() > 1)
			&& (!IsDigit(sTopicHeader[nLast - 2]))) {
		nLast--;
		INT nPos = nLast;
		INT nSense = StringToInt(sTopicHeader, nPos);
		ASSERT(nSense > 0);
		if (nSense > 0) {
			sSense.Format(szSENSE_FMT, nSense);
			}
		}
	sName = sTopicHeader.Left(nLast);

	// Add the name and optional sense indicator (no intervenig space)
	sResult += sName;
	sResult += sSense;

	// Add the ending tag
	sResult += szSGML_END_HEAD;

	// Format the rest with ad hoc formatting
	// TODO: key this off of the wordwheel name (eg, dictionary vs thesaurus)

	// Ignore the repeated head word
	CStringx sBody = sTopicBody;
	sBody.TrimLeft();
	INT nPos = 0;
	if (sBody.Find(sName) == 0) {
		nPos += sName.GetLength();
		}

	// Copy over the body, tagging part of speech indicators
	//
	// Here's an example for the dictionary entry of "scrap".
	//
	//		scrap (skrp) noun
	//		1.	A small piece or bit; a fragment.
	//		2.	scraps. Leftover bits of food.
	//		3.	Discarded waste material, especially metal suitable for reprocessing.
	//		4.	scraps. Crisp pieces of rendered animal fat; cracklings.
	//	
	//		verb, transitive
	//		scrapped, scrapping, scraps
	//		1.	To break down into parts for disposal or salvage.
	//		2.	To discard as worthless; junk.
	//		[Middle English, from Old Norse skrap, trifles, pieces.]
	//
	// Note that there are CR/NL characters after each line shown above.
	// Also, the parts of speech indicators appear only to occur at the end of
	// lines, so elsewhere they won't be tagged (in case definition text uses
	// partof speech word).
	//
	// TODO: tag the sense numbers
	//
	while (nPos < sBody.GetLength()) {
		CStringx sLine = ExtractToken(sBody, nPos, szCRLF);
		CString sPartOfSpeech;

		// Check for the most specific part of speech label that applies
		// The array is ordered with most-specific first (eg, "transitive verb"
		// before "verb")
		for (INT i = 0; i < m_asPartOfSpeech.GetSize(); i++) {
			INT nLinePos = sLine.Find(m_asPartOfSpeech[i]);
			if ((nLinePos != nINVALID_INDEX)
					&& ((nLinePos + m_asPartOfSpeech[i].GetLength())
						== sLine.GetLength())) {
				sLine = sLine.Left(nLinePos);
				sPartOfSpeech += szSGML_START_PART_OF_SPEECH;
				sPartOfSpeech += m_asPartOfSpeech[i];
				sPartOfSpeech += szSGML_END_PART_OF_SPEECH;
				break;
				}
			}

		// Add the line text plus optional part of speech label
		sResult += PlainTextToSgml(sLine);
		sResult += sPartOfSpeech;
		sResult += szCRLF;
		}

	return (sResult);
}


// ReferenceToLookupSgml: Format the topic name with lookup SGML.
// This will consist of the topic name within <head> ... </head> tags.
//
// Each SGML tag is placed on a separate line to facilate parsing:
//		<head>
//		teratoid
//		</head>
//
// NOTE: The parameter may be modified
//
// TODO: don't modify the parameters
//
CString CMediaView::ReferenceToLookupSgml (CString& sTopicName)
{
	DEBUG_OUT(TL_GARRULOUS, ("CMediaView::ReferenceToLookupSgml(%s)\n",
			  F1(sTopicName)));
	CString sResult;

	// Make sure the topic name is valid
	// Skip over the section indicator (eg, "4monster")
	sTopicName.TrimLeft();
	sTopicName.TrimRight();
	INT nNameStart = 0;
	if ((sTopicName.GetLength() > 0) && IsDigit(sTopicName[0])) {
		nNameStart += 1;
		}
	if (nNameStart == sTopicName.GetLength()) {
		DEBUG_OUT(TL_BASIC, ("ERROR: ReferenceToLookupSgml called with bad topic name ('%s')",
				  F1(sTopicName)));
		}

	// Format the reference topic name within SGML <head> tags:
	//		<head>teratoid</head>
	//

	// Add the starting tag
	sResult += szSGML_START_HEAD;

	// Isolate the name from the optional the sense indicator (eg, "scrap (2)")
	// Format the sense with braces instead of parens (eg, "{1}")
	CString sName;
	CString sSense;
	INT nSenseStart = Pos(sTopicName, cLEFT_PAREN);
	if (nSenseStart == nINVALID_INDEX) {
		nSenseStart = sTopicName.GetLength();
		}
	else {
		INT nPos = nSenseStart + 1;
		INT nSense = StringToInt(sTopicName, nPos);
		ASSERT(nSense > 0);
		if (nSense > 0) {
			sSense.Format(szSENSE_FMT, nSense);
			}
		}
	INT nNameLen = nSenseStart - nNameStart;
	sName = sTopicName.Mid(nNameStart, nNameLen);
	sName.TrimRight();

	// Add the name and optional sense indicator (no intervening space)
	sResult += sName;
	sResult += sSense;

	// Add the ending tag
	sResult += szSGML_END_HEAD;

	return (sResult);
}


// LookupPhrase: Lookup the topic text given a keyword phrase
// The results will optionally be formatted with pseudo-SGML
//
CString CMediaView::LookupPhrase (PCSZ pszLookupPhrase, 
								  BOOL bDefinition /* = FALSE */, 
								  BOOL bAddSgml /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::LookupPhrase(%s, %d)\n",
			  F(pszLookupPhrase), bAddSgml));
	CString sResult;
	CString sPhrase = pszLookupPhrase;
	ASSERT(m_hTitle != NULL);

	CMap<INT, INT, INT, INT> mapProcessed;
	mapProcessed.RemoveAll();

#if FALSE
	BOOL bAssocOK;
	HWND hWndNonScrollable = NULL;
	HWND hWndScrollable = NULL;
	if (GetDebugOption("UseTemporaryWindows", FALSE)) {
		hWndNonScrollable = CreateDummyWindow();
		bAssocOK = fMVSetWindow(m_pMvNonScrollable, hWndNonScrollable, &m_err);
		ASSERT(bAssocOK);
		hWndScrollable = CreateDummyWindow();
		bAssocOK = fMVSetWindow(m_pMvScrollable, hWndScrollable, &m_err);
		ASSERT(bAssocOK);
		}
#endif

	// Get the number of topics from the word wheel
	// TODO: figure out why extraneous topics are being returned
	PSZ pszPhrase = (PSZ)pszLookupPhrase;
	INT nNumKeyTopics = ::nMVKeyIndexGetCount(m_hWordWheel, (LPBYTE)pszPhrase);
	if (nNumKeyTopics == ((WORD)-1)) {
		DEBUG_OUT(TL_BASIC, ("error in nMVKeyIndexGetCount(%x, %s)\n",
				 m_hWordWheel, pszPhrase));
		return (szNULL);
		}	
	if (nNumKeyTopics > 100) {
		DEBUG_OUT(TL_DETAILED, ("NumKeyTopics reduced to 100 from %d\n", nNumKeyTopics));
		nNumKeyTopics = 100;
	}

	// Get the ADDR ptrs for the lookup
	CDWordArray adwKeyAddrs;
	adwKeyAddrs.SetAtGrow(nNumKeyTopics, 0);
	PDWORD pdwKeyAddrs = adwKeyAddrs.GetData();
	LRESULT lRet;
	lRet = ::lMVKeyIndexGetAddrs(m_hWordWheel,		// word wheel with topic keywords to check
								(LPBYTE)pszPhrase,	// keyword
								0,					// index from which to start copying
								nNumKeyTopics,		// number of addressed to copy
								(LPBYTE)pdwKeyAddrs);// array of key addresses
	if (lRet <= 0) {
		DEBUG_OUT(TL_BASIC, ("Error %d from lMVKeyIndexGetAddrs(%x, %s, 0, %d, %x)\n",
				  lRet, m_hWordWheel, pszPhrase, nNumKeyTopics, pdwKeyAddrs));
		ASSERT_ERROR();
		}

	// Add the text from each unique topic to the result buffer
	DEBUG_OUT(TL_VERBOSE, ("Index hits:\n"));
	for (INT i = 0; i < nNumKeyTopics; i++) {
		BOOL bProcess = FALSE;

		// Change the MediaView address in order to get at the name
		VA tmpVa = ::vaMVConvertAddr(m_hTitle, adwKeyAddrs[i]);
		::fMVSetAddress(m_pMvNonScrollable, tmpVa, nMV_NSR, 0, &m_err);
		::fMVSetAddress(m_pMvScrollable, tmpVa, nMV_SR, 0, &m_err);
		LPMV lpMV = GetMediaView();

		// Access topic title string
		HANDLE hName = hMVGetName(lpMV);
		PSZ pszName = (PSZ)GlobalLock(hName);

		// See if this topic was already processed
		INT nTopic = lMVTopicNumber(lpMV);
		DEBUG_OUT(TL_VERBOSE, ("\t%s\t%x\t[%d]\n", pszName, tmpVa, nTopic));
		INT nValue;
		bProcess = (mapProcessed.Lookup(nTopic, nValue) == FALSE);
		if (bDefinition && bProcess) {
			CONST CHAR szVALID_SUFFIXES[] = "es ed ing s ";
			bProcess = FALSE;

			// See if the entry should be included
			// For now, this just checks for entries with the phrase and common suffixes
			// or just the phrase itself.
			// NOTE: Some topic names start with a group number indicator, so this is ignored
			//		 Also, some have a sense indicator, which also must be ignored
			// TODO: find a better way of finding the right entries to include
			INT nPos = FindStringIgnoreCase(pszName, sPhrase);
			if (nPos != nINVALID_INDEX) {
				if ((nPos == 0)
					|| ((nPos == 1) && (IsDigit(pszName[0])))) {
					nPos += sPhrase.GetLength();
					CStringx sRem = ExtractToken(pszName, nPos, szLEFT_PAREN);
					sRem.TrimRight();
					sRem += cSPACE;
					if (sRem.IsEmpty()
						|| (FindStringIgnoreCase(szVALID_SUFFIXES, sRem) != nINVALID_INDEX)) {
						mapProcessed.SetAt(nTopic, TRUE);
						bProcess = TRUE;
						}
					}
				}
			}
		else {
			bProcess = (nNumKeyTopics != 100);
			}		

		//	Get the text from the topic if not already processed
		if (bProcess) {
			VA va = vaMVConvertAddr(m_hTitle, adwKeyAddrs[i]);
			::fMVSetAddress(m_pMvNonScrollable, va, nMV_NSR, 0, &m_err);
			::fMVSetAddress(m_pMvScrollable, va, nMV_SR, 0, &m_err);
			if (bAddSgml) {
				CString sNonScrollable;
				CString sScrollable;
				CopyTopicText(sNonScrollable, sScrollable);
				if (bDefinition) {
					sResult += DefinitionTextToLookupSgml(sNonScrollable, sScrollable);
					}
				else {
					sResult += TopicTextToLookupSgml(sNonScrollable, sScrollable);
					}
				}
			else {
				sResult += CopyTopicText();
				}
		}

		//	Free topic title string
		GlobalUnlock(hName);
		GlobalFree(hName);
	}

#if FALSE
	if (hWndNonScrollable != NULL) {
		bAssocOK = fMVSetWindow(m_pMvNonScrollable, m_hWndNonScrollable, &m_err);
		ASSERT(bAssocOK);
		m_pMvScrollable = lpMVNew(NULL, &m_err);
		bAssocOK = fMVSetWindow(m_pMvScrollable, m_hWndScrollable, &m_err);
		ASSERT(bAssocOK);
		DestroyWindow(hWndNonScrollable);
		DestroyWindow(hWndScrollable);
		}
#endif

	return (sResult);
}


// LookupCrossReferences: Return a list of topics that reference the current
// phrase. This is done by performing a full-text search (indexed of course)
//
CString CMediaView::LookupCrossReferences (PCSZ pszPhrase, BOOL bAddSgml /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::LookupCrossReferences(%s, %d):\n",
			  F(pszPhrase), bAddSgml));
	CString sResult;
	ASSERT(m_hTitle != NULL);
	HANDLE hResultsTopicList;
	CString sPhrase = pszPhrase;
	CString sSearch = sPhrase;

	// Put quotes around the search string if it contains embedded spaces
	// (without quotes) to ensure the words are not treated separately
	if ((sPhrase.Find(cSPACE) != nINVALID_INDEX)
		&& (sPhrase.Find(cDOUBLE_QUOTE) == nINVALID_INDEX)) {
		sSearch.Format("\"%s\"", (PCSZ)sPhrase);
	}

	// Do a full-text search on the topics for the phrase
	QUERYERR query_error;
	hResultsTopicList = hMVTopicListFromQuery(m_hTitle, // database handle
									   IMPLICIT_AND,	// search flags
									   sSearch,			// query text
									   m_hTopicList,	// topics to search
									   0,				// use default for nearness
									   NULL,			// ignore higlight information
									   &query_error);	// error information

	// Add the name of each topic to the results buffer
	if (hMVTopicListFromQuery) {
		INT nNumTopics = ::lMVTopicListLength(hResultsTopicList);
		DEBUG_OUT(TL_VERBOSE, ("Search results:\n"));
		CString sName;
		for (INT i = 0; i < nNumTopics; ++i) {
			// Get the name from the topic file
			PSZ pszName = sName.GetBuffer(nMAX_NAME);
			INT nTopic = ::lMVTopicListLookup(hResultsTopicList, i);
			::lMVTitleGetInfo(m_hTitle, TTLINF_TOPICTITLE, nTopic, (LONG)pszName);
			sName.ReleaseBuffer();

			// Add it to the results buffer
			DEBUG_OUT(TL_VERBOSE, ("\t%s\n", (PCSZ)sName));
			if (!bAddSgml) {
				sResult += sName;
				sResult += cMV_ENTRY_SEP;
				sResult += cSPACE;
				}
			else {
				sResult += ReferenceToLookupSgml(sName);
				}
			}

		// Clean up the results list
		::MVTopicListDestroy(hResultsTopicList);
		}
	else {
		// TODO: put this is a simple wrapper around hMVTopicListFromQuery
		DEBUG_OUT(TL_BASIC, ("Error %d from hMVTopicListFromQuery(%x, %d, %s, %x, 0, NULL, %x)\n",
				  m_hTitle, IMPLICIT_AND, (PCSZ)sSearch, m_hTopicList, &query_error));
		}

	return (sResult);
}


// CopyMediaText: Copy the text from the given MediaView object.
// This doesn't include any formatting indicators.
//
CString CMediaView::CopyMediaText (LPMV lpMV)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CopyMediaText(%x)\n", lpMV));
	CString sMediaText;
	HANDLE hMem = NULL;

	// Get a handle to a text block for the MediaView object
	m_err = wERRS_NONE;
	for (INT i = 0; i < 5; i++) {
		hMem = ::hMVCopyMedia(lpMV, &m_err);
		if (m_err != wERRS_PARTIAL) {
			break;
			}
		DEBUG_OUT(TL_DETAILED, ("Partial media copy; trying again after a nap\n"));
		Sleep(200);
		ASSERT(hMem);
		GlobalFree(hMem);
		}

	// Copy the text if accessible
	if (hMem) {
		PSZ pszMediaText = (PSZ)GlobalLock(hMem);
		ASSERT(pszMediaText);
		sMediaText = pszMediaText;
		GlobalUnlock(hMem);
		GlobalFree(hMem);
	}

	return (sMediaText);
}



// CopyTopicText: Copy the plain text from the current MediaView topic.
// This includes text from both the scrolling and nonscrolling regions.
//
VOID CMediaView::CopyTopicText (CString& sNonScrollable, CString& sScrollable)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CopyTopicText(%x, %x)\n",
			  &sNonScrollable, &sScrollable));

	// Copy both non-scrolling & scrolling regions
	sNonScrollable = CopyMediaText(m_pMvNonScrollable);
	sScrollable = CopyMediaText(m_pMvScrollable);
}


// CopyTopicText: Copy the plain text from the current MediaView topic.
// This version formats the nonscrollable region within angle brackets.
//
CString CMediaView::CopyTopicText (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMediaView::CopyTopicText()\n"));
	CString sTopicText;

	// Copy both non-scrolling & scrolling regions
	CString sNonScrollable;
	CString sScrollable;
	CopyTopicText(sNonScrollable, sScrollable);
	sTopicText.Format("<%s>\r\n%s", (PCSZ)sNonScrollable, (PCSZ)sScrollable);

	return (sTopicText);
}

