// MvLookupDlg.h : header file
//

CONST INT nMV_NSR = 0;
CONST INT nMV_SR = 1;


/////////////////////////////////////////////////////////////////////////////
// CKeyInfo
//
// Class for maintaining information about a MediaView keyword or topic group.
// Used in the collections for the group names and keyword-search sections
//

class CKeyInfo : public CObject
{
	DECLARE_DYNAMIC(CKeyInfo);
public:
	CKeyInfo (CONST CString& sExternalName, CONST CString& sInternalName, CONST CString& sInfo);
	CKeyInfo (VOID);
	~CKeyInfo ();
	CONST CKeyInfo& CKeyInfo::operator= (CONST CKeyInfo& other);
	CString m_sExternalName;
	CString m_sInternalName;
	CString m_sInfo;
};

MAKE_CLASS_CAST(CKeyInfo);

typedef	CMap<CString, PCSZ, CKeyInfo, CKeyInfo&> CMapKeyInfo;


/////////////////////////////////////////////////////////////////////////////
// CMediaView dialog

class CMediaView : public CObject
{
	DECLARE_DYNAMIC(CMediaView);

// Construction
public:
	CMediaView(VOID);
	~CMediaView(VOID);

public:
	CString m_sFile;					// filename for the MediaView title
	CString m_sTitleName;				// name of the title
	HANDLE m_hTopicList;				// object for current list of topics
	HANDLE m_hWordWheel;				// object for word wheel object
	HTITLE m_hTitle;					// object for title
	LPMV m_pMvNonScrollable;			// MediaView object for non-scrolling region
	LPMV m_pMvScrollable;				// MediaView object for scrolling region
	ERR m_err;							// error code for last operation
	CStringArray m_asPartOfSpeech;		// parts of speech labels used in Bookshelf

private:
	HWND m_hWndScrollable;				// window for rendering scrollable region
	HWND m_hWndNonScrollable;			// window for rendering non-scrollable region
	CMapKeyInfo	m_mapTopicGroups;		// association list for topic groups
	CMapKeyInfo m_mapKeywordGroups;		// association list for keyword groups

// Implementation
protected:

public:
	VOID InitMediaView (VOID);
	VOID CleanupMediaView (VOID);
	VOID OpenTitle (VOID);
	BOOL OpenTitle (CONST CString& sTitle);
	BOOL CloseTitle (VOID);
	BOOL OpenWordWheel (PCSZ pszWordWheel);
	VOID OpenWordWheel (VOID);
	VOID CloseWordWheel (VOID);
	BOOL OpenTopicList (PCSZ pszTopic);
	VOID OpenTopicList (VOID);
	VOID CloseTopicList (VOID);
	CString TopicTextToLookupSgml (CString& sNonScrollable, CString& sScrollable);
	CString DefinitionTextToLookupSgml (CString& sTopicHeader, CString& sTopicBody);
	CString ReferenceToLookupSgml (CString& sTopicName);
	CString LookupPhrase (PCSZ pszPhrase, BOOL bDefinition = FALSE, BOOL bAddSgml = TRUE);
	CString LookupDefinitionPhrase (PCSZ pszPhrase, BOOL bAddSgml = TRUE)
				{ return (LookupPhrase(pszPhrase, TRUE, bAddSgml)); }
	CString LookupCrossReferences (PCSZ pszPhrase, BOOL bAddSgml = TRUE);
	CString CopyMediaText (LPMV lpMV);
	VOID CopyTopicText (CString& sNonScrollable, CString& sScrollable);
	CString CopyTopicText (VOID);
	VOID EnableControls (VOID);

	LPMV GetMediaView (VOID)	
		{ return (fMVHasSR(m_pMvScrollable) ? m_pMvScrollable : m_pMvNonScrollable); }
};

MAKE_CLASS_CAST(CMediaView);
