// MvLookup.cpp : Defines the initialization routines for the DLL.
//

#include "stdafx.h"
#include "MvLookup.h"
#include "MediaView.h"

IMPLEMENT_DYNAMIC(CMvInfo, CObject);
IMPLEMENT_DYNAMIC(CMvLookup, CObject);

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

STATIC UINT DoMvInitProc (CMvInfo* pMvInfo);


CONST PCSZ apszLookupActions[] = { 
				"e_None",
				"e_LookupPhrase", "e_LookupReferences", "e_OpenTitle", 
				"e_CloseTitle", "e_OpenWordWheel", "e_OpenTopicList", "e_Exit",
				"e_Unknown" };

/////////////////////////////////////////////////////////////////////////////
// CMvLookupDll

BEGIN_MESSAGE_MAP(CMvLookupDll, CWinApp)
	//{{AFX_MSG_MAP(CMvLookupDll)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// The one and only CMvLookupDll object

CMvLookupDll theApp;

/////////////////////////////////////////////////////////////////////////////
// CMvLookupDll construction

CMvLookupDll::CMvLookupDll(VOID)
{
	INIT_DEBUGGING_("MvLookup.ini");
	DEBUG_OUT(TL_VERBOSE, ("CMvLookupDll::CMvLookupDll()\n"));
}


BOOL CMvLookupDll::InitInstance (VOID) 
{
	INIT_COMMON(AfxGetInstanceHandle(), NULL, "MvLookup");
	DEBUG_OUT(TL_VERBOSE, ("CMvLookupDll::InitInstance()\n"));

	return CWinApp::InitInstance();
}


//-----------------------------------------------------------------------------
//

STATIC UINT DoMvInitProc (CMvInfo* pMvInfo)
{
	DEBUG_OUT(TL_VERBOSE, ("DoMvInitProc(%x)\n", pMvInfo));
	BOOL bOK = FALSE;

	ASSERT(pMvInfo && pMvInfo->IsKindOf(RUNTIME_CLASS(CMvInfo)));
	if (pMvInfo && pMvInfo->IsKindOf(RUNTIME_CLASS(CMvInfo))) {
		pMvInfo->Init();
		pMvInfo->LookupWorker();
		}

	return (bOK);
}


BOOL WaitForEvent (CEvent& event, PCSZ pszName)
{
	DEBUG_OUT(TL_VERBOSE, ("WaitForEvent(%x, %s)\n", &event, F(pszName)));

	if (GetDebugOption("IgnoreEvents", FALSE)) {
		Sleep(100);	// sleep 1/10th of a second so the other guy becomes active
		return (TRUE);
		}

	// Wait to access the event/lock
	CSingleLock lock(&event);
	DWORD dwResult = lock.Lock(INFINITE);
	if (dwResult == 0) {
		DEBUG_OUT(TL_BASIC, ("Problem waiting for event %s\n", pszName));
		return (FALSE);
		}

	return (TRUE);
}


BOOL SignalEvent (CEvent& event, PCSZ pszName)
{
	DEBUG_OUT(TL_VERBOSE, ("SignalEvent(%x, %s)\n", &event, F(pszName)));
	BOOL bOK = event.SetEvent();
	if (!bOK) {
		DEBUG_OUT(TL_BASIC, ("Problem signaling event %s\n", pszName));
		}

	return (bOK);
}

#define WAIT_FOR_EVENT(event)	WaitForEvent(event, #event)
#define SIGNAL_EVENT(event)		SignalEvent(event, #event)


//-----------------------------------------------------------------------------
// CMvInfo
// 
// Interface class for accessing MediaView titles


CMvInfo::CMvInfo (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::CMvInfo()\n"));
	m_pMediaView = NULL;
	m_bAddSgml = TRUE;
	m_bDefinition = FALSE;
	m_pLookupThread = NULL;
	m_eLookupAction = e_None;
	if (GetDebugOption("UseLookupThread", TRUE)) {
		m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::DoMvInitProc, this);
		}
	else {
		Init();
		}
}


CMvInfo::~CMvInfo (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::~CMvInfo()\n"));

	if (m_pMediaView) {
		delete m_pMediaView;
		}
	if (m_pLookupThread) {
		delete m_pLookupThread;
		}
}


BOOL CMvInfo::Init (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::Init()\n"));

	m_pMediaView = new CMediaView();

#if TRUE
	if (!m_sTitle.IsEmpty()) {
		m_pMediaView->OpenTitle(m_sTitle);
		}
#endif

	return (TRUE);
}


BOOL CMvInfo::AcquireLock (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::AcquireLock()\n"));

	if (GetDebugOption("SkipLocks", FALSE)) {
		Sleep(100);	// sleep 1/10th of a second so the other guy becomes active
		return (TRUE);
		}

	// Wait to access the event/lock
	BOOL bOK = m_csMediaView.Lock(INFINITE);
	if (!bOK) {
		DEBUG_OUT(TL_BASIC, ("Problem waiting for critical section\n"));
		}

	return (bOK);
}


BOOL CMvInfo::ReleaseLock (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::ReleaseLock()\n"));

	BOOL bOK = m_csMediaView.Unlock();
	if (!bOK) {
		DEBUG_OUT(TL_BASIC, ("Problem unlocking critical section\n"));
		}

	return (TRUE);
}


// OpenTitle: Open a MediaView title
//
BOOL CMvInfo::OpenTitle (PCSZ pszTitle)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::OpenTitle('%s')\n", F(pszTitle)));

	//AcquireLock();
	BOOL bOK = m_pMediaView ? m_pMediaView->OpenTitle(pszTitle) : FALSE;
	//ReleaseLock();

	return (bOK);
}


// OpenTitle: Close the MediaView title
//
BOOL CMvInfo::CloseTitle (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::CloseTitle()\n"));

	//AcquireLock();
	BOOL bOK = m_pMediaView ? m_pMediaView->CloseTitle() : FALSE;
	//ReleaseLock();

	return (bOK);
}


// OpenWordWheel: Open the MediaView wordwheel with the given name
// Wordwheels are used for doing keyword lookup
//
BOOL CMvInfo::OpenWordWheel (PCSZ pszName)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::OpenWordWheel(%s)\n", F(pszName)));

	//AcquireLock();
	BOOL bOK = m_pMediaView ? m_pMediaView->OpenWordWheel(pszName) : FALSE;
	//ReleaseLock();

	return (bOK);
}


// OpenTopicList: Open the MediaView topic list with the given name
// Topic lists are used for doing full-text search (for cross-references)
//
BOOL CMvInfo::OpenTopicList (PCSZ pszName)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::OpenTopicList(%s)\n", F(pszName)));

	//AcquireLock();
	BOOL bOK = m_pMediaView ? m_pMediaView->OpenTopicList(pszName) : FALSE;
	//ReleaseLock();

	return (bOK);
}


// LookupPhrase: Lookup the topics in the database given a phrase keyword
//
CString CMvInfo::LookupPhrase (PCSZ pszPhrase, BOOL bDefinition /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::LookupPhrase(%s, %d)\n", F(pszPhrase), bDefinition));
	CString sResults;
	ASSERT_VALID(m_pMediaView);

	//AcquireLock();
	if (m_pMediaView) {
		sResults = m_pMediaView->LookupPhrase(pszPhrase, bDefinition, m_bAddSgml);
		}
	//ReleaseLock();

	return (sResults);
}


// LookupCrossReferences: Lookup the topics that contain a given phrase
//
CString CMvInfo::LookupCrossReferences (PCSZ pszPhrase)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::LookupCrossReferences(%s)\n", F(pszPhrase)));
	CString sResult;
	ASSERT_VALID(m_pMediaView);

	//AcquireLock();
	if (m_pMediaView) {
		sResult = m_pMediaView->LookupCrossReferences(pszPhrase, m_bAddSgml);
		}
	//ReleaseLock();

	return (sResult);
}


BOOL CMvInfo::MtOpenTitle (PCSZ pszTitle)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::MtOpenTitle('%s')\n", F(pszTitle)));

	// Wait for the event coordination lock
	AcquireLock();

	// Set the name and signal that an open title action is desired
	m_sTitle = pszTitle;
	m_eLookupAction = e_OpenTitle;
	SIGNAL_EVENT(m_eventLookup);

	// Wait for the results coordination lock
	WAIT_FOR_EVENT(m_eventResults);

	// Access the results and release the lock
	BOOL bOK = m_bOK;
	ReleaseLock();

	return (bOK);
}


BOOL CMvInfo::MtCloseTitle (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::MtCloseTitle()\n"));

	// Wait for the event coordination lock
	AcquireLock();

	// Signal that an close title event is desired
	m_eLookupAction = e_CloseTitle;
	SIGNAL_EVENT(m_eventLookup);

	// Wait for the results coordination lock
	WAIT_FOR_EVENT(m_eventResults);

	// Access the results and signal that it is available
	BOOL bOK = m_bOK;
	ReleaseLock();

	return (bOK);
}


BOOL CMvInfo::MtOpenWordWheel (PCSZ pszName)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::MtOpenWordWheel(%s)\n", F(pszName)));

	// Wait for the event coordination lock
	AcquireLock();

	// Set the name and signal that an open word wheel event is desired
	m_sWordWheel = pszName;
	m_eLookupAction = e_OpenWordWheel;
	SIGNAL_EVENT(m_eventLookup);

	// Wait for the results coordination lock
	WAIT_FOR_EVENT(m_eventResults);

	// Access the results and signal that it is available
	BOOL bOK = m_bOK;
	ReleaseLock();

	return (bOK);
}


BOOL CMvInfo::MtOpenTopicList (PCSZ pszName)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::MtOpenTopicList(%s)\n", F(pszName)));

	// Wait for the event coordination lock
	AcquireLock();

	// Set the name and signal that an open topic list event is desired
	m_sTopicList = pszName;
	m_eLookupAction = e_OpenTopicList;
	SIGNAL_EVENT(m_eventLookup);

	// Wait for the results coordination lock
	WAIT_FOR_EVENT(m_eventResults);

	// Access the results and signal that it is available
	BOOL bOK = m_bOK;
	ReleaseLock();

	return (bOK);
}


// MtLookupPhrase: Lookup the topics in the database given a phrase keyword
//
CString CMvInfo::MtLookupPhrase (PCSZ pszPhrase, BOOL bDefinition /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::MtLookupPhrase(%s, %d)\n", F(pszPhrase), bDefinition));
	CString sResult;

	// Wait for the event coordination lock
	AcquireLock();

	// Set the phrase and signal that a definition lookup is desired
	m_sPhrase = pszPhrase;
	m_bDefinition = bDefinition;
	m_eLookupAction = e_LookupPhrase;
	SIGNAL_EVENT(m_eventLookup);

	// Wait for the results coordination lock
	WAIT_FOR_EVENT(m_eventResults);

	// Access the results and signal that it is available
	sResult = m_sResults;
	ReleaseLock();

	return (sResult);
}


// MtLookupReferences: Lookup the topics in the database given a phrase keyword
//
CString CMvInfo::MtLookupCrossReferences (PCSZ pszReferences)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::MtLookupCrossReferences (%s)\n", F(pszReferences)));
	CString sResult;

	// Wait for the event coordination lock
	AcquireLock();

	// Set the phrase and signal that a cross reference lookup is desired
	m_sPhrase = pszReferences;
	m_eLookupAction = e_LookupReferences;
	SIGNAL_EVENT(m_eventLookup);

	// Wait for the results coordination lock
	WAIT_FOR_EVENT(m_eventResults);

	// Access the results and signal that it is available
	sResult = m_sResults;
	ReleaseLock();

	return (sResult);
}


BOOL CMvInfo::IsValid (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::IsValid()\n"));

	return (IsKindOf(RUNTIME_CLASS(CMvInfo)));
}


// LookupWorker: Workhorse routine for waiting upon lookup requests
//
VOID CMvInfo::LookupWorker (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvInfo::LookupWorker()\n"));

	// Keep processing lookup requests until the exit command is received
	CSingleLock locks(&m_eventLookup);
	for (;;) {
		DEBUG_OUT(TL_VERBOSE, ("LookupWorker waiting for work\n"));

		// Wait for the next request
		BOOL bOK = locks.Lock(INFINITE);
		if (!bOK) {
			DEBUG_OUT(TL_BASIC, ("Problem waiting for MvLookup events (last error=%lu)\n", GetLastError()));
			continue;
			}

		// Service the request
		DEBUG_OUT(TL_VERBOSE, ("LookupWorker servicing event %s\n",
				  apszLookupActions[max(0, min(m_eUnknown, m_eLookupAction - e_None))]));
		switch (m_eLookupAction) {
			case e_LookupPhrase:
				m_sResults = LookupPhrase(m_sPhrase, m_bDefinition);
				break;
			case e_LookupReferences:
				m_sResults = LookupCrossReferences(m_sPhrase);
				break;
			case e_OpenTitle:
				m_bOK = OpenTitle(m_sTitle);
				break;
			case e_CloseTitle:
				m_bOK = CloseTitle();
				break;
			case e_OpenWordWheel:
				m_bOK = OpenWordWheel(m_sWordWheel);
				break;
			case e_OpenTopicList:
				m_bOK = OpenTopicList(m_sTopicList);
				break;
			case e_Exit:
				AfxEndThread(0);
				break;
			default:
				DEBUG_OUT(TL_VERBOSE, ("Unknown lookup action: %d\n", m_eLookupAction));
				break;
			}

		// Signal that the results are ready
		m_eventResults.SetEvent();
		}
}


//-----------------------------------------------------------------------------
// CMvLookup
// 
// Interface class for actual lookup's


CMvLookup::CMvLookup (CMvInfo* pMvInfo)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::CMvLookup(%x)\n", pMvInfo));
	ASSERT_VALID(pMvInfo);

	m_pMvInfo = pMvInfo;
}


CMvLookup::~CMvLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::~CMvLookup()\n"));
}


// LookupPhrase: Lookup the topics in the database given a phrase keyword
//
CString CMvLookup::LookupPhrase (PCSZ pszPhrase, BOOL bDefinition /* = FALSE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::LookupPhrase(%s, %d)\n", F(pszPhrase), bDefinition));
	CString sResult;
	ASSERT_VALID(m_pMvInfo);

	if (m_pMvInfo) {
		sResult = m_pMvInfo->MtLookupPhrase(pszPhrase, bDefinition);
		}

	return (sResult);
}


CString CMvLookup::LookupCrossReferences (PCSZ pszPhrase)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::LookupCrossReferences(%s)\n", F(pszPhrase)));
	CString sResult;
	ASSERT_VALID(m_pMvInfo);

	if (m_pMvInfo) {
		sResult = m_pMvInfo->MtLookupCrossReferences(pszPhrase);
		}

	return (sResult);
}


// UseSgml: Indicate whether the results should be formatted with (pseudo) SGML
//
VOID CMvLookup::UseSgml (BOOL bUseSgml /* = TRUE */)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::UseSgml(%d)\n", bUseSgml));
	ASSERT_VALID(m_pMvInfo);

	if (m_pMvInfo) {
		m_pMvInfo->m_bAddSgml = bUseSgml;
		}
}


BOOL CMvLookup::IsValid (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMvLookup::IsValid()\n"));

	return (IsKindOf(RUNTIME_CLASS(CMvLookup)));
}



//-----------------------------------------------------------------------------
// Implementation of C-interface wrappers
//-----------------------------------------------------------------------------


// C-wrappers for CMvInfo
//

EXTERN_C PMVINFO DLL_EXPORT MviCreate (VOID)
{
	PMVINFO pMVI = new MVINFO;
	if (pMVI) {
		pMVI->pMvInfo = new CMvInfo();
		}

	return (pMVI);
}


EXTERN_C PMVINFO DLL_EXPORT MviDelete (PMVINFO pMVI)
{
	if (pMVI) {
		delete pMVI->pMvInfo;
		delete pMVI;
		}

	return (NULL);
}


EXTERN_C BOOL DLL_EXPORT MviOpenTitle (PMVINFO pMVI, PCSZ pszTitle)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->OpenTitle(pszTitle);
		}

	return (bOK);
}


EXTERN_C BOOL DLL_EXPORT MviCloseTitle (PMVINFO pMVI)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->CloseTitle();
		}

	return (bOK);
}


EXTERN_C BOOL DLL_EXPORT MviOpenWordWheel (PMVINFO pMVI, PCSZ pszName)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->OpenWordWheel(pszName);
		}

	return (bOK);
}


EXTERN_C BOOL DLL_EXPORT MviOpenTopicList (PMVINFO pMVI, PCSZ pszName)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->OpenTopicList(pszName);
		}

	return (bOK);
}


EXTERN_C PCSZ DLL_EXPORT MviLookupPhrase (PMVINFO pMVI, PCSZ pszPhrase, BOOL bDefinition)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		pMVI->sResult = pMVI->pMvInfo->LookupPhrase(pszPhrase, bDefinition);
		pszResult = pMVI->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MviLookupDefinitionPhrase (PMVINFO pMVI, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		pMVI->sResult = pMVI->pMvInfo->LookupDefinitionPhrase(pszPhrase);
		pszResult = pMVI->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MviLookupCrossReferences (PMVINFO pMVI, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		pMVI->sResult = pMVI->pMvInfo->LookupCrossReferences(pszPhrase);
		pszResult = pMVI->sResult;
		}

	return (pszResult);
}


// Multithreaded access functions
//

EXTERN_C BOOL DLL_EXPORT MviMtOpenTitle (PMVINFO pMVI, PCSZ pszTitle)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->MtOpenTitle(pszTitle);
		}

	return (bOK);
}


EXTERN_C BOOL DLL_EXPORT MviMtCloseTitle (PMVINFO pMVI)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->MtCloseTitle();
		}

	return (bOK);
}


EXTERN_C BOOL DLL_EXPORT MviMtOpenWordWheel (PMVINFO pMVI, PCSZ pszName)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->MtOpenWordWheel(pszName);
		}

	return (bOK);
}


EXTERN_C BOOL DLL_EXPORT MviMtOpenTopicList (PMVINFO pMVI, PCSZ pszName)
{
	BOOL bOK = FALSE;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		bOK = pMVI->pMvInfo->MtOpenTopicList(pszName);
		}

	return (bOK);
}


EXTERN_C PCSZ DLL_EXPORT MviMtLookupPhrase (PMVINFO pMVI, PCSZ pszPhrase, BOOL bDefinition)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		pMVI->sResult = pMVI->pMvInfo->MtLookupPhrase(pszPhrase, bDefinition);
		pszResult = pMVI->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MviMtLookupDefinitionPhrase (PMVINFO pMVI, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		pMVI->sResult = pMVI->pMvInfo->MtLookupDefinitionPhrase(pszPhrase);
		pszResult = pMVI->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MviMtLookupCrossReferences (PMVINFO pMVI, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVI && pMVI->pMvInfo);

	if (pMVI && pMVI->pMvInfo) {
		pMVI->sResult = pMVI->pMvInfo->MtLookupCrossReferences(pszPhrase);
		pszResult = pMVI->sResult;
		}

	return (pszResult);
}


// C-wrappers for CMvLookup
//

EXTERN_C PMVLOOKUP DLL_EXPORT MvlCreate (PMVINFO pMVI)
{
	PMVLOOKUP pMVL = NULL;

	if (pMVI && pMVI->pMvInfo) {
		pMVL = new MVLOOKUP;
		if (pMVL) {
			pMVL->pMvLookup = new CMvLookup(pMVI->pMvInfo);
			}
		}

	return (pMVL);
}


EXTERN_C PMVLOOKUP DLL_EXPORT MvlDelete (PMVLOOKUP pMVL)
{
	if (pMVL) {
		delete pMVL->pMvLookup;
		delete pMVL;
		}

	return (NULL);
}


EXTERN_C PCSZ DLL_EXPORT MvlLookupPhrase (PMVLOOKUP pMVL, PCSZ pszPhrase, BOOL bDefinition)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->sResult = pMVL->pMvLookup->LookupPhrase(pszPhrase, bDefinition);
		pszResult = pMVL->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MvlLookupDefinitionPhrase (PMVLOOKUP pMVL, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->sResult = pMVL->pMvLookup->LookupPhrase(pszPhrase, TRUE);
		pszResult = pMVL->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MvlLookupCrossReferences (PMVLOOKUP pMVL, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->sResult = pMVL->pMvLookup->LookupCrossReferences(pszPhrase);
		pszResult = pMVL->sResult;
		}

	return (pszResult);
}



EXTERN_C VOID DLL_EXPORT MvlUseSgml (PMVLOOKUP pMVL, BOOL bUseSgml)
{
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->pMvLookup->UseSgml(bUseSgml);
		}
}


EXTERN_C BOOL DLL_EXPORT MvlIsValid (PMVLOOKUP pMVL)
{
	BOOL bValid = FALSE;

	if (pMVL && pMVL->pMvLookup) {
		bValid = pMVL->pMvLookup->IsValid();
		}

	return (bValid);
}


// Multithreaded access functions
//

EXTERN_C PCSZ DLL_EXPORT MvlMtLookupPhrase (PMVLOOKUP pMVL, PCSZ pszPhrase, BOOL bDefinition)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->sResult = pMVL->pMvLookup->MtLookupPhrase(pszPhrase, bDefinition);
		pszResult = pMVL->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MvlMtLookupDefinitionPhrase (PMVLOOKUP pMVL, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->sResult = pMVL->pMvLookup->MtLookupPhrase(pszPhrase, TRUE);
		pszResult = pMVL->sResult;
		}

	return (pszResult);
}


EXTERN_C PCSZ DLL_EXPORT MvlMtLookupCrossReferences (PMVLOOKUP pMVL, PCSZ pszPhrase)
{
	PCSZ pszResult = NULL;
	ASSERT(pMVL && pMVL->pMvLookup);

	if (pMVL && pMVL->pMvLookup) {
		pMVL->sResult = pMVL->pMvLookup->MtLookupCrossReferences(pszPhrase);
		pszResult = pMVL->sResult;
		}

	return (pszResult);
}
