// MvLookup.h : main header file for the MVLOOKUP DLL
//

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols

#if !defined(DLL_EXPORT)
#define DLL_EXPORT	__declspec( dllexport )
#endif

// Constants
//
CONST CHAR cMV_ENTRY_SEP = ';';

#if FALSE
// Globals
//
EXTERN CMvLookupDlg* pDlg;				// built-in dialog for doing lookups
EXTERN HWND hDlg;						// handle for the build-in dialog
EXTERN VOID InitMvLookup(CWnd* pMainFrame);
EXTERN VOID ShutdownMvLookup(VOID);
#endif

/////////////////////////////////////////////////////////////////////////////
// CMvLookupDll
// See MvLookup.cpp for the implementation of this class
//

class CMvLookupDll : public CWinApp
{
public:
	CMvLookupDll();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMvLookupDll)
	public:
	virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CMvLookupDll)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


class CMvLookupDlg;
class CEvent;


//-----------------------------------------------------------------------------
// CMvInfo
// 
// Class for accessing MediaView. 
//
// OpenTitle: Open a MediaView title
//
// OpenTitle: Close the MediaView title
//
// OpenWordWheel: Open the MediaView wordwheel with the given name
// Wordwheels are used for doing keyword lookup
//
// OpenTopicList: Open the MediaView topic list with the given name
// Topic lists are used for doing full-text search (for cross-references)
//
// DoLookupDialog: Invoke the built-in lookup dialog
// This just activates the dialog since it is shamelessly used anyway.
//
// UseSgml: Indicate whether the results should be formatted with (pseudo) SGML
//

class DLL_EXPORT CMvInfo : public CWinThread
{
	DECLARE_DYNCREATE(CMvInfo);
public:
	//CMvInfo(CWnd* pParentWnd, PCSZ pszTitle);
	CMvInfo (VOID);
	~CMvInfo (VOID);
	BOOL Init();
	BOOL OpenTitle (PCSZ pszTitle);
	BOOL CloseTitle (VOID);
	BOOL OpenWordWheel (PCSZ pszName);
	BOOL OpenTopicList (PCSZ pszName);
	BOOL MtOpenTitle (PCSZ pszTitle);
	BOOL MtCloseTitle (VOID);
	BOOL MtOpenWordWheel (PCSZ pszName);
	BOOL MtOpenTopicList (PCSZ pszName);
	CWnd* DoLookupDialog (CWnd* pParentWnd);
	CMvLookupDlg* GetLookupDialog (VOID);
	CString LookupPhrase (PCSZ pszPhrase);
	CString LookupCrossReferences (PCSZ pszPhrase);
	VOID LookupWorker (VOID);
	CString MtLookupPhrase (PCSZ pszPhrase);
	CString MtLookupCrossReferences  (PCSZ pszReferences);
	virtual BOOL InitInstance();
	virtual BOOL ExitInstance();
	BOOL IsValid (VOID);
public:
	BOOL m_bAddSgml;					// true if SGML-formatting is desired
	CString m_sPhrase;					// phrase to lookup in the MV database
	CString m_sResults;					// results from the lookup
	CString m_sTitle;					// file name of the MediaView title
	CString m_sWordWheel;				// name of word wheel (eg, Dictionary)
	CString m_sTopicList;				// name of topic list (eg, Thesaurus)
	BOOL m_bOK;							// boolean status of last command
	HWND m_hWndParent;					// handle of parent window (for dialog)
private:
	CMvLookupDlg* m_pDlg;				// built-in dialog for lookups
	HWND m_hDlg;						// handle of said dialog
	CEvent m_eventLookup;				// main event for lookup coordination
	CEvent m_eventLookupPhrase;			// event for signalling phrase lookup
	CEvent m_eventLookupReferences;		// event for signalling xref lookup
	CEvent m_eventOpenTitle;			// event for opening the database
	CEvent m_eventCloseTitle;			// event for closing the database
	CEvent m_eventOpenWordWheel;		// event for opening a word wheel
	CEvent m_eventOpenTopicList;		// event for opening a topic list
	CEvent m_eventResults;				// event for signalling results ready
};

MAKE_CLASS_CAST(CMvInfo);


//-----------------------------------------------------------------------------
// CMvLookup
// 
// Class for doing MediaView lookups (designed particularly for Bookshelf)
// This must be used in conjunction with CMvInfo. These are separated for
// both resource efficiency and for multithreaded concerns.
//
// LookupPhrase: Lookup the topics in the database given a phrase keyword
//
// LookupCrossReferences: Lookup the topics that contain a given phrase
//
// DoLookupDialog: Invoke the built-in lookup dialog
// This just activates the dialog since it is shamelessly used anyway.
//
// UseSgml: Indicate whether the results should be formatted with (pseudo) SGML
//

class DLL_EXPORT CMvLookup : public CObject
{
	DECLARE_DYNAMIC(CMvLookup);
public:
	CMvLookup (CMvInfo* pMvInfo);
	~CMvLookup (VOID);

	CString LookupPhrase (PCSZ pszPhrase);
	CString LookupCrossReferences (PCSZ pszPhrase);
	CString MtLookupPhrase (PCSZ pszPhrase);
	CString MtLookupCrossReferences  (PCSZ pszReferences);
	VOID UseSgml (BOOL bUseSgml = TRUE);
	BOOL IsValid (VOID);

private:
	//CMvLookupDlg* m_pDlg;				// built-in dialog for doing lookups
	CMvInfo* m_pMvInfo;					// class for MediaView title info
	BOOL m_bAddSgml;					// true if SGML-formatting is desired
};

MAKE_CLASS_CAST(CMvLookup);

/////////////////////////////////////////////////////////////////////////////

EXTERN DLL_EXPORT CMvInfo* CreateMvInfo (VOID);
