//	Copyleft 1993-1996 Tom O'Hara		CWord source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// Options.cpp : implementation file
//

#include "stdafx.h"
#include "cword.h"
#include "Lookup.h"
#include "Options.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCWordOptionsDlg

IMPLEMENT_DYNAMIC(CCWordOptionsDlg, CPropertySheet)

CCWordOptionsDlg::CCWordOptionsDlg(CWnd* pParentWnd, UINT iSelectPage)
{
	CPropertySheet("CWord Options", pParentWnd, iSelectPage);
	CommonInit();
}

CCWordOptionsDlg::CCWordOptionsDlg(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
	CommonInit();
}

CCWordOptionsDlg::CCWordOptionsDlg(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	CommonInit();
}


VOID CCWordOptionsDlg::CommonInit (VOID)
{
	m_bSettingsChanged = FALSE;
	AddPage(&m_pageCWordOptions);
	AddPage(&m_pageBookshelfOptions);
	AddPage(&m_pageTrlOptions);
	AddPage(&m_pageWordNetOptions);
}


CCWordOptionsDlg::~CCWordOptionsDlg()
{
}


BEGIN_MESSAGE_MAP(CCWordOptionsDlg, CPropertySheet)
	//{{AFX_MSG_MAP(CCWordOptionsDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


INT CCWordOptionsDlg::DoModal (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CCWordOptionsDlg::DoModal()\n"));

	INT nResult = CPropertySheet::DoModal();
	if (nResult == IDOK) {
		m_bSettingsChanged = m_pageCWordOptions.m_bSettingsChanged;
		if (m_pageBookshelfOptions.m_bSettingsChanged ||
				m_pageTrlOptions.m_bSettingsChanged ||
				m_pageWordNetOptions.m_bSettingsChanged) {
			CWaitCursor wait_cursor;
			CLookup::ReinitLookup();
			}
		}

	return (nResult);
}

/////////////////////////////////////////////////////////////////////////////
// CCWordOptionsDlg message handlers

/////////////////////////////////////////////////////////////////////////////
// CCwordOptions property page

IMPLEMENT_DYNCREATE(CCwordOptions, CPropertyPage)

CCwordOptions::CCwordOptions() : CPropertyPage(CCwordOptions::IDD)
{
	//{{AFX_DATA_INIT(CCwordOptions)
	m_sScaleFactor = _T("");
	//}}AFX_DATA_INIT
	m_colorGridText = GetAppOption(szGRID_TEXT, clrBLACK);
	m_colorIndexText = GetAppOption(szGRID_INDEX_TEXT, clrBLACK);
	m_colorGridBorder = GetAppOption(szGRID_BORDER, clrBLACK);
	m_colorClueBackground = GetAppOption(szCLUE_BACKGROUND, clrLIGHT_GRAY);
	m_colorGridBackground = GetAppOption(szGRID_BACKGROUND, clrOFF_BLUE);
	m_colorClueText = GetAppOption(szCLUE_TEXT, clrBLACK);
	m_colorActiveCell = GetAppOption(szACTIVE_CELL, clrPALE_GREEN);
	m_colorBlackenedCell = GetAppOption(szBLACKENED_CELL, clrBLACK);
	m_nScaleFactor = GetAppOption(szSCALE_FACTOR, 100);
	m_sScaleFactor.Format("%d", m_nScaleFactor);

}

CCwordOptions::~CCwordOptions()
{
}


VOID CCwordOptions::HandleColorOption (COLORREF& colorCurrent)
{
	COLORREF colorSelected = GetColorOption(colorCurrent, this);
	if (colorCurrent != colorSelected) {
		colorCurrent = colorSelected;
		m_bSettingsChanged = TRUE;
		//SetModified(TRUE);
		}
}


// Now back to our regular program
//

void CCwordOptions::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCwordOptions)
	DDX_Control(pDX, IDC_DARK_CELL, m_btnDarkCell);
	DDX_Control(pDX, IDC_GRID_TEXT, m_btnGridText);
	DDX_Control(pDX, IDC_GRID_BORDER, m_btnGridBorder);
	DDX_Control(pDX, IDC_CLUE_BACKGROUND, m_btnClueBackground);
	DDX_Control(pDX, IDC_GRID_BACKGROUND, m_btnGridBackground);
	DDX_Control(pDX, IDC_CLUE_TEXT, m_btnClueText);
	DDX_Control(pDX, IDC_ACTIVE_CELL, m_btnActiveCell);
	DDX_CBString(pDX, IDC_SCALE_FACTOR, m_sScaleFactor);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCwordOptions, CPropertyPage)
	//{{AFX_MSG_MAP(CCwordOptions)
	ON_BN_CLICKED(IDC_ACTIVE_CELL, OnActiveCell)
	ON_BN_CLICKED(IDC_CLUE_BACKGROUND, OnClueBackground)
	ON_BN_CLICKED(IDC_CLUE_TEXT, OnClueText)
	ON_BN_CLICKED(IDC_GRID_BACKGROUND, OnGridBackground)
	ON_BN_CLICKED(IDC_GRID_BORDER, OnGridBorder)
	ON_BN_CLICKED(IDC_GRID_TEXT, OnGridText)
	ON_BN_CLICKED(IDC_DARK_CELL, OnDarkCell)
	ON_CBN_EDITCHANGE(IDC_SCALE_FACTOR, OnEditchangeScaleFactor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCwordOptions message handlers

void CCwordOptions::OnActiveCell() 
{
	HandleColorOption(m_colorActiveCell);
}


void CCwordOptions::OnClueBackground() 
{
	HandleColorOption(m_colorClueBackground);
}


void CCwordOptions::OnClueText() 
{
	HandleColorOption(m_colorClueText);
}


void CCwordOptions::OnGridBackground() 
{
	HandleColorOption(m_colorGridBackground);
}


void CCwordOptions::OnGridBorder() 
{
	HandleColorOption(m_colorGridBorder);
}


void CCwordOptions::OnGridText() 
{
	HandleColorOption(m_colorGridText);
}


BOOL CCwordOptions::ApplySettings (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CCwordOptions::ApplySettings()\n"));

	WriteAppOption(szGRID_TEXT, m_colorGridText);
	WriteAppOption(szGRID_INDEX_TEXT, m_colorIndexText);
	WriteAppOption(szGRID_BORDER, m_colorGridBorder);
	WriteAppOption(szCLUE_BACKGROUND, m_colorClueBackground);
	WriteAppOption(szGRID_BACKGROUND, m_colorGridBackground);
	WriteAppOption(szCLUE_TEXT, m_colorClueText);
	WriteAppOption(szBLACKENED_CELL, m_colorBlackenedCell);
	WriteAppOption(szACTIVE_CELL, m_colorActiveCell);
	m_nScaleFactor = StringToInt(m_sScaleFactor);
	WriteAppOption(szSCALE_FACTOR, m_nScaleFactor);

	return (TRUE);
}


void CCwordOptions::OnOK() 
{
	DEBUG_OUT(TL_VERBOSE, ("CCwordOptions::OnOK()\n"));

	ApplySettings();
	CPropertyPage::OnOK();
}


BOOL CCwordOptions::OnApply() 
{
	DEBUG_OUT(TL_VERBOSE, ("CCwordOptions::OnApply()\n"));

	ApplySettings();
	return CPropertyPage::OnApply();
}

void CCwordOptions::OnDarkCell() 
{
	HandleColorOption(m_colorBlackenedCell);
}

void CCwordOptions::OnEditchangeScaleFactor() 
{
	m_bSettingsChanged = TRUE;
	//SetModified(TRUE);
}
