// casefram.cpp : implementation file
//

#include "stdafx.h"
#include "common.h"
#include "pcommon.h"
#include "parser.h"
#include "concept.h"
#include "lexicon.h"
#include "casefram.h"
#include "transiti.h"
#include "graph.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(CCaseFrame, CCmdTarget, 1)
IMPLEMENT_SERIAL(CObjectFrame, CCaseFrame, 1)
IMPLEMENT_SERIAL(CActionFrame, CCaseFrame, 1)
IMPLEMENT_SERIAL(CSentFrame, CCaseFrame, 1)

#define new DEBUG_NEW

/////////////////////////////////////////////////////////////////////////////
// CCaseFrame

CCaseFrame::CCaseFrame(VOID)
{
	TRACEX(4, ("CCaseFrame::CCaseFrame()\n"));
	m_ptokens = NULL;
	m_bOriginal = TRUE;
	m_nPos = -1;
	m_pLexicon = NULL;
	m_nDepth = 0;
	m_sPopDestination.Empty();
}

//CCaseFrame::CCaseFrame (CONST CString& text, CONST CString& graph)

CCaseFrame::CCaseFrame (CONST CString& text)
{
	TRACEX(4, ("CCaseFrame::CCaseFrame(%s)\n", (PCSZ)text));
	m_ptokens = NULL;
	m_bOriginal = TRUE;
	m_nPos = -1;
	m_pLexicon = NULL;
	m_nDepth = 0;
	m_sPopDestination.Empty();
	InitTokens(text);
}

CCaseFrame::CCaseFrame (CONST CCaseFrame& other)
{
	TRACEX(4, ("CCaseFrame::CCaseFrame(%p)\n", &other));
	*this = other;
}

CCaseFrame::~CCaseFrame (VOID)
{
	TRACEX(4, ("CCaseFrame::~CCaseFrame()\n"));
#if FALSE
	if (m_bOriginal && m_ptokens) {
		if (!IsControlPressed()) {
			for (INT i = 0; i < m_ptokens->GetSize(); i++) {
				delete (*m_ptokens)[i];
				}
			}
		}
#endif
	if (IsOriginal() && m_ptokens) {
		delete m_ptokens;
		}
}

CONST CCaseFrame& CCaseFrame::operator= (CONST CCaseFrame& other)
{
	m_ptokens = other.m_ptokens;
	m_nPos = other.m_nPos;
	m_pLexicon = other.m_pLexicon;
	m_nDepth = other.m_nDepth;
	m_bOriginal = FALSE;
	m_sPopDestination = other.m_sPopDestination;
	return (*this);
}

VOID CCaseFrame::InitTokens (CONST CString& text)
{
	TRACEX(4, ("CCaseFrame::InitTokens('%s')\n", text));
	m_ptokens = new CStringArray;
	m_text = text;
	TokenizeText(*m_ptokens, m_text, " ,");
	m_nPos = 0;
}

BOOL CCaseFrame::CompatibleCase (CCaseFrame& other)
{
	return (m_case == other.m_case);
	
}

BOOL CCaseFrame::CompatibleNumber (CCaseFrame& other)
{
	return ((m_number == other.m_number) 
			|| (m_number == N_BOTH) || (other.m_number == N_BOTH));
}

CString& CCaseFrame::GetCurrentTokens (INT nNum, CHAR cDelim /* = cSPACE */)
{
	TRACEX(4, ("CCaseFrame::GetCurrentTokens(%d)\n", nNum));
//// modify JoinTokens to accomplish this
	CString tokens = "";
	for (INT i = GetPos(); i < GetPos() + nNum; i++) {
		if (i > GetPos()) {
			tokens += cDelim;
			}
		tokens += GetToken(i);
		}

	return (tokens);
}

VOID CCaseFrame::Serialize (CArchive& ar)
{
	CCmdTarget::Serialize(ar);
	if (ar.IsStoring()) {
		ar << m_ptokens << ToWord(GetPos()) << ToWord(GetDepth()) << ToWord(IsOriginal());
		ar << m_sPopDestination;
		}
	else {
		WORD wPos, wDepth, wOriginal;
		ar >> m_ptokens >> wPos >> wDepth >> wOriginal;
		ar >> m_sPopDestination;
		SetPos(wPos);
		SetDepth(wDepth);
		SetOriginal(wOriginal);
		}
}

#if defined(_DEBUG)
VOID CCaseFrame::AssertValid() CONST
{
	CObject::AssertValid();
	ASSERT_VALID(m_ptokens);
	ASSERT(GetPos() != -1);
	ASSERT_VALID(m_pLexicon);
}

VOID CCaseFrame::Dump(CDumpContext& dc) CONST
{
	CObject::Dump(dc);
	DUMP_VAR(m_nDepth);
	DUMP_VAR(m_bOriginal);
	DUMP_VAR(m_nPos);
	DUMP_VAR(m_sPopDestination);
	DUMPLN_VAR(m_ptokens);
//	dc << "m_ptoken = " << m_ptokens << "; m_pos = " << m_pos << "\n";
	dc << "m_pLexicon = " << &m_pLexicon << "\n";
}
#endif //_DEBUG

BEGIN_MESSAGE_MAP(CCaseFrame, CCmdTarget)
	//{{AFX_MSG_MAP(CCaseFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCaseFrame message handlers


/////////////////////////////////////////////////////////////////////////////
// CObjectFrame

CObjectFrame::CObjectFrame()
{
}

CObjectFrame::~CObjectFrame()
{
}

VOID CObjectFrame::Serialize (CArchive& ar)
{
	CCaseFrame::Serialize(ar);
	if (ar.IsStoring()) {
		ar << ToLong(m_bDirect);
		}
	else {
		LONG lDirect;
		ar >> lDirect;
		m_bDirect = ToInt(lDirect);
		}
}

#if defined(_DEBUG)
VOID CObjectFrame::AssertValid() CONST
{
	CCaseFrame::AssertValid();
}

VOID CObjectFrame::Dump(CDumpContext& dc) CONST
{
	CCaseFrame::Dump(dc);
	dc << "m_bDirect = " << m_bDirect << "\n";
}
#endif //_DEBUG

BEGIN_MESSAGE_MAP(CObjectFrame, CCaseFrame)
	//{{AFX_MSG_MAP(CCaseFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CActionFrame

CActionFrame::CActionFrame()
{
}

CActionFrame::~CActionFrame()
{
}

VOID CActionFrame::Serialize (CArchive& ar)
{
	CCaseFrame::Serialize(ar);
	if (ar.IsStoring()) {
		ar << ToLong(m_bImperative);
		}
	else {
		LONG lImperative;
		ar >> lImperative;
		m_bImperative = ToInt(lImperative);
		}
}

#if defined(_DEBUG)
VOID CActionFrame::AssertValid() CONST
{
	CCaseFrame::AssertValid();
}

VOID CActionFrame::Dump(CDumpContext& dc) CONST
{
	CCaseFrame::Dump(dc);
	dc << "m_bImperative = " << m_bImperative << "\n";
}
#endif //_DEBUG

BEGIN_MESSAGE_MAP(CActionFrame, CCaseFrame)
	//{{AFX_MSG_MAP(CCaseFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CSentFrame

CSentFrame::CSentFrame()
{
}

CSentFrame::~CSentFrame()
{
}

VOID CSentFrame::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		}
	else {
		}
	m_subject.Serialize(ar);
	m_action.Serialize(ar);
	m_target.Serialize(ar);
}

#if defined(_DEBUG)
VOID CSentFrame::AssertValid() CONST
{
	CObject::AssertValid();
	ASSERT_VALID(&m_subject);
	ASSERT_VALID(&m_action);
	ASSERT_VALID(&m_target);
}

VOID CSentFrame::Dump(CDumpContext& dc) CONST
{
	CObject::Dump(dc);
	dc << "m_subject: " << &m_subject << "\n";
	dc << "m_action: " << &m_action << "\n";
	dc << "m_target: " << &m_target << "\n";
}
#endif //_DEBUG

BEGIN_MESSAGE_MAP(CSentFrame, CCaseFrame)
	//{{AFX_MSG_MAP(CCaseFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

