// casefram.h : header file
//


/////////////////////////////////////////////////////////////////////////////
// CCaseFrame command target

class CCaseFrame : public CCmdTarget
{   
	DECLARE_SERIAL(CCaseFrame)
protected:
	CCaseFrame (VOID);			// protected constructor used by dynamic creation
public:
//	CCaseFrame (CONST CString& text, CONST CString& graph);
	CCaseFrame (CONST CString& text);
	CCaseFrame (CONST CCaseFrame& other);
	CONST CCaseFrame& operator= (CONST CCaseFrame& other);

// Attributes
public:
	typedef enum {N_SINGULAR, N_PLURAL, N_BOTH} NUMBER;
	typedef enum {C_SUBJECTIVE, C_OBJECTIVE} CASE;

	CStringArray& GetTokenArray (VOID)
				{ ASSERT(m_ptokens);  return (m_ptokens ? *m_ptokens : asNull); }
	CONST CString& GetToken (INT nPos)
				{ ASSERT(ValidIndex(nPos, GetTokenArray()));  
				  return  (ValidIndex(nPos, GetTokenArray()) ? GetTokenArray()[nPos] : sNULL); }
	CONST CString& GetCurrentToken (VOID)
				{ ASSERT(ValidPos() || (GetPos() == GetNumTokens()));
				  return (ValidPos() ? GetTokenArray()[GetPos()] : sNULL); }
	CString& GetCurrentTokens (INT nNum, CHAR cDelim = cSPACE);
	INT GetNumTokens (VOID)			{ return (GetTokenArray().GetSize()); }
	INT GetNumTokensLeft (VOID)
				//	NOTE: pos is zero-based
				{ INT nNumLeft = (GetNumTokens() > 0) ? (GetNumTokens() - GetPos()) : 0;
				  ASSERT(nNumLeft >= 0);
				  return (nNumLeft); }
	CLexicon* GetLexicon (VOID)		{ return (m_pLexicon); }
	INT GetPos (VOID) CONST			{ return (m_nPos); }
	BOOL ValidPos (VOID)			{ return (ValidIndex(GetPos(), GetTokenArray())); }
	BOOL NoMoreTokens (VOID)		{ return (GetNumTokensLeft() == 0); }
	INT GetDepth (VOID) CONST		{ return (m_nDepth); }
	BOOL IsOriginal (VOID) CONST	{ return (m_bOriginal); }
	CONST CString& GetPopDestination (VOID) { return (m_sPopDestination); }
	
// Future attributes
public:

// Operations
public:
	VOID SetLexicon (CLexicon* pLexicon)	{ m_pLexicon = pLexicon; }
//	CONST CCaseFrame operator= (CONST CCaseFrame& other);
	VOID AdvanceToken (INT nLen = 1)	{ m_nPos += nLen; }
	VOID SetPos (INT nPos)				{ m_nPos = nPos; }
	VOID SetDepth (INT nDepth)			{ m_nDepth = nDepth; }
	VOID IncDepth (VOID)				{ m_nDepth++; }
	VOID SetOriginal (BOOL bOriginal)	{ m_bOriginal = bOriginal; }
	VOID SetPopDestination (PCSZ pszDest)	{ m_sPopDestination = pszDest; }

// Future operations
public:
	VOID SetCase (CASE caseValue)	{ m_case = caseValue; }
	CASE GetCase (VOID)				{ return (m_case); }
	BOOL CompatibleCase (CCaseFrame& other);
	VOID SetNumber (NUMBER number)	{ m_number = number; }
	NUMBER GetNumber (VOID)		    { return (m_number); }
	BOOL CompatibleNumber (CCaseFrame& other);

// Implementation
public:
	CStringArray* m_ptokens;	// copied from the outermost caseframe (eg, Sent)
	BOOL m_bOriginal;			// if this is the original version of the frame
	INT m_nPos;					// current position in the token list
	INT m_nDepth;				// recursive depth
	CString m_sPopDestination;	// the label a POP destination resolves to
	// TODO; look into alternative techniques of making this available
	//		 eg, a global class
	CLexicon* m_pLexicon;
public:
	VOID InitTokens (CONST CString& text);
	virtual ~CCaseFrame();

	friend class CTransition;

// Future Implementation
private:
	NUMBER m_number;
	CASE m_case;
	CConcept m_concept;
	CString m_text;
	CString m_pre;
	CString m_head;
	CString m_post;

protected:
	// Generated message map functions
	//{{AFX_MSG(CCaseFrame)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) CONST;
	virtual VOID Dump (CDumpContext& dc) CONST;
#endif
};

/////////////////////////////////////////////////////////////////////////////

class CObjectFrame : public CCaseFrame
{
	DECLARE_SERIAL(CObjectFrame)
public:
	CObjectFrame (VOID);
protected:
	// Generated message map functions
	//{{AFX_MSG(CCaseFrame)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
// Implementation
public:
	~CObjectFrame (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) CONST;
	virtual VOID Dump (CDumpContext& dc) CONST;
#endif
private:
	BOOL m_bDirect;
};

class CActionFrame : public CCaseFrame
{
	DECLARE_SERIAL(CActionFrame)

public:
	CActionFrame (VOID);
protected:
	// Generated message map functions
	//{{AFX_MSG(CCaseFrame)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

// Implementation
public:
	~CActionFrame (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) CONST;
	virtual VOID Dump (CDumpContext& dc) CONST;
#endif
private:
	BOOL m_bImperative;
};

class CSentFrame : public CCaseFrame
{
	DECLARE_SERIAL(CSentFrame)

public:
	CSentFrame (VOID);
protected:
	// Generated message map functions
	//{{AFX_MSG(CCaseFrame)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

// Implementation
public:
	~CSentFrame (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) CONST;
	virtual VOID Dump (CDumpContext& dc) CONST;
#endif
private:
	CObjectFrame m_subject;
	CActionFrame m_action;
	CObjectFrame m_target;
};
