// concept.cpp : implementation file
//

#include "stdafx.h"
#include "common.h"
#include "pcommon.h"
#include "parser.h"
#include "concept.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(CConceptNode, CObject, 1)
IMPLEMENT_SERIAL(CRelationship, CObject, 1)
IMPLEMENT_SERIAL(CRelation, CObject, 1)
IMPLEMENT_SERIAL(CRelList, CObList, 1)
IMPLEMENT_SERIAL(CConcept, CObject, 1)
IMPLEMENT_SERIAL(CConceptMonger, CCmdTarget, 1);

#define new DEBUG_NEW

/////////////////////////////////////////////////////////////////////////////
// CConceptNode

CConceptNode::CConceptNode (VOID)
{
}

CConceptNode::~CConceptNode (VOID)
{
}
             
VOID CConceptNode::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		ar << m_desc;
		}
	else {
		ar >> m_desc;
		}
}

#if defined(_DEBUG)
VOID CConceptNode::AssertValid() const
{
	CObject::AssertValid();
	ASSERT(m_desc != "");
}

VOID CConceptNode::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "m_desc = " << m_desc << "\n";
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRelationship

CMapStringToOb CRelationship::m_typeMap;

CRelationship::CRelationship (VOID)
{
}

CRelationship::CRelationship (CString& sType)
{
	m_type = sType;
	ASSERT(FindRelationship(sType) == NULL);
	SetAssociation();
}

CRelationship::~CRelationship (VOID)
{
}

CRelationship* CRelationship::FindRelationship(CONST CString& sType)
{
	CObject* pObject;
	BOOL bOK = m_typeMap.Lookup(sType, pObject);

	return (bOK ? ToCRelationship(pObject) : NULL);
}

VOID CRelationship::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		ar << m_type;
		}
	else {
		ar >> m_type;
		SetAssociation();
		}
}

#if defined(_DEBUG)
VOID CRelationship::AssertValid() const
{
	CObject::AssertValid();
	ASSERT(m_type != "");
}

VOID CRelationship::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "m_type = " << m_type << "\n";
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRelation

CRelation::CRelation (VOID)
{
	Reset();
}

CRelation::CRelation (CRelationship* ptype, CConceptNode* pnode)
{
	m_ptype = ptype;
	m_pnode = pnode;
}

CRelation::CRelation (CONST CRelation& other)
{
	m_ptype = other.m_ptype;
	m_pnode = other.m_pnode;
}

CRelation::CRelation (CRelation* pother)
{
	Reset();
	ASSERT(pother);
	if (pother) {
		m_ptype = pother->m_ptype;
		m_pnode = pother->m_pnode;
		}
}

VOID CRelation::Reset (VOID)
{
	m_ptype = NULL;
	m_pnode = NULL;
}

CRelation::~CRelation (VOID)
{
}
             
VOID CRelation::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		ar << m_ptype << m_pnode;
		}
	else {
		ar >> m_ptype >> m_pnode;
		}
}

#if defined(_DEBUG)
VOID CRelation::AssertValid() const
{
	CObject::AssertValid();
	ASSERT_VALID(m_ptype);
	ASSERT_VALID(m_pnode);
}

VOID CRelation::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "m_type = " << m_ptype << "; m_pnode = " << m_pnode << "\n";
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRelList

CRelList::CRelList (VOID)
{
}

CRelList::CRelList (const CRelList& other)
{
	POSITION pos = other.IsEmpty() ? NULL : other.GetHead();
	while (pos != NULL) {
		CRelation* prel = new CRelation(other.GetAt(pos));
		AddRelation(prel);
		other.GetNext(pos);
		}
}

VOID CRelList::AddRelation (CRelation& rel)
{
	CRelation* prel = new CRelation(rel);
	AddTail(prel);
}

VOID CRelList::AddRelation (CRelation* prel)
{
	ASSERT(prel);
	if (prel) {
		AddTail(new CRelation(prel));
		}
}

CRelList::~CRelList (VOID)
{
	POSITION pos = IsEmpty() ? NULL : GetHead();
	while (pos != NULL) {
		DeleteRelationAt(pos); 
		GetNext(pos);
		}
}

VOID CRelList::Serialize (CArchive& ar)
{
	CObList::Serialize(ar);
	if (ar.IsStoring()) {
		}
	else {
		}
}

#if defined(_DEBUG)
VOID CRelList::AssertValid() const
{
	CObList::AssertValid();
}

VOID CRelList::Dump(CDumpContext& dc) const
{
	CObList::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CConcept

CConcept::CConcept(VOID)
{
}

CConcept::~CConcept(VOID)
{
}

VOID CConcept::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		}
	else {
		}
	m_node.Serialize(ar);
	m_links.Serialize(ar);
}

#if defined(_DEBUG)
VOID CConcept::AssertValid() const
{
	CObject::AssertValid();
	ASSERT_VALID(&m_node);
	ASSERT_VALID(&m_links);
}

VOID CConcept::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "m_node = " << &m_node << "\n";
	dc << "m_links = " << &m_links << "\n";
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CConceptMonger

CConceptMonger::CConceptMonger(VOID)
{
}

CConceptMonger::~CConceptMonger(VOID)
{
}

BEGIN_MESSAGE_MAP(CConceptMonger, CCmdTarget)
	//{{AFX_MSG_MAP(CConcept)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CConceptMonger::BuildConceptMap (VOID)
{
	TRACEX(4, ("CConceptMonger::BuildConceptMap \n"));
	BOOL bOK = FALSE;
	ASSERT(m_concepts.GetSize() > 0);
	for (INT i = 0; i < m_concepts.GetSize(); i++) {
		CConcept* pconcept = ToCConcept(m_concepts[i]);
		ASSERT(pconcept->IsKindOf(RUNTIME_CLASS(CConcept)));
		m_conceptMap.SetAt(pconcept->GetDescription(), pconcept);
		TRACEX(4, ("Adding association (%s,%p)\n", (PCSZ)pconcept->GetDescription(), pconcept));
		}
		
	return (bOK);
}

void CConceptMonger::AddConcept (CConcept& concept)
{
	m_concepts.Add(&concept);
	m_conceptMap.SetAt(concept.GetDescription(), &concept);
}

CConcept* CConceptMonger::FindConcept (CString& desc)
{
	CObject* pentry;
	BOOL bOK = m_conceptMap.Lookup(desc, pentry);
	
	return (bOK ? ToCConcept(pentry) : NULL);
}

VOID CConceptMonger::Serialize (CArchive& ar)
{
	CCmdTarget::Serialize(ar);
	m_concepts.Serialize(ar);
	if (ar.IsStoring()) {
		}
	else {
		BuildConceptMap();
		}
}

#if defined(_DEBUG)
VOID CConceptMonger::AssertValid(VOID) const
{
	CObject::AssertValid();
	ASSERT_VALID(&m_conceptMap);
}

VOID CConceptMonger::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "m_concepts: " << m_concepts << "\n";
	dc << "m_conceptMap: " << m_conceptMap << "\n";
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CConceptMonger message handlers

