// concept.h : header file
//


/////////////////////////////////////////////////////////////////////////////
// Classes to implement Roget-style classification
//
// Currently, these classes are inefficient since string values are
// used instead of atoms. Also, no attempt is made to eliminate
// redundancy (eg, through the use pointers).
//

class CConceptNode : public CObject
{
	DECLARE_SERIAL(CConceptNode);
public:
	CConceptNode (VOID);
// Attributes
	CString& GetDescription (VOID)		{ return (m_desc); }
// Implementation
private:
	CString m_desc;
public:
	~CConceptNode (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CConceptNode)


class CRelationship : public CObject
{
	DECLARE_SERIAL(CRelationship);
public:
	CRelationship (VOID);
	CRelationship (CString& type);
// Attributes
	CString& GetType (VOID)		{ return (m_type); }
// Operations
	STATIC CRelationship* FindRelationship (CONST CString& sType);
// Implementation
private:
	CString m_type;
	STATIC CMapStringToOb m_typeMap;
	VOID SetAssociation (VOID)		{ m_typeMap.SetAt(m_type, this); }
public:
	~CRelationship (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CRelationship)

class CRelation : public CObject
{
	DECLARE_SERIAL(CRelation);
public:
	CRelation (VOID);
	CRelation (CONST CRelation& other);
	CRelation (CRelation* pother);
	CRelation (CRelationship* ptype, CConceptNode* pnode);

// Attributes

// Operations
	VOID Reset (VOID);
	
// Implementation
public:
	~CRelation (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CRelationship* m_ptype;
	CConceptNode* m_pnode;
};

MAKE_CLASS_CAST(CRelation)


class CRelList : public CObList
{
	DECLARE_SERIAL(CRelList);
public:
	CRelList(VOID);
	CRelList(const CRelList& other);
	VOID AddRelation(CRelation& rel);
	VOID AddRelation (CRelation* prel);
	VOID DeleteRelationAt(POSITION& rPosition)
		{ delete GetAt(rPosition); }
// overridden operations
	CRelation* GetAt(POSITION position) const
		{ return (ToCRelation(CObList::GetAt(position))); }
	CRelation* GetNext(POSITION& position) const
		{ return (ToCRelation(CObList::GetNext(position))); }
	CRelation* GetPrev(POSITION& position) const
		{ return (ToCRelation(CObList::GetPrev(position))); }
// Implementation
public:
	~CRelList(VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
};

MAKE_CLASS_CAST(CRelList)


class CConcept : public CObject
{   
	DECLARE_SERIAL(CConcept);
public:
	CConcept(VOID);

// Attributes
public:
	CString& GetDescription (VOID)		{ return (m_node.GetDescription()); }
	CConceptNode& GetNode (VOID)		{ return (m_node); }
	CRelList& GetRelations (VOID)		{ return (m_links); }

// Operations
public:
	VOID AddRelation (CRelation& prel)	{ m_links.AddRelation(prel); }

// Implementation
public:
	virtual ~CConcept(VOID);

// Implementation
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CConceptNode m_node;
	CRelList m_links;
};

MAKE_CLASS_CAST(CConcept)


/////////////////////////////////////////////////////////////////////////////
// CConceptMonger command target (a glorified hash table)

class CConceptMonger : public CCmdTarget
{
	DECLARE_SERIAL(CConceptMonger)
public:
	CConceptMonger (VOID);

// Attributes
public:
//// make the concept array temporary since it is just for serialization
	CObArray m_concepts;
	CMapStringToOb m_conceptMap;

protected:
	// Generated message map functions
	//{{AFX_MSG(CConcept)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

// Operations
public:
	VOID AddConcept (CConcept& concept);
	CConcept* FindConcept (CString& desc);

// Implementation
public:
	~CConceptMonger (VOID);
	BOOL BuildConceptMap (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
};

MAKE_CLASS_CAST(CConceptMonger)

/////////////////////////////////////////////////////////////////////////////
