// graph.cpp : implementation file
//

#include "stdafx.h"
#include "common.h"
#include "pcommon.h"
#include "parser.h"
#include "concept.h"
#include "lexicon.h"
#include "casefram.h"
#include "transiti.h"
#include "graph.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(CGraphLink, CObject, 1)
IMPLEMENT_SERIAL(CGraphNode, CObject, 1)
IMPLEMENT_SERIAL(CGraph, CCmdTarget, 1)

#define new DEBUG_NEW

/////////////////////////////////////////////////////////////////////////////
// CGraphLink
             
CGraphLink::CGraphLink (VOID)
{
}
             
CGraphLink::CGraphLink (CONST CString& from, CONST CString& trans, CONST CString& to)
{
	TRACEX(TL_VERBOSE, ("CGraphLink::CGraphLink(%s, %s, %s)\n", 
				(PCSTR)from, (PCSTR)trans, (PCSTR)to));
	m_from = from;
	m_trans = trans;
	m_to = to;
	m_bPushLink = FALSE;
}
             
VOID CGraphLink::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		ar << m_from << m_trans << m_to << (LONG)m_bPushLink;
		}
	else {
		LONG lPushLink;
		ar >> m_from >> m_trans >> m_to >> lPushLink;
		//	TODO: define ToBool()
		m_bPushLink = ToInt(lPushLink);
		}
}

#if defined(_DEBUG)
VOID CGraphLink::AssertValid() const
{
	CObject::AssertValid();
	ASSERT(m_from != szNULL);
	ASSERT(m_trans != szNULL);
	ASSERT(m_to != szNULL);
}

VOID CGraphLink::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	DUMP_VAR(m_from);
	DUMP_VAR(m_trans);
	DUMP_VAR(m_to);
	DUMP_VAR(m_bPushLink);
//	dc << "From: " << m_from << " Trans: " << m_trans << " To: " << m_to;
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CGraphNode
             
CGraphNode::CGraphNode (VOID)
{
}
             
CGraphNode::CGraphNode (CONST CString& label)
{
	TRACEX(TL_VERBOSE, ("CGraphNode::CGraphNode(%s)\n", (PCSTR)label));
	ASSERT(label != szNULL);
	m_label = label;
}
             
CGraphNode::~CGraphNode (VOID)
{
	for (INT i = 0; i < m_links.GetSize(); i++) {
		CGraphLink* plink = ToCGraphLink(m_links[i]);
		ASSERT(plink);
		delete plink;
		}
}
             
CTransition* CGraphNode::GetTransition (INT nPos)
{
	TRACEX(TL_VERBOSE, ("CGraphNode::GetTransition(%d) on %s\n", nPos, (PCSZ)GetLabel()));
	CTransition* ptrans = NULL;
	CGraphLink* pLink = GetLink(nPos);
	ASSERT(pLink);
		
	if (pLink) {
		CString* pLabel = pLink->GetTransLabel();
		ASSERT(pLabel);
		if (pLabel) {
			ptrans = CTransition::FindTransition(*pLabel);
			}
		}

	return (ptrans);
}

CString* CGraphNode::GetToNode (INT nPos)
{
	CString* plabel = NULL;
	CGraphLink* pLink = GetLink(nPos);
	ASSERT(pLink);
		
	if (pLink) {
		plabel = pLink->GetToNode();
		}

	return (plabel);
}

VOID CGraphNode::AddTransition (CONST CString& trans, CONST CString& to, BOOL bPushLink)
{
	TRACEX(TL_VERBOSE, ("CGraphNode::AddTransition(%s, %s, %d)\n", (PCSTR)trans, (PCSTR)to, bPushLink));
	ASSERT(m_label != szNULL);
	CGraphLink* pLink = new CGraphLink(m_label, trans, to);
	m_links.Add(pLink);
	//	TODO: extend the graph link constructor to handle this
	if (bPushLink) {
		pLink->SetPushLink();
		}
}

VOID CGraphNode::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
		ar << m_label;
		}
	else {
		ar >> m_label;
		}
	m_links.Serialize(ar);
}

#if defined(_DEBUG)
VOID CGraphNode::AssertValid() const
{
	CObject::AssertValid();
	ASSERT(m_label != szNULL);
	ASSERT_VALID(&m_links);
}

VOID CGraphNode::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	DUMPLN();
	DUMP_VAR(m_label);
	DUMP_OBJ(m_links);
//	dc << "\nLabel: " << m_label << "\nLinks: " << &m_links;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraph

CMapStringToOb CGraph::graphMap;

CGraph::CGraph (VOID)
{
}

#if FALSE
CGraph::CGraph (CONST CString& name)
{
	n_name = name;
	Register(m_name);
}
#endif

CGraph::CGraph (CONST CString& filename)
{
	TRACEX(TL_VERBOSE, ("CGraph::CGraph(%s)\n", (PCSZ)filename));
	m_name = ExtractFileBase(filename);
	Register(m_name, this);
	BuildFromFile(filename);
}

CGraph::~CGraph (VOID)
{
	Deregister(m_name);
	for (INT i = 0; i < m_nodes.GetSize(); i++) {
		CGraphNode* pNode = ToCGraphNode(m_nodes[i]);
		ASSERT(pNode);
		delete pNode;
		}
}


VOID CGraph::Register (CONST CString& name, CGraph* pgraph)
{
	graphMap.SetAt(name, pgraph);
}

VOID CGraph::Deregister (CONST CString& name)
{
	graphMap.RemoveKey(name);
}

//	FindGraph - Find the graph object with the specified nam
//		assumes it hasn't been deleted (eg, isn't out of scope)
//
CGraph* CGraph::FindGraph (CONST CString& name)
{
	CObject* pentry;
//// should this function require input in uppercase (for efficiency)? 
	CString sName(name);
	sName.MakeUpper();
	BOOL bOK = graphMap.Lookup(sName, pentry);
	if (bOK) {
		ASSERT_VALID(pentry);
		}
	
	return (bOK ? ToCGraph(pentry) : NULL);
}


BOOL CGraph::BuildFromFile (CONST CString& filename)
{
	BOOL bOK = FALSE;

	ASSERT(filename != szNULL);
	TRACEX(TL_VERBOSE, ("CGraph::BuildFromFile(%s)\n", (PCSZ)filename));
	TRY {
		CStdioFile file (filename, CFile::modeRead | CFile::typeText);
		CHAR szLine[256];
		INT nLine = 0;
		for (;;) {
			LPSTR lpLine = file.ReadString(szLine, STR_LEN(szLine));
			if (!lpLine) {
				break;
				}
//// what if line > 256
			nLine++;
			TRACEX(5, ("L%d: %s\n", nLine, lpLine));
			CStringx line(lpLine);
//			line = line.RemoveComment(';');
			line = line.Trim();
			line.MakeUpper();
			if ((line != szNULL) && (line[0] != ';')) {
//// make this more efficient
				CStringArray tokens;
				TokenizeText(tokens, line, "\t ");
				if (tokens.GetSize() != 3) {
					CString error = "Invalid transition specification at " 
									+ filename + " line " + String(nLine);
					AfxMessageBox(error);
////
					return (FALSE);
					}
				CGraphNode* pNode = GetNodePtr(tokens[0]);
				if (pNode == NULL) {
					pNode = new CGraphNode(tokens[0]);
					m_nodeMap.SetAt(tokens[0], pNode);
					m_nodes.Add(pNode);
					}

				//	Special case handling of a push link:	PUSH(get_np)
				//	TODO: cleanup;
				BOOL bPushLink = FALSE;
				CONST CHAR szPUSH[] = "PUSH(";
				CONST CHAR szRIGHT_PAREN[] = ")";
				if (tokens[1].Find(szPUSH) == 0) {
					if (tokens[1].Right(1) != szRIGHT_PAREN) {
						CString error = "Invalid transition specification at " 
										+ filename + " line " + String(nLine);
						AfxMessageBox(error);
						}
					else {
						INT nStart = STR_LEN(szPUSH);
						INT nLen = tokens[1].GetLength() - nStart - 1;
						tokens[1] = tokens[1].Mid(nStart, nLen);
						bPushLink = TRUE;
						}
					}

				pNode->AddTransition(tokens[1], tokens[2], bPushLink);
				}
			}
		bOK = TRUE;
		}
	CATCH(CFileException, e) {
#if defined(_DEBUG)
		afxDump << "File exception on '" << filename << "'; " << e;
#endif
		}
	END_CATCH

#if FALSE
	if (bOK) {
		BuildNodeMap();
		}
#endif

	return (bOK);
}

BOOL CGraph::BuildNodeMap (VOID)
{
	TRACEX(TL_VERBOSE, ("CGraph::BuildNodeMap\n"));
	BOOL bOK = FALSE;
	ASSERT(m_nodes.GetSize() > 0);
	for (INT i = 0; i < m_nodes.GetSize(); i++) {
		CGraphNode* pnode = ToCGraphNode(m_nodes[i]);
		ASSERT(pnode->IsKindOf(RUNTIME_CLASS(CGraphNode)));
		m_nodeMap.SetAt(pnode->GetLabel(), pnode);
		TRACEX(TL_VERBOSE, ("Adding association (%s,%p)\n", (PCSZ)pnode->GetLabel(), pnode));
		}
		
	return (bOK);
}

CGraphNode* CGraph::GetNodePtr (CONST CString& node)
{
	ASSERT(node != szNULL);
	CObject* pEntry = NULL;
//// should this function require input in uppercase (for efficiency)? 
	CString sNode(node);
	sNode.MakeUpper();
	BOOL bOK = m_nodeMap.Lookup(sNode, pEntry);
	
	return (bOK ? (ToCGraphNode(pEntry)) : NULL);
}

INT CGraph::GetNumTransitions (CONST CString& node)
{
	INT nNumTransitions = -1;

	CGraphNode* pNode = GetNodePtr(node);
	if (pNode) {
		nNumTransitions = pNode->GetNumTransitions();
		}

	return (nNumTransitions);	
}

CString* CGraph::GetToNode (CONST CString& node, INT nPos)
{
	CString* pLabel = NULL;

	CGraphNode* pNode = GetNodePtr(node);
	if (pNode) {
		pLabel = pNode->GetToNode(nPos);
		}

	return (pLabel);
}

CTransition* CGraph::GetTransition (CONST CString& node, INT nPos)
{
	TRACEX(TL_VERBOSE, ("CGraph::GetTransition(%s, %d)\n", node, nPos));
	CTransition* pTrans = NULL;

	CGraphNode* pNode = GetNodePtr(node);
	if (pNode) {
		pTrans = pNode->GetTransition(nPos);
		}

	return (pTrans);
}

BEGIN_MESSAGE_MAP(CGraph, CCmdTarget)
	//{{AFX_MSG_MAP(CGraph)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

VOID CGraph::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	m_nodes.Serialize(ar);
	if (ar.IsStoring()) {
		// node map cannot be serialized (logically)
		}
	else {
		// rebuild node map
		BuildNodeMap();
		}
}

#if defined(_DEBUG)
VOID CGraph::AssertValid() const
{
	CObject::AssertValid();
	ASSERT_VALID(&m_nodes);
	ASSERT(m_nodes.GetSize() > 0);
	ASSERT_VALID(&m_nodeMap);
}

VOID CGraph::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	DUMPLN_OBJ(m_nodes);
	DUMPLN_OBJ(m_nodeMap);
//	dc << &m_nodes << &m_nodeMap;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraph message handlers

