// graph.h : header file
//


class CGraphLink : public CObject
{
	DECLARE_SERIAL(CGraphLink)
protected:
	CGraphLink (VOID);
public:
	CGraphLink (CONST CString& from, CONST CString& trans, CONST CString& to);
public:
	CString* GetFromNode (VOID)		{ return (&m_from); }
	CString* GetTransLabel (VOID)	{ return (&m_trans); }
	CString* GetToNode (VOID)		{ return (&m_to); }
	BOOL IsPushLink (VOID)			{ return (m_bPushLink); }
// Operations
	VOID SetPushLink (VOID)			{ m_bPushLink = TRUE; }
// Implementation
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CString m_from;
	CString m_trans;
	CString m_to;
	BOOL m_bPushLink;
};

MAKE_CLASS_CAST(CGraphLink);


class CGraphNode : public CObject
{
	DECLARE_SERIAL(CGraphNode);
protected:
	CGraphNode (VOID);
public:
	CGraphNode (CONST CString& label);
	~CGraphNode (VOID);
// Attributes
public:
	CString& GetLabel (VOID)			{ return (m_label); }
	INT GetNumTransitions (VOID)		{ return (m_links.GetSize()); }
	BOOL IsTerminal (VOID)				{ return (m_links.GetSize() == 0); }
	CGraphLink* GetLink (INT nPos)
		{ ASSERT(ValidIndex(nPos, m_links));
		  return (ValidIndex(nPos, m_links) ? ToCGraphLink(m_links[nPos]) : NULL); }
	CTransition* GetTransition (INT nPos);
	CString* GetToNode (INT nPos);
	BOOL IsPushLink (INT nPos)
		{ CGraphLink* pLink = GetLink(nPos);
		  return (pLink ? pLink->IsPushLink() : FALSE); }
	CString* GetTransLabel(INT nPos)
		{ CGraphLink* pLink = GetLink(nPos);
		  return (pLink ? pLink->GetTransLabel() : NULL); }
// Operations
	VOID AddTransition (CONST CString& trans, CONST CString& to, BOOL bPushLink);
// Implementation
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CString m_label;
	CObArray m_links;
};

MAKE_CLASS_CAST(CGraphNode);


/////////////////////////////////////////////////////////////////////////////
// CGraph command target

class CGraph : public CCmdTarget
{   
	DECLARE_SERIAL(CGraph)
protected:
	CGraph (VOID);			// protected constructor used by dynamic creation
public:
	CGraph (CONST CString& file);
#if FALSE
	CGraph (CONST CString& name);
#endif

// Attributes
public:

// Operations
public:
	BOOL BuildFromFile (CONST CString& filename);
	CGraphNode* GetNodePtr (CONST CString& node);
	INT GetNumTransitions (CONST CString& node);
	CTransition* GetTransition (CONST CString& node, INT nPos);
//// consider a name change ("get what to node?")
	CString* GetToNode (CONST CString& node, INT nPos);

// Implementation
public:
//	~CGraph (VOID);
//// make m_nodes temporary (only use it for serialization)
	CObArray m_nodes;
	CMapStringToOb m_nodeMap;
	virtual ~CGraph (VOID);
private:
	BOOL BuildNodeMap (VOID);
	CString m_name;

protected:
	// Generated message map functions
	//{{AFX_MSG(CGraph)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
	STATIC CMapStringToOb graphMap;
	STATIC VOID Register (CONST CString& name, CGraph* pgraph);
	STATIC VOID Deregister (CONST CString& name);
	STATIC CGraph* FindGraph (CONST CString& name);
};

MAKE_CLASS_CAST(CGraph);

/////////////////////////////////////////////////////////////////////////////
