// lexicon.h : header file
//


class CDefinition : public CObject
{
	DECLARE_SERIAL(CDefinition)
protected:
	CDefinition();			// protected constructor used by dynamic creation
public:
	CDefinition (CONST CString& phrase, CONST CString& cat, CONST CString& def, INT nSense /* = 1*/);
public:
	CONST CString& GetPhrase (VOID)	{ return (m_phrase); }
	CONST CString& GetCat (VOID)		{ return (m_cat); }
	CONST CString& GetDef (VOID)		{ return (m_def); }
	INT GetSense (VOID)			{ return (m_nSense); }
	
// Implementation
public:
	~CDefinition (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CString m_phrase;
	CString m_cat;
	CString m_def;
	INT m_nSense;
};

MAKE_CLASS_CAST(CDefinition);


class CDefGroup : public CObject
{
	DECLARE_SERIAL(CDefGroup)
public:
	CDefGroup(CONST CString& head);
protected:
	CDefGroup();			// protected constructor used by dynamic creation
public:
	CDefinition* LookupPhrase (CONST CString& phrase, CONST CString& cat, INT nSense = -1);
	VOID AddPhrase (CONST CString& phrase, CONST CString& cat, CONST CString& def, INT nSense = 1);
	CONST CString& GetHead (VOID)			{ return (m_head); }
	INT GetNumSenses (CONST CString& phrase, CONST CString& cat);
	BOOL PhraseHasCat (CONST CString& phrase, CONST CString& cat);
// Implementation
public:
	~CDefGroup (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CObArray m_entries;
	CString m_head;
};

MAKE_CLASS_CAST(CDefGroup);

/////////////////////////////////////////////////////////////////////////////
// CLexicon command target

class CLexicon : public CCmdTarget
{   
	DECLARE_SERIAL(CLexicon)
public:
	CLexicon();
	CLexicon(CONST CString& sFileName);

// Attributes
public:

// Operations
public:
	CDefinition* LookupPhrase (CONST CString& phrase, CONST CString& cat, INT nSense = -1);
	BOOL PhraseHasCat (CONST CString& word, CONST CString& cat);
	VOID AddPhrase (CONST CString& phrase, CONST CString& cat, CONST CString& def, INT nSense = 1);
	BOOL BuildFromFile (CONST CString& filename);
	BOOL BuildGroupMap (VOID);

// Future Operations
public:
	INT GetMaxPhraseSize (CONST CString& head)		{ return (1); }

private:
	CDefGroup* CreateGroup (CONST CString& head);
	CDefGroup* FindGroup (CONST CString& head);

// Implementation
public:
	virtual ~CLexicon();
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif

protected:
	// Generated message map functions
	//{{AFX_MSG(CLexicon)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
//// redundant definition group collections
//// make m_groups temporary
	CObArray m_groups;
	CMapStringToOb m_groupMap;
};

MAKE_CLASS_CAST(CLexicon);

/////////////////////////////////////////////////////////////////////////////
