// parseman.cpp : implementation file
//

#include "stdafx.h"
#include "common.h"
#include "pcommon.h"
#include "parser.h"
#include "lexicon.h"
#include "concept.h"
#include "casefram.h"
#include "transiti.h"
#include "graph.h"
#include "parseman.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

inline VOID DumpObject (CObject* pObject, INT nLevel = 3, BOOL bInDepth = TRUE)
{
#if defined(_DEBUG)
	ASSERT(pObject);
	if (nTraceLevel >= 3) {
		afxDump.SetDepth(bInDepth);
		pObject->Dump(afxDump);
		}
#endif
}

/////////////////////////////////////////////////////////////////////////////
// CParseManager

IMPLEMENT_SERIAL(CParseManager, CCmdTarget, 1)

CParseManager::CParseManager()
{
	m_pCF = NULL;
	m_pgraph = NULL;
	m_plex = NULL;
//	m_ptrans = NULL;
//	m_plabelNext = NULL;
}

CParseManager::CParseManager (CONST CString& lexicon, CONST CString& graph)
{
	m_pCF = NULL;
	m_pgraph = new CGraph(graph + ".INI");
	m_plex = new CLexicon(lexicon + ".INI");
}

CParseManager::~CParseManager()
{
	if (m_pCF) {
		delete m_pCF;
		}
	if (m_pgraph) {
		delete m_pgraph;
		}
	if (m_plex) {
		delete m_plex;
		}
}

inline CString Quote (CONST CString sText, CHAR cQuote = cQUOTE)
{	
	CString sQuotedText (cQuote);
	return (sQuotedText + sText + cQuote);
}

BOOL CParseManager::ParseIt (CONST CString& node, CONST CString* pText = NULL, BOOL bFirstTime /* = FALSE */)
{
	BOOL bOK = FALSE;

	//	Initialize for the parsing
	TRACEX(4, ("CParseManager::ParseIt(%s, %s, %d)\n", (PCSZ)node, 
					(PCSZ)(pText ? Quote(*pText) : "NULL"), bFirstTime));
	if (bFirstTime) {
		ASSERT(pText);
		if (m_pCF) {
			delete m_pCF;
			}
		m_pCF = new CCaseFrame (pText ? *pText : sNull);
//// betterize this
		m_pCF->SetLexicon(m_plex);
		}
	else {
		m_pCF->IncDepth();
		}
	BOOL bOriginal = m_pCF->IsOriginal();
	ASSERT_VALID(m_pCF);
	ASSERT_VALID(m_pgraph);
	TRACEX(4, ("Parsing position %d of [%s]\n", m_pCF->GetPos(), (PCSZ)JoinTokens(m_pCF->GetTokenArray())));

	CGraphNode* pnode = m_pgraph->GetNodePtr(node);
	if (!pnode) {
		TRACE("Invalid node '%s'\n", (PCSTR)node);
		return (FALSE);
		}

	//	Parse the tokens
#if FALSE
	if (m_pCF->GetNumTokensLeft() == 0) {
		if (*plabel == "STOP") {
			bOK = TRUE;
			}
		}
#endif
//	else {
//		CONST CString& token = m_pCF->GetCurrentToken();
#if FALSE
//// this is a tautology due to the definition of IsTerminal()
		ASSERT((pnode->GetNumTransitions() > 0) || (pnode->IsTerminal()));

		// End of graph but more tokens		
		if (pnode->IsTerminal()) {
			return (FALSE);
//			break;
			}
#endif
		
		//	Try each possible transition from the current ATN node
		for (INT i = 0; i < pnode->GetNumTransitions(); i++) {
			CTransition* ptrans = pnode->GetTransition(i);
			if (!ptrans) {
//// change String to ToString?; also, add Cstringx::FormatString
				CString sError ("Internal error: unable to resolve transition ");
				sError += String(i) + cSPACE + "of " + pnode->GetLabel();
				AfxMessageBox(sError);
				}
			TRACEX(3, ("Trying transition %s (%d/%d of %s)\n", 
						(PCSZ)ptrans->GetType(), i + 1, pnode->GetNumTransitions(), (PCSZ)pnode->GetLabel()));
			if (ptrans->CanProceed(m_pCF)) {

				//	Save the current state to allow for backtracking
//// optimize the backing up of the frame
				CCaseFrame backupFrame (*m_pCF);

				//	Process the transition and proceed with the parse
				ptrans->UpdateState(m_pCF);
				DumpObject(m_pCF);
#if TRUE
				CString* plabel = pnode->GetToNode(i);
				ASSERT(plabel);
				if (plabel) {
					if (*plabel == "STOP") {
						bOK = TRUE;
						}
					else {
						bOK = ParseIt(*plabel);
						}
					}
#else
				CString& label = pnode->GetToNode(i);
				Parse(label);
#endif

				//	If the parse succeeds, stop searching
				if (bOK && (m_pCF->GetNumTokensLeft() > 0)) {
					bOK = FALSE;
					}
				if (bOK) {
					break;
					}
					
				//	Otherwise, restore the saved state
				*m_pCF = backupFrame;
				}
			}
//		}

	m_pCF->SetOriginal(bOriginal);

	return (bOK);
}

VOID CParseManager::Serialize (CArchive& ar)
{
	CObject::Serialize(ar);
	if (ar.IsStoring()) {
//		ar << m_pCF << m_plex << m_pgraph << m_ptrans << m_plabelNext;
		ar << m_pCF << m_plex << m_pgraph;
		}                                
	else {
//		ar >> m_pCF >> m_plex >> m_pgraph >> m_ptrans >> m_plabelNext;
		ar >> m_pCF >> m_plex >> m_pgraph;
		}
}

#if defined(_DEBUG)
VOID CParseManager::AssertValid() const
{
	CObject::AssertValid();
	if (m_pCF) {
		ASSERT_VALID(m_pCF);
		}
	if (m_plex) {
		ASSERT_VALID(m_plex);
		}
	if (m_pgraph) {
		ASSERT_VALID(m_pgraph);
		}
//	ASSERT_VALID(m_ptrans);
//	ASSERT_VALID(m_pnextLabel);
}

VOID CParseManager::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
	dc << "\n";
	dc << "m_pCF = " << m_pCF << "\n";
	dc << "m_plex =" << m_plex << "\n";
	dc << "m_pgraph =" << m_pgraph << "\n";
//	dc << "m_ptrans =" << m_ptrans << "\n";
//	dc << "m_pnextLabel =" << m_plabelNext << "\n";
}
#endif //_DEBUG

BEGIN_MESSAGE_MAP(CParseManager, CCmdTarget)
	//{{AFX_MSG_MAP(CParseManager)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CParseManager message handlers

