// parsevw.cpp : implementation of the CParserView class
//

#include "stdafx.h"

#include "common.h"
#include "pcommon.h"
#include "parser.h"
#include "lexicon.h"
#include "concept.h"
#include "casefram.h"
#include "transiti.h"
#include "graph.h"
#include "parseman.h"

#include "parsedoc.h"
#include "parsevw.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CParserView

IMPLEMENT_DYNCREATE(CParserView, CFormView)

#define new DEBUG_NEW

BEGIN_MESSAGE_MAP(CParserView, CFormView)
	//{{AFX_MSG_MAP(CParserView)
	ON_BN_CLICKED(IDC_PARSE_IT, OnParseIt)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CParserView construction/destruction
CONST CHAR szPARSER_SECTION[] = 	"ParserSpec";
CONST CHAR szGRAPH[] =				"Graph";
CONST CHAR szLEXICON[] =			"Lexicon";

CParserView::CParserView()
	: CFormView(CParserView::IDD)
{
	//{{AFX_DATA_INIT(CParserView)
	CWinApp* pApp = AfxGetApp();
	m_sGraphName = pApp->GetProfileString(szPARSER_SECTION, szGRAPH, "SIMPLENP");
	m_sLexiconName = pApp->GetProfileString(szPARSER_SECTION, szLEXICON, "LEXICON");
	m_sPhrase = "";
	m_sResults = "";
	//}}AFX_DATA_INIT
	// TODO: add construction code here
	m_pParseMan = NULL;
	m_sPrevLexicon = "";
	m_sPrevGraph = "";
}

CParserView::~CParserView()
{
	CWinApp*pApp = AfxGetApp();
	pApp->WriteProfileString(szPARSER_SECTION, szGRAPH, m_sGraphName);
	pApp->WriteProfileString(szPARSER_SECTION, szLEXICON, m_sLexiconName);
	if (m_pParseMan) {
		delete m_pParseMan;
		}
}

void CParserView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CParserView)
	DDX_Control(pDX, IDC_PARSE_IT, m_btnParseIt);
	DDX_Text(pDX, IDC_GRAPH_NAME, m_sGraphName);
	DDV_MaxChars(pDX, m_sGraphName, 8);
	DDX_Text(pDX, IDC_LEXICON, m_sLexiconName);
	DDV_MaxChars(pDX, m_sLexiconName, 8);
	DDX_Text(pDX, IDC_PHRASE, m_sPhrase);
	DDX_Text(pDX, IDC_RESULTS, m_sResults);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// CParserView diagnostics

#ifdef _DEBUG
void CParserView::AssertValid() const
{
	CFormView::AssertValid();
}

void CParserView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

#if FALSE
CParserDoc* CParserView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CParserDoc)));
	return (CParserDoc*)m_pDocument;
}
#endif
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CParserView message handlers

void CParserView::OnParseIt()
{
	UpdateData();
	if (!m_pParseMan || (m_sGraphName != m_sPrevGraph) || (m_sLexiconName != m_sPrevLexicon)) {
		if (m_pParseMan) {
			delete m_pParseMan;
			}
		m_pParseMan = new CParseManager(m_sLexiconName, m_sGraphName);
		m_sPrevGraph = m_sGraphName;
		m_sPrevLexicon = m_sLexiconName;
#if defined(_DEBUG)
		afxDump.SetDepth(1);
		m_pParseMan->Dump(afxDump);
#endif
		}
	BOOL bOK = m_pParseMan->Parse(m_sPhrase);
	m_sResults = bOK ? "OK" : "NOT!";
#if defined(_DEBUG)
	afxDump.SetDepth(0);
	m_pParseMan->Dump(afxDump);
#endif
	UpdateData(FALSE);
}
