// transiti.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CTransition

class CTransition : public CObject
{
	DECLARE_SERIAL(CTransition)
public:
	CTransition(VOID);
	CTransition(CONST CString& type);

// Attributes
public:
	CONST CString& GetType() const		{ return (m_type); }
	
// Operations
public:
	virtual BOOL CanProceed (CCaseFrame* pCF)	{ return (TRUE); /* default to FALSE? */ }
	virtual VOID UpdateState (CCaseFrame* pCF)	{ /* nothing to do at this level */ }
	VOID Deregister (CONST CString& name);

// Implementation
public:
	CString m_type;
	virtual ~CTransition();
	STATIC CMapStringToOb m_transMap;
	STATIC VOID Register (CONST CString& name, CTransition* ptrans);
	STATIC CTransition* FindTransition (CONST CString& name);
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CTransition)


/////////////////////////////////////////////////////////////////////////////

class CNilTransition : public CTransition
{
	DECLARE_SERIAL(CNilTransition)
public:
	CNilTransition(VOID);

// Operations
public:
	virtual BOOL CanProceed (CCaseFrame* pCF)		
		{   TRACEX(TL_VERBOSE, ("%d <= CNilTransition::CanProceed(%p)\n", TRUE, pCF));	
			return (TRUE); }
	virtual VOID UpdateState (CCaseFrame* pCF)		{ /* nothing to update */ }

// Implementation
private:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CNilTransition)


class CCatTransition : public CTransition
{
	DECLARE_SERIAL(CCatTransition)
public:
	CCatTransition(VOID);
	CCatTransition (CONST CString& type, CONST CString& cat);

// Operations
public:
	virtual BOOL CanProceed (CCaseFrame* pCF);
	virtual VOID UpdateState (CCaseFrame* pCF);
	inline VOID SetCategory (CONST CString& cat)	{ m_cat = cat; }

// Implementation
public:
	CString m_cat;
public:
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CCatTransition)


class CAdjTransition : public CCatTransition
{
	DECLARE_SERIAL(CAdjTransition)
public:
	CAdjTransition(VOID);

// Operations
public:
#if FALSE
	virtual BOOL CanProceed (CCaseFrame* pCF);
	virtual VOID UpdateState (CCaseFrame* pCF);
#endif

// Implementation
public:
//	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CAdjTransition)


class CAdvTransition : public CCatTransition
{
	DECLARE_SERIAL(CAdvTransition)
public:
	CAdvTransition(VOID);

// Operations
public:
#if FALSE
	virtual BOOL CanProceed (CCaseFrame* pCF);
	virtual VOID UpdateState (CCaseFrame* pCF);
#endif

// Implementation
public:
//	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CAdvTransition)


class CDetTransition : public CCatTransition
{
	DECLARE_SERIAL(CDetTransition)
public:
	CDetTransition(VOID);

// Operations
public:
#if FALSE
	virtual BOOL CanProceed (CCaseFrame* pCF);
	virtual VOID UpdateState (CCaseFrame* pCF);
#endif

// Implementation
public:
//	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CDetTransition)


class CNounTransition : public CCatTransition
{
	DECLARE_SERIAL(CNounTransition)
public:
	CNounTransition(VOID);

// Operations
public:
#if FALSE
	virtual BOOL CanProceed (CCaseFrame* pCF);
	virtual VOID UpdateState (CCaseFrame* pCF);
#endif

// Implementation
public:
//	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CNounTransition)


class CVerbTransition : public CCatTransition
{
	DECLARE_SERIAL(CVerbTransition)
public:
	CVerbTransition(VOID);

// Operations
public:
#if FALSE
	virtual BOOL CanProceed (CCaseFrame* pCF);
	virtual VOID UpdateState (CCaseFrame* pCF);
#endif

// Implementation
public:
//	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
};

MAKE_CLASS_CAST(CVerbTransition)
