#include "common.h"

//	Support for debugging
#if defined(_DEBUG)
PFILE hTrace = NULL;                       // file for diagnostics
#endif


PFILE OpenFile (PSTR pszFileName, PSTR pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure)
{
	CloseFile(hFile, hDefaultFile);
    hFile = fopen(pszFileName, pszFileMode);
    if (!hFile && bExitOnFailure) {
        fprintf(stderr, "Error opening the file '%s'\n", pszFileName);
        exit(1);
        }

    return (hFile);
}

PFILE OpenDirFile (PSTR pszDir, PSTR pszFileName, PSTR pszFileMode, PFILE hFile, PFILE hDefaultFile, BOOL bExitOnFailure)
{
	CHAR szFile[PATH_MAX];
	sprintf(szFile, "%s\\%s", pszDir, pszFileName);
	
	return (OpenFile(szFile, pszFileMode, hFile, hDefaultFile, bExitOnFailure));
}

PFILE CloseFile (PFILE hFile,  PFILE hDefaultFile)
{
    if (hFile != hDefaultFile) {
        fclose(hFile);
        hFile = hDefaultFile;
        }

    return (hFile);
}

BOOL RemoveDirFile (PSTR pszDir, PSTR pszFileName)
{
	CHAR szFile[PATH_MAX];
	sprintf(szFile, "%s\\%s", pszDir, pszFileName);
	INT nResult = remove(szFile);

	return (nResult == 0);
}

INT ReadData (PVOID pData, INT nSize, INT nToRead, PFILE hFile)
{
	INT nRead = fread(pData, nSize, nToRead, hFile);
	Assert((nRead == nToRead) || feof(hFile));
//	BOOL bOK = ((nRead == nToRead) || feof(hFile));

	return (nRead);
}


INT GetString (PFILE hFile, PSTR pszBuffer, INT nMaxLen)
{
	INT nLen = 0;

	if (fgets(pszBuffer, nMaxLen, hFile)) {
		nLen = strlen(pszBuffer);
		if ((nLen > 0) && (pszBuffer[nLen - 1] == '\n')) {
			pszBuffer[nLen - 1] = NULL_CHAR;
			nLen--;
			}
		}

	return (nLen);
}


BOOL WriteText (PFILE hFile, PSTR pszFormat, ...)
{
	va_list vaArgs;
	va_start (vaArgs, pszFormat);
	vfprintf(hFile, pszFormat, vaArgs);
	va_end (vaArgs);
	
	return (TRUE);
}


LONG GetFilePos (PFILE hFile)
{
	LONG lPos = ftell(hFile);
	
	return (lPos);
}


BOOL SetFilePos (PFILE hFile, LONG lOffset, INT nFromWhere)
{
	fseek(hFile, lOffset, nFromWhere);
	
	return (TRUE);
}


LONG GetFileSize (PFILE hFile)
{
	LONG lPos = GetFilePos(hFile);
	SetFilePos(hFile, (LONG) 0, SEEK_END);
	LONG lNumChars = GetFilePos(hFile);
	SetFilePos(hFile, lPos, SEEK_SET);
	
	return (lNumChars);
}


VOID _ReportAssertion (PSTR pszExpr, PSTR pszFile, INT nLine)
{
	CHAR szMsg[256];
	sprintf(szMsg, "\nAssertion failed: %s (%s: %d)\n", pszExpr, pszFile, nLine);
	TraceOut("%s", szMsg);
	fprintf(stderr, "%s", szMsg);
	fprintf(stderr, "%s", "Press c to continue, anything else to quit\n");
	INT c = getchar();
	if (toupper(c) != 'C') {
		exit(1);
		}
}


VOID _ReportError (PSTR pszExpr, PSTR pszFile, INT nLine)
{
	CHAR szMsg[256];
	sprintf(szMsg, "\nUnrecoverable internal error; contact TPO (%s; %d)\n", 
			pszExpr, pszFile, nLine);
	TraceOut("%s", szMsg);
	fprintf(stderr, "%s", szMsg);
#ifdef QUICK_WIN
	_wsetexit(_WINEXITPERSIST);
#endif
	exit(1);
}


CONST INT nMAX_PAGES = 8192;
CONST INT nPAGE_SIZE = 4096;

BOOL DisplayAvailMem (VOID)
{
	WORD wListSize = (WORD)nMAX_PAGES * sizeof(PSTR);
	PPSTR apsz = (PPSTR)malloc(wListSize);
	if (!apsz) {
		TraceOut("Unable to allocate 64K buffer for DisplayAvailMem\n");
		return (FALSE);
		}
		
	INT i;
	INT nNumPages = 0;

	for (i = 0; i < nMAX_PAGES; i++) {
		apsz[i] = (PSTR)malloc(nPAGE_SIZE);
		if (!apsz[i]) {
			nNumPages = i;
			break;
			}
		}

	for (i = 0; i < nNumPages; i++) {
		free(apsz[i]);
		}
	free(apsz);

	LONG lFree = ((LONG)nNumPages * nPAGE_SIZE) + sizeof(apsz);
	TraceOut("Roughly, %l bytes available\n", lFree);
	
	return (TRUE);
}


INT StringToCode (PCODEC pCodeTable, INT nSize, PSTR pszString)
{
	INT nCode = pCodeTable[0].nCode;
	for (INT i = 1; i < nSize; i++) {
		if (!stricmp(pCodeTable[i].pszString, pszString)) {
			nCode = pCodeTable[i].nCode;
			break;
			}
		}
	
	return (nCode);
}


PSTR CodeToString (PCODEC pCodeTable, INT nSize, INT nCode)
{
	PSTR pszString = pCodeTable[0].pszString;
	for (INT i = 1; i < nSize; i++) {
		if (pCodeTable[i].nCode == nCode) {
			pszString = pCodeTable[i].pszString;
			break;
			}
		}
	
	return (pszString);
}
