//	Copyleft 1993-1996 Tom O'Hara		TrlLookup source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

//	IdxInfo
//
//	Module for interpreting an index from TRL's IDX files. This is useful
//	mainly for the almost-linear subindexes encountered in the dictionary and
//	the thesaurus IDX files.
//

#include "commonxx.h"
#include "TrlIndexInfo.h"


//	Determine whether the block from the IDX file is a (low-level)
//	index block. The index directory blocks are ignored.
//
BOOL IsIndexBlock (PIDXFILE_INFO pxi, PBLOCK_HDR pBlockHdr)
{
	BOOL bOK = (pBlockHdr->lCode1 == 0) 
				&& ((pBlockHdr->lNextBlock - pxi->lBlockNum) < 5);

#if FALSE				
	//	MISCARRIAGE: Restrict index blocks to those compatible with ones
	//				 already processed
	bOK = bOK && CloseCount(pBlockHdr->lCount1, lPrevCount1)
			  && CloseCount(pBlockHdr->lCount2, lPrevCount2);
	if (bOK) {
		pxi->lPrevBlock = pBlockHdr->lPrevBlock;
		pxi->lNextBlock = pBlockHdr->lNextBlock;
		}
#endif

	return (bOK);
}


//	Determine the entry descriptor. This indicates its size, number of
//	index referemces, the phrase start and length, and the pointer to 
//	the next entry among a few other things.
//
BOOL InterpretEntry (PBYTE pStart, PENTRY_INFO pEntry, PIDXFILE_INFO pxi)
{
	BOOL bOK = TRUE;
#if FALSE
	ZeroMemory(pEntry, sizeof(ENTRY_INFO));
#endif
	pEntry->pHdr = (PENTRY_HDR)pStart;
	pEntry->wSize = (WORD)pEntry->pHdr->bySize;
	pEntry->pNext = pStart + pEntry->wSize + 1;

	// check for filler at end
	if (pEntry->wSize == 0) {
		return (FALSE);
		}
//	Assert(wSize >= nMIN_ENTRY);
	Assert(pEntry->pHdr->byNull == 0);

	switch (pEntry->pHdr->byRefType) {
		case SINGLE_REF: 
		case QUOT_REF: 
			pEntry->pFirstRef = (PBYTE)(pStart + sizeof(ENTRY_HDR));
			pEntry->pszPhrase = (PCSZ)(pEntry->pFirstRef + sizeof(LONG));
			pEntry->nPhraseLen = StringLength(pEntry->pszPhrase);
			break;
		case MULTIPLE_REFS:
			pEntry->pszPhrase = (PCSZ)(pStart + sizeof(ENTRY_HDR));
			pEntry->nPhraseLen = StringLength(pEntry->pszPhrase);
			pEntry->pFirstRef = (PBYTE)(pEntry->pszPhrase + pEntry->nPhraseLen + 1);
			break;
		case UNKNOWN_REFS:
			if (pEntry->wSize >= nMIN_ENTRY) {
				pEntry->pszPhrase = (PCSZ)(pStart + sizeof(ENTRY_HDR));
				pEntry->nPhraseLen = StringLength(pEntry->pszPhrase);
				pEntry->pFirstRef = (PBYTE)(pEntry->pszPhrase + pEntry->nPhraseLen + 1);
				}
			else {
				pEntry->pszPhrase = szUNKOWN_REFS;
//// improve on this (note usage later on)
				pEntry->nPhraseLen = -1;
////
				pEntry->pFirstRef = (PBYTE)(pStart + sizeof(ENTRY_HDR));
				}
			break;
		default:
			AssertError();
			bOK = FALSE;
			break;
		}

	Assert(pEntry->nPhraseLen < (INT)pEntry->wSize);
	pEntry->nBaseSize = (sizeof(ENTRY_HDR) - 1) + (pEntry->nPhraseLen + 1);
	
	return (bOK);
}


//	Determine the number of references in the index entry and the start
//	of the references. Since some reference entries straddle index blocks,
//	special care is take not to trip over the residual bytes.
//
BOOL InterpretEntryRefs (PBYTE pStart, PENTRY_INFO pEntry, PIDXFILE_INFO pxi)
{
	//	Process residual bytes. These go into a special bit bucket in the
	//	index info structure.
	pEntry->bContinuation = (pxi->nBytesLeft > 0);
	INT nNumNeeded = 0;
	if (pEntry->bContinuation) {
		nNumNeeded = sizeof(LONG) - pxi->nBytesLeft;
		memcpy(pxi->abyResidual + pxi->nBytesLeft, pEntry->pFirstRef, nNumNeeded);
		pxi->nBytesLeft = 0;
		pEntry->pFirstRef += nNumNeeded;
		}

#if TRUE
	//	Advance the field pointer for the first reference until the offset therein
	//	is legitimate.
	for (INT i = 1; 
		 	((i < sizeof(LONG)) 
		 	 	&& (*(PLONG)pEntry->pFirstRef > pxi->lMaxDataOffset) 
		 	 	&& (pEntry->pFirstRef < pEntry->pNext));
		 	i++) {
		pEntry->pFirstRef++;
		nNumNeeded++;
		}
#endif

	//	Determine if the reference type is for a single reference or multiples ones.
	//	If for multiple, calculate the number of references.
	if ((pEntry->pHdr->byRefType == SINGLE_REF)
		|| (pEntry->pHdr->byRefType == QUOT_REF)) {
		pEntry->nNumRefs = 1;
		}
	else {
		pEntry->nNumRefs = (pEntry->pNext - pEntry->pFirstRef) / sizeof(LONG);
		}

	//	Determine the start of the references
	pEntry->plRefs = (PLONG)pEntry->pFirstRef;
	Assert(((pEntry->nNumRefs + (INT)pEntry->bContinuation) > 0) 
		   && ((WORD)pEntry->nNumRefs < (pEntry->wSize / sizeof(LONG))));

	//	Determine the number of bytes left in a incomplete reference entry, since some
	//	straddle index block boundaries. This can only be from 0 to 3 bytes.
	pxi->nBytesLeft = (pEntry->wSize - nNumNeeded) - (pEntry->nBaseSize + pEntry->nNumRefs * sizeof(LONG));
	nNumNeeded = sizeof(LONG) - pxi->nBytesLeft;
	Assert((pxi->nBytesLeft >= 0) && (pxi->nBytesLeft < sizeof(LONG)));

	//	Do sanity check for a single entry index (also for the last entry in 
	//	composite index).
	BOOL bSingleEntry = (pEntry->pHdr->byEntryPos == POS_MASK);
	if (bSingleEntry) {
		Assert(pxi->nBytesLeft == 0);
		}

	return (TRUE);
}
