//	Copyleft 1993-1996 Tom O'Hara		TrlLookup source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

//	TrlLookup
//
//	Module providing API for looking up entries in TRL's dictionary and thesaurus.
//	It could be used for the others, but some of them actually use a tree-like
//	index rather than the almost-linear one that this simple-mindedly interprets.
//

#include "commonxx.h"
#include "TrlLookup.h"

//  Constants
//
CONST BYTE START_BYTE = 0x02;
CONST BYTE END_BYTE = 0x06;
CONST BYTE START_BOLD = 0x01;
CONST BYTE END_BOLD = 0x06;
CONST BYTE WIDE_DASH = 0xC4;


CONST CHAR szAND[] = " AND ";
CONST INT nAND_LEN = STRING_SIZE(szAND);

#if defined(_WIN32)
CONST INT nMAX_REFS = 32767;
#else
CONST INT nMAX_REFS = 2048;
#endif
CONST nTEXT_BUFFER_MAX = 1024;

CONST INT nLEFT_BRACKET =	'[';
CONST INT nRIGHT_BRACKET =	']';

//CONST CHAR szSGML_START_X[] =		"\n<x>\n";
//CONST CHAR szSGML_END_X[] =		"\n</x>\n";


#if !defined(F)
#define F(arg)	PtrText(arg)
#endif


//  Global variabes
//
HANDLE hLibInst = NULL;

CODEC LookupCodes[] = {
    { eNONE,        "N/A" },
    { eCROSS_REFS,  "cross references" },
    { eDEFINITION,  "definition" },
    { eSYNONYMS,    "synonyms" },
    };


//  Accessor/manipulators for globals
//

inline LOOKUP_TYPE StringToLookupType (PCSZ pszType)
{
    return ((LOOKUP_TYPE)StringToCode(LookupCodes, NUM_ELEMENTS(LookupCodes), pszType));
}

inline PCSZ LookupTypeToString (INT nCode)
{
    return (CodeToString(LookupCodes, NUM_ELEMENTS(LookupCodes), nCode));
}


//  Function prototypes
//
#if defined(_WIN32)

BOOL WINAPI DllMain(
   		HINSTANCE  hinstDLL,			// handle of DLL module 
   		DWORD  fdwReason,				// reason for calling function 
   		LPVOID  lpvReserved 			// reserved 
   		);

#else

EXTERN_C INT CALLBACK LibMain(
        HINSTANCE hInst,                // handle of library instance
        WORD wDataSeg,                  // library data segment
        WORD cbHeapSize,                // default heap size
        PCSZ lpszCmdLine                // command-line arguments
        );
EXTERN_C INT CALLBACK WEP(
        INT nExitType               	// type of exit
        );

#endif

BOOL InitLib (HANDLE hInstance, PCSZ lpszCmdLine);
BOOL ReadBlockIndex (PCSZ pszBnxDir, PSRC_INFO pSrc);
LONG FindIdxOffset (PSRC_INFO pSrc, PCSZ pszPhrase);
INT MergeAnd (PLONG plRefs1, INT nLen1, PLONG plRefs2, INT nLen2);
INT GetPhraseEntries (PSRC_INFO pSrc, PCSZ pszPhrase, PLONG plRefs, INT nMaxRefs);
INT AddRefsToBuffer (PSRC_INFO pSrc, PLONG plRefs, INT nNumRefs, PSZ pszBuffer, 
                     INT nBufferLen, LOOKUP_TYPE eType);
VOID AddHeadToBuffer (PSRC_INFO pSrc, PSZ pszBuffer, INT& nBufferPos, INT nBufferLen, LOOKUP_TYPE eType);
VOID AddBodyToBuffer (PSRC_INFO pSrc, PSZ pszBuffer, INT& nBufferPos, INT nBufferLen, LOOKUP_TYPE eType);


//	Inline helpers
//

inline BOOL IsValidChar (INT cChar)
{
    return (isprint(cChar) || (cChar == WIDE_DASH));
}

inline INT TranslateChar (INT cChar, BOOL bMakeUpper)
{
    if (cChar == WIDE_DASH) {
        cChar = cDASH;
        }
    else if (bMakeUpper) {
        cChar = toupper(cChar);
        }
    return (cChar);
}

inline CHAR TranslateChar (INT cChar)
{
    if (cChar == WIDE_DASH) {
        cChar =  cDASH;
        }
    return (ToChar(cChar));
}


#if defined(_WIN32)


//	DllMain: Win32 DLL entry point
//
BOOL WINAPI DllMain(
   HINSTANCE  hInstDLL,					// handle of DLL module 
   DWORD  fdwReason,					// reason for calling function 
   LPVOID  lpvReserved 					// reserved 
   )
{
	BOOL bOK = TRUE;

	if (fdwReason == DLL_PROCESS_ATTACH) {
	    DEBUG_OUT(TL_ALWAYS, ("%s:DllMain(%x, %lx, %lx): DLL_PROCESS_ATTACH\n", 
                          	  szLIB_NAME, hInstDLL, fdwReason, lpvReserved));
    	DEBUG_OUT(TL_ALWAYS, ("&szLIB_NAME = %lx\n", szLIB_NAME));
    	bOK = InitLib(hInstDLL, szNULL_STR);
		}
	else if (fdwReason == DLL_PROCESS_DETACH) {
	    DEBUG_OUT(TL_ALWAYS, ("%s:DllMain(%x, %lx, %lx): DLL_PROCESS_DETACH\n", 
                          	  szLIB_NAME, hInstDLL, fdwReason, lpvReserved));
		}

	return (bOK);
}


#else

//  LibMain: Lib entry point
//
EXTERN_C INT CALLBACK LibMain(
        HINSTANCE hInst,                // handle of library instance
        WORD wDataSeg,                  // library data segment
        WORD cbHeapSize,                // default heap size
        PCSZ lpszCmdLine                // command-line arguments
        )
{
    DEBUG_OUT(TL_ALWAYS, ("%s:LibMain(%x,%x,%x,%lx => '%s')\n", 
                          szLIB_NAME, hInst, wDataSeg, cbHeapSize, lpszCmdLine, lpszCmdLine));
    DEBUG_OUT(TL_ALWAYS, ("&szLIB_NAME = %lx\n", szLIB_NAME));
    BOOL bOK = InitLib(hInst, lpszCmdLine);

    return (bOK);                   // return 1 for success; 0 for failure
}

CONST CHAR BASEDCS szWEP_FMT[] = "Lookup's WEP(%d)\n";

//  WEP: Lib exit point
//
EXTERN_C INT EXPORT CALLBACK WEP(
        INT nExitType               // type of exit
        )
{
    OUTPUT_DEBUG_TEXT((szWEP_FMT, nExitType));
    
    return (TRUE);
}


#endif


//	Initialize the library (DLL)
//
BOOL InitLib (HANDLE hInstance, PCSZ lpszCmdLine)
{
    hLibInst = hInstance;
    
    InitCommon(hInstance, NULL, szLIB_NAME);
    INIT_DEBUGGING_(szLOOKUP_INI);
    DEBUG_OUT(TL_VERBOSE, ("%s:InitLib(%x,%s)\n", 
                           szLIB_NAME, hInstance, lpszCmdLine));

    return (TRUE);
}


//	Read in the block index
//
BOOL ReadBlockIndex (PCSZ pszBnxDir, PSRC_INFO pSrc)
{
    DEBUG_OUTF(TL_VERBOSE, ("ReadBlockIndex(%s, %lx)\n", 
                            PtrText(pszBnxDir), pSrc));
    INT nLen;
    CHAR szBuffer[512]; 
    
    PFILE hBnxFile = NULL;
    ASSERT(HasTrailingPathDelim(pszBnxDir));
    wsprintf(szBuffer, "%s%s.BNX", pszBnxDir, pSrc->szIndex);
    hBnxFile = OpenTextFile(szBuffer, "r", hBnxFile, NULL, TRUE);
    if (!hBnxFile) {
        return (FALSE);
        }

    //  get the reference source name
    nLen = GetString(hBnxFile, pSrc->szSource, MAX_CHARS(pSrc->szSource));
    Assert(nLen > 0);
    MakeUpper(pSrc->szSource);

    // get the starting offset into the IDX file
    nLen = GetString(hBnxFile, szBuffer, MAX_CHARS(szBuffer));
    Assert(nLen > 0);
    pSrc->lIdxStart = StringToLong(szBuffer);

    // get the list of block-ending phrases
    pSrc->nNumGroups = 0;
    LONG lPrevFilePos = GetFilePos(hBnxFile);
    while (pSrc->nNumGroups < nMAX_GROUPS) {
        nLen = GetString(hBnxFile, szBuffer, MAX_CHARS(szBuffer));
        if (nLen == 0) {
            break;
            }
        LONG lFilePos = GetFilePos(hBnxFile);
        ASSERT(lFilePos > lPrevFilePos);
        lPrevFilePos = lFilePos;
#if TRUE
        //  truncate the key to the maximum size (eg, 5)
        nLen = min(nLen, nKEY_MAX);
        szBuffer[nLen] = NULL_CHAR;
#endif
        StringCopy(pSrc->apszLastPhrase[pSrc->nNumGroups], szBuffer);
        pSrc->nNumGroups++;
        }
    Assert((pSrc->nNumGroups < nMAX_GROUPS) || (nLen == 0));
    CloseTextFile(hBnxFile, NULL);

    return (TRUE);
}


//	Find the starting offset for the group containing an entry in the index file.
//
LONG FindIdxOffset (PSRC_INFO pSrc, PCSZ pszPhrase)
{
    DEBUG_OUTF(TL_VERBOSE, ("FindIdxOffset(%lx, %s)\n", pSrc, PtrText(pszPhrase)));

    INT nStart = 0;
    INT nEnd = pSrc->nNumGroups - 1;
    INT nMid = -1;
    while (nStart <= nEnd) {
        nMid = (nStart + nEnd)/2;
        PCSZ psz=pSrc->apszLastPhrase[nMid];
        INT nCompareResult = PhraseCompare(pszPhrase, pSrc->apszLastPhrase[nMid], FALSE);
        if (nCompareResult > 0) {
            nStart = nMid + 1;
            }
        else if (nCompareResult < 0) {
            nEnd = nMid - 1;
            }
        else {
            break;
            }
        }

    Assert((nMid >= 0) && (nMid < pSrc->nNumGroups));
    return (pSrc->lIdxStart + (LONG)nMid * wGROUP_SIZE);
}


//	Merge the two sorted lists by performing a boolean AND operation.
//	This is equivalent to an intersection.
//
INT MergeAnd (PLONG plRefs1, INT nLen1, PLONG plRefs2, INT nLen2)
{
    DEBUG_OUT(TL_VERBOSE, ("MergeAnd(%lx, %d, %lx, %d)\n",
                           plRefs1, nLen1, plRefs2, nLen2));
    INT nNum = 0;
    INT i = 0;
    INT j = 0;

    while ((i < nLen1) && (j < nLen2)) {
        if (plRefs1[i] < plRefs2[j]) {
            i++;
            }
        else if (plRefs1[i] == plRefs2[j]) {
            plRefs1[nNum] = plRefs1[i];
            nNum++;
            i++;
            j++;
            }
        else {
            j++;
            }
        }

    return (nNum);
}


//	Ye Olde Message Pump
//
void PumpMessages (VOID)
{
    //  Make note of the current cursor
    HCURSOR hCursor = GetCursor();

    //  Process messages until none remain
    MSG msg;
    while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
        if (msg.message == WM_QUIT) {
            return;
            }

        TranslateMessage(&msg);
        DispatchMessage(&msg);
        }

    //  Restore the cursor
    if (hCursor != NULL) {
        SetCursor(hCursor);
        }
}


//	Return a list of references to the entry from the index file. For dictionary
//	items this usually contains only a single element. But for cross references there
//	could be quite a few elements returned, especially for common words.
//
//	NOTE: The block index lists the last index entry in each group, so a binary
//	is done to find the appropriate group (eg, 32K chunk). Then brute force is used
//	to find the start of the entries in the index. 
//
//	ALSO NOTE: I had trouble deciphering the index's tree structure. Luckily, the
//	items are stored linearly for the most part (the business index is a big
//	exception). Thus the block index is used instead. Therefore, the scanning
//	proceeds a little past the desired phrase to account for "anomalies".
//
INT GetPhraseEntries (PSRC_INFO pSrc, PCSZ pszPhrase, PLONG plRefs, INT nMaxRefs)
{
    DEBUG_OUT(TL_VERBOSE, ("GetPhraseEntries(%lx, '%s', %lx, %d)\n",
                           pSrc, pszPhrase, plRefs, nMaxRefs));

	//	Determine the start of the index group containing the items
    LONG lIdxOffset = FindIdxOffset(pSrc, pszPhrase);
    
    /* allocate a large block for scanning the words */
    PBYTE pBlocks = ALLOC_ARRAY(wGROUP_SIZE, BYTE);

    //  initialize the entry residue vars    
    pSrc->xi.nBytesLeft = 0;
    *(PLONG)pSrc->xi.abyResidual = 0;

    /* try to find the phrase in the reference index */
    BOOL bFound = FALSE;
    BOOL bDone = FALSE;

	//	Goto the start of the index group and prepare for the hunt
    SetFilePos(pSrc->hIdxFile, lIdxOffset, SEEK_SET);
    pSrc->xi.lBlockNum = (lIdxOffset / nBLOCK_SIZE) - 1;
    Assert(pSrc->xi.lBlockNum >= 0);
    INT nBlocksLeft = (INT)((pSrc->lIdxSize - lIdxOffset) / nBLOCK_SIZE);
    INT nPos = 0;

    DEBUG_OUTF(TL_DETAILED, ("\nGetting entries for phrase %s\n", pszPhrase));
    INT nRefs = 0;
    INT nNumPast = 0;
    do {
        INT nToRead = min(nBLOCKS_PER_GROUP, nBlocksLeft);
        INT nRead = ReadData(pBlocks, nBLOCK_SIZE, nToRead, pSrc->hIdxFile);
        nBlocksLeft -= nRead;

        for (INT i = 0; i < nRead; i++) {
            PumpMessages();
            PBYTE pBlock = pBlocks + (i * nBLOCK_SIZE);
            PBLOCK_HDR pBlockHdr = (PBLOCK_HDR)pBlock;
            BOOL bBlockOK = IsIndexBlock(&pSrc->xi, pBlockHdr);
            if (bBlockOK) {
                PBYTE pBlockEnd = pBlock + nBLOCK_SIZE;
				PBYTE pNext = NULL;
                for (PBYTE pStart = pBlock + sizeof(BLOCK_HDR); 
                        pStart < pBlockEnd; 
                        pStart = pNext) {
					//	Fill in a descriptor for the current entry with items such as
					//	phrase start and length, index start, and next entry.
                	ENTRY_INFO Entry;
                    BOOL bEntryOK = InterpretEntry(pStart, &Entry, &pSrc->xi);
                    if (!bEntryOK) {
                        break;
                        }
					pNext = Entry.pNext;

					//	Compare the index entry's phrase with the lookup phrase
					//	Only signal an end when we are several entries past the item.
					//	This is a fudge factor to account for not-so-linear index entries.
                    INT nCompResult = PhraseCompare(Entry.pszPhrase, pszPhrase, FALSE);
                    if (nCompResult < 0) {
                        nNumPast = 0;
                        continue;
                        }
                    else if (nCompResult > 0) {
                        if (nNumPast++ > 10) {      // be conservative
                            bDone = TRUE;   // "done" but still check the rest of group
                            }
                        continue;               
                        }
                    bFound = TRUE;

					//	Determine the number of references and the starting offset
                    InterpretEntryRefs(pStart, &Entry, &pSrc->xi);
                    Assert(nMaxRefs > Entry.nNumRefs);

					//	Copy the reference pointers to the reference list
					//	TODO: handle residual references (ones that straddled blocks)
                    INT nToCopy = min(nMaxRefs, Entry.nNumRefs);
                    nMaxRefs -= nToCopy;
                    WORD wSize = ToWord(nToCopy * sizeof(LONG));
                    memcpy(plRefs + nRefs, Entry.plRefs, wSize);
                    nRefs += nToCopy;
                    }
                }

            lIdxOffset += nBLOCK_SIZE;
            pSrc->xi.lBlockNum++;
            }

        } while ((nBlocksLeft > 0) && !bDone);

    //	Release the large block
    FREE(pBlocks);

    return (nRefs);
}


//	Get the next character from the input file.
//
INT MyGetChar (PFILE hFile)
{
    DEBUG_OUT(TL_GARRULOUS, ("MyGetChar(%lx)\n", hFile));
	BYTE byBuffer[80];
    INT nRead = ReadData (byBuffer, 1, 1, hFile);
    BOOL bOK = (nRead == 1);
    return (bOK ? byBuffer[0] : EOF);
}


//	Add the dictionary entries for each item in the reference list to the text buffer.
//	If the lookup type is cross references, only the entry name is added.
//
//	The TRL file format is basically plain text with the exception of a few formatting 
//	codes added. To resolve a reference, the file offset is first set to the pointer value.
//	The initial character should be the "start entry name" code. The phrase name is then extracted
//	by accumulating all characters until the "end entry name" code. Then the phrase definition
//	contents is extracted by accumiating all characters until the next "start entry name" code.
//
//	The resulting buffer is formatted in a pseudo-SGML manner, with tags on separate lines
//	to faciliate parsing. For instance:
//
//		<head>
//		lamp n.
//		</head>
//		1. a container with a wick for burning oil, alcohol, etc to produce light or heat;
//		...
//		5.
//		<annotation>
//		Old Poet.
//		</annotation>
//		the sun, moon, a start, etc.
//		...
//
//	TODO: translate SGML special characters into character entities (eg, &lt.)
//
INT AddRefsToBuffer (PSRC_INFO pSrc, PLONG plRefs, INT nNumRefs, PSZ pszBuffer, 
                     INT nBufferLen, LOOKUP_TYPE eType)
{
    DEBUG_OUT(TL_VERBOSE, ("AddRefsToBuffer(%lx, %lx, %d, %lx, %d, %d)\n",
                           pSrc, plRefs, nNumRefs, pszBuffer, nBufferLen, eType));
    BOOL bJustKeys = (eType == eCROSS_REFS);

	//	Add each of the reference phrases to the buffer, optionally with the
	//	contents of the dictionary entry.
    INT nBufferPos = 0;
    for (INT i = 0; i < nNumRefs; i++) {
        PumpMessages();
        Assert((plRefs[i] >= 0) && (plRefs[i] <= pSrc->lDatSize));

		//	Goto the start of the entry in the data file
		//	The "start entry name" byte should be encountered.
        SetFilePos(pSrc->hDatFile, plRefs[i], SEEK_SET);
        INT nFirstChar = MyGetChar(pSrc->hDatFile);
        if (nFirstChar != START_BYTE) {
            DEBUG_OUTF(TL_BASIC, ("Reference %lx is suspect\n", plRefs[i]));
            continue;
            }

		//	Format the headword and add it to the buffer
	    AddHeadToBuffer(pSrc, pszBuffer, nBufferPos, nBufferLen, eType);

		//	Determine whether to keep scanning the entry or to go to the next one
        BOOL bDoRest = !bJustKeys;
#if _DEBUG_MODE
        bDoRest = bDoRest || (GetTraceLevel() >= TL_GARRULOUS);
	    DEBUG_OUTF(TL_GARRULOUS, ("\n"));
#endif

		//	Format the rest and add it to the buffer
        if (bDoRest) {
		    AddBodyToBuffer(pSrc, pszBuffer, nBufferPos, nBufferLen, eType);
			}
        }
    DEBUG_OUTF(TL_VERBOSE, ("\n"));

    return (nBufferPos);
}


//	Add the headword to the text buffer, formatted in proper pseudo-SGML fashion.
//	The buffer position is advanced past the new text.
//
VOID AddHeadToBuffer (PSRC_INFO pSrc, PSZ pszBuffer, INT& nBufferPos, INT nBufferLen, LOOKUP_TYPE eType)
{
    DEBUG_OUT(TL_GARRULOUS, ("AddHeadToBuffer(%lx, %lx, %d, %d, %d)\n",
                             pSrc, pszBuffer, nBufferPos, nBufferLen, eType));

	//	Scan for the end of the phrase name by looking for the "end entry name"
	//	byte. Translate the characters as appropriate (eg, make uppercase).
    CHAR szText[nTEXT_BUFFER_MAX];
    INT nTextLen = 0;
	while (nTextLen < STR_LEN(szText)) {      // rework this using block I/O
        INT nNextChar = MyGetChar(pSrc->hDatFile);
        if (nNextChar == END_BYTE) {
            break;
            }
        if (IsValidChar(nNextChar)) {
			szText[nTextLen++] = TranslateChar(nNextChar);
            }
        }

	//	Add the phrase name to the return buffer, except for the case
	//	of synonym lookup.
	//	TODO: exclude this for cross-references as well as synonyms?
	INT nNumChars = 0;
    Assert(nTextLen < STR_LEN(szText));
    szText[nTextLen] = NULL_CHAR;
    DEBUG_OUTF(TL_VERBOSE, ("%s ", szText));
    if (eType != eSYNONYMS) {
		CONST INT nEXTRA = STR_LEN(szSGML_START_HEAD) + STR_LEN(szSGML_END_HEAD)
						   + STR_LEN(szSGML_START_PARA);
        if (nBufferLen < (nBufferPos + nTextLen + nEXTRA)) {
            DEBUG_OUTF(TL_BASIC, ("Reference buffer insufficient!\n"));
            return;
            }
		// Quick Fix: Eliminate duplicate cross reference entries
		// so xref on 'shit' doesn't give "shat; shit; shit; shit; ...; shit;"
		INT nNewSize = sprintf(pszBuffer + nBufferPos, "%s%s%s", 
           	                   szSGML_START_HEAD, szText, szSGML_END_HEAD);
		if ((eType == eCROSS_REFS) 
			&& (FindStringIgnoreCase(pszBuffer, pszBuffer + nBufferPos) < nBufferPos)) {
			pszBuffer[nBufferPos] = cNULL;
			}
		else {
       		nBufferPos += nNewSize;
       		nBufferPos += sprintf(pszBuffer + nBufferPos, "%s", szSGML_START_PARA);
			}
        }

	return;
}


//	Helper inline for AddBodyToBuffer.
//	The buffer position is advanced past the new text.
//
INLINE VOID AddTextToBuffer (PSZ pszText, INT nTextLen, PSZ pszBuffer, 
							INT& nBufferPos, INT nBufferLen, LOOKUP_TYPE eType)
{
    DEBUG_OUT(TL_GARRULOUS, ("AddTextToBuffer(%x, %d, %d, %x, %d, %d): %s\n",
                             pszText, nTextLen, pszBuffer, nBufferPos, nBufferLen, pszText, eType));

    pszText[nTextLen] = NULL_CHAR;
    if (eType != eCROSS_REFS) {
		if (nBufferLen < (nBufferPos + nTextLen + STR_LEN(szSGML_START_PARA))) {
        	DEBUG_OUTF(TL_BASIC, ("Reference buffer insufficient!\n"));
			return;
			}
		nBufferPos += sprintf(pszBuffer + nBufferPos, "%s", pszText);
		if (eType != eSYNONYMS) {
			nBufferPos += sprintf(pszBuffer + nBufferPos, "%s", szSGML_START_PARA);
			}
        }
}


//	Add the rest of the entry text (eg, the definition) to the return buffer.
//	The buffer position is advanced past the new text.
//
VOID AddBodyToBuffer (PSRC_INFO pSrc, PSZ pszBuffer, INT& nBufferPos, INT nBufferLen, LOOKUP_TYPE eType)
{
    DEBUG_OUT(TL_GARRULOUS, ("AddBodyToBuffer(%lx, %lx, %d, %d, %d)\n",
                             pSrc, pszBuffer, nBufferPos, nBufferLen, eType));
    CHAR szText[nTEXT_BUFFER_MAX];

    INT nTextLen = 0;

	// Put a starting headword tag for the first synonym
	// NOTE: each synoynmo is separated by a newline character (0x0A)
	if (eType == eSYNONYMS) {
		nTextLen += sprintf(szText + nTextLen, "%s", szSGML_START_HEAD);
		}

    while (nBufferPos < nBufferLen) {

		//	Sanity check (the lines are less than 80 characters so this shouldn't happen)
	    if (nTextLen == STR_LEN(szText)) {
			AssertError();
			break;
			}

		//	Quit when the start of the next entry is encountered
        INT nNextChar = MyGetChar(pSrc->hDatFile);
        if (nNextChar == START_BYTE) {
            break;
            }

		//	Convert a newline into the entry separator string and
		//	flush the accumulation buffer into the return buffer.
        else if (nNextChar == cNEW_LINE) {
			AddTextToBuffer(szText, nTextLen, pszBuffer, nBufferPos, nBufferLen, eType);
            nTextLen = 0;
			// Put a ending headword tag for the last synonym
			if (eType == eSYNONYMS) {
				nTextLen += sprintf(szText + nTextLen, "%s", szSGML_END_HEAD);
				}
			}

		//	Turn on/off italics if when boldface codes are found (yes, bold => italics)
        else if (nNextChar == START_BOLD) {
			nTextLen += sprintf(szText + nTextLen, "%s", szSGML_START_ITALICS);
            }
        else if (nNextChar == END_BOLD) {
			nTextLen += sprintf(szText + nTextLen, "%s", szSGML_END_ITALICS);
            }

		//	Add annotation tag if within brackets for examples ([the miser 
		//	grudged his dog its food]) and dictionary annotations (eg, [Colloq.])
		else if (nNextChar == nLEFT_BRACKET) {
			nTextLen += sprintf(szText + nTextLen, "%s", szSGML_START_ANNOTATION);
			}
		else if (nNextChar == nRIGHT_BRACKET) {
			nTextLen += sprintf(szText + nTextLen, "%s", szSGML_END_ANNOTATION);
			}

		//	Add the next character to the accumulator buffer.
        else if (IsValidChar(nNextChar)) {
            szText[nTextLen++] = TranslateChar(nNextChar);
            }
        }

	//	Add any remaining text to the buffer
	if (nTextLen > 0) {
		AddTextToBuffer(szText, nTextLen, pszBuffer, nBufferPos, nBufferLen, eType);
		}

	return;
}


//-----------------------------------------------------------------------------
//	'Internal' Lookup API
//-----------------------------------------------------------------------------


//  Open a reference source, specified by the BNX file name w/o extension (eg, DICT4).
//  The BNX file is an ascii index file giving the file name for the reference source,
//  (eg, DICT), the starting offset for that index (eg, x9d7c00), and then the last
//  entries for each block starting at the offset (eg, ' Ahura Mazda', ..., ' zymurgy').
//
EXTERN_C PSRC_INFO DLL_EXPORT OpenSource (PCSZ pszSourceName, PCSZ pszBnxDir, PCSZ pszDataDir)
{
    DEBUG_OUTF(TL_VERBOSE, ("OpenSource(%s, %s, %s)\n", 
                            PtrText(pszSourceName), PtrText(pszBnxDir), PtrText(pszDataDir)));
    PSRC_INFO pSrc = ALLOC(SRC_INFO);

    wsprintf(pSrc->szIndex, "%.8s", pszSourceName);
    ReadBlockIndex(pszBnxDir, pSrc);
    BOOL bOK = FALSE;
    
    /* open the data file */
    CHAR szFile[PATH_MAX]; 
    pSrc->hDatFile = NULL;
    ASSERT(HasTrailingPathDelim(pszDataDir));
    wsprintf(szFile, "%s%s.DAT", pszDataDir, pSrc->szSource);
    pSrc->hDatFile = MyOpenFile(szFile, "rb", pSrc->hDatFile, NULL, TRUE);
    if (pSrc->hDatFile) {
        pSrc->lDatSize = MyGetFileSize(pSrc->hDatFile);
        pSrc->xi.lMaxDataOffset = pSrc->lDatSize;

        /* open the index files */
        pSrc->hIdxFile = NULL;
        wsprintf(szFile, "%s%s.IDX", pszDataDir, pSrc->szSource);
        pSrc->hIdxFile = MyOpenFile(szFile, "rb", pSrc->hIdxFile, NULL, TRUE);
        if (pSrc->hIdxFile) {
            pSrc->lIdxSize = MyGetFileSize(pSrc->hIdxFile);
            
            //  doctor up an Index Info entry
            pSrc->xi.hIdxFile = NULL;
            pSrc->xi.hIndex = NULL;
            pSrc->xi.bAsciiIndex = FALSE;
            pSrc->xi.hAsciiIndex = NULL;
            bOK = TRUE;
            }
        }

    //  Cleanuop up if things didn't go well
    if (!bOK) {
        DEBUG_OUTF(TL_BASIC, ("Trouble opening source '%s'; cleaning up\n", pszSourceName));
        FREE(pSrc);
        pSrc = NULL;
        }
    
    return (pSrc);
}


//	Close the spefied reference source, cleaning up memory and file handles, etc.
//	
EXTERN_C PSRC_INFO DLL_EXPORT CloseSource (PSRC_INFO pSrc)
{
    DEBUG_OUTF(TL_VERBOSE, ("CloseSource(%lx)\n", pSrc));
    if (pSrc) {
        MyCloseFile(pSrc->hDatFile, NULL);
        MyCloseFile(pSrc->hIdxFile, NULL);
#if FALSE
        for (INT i = 0; i < pSrc->nNumGroups; i++) {
            free(pSrc->apszLastPhrase[i]);
            }
        free(pSrc->apszLastPhrase);
#endif
        FREE(pSrc);
        }

    return (NULL);
}


//	Get a handle for a source from a list of open reference sources, given the ascii name
//	for the source (eg, DICT1).
//
EXTERN_C PSRC_INFO DLL_EXPORT GetSourceHandle (PPSRC_INFO ppSrc, INT nNumSources, PCSZ pszSource)
{
    DEBUG_OUTF(TL_VERBOSE, ("GetSourceHandle(%lx, %d, %s)\n", ppSrc, nNumSources, PtrText(pszSource)));
    ASSERT(ppSrc);

    PSRC_INFO pSrc = NULL;
    if (ppSrc) {
        for (INT i = 0; i < nNumSources; i++) {
            if (EQUAL_STR(pszSource, ppSrc[i]->szIndex)) {
                pSrc = ppSrc[i];
                }
            }
        }
    
    return (pSrc);
}


#if FALSE

//	Find a handle for the first source of a given type from a list of open reference sources.
//
EXTERN_C PSRC_INFO DLL_EXPORT FindSourceHandle (PPSRC_INFO ppSrc, INT nNumSources, LOOKUP_TYPE eType)
{
    DEBUG_OUTF(TL_VERBOSE, ("FindSourceHandle(%lx, %d, %d)\n", ppSrc, nNumSources, eType));
    ASSERT(ppSrc);

    PSRC_INFO pSrc = NULL;
    if (ppSrc) {
        for (INT i = 0; i < nNumSources; i++) {
            if (ppSrc[i]->eType == eType)) {
                pSrc = ppSrc[i];
                }
            }
        }
    
    return (pSrc);
}

#endif


//	Lookup a phase in one of the reference data files. For cross-reference lookups
//	the resulting list can be limited by using the AND operator.
//
//	The references to the phrase of the first conjunct is used to initialize the master 
//	reference list. For each remaining conjunct, the list of references is retrieved
//	and intersected with the master list.
//
EXTERN_C BOOL DLL_EXPORT DoLookupPhrase (PSRC_INFO pSrc, PCSZ pszInPhrase, PSZ pszBuffer, 
                                    INT nBufferLen, LOOKUP_TYPE eType)
{
    DEBUG_OUT(TL_VERBOSE, ("DoLookupPhrase(%lx, %s, %lx, %d, %d)\n",
                           pSrc, pszInPhrase, pszBuffer, nBufferLen, eType));
    ASSERT(pSrc);
	ASSERT(pszInPhrase);
	ASSERT(pszBuffer);

	//	Initialize things (like allocate the master and current reference lists)
    PLONG plRefs = ALLOC_ARRAY(nMAX_REFS, LONG);
    INT nRefs = 0;
    PLONG plNewRefs = ALLOC_ARRAY(nMAX_REFS, LONG);
    INT nNewRefs = 0;
    DEBUG_OUT(TL_DETAILED, ("\nLooking up %s for phrase '%s' in %s\n", 
                            LookupTypeToString(eType), pszInPhrase, pSrc->szSource));

	//	Initialize the return buffer to the empty string
	pszBuffer[0] = cNULL;

    //  Make a local uppercase copy of the phrase
    //	TODO: define DUPLICATE_STRING
    INT nPhraseLen = StringLength(pszInPhrase);
    PSZ pszPhrase = ALLOC_STRING(nPhraseLen);
    StringCopy(pszPhrase, pszInPhrase);
    MakeUpper(pszPhrase);

    PCSZ pszStart = pszPhrase;
    BOOL bFirstTime = TRUE;
    BOOL bMore = TRUE;
    do {
        //  Get the next subphrase from "phrase1 AND phrase3 AND ... phraseN"
        //
        CHAR szSubPhrase[256];
        PCSZ pszEnd = FindString(pszStart, szAND);
        if (!pszEnd) {
            pszEnd = pszStart + StringLength(pszStart);
            bMore = FALSE;
            }
        INT nSubLen = pszEnd - pszStart;
        Assert(nSubLen < STR_LEN(szSubPhrase));
        StringCopyN(szSubPhrase, pszStart, nSubLen);
        if (bMore) {
            pszEnd += nAND_LEN;
            }

        //  Get references for the subphrase
        //
        PLONG plTempSrcs = (bFirstTime ? plRefs : plNewRefs);       
        nNewRefs = GetPhraseEntries(pSrc, szSubPhrase, plTempSrcs, nMAX_REFS);

        //  Combine the result with the existing list, if any
        //
        if (bFirstTime) {
            nRefs = nNewRefs;
            }
        else {
            nRefs = MergeAnd(plRefs, nRefs, plTempSrcs, nNewRefs);
            }

        bFirstTime = FALSE;
        pszStart = pszEnd;
        } while (bMore);
    

	//	Resolve the references into the actual entry names
    INT nPos = AddRefsToBuffer(pSrc, plRefs, nRefs, pszBuffer, nBufferLen, eType);
	BOOL bOK = (nPos > 0);

	//	Cleanup things
    FREE_ARRAY(plRefs);
    FREE_ARRAY(plNewRefs);
    FREE_STRING(pszPhrase);

    return (bOK);
}


//-----------------------------------------------------------------------------
//  Exported Lookup API
//-----------------------------------------------------------------------------


//	Initialize the lookup library. This allocates memory for the lookup results
//	and determines where the TRL reference content files are located
//
EXTERN_C PLOOKUP_INFO DLL_EXPORT OldInitLookup (HINSTANCE hInst, HWND hWndFrame, PCSZ pszClientAppName)
{
    DEBUG_OUTF(TL_VERBOSE, ("OldInitLookup(%x, %x, '%s')\n", 
			   hInst, hWndFrame, F(pszClientAppName)));

	//	Allocate memory for the lookup process and results
    PLOOKUP_INFO pLI = ALLOC(LOOKUP_INFO);
    pLI->pszText = ALLOC_STRING(nLOOKUP_TEXT_MAX);

	//	Make note of the client application
    RegisterClientApp(hInst, hWndFrame, pszClientAppName);

	//	Determine where the user put the reference files
    INT nLen;
    nLen = GetPrivateProfileString(szSOURCE, szBNX_DIR, GetCurrentDir(), pLI->szBnxDir, STR_LEN(pLI->szBnxDir), szLOOKUP_INI);
    AddPathDelim(pLI->szBnxDir, nLen);
    nLen = GetPrivateProfileString(szSOURCE, szDATA_DIR, GetCurrentDir(), pLI->szDataDir, STR_LEN(pLI->szDataDir), szLOOKUP_INI);
    AddPathDelim(pLI->szDataDir, nLen);
        
    return (pLI);
}


EXTERN_C PLOOKUP_INFO DLL_EXPORT InitLookup (HINSTANCE hInst, HWND hWndFrame, 
										    PCSZ pszClientAppName, PCSZ pszDataDir, PCSZ pszIndexDir)
{
    DEBUG_OUTF(TL_VERBOSE, ("InitLookup(%x, %x, '%s', '%s', '%s')\n", 
			  hInst, hWndFrame, F(pszClientAppName), F(pszDataDir), F(pszIndexDir)));

	PLOOKUP_INFO pLI = OldInitLookup(hInst, hWndFrame, pszClientAppName);
	if (pLI && pszDataDir) {
		StringCopy(pLI->szDataDir, pszDataDir);
		AddPathDelim(pLI->szDataDir);
		}
	if (pLI && pszIndexDir) {
		StringCopy(pLI->szBnxDir, pszIndexDir);
		AddPathDelim(pLI->szBnxDir);
		}

	return (pLI);
}


//	Shutdown the lookup library, This frees memory used for the lookup results.
//
EXTERN_C BOOL DLL_EXPORT EndLookup (HINSTANCE hInst, PLOOKUP_INFO pLI)
{
    DEBUG_OUTF(TL_VERBOSE, ("EndLookup(%lx)\n", pLI));
    BOOL bOK = FALSE;

	//	Free the memory used in the lookup process and for the results
	//	Also, close any sources that have been left open.
    ASSERT(pLI);
    if (pLI) {
        FREE(pLI->pszText);
        for (INT i = 0; i < pLI->nNumSources; i++) {
            CloseSource(pLI->apSrc[i]);
            }
        FREE(pLI);
        bOK = TRUE;
        }

	//	Forget about the client application
    UnregisterClientApp(hInst);

    return (bOK);
}


//	Open and add a source to the list of open reference sources.
//
EXTERN_C BOOL DLL_EXPORT AddSource (PLOOKUP_INFO pLI, PCSZ pszSource)
{
    DEBUG_OUTF(TL_VERBOSE, ("AddSource(%lx, %s)\n", pLI, PtrText(pszSource)));
    ASSERT(pszSource);

    BOOL bOK = FALSE;
    ASSERT(pLI);
    if (pLI) {
        BOOL bTempSrc = FALSE;

		//	Make sure there's room (TODO: dynamic resizing of the list)
        ASSERT(pLI->nNumSources < nMAX_SOURCES);
        if (pLI->nNumSources >= nMAX_SOURCES) {
            DEBUG_OUTF(TL_BASIC, ("AddSource: maximum number of sources (%d) already open\n", pLI->nNumSources));
            return (FALSE);
            }

		//	Open the reference source
        pLI->apSrc[pLI->nNumSources] = OpenSource(pszSource, pLI->szBnxDir, pLI->szDataDir);
        if (pLI->apSrc[pLI->nNumSources]) {
            pLI->nNumSources++;
            bOK = TRUE;
            bTempSrc = TRUE;
            }
        if (bTempSrc) {		// what am I doing?
            CloseSource(pLI->apSrc[pLI->nNumSources]);
            pLI->nNumSources--;
            }
        }

    return (bOK);
}


//	Lookup a phrase from the specified source. If the source is not active, it is opened
//	and added to the list of active ones.
//
EXTERN_C BOOL DLL_EXPORT LookupPhrase (PLOOKUP_INFO pLI, PCSZ pszPhrase, PCSZ pszSource, LOOKUP_TYPE eType)
{
    DEBUG_OUTF(TL_VERBOSE, ("LookupPhrase(%lx, %s, %d)\n",
                            pLI, PtrText(pszPhrase), PtrText(pszSource), eType));
    BOOL bOK = FALSE;

    ASSERT(pLI);
    if (pLI) {
		//	See if the source is already open
        PSRC_INFO pSrc = GetSourceHandle(pLI->apSrc, pLI->nNumSources, pszSource);
        if (!pSrc) {
			//	If not, open it and add it to the list
            bOK = AddSource(pLI, pszSource);
            if (bOK) {
                pLI->nNumSources++;
                pSrc = pLI->apSrc[pLI->nNumSources - 1];
                }
            }
		
		//	Lookup the phrase
        if (pSrc) {
            bOK = DoLookupPhrase(pSrc, pszPhrase, pLI->pszText, nLOOKUP_TEXT_MAX, eType);
            }
        }

    return (bOK);
}


//	Retrieve a text pointer to the results of the last lookup operation.
//
EXTERN_C PCSZ DLL_EXPORT GetResultsPtr (PLOOKUP_INFO pLI)
{
    DEBUG_OUTF(TL_VERBOSE, ("GetResultsPtr(%lx)\n", pLI));
    PCSZ pszResults = NULL;

    ASSERT(pLI);
    if (pLI) {
        pszResults = pLI->pszText;
        }
        
    return (pszResults);
}
