#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <X11/X.h>
#include "ic.h"
#ifdef NEXT
extern char *strdup();
#endif

/*
   ic.c - interclient communication dealing with escort, wordnet,
   and context.
*/

static int CheckForTone(int);
static char *EncodeMessage(int, char *, int, int [], int [], char *);
static Packet_p DecodeMessage(char *);

static Display *display;

void InitializeCommunication(int type)
{
    char *dname;

    app_type = type;
    dname = getenv("DISPLAY");
    if(!dname) {
	fprintf(stderr,"You must set your DISPLAY variable before using the interclient communication\n");
	fprintf(stderr,"library.\n");
	exit(1);
    }
    if(!display) display = XOpenDisplay(dname);
    HangUpLine(type);
}

void SendPacket(int to, char *word, int num, int senses[MAXNUM], 
		int pos[MAXNUM], char *extra)
{
    char *message;
    Status s;
    XEvent event;
    int nbytes;

    if(!CheckForTone(to)) return;
    message = EncodeMessage(to, word, num, senses, pos, extra);
    XStoreBuffer(display , message, strlen(message)+1, to);
    free(message);
    message = XFetchBuffer(display, &nbytes, to);
    free(message);
}

Packet_p ReceivePacket()
{
    int openline, nbytes;
    char *message;
    char empty[] = "Empty";
    Packet_p packet;

    message = XFetchBuffer(display, &nbytes, app_type);
    if(!message || strncmp(message, DELN, 3)) {
	if(message) free(message);
	return(NULL);
    }
    packet = DecodeMessage(message);
    free(message);
    XStoreBuffer(display, empty, 5, app_type);
    message = XFetchBuffer(display, &nbytes, app_type);
    free(message);

    return(packet);
}

static int CheckForTone(int buf)
{
    int i, nbytes;
    char *message;

    message = XFetchBuffer(display, &nbytes, buf);
    if(!message) return(1);
    else if(!strcmp(message, "Busy")) { free(message); return(0); }

    free(message);
    return(1);
}

void PickUpLine(int type)
{
    char *message;
    int nbytes;

    XStoreBuffer(display, "Busy", 4, type);
    message = XFetchBuffer(display, &nbytes, type);
    free(message);
}

void HangUpLine(int type)
{
    char *message;
    int nbytes;

    XStoreBuffer(display, "Empty", 5, type);
    message = XFetchBuffer(display, &nbytes, type);
    free(message);
}

static char *EncodeMessage(int to, char *word, int num, int senses[MAXNUM],
		    int pos[MAXNUM], char *extra)
{
    char *message;
    char temp[512];
    int i;

    message = (char *) malloc (4 + strlen(word) +
			       num*(4) + /* Sense & Pos numbers */
			       ((extra) ? strlen(extra) : 0) +		
			       3*(3+(num*2)) + /* Delineators */
			       5 /* to(1 char)+num(2 char)+NULL */
			       /* and from */
			       );

    sprintf(message,"%s%d%s%d%s%s%s%d%s",
	    DELN,app_type,DELN,to,DELN,word,DELN,num,DELN);

    for(i = 0; i < num; i++) {
	sprintf(temp,"%d%s%d%s",senses[i],DELN,pos[i],DELN);
	strcat(message,temp);
    }

    if(extra) strcat(message,extra);

    return(message);
}

static Packet_p DecodeMessage(char *message)
{
    int i, to, from, num = 0, senses[MAXNUM], pos[MAXNUM];
    char *word, *extra, *temp, *eom;
    Packet_p p;
					
    p = (Packet_p) malloc (sizeof(Packet));

    for(eom = message; *eom != '\0'; eom++);

    message = message + 3;	/* Get rid of first DELN */
    temp = strstr(message, DELN);
    *temp = '\0';
    p->from = atoi(message);

    message = temp+3;

    temp = strstr(message, DELN);
    *temp = '\0';
    p->to = atoi(message);

    message = temp+3;

    temp = strstr(message, DELN);
    *temp = '\0';
    p->word = strdup(message);

    message = temp+3;

    temp = strstr(message, DELN);
    *temp = '\0';
    p->num = atoi(message);

    message = temp+3;

    for(i = 0; i < p->num; i++) {

	temp = strstr(message, DELN);
	*temp = '\0';
	p->senses[i] = atoi(message);

	message = temp + 3;

	temp = strstr(message, DELN);
	*temp = '\0';
	p->pos[i] = atoi(message);

	message = temp + 3;
    }

    if(message != eom) p->extra = message;

    return(p);
}

void SetTimer(XtAppContext c, unsigned long i, XtTimerCallbackProc p) 
{
    /* Set timer for app context c, interval i, to procedure p */
    XtAppAddTimeOut(c, i, p, (XtPointer)0);
}

