/*
  
  search.c - WordNet library of search code
  
*/

#ifdef MSDOS
#include <windows.h>
#include <windowsx.h>
#endif
#include "wnconsts.h"
#include <stdio.h>
#include <ctype.h>
#ifdef MAC
/* Needed for strtok() */
#include "macfns.h"
#include "wngrep.h"
#include <string.h>
/* Needed for atoi() */
#include "stdlib.h"
#else
#include <string.h>
#endif
#ifndef MAC
#include <stdlib.h>
#include <assert.h>
#endif
#ifdef unix
#ifdef MACH
/* don't include <malloc.h> for MACH */
#else
#ifndef NEXT
#include <malloc.h>
#endif
#endif
#endif
#include "wn.h"
#include "wnglobals.h"

#include "myerror.h"

static char *Id = "$Id: search.c,v 1.94 1995/02/16 18:13:10 wn Exp $";

/* For adjectives, indicates synset type */

#define DONT_KNOW   0
#define DIRECT_ANT  1   /* direct antonyms (cluster head) */
#define INDIRECT_ANT    2   /* indrect antonyms (similar) */
#define PERTAINYM   3   /* no antonyms or similars (pertainyms) */

#define ALLWORDS    0   /* print all words */
#define SKIP_ANTS   0   /* skip printing antonyms in printsynset() */
#define PRINT_ANTS  1   /* print antonyms in printsynset() */
#define SKIP_MARKER 0   /* skip printing adjective marker */
#define PRINT_MARKER    1   /* print adjective marker */

/* Trace types used by printspaces() to determine print sytle */

#define TRACEP      1   /* traceptrs() */
#define TRACEC      2   /* tracecoords() */
#define TRACEI      3   /* traceinherit() */

#define DEFON 1
#define DEFOFF 0

/* Forward declarations */

static void findsisters(), findtwins(), findcousins(),
     add_relatives(), free_rellist();
static void printsynset(), printantsynset(),
     printbuffer(), printsense(), printword(),
     printspaces(), readfnames(), printstr(), printrelatives();
static int HasPtr(), getsearchsense(), depthcheck(), groupexc();
static char *printant();
static void wngrep(char *string, int dbase);

static int prflag, sense, prsense;
static int overflag;        /* set when output buffer overflows */
static char *searchbuffer = NULL;
static long searchbufsize = SEARCHBUF; /* init to default size */
static char tmpbuf[256];    /* general purpose printing buffer */
static char wdbuf[WORDBUF]; /* general purpose word buffer */
static int adj_marker;
static char *markers[] = {
    "",
    "(prenominal)",
    "(postnominal)",
    "(predicate)",
};

static char *freqcats[] = {
    "extremely rare","very rare","rare","uncommon","common",
    "familiar","very familiar","extremely familiar"
};

#if 1
static bDebug = 0;
static bAddSense = 0;
#endif


/* Find word in index file and return parsed entry in data structure.
   Input word must be exact match of string in database. */

IndexPtr index_lookup(char *word, int dbase)
{
    register char *ptrtok;
    register IndexPtr idx = NULL;
    register int j;
    FILE *fp;
    char *line;

    if ((fp = indexfps[dbase]) == NULL) {
    DisplayError("Error: %s indexfile not open\n", partnames[dbase]);
    return(NULL);
    }

    if ((line = bin_search(word, fp)) != NULL) {
        idx = (IndexPtr)malloc(sizeof(Index));
        assert(idx);
    
        idx->wd='\0';
        idx->pos='\0';
        idx->off_cnt=0;
        idx->sense_cnt=0;
        idx->offset='\0';
        idx->ptruse_cnt=0;
        idx->ptruse='\0';
    
    /* get the word */
    ptrtok=strtok(line," \n");
    
    idx->wd=(char *) malloc(strlen(ptrtok) + 1);
    assert(idx->wd);
    strcpy(idx->wd, ptrtok);
    
    /* get the part of speech */
    ptrtok=strtok(NULL," \n");
    idx->pos=(char *) malloc(strlen(ptrtok) + 1);
    assert(idx->pos);
    strcpy(idx->pos, ptrtok);
    
    /* get the collins count */
    ptrtok=strtok(NULL," \n");
    idx->sense_cnt = atoi(ptrtok);
    
    /* get the number of pointers types */
    ptrtok=strtok(NULL," \n");
    idx->ptruse_cnt = atoi(ptrtok);

    if (idx->ptruse_cnt) {
        idx->ptruse = (int *) malloc(idx->ptruse_cnt * (sizeof(int)));
        assert(idx->ptruse);
    
        /* get the pointers types */
        for(j=0;j < idx->ptruse_cnt; j++) {
        ptrtok=strtok(NULL," \n");
        idx->ptruse[j] = getptrtype(ptrtok);
        }
    }
    
    /* get the number of offsets */
    ptrtok=strtok(NULL," \n");
    idx->off_cnt = atoi(ptrtok);
    
    /* make space for the offsets */
    idx->offset = (long *) malloc(idx->off_cnt * (sizeof(long)));
    assert(idx->offset);
    
    /* get the offsets */
    for(j=0;j<idx->off_cnt;j++) {
        ptrtok=strtok(NULL," \n");
        idx->offset[j] = atol(ptrtok);
    }
    }
    return(idx);
}

#define MAX_TRIES   5

/* 'smart' search of index file.  Find word in index file, trying different
   techniques - replace hyphens with underscores, replace underscores with
   hyphens, strip hyphens and underscores, strip periods. */

IndexPtr getindex(char *searchstr, int dbase)
{
    register int offset = 0, i, j, k;
    register char c;
    IndexPtr idx[MAX_TRIES];
    char strings[MAX_TRIES][WORDBUF]; /* vector of search strings */
    
    /* Prepare vector of search strinsg */
    
    for (i = 0; i < MAX_TRIES; i++) {
    strcpy(strings[i], searchstr);
    idx[i] = NULL;
    }
    strsubst(strings[1], '_', '-');
    strsubst(strings[2], '-', '_');

    /* remove all spaces and hyphens from last search string, then
       all periods */
    for (i = j = k = 0; (c = searchstr[i]) != '\0'; i++) {
    if (c != '_' && c != '-')
        strings[3][j++] = c;
    if (c != '.')
        strings[4][k++] = c;
    }
    strings[3][j] = c;      /* copy final NULL */
    strings[4][k] = c;      /* copy final NULL */
    
    /* Eliminate duplications */

    for (i = 1; i < MAX_TRIES; i++)
    if (!strcmp(strings[0], strings[i]))
        strings[i][0] = '\0';   
    
    /* Get index entries for all versions of search string, and
       return lowest numbered offset (assume if more than one
       found, they're in the same syset as alternate spellings) */
    
    for (i = (MAX_TRIES - 1); i >= 0; i--)
    if (strings[i][0] != '\0')
        if (idx[i] = index_lookup(strings[i], dbase))
        offset = i;

    for (i = 0; i < MAX_TRIES; i++) /* free up unused index structures */
    if (i != offset && idx[i])
        free_index(idx[i]);
    return(idx[offset]);
}

/* Read synset from data file at byte offset passed and return parsed
   entry in data structure. */

SynsetPtr read_synset(int dbase, long boffset, char *word)
{
    FILE *fp;

    if((fp = datafps[dbase]) == NULL) {
    DisplayError("Error: %s datafile not open\n", partnames[dbase]);
    return(NULL);
    }
    
    fseek(fp, boffset, 0);  /* position file to byte offset requested */

    return(parse_synset(fp, dbase, word)); /* parse synset and return */
}

/* Read synset at current byte offset in file and return parsed entry
   in data structure. */

SynsetPtr parse_synset(FILE *fp, int dbase, char *word)
{
    static char line[LINEBUF];
    register char *ptrtok;
    char *tmpptr;
    char wdnum[3];
    register int i;
    register SynsetPtr synptr;
    
    if ((tmpptr = fgets(line, LINEBUF, fp)) == NULL)
    return(NULL);
    
    synptr = (SynsetPtr)malloc(sizeof(Synset));
    assert(synptr);
    
    synptr->hereiam=0;
    synptr->sstype=DONT_KNOW;
    synptr->fnum=0;
    synptr->pos='\0';
    synptr->wcount=0;
    synptr->words='\0';
    synptr->whichword=0;
    synptr->ptrcount=0;
    synptr->ptrtyp='\0';
    synptr->ptroff='\0';
    synptr->ppos = '\0';
    synptr->pto='\0';
    synptr->pfrm='\0';
    synptr->fcount=0;
    synptr->frmid='\0';
    synptr->frmto='\0';
    synptr->defn='\0';
    synptr->nextss = NULL;
    synptr->searchtype = -1;
    synptr->ptrlist = NULL;
    synptr->headword = NULL;
    synptr->headsense = 0;

    ptrtok=line;
    
    /* looking at offset */
    ptrtok=strtok(line," \n");
    synptr->hereiam = atol(ptrtok);
    
    /* looking at FNUM */
    ptrtok=strtok(NULL," \n");
    synptr->fnum=atoi(ptrtok);
    
    /* looking at POS */
    ptrtok=strtok(NULL," \n");
    synptr->pos = (char *)malloc(strlen(ptrtok) + 1);
    assert(synptr->pos);
    strcpy(synptr->pos, ptrtok);
    if (getsstype(synptr->pos) == SATELLITE)
    synptr->sstype = INDIRECT_ANT;
    
    /* looking at numwords */
    ptrtok=strtok(NULL," \n");
    synptr->wcount=strtol(ptrtok,NULL,16);
    
    synptr->words=(char **)malloc(synptr->wcount  * sizeof(char *));
    assert(synptr->words);
    synptr->wnsense = (int *)malloc(synptr->wcount * sizeof(int));
    assert(synptr->wnsense);
    
    for(i=0;i<synptr->wcount; i++) {
    ptrtok=strtok(NULL," \n");
    synptr->words[i]= (char *)malloc(strlen(ptrtok) + 1);
    assert(synptr->words[i]);
    strcpy(synptr->words[i], ptrtok);
    
    /* is this the word we're looking for? */
    
    if(!strcmp(word,strtolower(ptrtok))) /* strtolower does deadjify */
        synptr->whichword = i+1;
    
    ptrtok = strtok(NULL, " \n");
#if defined(_WIN32)
    sscanf(ptrtok, "%x", &synptr->wnsense[i]);
#else
    synptr->wnsense[i] = strtol(ptrtok, NULL, 16);
#endif
    }
    
    /* get the pointer count */
    ptrtok=strtok(NULL," \n");
    synptr->ptrcount=atoi(ptrtok);

    if (synptr->ptrcount) {

    /* alloc storage for the pointers */
    synptr->ptrtyp=(int *)malloc(synptr->ptrcount * sizeof(int));
    assert(synptr->ptrtyp);
    synptr->ptroff=(long *)malloc(synptr->ptrcount * sizeof(long));
    assert(synptr->ptroff);
    synptr->ppos = (int *)malloc(synptr->ptrcount * sizeof(int));
    assert(synptr->ppos);
    synptr->pto=(int *)malloc(synptr->ptrcount * sizeof(int));
    assert(synptr->pto);
    synptr->pfrm=(int *)malloc(synptr->ptrcount * sizeof(int));
    assert(synptr->pfrm);
    
    for(i=0;i<synptr->ptrcount;i++) {
        /* get the pointer type */
        ptrtok=strtok(NULL," \n");
        synptr->ptrtyp[i] = getptrtype(ptrtok);
        if (dbase == ADJ && synptr->sstype == DONT_KNOW) {
        if (synptr->ptrtyp[i] == PERTPTR)
            synptr->sstype = PERTAINYM;
        else if (synptr->ptrtyp[i] == ANTPTR)
            synptr->sstype = DIRECT_ANT;
        }

        /* get the pointer offset */
        ptrtok=strtok(NULL," \n");
        synptr->ptroff[i] = atol(ptrtok);
    
        /* get the pointer part of speech */
        ptrtok = strtok(NULL, " \n");
        synptr->ppos[i] = getpos(ptrtok);
    
        /* get the lexp to/from restrictions */
        ptrtok = strtok(NULL," \n");
    
        tmpptr = ptrtok;
        strncpy(wdnum,tmpptr,2);
        wdnum[2] = '\0';
        synptr->pfrm[i] = strtol(wdnum, (char **)NULL, 16);
    
        tmpptr += 2;
        strncpy(wdnum,tmpptr,2);
        wdnum[2] = '\0';
        synptr->pto[i] = strtol(wdnum, (char **)NULL, 16);
    }
    }
    
    /* retireve optional information from verb synset */
    if(dbase == VERB) {
    ptrtok=strtok(NULL," \n");
    synptr->fcount=atoi(ptrtok);
    
    /* allocate frame storage */
    
    synptr->frmid=(int *)malloc(synptr->fcount * sizeof(int));  
    assert(synptr->frmid);
    synptr->frmto=(int *)malloc(synptr->fcount * sizeof(int));  
    assert(synptr->frmto);
    
    for(i=0;i<synptr->fcount;i++) {
        /* skip the frame pointer (+) */
        ptrtok=strtok(NULL," \n");
        
        ptrtok=strtok(NULL," \n");
        synptr->frmid[i]=atoi(ptrtok);
        
        ptrtok=strtok(NULL," \n");
        synptr->frmto[i]=atoi(ptrtok);
    }
    }
    
    /* get the optional definition */
    
    ptrtok=strtok(NULL," \n");
    if(ptrtok) {
    ptrtok=strtok(NULL," \n");
    sprintf(tmpbuf,"");
    while(ptrtok != NULL) {
        strcat(tmpbuf,ptrtok);
        ptrtok=strtok(NULL," \n");
        if(ptrtok)
        strcat(tmpbuf," ");
    }
    synptr->defn=(char *)malloc(strlen(tmpbuf) + 4);
    assert(synptr->defn);
    sprintf(synptr->defn,"(%s)",tmpbuf);
    }

    return(synptr);
}

/* Free a synset linked list allocated by findtheinfo_ds() */

void free_syns(SynsetPtr synptr)
{
    SynsetPtr cursyn, nextsyn;

    if (synptr) {
    cursyn = synptr;
    nextsyn = synptr->nextss;
    while(cursyn) {
        nextsyn = cursyn->nextss;
        free_synset(cursyn);
        cursyn = nextsyn;
    } 
    }
}

/* Free a synset */

void free_synset(SynsetPtr synptr)
{
    int i;
    
    free(synptr->pos);
    for (i = 0; i < synptr->wcount; i++){
    free(synptr->words[i]);
    }
    free(synptr->words);
    free(synptr->wnsense);
    if (synptr->ptrcount) {
    free(synptr->ptrtyp);
    free(synptr->ptroff);
    free(synptr->ppos);
    free(synptr->pto);
    free(synptr->pfrm);
    }
    if (synptr->fcount) {
    free(synptr->frmid);
    free(synptr->frmto);
    }
    if (synptr->defn)
    free(synptr->defn);
    if (synptr->headword)
    free(synptr->headword);
    if (synptr->ptrlist)
    free_syns(synptr->ptrlist); /* changed from free_synset() */
    free(synptr);
}

/* Free an index structure */

void free_index(IndexPtr idx)
{
    free(idx->wd);
    free(idx->pos);
    if (idx->ptruse)
    free(idx->ptruse);
    free(idx->offset);
    free(idx);
}

/* Recursive search algorithm to trace a pointer tree */

static void traceptrs(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
{
    register int i;
    int extraindent = 0;
    register SynsetPtr cursyn;
    char prefix[40], tbuf[20];

    if (abortsearch)
    return;

    if (ptrtyp < 0) {
    ptrtyp = -ptrtyp;
    extraindent = 2;
    }

    for(i=0;i<synptr->ptrcount;i++) {
    if((synptr->ptrtyp[i] == ptrtyp) &&
       ((synptr->pfrm[i] == 0) ||
        (synptr->pfrm[i] == synptr->whichword))) {
        
        if(!prflag) {
        printsense(synptr, sense + 1);
        printsynset("",synptr,"\n",DEFON, ALLWORDS, PRINT_ANTS);
        prflag = 1;
        }

        printspaces(TRACEP, depth + extraindent);

        switch(ptrtyp) {
        case PERTPTR:
        if (dbase == ADV) 
            sprintf(prefix, "Derived from %s -> ",
                partnames[synptr->ppos[i]]);
        else
            sprintf(prefix, "Pertains to %s -> ",
                partnames[synptr->ppos[i]]);
        break;
        case PPLPTR:
        sprintf(prefix, "Participle of verb -> ");
        break;
        case HASMEMBERPTR:
        sprintf(prefix, "   HAS MEMBER: ");
        break;
        case HASSTUFFPTR:
        sprintf(prefix, "   HAS SUBSTANCE: ");
        break;
        case HASPARTPTR:
        sprintf(prefix, "   HAS PART: ");
        break;
        case ISMEMBERPTR:
        sprintf(prefix, "   MEMBER OF: ");
        break;
        case ISSTUFFPTR:
        sprintf(prefix, "   SUBSTANCE OF: ");
        break;
        case ISPARTPTR:
        sprintf(prefix, "   PART OF: ");
        break;
        default:
        sprintf(prefix, "=> ");
        break;
        }


        cursyn=read_synset(synptr->ppos[i], synptr->ptroff[i], "");

        if ((ptrtyp == PERTPTR || ptrtyp == PPLPTR) &&
        synptr->pto[i] != 0) {
        sprintf(tbuf, " (Sense %d)\n",
            getsearchsense(cursyn, synptr->pto[i]));
        printsynset(prefix, cursyn, tbuf, DEFOFF, synptr->pto[i],
                SKIP_ANTS);
        } else 
#if 0
{
	strcpy(tbuf, "\n");
	sprintf(tbuf, " (Sense %d)\n", getsearchsense(cursyn, cursyn->whichword));
        printsynset(prefix, cursyn, tbuf, DEFON, ALLWORDS, PRINT_ANTS);
}
#else
        printsynset(prefix, cursyn, "\n", DEFON, ALLWORDS, PRINT_ANTS);
#endif

        if (ptrtyp == PPLPTR) {
        printsynset("      =>", cursyn, "\n", DEFON, ALLWORDS, PRINT_ANTS);
        traceptrs(cursyn, HYPERPTR, getpos(cursyn->pos), 0);
        }
        if(depth) {
        depth = depthcheck(depth, cursyn);
        traceptrs(cursyn,ptrtyp,dbase,(depth+1));

        free_synset(cursyn);
        } else
        free_synset(cursyn);
    }
    }
}

static void tracecoords(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
{
    register int i;
    register SynsetPtr cursyn;
    
    for(i=0;i<synptr->ptrcount;i++) {
    if((synptr->ptrtyp[i] == HYPERPTR) &&
       ((synptr->pfrm[i] == 0) ||
        (synptr->pfrm[i] == synptr->whichword))) {
        
        if(!prflag) {
        printsense(synptr, sense + 1);
        printsynset("",synptr,"\n",DEFON, ALLWORDS, PRINT_ANTS);
        prflag = 1;
        }
        printspaces(TRACEC, depth);

        cursyn=read_synset(dbase,synptr->ptroff[i],"");

        printsynset("-> ",cursyn,"\n",DEFON, ALLWORDS, SKIP_ANTS);

        traceptrs(cursyn,ptrtyp,NOUN,depth);
        
        if(depth) {
        depth = depthcheck(depth, cursyn);
        tracecoords(cursyn,ptrtyp,dbase,(depth+1));
        free_synset(cursyn);
        } else
        free_synset(cursyn);
    }
    }
}

/* Trace through the hypernym tree and print all MEMBER, STUFF
   and PART info. */

static void traceinherit(SynsetPtr synptr, int ptrbase, int dbase, int depth)
{
    register int i;
    register SynsetPtr cursyn;
    
    for(i=0;i<synptr->ptrcount;i++) {
    if((synptr->ptrtyp[i] == HYPERPTR) &&
       ((synptr->pfrm[i] == 0) ||
        (synptr->pfrm[i] == synptr->whichword))) {
        
        if(!prflag) {
        printsense(synptr, sense + 1);
        printsynset("",synptr,"\n",DEFON, ALLWORDS, PRINT_ANTS);
        prflag = 1;
        }
        printspaces(TRACEI, depth);
        
        cursyn=read_synset(dbase,synptr->ptroff[i],"");

        printsynset("=> ",cursyn,"\n",DEFON, ALLWORDS, SKIP_ANTS);
        
        traceptrs(cursyn,ptrbase,NOUN,depth);
        traceptrs(cursyn,ptrbase + 1,NOUN,depth);
        traceptrs(cursyn,ptrbase + 2,NOUN,depth);
        
        if(depth) {
        depth = depthcheck(depth, cursyn);
        traceinherit(cursyn,ptrbase,dbase,(depth+1));
        free_synset(cursyn);
        } else
        free_synset(cursyn);
    }
    }
}

static void traceadjant(SynsetPtr synptr)
{
    SynsetPtr newsynptr;
    register int i, j;
    int anttype = DIRECT_ANT;
    register SynsetPtr simptr, antptr;
    static char similar[] = "        => ";

    if (stopsearch())       /* check flag before proceeding */
    return;

    /* This search is only applicable for ADJ synsets which have
       either direct or indirect antonyms (not valid for pertainyms). */
    
    if (synptr->sstype == DIRECT_ANT || synptr->sstype == INDIRECT_ANT) {
    printsense(synptr, sense + 1);
    printsynset("", synptr, "\n\n", DEFON, ALLWORDS, PRINT_ANTS);
    
    /* if indirect, get cluster head */
    
    if(synptr->sstype == INDIRECT_ANT) {
        anttype = INDIRECT_ANT;
        i = 0;
        while (synptr->ptrtyp[i] != SIMPTR) i++;
        newsynptr = read_synset(ADJ, synptr->ptroff[i], "");
    } else
        newsynptr = synptr;
    
    /* find antonyms - if direct, make sure that the antonym
       ptr we're looking at is from this word */
    
    for (i = 0; i < newsynptr->ptrcount; i++) {
        if (stopsearch()) { /* check flag before proceeding */
        free_synset(newsynptr);
        return;
        }

        if (newsynptr->ptrtyp[i] == ANTPTR &&
        ((anttype == DIRECT_ANT &&
          newsynptr->pfrm[i] == newsynptr->whichword) ||
         (anttype == INDIRECT_ANT))) {
        
        /* read the antonym's synset and print it.  if a
           direct antonym, print it's satellites. */
        
        antptr = read_synset(ADJ, newsynptr->ptroff[i], "");
    
        if (anttype == DIRECT_ANT) {
            printsynset("", antptr, "\n", DEFON, ALLWORDS, PRINT_ANTS);
            for(j = 0; j < antptr->ptrcount; j++) {
            if(antptr->ptrtyp[j] == SIMPTR) {
                simptr = read_synset(ADJ, antptr->ptroff[j], "");
                printsynset(similar, simptr, "\n",
                    DEFON, ALLWORDS, SKIP_ANTS);
                free_synset(simptr);
            }
            }
        } else
            printantsynset(antptr, "\n", anttype, DEFON);

        free_synset(antptr);
        }
    }
    if (newsynptr != synptr)
        free_synset(newsynptr);
    }
}

static void partsall(SynsetPtr synptr, int ptrtyp)
{
    register int ptrbase;
    int hasptr = 0;
    
    ptrbase = (ptrtyp == HMERONYM) ? HASMEMBERPTR : ISMEMBERPTR;
    
    /* First, print out the MEMBER, STUFF, PART info for this synset */
    
    if (HasPtr(synptr, ptrbase)) {
    traceptrs(synptr, ptrbase, NOUN, 1);
    hasptr++;
    }
    if (HasPtr(synptr, ptrbase + 1)) {
    traceptrs(synptr, ptrbase + 1, NOUN, 1);
    hasptr++;
    }
    if (HasPtr(synptr, ptrbase + 2)) {
    traceptrs(synptr, ptrbase + 2, NOUN, 1);
    hasptr++;
    }
    
    /* Print out MEMBER, STUFF, PART info for hypernyms on
       HMERONYM search only */
    
    if (hasptr && ptrtyp == HMERONYM) 
    traceinherit(synptr, ptrbase, NOUN, 1);
}

static void printframe(SynsetPtr synptr, int prsynset)
{
    register int i;

    if (prsynset) {
    printsense(synptr, sense + 1);
    printsynset("",synptr,"\n",DEFON, ALLWORDS, PRINT_ANTS);
    }
    
    for(i = 0; i < synptr->fcount; i++) {
    if ((synptr->frmto[i] == synptr->whichword) ||
        (synptr->frmto[i] == 0)) {
        if (synptr->frmto[i] == synptr->whichword)
        printbuffer("          => ");
        else
        printbuffer("          *> ");
        printbuffer(frametext[synptr->frmid[i]]);
        printbuffer("\n");
    }
    }
}

static void printseealso(SynsetPtr synptr, int dbase)
{
    register SynsetPtr cursyn;
    register int i, first = 1;
    static char firstline[] = "          Also See-> ";
    static char otherlines[] = ", ";
    char *prefix = firstline;


    /* Find all SEEALSO pointers from the searchword and print the
       word or synset pointed to. */

    for(i = 0; i < synptr->ptrcount; i++) {
    if ((synptr->ptrtyp[i] == SEEALSOPTR) &&
        ((synptr->pfrm[i] == 0) ||
         (synptr->pfrm[i] == synptr->whichword))) {

        cursyn = read_synset(dbase, synptr->ptroff[i], "");

        if (synptr->pto[i] == 0)
        printsynset(prefix, cursyn, "", DEFOFF,
                ALLWORDS, SKIP_ANTS);
        else
        printsynset(prefix, cursyn, "", DEFOFF,
                synptr->pto[i], SKIP_ANTS);
        free_synset(cursyn);

        if (first) {
        prefix = otherlines;
        first = 0;
        }
    }
    }
    if (!first)
    printbuffer("\n");
}

static void freq_word(register IndexPtr index)
{
    register int familiar=0;
    register int cnt;
    static char *a_an[] = {
    "", "a noun", "a verb", "an adjective", "an adverb" };
    
    if(index) {
    cnt = index->sense_cnt;
    if (cnt == 0) familiar = 0;
    if (cnt == 1) familiar = 1;
    if (cnt == 2) familiar = 2;
    if (cnt >= 3 && cnt <= 4) familiar = 3;
    if (cnt >= 5 && cnt <= 8) familiar = 4;
    if (cnt >= 9 && cnt <= 16) familiar = 5;
    if (cnt >= 17 && cnt <= 32) familiar = 6;
    if (cnt > 32 ) familiar = 7;
    
    sprintf(tmpbuf,
        "%s used as %s is %s (polysemy count = %d)\n",
        index->wd, a_an[getpos(index->pos)], freqcats[familiar], cnt);
    printbuffer(tmpbuf);
    }
}

static void wngrep(char *string, int dbase)
{
    register int c;
    register char *tp;
    FILE *fp;

    strsubst(strcpy(wdbuf, string), '_', ' ');

    unlink(GREPFILE);

#ifdef HASGREP
    do_wngrep(wdbuf, dbase, GREPFILE);
#else
    #if defined(_WINDOWS)
        strcpy(tmpbuf, "Grep not supported!!!\n");
    #else
        sprintf(tmpbuf, "wngrep \"%s\" %s >%s", wdbuf, partnames[dbase], GREPFILE);
        system(tmpbuf);
        WinExec
    #endif
#endif

    if ((fp = fopen(GREPFILE, "r")) != NULL) {
#ifdef HASGREP
    sprintf(tmpbuf, "Grep of %s %s\n", partnames[dbase], wdbuf);
    printbuffer(tmpbuf);
#endif
    tp = tmpbuf;
    *tp++ = '\t';
    while ((c = getc(fp)) != EOF) {
        *tp++ = c;
        if (c == '\n') {
        *tp = '\0';
        printbuffer(tmpbuf);
        tp = tmpbuf;
        *tp++ = '\t';
        }
    }
    fclose(fp);
    unlink(GREPFILE);
    } else
    DisplayError("Error: can't open temp file \"%s\"\n", GREPFILE);
}


/* Determine if any of the synsets that this word is in have inherited
   meronyms or holonyms. */

static int HasHoloMero(IndexPtr index, int ptrtyp)
{
    register int i;
    register SynsetPtr synset;
    register int found=0;
    int ptrbase;
    
    ptrbase = (ptrtyp == HMERONYM) ? HASMEMBERPTR : ISMEMBERPTR;
    
    for(i = 0; i < index->off_cnt; i++) {
    synset = read_synset(NOUN, index->offset[i], "");
    found += HasPtr(synset, ptrbase);
    found += HasPtr(synset, ptrbase + 1);
    found += HasPtr(synset, ptrbase + 2);
    free_synset(synset);
    }
    return(found);
}

static int HasPtr(SynsetPtr synptr, int ptrtyp)
{
    register int i;
    
    for(i = 0; i < synptr->ptrcount; i++) {
        if(synptr->ptrtyp[i] == ptrtyp) {
        return(1);
    }
    }
    return(0);
}

/* Stucture to keep track of 'relative groups'.  All senses in a relative
   group are displayed together at end of search.  Transitivity is
   supported, so if either of a new set of related senses is already
   in a 'relative group', the other sense is added to that group as well. */

struct relgrp {
    int senses[MAXSENSE];
    struct relgrp *next;
};
static struct relgrp *rellist;

static struct relgrp *mkrellist(void);
static void free_rellist(void);

/* Array of bytes offsets of cousin top node pairs (cousin.tops) */
#define MAXCOUSINTOPS   256
unsigned long ctop_offsets[MAXCOUSINTOPS][2];

/* Find relative groups for all senses of target word in given part
   of speech (in version 1.5, only NOUNS) */

static void relatives(IndexPtr idx, int dbase)
{
    rellist = NULL;

    switch(dbase) {
    case NOUN:
    findsisters(idx, NOUN);
    if (abortsearch)
        return;
    findtwins(idx, NOUN);
    if (abortsearch)
        return;
    findcousins(idx, NOUN);
    if (abortsearch)
        return;
    printrelatives(idx, NOUN);
    break;
    case VERB:
    break;
    case ADJ:
    break;
    case ADV:
    break;
    default:
    break;
    }

    free_rellist();
}

/* Look for 'twins' - synsets with 3 or more words in common. */

static void findtwins(IndexPtr idx, int dbase)
{
    SynsetPtr cursyn, twin, lng, shrt;
    IndexPtr twinidx;
    char buf1[256], buf2[256];
    int i, j, k, l;
    int twincnt;

    /* For each sense of search word, read the synset.  If it has
       at least 3 words in it, read all of the other synsets for
       search word with more than 3 words in it, and see if you
       can find 3 or more words in common.  If so, synsets are
       twins and are added to a relative group. */

    for (i = 0; i < idx->off_cnt - 1; i++) {
    if (stopsearch()) { /* check flag before proceeding */
        return;
    }
    cursyn = read_synset(dbase, idx->offset[i], "");
    if (cursyn->wcount < 3)
        continue;
    for (j = i + 1; j < idx->off_cnt; j++) {
        if (stopsearch()) { /* check flag before proceeding */
        free_synset(cursyn);
        return;
        }
        twin = read_synset(dbase, idx->offset[j], "");
        if (twin->wcount < 3)
        continue;
        if (cursyn->wcount > twin->wcount) {
        lng = cursyn;
        shrt = twin;
        } else{
        lng = twin;
        shrt = cursyn;
        }
        twincnt = 0;
        for (k = 0; (k < shrt->wcount) && (twincnt < 3); k++) {
        if (stopsearch()) { /* check flag before proceeding */
            free_synset(cursyn);
            free_synset(twin);
            return;
        }
        strtolower(strcpy(buf1, shrt->words[k]));
        for (l = 0; l < lng->wcount; l++) {
            strtolower(strcpy(buf2, lng->words[l]));
            if (!strcmp(buf1, buf2)) {
            twincnt++;
            break;
            }
        }
        }
        if (twincnt >= 3)
        add_relatives(idx, i, j);
        free_synset(twin);
    }
    free_synset(cursyn);
    }
}

/* Look for 'sisters' - senses of the search word with a common parent. */

static void findsisters(IndexPtr idx, int dbase)
{
    SynsetPtr synptr;
    int i, j, k, hyperidx;
    unsigned long hypers[MAXSENSE][4];

    /* clear out array of hypernym offsets */

    for (i = 0; i < MAXSENSE; i++)
    for (j = 0; j < 4; j++)
        hypers[i][j] = 0L;

    /* Read the synset for each sense of the search word and
       examine all its pointers.  Save all the hypernyms in an
       array for comparison later. */

    for (i = 0; i < idx->off_cnt; i++) {
    synptr = read_synset(dbase, idx->offset[i], idx->wd);
    hyperidx = 0;
    for (j = 0; j < synptr->ptrcount; j++) {
        if (stopsearch()) {
        free_synset(synptr);
        return;
        }
        if (synptr->ptrtyp[j] == HYPERPTR) {
        hypers[i][hyperidx++] = synptr->ptroff[j];
        if (hyperidx == 4)
            break;
        }
    }
    free_synset(synptr);
    }

    /* Examine all hypernyms for all senses of search word.  Any
       different senses with same hypernym are 'sisters'. */

    for (i = 0; i < (idx->off_cnt - 1); i++)
    for (j = i + 1; j < idx->off_cnt; j++)
        for (k = 0; k < 4; k++) {
        if (hypers[i][k] == 0)
            break;
        for (hyperidx = 0; hyperidx < 4; hyperidx++) {
            if (stopsearch())
            return;
            if (hypers[j][hyperidx] == 0)
            break;
            if (hypers[i][k] == hypers[j][hyperidx])
            add_relatives(idx, i, j);
        }
        }
}
        
#ifdef FOOP
static void findsisters(IndexPtr idx, int dbase)
{
    SynsetPtr synptr, hypernym;
    int i, j, k, l;

    for (l = 0; l < idx->off_cnt; l++) {
    synptr = read_synset(dbase, idx->offset[l], idx->wd);

    /* For each hypernym of this synset, find all its hyponyms
       and see if any hyponym is also in the index entry passed. */

    for(i = 0; i < synptr->ptrcount; i++) {
        if (stopsearch()) { /* check flag before proceeding */
        free_synset(synptr);
        return;
        }
        if(synptr->ptrtyp[i] == HYPERPTR) {

        /* First, get hypernym of this synset */
        hypernym = read_synset(dbase, synptr->ptroff[i], "");

        for (j = 0; j < hypernym->ptrcount; j++) {
            if (stopsearch()) { /* check flag before proceeding */
            free_synset(hypernym);
            free_synset(synptr);
            return;
            }

            /* Next, find all hyponyms which are not this synset
               and see if it is in the index entry for the
               searchword.  If it is, it must be a sister. */

            if ((hypernym->ptrtyp[j] == HYPOPTR) &&
            (hypernym->ptroff[j] != synptr->hereiam)) {
            for (k = 0; k < idx->off_cnt; k++) {
                if (idx->offset[k] == hypernym->ptroff[j])
                add_relatives(idx, k, l);
            }
            }
        }
        free_synset(hypernym);
        }
    }
    free_synset(synptr);
    }
}
#endif

/* Look for 'cousins' - two senses, each under a different predefined
   top node pair.  Top nodes offset pairs are stored in the
   cousin.tops file. */

static void findcousins(IndexPtr idx, int dbase)
{
    SynsetPtr cursyn, cousin;
    int i, j, cidx;
    int other;

    /* Read in cousin.tops file */

    read_cousintops();

    /* Go through all the cousin top node pairs looking to see if
       an offset for the search word is in either tree for the
       current pair.  If it is, check all the other offsets for
       the word.  If found in the other tree, they must be
       cousins - unless on the exception list.
    */

    for (cidx = 0; ctop_offsets[cidx][0]; cidx++) {
    if (stopsearch())
        return;
    for (i = 0; i < idx->off_cnt - 1; i++) {
        if (stopsearch())
        return;
        cursyn = read_synset(dbase, idx->offset[i], "");
        if (trace_cousin(cursyn, ctop_offsets[cidx][0], 1))
        other = 1;
        else if (trace_cousin(cursyn, ctop_offsets[cidx][1], 1))
        other = 0;
        else
        other = -1;
        if (other >= 0) {
        for (j = i + 1; j < idx->off_cnt; j++) {
            if (stopsearch())
            return;
            cousin = read_synset(dbase, idx->offset[j], "");
            if (trace_cousin(cousin, ctop_offsets[cidx][other], 1))
            add_relatives(idx, i, j);
            free_synset(cousin);
        }   
        }
        free_synset(cursyn);
    }
    }
}

/* Trace cousin tree */
int trace_cousin(SynsetPtr synptr, unsigned long top, int depth)
{
    SynsetPtr cursyn;
    int i, ret;

    if (stopsearch())       /* check flag before proceeding */
    return(0);

    if (depth >= MAXDEPTH)
    return(0);

    for(i = 0; i < synptr->ptrcount; i++) {
    if(synptr->ptrtyp[i] == HYPERPTR) {
        if (synptr->ptroff[i] == top)
        return(1);
        cursyn = read_synset(synptr->ppos[i],
                 synptr->ptroff[i],
                 "");
        ret = trace_cousin(cursyn, top, depth + 1);
        free_synset(cursyn);
        if (ret)
        return(ret);
    }
    }
    return(0);
}

/* Read cousin.tops file into array */
void read_cousintops(void)
{
    static int done = 0;
    int i = 0;
#if !defined(_WIN32)
    char szCouz[20];
    char* pszNum;
#endif

    if (done)
    return;

    rewind(cousinfp);
#if defined(_WIN32)
    while (fscanf(cousinfp, "%ld %ld\n",
          &ctop_offsets[i][0], &ctop_offsets[i][1]) != EOF)
    i++;
#else
    while (fgets(szCouz, sizeof(szCouz), cousinfp) != EOF) {
        pszNum = strtok(szCouz, " \t");
        ctop_offsets[i][0] = atol(pszNum);
        pszNum = strtok(NULL, " \t");
        ctop_offsets[i][1] = atol(pszNum);
        i++;
        }
    ctop_offsets[i][0] = ctop_offsets[i][1] = 0;
#endif
#ifndef LABONLY
    done = 1;
#endif
    return;
}
    
static void add_relatives(IndexPtr idx, int rel1, int rel2)
{
    int i;
    struct relgrp *rel, *last, *r;

    /* First make sure that senses are not on the excpetion list */

    if (groupexc(idx->offset[rel1], idx->offset[rel2]))
    return;

    /* If either of the new relatives are already in a relative group,
       then add the other to the existing group (transitivity).
       Otherwise create a new group and add these 2 senses to it. */

    for (rel = rellist; rel; rel = rel->next) {
    if (rel->senses[rel1] == 1 || rel->senses[rel2] == 1) {
        rel->senses[rel1] = rel->senses[rel2] = 1;

        /* If part of another relative group, merge the groups */
        for (r = rellist; r; r = r->next) {
        if (r != rel &&
            (r->senses[rel1] == 1 || r->senses[rel2] == 1)) {
            for (i = 0; i < MAXSENSE; i++)
            rel->senses[i] |= r->senses[i];
        }
        }
        return;
    }
    last = rel;
    }
    rel = mkrellist();
    rel->senses[rel1] = rel->senses[rel2] = 1;
    if (rellist == NULL)
    rellist = rel;
    else
    last->next = rel;
}
    
static int groupexc(unsigned long off1, unsigned long off2)
{
    char buf[8], *p, linebuf[1024];

    sprintf(buf, "%8.8d", (off1 < off2 ? off1 : off2));

    if ((p = bin_search(buf, cousinexcfp)) != NULL) {
    sprintf(buf, "%8.8d", (off2 > off1 ? off2 : off1));
    strcpy(linebuf, p + 9); /* don't copy key */
    linebuf[strlen(linebuf) - 1] = '\0'; /* strip off newline */
    p = strtok(linebuf, " ");
    while (p && strcmp(p, buf))
        p = strtok(NULL, " ");
    }
    return(p ? 1 : 0);
}

static struct relgrp *mkrellist(void)
{
    struct relgrp *rel;
    int i;

    rel = (struct relgrp *) malloc(sizeof(struct relgrp));
    for (i = 0; i < MAXSENSE; i++)
    rel->senses[i] = 0;
    rel->next = NULL;
    return(rel);
}

static void free_rellist(void)
{
    struct relgrp *rel, *next;

    rel = rellist;
    while(rel) {
    next = rel->next;
    free(rel);
    rel = next;
    }
}

static void printrelatives(IndexPtr idx, int dbase)
{
    SynsetPtr synptr;
    struct relgrp *rel;
    int i, flag;
    int outsenses[MAXSENSE];

    for (i = 0; i < idx->off_cnt; i++)
    outsenses[i] = 0;
    prflag = 1;

    for (rel = rellist; rel; rel = rel->next) {
    flag = 0;
    for (i = 0; i < idx->off_cnt; i++) {
        if (rel->senses[i] && !outsenses[i]) {
        flag = 1;
        synptr = read_synset(dbase, idx->offset[i], "");
        printsense(synptr, i + 1);
        printsynset("", synptr, "\n", DEFON, ALLWORDS, SKIP_ANTS);
        traceptrs(synptr, HYPERPTR, dbase, 0);
        outsenses[i] = 1;
        free_synset(synptr);
        }
    }
    if (flag)
        printstr("--------------");
    }

    for (i = 0; i < idx->off_cnt; i++) {
    if (!outsenses[i]) {
        synptr = read_synset(dbase, idx->offset[i], "");
        printsense(synptr, i + 1);
        printsynset("", synptr, "\n", DEFON, ALLWORDS, SKIP_ANTS);
        traceptrs(synptr, HYPERPTR, dbase, 0);
        printstr("--------------");
        free_synset(synptr);
    }
    }
}


/*
  Search code interfaces to WordNet database

  findtheinfo() - print search results and return ptr to output buffer
  findtheinfo_ds() - return search results in linked list data structrure
*/

char *findtheinfo(char *searchstr, int dbase, int ptrtyp, int whichsense)
{
    register SynsetPtr cursyn;
    register IndexPtr idx = NULL;
    register int depth = 0;
    
    if (!searchbuffer) {    /* initialize search buffer */
    searchbuffer = (char *) malloc(SEARCHBUF * sizeof(char));
    assert(searchbuffer);
    }

    sprintf(searchbuffer,"");

    abortsearch = 0;        /* clear flag */

    if(ptrtyp < 0) {        /* negative type passed for recursive search */
    ptrtyp = -ptrtyp;
    depth = 1;
    }
    
    if(ptrtyp == FREQ) {
    if ((idx = getindex(searchstr, dbase)) == NULL)
        return(searchbuffer);
    SenseCount = idx->off_cnt; /* number of senses word has */
    freq_word(idx);
    } else if (ptrtyp == WNGREP) {
    wngrep(searchstr, dbase);
    } else if (ptrtyp == WNESCORT) {
    sprintf(searchbuffer,
        "Sentences that contain %s will be displayed in the Escort window.",
        searchstr);
    } else if (ptrtyp == RELATIVES) {
    if ((idx = getindex(searchstr, dbase)) == NULL)
        return(searchbuffer);
    relatives(idx, dbase);
    } else {
    if ((idx = getindex(searchstr, dbase)) == NULL)
        return(searchbuffer);

    OutSenseCount = 0;  /* keep track of # senses printed */
    SenseCount = idx->off_cnt; /* number of senses word has */

    /* Only bother with extra sense msgs if looking at all senses */
    if (whichsense == ALLSENSES) {
        sprintf(tmpbuf,
"                                                                         \n");
        printbuffer(tmpbuf);
    }
    
    /* Go through all of the searchword's senses in the
       database and perform the search requested. */

    for(sense = 0; sense < idx->off_cnt; sense++) {

        if (whichsense == ALLSENSES || whichsense == sense + 1) {

        prflag=0;
        cursyn=read_synset(dbase,idx->offset[sense],idx->wd);
        
        switch(ptrtyp) {
        case ANTPTR:
            if(dbase == ADJ)
            traceadjant(cursyn);
            else
            traceptrs(cursyn,ANTPTR,dbase,depth);
            break;
            
        case COORDS:
            tracecoords(cursyn,HYPOPTR,dbase,depth);
            break;

            
        case FRAMES:
            printframe(cursyn, 1);
            break;
            
        case MERONYM:
            traceptrs(cursyn,HASMEMBERPTR,dbase,depth);
            traceptrs(cursyn,HASSTUFFPTR,dbase,depth);
            traceptrs(cursyn,HASPARTPTR,dbase,depth);
            break;
            
        case HOLONYM:
            traceptrs(cursyn,ISMEMBERPTR,dbase,depth);
            traceptrs(cursyn,ISSTUFFPTR,dbase,depth);
            traceptrs(cursyn,ISPARTPTR,dbase,depth);
            break;
            
        case HMERONYM:
            partsall(cursyn, HMERONYM);
            break;
            
        case HHOLONYM:
            partsall(cursyn, HHOLONYM);
            break;
            
        case SEEALSOPTR:
            printseealso(cursyn, dbase);
            break;

#ifdef FOOP
        case PPLPTR:
            traceptrs(cursyn, ptrtyp, dbase, depth);
            traceptrs(cursyn, PPLPTR, dbase, depth);
            break;
#endif
            
        case SIMPTR:
        case SYNS:
        case HYPERPTR:
            printsense(cursyn, sense + 1);
            
            /* Don't print antonyms on a satellite synset */
            printsynset("", cursyn, "\n", DEFON, ALLWORDS,
                ((getsstype(cursyn->pos) == SATELLITE)
                 ? SKIP_ANTS : PRINT_ANTS));
            prflag = 1;
            
            traceptrs(cursyn,ptrtyp,dbase,depth);
            
            if (dbase == ADJ) {
            traceptrs(cursyn, PERTPTR, dbase, depth);
            traceptrs(cursyn, PPLPTR, dbase, depth);
            } else if (dbase == ADV) {
            traceptrs(cursyn, PERTPTR, dbase, depth);
            }

            if (saflag) /* print SEE ALSO pointers if requested */
            printseealso(cursyn, dbase);
            
            if (frflag)
            printframe(cursyn, 0);
            break;
            
        default:
            traceptrs(cursyn,ptrtyp,dbase,depth);
            break;
            
        }
        free_synset(cursyn);
        }

        if (abortsearch || (whichsense == sense + 1))
        break;
    }
    if (!abortsearch && whichsense == ALLSENSES)
        printbuffer("\n");

    /* patch in the number of senses output if looked at all senses */

    if (overflag) {
        sprintf(searchbuffer,
            "Search too large.  Narrow search and try again\n");
        overflag = 0;
    } else if (!abortsearch && whichsense == ALLSENSES) {
        if (OutSenseCount == idx->off_cnt) {
        if (OutSenseCount == 1)
            sprintf(tmpbuf, "1 sense of %s", idx->wd);
        else
            sprintf(tmpbuf, "%d senses of %s", OutSenseCount, idx->wd);
        } else
        sprintf(tmpbuf, "%d of %d senses of %s",
            OutSenseCount, idx->off_cnt, idx->wd);
        strncpy(searchbuffer, tmpbuf, strlen(tmpbuf));
    }
    }
    if (idx)
    free_index(idx);
    if (abortsearch) {
    sprintf(searchbuffer, "Search Aborted...");
    abortsearch = 0; /* clear flag */
    }
    
    return(strsubst(searchbuffer,'_',' '));
}

SynsetPtr findtheinfo_ds(char *searchstr, int dbase, int ptrtyp, int whichsense)
{
    IndexPtr idx;
    SynsetPtr cursyn;
    SynsetPtr synlist = NULL, lastsyn = NULL;
    int depth = 0;
    
    if ((idx = getindex(searchstr, dbase)) != NULL) {
    
    if(ptrtyp < 0) {
        ptrtyp = -ptrtyp;
        depth = 1;
    }

    SenseCount = idx->off_cnt; /* number of senses word has */
    
    /* Go through all of the searchword's senses in the
       database and perform the search requested. */
    
    for(sense = 0; sense < idx->off_cnt; sense++) {
        if (whichsense == ALLSENSES || whichsense == sense + 1) {
        cursyn=read_synset(dbase,idx->offset[sense],idx->wd);
        if (lastsyn)
            lastsyn->nextss = cursyn;
        if (!synlist)
            synlist = cursyn;
        
        cursyn->searchtype = ptrtyp;
        cursyn->ptrlist = traceptrs_ds(cursyn,ptrtyp,dbase,depth);
        
        lastsyn = cursyn;

        if (whichsense == sense + 1)
            break;
        }
    }      
    free_index(idx);
    }
    return(synlist);
}

/* Recursive search algorithm to trace a pointer tree and return results
   in linked list of data structures. */

SynsetPtr traceptrs_ds(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
{
    register int i;
    register SynsetPtr cursyn, synlist = NULL, lastsyn = NULL;
    
    /* If synset is a satellite, find the head word of its
       head synset and the head word's sense number. */

    if (getsstype(synptr->pos) == SATELLITE) {
    for (i = 0; i < synptr->ptrcount; i++)
        if (synptr->ptrtyp[i] == SIMPTR) {
        cursyn = read_synset(synptr->ppos[i],
                      synptr->ptroff[i],
                      "");
        synptr->headword = malloc(strlen(cursyn->words[0]) + 1);
        assert(synptr->headword);
        strcpy(synptr->headword, cursyn->words[0]);
        synptr->headsense = cursyn->wnsense[0];
        free_synset(cursyn);
        break;
        }
    }
        
    for(i=0;i<synptr->ptrcount;i++) {
    if((synptr->ptrtyp[i] == ptrtyp) &&
       ((synptr->pfrm[i] == 0) ||
        (synptr->pfrm[i] == synptr->whichword))) {
        
        cursyn=read_synset(synptr->ppos[i], synptr->ptroff[i], "");
        cursyn->searchtype = ptrtyp;

        if (lastsyn)
        lastsyn->nextss = cursyn;
        if (!synlist)
        synlist = cursyn;
        lastsyn = cursyn;

        if(depth) {
        depth = depthcheck(depth, cursyn);
        cursyn->ptrlist = traceptrs_ds(cursyn,ptrtyp,dbase,(depth+1));
        }   
    }
    }
    return(synlist);
}

/* Do requested search on synset passed, returning output in buffer. */

char *do_trace(SynsetPtr synptr, int ptrtyp, int dbase, int depth)
{
    if (!searchbuffer) {    /* initialize search buffer */
    searchbuffer = (char *) malloc(SEARCHBUF * sizeof(char));
    assert(searchbuffer);
    }
    traceptrs(synptr, ptrtyp, dbase, depth);
    return(searchbuffer);
}

/* Set bit for each search type that is valid for the search word
   passed and return bit mask. */
  
unsigned long is_defined(char *searchstr, int dbase)
{
    register IndexPtr index;
    register int i;
    unsigned long retval=0;
    
    if((index = getindex(searchstr, dbase)) != NULL) {

    SenseCount = index->off_cnt; /* number of senses word has */
    
    /* set bits that must be true for all words */
    
    retval = bit(SIMPTR) | bit(FREQ) | bit(SYNS)| bit(WNGREP);

    for(i = 0; i < index->ptruse_cnt; i++) {
        
        retval |= bit(index->ptruse[i]); /* set bit for this ptr */

        if (dbase == NOUN) {

        /* set generic HOLONYM and/or MERONYM bit if necessary */
        
        if(index->ptruse[i] >= ISMEMBERPTR &&
           index->ptruse[i] <= ISPARTPTR)
            retval |= bit(HOLONYM);
        else if(index->ptruse[i] >= HASMEMBERPTR &&
            index->ptruse[i] <= HASPARTPTR)
            retval |= bit(MERONYM);
        } else if (dbase == ADJ && index->ptruse[i] == SIMPTR)
        retval |= bit(ANTPTR);
    }

    if (dbase == NOUN) {
        retval |= bit(RELATIVES);
        if (HasHoloMero(index, HMERONYM))
        retval |= bit(HMERONYM);
        if (HasHoloMero(index, HHOLONYM))
        retval |= bit(HHOLONYM);
        if (retval & bit(HYPERPTR))
        retval |= bit(COORDS);
    }   
    free_index(index);
    }
    return(retval);
}

/* Set bit for each POS that search word is in.  0 returned if
   word is not in WordNet. */

unsigned int in_wn(char *word, int pos)
{
    int i;
    unsigned int retval = 0;

    if (pos == ALL_POS) {
    for (i = 1; i < NUMPARTS + 1; i++)
        if (indexfps[i] != NULL && bin_search(word, indexfps[i]) != NULL)
        retval |= bit(i);
    } else if (indexfps[pos] != NULL && bin_search(word,indexfps[pos]) != NULL)
        retval |= bit(pos);
    return(retval);
}

static int depthcheck(int depth, SynsetPtr synptr)
{
    if (stopsearch())       /* check flag before proceeding */
    return(-1);
    if(depth >= MAXDEPTH) {
        DisplayError("Error Cycle detected\n");
        DisplayError("   %s\n", synptr->words[0]);
        depth = -1;     /* reset to get one more trace then quit */
    }
    return(depth);
}

/* Strip off () enclosed comments from a word */

static char *deadjify(char *word)
{
    register char *y;
    
    adj_marker = UNKNOWN_MARKER; /* default if not adj or unknown */
    
    y=word;
    while(*y) {
    if(*y == '(') {
        if (!strncmp(y, "(a)", 3))
        adj_marker = ATTRIBUTIVE;
        else if (!strncmp(y, "(ip)", 4))
        adj_marker = IMMED_POSTNOMINAL;
        else if (!strncmp(y, "(p)", 3))
        adj_marker = PREDICATIVE;
        *y='\0';
    } else 
        y++;
    }
    return(word);
}

static int getsearchsense(SynsetPtr synptr, int whichword)
{
    register IndexPtr idx;
    register int i;

    strsubst(strcpy(wdbuf, synptr->words[whichword - 1]), '_', ' ');
    strtolower(wdbuf);
               
    if (idx = index_lookup(wdbuf, getpos(synptr->pos))) {
    for (i = 0; i < idx->off_cnt; i++)
        if (idx->offset[i] == synptr->hereiam) {
        free_index(idx);
        return(i + 1);
        }
    free_index(idx);
    }
    return(0);
}

static void printsynset(char *head, SynsetPtr synptr, char *tail, int definition, int wdnum, int antflag)
{
    register int i, wdcnt;
    char *str;
    char tbuf[SMLINEBUF];
    
    sprintf(tbuf,"");
    
#if 1
    bAddSense = getenv("WN_ADDSENSE");
#endif

    if((str = getenv("WNDEBUG")) && *str != '\0') {
#if 1
	    bDebug = 1;
#endif
    sprintf(tbuf,"<%s>", wn_filenames[synptr->fnum]);
    prsense = 1;
    } else
    prsense = 0;
    
    strcat(tbuf,head);

    if (wdnum)          /* print only specific word asked for */
    printword(tbuf, synptr, wdnum - 1, PRINT_MARKER, antflag);
    else            /* print all words in synset */
    for(i=0, wdcnt = synptr->wcount; i < wdcnt; i++) {
        printword(tbuf, synptr, i, PRINT_MARKER, antflag);
        if (i < wdcnt - 1)
        strcat(tbuf, ", ");
    }
    
    if(definition && dflag && synptr->defn) {
    strcat(tbuf," -- ");
    strcat(tbuf,synptr->defn);
    }
    
    strcat(tbuf,tail);
    printbuffer(tbuf);
}

static void printantsynset(SynsetPtr synptr, char *tail, int anttype, int definition)
{
    register int i, wdcnt;
    char *str;
    char tbuf[SMLINEBUF];
    int first = 1;
    
    sprintf(tbuf,"");
    
    if((str = getenv("WNDEBUG")) && *str != '\0') {
    sprintf(tbuf,"<%s>", wn_filenames[synptr->fnum]);
    prsense = 1;
    } else
    prsense = 0;
    
    /* print anotnyms from cluster head (of indirect ant) */
    
    strcat(tbuf, "INDIRECT (VIA ");
    for(i=0, wdcnt = synptr->wcount; i < wdcnt; i++) {
    if (first) {
        str = printant(ADJ, synptr, i + 1, "%s", ", ");
        first = 0;
    } else
        str = printant(ADJ, synptr, i + 1, ", %s", ", ");
    if (*str)
        strcat(tbuf, str);
    }
    strcat(tbuf, ") -> ");
    
    /* now print synonyms from cluster head (of indirect ant) */
    
    for (i = 0, wdcnt = synptr->wcount; i < wdcnt; i++) {
    printword(tbuf, synptr, i, SKIP_MARKER, SKIP_ANTS);
    if (i < wdcnt - 1)
        strcat(tbuf, ", ");
    }
    
    if(dflag && synptr->defn && definition) {
    strcat(tbuf," -- ");
    strcat(tbuf,synptr->defn);
    }
    
    strcat(tbuf,tail);
    printbuffer(tbuf);
}

static void printword(char *buf, SynsetPtr synptr, int wdnum, int adjmarker, int antflag)
{
    int pos;
    static char vs[] = " (vs. %s)";

    /* copy the word (since deadjify() changes original string),
       deadjify() the copy and append to buffer.  then append
       adjective marker string if there is one. */
    
    strcpy(wdbuf, synptr->words[wdnum]);
    strcat(buf, deadjify(wdbuf));
    
    /* if debug is set, add WordNet sense number if non-0 */
    
    if (prsense && (synptr->wnsense[wdnum] != 0))
    sprintf(buf + strlen(buf), "%d", synptr->wnsense[wdnum]);
    
    pos = getpos(synptr->pos);

    /* append adjective marker if present */

    if (pos == ADJ) {
    if (adjmarker)
        if (adj_marker != UNKNOWN_MARKER) /* set in deadjify() */
        strcat(buf, markers[adj_marker]);
    if (antflag == PRINT_ANTS)
        strcat(buf,
           printant(getpos(synptr->pos), synptr, wdnum + 1, vs, ""));
    }

#if 1
    if (bAddSense) {
	sprintf(buf + strlen(buf), "#%d", getsearchsense(synptr, 1 + wdnum));
	}
#endif
}

static char *printant(int dbase, SynsetPtr synptr, int wdnum, char *template, char *tail)
{
    register int i, j, wdoff;
    register SynsetPtr psynptr;
    char tbuf[WORDBUF];
    static char retbuf[SMLINEBUF];
    int first = 1;
    
    retbuf[0] = '\0';
    
    /* Go through all the pointers looking for anotnyms from the word
       indicated by wdnum.  When found, print all the antonym's
       antonym pointers which point back to wdnum. */
    
    for (i = 0; i < synptr->ptrcount; i++) {
    if (synptr->ptrtyp[i] == ANTPTR && synptr->pfrm[i] == wdnum) {

        psynptr = read_synset(dbase, synptr->ptroff[i], "");

        for (j = 0; j < psynptr->ptrcount; j++) {
        if (psynptr->ptrtyp[j] == ANTPTR &&
            psynptr->pto[j] == wdnum &&
            psynptr->ptroff[j] == synptr->hereiam) {

            wdoff = (psynptr->pfrm[j]) ? psynptr->pfrm[j] - 1 : 0;
            strcpy(wdbuf, psynptr->words[wdoff]);
            if (prsense && (psynptr->wnsense[wdoff] != 0))
            sprintf(tbuf, "%s%d", deadjify(wdbuf),
                psynptr->wnsense[wdoff]);
            else
            sprintf(tbuf, "%s", deadjify(wdbuf));

            if (!first)
            strcat(retbuf, tail);
            else
            first = 0;
            sprintf(retbuf + strlen(retbuf), template, tbuf);
        }
        }
        free_synset(psynptr);
    }
    }
    return(retbuf);
}

static void printbuffer(char *string)
{
    register long delta;

    if (overflag)
    return;
    delta = (searchbufsize - strlen(searchbuffer)) - strlen(string);
    if(delta <= 0) {
    delta = -delta;
    if ((searchbufsize + (delta + SEARCHBUF)) > MAXSEARCHBUF) {
        overflag = 1;
        return;
    }
    searchbufsize += (delta + SEARCHBUF);
    searchbuffer = realloc(searchbuffer, searchbufsize);
    assert(searchbuffer);
    }
    strcat(searchbuffer,string);
}

static void printstr(char *str)
{
    sprintf(tmpbuf,"%s\n", str);
    printbuffer(tmpbuf);
}

static void printsense(SynsetPtr synptr, int sense)
{
    sprintf(tmpbuf,"\nSense %d", sense);

    /* Append lexicographer filename after Sense # if flag is set. */

    if (fnflag)
    sprintf(tmpbuf + strlen(tmpbuf)," in file \"%s\"",
        wn_filenames[synptr->fnum]);
    strcat(tmpbuf, "\n");
    printbuffer(tmpbuf);
    OutSenseCount++;
}

static void printspaces(int trace, int depth)
{
    int j;

    for (j = 0; j < depth; j++)
    printbuffer("    ");

    switch(trace) {
    case TRACEP:        /* traceptrs() */
    if (depth)
        printbuffer("   ");
    else
        printbuffer("       ");
    break;

    case TRACEC:        /* tracecoords() */
    if (!depth)
        printbuffer("    ");
    break;

    case TRACEI:            /* traceinherit() */
    if (!depth)
        printbuffer("\n    ");
    break;
    }
}

/*
  Revision log:
  
  $Log: search.c,v $
 * Revision 1.94  1995/02/16  18:13:10  wn
 * change MAXSENSES to MAXSENSE
 *
 * Revision 1.93  1995/02/14  15:01:06  wn
 * fixed msg in wngrep if can't open file
 *
 * Revision 1.92  1995/02/07  18:14:37  wn
 * *** empty log message ***
 *
 * Revision 1.91  1995/02/07  16:16:25  wn
 * cleanup, added prototyes, new findsisters functin
 *
 * Revision 1.90  1995/01/17  21:30:36  wn
 * added prototypes
 *
 * Revision 1.89  1995/01/17  20:56:13  wn
 * added prototype for in_wn()
 *
 * Revision 1.88  1995/01/12  15:52:56  wn
 * cleaned up forward declarations for gcc
 *
 * Revision 1.87  1995/01/06  21:24:50  wn
 * added code to test for stopsearch
 * d
 *
 * Revision 1.86  1995/01/04  20:49:29  wn
 * fixed some cousin bugs
 *
 * Revision 1.85  1994/12/15  18:42:13  wn
 * MAC stuff added.
 *
 * Revision 1.84  1994/12/01  22:01:55  wn
 * typo in call to trace_cousin
 *
 * Revision 1.83  1994/11/30  16:50:48  wn
 * made read_cousintops and trace_cousins external functions
 *
 * Revision 1.82  1994/11/30  16:12:40  wn
 * added cousins nd fixed bugs
 *
 * Revision 1.81  1994/10/14  16:10:39  wn
 * *** empty log message ***
 *
 * Revision 1.80  1994/10/07  14:39:35  wn
 * broke read_synset into read_synset and parse_synset
 *
 * Revision 1.79  1994/10/07  14:33:22  wn
 * added code for PPLPTR
 *
 * Revision 1.78  1994/09/27  14:00:16  wn
 * changed wn_filenames to 1 dimensional array
 *
 * Revision 1.77  1994/06/15  20:52:06  wn
 * fixed bug in free_syns
 *
 * Revision 1.76  1993/12/22  14:23:44  wn
 * added WNESCORT
 *
 * Revision 1.74  93/07/29  13:46:08  wn
 * *** empty log message ***
 * 
 * Revision 1.73  93/07/29  13:41:56  wn
 * *** empty log message ***
 * 
 * Revision 1.72  93/07/28  16:32:32  wn
 * added re_wninit
 * 
 * Revision 1.71  93/07/08  15:44:03  wn
 * cleanups for 1.4
 * 
 * Revision 1.70  93/07/01  17:10:27  wn
 * removed all exit() statements - converted to return(NULL)
 * 
 * Revision 1.69  93/06/09  14:53:51  wn
 * changed wninit to open fps, wrote in_wn()
 * 
 * Revision 1.68  93/06/08  15:23:06  wn
 * added code to getindex to strip '.' from strings before lookup
 * 
 * Revision 1.67  93/06/08  14:31:36  wn
 * no changes
 * 
 * Revision 1.66  93/04/22  11:58:51  wn
 * no changes
 * 
 * Revision 1.65  93/04/13  13:47:57  wn
 * made free code global
 * 
 * Revision 1.64  93/04/01  10:52:56  wn
 * added MACH #ifdefs and changed free_synset() call to free_syns()
 * in free_synset() ala marti hearst
 * 
 * Revision 1.63  93/03/02  16:22:02  wn
 * fixed bug in read_synset
 * 
 * Revision 1.62  93/01/26  13:55:52  wn
 * make getindex external instead of static
 * 
 * 
 */
