/*

  wnutil.c - utility functions used by WordNet code

*/

#include <stdio.h>
#include <ctype.h>

#include "wnconsts.h"       /*So that sys-type is defined.*/ 

#ifdef MAC
/* Needed for strtok() */
#include "macfns.h"
#include <string.h>
/* Needed for atoi() */
#include <stdlib.h>
#else
#include <string.h>
#endif
#ifndef MAC
#include <stdlib.h>
#include <assert.h>
#endif
#ifdef unix
#ifdef MACH
/* don't include <malloc.h> for MACH */
#else
#ifndef NEXT
#include <malloc.h>
#endif
#endif
#endif
#include "wn.h"
#include "wnglobals.h"
#include "license.h"

#include "myerror.h"

static int do_init();

/* Initialization functions */

static void readfnames(), closefps();

static int done = 0;
static int openerr = 0;

int wninit(void)
{

    if (done)
    return(openerr);

    openerr = do_init();
    if (!openerr) { 
    done = 1;
    OpenDB = 1;
    readfnames();
    openerr = morphinit();
    }

    return(openerr);
}

int re_wninit(void)
{
    //int openerr;

    closefps();

    openerr = do_init();
    if (!openerr) {
    OpenDB = 1;
    readfnames();
    openerr = re_morphinit();
    }

    return(openerr);
}

static void closefps(void) 
{
    int i;

    for (i = 1; i < NUMPARTS + 1; i++) {
    fclose(datafps[i]);
    fclose(indexfps[i]);
    }
	if (sensefp)
    	fclose(sensefp);
    fclose(cousinfp);
    fclose(cousinexcfp);
    done = 0;
}

#if 1
int wnclose (void)
{
    closefps();
	openerr = 0;
	done = 0;
	morphclose();
    OpenDB = 0;
}
#endif

static int do_init(void)
{
    int i, openerr;
    char searchdir[256], tmpbuf[256];
    char *env;
 
    openerr = 0;
    strcpy(searchdir,((env = getenv("WNSEARCHDIR")) ? env : DEFAULTPATH));

    for (i = 1; i < NUMPARTS + 1; i++) {
    sprintf(tmpbuf, DATAFILE, searchdir, partnames[i]);
    if((datafps[i] = fopen(tmpbuf, "r")) == NULL) {
        DisplayError("Error: can't open datafile(%s)\n",tmpbuf);
        openerr = -1;
    }
    sprintf(tmpbuf, INDEXFILE, searchdir, partnames[i]);
    if((indexfps[i] = fopen(tmpbuf, "r")) == NULL) {
        DisplayError("Error: can't open indexfile(%s)\n",tmpbuf);
        openerr = -1;
    }
    }

    sprintf(tmpbuf, SENSEIDXFILE, searchdir);
    sensefp = fopen(tmpbuf, "r");

    sprintf(tmpbuf, COUSINFILE, searchdir);
    if ((cousinfp = fopen(tmpbuf, "r")) == NULL) {
    DisplayError("Error: can't open cousin tops file(%s)\n", tmpbuf);
    openerr = -1;
    }
    sprintf(tmpbuf, COUSINEXCFILE, searchdir);
    if ((cousinexcfp = fopen(tmpbuf, "r")) == NULL) {
    DisplayError("Error: can't open exception file(%s)\n", tmpbuf);
    openerr = -1;
    }
    return(openerr);
}

static void readfnames(void)
{
    FILE *fp;
    register int i = 0, len;
    char line[SMLINEBUF];
    char c;
    char *tmpptrs[100];     /* place to hold ptrs to fnames */

    fp = datafps[NOUN];     /* just get from noun file */
    fseek(fp, (long)DBLICENSE_SIZE - 1, 0); /* skip the license */

    while((c = fgetc(fp)) != '0') {
    ungetc(c, fp);
    fgets(line, SMLINEBUF, fp);
    len = 0;
    while (line[len++] != ' ');
    tmpptrs[i] = malloc(len);
    assert(tmpptrs[i]);
    strncpy(tmpptrs[i], line, len - 1);
    tmpptrs[i][len - 1] = '\0'; /* append null */
    i++;
    }

    ungetc(c, fp);
    datastart = ftell(fp);
    wn_filenames = (char **)malloc(i * sizeof(char *));
    assert(wn_filenames);

    while (i--)
    wn_filenames[i] = tmpptrs[i];
    return;
}

/* Count the number of underscore or space separated words in a string. */

int cntwords(char *s, char separator)
{
    register int wdcnt = 0;

    while (*s) {
    if (*s == separator || *s == ' ' || *s == '_') {
        wdcnt++;
        while (*s && (*s == separator || *s == ' ' || *s == '_'))
        s++;
    } else
        s++;
    }
    return(++wdcnt);
}

/* Convert string to lower case remove trailing adjective marker if found */

char *strtolower(char *word)
{
    register char *y;

    y=word;
    while(*y){
    if(isupper(*y))
        *y=tolower(*y);
    if(*y == '(')
        *y='\0';
    else
        y++;
    }
    return(word);
}

/* Convert string passed to lower case */

char *ToLowerCase(char *str)
{
    register char *s = str;

    while(*s != '\0') {
    if(*s >= 'A' && *s <= 'Z')
        *s += 32;
    s++;
    }
    return(str);
}

/* Replace all occurences of 'from' with 'to' in 'str' */

char *strsubst(char *str, char from, char to)
{
    register char *p;

    for (p = str; *p != 0; ++p)
    if (*p == from)
        *p = to;
    return str;
}

/* Return pointer code for pointer type characer passed. */

int getptrtype(char *ptrstr)
{
    register int i;

    for(i = 1; i <= MAXSEARCH; i++)
    if(!strcmp(ptrstr, ptrtyp[i]))
        return(i);
}

/* Return part of speech code for string passed */

int getpos(char *s)
{
    switch (*s) {
    case 'n':
    return(NOUN);
    case 'a':
    case 's':
    return(ADJ);
    case 'v':
    return(VERB);
    case 'r':
    return(ADV);
    default:
    DisplayError("Error: unknown part of speech %s\n", s);
#if FALSE
    exit(-1);
#else
    return (-1);
#endif
    }
}

/* Return synset type code for string passed. */

int getsstype(char *s)
{
    switch (*s) {
    case 'n':
    return(NOUN);
    case 'a':
    return(ADJ);
    case 'v':
    return(VERB);
    case 's':
    return(SATELLITE);
    case 'r':
    return(ADV);
    default:
    DisplayError("Error: unknown synset type %s\n", s);
#if FALSE
    exit(-1);
#else
    return (-1);
#endif
    }
}

/* Pass in string for POS, return corresponding integer value */

int StrToPos(char *str)
{
    if (!strcmp(str, "noun"))
    return(NOUN);
    else if (!strcmp(str, "verb"))
    return(VERB);
    else if (!strcmp(str, "adj"))
    return(ADJ);
    else if (!strcmp(str, "adv"))
    return(ADV);
    else {
    return(-1);
    }
}

#define MAX_TRIES   5

/* Find string for 'searchstr' as it is in index file */

char *GetWNStr(char *searchstr, int dbase)
{
    register int i, j, k, offset = 0;
    register char c;
    char *underscore = NULL, *hyphen = NULL, *period = NULL;
    static char strings[MAX_TRIES][WORDBUF];
    
    ToLowerCase(searchstr);

    if (!(underscore = strchr(searchstr, '_')) &&
    !(hyphen = strchr(searchstr, '-')) &&
    !(period = strchr(searchstr, '.')))
    return (strcpy(strings[0],searchstr));

    for(i = 0; i < 3; i++)
    strcpy(strings[i], searchstr);
    if (underscore != NULL) strsubst(strings[1], '_', '-');
    if (hyphen != NULL) strsubst(strings[2], '-', '_');
    for(i = j = k = 0; (c = searchstr[i]) != '\0'; i++){
    if(c != '_' && c != '-') strings[3][j++] = c;
    if(c != '.') strings[4][k++] = c;
    }
    strings[3][j] = '\0';
    strings[4][k] = '\0';
    
    for(i = 1; i < MAX_TRIES; i++)
    if(strcmp(strings[0], strings[i]) == 0) strings[i][0] = '\0';
    
    for (i = (MAX_TRIES - 1); i >= 0; i--)
    if (strings[i][0] != '\0')
        if (bin_search(strings[i], indexfps[dbase]) != NULL)
        offset = i;
    
    return(strings[offset]);
}

/* Return synset for sense key passed. */

SynsetPtr GetSynsetForSense(char *sense)
{
    long offset;

    /* Pass in sense key and return parsed sysnet structure */

    if ((offset = GetDataOffset(sense)))
    return(read_synset(GetPOS(sense),
               offset,
               GetWORD(sense)));
    else
    return(NULL);
}

/* Find offset of sense key in data file */

long GetDataOffset(char *sense)
{
    char *line;
    char senseidx[256], *env;

    /* Pass in encoded sense string, return byte offset of corresponding
       synset in data file. */

    if (sensefp == NULL) {
    DisplayError("Error: Sense index file not open\n");
    return(0L);
    }
    line = bin_search(sense, sensefp);
    if (line) {
    while (*line++ != ' ');
    return(atol(line));
    } else
    return(0L);
}

/* Find polysemy (collins) count for sense key passed. */

int GetPolyCount(char *sense)
{
    IndexPtr idx;
    int sense_cnt;

    /* Pass in encoded sense string and return polysemy count
       for word in corresponding POS */

    idx = index_lookup(GetWORD(sense), GetPOS(sense));
    sense_cnt = idx->sense_cnt;
    free_index(idx);
    return(sense_cnt);
}

/* Return word part of sense key */
char *GetWORD(char *s)
{
    static char word[100];
    int i=0;

    /* Pass in encoded sense string and return WORD */

    while ((word[i++] = *s++) != '%');
    word[i - 1] = '\0';
    return(word);
}

/* Return POS code for sense key passed. */

int GetPOS(char *s)
{
    int pos;

    /* Pass in encoded sense string and return POS */

    while (*s++ != '%');    /* skip over WORD */
#if defined(_WIN32)
    sscanf(s, "%1d", &pos);
#else
    pos = atoi(s);
#endif
    return(pos == SATELLITE ? ADJ : pos);
}

//#if TRUE
int GetPartOfSpeach (char *s)
{
    int n = 666;
    int m = GetPOS(s);
    n *= (int)s;
    return (m);
}
//#endif



/* Reconstruct synset from synset pointer and return ptr to buffer */

char *FmtSynset(SynsetPtr synptr, int defn)
{
    int i;
    static char synset[512];

    strcpy(synset, "{ ");
    for (i = 0; i < (synptr->wcount - 1); i++) {
    strcat(synset, synptr->words[i]);
    strcat(synset, ", ");
    }
    strcat(synset, synptr->words[i]);
    if (defn && synptr->defn) {
    strcat(synset, " (");
    strcat(synset, synptr->defn);
    strcat(synset, ") ");
    }
    strcat(synset, " }");
    return(synset);
}

/* Convert WordNet sense number passed of IndexPtr entry to sense key. */
char *WNSnsToStr(IndexPtr idx, int sense)
{
    SynsetPtr sptr, adjss;
    char snsstring[512], lowerword[256];
    int j, sstype, pos;

    pos = getpos(idx->pos);
    sptr = read_synset(pos, idx->offset[sense - 1], "");

    if ((sstype = getsstype(sptr->pos)) == SATELLITE) {
    for (j = 0; j < sptr->ptrcount; j++) {
        if (sptr->ptrtyp[j] == SIMPTR) {
        adjss = read_synset(sptr->ppos[j],sptr->ptroff[j],"");
        sptr->headword = malloc (strlen(adjss->words[0]) + 1);
        if(!sptr->headword) {
            free_synset(sptr);
            return(NULL);
        }
        strcpy(sptr->headword, adjss->words[0]);
        sptr->headsense = adjss->wnsense[0];
        free_synset(adjss);
        break;
        }
    }
    }

    for (j = 0; j < sptr->wcount; j++) {
    strcpy(lowerword, sptr->words[j]);
    strtolower(lowerword);
    if(!strcmp(lowerword, idx->wd))
        break;
    }

    if (j == sptr->wcount) {
    free_synset(sptr);
    return(NULL);
    }

    if (sstype == SATELLITE) 
    sprintf(snsstring,"%s%%%-1.1d:%-2.2d:%-2.2d:%s:%-2.2d",
        idx->wd, SATELLITE, sptr->fnum,
        sptr->wnsense[j], sptr->headword,sptr->headsense);
    else 
    sprintf(snsstring,"%s%%%-1.1d:%-2.2d:%-2.2d::",
        idx->wd, pos, sptr->fnum, sptr->wnsense[j]);

    free_synset(sptr);
    return(strdup(snsstring));
}

/* Search for string and/or baseform of word in database and return
   index structure for word if found in database. */

IndexPtr GetValidIndexPointer(char *word, int pos)
{
    IndexPtr idx = NULL;
    char *morphword;

    idx = getindex(word, pos);

    if(!idx) {
    if((morphword = morphstr(word, pos)) != NULL)
        while (morphword) {
        idx = getindex(morphword, pos);
        if(idx) break;
        morphword = morphstr(NULL, pos);
        }
    }
    return (idx);
}

#ifdef FOOP
int GetFileNum(char *string)
{
    char *head = string;

    while (*string != ' ') string++;
    head = string++;
    while (*string != ' ') string++;
    *string = '\0';
    return (atoi(head));
}

int GetNextSense(char *string)
{
    char *copy , *head = (char *) malloc (strlen(string)+1);
    int answer;

    if(!head) return(-1);
    copy = head;
    while(*string != '\0' && !isdigit(*string)) string++;
    while(*string != '\0' && isdigit(*string)) *copy++ = *string++;
    *copy = '\0';
    answer = atoi(head);
    free(head);
    return(answer);
}   
#endif

#ifdef NEXT
char *strdup(char *s)
{
    char *t;

    t = (char *) malloc (strlen(s) +1);
    if(t) strcpy(t, s);
    return(t);
}
#endif

/* Return sense number in database for word and lexsn passed. */

int GetWNSense(char *word, char *lexsn)
{
    char *idxbuf;
    char buf[256];
    char key[256], loc[8];
    int wnsense;

    sprintf(buf, "%s%%%s", word, lexsn);
    if ((idxbuf = bin_search(buf, sensefp)) != NULL) {
#if defined(_WIN32)
    sscanf(idxbuf, "%s %s %d\n", key, loc, &wnsense);
#else
    char* pszPos;
    pszPos = strtok(idxbuf, " \t");
    if (pszPos) strcpy(key, pszPos);
    pszPos = strtok(NULL, " \t");
    if (pszPos) strcpy(loc, pszPos);
    pszPos = strtok(NULL, " \t");
    if (pszPos) wnsense = atoi(pszPos);
#endif
    return(wnsense);
    } else 
    return(0);
}
