/* 

  wn.c - Command line interface to WordNet

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "wn.h"
#include "wnconsts.h"
#include "wnglobals.h"
#include "wnhelp.h"
#include "license.h"

static char *Id = "$Id: wn.c,v 1.50 1995/03/03 20:06:38 wn Exp $";

static struct {
    char *option;
    int search;
    int pos;
    int errmsg;
    char *label;
} optlist [] = {
    { "-synsa", SIMPTR, ADJ, 0, "Similarity" },
    { "-antsa", ANTPTR, ADJ, 1, "Antonyms" },
    { "-perta", PERTPTR, ADJ, 0, "Pertainyms" },
    { "-famla", FREQ, ADJ, 3, "Familiarity" },
    { "-attra", ATTRIBUTE, ADJ, 2, "Attributes" },
    { "-grepa", WNGREP, ADJ, 4, "Grep" },
    { "-antsn", ANTPTR, NOUN, 2, "Antonyms" },
    { "-hypen", -HYPERPTR, NOUN, 4, "Synonyms/Hypernyms (Ordered by Frequency)" },
    { "-hypon", HYPOPTR, NOUN, 5, "Hyponyms" },
    { "-meron", MERONYM, NOUN, 9, "Meronyms" },
    { "-holon", HOLONYM, NOUN, 7, "Holonyms" },
    { "-treen", -HYPOPTR, NOUN, 6, "Hyponyms" },
    { "-famln", FREQ, NOUN, 12, "Familiarity" },
    { "-synsn", HYPERPTR, NOUN, 0, "Synonyms/Hypernyms (Ordered by Frequency)" },
    { "-simsn", RELATIVES, NOUN, 1, "Synonyms (Grouped by Similarity of Meaning)" },
    { "-subsn", HASSTUFFPTR, NOUN, 9, "Substance Meronyms" },
    { "-partn", HASPARTPTR, NOUN, 9, "Part Meronyms" },
    { "-membn", HASMEMBERPTR, NOUN, 9, "Member Meronyms" },
    { "-sprtn", ISPARTPTR, NOUN, 7, "Part Holonyms" },
    { "-smemn", ISMEMBERPTR, NOUN, 7, "Member Holonyms" },
    { "-ssubn", ISSTUFFPTR, NOUN, 7, "Substance Holonyms" },
    { "-hmern", -HMERONYM, NOUN, 10, "Meronyms" },
    { "-hholn", -HHOLONYM, NOUN, 8, "Holonyms" },
    { "-coorn", COORDS, NOUN, 3, "Coordinate Terms" },
    { "-attrn", ATTRIBUTE, NOUN, 11, "Attributes" },
    { "-grepn", WNGREP, NOUN, 13, "Grep" },
    { "-antsv", ANTPTR, VERB, 1, "Anotnyms" },
    { "-synsv", HYPERPTR, VERB, 0, "Synonyms/Hypernyms (Ordered by Frequency)" },
    { "-famlv", FREQ, VERB, 7, "Familiarity" },
    { "-hypev", -HYPERPTR, VERB, 2, "Synonyms/Hypernyms (Ordered by Frequency)" },
    { "-treev", -HYPOPTR, VERB, 3, "Hyponyms" },
    { "-hypov", HYPOPTR, VERB, 3, "Hyponyms" },
    { "-entav", ENTAILPTR, VERB, 4, "Entailment" },
    { "-framv", FRAMES, VERB, 6, "Sample Sentences" },
    { "-causv", CAUSETO, VERB, 5, "\'Cause To\'" },
    { "-grepv", WNGREP, VERB, 8, "Grep" },
    { "-synsr", SYNS, ADV, 0, "Synonyms" },
    { "-antsr", ANTPTR, ADV, 1, "Anotnyms" },
    { "-pertr", PERTPTR, ADV, 0, "Pertainyms" },
    { "-famlr", FREQ, ADV, 2, "Familiarity" },
    { "-grepr", WNGREP, ADV, 3, "Grep" },
    { NULL, 0, 0, 0, NULL }
};

struct {
    char *template;
    char *option;
    char *helpstr;
} searchstr[] = {
    { NULL, NULL, NULL },
    { "-ants%c",    "-ants{n|v|a|r}",   "\t\tAntonyms", },
    { "-hype%c",    "-hype{n|v}",   "\t\tHypernyms", },
    { "-hypo%c, -tree%c",   "-hypo{n|v}, -tree{n|v}",
      "\tHyponyms & Hyponym Tree", },
    { "-enta%c",    "-entav",   "\t\t\tVerb Entailment", },
    { "-syns%c",    "-syns{n|v|a|r}",   "\t\tSynonyms (ordered by frequency)", },
    { "-smem%c",    "-smemn",   "\t\t\tMember of Holonyms", },
    { "-ssub%c",    "-ssubn",   "\t\t\tSubstance of Holonyms", },
    { "-sprt%c",    "-sprtn",   "\t\t\tPart of Holonyms", },
    { "-memb%c",    "-membn",   "\t\t\tHas Member Meronyms", },
    { "-subs%c",    "-subsn",   "\t\t\tHas Substance Meronyms", },
    { "-part%c",    "-partn",   "\t\t\tHas Part Meronyms", },
    { "-mero%c",    "-meron",   "\t\t\tAll Meronyms", },
    { "-holo%c",    "-holon",   "\t\t\tAll Holonyms", },
    { "-caus%c",    "-causv",   "\t\t\tVerb Cause to", },
    { NULL, NULL, NULL },   /* verb caused by - not used */
    { NULL, NULL, NULL },   /* see also - no specific search */
    { "-pert%c",    "-pert{a|r}",   "\t\tPertainyms", },
    { "-attr%c",    "-attr{n|a}",   "\t\tAttributes", },
    { NULL, NULL, NULL },   /* syns - taken care of w/SIMPTR */
    { "-faml%c",    "-faml{n|v|a|r}",   "\t\tFamiliarity & Polysemy Count", },
    { "-fram%c",    "-framv",   "\t\t\tVerb Frames", },
    { "-coor%c",    "-coorn",   "\t\t\tCoordinate Sisters", },
    { "-sims%c",    "-simsn",   "\t\t\tSynonyms (grouped by similarity of meaning)", },
    { "-hmer%c",    "-hmern",   "\t\t\tHierarchical Meronyms", },
    { "-hhol%c",    "-hholn",   "\t\t\tHierarchical Holonyms" },
    { NULL, NULL, NULL },   /* wnescort - used by xwn only */
    { "-grep%c",    "-grep{n|v|a|r}",   "\t\tList of Compound Words" },
};

static int do_search(int, char *[]);
static int do_is_defined(char *);
static void printsearches(char *, int, unsigned long);

main(int argc,char *argv[])
{
    int i;
    
    if(argc < 2) {
    fprintf(stdout,
"\nusage: wn word [-hgla] [-n#] -searchtype [-searchtype...]\n");
    fprintf(stdout, "       wn [-l]\n\n");
    fprintf(stdout, "\t-h\t\tDisplay help text\n");
    fprintf(stdout, "\t-g\t\tDisplay optional gloss\n");
    fprintf(stdout, "\t-l\t\tDisplay license and copyright notice\n");
    fprintf(stdout, "\t-a\t\tDisplay lexicographer file information\n");
    fprintf(stdout, "\t-n#\t\tSearch only sense number #\n");
    fprintf(stdout,"\nsearchtype is one of the following:\n");
    for (i = 1; i <= MAXSEARCH; i++)
        if (searchstr[i].option)
        fprintf(stdout, "\t%s%s\n",
            searchstr[i].option, searchstr[i].helpstr);
    exit(-1);
    }

    if (wninit()) {
    fprintf(stdout, "wn: error - cannot open WordNet database\n");
    exit (-1);
    }

    if (argc == 2) {
    if (!strcmp("-l", argv[1])) {
        printf("WordNet Release %s\n\n", wnrelease);
        printf(license);
    } else
        exit(do_is_defined(argv[1]));
    } else
    exit(do_search(argc, argv));
}

static int do_search(int ac, char *av[])
{
    int i;
    int j = 1;          /* initialize to skip over word */
    int whichsense = ALLSENSES, help = 0;
    int errcount = 0;
    int sensecount = 0;
    char *morphword;
    int lexinfo;
    
    dflag = 0;          /* turn off definitions by default */

    /* Parse command line options */

    for(i = 1; i < ac; i++) {
    if(!strcmp("-g",av[i]))
        dflag++;
    else if (!strcmp("-h",av[i]))
        help++;
    else if (!strcmp("-l", av[i]))
        printf(license);
    else if (!strncmp("-n", av[i], 2))
        whichsense = atoi(av[i] + 2);
    else if (!strcmp("-a", av[i]))
#ifndef NEXT
        putenv("WNDEBUG=on");
#else
        ;
#endif
    }

    lexinfo = (getenv("WNDEBUG") == NULL) ? 0 : 1;

    strtolower(strsubst(av[1], ' ', '_'));

    while(av[++j]) {
    if(strcmp("-g", av[j]) &&
       strcmp("-h", av[j]) &&
       strcmp("-l", av[j]) &&
       strncmp("-n", av[j], 2)
       && strcmp("-a", av[j])
       ) {
        i = 0;
        
        while(optlist[i].option) {
        if(!strcmp(optlist[i].option, av[j])) {
            if (help)
            printf("%s\n",
                   helptext[optlist[i].pos][optlist[i].errmsg]);
            printf("%s of %s %s\n",
               optlist[i].label,
               partnames[optlist[i].pos],
               av[1]);
            printf(findtheinfo(av[1],
                       optlist[i].pos,
                       optlist[i].search,
                       whichsense));
            sensecount = OutSenseCount;
            if (morphword = morphstr(av[1], optlist[i].pos))
            do
                printf(findtheinfo(morphword,
                           optlist[i].pos,
                           optlist[i].search,
                           whichsense));
            while (morphword = morphstr(NULL,
                            optlist[i].pos));
            break;
        }
        i++;
        }
        if(!optlist[i].option) {
        printf("wn: invalid option : %s\n",av[j]);
        errcount++;
        }
    }
    }
#ifndef NEXT
    /* reset WNDEBUG (lex. file info) as it was before wn was run */
    if (lexinfo)
    putenv("WNDEBUG=on");
    else
    putenv("WNDEBUG=");
#endif
    return(errcount ? -errcount : sensecount);
}

static int do_is_defined(char *searchword)
{
    int i, found = 0;
    unsigned long search;
    char *morphword;
    
    /* Print all valid searches for word in all parts of speech */

    strtolower(strsubst(searchword, ' ', '_'));
        
    for (i = 1; i <= NUMPARTS; i++) {
    if ((search = is_defined(searchword, i)) != 0) {
        printsearches(searchword, i, search);
        found = 1;
    } else
        printf("\nNo information available for %s %s\n",
           partnames[i], searchword);

    if ((morphword = morphstr(searchword, i)) != NULL)
        do {
        if ((search = is_defined(morphword, i)) != 0) {
            printsearches(morphword, i, search);
            found = 1;
        } else
            printf("\nNo information available for %s %s\n",
               partnames[i], morphword);
        } while ((morphword = morphstr(NULL, i)) != NULL );
    }
    return(found);
}

static void printsearches(char *word, int dbase, unsigned long search)
{
    int j;

    printf("\nInformation available for %s %s\n", partnames[dbase], word);
    for (j = 1; j <= MAXSEARCH; j++)
    if ((search & bit(j)) && searchstr[j].option) {
        printf("\t");
        printf(searchstr[j].template,
           partchars[dbase], partchars[dbase]);
        printf(searchstr[j].helpstr);
        printf("\n");
    }
}

/*
  Revision log:
  
  $Log: wn.c,v $
 * Revision 1.50  1995/03/03  20:06:38  wn
 * ifdef out putenv stuff - not available on next
 *
 * Revision 1.49  1995/03/03  14:46:10  wn
 * fixed warnings found by linux gcc
 *
 * Revision 1.48  1995/02/07  17:06:03  wn
 * cleanup, added prototypes
 *
 * Revision 1.47  1995/02/02  21:51:47  wn
 * removed HASGREP stuff
 *
 * Revision 1.46  1995/01/12  18:03:25  wn
 * changed relsn to simsn
 *
 * Revision 1.45  1995/01/12  18:01:15  wn
 * added relative search
 *
 * Revision 1.44  1994/11/29  22:29:37  wn
 * added place holder for relative search
 *
 * Revision 1.43  1994/10/14  16:11:10  wn
 * *** empty log message ***
 *
 * Revision 1.42  1994/01/05  12:32:04  wn
 * changed do_is_defined to return 1 or 0
 *
 * Revision 1.41  93/07/29  13:51:03  wn
 * *** empty log message ***
 * 
 * Revision 1.40  93/07/27  15:28:47  wn
 * fixed error msgs
 * 
 * Revision 1.39  93/07/01  16:36:51  wn
 * moved wninit call
 * 
 * Revision 1.38  93/04/21  14:06:18  wn
 * added 'attrn' and 'attra'
  * 
  * Revision 1.1  91/09/17  15:51:09  wn
  * Initial revision
  * 
  */
