//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// mainfrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "common.h"
#include "wordz.h"
#include "lookup.h"
#include "Options.h"

#include "mainfrm.h"

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

#if defined(SubclassWindow)
// Hide windows crackers SubclassWindow macro
#undef SubclassWindow
#endif


// Globals
//
CMainFrame* pTheOneAndOnlyMainFrame = NULL;


/////////////////////////////////////////////////////////////////////////////
// CMDIClientWnd
//
// Class basically used for changing the frame background color based on 
// the knowledge base article Q129471 (How to SubClass the MDIClient by Using MFC)
//

CMDIClientWnd::CMDIClientWnd()
{
	m_clrBackground = clrGRAY;
}

CMDIClientWnd::~CMDIClientWnd()
{
}


BEGIN_MESSAGE_MAP(CMDIClientWnd, CWnd)
	//{{AFX_MSG_MAP(CMDIClientWnd)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CMDIClientWnd message handlers

// OnEraseBkgnd: Erase the frame background
//
BOOL CMDIClientWnd::OnEraseBkgnd(CDC* pDC) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CMDIClientWnd::OnEraseBkgnd(%x)\n", pDC));

	CBrush brush;
	brush.CreateSolidBrush(m_clrBackground);
	brush.UnrealizeObject();
	CBrush* pOldBrush = pDC->SelectObject(&brush);

	CRect rect;
	pDC->GetClipBox(rect);
	DEBUG_OUT(TL_GARRULOUS, ("CMDIClientWnd::OnEraseBkgnd: color=0x%08x: ", m_clrBackground));
	DUMP_RECT(TL_GARRULOUS, "clip rect=", rect, "\n");

	pDC->PatBlt(rect.left, rect.top, rect.Width(), rect.Height(), PATCOPY);
	pDC->SelectObject(pOldBrush);

	return (TRUE);
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_MISC_OPTIONS, OnMiscOptions)
	ON_COMMAND(ID_APP_EXIT, OnAppExit)
	ON_COMMAND(ID_APP_PUNT, OnAppPunt)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// arrays of IDs used to initialize control bars
	
// toolbar buttons - IDs are command buttons
static UINT BASED_CODE buttons[] =
{
	// same order as in the bitmap 'toolbar.bmp'
	ID_FILE_NEW,
	ID_FILE_OPEN,
	ID_FILE_SAVE,
		ID_SEPARATOR,
	ID_EDIT_CUT,
	ID_EDIT_COPY,
	ID_EDIT_PASTE,
		ID_SEPARATOR,
	ID_FILE_PRINT,
	ID_APP_ABOUT,
};

static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};


/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::CMainFrame()\n"));

	pTheOneAndOnlyMainFrame = this;
	m_pWndCurrentChild = NULL;
	GetOptions();
}


CMainFrame::~CMainFrame()
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::~CMainFrame()\n"));

	CLookup::CleanupLookup();
}


VOID CMainFrame::GetOptions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::GetOptions()\n"));

	m_clrBackground = GetAppOption("WindowBackground", clrLIGHT_GRAY);
#if TRUE
	m_wndMDIClientWnd.m_clrBackground = m_clrBackground;
#endif
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::OnCreate(%x)\n", lpCreateStruct));

	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1) {
		DEBUG_OUT(TL_BASIC, ("CMDIFrameWnd::OnCreate failed\n"));
		return (-1);
		}

	// Subclass the window to handle changing the background color
	if (!m_wndMDIClientWnd.SubclassWindow(m_hWndMDIClient)) {
		DEBUG_OUT(TL_BASIC, ("Failed to subclass MDI client window\n"));
		return (-1);
		}

	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadBitmap(IDR_MAINFRAME) ||
		!m_wndToolBar.SetButtons(buttons, sizeof(buttons)/sizeof(UINT))) {
		DEBUG_OUT(TL_BASIC, ("Failed to create toolbar\n"));
		return -1;      // fail to create
		}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators, sizeof(indicators)/sizeof(UINT))) {
		DEBUG_OUT(TL_BASIC, ("Failed to create status bar\n"));
		return -1;      // fail to create
		}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// TODO: Remove this if you don't want tool tips
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() | CBRS_TOOLTIPS | CBRS_FLYBY);

	// Cheesy way to disable the status dialog
	// TODO: base this opon user preference
	PostMessage(WM_COMMAND, ID_VIEW_STATUS_BAR);
	PostMessage(WM_COMMAND, ID_VIEW_TOOLBAR);

	//CLookup::InitLookup(this);

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

BOOL CMainFrame::OnEraseBkgnd(CDC* pDC) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CMainFrame::OnEraseBkgnd(%x)\n", pDC));

	CBrush brush;
	brush.CreateSolidBrush(m_clrBackground);
	brush.UnrealizeObject();
	CBrush* pOldBrush = pDC->SelectObject(&brush);

	CRect rect;
	pDC->GetClipBox(rect);
	DEBUG_OUT(TL_GARRULOUS, ("CMainFrame::OnEraseBkgnd: color=0x%08x: ", m_clrBackground));
	DUMP_RECT(TL_GARRULOUS, "", rect, "\n");
	pDC->PatBlt(rect.left, rect.top, rect.right, rect.bottom, PATCOPY);
	pDC->SelectObject(pOldBrush);

	return (TRUE);
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs) 
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::PreCreateWindow(%x)\n", &cs));
#if TRUE
	if (GetDebugOption("AddClipChildren", FALSE)) {
		cs.style |= WS_CLIPCHILDREN;
		}
#endif

	if (GetDebugOption("UseStupidCMainFrame", FALSE)) {
		STATIC CONST CHAR szCLASS[] = "StupidCMainFrame";
		WNDCLASS WndClass;
		//GetClassInfo(AfxGetInstanceHandle(), cs.lpszClass, &WndClass);
		// TODO: add WS_CLIPCHILDREN;
		WndClass.style = CS_BYTEALIGNCLIENT | CS_BYTEALIGNWINDOW;
		WndClass.lpfnWndProc = AfxGetAfxWndProc();
		WndClass.lpszClassName = szCLASS;
		WndClass.cbClsExtra = 0;
		WndClass.cbWndExtra = 0;
		WndClass.hInstance = AfxGetInstanceHandle();
		WndClass.hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
		WndClass.hCursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
		WndClass.hbrBackground = (HBRUSH)(1 + COLOR_GRAYTEXT);
		WndClass.lpszMenuName = MAKEINTRESOURCE(IDR_MAINFRAME);
		AfxRegisterClass(&WndClass);
		cs.lpszClass = szCLASS;
		}
	
	return CMDIFrameWnd::PreCreateWindow(cs);
}


BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::OnCreateClient(%x, %x)\n", lpcs, pContext));

	UNREFERENCED_PARAMETER(pContext);
	if (GetDebugOption("UseStupidCMainFrame", FALSE)) {
		return (MyStupidCreateClient(lpcs, NULL));
		}

	return (CMDIFrameWnd::OnCreateClient(lpcs, NULL));
}


BOOL CMainFrame::MyStupidCreateClient (LPCREATESTRUCT lpCreateStruct, CMenu* pWindowMenu)
{
	DEBUG_OUT(TL_VERBOSE, ("CMainFrame::MyStupidCreateClient()\n"));

	ASSERT(m_hWnd != NULL);
	ASSERT(m_hWndMDIClient == NULL);
	DWORD dwStyle = WS_VISIBLE | WS_CHILD | WS_BORDER |
		WS_CLIPCHILDREN | WS_CLIPSIBLINGS |
		MDIS_ALLCHILDSTYLES;    // allow children to be created invisible
	DWORD dwExStyle = 0;
	// will be inset by the frame

#if FALSE
	if (afxData.bWin4)
	{
		// special styles for 3d effect on Win4
		dwStyle &= ~WS_BORDER;
		dwExStyle = WS_EX_CLIENTEDGE;
	}
#endif

	CONST CHAR szMID_CLIENT[] = _T("mdiclient");
	STATIC CONST CHAR szMY_CLASS[] = _T("my_stupid_mdiclient");
	WNDCLASS WndClass;
#if FALSE
	WndClass.style = CS_BYTEALIGNCLIENT | CS_BYTEALIGNWINDOW;
	WndClass.lpfnWndProc = AfxGetAfxWndProc();
	WndClass.lpszClassName = szMY_CLASS;
	WndClass.cbClsExtra = 0;
	WndClass.cbWndExtra = 0;
	WndClass.hInstance = AfxGetInstanceHandle();
	WndClass.hIcon = NULL;
	WndClass.hCursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
	WndClass.hbrBackground = (HBRUSH)(1 + COLOR_GRAYTEXT);
	WndClass.lpszMenuName = NULL;
#endif
	GetClassInfo(AfxGetInstanceHandle(), szMID_CLIENT, &WndClass);
	WndClass.lpszClassName = szMY_CLASS;
	//WndClass.hbrBackground = (HBRUSH)(1 + COLOR_GRAYTEXT);
	WndClass.hbrBackground = NULL;
	AfxRegisterClass(&WndClass);
	
	CLIENTCREATESTRUCT ccs;
	ccs.hWindowMenu = pWindowMenu->GetSafeHmenu();
		// set hWindowMenu for MFC V1 backward compatibility
		// for MFC V2, window menu will be set in OnMDIActivate
	ccs.idFirstChild = AFX_IDM_FIRST_MDICHILD;

	if (lpCreateStruct->style & (WS_HSCROLL|WS_VSCROLL)) {
		// parent MDIFrame's scroll styles move to the MDICLIENT
		dwStyle |= (lpCreateStruct->style & (WS_HSCROLL|WS_VSCROLL));

		// fast way to turn off the scrollbar bits (without a resize)
		ModifyStyle(WS_HSCROLL|WS_VSCROLL, 0, SWP_NOREDRAW|SWP_FRAMECHANGED);
	}

	// Create MDICLIENT control with special IDC
	if ((m_hWndMDIClient = ::CreateWindowEx(dwExStyle, szMY_CLASS, NULL,
		dwStyle, 0, 0, 0, 0, m_hWnd, (HMENU)AFX_IDW_PANE_FIRST,
		AfxGetInstanceHandle(), (LPVOID)&ccs)) == NULL) {
		TRACE0("Warning: CMDIFrameWnd::OnCreateClient: failed to create MDICLIENT.\n");
		return FALSE;
		}
	// Move it to the top of z-order
	::BringWindowToTop(m_hWndMDIClient);

	return TRUE;
}


void CMainFrame::OnMiscOptions (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::OnMiscOptions()\n"));

	CWordzOptionsDlg dlgOptions(this);
	INT nResult = dlgOptions.DoModal();
	if (nResult == IDOK) {
		COLORREF colorOld = m_clrBackground;
		if (dlgOptions.m_bSettingsChanged) {
			GetOptions();
			InvalidateRect(NULL);
			UpdateWindow();
#if TRUE
			// Repaint the child frame windows
			// TODO: figure out a better way to to do (is it really needed?)
			if (GetDebugOption("UpdateChildren", TRUE)) {
				UpdateChildren();
				}
#endif
			UpdateAllViews();
			}
		}
}


VOID CMainFrame::UpdateAllViews (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::UpdateAllViews()\n"));

	// Get the template object so we can iterate the documents
	POSITION posTemplate = AfxGetApp()->GetFirstDocTemplatePosition();
	ASSERT(posTemplate != NULL);
	CDocTemplate* pTemplate = AfxGetApp()->GetNextDocTemplate(posTemplate);
	ASSERT(pTemplate);
	ASSERT(posTemplate == NULL);

	// Iterate through the documents and update their views
	POSITION posDocument = pTemplate->GetFirstDocPosition();
	//ASSERT(posDocument);
	while (posDocument) {
		CDocument* pDoc = pTemplate->GetNextDoc(posDocument);
		pDoc->UpdateAllViews(NULL,					// view that modified the doc
							 lHINT_OPTION_CHANGE);  // update hint
		}
}


void CMainFrame::OnAppExit (VOID) 
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::OnAppExit()\n"));

	DestroyWindow();
}


// UpdateChildren: Interate through the MDI child windows to force
// an update on their windows and frames
//
VOID CMainFrame::UpdateChildren (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::UpdateChildren()\n"));

	for (CMDIChildWnd* pChild = GetFirstMDIChildWnd();
		 pChild;
		 pChild = GetNextMDIChildWnd()) {
		pChild->InvalidateRect(NULL);
		//pChild->UpdateWindow();
		pChild->RedrawWindow(NULL, NULL, // update rectangle & region
							 RDW_FRAME | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE); 

		}
}


CMDIChildWnd* CMainFrame::GetFirstMDIChildWnd (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::GetFirstMDIChildWnd()\n"));

	m_pWndCurrentChild = NULL;
	return (GetNextMDIChildWnd());
}


// GetNextMDIChildWnd: Finds the CMDIChildWnd in the list of windows
// maintained by the application's MDIClient window following the
// one pointed to by the member variable m_pWndCurrentChild. If no
// further CMDIChildWnds are in the list, NULL is returned.
//
// NOTE: From knowledge base article (Q131994):
//		 How to Use MFC to Step Through Child Windows of the MDIClient
//
 
CMDIChildWnd* CMainFrame::GetNextMDIChildWnd (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::GetNextMDIChildWnd()\n"));

	if (!m_pWndCurrentChild) {
		// Get the first child window.
		m_pWndCurrentChild = m_wndMDIClientWnd.GetWindow(GW_CHILD);
		}
	else {
		// Get the next child window in the list.
		m_pWndCurrentChild = m_pWndCurrentChild->GetWindow(GW_HWNDNEXT);
		}
	
	if (!m_pWndCurrentChild) {
		// No child windows exist in the MDIClient,
		// or you are at the end of the list. This check
		// will terminate any recursion.
		return (NULL);
		}
	
	// Check the kind of window
	if (!m_pWndCurrentChild->GetWindow(GW_OWNER)) {
		if (m_pWndCurrentChild->IsKindOf(RUNTIME_CLASS(CMDIChildWnd))) {
			// CMDIChildWnd or a derived class.
			return (CMDIChildWnd*)m_pWndCurrentChild;
			}
		else {
			// Window is foreign to the MFC framework.
			// Check the next window in the list recursively.
			return GetNextMDIChildWnd();
			}
		}
	else {
		// Title window associated with an iconized child window.
		// Recurse over the window manager's list of windows.
		return GetNextMDIChildWnd();
		}

	ASSERT_ERROR();
	return (NULL);
}

void CMainFrame::OnAppPunt() 
{
    DEBUG_OUT(TL_VERBOSE, ("CMainFrame::OnAppPunt()\n"));

	DestroyWindow();
	AfxOleTerm();
	ExitProcess(666);
}
