//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// Options.cpp : implementation file
//

#include "stdafx.h"
#include "Wordz.h"
#include "Lookup.h"
#include "Options.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWordzOptionsDlg


IMPLEMENT_DYNAMIC(CWordzOptionsDlg, CPropertySheet)

CWordzOptionsDlg::CWordzOptionsDlg(CWnd* pParentWnd, UINT iSelectPage)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptionsDlg::CWordzOptionsDlg(%x, %d)\n",
			  pParentWnd, iSelectPage));

	CPropertySheet("Wordz Options", pParentWnd, iSelectPage);
	CommonInit();
}


CWordzOptionsDlg::CWordzOptionsDlg(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptionsDlg::CWordzOptionsDlg(%d, %x, %d)\n",
			  nIDCaption, pParentWnd, iSelectPage));

	CommonInit();
}


CWordzOptionsDlg::CWordzOptionsDlg(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptionsDlg::CWordzOptionsDlg(%s, %x, %d)\n",
			  F(pszCaption), pParentWnd, iSelectPage));

	CommonInit();
}


CWordzOptionsDlg::~CWordzOptionsDlg (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptionsDlg::~CWordzOptionsDlg()\n"));
}


VOID CWordzOptionsDlg::CommonInit (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptionsDlg::CommonInit()\n"));

	m_bSettingsChanged = FALSE;
	//LPCDLGTEMPLATE pResource;

	m_pageWordzOptions.PrepareMisc();
	AddPage(&m_pageWordzOptions);

	m_pageBookshelfOptions.PrepareMisc();
	AddPage(&m_pageBookshelfOptions);

	m_pageTrlOptions.PrepareMisc();
	AddPage(&m_pageTrlOptions);

	m_pageWordNetOptions.PrepareMisc();
	AddPage(&m_pageWordNetOptions);
}


BEGIN_MESSAGE_MAP(CWordzOptionsDlg, CPropertySheet)
	//{{AFX_MSG_MAP(CWordzOptionsDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


int CWordzOptionsDlg::DoModal (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptionsDlg::DoModal()\n"));

	INT nResult = CPropertySheet::DoModal();
	if (nResult == IDOK) {
		// TODO: check if the settings for the individual property pages have changed
		m_bSettingsChanged = TRUE;
		if (m_pageBookshelfOptions.m_bSettingsChanged ||
				m_pageTrlOptions.m_bSettingsChanged ||
				m_pageWordNetOptions.m_bSettingsChanged) {
			CWaitCursor wait_cursor;
			CLookup::ReinitLookup();
			}
		}

	return (nResult);
}

/////////////////////////////////////////////////////////////////////////////
// CWordzOptionsDlg message handlers


/////////////////////////////////////////////////////////////////////////////
// CWordzOptions property page

IMPLEMENT_DYNCREATE(CWordzOptions, CPropertyPage)

CWordzOptions::CWordzOptions() : CPropertyPage(CWordzOptions::IDD)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::CWordzOptions()\n"));

	//{{AFX_DATA_INIT(CWordzOptions)
	m_bDontAskToSave = GetAppOption(szDONT_ASK_TO_SAVE, FALSE);
	m_bBorderlessWindows = GetAppOption(szBORDERLESS_WINDOWS, FALSE);
	m_bSaveLayout = GetAppOption(szSAVE_LAYOUT, TRUE);
	m_bIncludeDefinitions = GetAppOption(szLOOKUP_DEFINITION, TRUE);
	m_bIncludeDescendants = GetAppOption(szLOOKUP_DESCENDANTS, FALSE);
	m_bIncludeReferences = GetAppOption(szLOOKUP_REFERENCES, FALSE);
	m_bIncludeSynonyms = GetAppOption(szLOOKUP_SYNONYMS, TRUE);
	//}}AFX_DATA_INIT
}

CWordzOptions::~CWordzOptions()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::~CWordzOptions()\n"));
}


VOID CWordzOptions::PrepareMisc (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::PrepareMisc()\n"));
	m_psp.hInstance = AfxGetInstanceHandle();
}


VOID HandleColorOption (PCSZ pszOption, COLORREF colorDefault, CWnd* pWnd)
{
	DEBUG_OUT(TL_VERBOSE, ("HandleColorOption(%s, %08x, 0x%x)\n", 
			  F(pszOption), colorDefault, pWnd));

	COLORREF colorInitial = (INT)GetAppOption(pszOption, (INT)colorDefault);
	COLORREF colorSelected = GetColorOption(colorInitial, pWnd);
	if (colorSelected != colorInitial) {
		// TODO: only write the value upon OK or APPLY
		WriteAppOption(pszOption, (INT)colorSelected);
		// Refresh display
	}
}


// Now back to our regular program
//

void CWordzOptions::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWordzOptions)
	DDX_Check(pDX, IDC_DONT_ASK_TO_SAVE, m_bDontAskToSave);
	DDX_Check(pDX, IDC_RESIZABLE_WINDOWS, m_bBorderlessWindows);
	DDX_Check(pDX, IDC_SAVE_LAYOUT, m_bSaveLayout);
	DDX_Check(pDX, IDC_INCLUDE_DEFINITIONS, m_bIncludeDefinitions);
	DDX_Check(pDX, IDC_INCLUDE_DESCENDANTS, m_bIncludeDescendants);
	DDX_Check(pDX, IDC_INCLUDE_REFERENCES, m_bIncludeReferences);
	DDX_Check(pDX, IDC_INCLUDE_SYNONYMS, m_bIncludeSynonyms);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWordzOptions, CPropertyPage)
	//{{AFX_MSG_MAP(CWordzOptions)
	ON_BN_CLICKED(IDC_TEXT_BACKGROUND, OnTextBackground)
	ON_BN_CLICKED(IDC_TEXT_COLOR, OnTextColor)
	ON_BN_CLICKED(IDC_WINDOW_BACKGROUND, OnWindowBackground)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWordzOptions message handlers

#if FALSE
void CWordzOptions::OnActiveCell() 
{
	HandleColorOption("ActiveCell", clrPALE_GREEN, this);
}
#endif


void CWordzOptions::OnTextBackground (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::OnTextBackground()\n"));

	HandleColorOption("TextBackground", clrLIGHT_GRAY, this);
}


void CWordzOptions::OnTextColor (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::OnTextColor()\n"));

	HandleColorOption("TextColor", clrBLACK, this);
}


void CWordzOptions::OnWindowBackground (VOID) 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::OnWindowBackground()\n"));

	HandleColorOption("WindowBackground", clrLIGHT_GRAY, this);
}


void CWordzOptions::OnOK (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::OnOK()\n"));

	ApplySettings();

	CPropertyPage::OnOK();
}


BOOL CWordzOptions::OnApply (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::OnApply()\n"));

	ApplySettings();

	return CPropertyPage::OnApply();
}


BOOL CWordzOptions::ApplySettings (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzOptions::ApplySettings()\n"));

	WriteAppOption(szDONT_ASK_TO_SAVE, m_bDontAskToSave);
	WriteAppOption(szBORDERLESS_WINDOWS, m_bBorderlessWindows);
	WriteAppOption(szSAVE_LAYOUT, m_bSaveLayout);
	WriteAppOption(szLOOKUP_DEFINITION, m_bIncludeDefinitions);
	WriteAppOption(szLOOKUP_DESCENDANTS, m_bIncludeDescendants);
	WriteAppOption(szLOOKUP_REFERENCES, m_bIncludeReferences);
	WriteAppOption(szLOOKUP_SYNONYMS, m_bIncludeSynonyms);

	return (TRUE);
}
