//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// srvritem.cpp : implementation of the CWordzSrvrItem class
//

#include "stdafx.h"
#include "Wordz.h"

#include "rtfedit.h"
#include "lookup.h"
#include "wordzvw.h"
#include "Wordzdoc.h"
#include "srvritem.h"

IMPLEMENT_DYNAMIC(CWordzSrvrItem, COleServerItem)

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEDBUG_NEW
#endif


/////////////////////////////////////////////////////////////////////////////
// CWordzSrvrItem implementation


CWordzSrvrItem::CWordzSrvrItem(CWordzDoc* pContainerDoc)
	: COleServerItem(pContainerDoc, TRUE)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzSrvrItem::CWordzSrvrItem(%x)\n", pContainerDoc));
	// TODO: add one-time construction code here
	//  (eg, adding additional clipboard formats to the item's data source)
	//m_sizeDefault = CSize(5000, 5000);   // 10000 x 10000 HIMETRIC units
	//m_sizeDefault = CSize(300, 400);   // 3x4 ""
	m_sizeDefault = CSize(400, 200);   // 4x2 inches
}


CWordzSrvrItem::~CWordzSrvrItem()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzSrvrItem::~CWordzSrvrItem(%x)\n"));
	// TODO: add cleanup code here
}


void CWordzSrvrItem::Serialize(CArchive& ar)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzSrvrItem::Serialize(%x)\n", ar));
	// CWordzSrvrItem::Serialize will be called by the framework if
	//  the item is copied to the clipboard.  This can happen automatically
	//  through the OLE callback OnGetClipboardData.  A good default for
	//  the embedded item is simply to delegate to the document's Serialize
	//  function.  If you support links, then you will want to serialize
	//  just a portion of the document.

	if (!IsLinkedItem())
	{
		CWordzDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		pDoc->Serialize(ar);
	}
}


VOID SetDefaultMapMode (CDC* pDC, CSize& sizeExtent)
{
	DEBUG_OUT(TL_VERBOSE, ("SetDefaultMapMode(%x, %x)\n", pDC, &sizeExtent));
	pDC->SetMapMode(MM_ANISOTROPIC);
	pDC->SetViewportExt(pDC->GetDeviceCaps(LOGPIXELSX), pDC->GetDeviceCaps(LOGPIXELSY));
	pDC->SetWindowOrg(0,0);
	//pDC->SetWindowExt(100, -100);		// "logical" LOENGLISH
	pDC->SetWindowExt(100, -100);
	//pDC->SetWindowExt(sizeExtent);
}

BOOL CWordzSrvrItem::OnGetExtent(DVASPECT dwDrawAspect, CSize& rSize)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzSrvrItem::OnGetExtent(%x, %x)\n", dwDrawAspect, rSize));
	// Most applications, like this one, only handle drawing the content
	//  aspect of the item.  If you wish to support other aspects, such
	//  as DVASPECT_THUMBNAIL (by overriding OnDrawEx), then this
	//  implementation of OnGetExtent should be modified to handle the
	//  additional aspect(s).

	BOOL bOK = FALSE;
	if (dwDrawAspect != DVASPECT_CONTENT) {
		bOK = COleServerItem::OnGetExtent(dwDrawAspect, rSize);
		}
	else {
		// CWordzSrvrItem::OnGetExtent is called to get the extent in
		//  HIMETRIC units of the entire item.  The default implementation
		//  here simply returns a hard-coded number of units.
		CWordzDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		// TODO: replace this arbitrary size
		rSize = m_sizeDefault;
		CClientDC dc(NULL);
		SetDefaultMapMode(&dc, rSize);
		dc.LPtoHIMETRIC(&rSize);
		bOK = TRUE;
		}
	DEBUG_OUT(TL_VERBOSE, ("%d <= OnGetExtent: rSize = (%d, %d)\n", 
			  bOK, rSize.cx, rSize.cy));

	return (bOK);
}


//	Convert a rectange in HIMETRIC coordinates into twips.
//	NOTE: MM_HIMETRIC (himm) is in units of 0.01 mm
//
//	Since 1 inch = 25.4 mm
//		1 twip = 1/20th point = 1/1440 inch = 0.0176 mm
//	so  1 mm = 56.8 twips and 1 himm = 0.568 twips
//
//
STATIC CRect HiMetricToTwips (CRect& rectHiMetric)
{
	DEBUG_OUTF(TL_VERBOSE, ("HiMetricToTwips(%x)\n", &rectHiMetric));

	//	First convert the himetrics coordinates into twips using following conversion:
	//		1 himetric unit = 0.568 twips (see comment above)
	CRect rectTwips;
	rectTwips.left = MulDiv(rectHiMetric.left, 568, 1000);
	rectTwips.top = MulDiv(rectHiMetric.top, 568, 1000);
	rectTwips.right = MulDiv(rectHiMetric.right, 568, 1000);
	rectTwips.bottom = MulDiv(rectHiMetric.bottom, 568, 1000);

	return (rectTwips);
}


//	Convert a rectange in LOENGLISH coordinates into twips.
//	NOTE: LOENGLISH  (loinch) is in units of 0.01 inch
//
//		1 twip = 1/20th point = 1/1440 inch
//	so  1 inch = 1440 twips and 1 loinch = 14.40 twips
//
//
STATIC CRect LoEnglishToTwips (CRect& rectLoEnglish)
{
	DEBUG_OUTF(TL_VERBOSE, ("LoEnglishToTwips(%x)\n", &rectLoEnglish));

	//	First convert the LoEnglishs coordinates into twips using following conversion:
	//		1 LoEnglish unit = 14.40 twips (see comment above)
	CRect rectTwips;
	rectTwips.left = MulDiv(rectLoEnglish.left, 1440, 100);
	rectTwips.top = MulDiv(rectLoEnglish.top, 1440, 100);
	rectTwips.right = MulDiv(rectLoEnglish.right, 1440, 100);
	rectTwips.bottom = MulDiv(rectLoEnglish.bottom, 1440, 100);

	return (rectTwips);
}


BOOL CWordzSrvrItem::OnDraw(CDC* pDC, CSize& rSize)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzSrvrItem::OnDraw(%x, (%d, %d))\n", pDC, rSize.cx, rSize.cy));
	CWordzDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: set mapping mode and extent
	//  (The extent is usually the same as the size returned from OnGetExtent)
	SetDefaultMapMode(pDC, m_sizeDefault);

    CWordInfo* pInfo = pDoc->GetInfo(0); // for now, just use the first phrase in doc's list
   	ASSERT_VALID(pInfo);

	// TODO: add drawing code here.  Optionally, fill in the HIMETRIC extent.
	//  n.b.: All drawing takes place in the metafile device context (pDC).
	CRect rectExtent (CPoint(0, 0), m_sizeDefault);
#if TRUE
	rectExtent.bottom = -rectExtent.bottom/2;
#endif
	INT nBase = 0;
	DUMP_RECT_(TL_VERBOSE, "extent rect", rectExtent);

#if FALSE
	//	Draw the text for the word
	pDC->DrawText(pInfo->GetWord(), -1, rect, DT_NOPREFIX | DT_CALCRECT);
	rect.OffsetRect(0, nBase);
	pDC->DrawText(pInfo->GetWord(), -1, rect, DT_NOPREFIX);
	nBase = rect.bottom;

	//	Draw the text for the definition
	pDC->DrawText(pInfo->GetDefinition(), -1, rect, DT_WORDBREAK | DT_NOPREFIX | DT_CALCRECT);
	rect.OffsetRect(0, nBase);
	pDC->DrawText(pInfo->GetDefinition(), -1, rect, DT_WORDBREAK | DT_NOPREFIX);
	nBase = rect.bottom;

	//	Draw the text for the synonyms
	pDC->DrawText(pInfo->GetSynonyms(), -1, rect, DT_WORDBREAK | DT_NOPREFIX | DT_CALCRECT);
	rect.OffsetRect(0, nBase);
	pDC->DrawText(pInfo->GetSynonyms(), -1, rect, DT_WORDBREAK | DT_NOPREFIX);
	nBase = rect.bottom;

	//	Draw the text for the cross-references
	pDC->DrawText(pInfo->GetCrossRefs(), -1, rect, DT_WORDBREAK | DT_NOPREFIX | DT_CALCRECT);
	rect.OffsetRect(0, nBase);
	pDC->DrawText(pInfo->GetCrossRefs(), -1, rect, DT_WORDBREAK | DT_NOPREFIX);
	nBase = rect.bottom;
#endif

//virtual BOOL ExtTextOut( int x, int y, UINT nOptions, LPCRECT lpRect, LPCTSTR lpszString, UINT nCount, LPINT lpDxWidths );

	//	Misc setup
	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	//lf.lfHeight = MulDiv(10, rect.Height(), 100);
	//lf.lfHeight = MulDiv(5, pDC->GetWindowExt().cy, 100);
	lf.lfHeight = MulDiv(5, rectExtent.Height(), 100);
	//lf.lfHeight = MulDiv(10, rectExtent.Height(), 100);
	CFont font;
	font.CreateFontIndirect(&lf);
	CFont* pOldFont = pDC->SelectObject(&font);

	//pDC->SetBkMode(TRANSPARENT);
	
	CWordzView* pView = pDoc->FirstView();
	ASSERT(pView);
	CPen penBorder;
	CPen* pOldPen;
	
	//	word
	penBorder.CreatePen(PS_DASH, 1, RGB(192, 192, 192));
	pOldPen = pDC->SelectObject(&penBorder);
	//pDC->DrawText(pInfo->GetWord(), -1, rect, DT_NOPREFIX);
	CRect rectWord = rectExtent;
	rectWord.bottom = rectWord.top + MulDiv(10, rectExtent.Height(), 100);
	DUMP_RECT_(TL_VERBOSE, "word rect", rectWord);
	pDC->Rectangle(rectWord);
	pDC->ExtTextOut(rectWord.left, rectWord.top, 0, rectWord, pInfo->GetWord(), pInfo->GetWord().GetLength(), NULL);
	pDC->SelectObject(pOldPen);
	penBorder.DeleteObject();

	CRect rectOther = rectExtent;
	INT nOtherHeight = MulDiv(30, rectExtent.Height(), 100);
	rectOther.top = rectWord.bottom;
	rectOther.bottom = rectOther.top + nOtherHeight;
	//rectOther.OffsetRect(0, rectWord.Height());

	
	CClientDC dcClient(pView);
	CRect rectClient;
	pView->GetClientRect(&rectClient);
	rectClient.right = MulDiv(rectClient.right, 1440, dcClient.GetDeviceCaps(LOGPIXELSX));

	//	restore the usual mapping mode (MM_ANISOTROPIC@10000x10000)
	SetDefaultMapMode(pDC, m_sizeDefault);
	//pOldFont = pDC->SelectObject(&font);

	//	syns
	penBorder.CreatePen(PS_DOT, 1, RGB(0, 192, 0));
	pOldPen = pDC->SelectObject(&penBorder);
	//pDC->DrawText(pInfo->GetSynonyms(), -1, rect, DT_WORDBREAK | DT_NOPREFIX);
	DUMP_RECT_(TL_VERBOSE, "syns rect", rectOther);
	pDC->Rectangle(rectOther);
	pDC->ExtTextOut(rectOther.left, rectOther.top, 0, rectOther, pInfo->GetSynonyms(), pInfo->GetSynonyms().GetLength(), NULL);
	//pDC->DrawText(pInfo->GetSynonyms(), -1, rectOther, DT_WORDBREAK | DT_NOPREFIX);
	rectOther.OffsetRect(0, nOtherHeight);
	pDC->SelectObject(pOldPen);
	penBorder.DeleteObject();

	//	xrefs
	penBorder.CreatePen(PS_DOT, 1, RGB(0, 0, 192));
	pOldPen = pDC->SelectObject(&penBorder);
	//pDC->DrawText(pInfo->GetCrossRefs(), -1, rect, DT_WORDBREAK | DT_NOPREFIX);
	DUMP_RECT_(TL_VERBOSE, "xrefs rect", rectOther);
	pDC->Rectangle(rectOther);
	pDC->ExtTextOut(rectOther.left, rectOther.top, 0, rectOther, pInfo->GetCrossRefs(), pInfo->GetCrossRefs().GetLength(), NULL);
	rectOther.OffsetRect(0, nOtherHeight);
	pDC->SelectObject(pOldPen);
	penBorder.DeleteObject();

	//	def
	penBorder.CreatePen(PS_DOT, 1, RGB(192, 0, 0));
	pOldPen = pDC->SelectObject(&penBorder);
	DUMP_RECT_(TL_VERBOSE, "def rect", rectOther);
	pDC->Rectangle(rectOther);
	if (IsControlPressed()) {
		//pDC->SelectObject(pOldFont);
		//pDC->DrawText(pInfo->GetDefinition(), -1, rect, DT_WORDBREAK | DT_NOPREFIX);
		CRect rectDef = rectOther;
		if (!IsShiftPressed()) {
			pDC->SetMapMode(MM_TEXT);
			rectDef.top = -rectDef.top;
			rectDef.bottom = -rectDef.bottom;
			}
		//pDC->Rectangle(rectOther);
		//pDC->ExtTextOut(rectOther.left, rectOther.top, ETO_OPAQUE, rectOther, pInfo->GetDefinition(), pInfo->GetDefinition().GetLength(), NULL);
		//CRect rectDefView (0, 0, 32767, 32767);
		//CRect rectDefOut = rectOther;
		//LRESULT lResult = pView->m_edDef.SplatTextToDC(pDC, rectDefView, rectDefOut, FALSE);
		if (!IsAltPressed()) {
			rectDef = LoEnglishToTwips(rectDef);
			}
#if FALSE
		LRESULT lResult = pView->m_edDef.SplatTextToDC(pDC, rectDef, rectDef, FALSE);
		if (lResult == 0) {
			// what should we do now?
			}
#endif
		ASSERT(pDC->RectVisible(rectDef));
		pView->m_edDef.SplatTextToDC(pDC, rectDef, rectDef, TRUE);
		}
	pDC->SelectObject(pOldPen);
	penBorder.DeleteObject();
	rectOther.OffsetRect(0, nOtherHeight);

	//	clean up
	pDC->SelectObject(pOldFont);

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CWordzSrvrItem diagnostics


#ifdef _DEBUG


void CWordzSrvrItem::AssertValid() const
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzSrvrItem::AssertValid()\n"));
	COleServerItem::AssertValid();
}


void CWordzSrvrItem::Dump(CDumpContext& dc) const
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzSrvrItem::Dump(%x)\n", dc));
	COleServerItem::Dump(dc);
}


#endif

/////////////////////////////////////////////////////////////////////////////
