//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

//  wordz.cpp: class implementation for the Wordz application
//

#include "stdafx.h"
#include "Wordz.h"

#include "mainfrm.h"
#include "wordzfrm.h"
#include "ipframe.h"
#include "lookup.h"
#include "wordzdoc.h"
//#include "rtfedit.h"
#include "wordzvw.h"
#include "about.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

// Constants
//
//CONST CHAR szWORDZ_REG_KEY[] = "Nonesuch\\Wordz";
CONST CHAR szWORDZ_REG_KEY[] = "Nonesuch";

/////////////////////////////////////////////////////////////////////////////
// CWordzApp

BEGIN_MESSAGE_MAP(CWordzApp, CWinApp)
	//{{AFX_MSG_MAP(CWordzApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWordzApp construction

CWordzApp::CWordzApp (VOID)
{
	INIT_DEBUGGING_("wordz.ini");
	DEBUG_OUT(TL_VERBOSE, ("CWordzApp::CWordzApp()\n"));
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CWordzApp object

CWordzApp theApp;

// This identifier was generated to be statistically unique for your app.
// You may change it if you prefer to choose a specific identifier.
static const CLSID BASED_CODE clsid =
{ 0x32035e60, 0xb4cd, 0x11ce, { 0x96, 0x16, 0x44, 0x45, 0x53, 0x54, 0x0, 0x0 } };

/////////////////////////////////////////////////////////////////////////////
// CWordzApp initialization

BOOL CWordzApp::InitInstance()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzApp::InitInstance()\n"));
	InitCommon(m_hInstance, NULL, m_pszExeName);

	// Initialize OLE libraries
	if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

	SetDialogBkColor();
	Enable3dControls();

	BOOL bUseRegistry = GetDebugOption("UseRegistry", TRUE);
	if (bUseRegistry) {
		SetRegistryKey(szWORDZ_REG_KEY);
	}
	LoadStdProfileSettings(8);  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_WORDZTYPE,
		RUNTIME_CLASS(CWordzDoc),
		//RUNTIME_CLASS(CMDIChildWnd),        // standard MDI child frame
		RUNTIME_CLASS(CWordzFrame),			  // slightly non-standard MDI child frame
		RUNTIME_CLASS(CWordzView));
	pDocTemplate->SetServerInfo(
		IDR_WORDZTYPE_SRVR_EMB, IDR_WORDZTYPE_SRVR_IP,
		RUNTIME_CLASS(CInPlaceFrame));
	AddDocTemplate(pDocTemplate);

	// Connect the COleTemplateServer to the document template.
	//  The COleTemplateServer creates new documents on behalf
	//  of requesting OLE containers by using information
	//  specified in the document template.
	m_server.ConnectTemplate(clsid, pDocTemplate, FALSE);

	// Register all OLE server factories as running.  This enables the
	//  OLE libraries to create objects from other applications.
	COleTemplateServer::RegisterAll();
		// Note: MDI applications register all server objects without regard
		//  to the /Embedding or /Automation on the command line.

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Set the application window for anchoring error dialogs
	RegisterAppWindow(m_hInstance, m_pMainWnd->m_hWnd);

	// Initialize the static part of the lookup class (eg, MediaView access)
	CLookup::InitLookup(m_pMainWnd);

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes();

	// Parse the command line to see if launched as OLE server
	// NOTE: These modify the command line string
	if (RunEmbedded() || RunAutomated())
	{
		// Application was run with /Embedding or /Automation.  Don't show the
		//  main window in this case.
		return TRUE;
	}

	// When a server application is launched stand-alone, it is a good idea
	//  to update the system registry in case it has been damaged.
	m_server.UpdateRegistry(OAT_INPLACE_SERVER);
	COleObjectFactory::UpdateRegistryAll();

	// simple command line parsing
	CString sRem = ParseCmdLine();
	if (m_sFile.IsEmpty())
	{
		// create a new (empty) document
		OnFileNew();
	}
	else
	{
		// open an existing document
		OpenDocumentFile(m_sFile);
	}

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	return TRUE;
}


BOOL CWordzApp::ExitInstance()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzApp::ExitInstance()\n"));

	// Cleanup the static part of the lookup class (eg, MediaView access)
	CLookup::CleanupLookup();

	// Invoke the base class cleanup (eg, flush INI settings)
	CWinApp::ExitInstance();

	END_DEBUGGING();

	return (TRUE);
}


/////////////////////////////////////////////////////////////////////////////
// CWordzApp commands


// App command to run the dialog
void CWordzApp::OnAppAbout()
{
	CAboutDlg aboutDlg(AfxGetMainWnd());
	aboutDlg.DoModal();
}


CString CWordzApp::ParseCmdLine (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzApp::ParseCmdLine()\n"));
    INT i = 0;  
	CString sRemainder;

	CONST CStringx& line = m_lpCmdLine;
	::ParseCmdLine(line);
    while (i < line.GetLength()) {
        SkipSpaces(line, i);
		INT nStartPos = i;
        if (i == line.GetLength()) {
            break;
            }
        if (IsCommandSwitchDelim(line[i])) {
			i++;
            CHAR cOption = line[i++];
            switch (ToUpper(cOption)) {
				case '?':
					DEBUG_OUT(TL_DETAILED, ("Usage: wordz [filename]\n"));
					break;
                default:
					i -= 2;
					sRemainder += ExtractToken(line, i, " \t");
					sRemainder += cSPACE;
                    break;
                }
            }
		else if (m_sFile.IsEmpty()) {
			m_sFile = ExtractToken(line, i, " \t");
		}
		else {
			sRemainder += ExtractToken(line, i, " \t");
			sRemainder += cSPACE;
		}
		if (nStartPos == i) {
			ASSERT_ERROR();
			i++;
		}
	}
	DEBUG_OUT(TL_VERBOSE, ("CWordzApp::ParseCmdLine => '%s'\n", F(sRemainder)));

	return (sRemainder);
}

