//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// wordzdoc.cpp : implementation of the CWordzDoc class
//

#include "stdafx.h"
#include "Wordz.h"

#include "mainfrm.h"
#include "wordzfrm.h"
#include "wordzdoc.h"
#include "srvritem.h"
#include "lookup.h"
#include "rtfedit.h"
#include "wordzvw.h"
#include "options.h"


IMPLEMENT_DYNCREATE(CWordzDoc, COleServerDoc)
IMPLEMENT_SERIAL(CWordInfo, CObject, 5)


#if defined(_DEBUG)
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


/////////////////////////////////////////////////////////////////////////////
// CWordInfo

CWordInfo::CWordInfo (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordInfo::CWordInfo()\n"));
    m_Definition = "";
    m_CrossRefs = "";
    m_Synonyms = "";
    m_Word = "";
    m_Descendants = "";
	m_rect.SetRectEmpty();
	m_nPreviousItem = nINVALID_INDEX;
    //m_bLookupDef = TRUE;
    //m_bLookupSyns = TRUE;
    //m_bLookupXrefs = FALSE;
#if FALSE
    m_bLookupDef = AfxGetApp()->GetProfileInt("Options", "LookupDefinition", TRUE);
    m_bLookupSyns = AfxGetApp()->GetProfileInt("Options", "LookupSynonyms", TRUE);
    m_bLookupXrefs = AfxGetApp()->GetProfileInt("Options", "LookupCrossRefs", TRUE);
    m_bLookupDescs = AfxGetApp()->GetProfileInt("Options", "LookupDescendants", TRUE);
#endif
    m_bLookupDef = GetAppOption(szLOOKUP_DEFINITION, TRUE);
    m_bLookupSyns = GetAppOption(szLOOKUP_SYNONYMS, TRUE);
    m_bLookupXrefs = GetAppOption(szLOOKUP_REFERENCES, FALSE);
    m_bLookupDescs = GetAppOption(szLOOKUP_DESCENDANTS, FALSE);

	DEBUG_OUT(TL_VERBOSE, ("m_bLookup{Def, Syns, Xrefs, Desc} = (%d, %d, %d, %d)\n",
			  m_bLookupDef, m_bLookupSyns, m_bLookupXrefs, m_bLookupDescs));
}
             
CWordInfo::~CWordInfo (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordInfo::~CWordInfo()\n"));
}


VOID CWordInfo::Serialize (CArchive& ar)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordInfo::Serialize(%x)\n", ar));
    CObject::Serialize(ar);
    if (ar.IsStoring()) {
        ar << m_Word << m_Definition << m_CrossRefs << m_Synonyms << m_Descendants;
        ar << ToLong(m_bLookupDef) << ToLong(m_bLookupSyns) << ToLong(m_bLookupXrefs)
		   << ToLong(m_bLookupDescs) << ToLong(m_nPreviousItem);
		ar << m_rect.left << m_rect.top << m_rect.right << m_rect.bottom;
        }
    else {
        ar >> m_Word >> m_Definition >> m_CrossRefs >> m_Synonyms >> m_Descendants;
        LONG lValue;
        ar >> lValue;
        m_bLookupDef = ToInt(lValue);
        ar >> lValue;
        m_bLookupSyns = ToInt(lValue);
        ar >> lValue;
        m_bLookupXrefs = ToInt(lValue);
        ar >> lValue;
        m_bLookupDescs = ToInt(lValue);
        ar >> lValue;
        m_nPreviousItem = ToInt(lValue);
		ar >> m_rect.left >> m_rect.top >> m_rect.right >> m_rect.bottom;
        }
}


#if defined(_DEBUG)

VOID CWordInfo::AssertValid() const
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordInfo::AssertValid()\n"));
    CObject::AssertValid();
}


VOID CWordInfo::Dump(CDumpContext& dc) const
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordInfo::Dump(%x)\n", dc));
    CObject::Dump(dc);

	DUMP_VAR(m_Word);
	DUMP_VAR(m_Definition);
	DUMP_VAR(m_Synonyms);
	DUMP_VAR(m_CrossRefs);
	DUMP_VAR(m_Descendants);
	DUMP_VAR(m_bLookupDef);
	DUMP_VAR(m_bLookupSyns);
	DUMP_VAR(m_bLookupXrefs);
	DUMP_VAR(m_bLookupDescs);
	DUMP_VAR(m_nPreviousItem);
	DUMP_VAR(m_rect);
#if FALSE
	CString sRect;
	sRect.Format("(%d, %d, %d, %d)", 
				 m_rect.left, m_rect.top, m_rect.right, m_rect.bottom);
	dc << "m_rect =" << sRect << "\n";
#endif
}

#endif // defined(_DEBUG)


/////////////////////////////////////////////////////////////////////////////
// CWordzDoc

BEGIN_MESSAGE_MAP(CWordzDoc, COleServerDoc)
	//{{AFX_MSG_MAP(CWordzDoc)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CWordzDoc, COleServerDoc)
	//{{AFX_DISPATCH_MAP(CWordzDoc)
	DISP_FUNCTION(CWordzDoc, "LookupWord", LookupWord, VT_BSTR, VTS_BSTR)
	DISP_FUNCTION(CWordzDoc, "RefreshWindow", Refresh, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CWordzDoc, "ShowWindow", ShowWindow, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CWordzDoc, "LookupDef", LookupDef, VT_EMPTY, VTS_BSTR)
	DISP_FUNCTION(CWordzDoc, "LookupSyns", LookupSyns, VT_EMPTY, VTS_BSTR)
	DISP_FUNCTION(CWordzDoc, "LookupXrefs", LookupXrefs, VT_EMPTY, VTS_BSTR)
	DISP_FUNCTION(CWordzDoc, "LookupDescs", LookupDescs, VT_EMPTY, VTS_BSTR)
	DISP_FUNCTION(CWordzDoc, "TestConnection", TestConnection, VT_BOOL, VTS_NONE)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWordzDoc construction/destruction

CWordzDoc::CWordzDoc()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::CWordzDoc()\n"));
	//	So instance initialization
    m_nCurrentPos = 0;
	m_bJustSerialized = FALSE;
	m_posView = NULL;
	m_bDontAskToSave = TRUE;
	m_pCurrentView = NULL;

	//	Do OLE support initialization
	EnableAutomation();
	AfxOleLockApp();

	// m_bDontAskToSave = AfxGetApp()->GetProfileInt("Options", "DontAskToSave", TRUE);
	m_bDontAskToSave = GetAppOption(szDONT_ASK_TO_SAVE, TRUE);
	DEBUG_OUT(TL_VERBOSE, ("m_bDontAskToSave=%d\n", m_bDontAskToSave));
}


CWordzDoc::~CWordzDoc()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::~CWordzDoc()\n"));

	//	Instance cleanup
    Empty();

	//	OLE support cleanup
	AfxOleUnlockApp();
}


INT CWordzDoc::GetNextIndex (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::GetNextIndex()\n"));
    INT nNextPos = m_nCurrentPos++;

    if (nNextPos == m_anPlacement.GetSize()) {
        CWordInfo* pinfo = new CWordInfo;
        m_words.Add(pinfo);
		m_anPlacement.Add(nNextPos);
        }

    return (m_anPlacement[nNextPos]);
}


VOID CWordzDoc::Empty (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::Empty()\n"));

    for (INT i = 0; i < m_words.GetSize(); i++) {
        CWordInfo* pinfo = ToCWordInfo(m_words[i]);
        ASSERT_VALID(pinfo);
        delete pinfo;
        }
    m_words.RemoveAll();
    m_anPlacement.RemoveAll();
}

             
BOOL CWordzDoc::OnNewDocument()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::OnNewDocument()\n"));

	if (!COleServerDoc::OnNewDocument()) {
		DEBUG_OUT(TL_BASIC, ("Couldn't create a new ole server doc\n"));
		return FALSE;
		}

    Empty();
    m_nCurrentPos = 0;

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CWordzDoc server implementation


COleServerItem* CWordzDoc::OnGetEmbeddedItem()
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::OnGetEmbeddedItem()\n"));
	// OnGetEmbeddedItem is called by the framework to get the COleServerItem
	//  that is associated with the document.  It is only called when necessary.

	CWordzSrvrItem* pItem = new CWordzSrvrItem(this);
	ASSERT_VALID(pItem);
	return pItem;
}


/////////////////////////////////////////////////////////////////////////////
// CWordzDoc serialization


// DetermineItemOrder: order the placement array by reverse z-order for the
// items. This is only done when the document is read since the view maintains
// it implicitly during a session.
//
// TODO: optimize this topological sort
//
void CWordzDoc::DetermineItemOrder (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::DetermineItemOrder()\n"));
	m_anPlacement.SetSize(m_words.GetSize());
	if (m_anPlacement.GetSize() == 0) {
		return;
	}

	CArray<INT, INT> abProcessed;
	abProcessed.SetSize(m_words.GetSize());
	PLONG pData = (PLONG)abProcessed.GetData();
	ZeroMemory(pData, abProcessed.GetSize() * sizeof(abProcessed[0]));

	// Fill the placement array in reverse z-order, so that the rearmost
	// item will be processed first.
	DEBUG_OUT(TL_VERBOSE, ("Item arrangement: "));
	for (INT i = m_words.GetSize() - 1; i >= 0; i--) {
		DEBUG_OUT(TL_VERBOSE, ("(%d, %d) ", i, ToCWordInfo(m_words[i])->m_nPreviousItem));
		// Find next item that is behind no other remaining windows
		INT nNext = nINVALID_INDEX;
		for (INT j = 0; j < m_words.GetSize(); j++) {
			CWordInfo* pInfo = ToCWordInfo(m_words[j]);
			if ((!abProcessed[j]) &&
				((pInfo->m_nPreviousItem == nINVALID_INDEX) 
					|| (abProcessed[pInfo->m_nPreviousItem]))) {
				nNext = j;
				abProcessed[j] = TRUE;
				break;
			}
		}
		ASSERT(nNext != nINVALID_INDEX);
		m_anPlacement[i] = nNext;
	}
	DEBUG_OUT(TL_VERBOSE, ("\n"));

	// Dump the results of the topological sort
	DEBUG_OUT(TL_VERBOSE, ("Item order: "));
	for (i = 0; i < m_anPlacement.GetSize(); i++) {
		DEBUG_OUT(TL_VERBOSE, ("%d ", m_anPlacement[i]));
	}
	DEBUG_OUT(TL_VERBOSE, ("\n"));
}


void CWordzDoc::Serialize(CArchive& ar)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::Serialize(%x)\n", ar));
	m_bJustSerialized = TRUE;
    CDocument::Serialize(ar);
    m_words.Serialize(ar);
    if (ar.IsStoring()) {
        }
    else {
		m_nCurrentPos = 0;
#if TRUE
		// Topologically sort the word objects by z-order, so that the placement
		// is the same as when the document was saved
		DetermineItemOrder();
#endif
		if (!IsEmbedded()) {
        	OpenOtherWindows();
			}
        }
	//m_bJustSerialized = FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// CWordzDoc diagnostics

#if defined(_DEBUG)

void CWordzDoc::AssertValid() const
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzDoc::AssertValid()\n"));
	COleServerDoc::AssertValid();
	m_words.AssertValid();
	m_anPlacement.AssertValid();
}


void CWordzDoc::Dump(CDumpContext& dc) const
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzDoc::Dump(%x)\n", dc));
	COleServerDoc::Dump(dc);

	DUMP_OBJ(m_words);
	DUMP_VAR(m_nCurrentPos);
	DUMP_VAR(m_bJustSerialized);
	DUMP_VAR(m_posView);
	DUMP_VAR(m_bDontAskToSave);
	DUMP_VAR(m_pCurrentView);
	DUMP_OBJ(m_anPlacement);
}

#endif // defined(_DEBUG)


/////////////////////////////////////////////////////////////////////////////
// CWordzDoc commands


void CWordzDoc::OnFileClose (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::OnFileClose()\n"));
    CDocument::OnFileClose();

#if FALSE
    POSITION pos = GetFirstViewPosition();
    while (pos != NULL) {
        CView* pview = GetNextViewPosition(pos);
        pView->Close
        }
#endif
}


// Create additional windows for the word entries
//
#if FALSE
VOID CWordzDoc::OpenOtherWindows (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::OpenOtherWindows()\n"));
	ASSERT(!IsEmbedded());
    //POSITION pos = GetFirstViewPosition();
    //ASSERT(pos);
    //if (pos != NULL) {
        //CView* pview = GetFirstView(pos);
        for (INT i = 1; i < m_words.GetSize(); i++) {
            //  pview->SendMessage(ID_WINDOW_NEW);
            // AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_WINDOW_NEW);
			GetFrame()->PostMessage(WM_COMMAND, ID_WINDOW_NEW);
            }
        //}
}
#endif


VOID CWordzDoc::OpenOtherWindows (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::OpenOtherWindows()\n"));
	ASSERT(!IsEmbedded());

	for (INT i = 1; i < m_words.GetSize(); i++) {
		GetFrame()->PostMessage(WM_COMMAND, ID_WINDOW_NEW);
	}
}


VOID CWordzDoc::OnEditCopy (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::OnEditCopy()\n"));

	CWordzSrvrItem* pItem = GetEmbeddedItem();
	pItem->CopyToClipboard(TRUE);
}


// Refresh: update the word item view from the document
// If no view is specified, all views are updated
//
// NOTE: Currently, the client must call this via RefreshWindow in order
//		 for the view window to be induced.
//

VOID CWordzDoc::Refresh (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::Refresh()\n"));

	Refresh(NULL);

#if TRUE
	// Maximize the view (ie, behave like an SDI app)
	CWordzView* pView = (m_pCurrentView ? m_pCurrentView : FirstView());
	ASSERT(pView);
	if (pView) {
		CWordzFrame* pFrameWnd = ToCWordzFrame(pView->GetParentFrame());
		ASSERT(pFrameWnd);
		pFrameWnd->MDIMaximize();
		}
#endif
}


VOID CWordzDoc::Refresh (CWordzView* pWordzView)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::Refresh(%x)\n", pWordzView));

	SetModifiedFlag();
	if (pWordzView) {
		pWordzView->UpdateView();
		}
	else {
		UpdateAllViews(NULL);
		}
}


#if FALSE
VOID CWordzDoc::ShowWindow (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::ShowWindow()\n"));

	POSITION pos = GetFirstViewPosition();
	while (pos != NULL) {
		CView* pView = GetNextView(pos);
		if (pView != NULL) {
			CFrameWnd* pFrameWnd = pView->GetParentFrame();
			pFrameWnd->ActivateFrame(SW_SHOW);
			pFrameWnd = pFrameWnd->GetParentFrame();
			if (pFrameWnd != NULL) {
				pFrameWnd->ActivateFrame(SW_SHOW);
				}
			}
		}
}
#endif


// ShowWindow: make the view visible along with its associated frame window
// If no view is specified all views are made visible (ie, activated).
//

VOID CWordzDoc::ShowWindow (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::ShowWindow()\n"));

	CWordzView* pView = FirstView();
	while (pView != NULL) {
		ShowWindow(pView);
		pView = NextView();
		}
}


VOID CWordzDoc::ShowWindow (CWordzView* pView)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzDoc::ShowWindow(%x)\n", pView));

	CFrameWnd* pFrameWnd = pView->GetParentFrame();
	pFrameWnd->ActivateFrame(SW_SHOW);
	pFrameWnd = pFrameWnd->GetParentFrame();
	if (pFrameWnd != NULL) {
		pFrameWnd->ActivateFrame(SW_SHOW);
		}
	pView->SetFocus();
}


// BringToForeground: bring the mainframe window to the foreground
//

VOID CWordzDoc::BringToForeground (VOID)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::FirstView()\n"));

	GetFrame()->SetForegroundWindow();
}


// FirstView: return the first view for the document 
// This also update the current position to the next view to support iteration
//

CWordzView* CWordzDoc::FirstView (VOID)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::FirstView()\n"));

	m_posView = GetFirstViewPosition();
	return (NextView());
}


// NextView: return the next view for the document and update the current 
// position indicator to the subsequent view (to support iteration)
//

CWordzView* CWordzDoc::NextView (VOID)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::NextView()\n"));
	CWordzView* pView = NULL;

	if (m_posView != NULL) {
		pView = (CWordzView*)GetNextView(m_posView);
		}

	return (pView);
}


// LookupWord: lookup the definition of the word, display it, and return it
// TODO: give this a more meaningful name
//

BSTR CWordzDoc::LookupWord (LPCTSTR pszWord) 
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::LookupWord(%s)\n", F(pszWord)));
	ASSERT(m_words.GetSize() > 0);
	CString sResult;
	
	if (m_words.GetSize() > 0) {
		CWordInfo* pInfo = GetCurrentInfo();
		if (pInfo) {
			pInfo->SetWord(pszWord);
			//CWordzView* pView = FirstView();
			CWordzView* pView = (m_pCurrentView ? m_pCurrentView : FirstView());
			//ShowWindow(pView);
			Refresh(pView);
			if (pView != NULL) {
				pView->JustDoIt();
				}
			sResult = pInfo->GetDefinition();
			BringToForeground();
			}
		else {
			DEBUG_OUT(TL_BASIC, ("ERROR: No current word item\n"));
			}
		}
	else {
		DEBUG_OUT(TL_BASIC, ("ERROR: No word information entries\n"));
		}
	
	return (sResult.AllocSysString());
}


// LookupDef: lookup the definition of the word and display it
// TODO: return it as well (the client then can ignore it at will)
//

VOID CWordzDoc::LookupDef (LPCTSTR pszWord)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::LookupDef(%s)\n", F(pszWord)));
	ASSERT(m_words.GetSize() > 0);

	if (m_words.GetSize() > 0) {
	    CWordInfo* pInfo = GetCurrentInfo();
		if (pInfo) {
			pInfo->SetWord(pszWord);
			pInfo->m_bLookupDef = TRUE;
			pInfo->m_bLookupSyns = FALSE;
			pInfo->m_bLookupXrefs = FALSE;
			pInfo->m_bLookupDescs = FALSE;
		}
		//CWordzView* pView = FirstView();
		CWordzView* pView = (m_pCurrentView ? m_pCurrentView : FirstView());
		//ShowWindow(pView);
		Refresh(pView);
		if (pView != NULL) {
			pView->JustDoIt();
			}

		BringToForeground();
		}
}


// LookupSyns: lookup the synonyms of the word and display them
// TODO: return them as well (the client then can ignore it at will)
//

VOID CWordzDoc::LookupSyns (LPCTSTR pszWord)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::LookupSyns(%s)\n", F(pszWord)));
	ASSERT(m_words.GetSize() > 0);

	if (m_words.GetSize() > 0) {
	    CWordInfo* pInfo = GetCurrentInfo();
		if (pInfo) {
			pInfo->SetWord(pszWord);
			pInfo->m_bLookupDef = FALSE;
			pInfo->m_bLookupSyns = TRUE;
			pInfo->m_bLookupXrefs = FALSE;
			pInfo->m_bLookupDescs = FALSE;
		}
		//CWordzView* pView = FirstView();
		CWordzView* pView = (m_pCurrentView ? m_pCurrentView : FirstView());
		//ShowWindow(pView);
		Refresh(pView);
		if (pView != NULL) {
			pView->JustDoIt();
			}

		BringToForeground();
		}
}


// LookupXrefs: lookup the cross-references of the word and display them
// TODO: return them as well (the client then can ignore it at will)
//

VOID CWordzDoc::LookupXrefs (LPCTSTR pszWord)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::LookupXrefs(%s)\n", F(pszWord)));
	ASSERT(m_words.GetSize() > 0);

	if (m_words.GetSize() > 0) {
	    CWordInfo* pInfo = GetCurrentInfo();
		if (pInfo) {
			pInfo->SetWord(pszWord);
			pInfo->m_bLookupDef = FALSE;
			pInfo->m_bLookupSyns = FALSE;
			pInfo->m_bLookupXrefs = TRUE;
			pInfo->m_bLookupDescs = FALSE;
		}
		//CWordzView* pView = FirstView();
		CWordzView* pView = (m_pCurrentView ? m_pCurrentView : FirstView());
		//ShowWindow(pView);
		Refresh(pView);
		if (pView != NULL) {
			pView->JustDoIt();
			}

		BringToForeground();
		}
}


// LookupXrefs: lookup the descendants of the word and display them
// The descendants are from the WordNet hyponym tree.
// TODO: return it as well (the client then can ignore it at will)
//

VOID CWordzDoc::LookupDescs (LPCTSTR pszWord)
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::LookupDescs(%s)\n", F(pszWord)));
	ASSERT(m_words.GetSize() > 0);

	if (m_words.GetSize() > 0) {
	    CWordInfo* pInfo = GetCurrentInfo();
		if (pInfo) {
			pInfo->SetWord(pszWord);
			pInfo->m_bLookupDef = FALSE;
			pInfo->m_bLookupSyns = FALSE;
			pInfo->m_bLookupXrefs = FALSE;
			pInfo->m_bLookupDescs = TRUE;
		}
		//CWordzView* pView = FirstView();
		CWordzView* pView = (m_pCurrentView ? m_pCurrentView : FirstView());
		ShowWindow(pView);
		Refresh(pView);
		if (pView != NULL) {
			pView->JustDoIt();
			}

		BringToForeground();
		}
}


// TestConnection: Determine if the OLE connection is valid
//

BOOL CWordzDoc::TestConnection (VOID) 
{
	DEBUG_OUTF(TL_VERBOSE, ("CWordzDoc::TestConnection()\n"));
    ASSERT_VALID(this);

	return (TRUE);
}
