//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// wordzdoc.h : interface of the CWordzDoc class
//
/////////////////////////////////////////////////////////////////////////////

class CWordzSrvrItem;

class CWordzView;


class CWordInfo : public CObject
{
	DECLARE_SERIAL(CWordInfo)
public:
	CWordInfo (VOID);

// Attributes
public:
	const CStringx& GetWord (VOID)			{ return (m_Word); }
	const CStringx& GetDefinition (VOID)	{ return (m_Definition); }
	const CStringx& GetCrossRefs (VOID)		{ return (m_CrossRefs); }
	const CStringx& GetSynonyms (VOID)		{ return (m_Synonyms); }
	const CStringx& GetDescendants (VOID)	{ return (m_Descendants); }
	const CRect& GetRect (VOID)				{ return (m_rect); }

// Operations
public:
	VOID SetWord (PCSZ pszWord)				{ m_Word = pszWord; }
	VOID SetDefinition (PCSZ pszDefinition)	{ m_Definition = pszDefinition; }
	VOID SetCrossRefs (PCSZ pszCrossRefs)	{ m_CrossRefs = pszCrossRefs; }
	VOID SetSynonyms (PCSZ pszSynonyms)		{ m_Synonyms = pszSynonyms; }
	VOID SetDescendants (PCSZ pszDescendants)	{ m_Descendants = pszDescendants; }
	VOID SetRect (CONST CRect rect)			{ m_rect = rect; }

// Implementation
public:
	~CWordInfo (VOID);
	virtual VOID Serialize (CArchive& ar);
#if defined(_DEBUG)
	virtual VOID AssertValid (VOID) const;
	virtual VOID Dump (CDumpContext& dc) const;
#endif
private:
	CStringx	m_Word;					// the word or phrase of interest (headword)
	CStringx	m_Definition;			// definition for the entry
	CStringx	m_CrossRefs;			// list of other entries that reference the word
	CStringx	m_Synonyms;				// list of synonyms for the headword
	CStringx	m_Descendants;			// list of words in WordNet hyponym tree
	BOOL	m_bLookupDef;				// indicates dictionary lookup desired
	BOOL	m_bLookupSyns;				// indicates synonym lookup desired
	BOOL	m_bLookupXrefs;				// indicates cross-reference lookup desired
	BOOL	m_bLookupDescs;				// indicates descendant tree lookup desired
	CRect	m_rect;						// window rectangle within view
	INT		m_nPreviousItem;			// index of previous item in z-order
private:
	friend class CWordzDoc;
	friend class CWordzView;
};

MAKE_CLASS_CAST(CWordInfo);


class CWordzDoc : public COleServerDoc
{
protected: // create from serialization only
	CWordzDoc();
	DECLARE_DYNCREATE(CWordzDoc)

// Attributes
public:
	CWordzSrvrItem* GetEmbeddedItem (VOID)
		{ return (CWordzSrvrItem*)COleServerDoc::GetEmbeddedItem(); }
	INT GetCurrentIndex (VOID)
		{ ASSERT(VALID_INDEX(m_nCurrentPos - 1, m_anPlacement));
		  return m_anPlacement[m_nCurrentPos - 1]; }
	CWordInfo* GetInfo (INT nPos)	
		{ ASSERT(ValidIndex(nPos, m_words));
		  //return ((CWordInfo*)m_words[nPos]); }
		  return (ToCWordInfo(m_words[nPos])); }
	CWordInfo* GetCurrentInfo (VOID)
		{ return (GetInfo(GetCurrentIndex())); }
	BOOL MoreEntries (VOID)	
		{ return (m_nCurrentPos < m_anPlacement.GetSize()); }
	BOOL MoreEntries (INT nPos)	
		{ return ((nPos + 1) < m_anPlacement.GetSize()); }


// Operations
public:
	INT GetNextIndex();
	VOID Empty(VOID);
	CWordzView* FirstView (VOID);
	CWordzView* NextView (VOID);
	VOID ShowWindow (CWordzView* pView);
	VOID Refresh (CWordzView* pView);
	VOID BringToForeground (VOID);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CWordzDoc)
	protected:
	virtual COleServerItem* OnGetEmbeddedItem();
	public:
	virtual BOOL OnNewDocument();
	//}}AFX_VIRTUAL
	virtual BOOL SaveModified (VOID)	{ return (m_bDontAskToSave); }

// Implementation
public:
	virtual ~CWordzDoc();
	virtual void Serialize(CArchive& ar);   // overridden for document i/o
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	VOID DetermineItemOrder (VOID);
	VOID OpenOtherWindows (VOID);
private:
	CObArray m_words;						// collection of word information
	CArray<INT, INT> m_anPlacement;			// placement order for word items
	INT m_nCurrentPos;						// current position in placement array
	BOOL m_bJustSerialized;					// was the document just input or output?
	POSITION m_posView;						// position for iterating views
	BOOL	m_bDontAskToSave;				// dont prompt to save document at end
	CWordzView* m_pCurrentView;				// HACK: current view being displayed
	friend class CWordzView;

// Generated message map functions
protected:
	//{{AFX_MSG(CWordzDoc)
	afx_msg void OnFileClose();
	afx_msg void OnEditCopy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(CWordzDoc)
	afx_msg BSTR LookupWord(LPCTSTR pszWord);
	afx_msg void Refresh();
	afx_msg void ShowWindow();
	afx_msg void LookupDef(LPCTSTR pszWord);
	afx_msg void LookupSyns(LPCTSTR pszWord);
	afx_msg void LookupXrefs(LPCTSTR pszWord);
	afx_msg void LookupDescs(LPCTSTR pszWord);
	afx_msg BOOL TestConnection();
	//}}AFX_DISPATCH
	DECLARE_DISPATCH_MAP()
};

MAKE_CLASS_CAST(CWordzDoc);


/////////////////////////////////////////////////////////////////////////////
