//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// wordzvw.cpp : implementation of the CWordzView class
//

#include "stdafx.h"
#include "Wordz.h"
#include "lookup.h"
#include "wordzdoc.h"
#include "rtfedit.h"
#include "wordzvw.h"
#include "mainfrm.h"
#include "Options.h"

// for lookup.h
INLINE CString ToRtf (CONST CString& sText)
{
	CString sRtfText (szRTF_HEADER);
	sRtfText += sText;
	sRtfText += szRTF_TRAILER;

	return (sRtfText);
}


#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

//  Local functions
void StripTrailer (CStringx& text);

//	Other functions
CRect ToTwips (CDC* pDC, CRect& rectLogical);	// in rtfedit.cpp


//	Constants
//
//COLORREF clrLIGHT_GRAY = RGB(192, 192, 192);
//COLORREF clrGRAY = RGB(128, 128, 128);

// Slimy Globals
//
CWnd* pLastEditWnd = NULL;			// last edit window to receive click

/////////////////////////////////////////////////////////////////////////////
// CWordzView

IMPLEMENT_DYNCREATE(CWordzView, CFormView)

BEGIN_MESSAGE_MAP(CWordzView, CFormView)
	//{{AFX_MSG_MAP(CWordzView)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
    ON_BN_DOUBLECLICKED(IDC_DO_DEF, OnDoubleclickedDef)
    ON_COMMAND(IDM_DOIT, OnDoit)
    ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
    ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
    ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
    ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
    ON_COMMAND(ID_EDIT_CUT, OnEditCut)
    ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
    ON_WM_LBUTTONDBLCLK()
    ON_COMMAND(ID_EDIT_CHANGEFONT, OnEditChangefont)
    ON_WM_SIZE()
    ON_COMMAND(IDC_DO_DEF, OnDoDef)
    ON_COMMAND(IDC_DO_SYNS, OnDoSyns)
    ON_COMMAND(IDC_DO_XREFS, OnDoXrefs)
    ON_COMMAND(IDC_DO_DESCS, OnDoDescs)
    ON_UPDATE_COMMAND_UI(IDC_DO_DEF, OnUpdateDoDef)
    ON_UPDATE_COMMAND_UI(IDC_DO_SYNS, OnUpdateDoSyns)
    ON_UPDATE_COMMAND_UI(IDC_DO_XREFS, OnUpdateDoXrefs)
    ON_UPDATE_COMMAND_UI(IDC_DO_DESCS, OnUpdateDoDescs)
	ON_WM_CTLCOLOR()
	ON_COMMAND(ID_VIEW_FONT, OnViewFont)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FONT, OnUpdateViewFont)
	ON_UPDATE_COMMAND_UI(IDM_DOIT, OnUpdateDoit)
	ON_WM_TIMER()
    ON_COMMAND(ID_VIEW_LOOKUP, OnDoit)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LOOKUP, OnUpdateDoit)
    ON_BN_CLICKED(IDC_DOIT, OnDoit)
	ON_WM_MOVE()
	ON_WM_WINDOWPOSCHANGED()
	//}}AFX_MSG_MAP
    //  Entries to force printing support
    ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
    ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	//ON_COMMAND(ID_PROCESS_RESULTS, ProcessLookupResults)
	//	My edit extension notifications
	ON_NOTIFY(WM_KEYDOWN, IDC_WORD, OnDefinitionKeyDown)
	ON_NOTIFY(WM_LBUTTONDBLCLK, IDC_DEFINITION, OnNotifyDblClick)
	ON_NOTIFY(WM_LBUTTONDBLCLK, IDC_SYNONYMS, OnNotifyDblClick)
	ON_NOTIFY(WM_LBUTTONDBLCLK, IDC_CROSSREFS, OnNotifyDblClick)
	ON_NOTIFY(WM_LBUTTONDBLCLK, IDC_DESCENDANTS, OnNotifyDblClick)
	ON_NOTIFY(WM_LBUTTONDOWN, IDC_DEFINITION, OnNotifyButtonDown)
	ON_NOTIFY(WM_LBUTTONDOWN, IDC_SYNONYMS, OnNotifyButtonDown)
	ON_NOTIFY(WM_LBUTTONDOWN, IDC_CROSSREFS, OnNotifyButtonDown)
	ON_NOTIFY(WM_LBUTTONDOWN, IDC_DESCENDANTS, OnNotifyButtonDown)
	//	Internal lookup commands
    ON_COMMAND(ID_LOOKUP_DEFINITION, LookupDef)
    ON_COMMAND(ID_LOOKUP_SYNONYMS, LookupSyns)
    ON_COMMAND(ID_LOOKUP_REFERENCES, LookupXrefs)
    ON_COMMAND(ID_LOOKUP_DESCENDANTS, LookupDescs)
    ON_COMMAND(ID_PROCESS_DEFINITION, ProcessDef)
    ON_COMMAND(ID_PROCESS_SYNONYMS, ProcessSyns)
    ON_COMMAND(ID_PROCESS_REFERENCES, ProcessXrefs)
    ON_COMMAND(ID_PROCESS_DESCENDANTS, ProcessDescs)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWordzView construction/destruction

CWordzView::CWordzView (VOID) :
    CFormView(CWordzView::IDD)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::CWordzView()\n"));
    //{{AFX_DATA_INIT(CWordzView)
    m_Word = "";
    m_Definition = "";
    m_Synonyms = "";
    m_CrossRefs = "";
    m_Descendants = "";
    m_bLookupDef = FALSE;
    m_bLookupSyns = FALSE;
    m_bLookupXrefs = FALSE;
    m_bLookupDescs = FALSE;
    //}}AFX_DATA_INIT

	m_pLookup = NULL;
    m_bInitialLookup = FALSE;
    m_nDocIndex = nINVALID_INDEX;
	m_rect.SetRectEmpty();
	SetDuringLookup(FALSE);
	m_nNextCommand = nINVALID_INDEX;
	m_nPreviousItem = nINVALID_INDEX;
	m_auLookupCommands.Add(ID_LOOKUP_DEFINITION);
	m_auLookupCommands.Add(ID_LOOKUP_SYNONYMS);
	m_auLookupCommands.Add(ID_LOOKUP_REFERENCES);
	m_auLookupCommands.Add(ID_LOOKUP_DESCENDANTS);
	m_clrBackground = clrLIGHT_GRAY;

	// Get the options for the view
	GetOptions();
}


CWordzView::~CWordzView()
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::~CWordzView()\n"));
#if FALSE
	if (m_pLookupThread) {
		TerminateThread(m_pLookupThread->m_hThread, 666);
		m_pLookupThread = NULL;
		}
#endif

	if (m_pLookup) {
		delete m_pLookup;
		m_pLookup = NULL;
	}
	//DeleteCriticalSection(&m_csLookup);
	//CloseHandle(m_hLookupEvent);
	pLastEditWnd = NULL;		// clear so we don't get stale pointers
}


// GetOptions: Get the user options that affect Wordz views. The main purpose
// is to get the text and background colors.
//
// NOTE: Since SetBkColor doesn't support dithered colors, the color will
//	     be converted to the nearest solid color
//		 See KB article Q69885: "SetBkColor() Does Not Support Dithered Colors"
//
VOID CWordzView::GetOptions (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::GetOptions()\n"));

	m_clrText = GetAppOption(szTEXT_COLOR, clrBLACK);

	// Do special case handling for the clue text background
	//		get nearest solid color
	//		update the brush used for painting backgrounds
	COLORREF colorBackground = GetAppOption(szTEXT_BACKGROUND, clrLIGHT_GRAY);
	CWindowDC dc(NULL);
	m_clrBackground = dc.GetNearestColor(colorBackground);
	DEBUG_OUT(TL_VERBOSE, ("m_clrBackground = 0x%08x (\"TextBackground\" = 0x%08x)\n", 
			  m_clrBackground, colorBackground));
	if (NULL != m_brushBackground.GetSafeHandle()) {
		m_brushBackground.DeleteObject();
		}
	m_brushBackground.CreateSolidBrush(m_clrBackground);
}


BOOL CWordzView::Create (LPCSTR lpszClassName, LPCSTR lpszWindowName, DWORD dwStyle, 
                        const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::Create(%s, %s, %x, %x, %x, %x, %x)\n",
    		  F1(lpszClassName), F2(lpszWindowName), dwStyle, rect, pParentWnd, nID, pContext));
    dwStyle |= CS_DBLCLKS;
    return (CFormView::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext));
}


void CWordzView::OnInitialUpdate (void) 
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnInitialUpdate()\n"));
    CWordzDoc* pDoc = GetDocument();

	//	Override the edit box creation (bogus size at first)
    CRect rect(666, 666, 666, 666);
	DWORD dwEdStyle = WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_MULTILINE 
					  | WS_VSCROLL;
	m_edWord.Create(dwEdStyle, rect, this, IDC_WORD);
	dwEdStyle |= ES_READONLY;
	m_edDef.Create(dwEdStyle, rect, this, IDC_DEFINITION);
	m_edSyns.Create(dwEdStyle, rect, this, IDC_SYNONYMS);
	m_edXref.Create(dwEdStyle, rect, this, IDC_CROSSREFS);
	m_edDescs.Create(dwEdStyle, rect, this, IDC_DESCENDANTS);

#if FALSE
	//	TODO: add a CRtfEdit wrapper for this
	m_edWord.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)m_clrBackground);
	m_edDef.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)m_clrBackground);
	m_edSyns.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)m_clrBackground);
	m_edXref.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)m_clrBackground);
	m_edDescs.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)m_clrBackground);
#endif
	SetWindowColors();

    //  See if the old view had an edit selection active
    //  If so, use the text for the lookup
    //CWnd* pwnd = GetFocus();
	//CWnd* pwnd = FromHandlePermanent(::GetFocus());
	CWnd* pwnd = pLastEditWnd;
    if ((!pDoc->m_bJustSerialized) && pwnd) {
        CWnd* pParentWnd = pwnd->GetParent();
        if (pParentWnd && pParentWnd->IsKindOf(RUNTIME_CLASS(CWordzView))) {
            CWordzView* pWordForm = (CWordzView*)pParentWnd;
            if (pWordForm->IsAnEditControl(pwnd)) {
                CEditx* pedit = (CEditx*)pwnd;
                if (IsSelectionSet(pedit)) {
                    pedit->Copy();
                    m_bInitialLookup = TRUE;
					//	Copy over options from the other view
				    m_bLookupDef = pWordForm->m_bLookupDef;
    				m_bLookupSyns = pWordForm->m_bLookupSyns;
    				m_bLookupXrefs = pWordForm->m_bLookupXrefs;
    				m_bLookupDescs = pWordForm->m_bLookupDescs;
                    }
                }
            }
        }
	pLastEditWnd = NULL;		// clear so we don't get stale pointers

	// Determine which word entry is viewed
    m_nDocIndex = pDoc->GetNextIndex();
//  m_nDocIndex = pDoc->m_words.GetSize();
    if (m_bInitialLookup) {
		//	Set the document options based on settings previously had from other view
	    CWordInfo* pinfo = pDoc->GetInfo(m_nDocIndex);
    	ASSERT_VALID(pinfo);
    	pinfo->m_bLookupDef = m_bLookupDef;
    	pinfo->m_bLookupSyns = m_bLookupSyns;
	    pinfo->m_bLookupXrefs = m_bLookupXrefs;
	    pinfo->m_bLookupDescs = m_bLookupDescs;
		}
    
#if TRUE
	//	Turn off the document's just-serialized flag in this the the last word
	//	TODO: find a better spot for this hack
	if (pDoc->m_bJustSerialized && !pDoc->MoreEntries(m_nDocIndex))
		{
		pDoc->m_bJustSerialized = FALSE;
		}
#endif

	//	Now, let the base class do its usual stuff
    CFormView::OnInitialUpdate();

	//	Create object for doing the lookups for clues
	m_pLookup = new CLookup(this);

	//	Select the word control
    //CEditx* pedit = (CEditx*)GetDlgItem(IDC_WORD);
    //((CDialog*) this)->GotoDlgCtrl(pedit);
	//GotoDlgCtrl(m_edWord);
	m_edWord.SetFocus();
    if (m_bInitialLookup) {
        //pedit->Paste();
		m_edWord.Paste();
        UpdateData(TRUE);
        StripTrailer(m_Word);
        UpdateData(FALSE);
        OnDoit();
        }

	//	Make the doc as not just serialized (HACK)
	//pDoc->m_bJustSerialized = FALSE;

	//	Have the embedded edit controls notify us of double clicks
	ASSERT(m_edDef.m_hWnd != NULL);
	m_edWord.NotifyOfKeystrokes();
	m_edDef.NotifyOfClicks();
	m_edSyns.NotifyOfClicks();
	m_edXref.NotifyOfClicks();
	m_edDescs.NotifyOfClicks();
}


VOID CWordzView::SetWindowColors (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::SetWindowColors()\n"));

	m_edWord.ChangeTextColor(m_clrText);
	m_edDef.ChangeTextColor(m_clrText);
	m_edSyns.ChangeTextColor(m_clrText);
	m_edXref.ChangeTextColor(m_clrText);
	m_edDescs.ChangeTextColor(m_clrText);

	m_edWord.ChangeBackgroundColor(m_clrBackground);
	m_edDef.ChangeBackgroundColor(m_clrBackground);
	m_edSyns.ChangeBackgroundColor(m_clrBackground);
	m_edXref.ChangeBackgroundColor(m_clrBackground);
	m_edDescs.ChangeBackgroundColor(m_clrBackground);
}


void CWordzView::OnUpdate (CView* pChangedView, LPARAM lParam, CObject* pObject)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnUpdate(%x, %x, %x)\n", 
			  pChangedView, lParam, pObject));

	if ((pChangedView != NULL) && (pChangedView != this)) {
		DEBUG_OUT(TL_VERBOSE, ("Changed view is not this view; ignoring OnUpdate\n"));
		return;
	}
	if (m_nDocIndex == nINVALID_INDEX) {
		OnInitialUpdate();
		}

#if TRUE
	if (lParam == lHINT_OPTION_CHANGE) {
		GetOptions();
		SetWindowColors();
		Invalidate();
		UpdateWindow();
		return;
		}
#endif

    //  Get the data from the document
    CWordzDoc* pDoc = GetDocument();
    CWordInfo* pinfo = pDoc->GetInfo(m_nDocIndex);
    ASSERT_VALID(pinfo);
    m_Word = pinfo->m_Word;
    m_Definition = pinfo->m_Definition;
    m_Synonyms = pinfo->m_Synonyms;
    m_CrossRefs = pinfo->m_CrossRefs;
    m_Descendants = pinfo->m_Descendants;
    m_bLookupDef = pinfo->m_bLookupDef;
    m_bLookupSyns = pinfo->m_bLookupSyns;
    m_bLookupXrefs = pinfo->m_bLookupXrefs;
    m_bLookupDescs = pinfo->m_bLookupDescs;
	if (!pinfo->m_rect.IsRectEmpty() && (m_rect != pinfo->m_rect)) {
		m_rect = pinfo->m_rect;
		
		// Move the window to the saved spot, but ensure it is visible
		if (GetAppOption(szSAVE_LAYOUT, TRUE)) {
			DUMP_RECT(TL_VERBOSE, "saved view rect", m_rect, "\n");
			CRect rectFrame;
			pTheOneAndOnlyMainFrame->GetClientRect(rectFrame);
			m_rect.left = min(m_rect.left, rectFrame.right - rectFrame.Width()/4);
			m_rect.right = min(m_rect.right, rectFrame.right);
			m_rect.top = min(m_rect.top, rectFrame.bottom - rectFrame.Height()/4);
			m_rect.bottom = min(m_rect.bottom, rectFrame.bottom);
			DUMP_RECT(TL_VERBOSE, "moving view to ", m_rect, "\n");
			GetParent()->MoveWindow(m_rect, TRUE);
		}
	}

	// Disable form scrolling
    SetScrollSizes(MM_TEXT, CSize(0,0));

    // Initialize the form
    UpdateData(FALSE);
    DoResize();
	SetViewTitle();
    Invalidate();
}


void CWordzView::DoDataExchange (CDataExchange* pDX)
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzView::DoDataExchange(%x)\n", pDX));
    CFormView::DoDataExchange(pDX);

    //{{AFX_DATA_MAP(CWordzView)
    DDX_Control(pDX, IDC_PHRASE_FRAME, m_PhraseFrame);
    DDX_Control(pDX, IDC_DEFINITION_FRAME, m_DefFrame);
    DDX_Control(pDX, IDC_SYNONYMS_FRAME, m_SynsFrame);
    DDX_Control(pDX, IDC_XREFS_FRAME, m_XrefsFrame);
    DDX_Control(pDX, IDC_DESCS_FRAME, m_DescsFrame);
    //DDX_Control(pDX, IDC_WORD, m_edWord);
    //DDX_Control(pDX, IDC_DEFINITION, m_edDef);
    //DDX_Control(pDX, IDC_SYNONYMS, m_edSyns);
    //DDX_Control(pDX, IDC_CROSSREFS, m_edXref);
    //DDX_Text(pDX, IDC_WORD, m_Word);
    //DDX_Text(pDX, IDC_DEFINITION, m_Definition);
    //DDX_Text(pDX, IDC_SYNONYMS, m_Synonyms);
    //DDX_Text(pDX, IDC_CROSSREFS, m_CrossRefs);
    //}}AFX_DATA_MAP

	if (pDX->m_bSaveAndValidate) {
		m_edWord.GetWindowText(m_Word);
		}
	else {
		// TODO: figure out how to change the color just once
		//m_edWord.SetWindowText(m_Word);
		m_edWord.SetText(ToRtf(m_Word));
		m_edDef.SetText(m_Definition);
		m_edSyns.SetText(m_Synonyms);
		m_edXref.SetText(m_CrossRefs);
		m_edDescs.SetText(m_Descendants);
		if (GetDebugOption("ResetWindowColors", FALSE)) {
			SetWindowColors();
			}
		}
}


/////////////////////////////////////////////////////////////////////////////
// CWordzView drawing

void CWordzView::OnDraw(CDC* pDC)
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzView::OnDraw(%x)\n", pDC));
	CWordzDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
#if FALSE
    PaintChildWindows(m_hWnd, pDC, GetDeviceScrollPosition());
#endif
}


/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CWordzView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CWordzView diagnostics

#ifdef _DEBUG
void CWordzView::AssertValid() const
{
	CFormView::AssertValid();
}

void CWordzView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

#endif //_DEBUG

BOOL CWordzView::ValidDoc (VOID)
{
	BOOL bDocOK = m_pDocument->IsKindOf(RUNTIME_CLASS(CWordzDoc));
	ASSERT(bDocOK);
    ASSERT_VALID(m_pDocument);
	return (bDocOK);
}

/////////////////////////////////////////////////////////////////////////////
// CWordzView message handlers

void CWordzView::OnOK (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnOK\n"));

   	UpdateData(TRUE);
	SetViewTitle();
#if FALSE
	BOOL bUseThread = !IsKeyToggled(VK_SCROLL);
	if (bUseThread) {
		if (m_pLookupThread == NULL) {
    		m_pLookupThread = AfxBeginThread((AFX_THREADPROC)::LookupWordProc, this);
	 		SetTimer(666, 200, NULL);
			}
		}
	else {
		BeginWaitCursor();
		LookupWord();
		ProcessLookupResults();
		}
#endif
	ASSERT(!DuringLookup());
	ASSERT(m_nNextCommand == nINVALID_INDEX);
	if (!DuringLookup()) {
		DoNextCommand();
		}
	//StartLookup();
	//PostMessage(WM_COMMAND, ID_LOOKUP_DEFINITION);
	//PostMessage(WM_COMMAND, ID_LOOKUP_SYNONYMS);
	//PostMessage(WM_COMMAND, ID_LOOKUP_REFERENCES);
	//EndLookup();
}


#if FALSE
CWordzView::ProcessLookupResults (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::ProcessLookupResults()\n"));
	if (m_pLookupThread) {
		m_pLookupThread = NULL;
		KillTimer(666);
		}
	else {
		EndWaitCursor();
		}
    UpdateData(FALSE);
    ReviseDoc();
    DoResize();			// resized proportionately to content
}
#endif


void CWordzView::SetViewTitle (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::SetViewTitle()\n"));
	if (!m_Word.IsEmpty()) {
		//SetWindowText(pDoc->GetTitle() + ":" + m_Word);

	    CString title = GetDocument()->GetTitle();
		INT nPos = title.Find(" - ");
		if (nPos != nINVALID_INDEX) {
		    title = title.Left(nPos);
		    }
		title += " - " + m_Word;
		//GetDocument()->SetTitle(title);

	    CWnd* pwnd = GetParent();
    	//ASSERT(pwnd && pwnd->IsKindOf(RUNTIME_CLASS(CMDIChildFrame)));
    	pwnd->SetWindowText(title);
		}
}

void CWordzView::ReviseDoc (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::ReviseDoc()\n"));
	if (m_nDocIndex == nINVALID_INDEX) {
		return;
	}
    CWordzDoc* pDoc = GetDocument();
    CWordInfo* pinfo = pDoc->GetInfo(m_nDocIndex);
    ASSERT_VALID(pinfo);
    pinfo->m_Word = m_Word;
    pinfo->m_Definition = m_Definition;
    pinfo->m_Synonyms = m_Synonyms;
    pinfo->m_CrossRefs = m_CrossRefs;
    pinfo->m_Descendants = m_Descendants;
    pinfo->m_bLookupDef = m_bLookupDef;
    pinfo->m_bLookupSyns = m_bLookupSyns;
    pinfo->m_bLookupXrefs = m_bLookupXrefs;
    pinfo->m_bLookupDescs = m_bLookupDescs;
	pinfo->m_rect = m_rect;
	pinfo->m_nPreviousItem = m_nPreviousItem;
    pDoc->SetModifiedFlag();
    pDoc->UpdateAllViews(this);
	pDoc->NotifyChanged();
}

#if FALSE
void CWordzView::OnDoubleclickedOk (VOID)
{
    OnOK();
}
#endif


//  Other operations
//

#if FALSE
// see lookup.cpp
CStringx FormatDefinition (const CStringx& sTextDef)
{
	DEBUG_OUT(TL_VERBOSE, ("FormatDefinition(%x)\n", sTextDef));
	CStringx sRtfDef (szRTF_HEADER);
	CStringx sCRNL ("\r\n");
	INT i = 0;

	//	Make the header word boldface
	sRtfDef += szSTART_BOLD;

	//	Convert carraige return/new line pairs into "\par";
	while (i < sTextDef.GetLength()) {
		BOOL bFirstLine = (i == 0);
		sRtfDef += ExtractToken(sTextDef, i, sCRNL);
		if (bFirstLine) {
			//sRtfDef += szEND_BOLD;	// end boldfaced headword
			}
		sRtfDef += szNEW_PARA;
		}
	
	sRtfDef += szRTF_TRAILER;

	return (sRtfDef);
}
#endif


#if FALSE
void CWordzView::LookupWord (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::LookupWord()\n"));
	ASSERT(m_pLookup);
    if (m_bLookupDef) {
        DEBUG_OUT(TL_VERBOSE, ("Looking up definition for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetDefinition(m_Word);
        //m_Definition = FormatDefinition(m_pLookup->buffer);
		m_Definition = m_pLookup->buffer;
        }
    if (m_bLookupSyns) {
        DEBUG_OUT(TL_VERBOSE, ("Looking up synonyms for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetSynonyms(m_Word);
        m_Synonyms = m_pLookup->buffer;
        }
    if (m_bLookupXrefs) {
        DEBUG_OUT(TL_VERBOSE, ("Looking up cross references for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetReferences(m_Word);
        m_CrossRefs = m_pLookup->buffer;
        }
}
#endif


void CWordzView::LookupDef (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::LookupDef()\n"));
	ASSERT(m_pLookup);
    if (m_bLookupDef) {
		//StartLookup();
        DEBUG_OUT(TL_VERBOSE, ("Looking up definition for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetDefinition(m_Word, ID_PROCESS_DEFINITION);
        }
	else {
		DoNextCommand();
		}
}


VOID CWordzView::DoNextCommand (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::DoNextCommand(): m_nNextCommand=%d\n", 
					       m_nNextCommand));

	if (m_nNextCommand == nINVALID_INDEX) {
		StartLookup();
		m_nNextCommand = 0;
		DoNextCommand();
		}
	else if (m_nNextCommand == m_auLookupCommands.GetSize()) {
		EndLookup();
		m_nNextCommand = nINVALID_INDEX;
		}
	else {
		ASSERT(m_nNextCommand < m_auLookupCommands.GetSize())
		PostMessage(WM_COMMAND, m_auLookupCommands[m_nNextCommand]);
		m_nNextCommand++;
		}
}


void CWordzView::ProcessDef (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::ProcessDef()\n"));
	ASSERT(m_pLookup);
	ASSERT(m_bLookupDef);

    if (m_bLookupDef) {
		m_Definition = m_pLookup->buffer;
		//EndLookup();
		}
	////	HACK: kill the cascading processing
	//PostMessage(WM_COMMAND, ID_LOOKUP_SYNONYMS);
	DoNextCommand();
}


void CWordzView::LookupSyns (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::LookupSyns()\n"));
	ASSERT(m_pLookup);
    if (m_bLookupSyns) {
		//StartLookup();
        DEBUG_OUT(TL_VERBOSE, ("Looking up synonyms for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetSynonyms(m_Word, ID_PROCESS_SYNONYMS);
        }
	else {
		//ProcessSyns();
		DoNextCommand();
		}
}


void CWordzView::ProcessSyns (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::ProcessSyns()\n"));
	ASSERT(m_pLookup);
	ASSERT(m_bLookupSyns);

	if (m_bLookupSyns) {
	    m_Synonyms = m_pLookup->buffer;
		//EndLookup();
		}
	////	HACK: kill the cascading processing
	//PostMessage(WM_COMMAND, ID_LOOKUP_REFERENCES);
	DoNextCommand();
}


void CWordzView::LookupXrefs (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::LookupXrefs()\n"));
	ASSERT(m_pLookup);

    if (m_bLookupXrefs) {
		//StartLookup();
        DEBUG_OUT(TL_VERBOSE, ("Looking up cross references for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetReferences(m_Word, ID_PROCESS_REFERENCES);
        }
	else {
		DoNextCommand();
		//ProcessXrefs();
		}
}


void CWordzView::ProcessXrefs (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::ProcessXrefs()\n"));
	ASSERT(m_pLookup);
	ASSERT(m_bLookupXrefs);

	if (m_bLookupXrefs) {
	    m_CrossRefs = m_pLookup->buffer;
		//EndLookup();
		}
	//EndLookup();
	DoNextCommand();
}


void CWordzView::LookupDescs (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::LookupDescs()\n"));
	ASSERT(m_pLookup);

    if (m_bLookupDescs) {
		//StartLookup();
        DEBUG_OUT(TL_VERBOSE, ("Looking up descendants for '%s'\n", (PCSZ)m_Word));
        m_pLookup->GetDescendants(m_Word, ID_PROCESS_DESCENDANTS);
        }
	else {
		DoNextCommand();
		//ProcessXrefs();
		}
}


void CWordzView::ProcessDescs (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::ProcessDescs()\n"));
	ASSERT(m_pLookup);
	ASSERT(m_bLookupDescs);

	if (m_bLookupDescs) {
	    m_Descendants = m_pLookup->buffer;
		//EndLookup();
		}
	//EndLookup();
	DoNextCommand();
}


void CWordzView::OnDoubleclickedDef (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnDoubleclickedDef()\n"));
    OnOK();
}


void CWordzView::OnDoit (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnDoit\n"));
    OnOK();
}



void CWordzView::OnUpdateDoit(CCmdUI* pCmdUI) 
{
	//BOOL bEnable = (m_pLookupThread == NULL);
	BOOL bEnable = (!DuringLookup());
	pCmdUI->Enable(bEnable);
}


/////////////////////////////////////////////////////////////////////////////
// CFormView printing (based on the VIEWEX sample)

BOOL CWordzView::OnPreparePrinting(CPrintInfo* pInfo)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnPreparePrinting(%x)\n", pInfo));
    // default preparation
    return DoPreparePrinting(pInfo);
}

void CWordzView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CWordzView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


VOID CWordzView::PrintWordBox (CDC *pDC, CRtfEdit& edWordBox, CSize& sScale)
{
	CRect rectBox;
	edWordBox.GetWindowRect(rectBox);
    ScreenToClient(rectBox);
	rectBox = ToTwips(pDC, rectBox);
	rectBox.left *= sScale.cx;
	rectBox.right *= sScale.cx;
	rectBox.top *= sScale.cy;
	rectBox.bottom *= sScale.cy;
	edWordBox.SplatTextToDC(pDC, rectBox, rectBox, TRUE);
}


void CWordzView::OnPrint(CDC* pDC, CPrintInfo*)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnPrint(%x, _)\n", pDC));

    CClientDC dcScreen(NULL);
	CRect rectView;
	GetClientRect(rectView);
	CSize sScale;
	sScale.cx = MulDiv(1, pDC->GetDeviceCaps(LOGPIXELSX), dcScreen.GetDeviceCaps(LOGPIXELSX));
	sScale.cy = MulDiv(-1, pDC->GetDeviceCaps(LOGPIXELSY), dcScreen.GetDeviceCaps(LOGPIXELSY));

	PrintWordBox(pDC, m_edWord, sScale);
	if (m_bLookupDef) {
		PrintWordBox(pDC, m_edDef, sScale);
		}
	if (m_bLookupSyns) {
		PrintWordBox(pDC, m_edSyns, sScale);
		}
	if (m_bLookupXrefs) {
		PrintWordBox(pDC, m_edXref, sScale);
		}
	if (m_bLookupDescs) {
		PrintWordBox(pDC, m_edDescs, sScale);
		}
}


BOOL EqualWnd (CWnd* pwnd1, CWnd* pwnd2)
{
	DEBUG_OUT(TL_VERBOSE, ("EqualWnd(%x, %x)\n", pwnd1, pwnd2));
    return (pwnd1 && pwnd2 && (pwnd1->m_hWnd == pwnd2->m_hWnd));
}


BOOL CWordzView::IsAnEditControl (CWnd* pwnd)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::IsAnEditControl(%x)\n", pwnd));
#if FALSE
    return ((pwnd == GetDlgItem(IDC_DEFINITION))
            || (pwnd == GetDlgItem(IDC_SYNONYMS))
            || (pwnd == GetDlgItem(IDC_CROSSREFS))
            || (pwnd == GetDlgItem(IDC_DESCENDANTS))
            || (pwnd == GetDlgItem(IDC_WORD)));
	//#else
    //CEditx* pedit = (CEditx*)pwnd;
	//  return ((pedit == m_edXref) || (pedit == m_edSyns) || (pedit == m_edWord) || (pedit == m_edDef));
	//	TODO: use RUNTIME_CLASS
	//#else
    //return (EqualWnd(pedit, &m_edXref) || EqualWnd(pedit, &m_edSyns) 
    //        || EqualWnd(pedit, &m_edWord) || EqualWnd(pedit, &m_edDef));
#else
	return (pwnd && pwnd->IsKindOf(RUNTIME_CLASS(CEditx)));
#endif
}

BOOL CWordzView::IsSelectionSet (CEditx* pedit)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::IsSelectionSet(%x)\n", pedit));
    INT nStart;
    INT nEnd;
    pedit->GetSel(nStart, nEnd);

    return ((nStart != -1) && (nStart != nEnd));
}

void CWordzView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateEditCopy(%x)\n", pCmdUI));
    CWnd* pwnd = GetFocus();
    if (IsAnEditControl(pwnd)) {
        pCmdUI->Enable(TRUE);
        }
    else {
        pCmdUI->Enable(FALSE);
        }
}

void CWordzView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateEditCut(%x)\n", pCmdUI));
    CWnd* pwnd = GetFocus();
    if (IsAnEditControl(pwnd)) {
        CEditx* pedit = (CEditx*)pwnd;
        pCmdUI->Enable(IsSelectionSet(pedit));
        }
    else {
        pCmdUI->Enable(FALSE);
        }
}

void CWordzView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateEditPaste(%x)\n", pCmdUI));
    CWnd* pwnd = GetFocus();
    if (IsAnEditControl(pwnd)) {
        pCmdUI->Enable(TRUE);
        }
    else {
        pCmdUI->Enable(FALSE);
        }
}

void CWordzView::OnEditCopy (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnEditCopy()\n"));
    CWnd* pwnd = GetFocus();
    if (IsAnEditControl(pwnd)) {
        CEditx* pedit = (CEditx*)pwnd;
        pedit->Copy();
        }
}

void CWordzView::OnEditCut (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnEditCut()\n"));
    CWnd* pwnd = GetFocus();
    if (IsAnEditControl(pwnd)) {
        CEditx* pedit = (CEditx*)pwnd;
        pedit->Cut();
        }
}

void CWordzView::OnEditPaste (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnEditPaste()\n"));
    CWnd* pwnd = GetFocus();
    if (IsAnEditControl(pwnd)) {
        CEditx* pedit = (CEditx*)pwnd;
        pedit->Paste();
        }
}

#if FALSE
void CWordzView::OnUpdateWindowNew(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateWindowNew(%x)\n", pCmdUI));
    pCmdUI->Enable(FALSE);  
}
#endif

void CWordzView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnLButtonDblClk(%x, (%d, %d))\n",
			  nFlags, point.x, point.y));
    CFormView::OnLButtonDblClk(nFlags, point);
    if (IsShiftPressed()) {
        CWnd* pwnd = GetFocus();
        if (IsAnEditControl(pwnd)) {
            if (IsControlPressed()) {
                //  ::SendMessage(GetMainHwnd(), WM_COMMAND, ID_FILE_NEW);
                AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_NEW);
                }
            else {
                //  CWnd* pwnd = GetParent();
                //  pwnd->SendMessage(WM_COMMAND, ID_WINDOW_NEW);
                AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_WINDOW_NEW);
                }
            }
        }
}

void CWordzView::OnEditChangefont (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnEditChangefont()\n"));
    CWnd* pwnd = GetFocus();
    if (!pwnd) {
        AfxMessageBox("No window has the focus");
        return;
        }
    CEditx::InvokeChangeFont(pwnd);
}

#if FALSE
void CWordzView::OnWindowNew (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnWindowNew \n"));
    CFormView::OnWindowNew();
}
#endif

//  Misc functions
//

void StripTrailer (CStringx& text)
{
	DEBUG_OUT(TL_VERBOSE, ("StripTrailer(%s)\n", F1(text)));
    for (INT nPos = text.GetLength() - 1; nPos >= 0; nPos--) {
        if (!IsSpace(text[nPos]) && (text[nPos] != ';')) {
            break;
            }
        }
    
    text = text.Left(nPos + 1);
}

void CWordzView::OnSize(UINT nType, int cx, int cy)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnSize(%d, %d, %d)\n", nType, cx, cy));
    CFormView::OnSize(nType, cx, cy);
    DoResize(cx, cy);
	GetParent()->GetWindowRect(m_rect);

    pTheOneAndOnlyMainFrame->ScreenToClient(m_rect);
#if TRUE
	ReviseDoc();
#endif
}

void CWordzView::DoResize (void)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::DoResize()\n"));
    CRect rect;
    GetClientRect(rect);
    DoResize(rect.right, rect.bottom);
}


CONST INT nOFFSET = 16;

void ResizeWordBox (CStatic& frame, CEditx& edit, INT nTop, INT nHeight, INT nWidth, INT nSub)
{
	DEBUG_OUT(TL_VERBOSE, ("ResizeWordBox(%lx, %lx, %d, %d, %d, %d)\n", 
			   &frame, &edit, nTop, nHeight, nWidth, nSub));
	CRect rect;

    frame.GetClientRect(rect);
    rect.right = nWidth;
    //rect.top = nBase + (nSub * nHeight);
	rect.top = nTop;
    rect.bottom = rect.top + nHeight;
	CString sCaption;
	frame.GetWindowText(sCaption);
    DEBUG_OUT(TL_VERBOSE, ("Moving %s word box to (%d, %d, %d, %d)\n",
               F1(sCaption), rect.left, rect.top, rect.right, rect.bottom));
    frame.MoveWindow(rect);
    frame.ShowWindow(SW_SHOW);
    //rect.InflateRect(-nOFFSET, -nOFFSET);
    //rect.OffsetRect(nOFFSET - 4, nOFFSET -4);
	rect.left += nOFFSET;
	rect.top += (3 * nOFFSET)/2;
	rect.right -= 2;
	rect.bottom -= 2;
    edit.MoveWindow(rect);
    edit.ShowWindow(SW_SHOW);
}


void HideWordBox (CStatic& frame, CEditx& edit)
{
	DEBUG_OUT(TL_VERBOSE, ("HideWordBox(%lx, %lx)\n", 
			   frame, edit));

    frame.ShowWindow(SW_HIDE);
    edit.ShowWindow(SW_HIDE);
}


void CWordzView::DoResize (int cx, int cy)
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::DoResize(%d, %d)\n", cx, cy));
    // Resize the embedded edit windows
    CRect rect;
    
    //INT nWidth = ((LONG)cx * 95)/100;
    INT nWidth = cx;

    //  Resize the phrase box to fit the width but keep the size from resource
	if (m_PhraseFrame.m_hWnd == NULL) {
		return;
		}
    m_PhraseFrame.GetClientRect(rect);
    INT nBase = ((LONG)rect.bottom * 105) / 100;
    INT nRemHeight = max(0, cy - nBase);

	ResizeWordBox(m_PhraseFrame, m_edWord, 0, rect.bottom, nWidth, 0);


    INT nNumSubs = m_bLookupDef + m_bLookupSyns + m_bLookupXrefs+ m_bLookupDescs;
    ASSERT((nNumSubs >= 0) && (nNumSubs <= 4));
    //INT nAvgHeight = (nNumSubs > 0) ? (nRemHeight / nNumSubs) : 0;

#if TRUE
	//	Try to size the boxes proportionately based on content
	//	NOTE: Each box gets two extra lines: one for the title and one for good measure
	INT nTotalLines = ((m_bLookupDef * (2 + m_edDef.GetLineCount()))
					  +	(m_bLookupSyns * (2 + m_edSyns.GetLineCount()))
					  + (m_bLookupXrefs * (2 + m_edXref.GetLineCount()))
					  + (m_bLookupDescs * (2 + m_edDescs.GetLineCount())))
					  ;
	INT nTotalLength = ((m_bLookupDef * (1 + m_Definition.GetLength()))
					  +	(m_bLookupSyns * (1 + m_Synonyms.GetLength()))
					  + (m_bLookupXrefs * (1 + m_CrossRefs.GetLength()))
					  + (m_bLookupDescs * (1 + m_Descendants.GetLength())));
#endif

	//	Determine the box heights, adjusting so small ones aren't squished
	INT nSpacing = 6;		// TODO: make this proportionate to the font
	INT nFontSize = 16;
	INT nMaxMinHeight = 4 * nFontSize;
	CIntArray anHeight;
	CIntArray anMinHeight;
    if (m_bLookupDef) {
		anHeight.Add(MulDiv((2 + m_edDef.GetLineCount()), nRemHeight, nTotalLines));
		anMinHeight.Add(min((2 + m_edDef.GetLineCount()) * nFontSize, nMaxMinHeight));
		}
    if (m_bLookupSyns) {
		anHeight.Add(MulDiv((2 + m_edSyns.GetLineCount()), nRemHeight, nTotalLines));
		anMinHeight.Add(min((2 + m_edSyns.GetLineCount()) * nFontSize, nMaxMinHeight));
		}
    if (m_bLookupXrefs) {
		anHeight.Add(MulDiv((2 + m_edXref.GetLineCount()), nRemHeight, nTotalLines));
		anMinHeight.Add(min((2 + m_edXref.GetLineCount()) * nFontSize, nMaxMinHeight));
		}
    if (m_bLookupDescs) {
		anHeight.Add(MulDiv((2 + m_edDescs.GetLineCount()), nRemHeight, nTotalLines));
		anMinHeight.Add(min((2 + m_edDescs.GetLineCount()) * nFontSize, nMaxMinHeight));
		}
	for (INT i = 0; i < anHeight.GetSize(); i++) {
		if (anHeight[i] < anMinHeight[i]) {
			INT nHeightAdjustment = (nNumSubs > 0) ? (anMinHeight[i] / nNumSubs) : 0;
			anHeight[i] = anMinHeight[i];
			for (INT j = 0; j < anHeight.GetSize(); j++) {
				if (i != j) {
					anHeight[j] = max(anHeight[j] - nHeightAdjustment, nFontSize + nSpacing);
					//anHeight[j] = max(anHeight[j] - nHeightAdjustment, anMinHeight[j]);
					}
				}
			}
		}

    //  Resize the definition window
	//	TODO: move some of the redundant logic into ResizeWordBox
    INT nSub = 0;
    if (m_bLookupDef) {
		DEBUG_OUT(TL_VERBOSE, ("def box height is %d (~max(%d/%d * %d, %d))\n", 
				  anHeight[nSub], m_edDef.GetLineCount(), nTotalLines, nRemHeight, anMinHeight[nSub]));
		//INT nTheHeight = MulDiv(m_edDef.GetLineCount(), nRemHeight, nTotalLines);
		//INT nTheHeight = MulDiv(1 + m_Definition.GetLength(), nRemHeight, nTotalLength);
		//nTheHeight = max(28, nTheHeight);	// TODO: account for box's font size
		INT nTheHeight = anHeight[nSub];
		ResizeWordBox(m_DefFrame, m_edDef, nBase, nTheHeight - nSpacing, nWidth, nSub);
		nBase += nTheHeight;
        nSub++;
        }
    else {
		HideWordBox(m_DefFrame, m_edDef);
        }

    //  Resize the synonyms window
    if (m_bLookupSyns) {
		DEBUG_OUT(TL_VERBOSE, ("syns box height is %d (~max(%d/%d * %d, %d))\n", 
				  anHeight[nSub], m_edSyns.GetLineCount(), nTotalLines, nRemHeight, anMinHeight[nSub]));
		//INT nTheHeight = MulDiv(m_edSyns.GetLineCount(), nRemHeight, nTotalLines);
		//INT nTheHeight = MulDiv(1 + m_Synonyms.GetLength(), nRemHeight, nTotalLength);
		//nTheHeight = max(28, nTheHeight);	// TODO: account for box's font size
		INT nTheHeight = anHeight[nSub];
		ResizeWordBox(m_SynsFrame, m_edSyns, nBase, nTheHeight - nSpacing, nWidth, nSub);
		nBase += nTheHeight;
        nSub++;
        }
    else {
		HideWordBox(m_SynsFrame, m_edSyns);
        }

    //  Resize the cross references window
    if (m_bLookupXrefs) {
		DEBUG_OUT(TL_VERBOSE, ("xref box height is %d (~max(%d/%d * %d, %d))\n", 
				  anHeight[nSub], m_edXref.GetLineCount(), nTotalLines, nRemHeight, anMinHeight[nSub]));
		//INT nTheHeight = MulDiv(m_edXref.GetLineCount(), nRemHeight, nTotalLines);
		//INT nTheHeight = MulDiv(1 + m_CrossRefs.GetLength(), nRemHeight, nTotalLength);
		//nTheHeight = max(28, nTheHeight);	// TODO: account for box's font size
		INT nTheHeight = anHeight[nSub];
		ResizeWordBox(m_XrefsFrame, m_edXref, nBase, nTheHeight - nSpacing, nWidth, nSub);
		nBase += nTheHeight;
        nSub++;
        }
    else {
		HideWordBox(m_XrefsFrame, m_edXref);
        }

    //  Resize the descendants window
    if (m_bLookupDescs) {
		DEBUG_OUT(TL_VERBOSE, ("desc box height is %d (~max(%d/%d * %d, %d))\n", 
				  anHeight[nSub], m_edDescs.GetLineCount(), nTotalLines, nRemHeight, anMinHeight[nSub]));
		INT nTheHeight = anHeight[nSub];
		ResizeWordBox(m_DescsFrame, m_edDescs, nBase, nTheHeight - nSpacing, nWidth, nSub);
		nBase += nTheHeight;
        nSub++;
        }
    else {
		HideWordBox(m_DescsFrame, m_edDescs);
        }

    Invalidate();
}


void CWordzView::OnDoDef()
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnDoDef()\n"));
    m_bLookupDef = !m_bLookupDef;
    DoResize();
}

void CWordzView::OnDoSyns()
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnDoSyns()\n"));
    m_bLookupSyns = !m_bLookupSyns;
    DoResize();
}

void CWordzView::OnDoXrefs()
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnDoXrefs()\n"));
    m_bLookupXrefs = !m_bLookupXrefs;    
    DoResize();
}

void CWordzView::OnDoDescs()
{
    DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnDoDescs()\n"));
    m_bLookupDescs = !m_bLookupDescs;
    DoResize();
}

void CWordzView::OnUpdateDoDef(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateDoXrefs(%x)\n", pCmdUI));
    pCmdUI->SetCheck(m_bLookupDef);
}

void CWordzView::OnUpdateDoSyns(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateDoXrefs(%x)\n", pCmdUI));
    pCmdUI->SetCheck(m_bLookupSyns);
}

void CWordzView::OnUpdateDoXrefs(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateDoXrefs(%x)\n", pCmdUI));
    pCmdUI->SetCheck(m_bLookupXrefs);
}

void CWordzView::OnUpdateDoDescs(CCmdUI* pCmdUI)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CWordzView::OnUpdateDoDescs(%x)\n", pCmdUI));
    pCmdUI->SetCheck(m_bLookupDescs);
}

#if FALSE
void CWordzView::OnWindowNew() 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnWindowNew()\n"));
}
#endif

VOID CWordzView::OpenNewWindow (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OpenNewWindow()\n"));
}

HBRUSH CWordzView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzView::OnCtlColor(%x, %x, %x)\n",
			  pDC, pWnd, nCtlColor));
	//	Set the background for the edit box to gray (instead of white)
	//if (nCtlColor == CTLCOLOR_EDIT) {
		//pDC->SetBkColor(clrLIGHT_GRAY);
		//HBRUSH hBrush = (HBRUSH)::GetStockObject(LTGRAY_BRUSH);
		pDC->SetTextColor(m_clrText);
		pDC->SetBkColor(m_clrBackground);
		HBRUSH hBrush = (HBRUSH)m_brushBackground.GetSafeHandle();
		return (hBrush);
		//}
#if FALSE
	if (nCtlColor == CTLCOLOR_DLG) {
		pDC->SetBkColor(clrLIGHT_GRAY);
		HBRUSH hBrush = (HBRUSH)::GetStockObject(LTGRAY_BRUSH);
		return (hBrush);
		}
#endif
	
	return CFormView::OnCtlColor(pDC, pWnd, nCtlColor);
}

VOID CWordzView::OnNotifyDblClick (NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzView::OnNotifyDblClick(%x, %x)\n", 
			  pNMHDR, pResult));
	NMHDREX* pNMHDREX = (NMHDREX*)pNMHDR;
	ASSERT(IsValidNMHDREX(pNMHDREX));
	pLastEditWnd = FromHandlePermanent(pNMHDR->hwndFrom);
	ASSERT(pLastEditWnd);
    if (IsControlPressed()) {
        //  ::SendMessage(GetMainHwnd(), WM_COMMAND, ID_FILE_NEW);
        AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_NEW);
        }
    else {
        //  CWnd* pwnd = GetParent();
        //  pwnd->SendMessage(WM_COMMAND, ID_WINDOW_NEW);
        AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_WINDOW_NEW);
        }
	*pResult = 1;						// standard MFC notification processed indicator
}


VOID CWordzView::OnNotifyButtonDown (NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzView::OnNotifyButtonDown(%x, %x)\n", 
			  pNMHDR, pResult));
	NMHDREX* pNMHDREX = (NMHDREX*)pNMHDR;
	ASSERT(IsValidNMHDREX(pNMHDREX));
	pLastEditWnd = FromHandlePermanent(pNMHDR->hwndFrom);
	ASSERT(pLastEditWnd);
	*pResult = 1;						// standard MFC notification processed indicator
}


VOID CWordzView::OnDefinitionKeyDown (NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_GARRULOUS, ("CWordzView::OnDefinitionKeyDown(%x, %x)\n", 
			  pNMHDR, pResult));
	NMHDREX* pNMHDREX = (NMHDREX*)pNMHDR;
	ASSERT(IsValidNMHDREX(pNMHDREX));
	UINT uKeyCode = pNMHDREX->wParam;
	if (uKeyCode == VK_RETURN) {
		OnDoit();
	}
	else if ((uKeyCode == VK_ESCAPE) && DuringLookup()) {
		m_pLookup->AbortLookup();
		EndLookup();
		m_nNextCommand = nINVALID_INDEX;
	}
	*pResult = 1;						// standard MFC notification processed indicator
}


void CWordzView::OnViewFont() 
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnMiscChangefont()\n"));
    CWnd* pwnd = GetFocus();
    CEditx::InvokeChangeFont(pwnd);
}

void CWordzView::OnUpdateViewFont(CCmdUI* pCmdUI) 
{
    CWnd* pwnd = GetFocus();
	BOOL bEnableIt = IsAnEditControl(pwnd);
    pCmdUI->Enable(bEnableIt);
}


VOID CWordzView::StartLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::StartLookup()\n"));
	//EnterCriticalSection(&m_csLookup);
	//WaitForSingleObject(m_hLookupEvent, INFINITE);
	//ResetEvent(m_hLookupEvent);
	AfxGetApp()->DoWaitCursor(1);		// start with MFC's wait cursor (overridden in timer proc)
	SetTimer(666, 333, NULL);
	SetDuringLookup(TRUE);
}


VOID CWordzView::EndLookup (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::EndLookup()\n"));
	if (DuringLookup()) {
		KillTimer(666);
		AfxGetApp()->DoWaitCursor(-1);		// restore cursor that MFC thinks as the old one
		SetDuringLookup(FALSE);
	    UpdateData(FALSE);
    	ReviseDoc();
    	DoResize();			// resized proportionately to content
		//LeaveCriticalSection(&m_csLookup);
		//SetEvent(m_hLookupEvent);
		}
}


void CWordzView::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == 666) {
		//AfxGetApp()->DoWaitCursor(1);
		::SetCursor(AfxGetApp()->LoadCursor(IDC_COFFEE));
		}
	
	CFormView::OnTimer(nIDEvent);
}


//-----------------------------------------------------------------------------
//	Other functions
//-----------------------------------------------------------------------------

#if FALSE
UINT LookupWordProc (CWordzView* pWordzView)
{
	DEBUG_OUT(TL_VERBOSE, ("LookupWordProc(%x)\n", pWordzView));
	BOOL bOK = FALSE;

	ASSERT(pWordzView && pWordzView->IsKindOf(RUNTIME_CLASS(CWordzView)));
	if (pWordzView && pWordzView->IsKindOf(RUNTIME_CLASS(CWordzView))) {
		pWordzView->LookupWord();
		//pWordzView->ProcessLookupResults();
		pWordzView->PostMessage(WM_COMMAND, ID_PROCESS_RESULTS, 0);
		}

	return (bOK);
}
#endif


// OnMove: Trap the move notification to keep track of window coordinates
//
// TODO: have the child frame notify us since this is not getting
//		 notification
//
void CWordzView::OnMove(int x, int y) 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnMove(%d, %d)\n", x, y));
	CFormView::OnMove(x, y);

	//m_rect.left = x;	
	//m_rect.right = y;	
	GetParent()->GetWindowRect(m_rect);
    pTheOneAndOnlyMainFrame->ScreenToClient(m_rect);
#if TRUE
	ReviseDoc();
#endif
}


// OnWindowPosChanging: when the z-order changes, revise the placement value
// of the corresponding document item
//
// TODO: optimize/batch the document updates
//
void CWordzView::OnWindowPosChanged( WINDOWPOS* lpwndpos )
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnWindowPosChanged(%x)\n", lpwndpos));
	CFormView::OnWindowPosChanged(lpwndpos);

#if FALSE
	CWordzView* pView = ToCWordzView(FromHandle(lpwndpos->hwndInsertAfter));
	m_nPreviousItem = pView ? pView->m_nDocIndex : nINVALID_INDEX;

	ReviseDoc();
#endif
}

void CWordzView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	DEBUG_OUT(TL_VERBOSE, ("CWordzView::OnActivateView(%d, %x, %x)\n", 
			  bActivate, pActivateView, pDeactiveView));
	
	CFormView::OnActivateView(bActivate, pActivateView, pDeactiveView);
#if FALSE
	CWordzView* pToView = ToCWordzView(pActivateView);
	CWordzView* pFromView = ToCWordzView(pDeactiveView);
	if (bActivate && (this == pToView) && (pToView != pFromView)) {
		m_nPreviousItem = pFromView ? pFromView->m_nDocIndex : nINVALID_INDEX;
		ReviseDoc();
	}
#else
	// TODO: be more careful; this was the source of disappearing clue word in the
	//		 Wordz automation server
	if (bActivate && (pActivateView != pDeactiveView)) {
		CWordzView* pLastView = GetDocument()->m_pCurrentView;
		if (pLastView) {
			INT nLastIndex = pLastView->m_nDocIndex;
			if (pLastView->m_nPreviousItem == nINVALID_INDEX) {
				pLastView->m_nPreviousItem = m_nDocIndex;
				pLastView->ReviseDoc();
				DEBUG_OUT(TL_VERBOSE, ("z-order: (%d, %d)\n", 
					      pLastView->m_nDocIndex, pLastView->m_nPreviousItem));
			}
			m_nPreviousItem = nLastIndex;
		}
		m_nPreviousItem = nINVALID_INDEX;
		ReviseDoc();
		DEBUG_OUT(TL_VERBOSE, ("z-order: (%d, %d)\n", 
			      m_nDocIndex, m_nPreviousItem));
		GetDocument()->m_pCurrentView = this;
	}
#endif
}
