//	Copyleft 1993-1996 Tom O'Hara		Wordz source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// wordzvw.h : interface of the CWordzView class
//
/////////////////////////////////////////////////////////////////////////////

class CWordzDoc;

class CWordzView : public CFormView
{
protected: // create from serialization only
	CWordzView();
    virtual BOOL Create(LPCSTR lpszClassName, LPCSTR lpszWindowName, DWORD dwStyle, 
					    const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
	DECLARE_DYNCREATE(CWordzView)

// Form Data
public:
    //{{AFX_DATA(CWordForm)
    enum { IDD = IDR_WORDZTYPE };
    CStatic m_PhraseFrame;
    CStatic m_DefFrame;
    CStatic m_SynsFrame;
    CStatic m_XrefsFrame;
    CStatic m_DescsFrame;
    CButton m_DoIt;
    CRtfEdit  m_edWord;
    CRtfEdit  m_edDef;
    CRtfEdit  m_edSyns;
    CRtfEdit  m_edXref;
    CRtfEdit  m_edDescs;
    CStringx    m_Word;
    CStringx    m_Definition;
    CStringx    m_Synonyms;
    CStringx    m_CrossRefs;
    CStringx    m_Descendants;
    BOOL    m_bLookupDef;
    BOOL    m_bLookupSyns;
    BOOL    m_bLookupXrefs;
    BOOL    m_bLookupDescs;
    //}}AFX_DATA

// Attributes
public:
    //CLookup lu;
	CLookup* m_pLookup;					// the class for doing actual lookup's
	BOOL m_bDuringLookup;				// true if during a lookup operation
    BOOL m_bInitialLookup;				// should an initial lookup be done?
    INT m_nDocIndex;					// index of Info item in doc collection
	//CRITICAL_SECTION m_csLookup;		// obsolete lookup critical section
	//HANDLE m_hLookupEvent;			// obsolete lookup event handle
	INT m_nNextCommand;					// next lookup command ID
	CUIntArray m_auLookupCommands;		// lookup command ID's
	CRect m_rect;						// (saved) window coordinates
	INT m_nPreviousItem;				// previous view item in z-order
	COLORREF m_clrText;					// color of the text in the edit boxes
	COLORREF m_clrBackground;			// color of the edit box background
	CBrush m_brushBackground;			// brush for child control backgrounds

// Attributes
public:

// Operations
public:
	VOID GetOptions (VOID);
	void JustDoIt (VOID)					{ OnOK(); }
    void LookupWord (void);
	void SetViewTitle (VOID);
	VOID OpenNewWindow (VOID);
	VOID UpdateView (VOID)
			{ OnUpdate(this, 		// view that modified the document
						0,			// (no) hint information
						NULL); }	// (no) object with hint information
protected:
    void ReviseDoc (VOID);
    virtual void OnUpdate (CView* pSender, LPARAM lHint, CObject* pHint);
    virtual void OnInitialUpdate (void);
	VOID SetWindowColors (VOID);
    void DoResize (VOID);
    void DoResize (int cx, int cy);
	BOOL ValidDoc (VOID);

    BOOL IsAnEditControl (CWnd* pwnd);
    BOOL IsSelectionSet (CEditx* pedit);
    CWordzDoc* GetDocument (VOID)			
    	{ ASSERT(ValidDoc()); 
          return ((CWordzDoc*)m_pDocument); }
	VOID StartLookup (VOID);
	VOID EndLookup (VOID);
	inline VOID SetDuringLookup (BOOL bActive)	{ m_bDuringLookup = bActive; }
	inline BOOL DuringLookup (VOID)				{ return (m_bDuringLookup); }
	VOID DoNextCommand (VOID);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CWordzView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CWordzView();
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CWordzView)
	afx_msg void OnCancelEditSrvr();
    virtual void OnOK();
    afx_msg void OnDoubleclickedDef();
    afx_msg void OnDoit();
    afx_msg void OnUpdateEditCopy(CCmdUI* pCmdUI);
    afx_msg void OnUpdateEditCut(CCmdUI* pCmdUI);
    afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
    afx_msg void OnEditCopy();
    afx_msg void OnEditCut();
    afx_msg void OnEditPaste();
    afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
    afx_msg void OnEditChangefont();
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnDoDef();
    afx_msg void OnDoSyns();
    afx_msg void OnDoXrefs();
    afx_msg void OnDoDescs();
    afx_msg void OnUpdateDoDef(CCmdUI* pCmdUI);
    afx_msg void OnUpdateDoSyns(CCmdUI* pCmdUI);
    afx_msg void OnUpdateDoXrefs(CCmdUI* pCmdUI);
    afx_msg void OnUpdateDoDescs(CCmdUI* pCmdUI);
	afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
	afx_msg void OnViewFont();
	afx_msg void OnUpdateViewFont(CCmdUI* pCmdUI);
	afx_msg void OnUpdateDoit(CCmdUI* pCmdUI);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnMove(int x, int y);
	afx_msg void OnWindowPosChanged(WINDOWPOS* lpwndpos);
	//}}AFX_MSG
	afx_msg VOID OnNotifyDblClick (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg VOID OnNotifyButtonDown (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg VOID OnDefinitionKeyDown (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void LookupDef (void);
	afx_msg void ProcessDef (void);
	afx_msg void LookupSyns (void);
	afx_msg void ProcessSyns (void);
	afx_msg void LookupXrefs (void);
	afx_msg void ProcessXrefs (void);
	afx_msg void LookupDescs (void);
	afx_msg void ProcessDescs (void);
	DECLARE_MESSAGE_MAP()

    //  For forcing printing support
	VOID PrintWordBox (CDC *pDC, CRtfEdit& m_edWordBox, CSize& sScale);
    virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
};


MAKE_CLASS_CAST(CWordzView);

/////////////////////////////////////////////////////////////////////////////
