//	Copyleft 1993-1996 Tom O'Hara		CWord source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// gridview.cpp : implementation file
//

#include "stdafx.h"
#include "cword.h"
#include "lookup.h"
#include "gridview.h"
#include "options.h"
#include "mainfrm.h"
#include "wordz.h"

IMPLEMENT_DYNCREATE(CGridView, CFormView)

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


CONST CHAR szACROSS[] =			"ACROSS";
CONST CHAR szDOWN[] =			"DOWN";
CONST CHAR szGRID[] =			"GRID";
CONST CHAR szSTATE[] =			"STATE";
CONST CHAR szNUM_PAIR_FMT[] =	"%d, %d\n";
CONST CHAR szHEADER_FMT[] =		"%s\n";
CONST CHAR szCLUE_FMT[] =		"%s\n";
CONST CHAR szSTATE_FMT[] =		"\"%s\"\n";


CONST CHAR szRTF_SMALL_FONT[] = "\\fs16 ";	// 8 pt font
CONST CHAR szCLUE[] = "Clue: ";

inline PCSZ ClueText(CString& sClue)	{ return ((PCSZ)sClue + STR_LEN(szCLUE)); }



INLINE CString ToRtf (CONST CString& sText)
{
	CString sRtfText (szRTF_HEADER);
	sRtfText += szRTF_SMALL_FONT;
	sRtfText += sText;
	sRtfText += szRTF_TRAILER;

	return (sRtfText);
}



// Globals
//
// HACK
BOOL bJustReadIn = FALSE;
BOOL bAllowSingletonCells = FALSE;


/////////////////////////////////////////////////////////////////////////////
// CGridView

CONST INT nBUFLEN = SHRT_MAX - 1;


CGridView::CGridView()
    : DEF_NUM_ROWS(15),
      DEF_NUM_COLS(15),
      CFormView(CGridView::IDD)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::CGridView()\n"));
    //{{AFX_DATA_INIT(CGridView)
    m_sChar = "";
	//}}AFX_DATA_INIT

	m_nScaleFactor = 100;
#if TRUE
    m_nCellHeight = 25; // 1/4 inch
	//m_nCellHeight = 38; 			// 6/16 inch
    m_nCellWidth = 25;	// 1/4 inch
	//m_nCellWidth = 38; 			// 6/16 inch
#endif
    m_nRow = 0;
    m_nCol = 0;
	m_pcRow = NULL;
	m_puIndex = NULL;
	m_pLookup = NULL;


	InitGrid(DEF_NUM_ROWS, DEF_NUM_COLS);

	//	The indices are assigned numerically in row-order for all cells in row 0
	//	or column 0; all other cells receive the null index (uNO_INDEX).
	ReviseIndices();

	//	have the cell character box reflect keystrokes
	m_cwordChar.NotifyOfKeystrokes();

	SetDuringLookup(FALSE);

	m_piWordz = NULL;
#if FALSE
	//	Create the Wordz automation object
	ConnectToWordz();
#endif

	CString sEntries;
	m_sCluesAcross.Format("%s%sACROSS%s%s", 
			szRTF_HEADER, szRTF_START_BOLD, szRTF_END_BOLD, szRTF_NEW_PARA);
	sEntries.Format("1 Medical Chart Info%s7 Jeans, restyled%s14 Baseball's George and Ken",
					szRTF_NEW_PARA, szRTF_NEW_PARA, szRTF_NEW_PARA);
	m_sCluesAcross += sEntries;
	m_sCluesAcross += szRTF_TRAILER;
	m_sCluesDown.Format("%s%sDOWN%s%s", 
			szRTF_HEADER, szRTF_START_BOLD, szRTF_END_BOLD, szRTF_NEW_PARA);
	sEntries.Format("1 What a magician may wave%s2 Actor Morales%s3 Beatles' \"Let ----\"%s",
					szRTF_NEW_PARA, szRTF_NEW_PARA, szRTF_NEW_PARA);
	m_sCluesDown += sEntries;
	m_sCluesDown += szRTF_TRAILER;

	// Get the user preferences (eg, so everything isn't initially black)
	GetOptions();
}


CGridView::~CGridView()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::~CGridView()\n"));
	CleanupGridView();
}


VOID CGridView::CleanupGridView (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::CleanupGridView()\n"));

	if (m_pcRow) {
		delete [] m_pcRow;
		m_pcRow = NULL;
		}
	if (m_puIndex) {
		delete [] m_puIndex;
		m_puIndex = NULL;
		}
	if (m_pLookup) {
		delete m_pLookup;
		m_pLookup = NULL;
		}
	DeleteRtfColumns(m_aedCluesAcross);
	DeleteRtfColumns(m_aedCluesDown);
	DisconnectWordz();
}


// GetOptions: Get the user options that affect the crossword grid display
// nad the clue display
//
// NOTE: Since SetBkColor doesn't support dithered colors, the background color will
//	     be converted to the nearest solid color
//		 See KB article Q69885: "SetBkColor() Does Not Support Dithered Colors"
//
VOID CGridView::GetOptions (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::GetOptions()\n"));

	m_colorGridText = GetAppOption(szGRID_TEXT, clrBLACK);
	m_colorIndexText = GetAppOption(szGRID_INDEX_TEXT, clrBLACK);
	m_colorGridBorder = GetAppOption(szGRID_BORDER, clrBLACK);
	m_colorGridBackground = GetAppOption(szGRID_BACKGROUND, clrOFF_BLUE);
	m_colorBlackened = 	GetAppOption(szBLACKENED_CELL, clrBLACK);
	m_colorActiveCell = GetAppOption(szACTIVE_CELL, clrPALE_GREEN);
	m_colorClueText = GetAppOption(szCLUE_TEXT, clrBLACK);

	m_nScaleFactor = GetAppOption(szSCALE_FACTOR, 100);
    m_nCellHeight = MulDiv(25, m_nScaleFactor, 100);
    m_nCellWidth = MulDiv(25, m_nScaleFactor, 100);

	// Do special case handling for the clue text background
	//		get nearest solid color
	//		update the brush used for painting backgrounds
	COLORREF colorBackground = GetAppOption(szCLUE_BACKGROUND, clrLIGHT_GRAY);
	CWindowDC dc(NULL);
	m_colorClueBackground = dc.GetNearestColor(colorBackground);
	DEBUG_OUT(TL_VERBOSE, ("m_colorClueBackground = 0x%08x (\"TextBackground\" = 0x%08x)\n", 
			  m_colorClueBackground, colorBackground));
	if (NULL != m_brushClueBackground.GetSafeHandle()) {
		m_brushClueBackground.DeleteObject();
		}
	m_brushClueBackground.CreateSolidBrush(m_colorClueBackground);

	bAllowSingletonCells = GetAppOption("AllowSingletonCells", FALSE);
}


// ConnectToWordz: Make or verify an OLE connection to the Wordz automation server
// If no connection is active, this just establishes a new connection. But if there
// is one active, this tests the connection, re-estabishing it if necessary.
//
// Note: Each connection corresponds to one Wordz document and thus one view due to
// the current Wordz implementation. If more than one view appear, there might be a 
// problem in this connection (as was the case in a buggy version of this function).
//

VOID CGridView::ConnectToWordz (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::ConnectToWordz()\n"));

	// First, test out any existing connection
	BOOL bConnectionOK = FALSE;
	if (m_piWordz) {
		try {
			bConnectionOK = m_piWordz->TestConnection();
		}
		catch (...) {
			DEBUG_OUT(TL_BASIC, ("Problem with Wordz connection ... reconnecting\n"));
			DisconnectWordz();
		}
	}
	if (bConnectionOK) {
		return;
	}

	// Make a new connection
	COleException excOle;
	DEBUG_OUT(TL_VERBOSE, ("Establishing new Wordz connection\n"));
	m_piWordz = new IWordz();
	if (!m_piWordz->CreateDispatch("Wordz.Document.1", &excOle)) {
		AfxMessageBox("Unable to connect to Wordz (via OLE automation)");
		}
	else {
		m_piWordz->RefreshWindow();		// needed to induce a view
		m_piWordz->ShowWindow();		// needed to make Wordz visible
		}
}


// DisconnectWordz: Close the OLE connection to the Wordz automation server
//
VOID CGridView::DisconnectWordz (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::DisconnectWordz()\n"));
	
	if (m_piWordz) {
		delete m_piWordz;
		m_piWordz = NULL;
	}
}


VOID CGridView::InitGrid (INT nNumRows, INT nNumCols)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::InitGrid(%d, %d)\n", nNumRows, nNumCols));
	m_nNumRows = nNumRows;
	m_nNumCols = nNumCols;

	// Clean up existing structures
    if (m_pcRow) {
		delete [] m_pcRow;
		m_pcRow = NULL;
	}
	if (m_puIndex) {
		delete [] m_puIndex;
	}

    //	Allocate the crossword structures: the letter grid and the index grid
    m_pcRow = new CByteArray[m_nNumRows];
	m_puIndex = new CUIntArray[m_nNumRows];

    //	Initialize internal representation of the CrossWord to all blanks.
    for (INT i = 0; i < m_nNumRows; i++) {
        m_pcRow[i].SetAtGrow(m_nNumRows, cNULL);		// null char to end string
		m_puIndex[i].SetAtGrow(m_nNumRows, uNO_INDEX);	// trailer entry for array
        for (INT j = 0; j < m_nNumCols; j++) {
            SetEmpty(i, j);
			m_puIndex[i].SetAt(j, uNO_INDEX);
            }
        }
}


// Check for the section header (eg, GRID) in the text stream. It must occur
// alone on a line, but case is not significant.
//
BOOL CGridView::ReadSectionHeader (CStdioFile& fileDesc, PCSZ pszHeader)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadSectionHeader(%x, %s)\n", &fileDesc, F(pszHeader)));
	ASSERT(fileDesc.m_hFile != CFile::hFileNull);
	CString sHeader (pszHeader);

	// NOTE: exceptions must be handled below (by caller or caller's caller, etc)
	CStringx sLine;
	BOOL bOK = TRUE;
	while (bOK && (sHeader.CompareNoCase(sLine) != 0)) {
		bOK = fileDesc.ReadString(sLine);
		if (bOK) {
			DEBUG_OUT(TL_VERBOSE, ("line: %s\n", (PCSZ)sLine));
			sLine.Trim();
		}
	}

	// Complain if not found
	if (!bOK) {
		CString sError;
		sError.Format("Missing %s section in desc", pszHeader);
		MessageBox(sError);
		}

	return (bOK);
}


// Read the clues of a given section (across or down). The stream contains
// one line per clue consisting of a number-text pair (eg, "7 fubar").
// The resulting text will be formatted with RTF.
//
BOOL CGridView::ReadSectionClues (CStdioFile& fileDesc, PCSZ pszHeader,
								  CStringArray& asClues, INT& nCluesLen)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadClues(%x, %s, %x, %x)\n", 
			  &fileDesc, pszHeader, &asClues, &nCluesLen));

	asClues.RemoveAll();
	ReadSectionHeader(fileDesc, pszHeader);
#if FALSE
	// Initialize the string buffer with the RTF header ("{\rtf1\ansi ...").
	// Also, format the clue section name in boldface
	// TODO: add a font spec (based on user preference)
	sClues.Format("%s%s%s%s%s", szRTF_HEADER, 
			      szRTF_START_BOLD, pszHeader, szRTF_END_BOLD, szRTF_NEW_PARA);
#endif

	// Read each clue spec from the file, and add to RTF buffer
	// TODO: catch any exceptoion
	nCluesLen = 0;
	CStringx sLine;
	INT nLastNum = 0;
	BOOL bOK = TRUE;
	while (bOK) {
		bOK = fileDesc.ReadString(sLine);
		if (bOK) {
			sLine.Trim();
			if (sLine.IsEmpty()) {
				break;
			}

			INT nPos = 0;
			CString sNum = ExtractToken(sLine, nPos, " \t");
			INT nNum = StringToInt(sNum);
			bOK = (nNum > 0);
			if (bOK) {
				CString sText = sLine.Mid(nPos);
				CString sEntry;
				DEBUG_OUT(TL_VERBOSE, ("\t%d %s\n", nNum, sText));
#if FALSE
				sEntry.Format("%d %s%s",
							  nNum, (PCSZ)sText, szRTF_NEW_PARA);
				sClues += sEntry;
#else
				sEntry.Format("%d %s", nNum, (PCSZ)sText);
				nCluesLen += sEntry.GetLength();
				asClues.Add(sEntry);
#endif
				ASSERT(nNum > nLastNum);
				nLastNum = nNum;
			}
			else {
				CString sError;
				sError.Format("Invalid clue spec (sequence disorder): %s", sLine);
				MessageBox(sError);
			}
		}
	}

#if FALSE
	// Add the RTF trailer ("}") to the string
	sClues += szRTF_TRAILER;
#endif

	return (bOK);
}


// Read the clues from the text stream. There is one clue per line as follows:
//
//		ACROSS
//		1 Energy source 
//		5 Trolley's "garage" 
//		9 Bassoon's kin 
//		...
//		DOWN
//		1 Where Saint Moritz is 
//		2 Fight on horseback 
//		3 Bridge expert Sharif 
//		...
//

BOOL CGridView::ReadClues (CStdioFile& fileDesc)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadClues(%x)\n", &fileDesc));
	ASSERT(fileDesc.m_hFile != CFile::hFileNull);
	BOOL bOK = TRUE;

	// Read the phrases for the across clues
	bOK = ReadSectionClues(fileDesc, szACROSS, m_asCluesAcross, m_nCluesAcrossLen);
	FillColumnsWithClues(m_aedCluesAcross, m_asCluesAcross, m_nCluesAcrossLen, szACROSS);

	// Read the phrases for the down clues
	bOK = bOK && ReadSectionClues(fileDesc, szDOWN, m_asCluesDown, m_nCluesDownLen);
	FillColumnsWithClues(m_aedCluesDown, m_asCluesDown, m_nCluesDownLen, szDOWN);

	return (bOK);
}


// Read the grid spec from the text stream. After the grid dimensions are given,
// there is one line for each blacked cell as follows. (The coordinates are 0-based.)
//
//		GRID
//		15 15
//		4, 0 
//		9, 0 
//		10, 0 
//		4, 1 
//		...
//
BOOL CGridView::ReadGrid (CStdioFile& fileDesc)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadGrid(%x)\n", &fileDesc));
	ASSERT(fileDesc.m_hFile != CFile::hFileNull);
	BOOL bOK = TRUE;

	// Make sure we're positioned at the grid section
	bOK = ReadSectionHeader(fileDesc, "GRID");
	if (!bOK) {
		return (FALSE);
	}

	// Extract the grid dimensions and validate them
	// If ok, initialize the grid
	CStringx sLine;
	CStringx sNum;
	INT nPos = 0;
	bOK = fileDesc.ReadString(sLine);
	sNum = ExtractToken(sLine, nPos, " \t,");
	INT nNumRows = StringToInt(sNum);
	sNum = ExtractToken(sLine, nPos, " \t,");
	INT nNumCols = StringToInt(sNum);
	DEBUG_OUT(TL_BASIC, ("grid size spec: %dx%d\n", nNumRows, nNumCols));
	if ((nNumRows <= 0) || (nNumCols <= 0)) {
		CString sError;
		sError.Format("Invalid grid spec: %d x %d\n", nNumRows, nNumCols);
		MessageBox(sError);
		return (FALSE);
	}
	InitGrid(nNumRows, nNumCols);

	// Determine the position of each dark cell
	while (bOK) {
		bOK = fileDesc.ReadString(sLine);
		if (bOK) {
			sLine.Trim();
			nPos = 0;
			// Stop if an emty line is encountered
			if (sLine.GetLength() == 0) {
				break;
				}

			//	Get the coordinate pair: row, col
			sNum = ExtractToken(sLine, nPos, " \t,");
			INT nCol = StringToInt(sNum);
			sNum = ExtractToken(sLine, nPos, " \t,");;
			INT nRow = StringToInt(sNum);

			// Validate and blacken appropriate square
			bOK = ((nRow >= 0) && (nRow < m_nNumRows)
				   && (nCol >= 0) && (nCol < m_nNumCols));
			if (bOK) {
				DEBUG_OUT(TL_VERBOSE, ("\t(%d, %d)\n", nRow, nCol));
				ASSERT(IsEmpty(nRow, nCol));
				SetBlackened(nRow, nCol);
			}
			else {
				CString sError;
				sError.Format("Invalid dark cell spec: %d %d\n", nRow, nCol);
				MessageBox(sError);
			}
		}
	}

	// Assign cell indices based on the dark cells
	ReviseIndices();

	return (bOK);
}


BOOL CGridView::ReadState (CStdioFile& fileDesc)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadState(%x)\n", &fileDesc));
	ASSERT(fileDesc.m_hFile != CFile::hFileNull);
	BOOL bOK = TRUE;

	// Make sure we're positioned at the grid section
	bOK = ReadSectionHeader(fileDesc, szSTATE);
	if (!bOK) {
		return (FALSE);
		}

	// Read the contents of the each squares
	// TODO: validate the contents
	CONST CHAR szSPACES[] = " \t";
	CString sLine;
	INT nRow = 0;
	INT nNumCols = -1;
	while (bOK && (nNumCols != 0)) {
		bOK = fileDesc.ReadString(sLine);
		INT nPos = 0;
		// Get the cells, which should be enclosed in quotes
		CString sCells = ExtractToken(sLine, nPos, szNEW_LINE);
		DEBUG_OUT(TL_VERBOSE,("cells='%s'\n", (PCSZ)sCells));
		if (bOK) {
			nNumCols = sCells.GetLength();
			if ((nNumCols > m_nNumCols) || (nRow > m_nNumRows)) {
				InitGrid(nRow, nNumCols);
				}
	        for (INT nCol = 0; nCol < nNumCols; nCol++) {
				m_pcRow[nRow][nCol] = (BYTE)sCells[nCol];
				}
			}
		nRow++;
		}

	// Assign cell indices based on the dark cells
	ReviseIndices();

	return (bOK);
}


//	Read in a puzzle description, containing the clues and grid layout.
//	There are three sections in the file: across, down, and grid. Each
//	section starts with its correspeconding label (eg, DOWN) and ends
//	with a blank line.
//
//		ACROSS
//		1 Energy source
//		...
//
//		DOWN
//		1 Where Saint Moritz is
//		...
//
//		GRID
//		15 15
//		4, 0
//		...
//

BOOL CGridView::ReadPuzzle (CStdioFile& fileDesc)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadPuzzle(%x)\n", &fileDesc));
	BOOL bOK = TRUE;

#if TRUE
	if (m_hWnd != NULL) {
		OnInitialUpdate();
		}
#endif

	try {
		bOK = bOK && ReadClues(fileDesc);
		bOK = bOK && ReadGrid(fileDesc);
		ReadState(fileDesc);			//  the state is optional
		bJustReadIn = TRUE;
	}
	catch (CException* pExc) {
		// TODO: customize the error dialog
		pExc->ReportError();
	}

	return (bOK);
}


BOOL CGridView::ReadPuzzle (CONST CString& sFile)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::ReadPuzzle(%s)\n",
			  (PCSZ)sFile));

	BOOL bOK = FALSE;
	CStdioFile fileDesc;
	bOK = fileDesc.Open(sFile, CFile::modeRead);
	if (bOK) {
		SetRedraw(FALSE);
		bOK = ReadPuzzle(fileDesc);
		fileDesc.Close();
		SetRedraw(TRUE);
		Invalidate();
		}
	else {
		CString sError;
		sError.Format("Unable to read %s\n", (PCSZ)sFile);
		MessageBox(sError);
	}

	return (bOK);
}


// Write the grid description, including the position of the blackened
// squares and the state of each cell.
//
// TODO: Make sure the word list is up-to-date with the RTF columns
//
BOOL CGridView::WriteClues(CStdioFile& fileDesc, PCSZ pszHeader,
						   CStringArray& asClues)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::WriteClues(%x, %s, %x)\n", 
			  &fileDesc, pszHeader, &asClues));
	ASSERT(fileDesc.m_hFile != CFile::hFileNull);
	CString sLine;
	BOOL bOK = TRUE;

	// Write the ACROSS or DOWN header
	sLine.Format(szHEADER_FMT, pszHeader);
	fileDesc.WriteString(sLine);
	for (INT i = 0; i < asClues.GetSize(); i++) {
		sLine.Format(szCLUE_FMT, (PCSZ)asClues[i]);
		fileDesc.WriteString(sLine);
		}
	fileDesc.WriteString(szNEW_LINE);

	return (bOK);
}


// Write the grid description, including the position of the blackened
// squares and the state of each cell.
//
BOOL CGridView::WriteGrid (CStdioFile& fileDesc)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::WriteGrid(%x)\n", &fileDesc));
	ASSERT(fileDesc.m_hFile != CFile::hFileNull);
	CString sLine;
	BOOL bOK = TRUE;

	// Write the GRID header and the dimensions
	sLine.Format(szHEADER_FMT, szGRID);
	fileDesc.WriteString(sLine);
	sLine.Format(szNUM_PAIR_FMT, m_nNumRows, m_nNumCols);
	fileDesc.WriteString(sLine);

	// Write the coordinates of the blackened squares
    for (INT i = 0; i < m_nNumRows; i++) {
        for (INT j = 0; j < m_nNumCols; j++) {
			if (IsBlackened(i, j)) {
				sLine.Format(szNUM_PAIR_FMT, i, j);
				fileDesc.WriteString(sLine);
				}
			m_pcRow[i][j];
			}
		}
	fileDesc.WriteString(szNEW_LINE);

	return (bOK);
}


BOOL CGridView::WriteState (CStdioFile& fileDesc)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::WriteState(%x)\n", &fileDesc));

	// Write the STATE header
	CString sLine;
	sLine.Format(szHEADER_FMT, szSTATE);
	fileDesc.WriteString(sLine);
	BOOL bOK = TRUE;

	// Write the contents of the each squares
    for (INT i = 0; i < m_nNumRows; i++) {
		CString sCells;
        for (INT j = 0; j < m_nNumCols; j++) {
			sCells += m_pcRow[i][j];
			}
		sLine.Format(szSTATE_FMT, (PCSZ)sCells);
		fileDesc.WriteString(sLine);
		}
	fileDesc.WriteString(szNEW_LINE);

	return (bOK);
}


BOOL CGridView::WritePuzzle (CONST CString& sFile)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::WritePuzzle(%s)\n",
			  (PCSZ)sFile));

	BOOL bOK = FALSE;
	CStdioFile fileDesc;
	bOK = fileDesc.Open(sFile, CFile::modeWrite | CFile::modeCreate);
	if (bOK) {
		try {
			bOK = bOK && WriteClues(fileDesc, szACROSS, m_asCluesAcross);
			bOK = bOK && WriteClues(fileDesc, szDOWN, m_asCluesDown);
			bOK = bOK && WriteGrid(fileDesc);
			bOK = bOK && WriteState(fileDesc);
		}
		catch (CException* pExc) {
			// TODO: customize the error dialog
			pExc->ReportError();
			}
		}
	else {
		CString sError;
		sError.Format("Unable to open '%s' for writing\n", (PCSZ)sFile);
		AfxMessageBox(sError);
		}

	return (bOK);
}


//	Draw the numeric text for each index in the crossword. These values
//	are stored in an array, so they don't need to be determined on the fly.
//
VOID CGridView::DrawIndices (CDC* pDC)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::DrawIndices(%x)\n", pDC));

    //  Create the index font with the character cell proportional to the grid cell
	//	TODO: cache the index font
	CFont fontIndex;
	INT nFontSize = -4 * m_nCellHeight/10;		// roughly 40% of the cell height
    fontIndex.CreateFont(nFontSize, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "Arial");
    CFont* pOldFont = pDC->SelectObject(&fontIndex);

	// Make sure the text background is not rendered
	pDC->SetBkMode(TRANSPARENT);

	//  Fill in the indices for the cells that aren't blackened and that
	//	qualify for an index (see ReviseIndices for the rules)
	COLORREF colorOld = pDC->SetTextColor(m_colorIndexText);
    for (INT i = 0; i < m_nNumRows; i++) {
        for (INT j = 0; j < m_nNumCols; j++) {
            CRect rect = GetCellRect(i, j);
			// pDC->LPtoDP(rect);
			InflateRect(rect, 0, -1);
			if (m_puIndex[i][j] != uNO_INDEX) {
				CString sIndex;
				sIndex.Format("%u", m_puIndex[i][j]);
                pDC->DrawText(sIndex, -1, rect, DT_TOP | DT_LEFT);
				}
			}
        }

    //	Restore the old GDI resources
	pDC->SetTextColor(colorOld);
    pDC->SelectObject(pOldFont);
}


//	Update the crossword grid and cell contents.
//
VOID CGridView::OnDraw (CDC* pDC)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnDraw(%x)\n", pDC));
    INT i;
    INT j;
	CBrush* pOldBrush;
	CPen* pOldPen;

	//	Set the mapping mode for 0.01" for each logical point
    pDC->SetMapMode(MM_LOENGLISH);

	//	Check out the invalid rectange
#if defined(_DEBUG)
	CRect rectClip;
	INT nClipType = pDC->GetClipBox(&rectClip); // COMPLEXREGION
	DEBUG_OUTF(TL_VERBOSE, ("clip rect (type %d) ", nClipType));
	DUMP_RECT(TL_VERBOSE, "in LP: ", rectClip, "; ");
	// pDC->LPtoDP(&rectClip);
	DUMP_RECT(TL_VERBOSE, "in DP: ", rectClip, "\n");
#endif

	//	Paint the grid background
	//	TODO: make this more efficient in regard to clipping
	//		  a separate window would be nice
	CBrush brushGrid (m_colorGridBackground);
    pOldBrush = pDC->SelectObject(&brushGrid);
	pDC->SelectObject(&brushGrid);
	CRect rectGrid (m_rectGrid.left, 0, 
					m_rectGrid.left + m_nCellWidth * m_nNumRows, -m_nCellHeight * m_nNumCols);
	// pDC->LPtoDP(rectGrid);
    pDC->Rectangle(rectGrid);
	pDC->SelectObject(pOldBrush);
	// ASSERT(rectGrid == m_rectGrid);

    //  Draw the crossword grid lines
	CPen penGrid (PS_SOLID, 0, m_colorGridBorder);
    pOldPen = pDC->SelectObject(&penGrid);
    for (i = 0; i <= m_nNumRows; i++) {
        pDC->MoveTo(m_rectGrid.left, -i * m_nCellHeight);
        pDC->LineTo(m_rectGrid.left + m_nNumCols * m_nCellWidth, -i * m_nCellHeight);
        for (j = 0; j <= m_nNumCols; j++) {
            pDC->MoveTo(m_rectGrid.left + j * m_nCellWidth, 0);
            pDC->LineTo(m_rectGrid.left + j * m_nCellWidth, -m_nNumRows * m_nCellHeight);
            }
        }
	pDC->SelectObject(pOldPen);

            
    //  Draw a highlight rectangle for the current cell
	CBrush brushHighlight (m_colorActiveCell);
    pOldBrush = pDC->SelectObject(&brushHighlight);
	CRect rect = GetCurrentCellRect();
    pDC->Rectangle(rect);
    pDC->SelectObject(pOldBrush);

	//	Draw the indices first to give the letters precedence.
	DrawIndices(pDC);

    //  Create the letter font, matching the grid cell height to the character cell height
	//	TODO: cache the font
    CFont letter_font;
	INT nFontSize = -8 * m_nCellHeight/10;		// roughly 80% of the cell
    letter_font.CreateFont(nFontSize, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, "Arial");
    CFont* pOldFont = pDC->SelectObject(&letter_font);

    //  Create black blush for filling a blackened cell
    CBrush brushBlackenedCell(m_colorBlackened);
    pOldBrush = pDC->SelectObject(&brushBlackenedCell);

	//	Make sure the text background is not rendered
	pDC->SetBkMode(TRANSPARENT);

    //  Fill in the cells that the user typed letters into.
    //  And, copy the bitmap over each of the "blackened" squares
	COLORREF colorOld = pDC->SetTextColor(m_colorGridText);
    for (i = 0; i < m_nNumRows; i++) {
        for (j = 0; j < m_nNumCols; j++) {
            CRect rect = GetCellRect(i, j);
			// pDC->LPtoDP(rect);
			InflateRect(rect, 0, -1);
            if (IsBlackened(i, j)) {
                DEBUG_OUT(TL_VERBOSE, ("blackening cell [%d,%d] (h=%d)", 
                          i, j, m_nCellHeight));
				DUMP_RECT(TL_VERBOSE, " at ", rect, "\n");
                pDC->Rectangle(rect);
                }
            else if (!IsEmpty(i, j)) {
                DEBUG_OUT(TL_VERBOSE, ("setting letter %c in cell [%d,%d]", 
                            m_pcRow[i][j], i, j));
				DUMP_RECT(TL_VERBOSE, " at ", rect, "\n");
                CStringx letter (m_pcRow[i][j]);
                //pDC->DrawText(letter, -1, rect, DT_CENTER | DT_VCENTER | DT_EXTERNALLEADING);
				//pDC->DrawText(letter, -1, rect, DT_RIGHT | DT_BOTTOM);
				//pDC->DrawText(letter, -1, rect, DT_RIGHT | DT_BOTTOM);
				pDC->DrawText(letter, -1, rect, DT_CENTER | DT_VCENTER);
                }
            }
        }

    //  restore the old GDI resources
    pDC->SelectObject(pOldFont);
    pDC->SelectObject(pOldBrush);
	pDC->SetTextColor(colorOld);

	//	Let the base class do the dialog-control updates    
    CFormView::OnDraw(pDC);
}


MAKE_CLASS_CAST(CRtfEdit);	// put into rtfedit.h

VOID CGridView::CreateRtfColumns (CRtfEditArray& aedRtfColumn, INT nResourceIdBase)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::CreateColumnControls(%x, %d)\n",
			  &aedRtfColumn, nResourceIdBase));
	
    CRect rect(666, 666, 666, 666);
	DWORD dwEdStyle = WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_MULTILINE | ES_READONLY;
					  // | WS_VSCROLL;
	aedRtfColumn.SetSize(nMAX_COLUMNS);
	for (INT i = 0; i < aedRtfColumn.GetSize(); i++) {
		CRtfEdit* pRtfEdit = new CRtfEdit;
		pRtfEdit->Create(dwEdStyle, rect, this, nResourceIdBase + i);
		//pRtfEdit->SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)clrLIGHT_GRAY);
		pRtfEdit->ChangeTextColor(m_colorClueText);
		pRtfEdit->ChangeBackgroundColor(m_colorClueBackground);
		pRtfEdit->NotifyOfClicks();
		aedRtfColumn[i] = pRtfEdit;
	}
}


VOID CGridView::DeleteRtfColumns (CRtfEditArray& aedRtfColumn)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::DeleteColumnControls(%x)\n",
			  &aedRtfColumn));

	INT nSize = aedRtfColumn.GetSize();
	for (INT i = 0; i < nSize; i++) {
		CRtfEdit* pRtfEdit = aedRtfColumn[i];
		ASSERT(ToCRtfEdit(pRtfEdit) != NULL);
		delete pRtfEdit;
	}
	aedRtfColumn.SetSize(0);
}


VOID CGridView::ColorRtfColumns (CRtfEditArray& aedRtfColumn)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::ColorRtfColumns(%x)\n",
			  &aedRtfColumn));

	for (INT i = 0; i < aedRtfColumn.GetSize(); i++) {
		CRtfEdit* pRtfEdit = aedRtfColumn[i];
		ASSERT_VALID(pRtfEdit);
		pRtfEdit->ChangeTextColor(m_colorClueText);
		pRtfEdit->ChangeBackgroundColor(m_colorClueBackground);
		}
}


// MoveRtfColumns: move the individual columns to their respective positions within
// the rectange (in device coordinates)
//
// for now the rectangle is just divided into 3 columns
//
VOID CGridView::MoveRtfColumns (CRtfEditArray& aedRtfColumn, CRect& rectClues, CDC* pDC)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::MoveColumnControls(%x, %x, %x)\n",
			  &aedRtfColumn, &rectClues, pDC));

	INT nNumColumns = min(3, aedRtfColumn.GetSize());
	INT nColumnWidth = MulDiv(rectClues.Width(), 1, 3);
	for (INT i = 0; i < nNumColumns; i++) {
		CRect rectColumn (rectClues.left + i * nColumnWidth, rectClues.top,
						  rectClues.left + (i + 1) * nColumnWidth, rectClues.bottom);
		CRtfEdit* pRtfEdit = aedRtfColumn[i];
		ASSERT(ToCRtfEdit(pRtfEdit) != NULL);
		if (pRtfEdit) {
			pRtfEdit->MoveWindow(rectColumn);
			DEBUG_OUT(TL_VERBOSE, ("clues[%d] ", i));
			DUMP_RECT(TL_VERBOSE, "rect: ", rectColumn, "\n");
		}
	}
}


VOID CGridView::FillColumnsWithClues(CRtfEditArray& aedRtfColumn, CStringArray& asClues,
									 INT nSectionTextLen, PCSZ pszSection)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::FillColumnsWithClues(%x, %x, %s)\n",
			  &aedRtfColumn, &asClues, F(pszSection)));

	INT nNumColumns = min(3, asClues.GetSize());
	INT nAvgColumnTextLen = MulDiv(nSectionTextLen, 1, nNumColumns);
	INT nPos = 0;
	INT nLastColumn = nNumColumns - 1;
	for (INT i = 0; i < nNumColumns; i++) {
		CString sClues;
		// Initialize the string buffer with the RTF header ("{\rtf1\ansi ...").
		// Also, format the clue section name in boldface if column 1
		// TODO: add a font spec (based on user preference)
		sClues.Format(szRTF_HEADER);
		sClues += szRTF_SMALL_FONT;
		sClues += szRTF_HANGING_INDENT;
		INT nColumnTextLen = 0;
		if (i == 0) {
			CString sSectionHead;
			sSectionHead.Format("%s %s %s %s\r\n", szRTF_NEW_PARA,
						szRTF_START_BOLD, pszSection, szRTF_END_BOLD);
			sClues += sSectionHead;
			// TODO: account for the effect of small entries
			nColumnTextLen += StringLength(pszSection);
		}
		while ((nColumnTextLen <= nAvgColumnTextLen) || (i == nLastColumn)) {
			if (nPos == asClues.GetSize()) {
				break;
			}
			PCSZ pszText = asClues[nPos];
			CString sEntry;
			sEntry.Format("%s %s\r\n", szRTF_NEW_PARA, pszText);
			sClues += sEntry;
			nColumnTextLen += asClues[nPos].GetLength();
			nPos++;
		}

		// Add the RTF trailer ("}") to the string
		DEBUG_OUT(TL_GARRULOUS, ("col%d: %s\n", i, sClues));
		sClues += szRTF_TRAILER;

		// Send the text to the control
		CRtfEdit* pRtfEdit = aedRtfColumn[i];
		ASSERT(ToCRtfEdit(pRtfEdit) != NULL);
		if (pRtfEdit) {
			pRtfEdit->SetText(sClues);
		}
	}
}


VOID CGridView::OnInitialUpdate (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnInitialUpdate()\n"));
    CFormView::OnInitialUpdate();       // tie into Document

    //  Create a small font for definitions
    //m_def.CreateFont(-8, "Arial");
	//m_def.CreateFont(8, "Arial");

	// If a puzzle was just read in, don't reinitialize things
	// TODO: get the MFC interface down
	if (bJustReadIn) {
		ASSERT(m_edClue.m_hWnd != NULL);
	    UpdateData(bINITIALIZE);

		// Set the colors for the embedded controls (mainly the RTF ones)
		SetWindowColors();

		//	Reposition the controls around the crossword
		ResizeControls();
		Invalidate();
		bJustReadIn = FALSE;
		return;
		}

	// If this is the second or so time around, delete the dynamic data
	// and start anew (well almost, since some of the embbeded controls
	// are not dynamic, such as m_edClue)
	// TODO: make all embedded controls dynamic
	BOOL bFirstTime = TRUE;
	if (m_edClue.m_hWnd != NULL) {
		CleanupGridView();
		InitGrid(DEF_NUM_ROWS, DEF_NUM_COLS);
		bFirstTime = FALSE;
		}

	//	Override the edit box creation (bogus size at first)
	// TODO: use MFC's control
	if (bFirstTime) {
		CRect rect(666, 666, 666, 666);
		DWORD dwEdStyle = WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_MULTILINE 
						  | WS_VSCROLL;
		m_edClue.Create(dwEdStyle, rect, this, IDC_CLUE);
#if defined(_DEBUG)
#else
		dwEdStyle |= ES_READONLY;
#endif
		m_edWords.Create(dwEdStyle, rect, this, IDC_WORDS);
		//m_edCluesAcross.Create(dwEdStyle, rect, this, IDC_CLUES_ACROSS);
		//m_edCluesDown.Create(dwEdStyle, rect, this, IDC_CLUES_DOWN);
		}

	SetRedraw(FALSE);

	// Create the RtfEdit controls for the columns
	CreateRtfColumns(m_aedCluesAcross, IDC_CLUES_ACROSS_BASE);
	CreateRtfColumns(m_aedCluesDown, IDC_CLUES_DOWN_BASE);

#if FALSE
	//	TODO: add a CRtfEdit wrapper for this
	m_edWords.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)clrLIGHT_GRAY);
	//m_edCluesAcross.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)clrLIGHT_GRAY);
	//m_edCluesDown.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)clrLIGHT_GRAY);
	m_edClue.SendMessage(EM_SETBKGNDCOLOR, FALSE, (LPARAM)clrLIGHT_GRAY);
#endif

	//	Create object for doing the lookups for clues
	m_pLookup = new CLookup(this);
    //  specify the mouse cursor to replace the hourglass during lookups
    //m_pLookup->SetWaitCursor(IDC_COFFEE);

	// Read in the puzzle description
	//CString sFile ("xword_desc.txt");
	if (bFirstTime) {
		CString sFile;
		if (AfxGetApp()->m_lpCmdLine[0] != NULL) {
			// TODO: option switch (eg, /x nyt19may96.lst)
			sFile = AfxGetApp()->m_lpCmdLine;
			}
		if (!sFile.IsEmpty()) {
			ReadPuzzle(sFile);
			}
		}
    
	// Set the clue prompt and protect it
	// This is entirely stupid
	DWORD dwEventMask = m_edClue.SendMessage(EM_GETEVENTMASK);
	m_edClue.SendMessage(dwEventMask | ENM_CHANGE);
	m_sClue = szCLUE;
	m_edClue.SetText(ToRtf(m_sClue));
	//INT nStart;
	//INT nEnd;
	//m_edClue.GetSel(nStart, nEnd);
	m_edClue.SetSel(0, m_sClue.GetLength(), TRUE);
    CHARFORMAT cf;
   	cf.cbSize = sizeof(cf);
	cf.dwMask = CFM_PROTECTED;
	cf.dwEffects = CFE_PROTECTED;
    LRESULT lRes = m_edClue.SendMessage(EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	ASSERT(lRes);
	//	Restore the selection
	//m_edClue.SetSel(nStart, nEnd, TRUE);
	m_edClue.SetSel(m_sClue.GetLength(), m_sClue.GetLength(), TRUE);
	m_edClue.SetRedraw(TRUE);

	// Set grid scroll size
    SetScrollSizes(MM_LOENGLISH, CSize(0,0));

#if FALSE
    CWnd* pGridFrame = GetDlgItem(IDC_GRID_FRAME);
    ASSERT(pGridFrame);
    CRect rectFrame;
    if (pGridFrame) {
        pGridFrame->GetClientRect(rectFrame);
        m_nCellHeight = (rectFrame.bottom + m_nNumRows - 1) / m_nNumRows;
        m_nCellWidth = (rectFrame.right + m_nNumCols - 1) / m_nNumCols;
        DEBUG_OUT(TL_VERBOSE, ("rectGridFrame: (%d, %d, %d, %d); cell dimension: %dx%d\n",
              rectFrame.left, rectFrame.top, rectFrame.right, rectFrame.bottom,
              m_nCellWidth, m_nCellHeight));
        }
#endif

    UpdateData(bINITIALIZE);

	// Set the colors for the embedded controls (mainly the RTF ones)
	SetWindowColors();

	//	Reposition the controls around the crossword
	ResizeControls();

	SetRedraw(TRUE);
	Invalidate();
}


VOID CGridView::OnUpdate(CView* pChangedView, LPARAM lHint, CObject* pHint)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnUpdate(%x, %x, %x)\n",
			  pChangedView, lHint, pHint));
	UNREFERENCED_PARAMETER(pHint);

	if (lHint == lHINT_OPTION_CHANGE) {
		GetOptions();
		SetWindowColors();
		ResizeControls();
		Invalidate();
		UpdateWindow();
		}

	CView::OnUpdate(pChangedView, lHint, pHint);
}


VOID CGridView::SetWindowColors (VOID)
{
    DEBUG_OUT(TL_VERBOSE, ("CGridView::SetWindowColors()\n"));

	// Just change the colors in the RTF controls since the others get
	// handled in OnCtlColor
	m_edWords.ChangeTextColor(m_colorClueText);
	m_edWords.ChangeBackgroundColor(m_colorClueBackground);
	m_edClue.ChangeTextColor(m_colorClueText);
	m_edClue.ChangeBackgroundColor(m_colorClueBackground);
	ColorRtfColumns(m_aedCluesAcross);
	ColorRtfColumns(m_aedCluesDown);
}


//	Reposition the controls based on the size of the crossword. The layout
//	should be as follows:
//
//	+------+--------------------+
//	|across| crossword          |
//	   :	   ...				
//	|  :   |   grid             |
//	+------+--------------------+
//	|clue  +  down     :        |
//	+sugg  +     :     :        |
//	+---------------------------+
//
//  NOTE: the across and down sections are divided into columns of Rtf edit
//	      controls
//
//	TODO: use separate views or windows to make this less cluttered
//

VOID CGridView::ResizeControls (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::ResizeControls()\n"));

	//	Make sure the controls are initialized
	CWnd* pEditWnd = GetDlgItem(IDC_CWORD_CHAR);
	if ((!pEditWnd) || (pEditWnd->m_hWnd == NULL)) {
		DEBUG_OUTF(TL_VERBOSE, ("Control windows not set yet\n"));
		return;
		}

	if ((m_aedCluesAcross.GetSize() == 0) || (m_aedCluesAcross[1]->m_hWnd == NULL)) {
		DEBUG_OUTF(TL_VERBOSE, ("Control windows not yet set\n"));
		return;
	}

	//	Make sure the controls are there
#if TRUE
	//	TODO: use the DDX members m_def, et al, stupid
	//CWnd* pDefWnd = GetDlgItem(IDC_DEF);
	CWnd* pClueWnd = GetDlgItem(IDC_CLUE);
	//CWnd* pClueLabelWnd = GetDlgItem(IDC_CLUE_LABEL);
	//x// CWnd* pSuggestWnd = GetDlgItem(IDC_SUGGESTIONS);
	CWnd* pWords = GetDlgItem(IDC_WORDS);
	ASSERT(pClueWnd && pWords);
	if (!(pClueWnd && pWords)) {
		return;
		}
#endif

	// Compute the size of the grid in logical coordinates 
	// (Add extra row & column for spacing).
	// NOTE: the grid is fixed width, so it is placed flush right
	//		 at the top
	CClientDC dc(this);
	dc.SetMapMode(MM_LOENGLISH);
#if FALSE
	CRect rectGrid (0, 0,
					(m_nCellWidth + 1) * m_nNumRows, 
					-(m_nCellHeight + 1) * m_nNumRows);
#else
	CRect rectGrid (0, 0,
					m_nCellWidth * m_nNumRows, 
					-m_nCellHeight * m_nNumCols);
#endif

	//	Convert to device coordinates
	dc.LPtoDP(rectGrid);
	DUMP_RECT(TL_VERBOSE, "grid dimensions (DP): ", rectGrid, "\n");

	// Get the window dimensions
	CRect rectWindow;
	GetClientRect(rectWindow);
	//dc.LPtoDP(rectWindow);
	m_rectGrid.left = max(0, rectWindow.right - rectGrid.right);
	INT nRemHeight = max(0, rectWindow.bottom - rectGrid.bottom);
	INT nRemWidth = m_rectGrid.left;

	// Logically reposition the grid: top-right corner
	rectGrid.OffsetRect(nRemWidth, 0);
	DUMP_RECT(TL_VERBOSE, "grid position: ", rectGrid, "\n");
	//?//m_frameGrid.MoveWindow(rectGrid);
	//?//dc.BitBlt(rectGrid ... );

	// Save rect position in logical coordinates for later use
	m_rectGrid = rectGrid;
	dc.DPtoLP(m_rectGrid);

#if FALSE
	//	Resize the definition box
	CRect rectDef(rectGrid.right, 0, rectWindow.right, rectWindow.bottom);
	pDefWnd->MoveWindow(rectDef);
	DUMP_RECT(TL_VERBOSE, "def rect: ", rectDef, "\n");
#endif


	// Resize the Clues Across box: top-left corner (adjacent to grid)
	//???// CRect rectCluesAcross (0, 0, rectGrid.left, rectGrid.bottom);
	CRect rectCluesAcross (0, 0, rectGrid.left, rectGrid.bottom);
	//m_edCluesAcross.MoveWindow(rectCluesAcross);
	DUMP_RECT(TL_VERBOSE, "clues across rect: ", rectCluesAcross, "\n");
	MoveRtfColumns(m_aedCluesAcross, rectCluesAcross, &dc);

	// Resize the Clues Down box: bottom-right corner (adjacent to grid)
	CRect rectCluesDown (rectGrid.left / 2, rectGrid.bottom, rectWindow.right, rectWindow.bottom);
	//m_edCluesDown.MoveWindow(rectCluesDown);
	DUMP_RECT(TL_VERBOSE, "clues down rect: ", rectCluesDown, "\n");
	MoveRtfColumns(m_aedCluesDown, rectCluesDown, &dc);

	// Resize the clue box & label; always maintain the left position and height
	TEXTMETRIC tm;
	dc.GetTextMetrics(&tm);
	CFont* pOldFont = dc.SelectObject(GetFont());
	INT nClueHeight =  3 * tm.tmHeight / 2;
	dc.SelectObject(pOldFont);
	//CRect rectClueLabel(0, rectGrid.bottom);
	//					0, rectGrid.bottom + nClueHeight)
#if FALSE
	CRect rectClueLabel;
	pClueLabelWnd->GetClientRect(rectClueLabel);
	rectClueLabel.top = rectGrid.bottom;
	rectClueLabel.bottom = rectGrid.bottom + nClueHeight;
	pClueLabelWnd->MoveWindow(rectClueLabel);
	DUMP_RECT(TL_VERBOSE, "clue label rect: ", rectClueLabel, "\n");
	CRect rectClue;
	pClueWnd->GetClientRect(rectClue);
	//INT nClueHeight = rectClue.bottom - rectClue.top;
	rectClue.left = rectClueLabel.right;
	rectClue.top = rectGrid.bottom;
	rectClue.bottom = rectClue.top + nClueHeight;
	rectClue.right = nRemWidth;
#else
	CRect rectClue;
	pClueWnd->GetClientRect(rectClue);
	//INT nClueHeight = rectClue.bottom - rectClue.top;
	rectClue.left = 0;
	rectClue.top = rectGrid.bottom;
	rectClue.bottom = rectClue.top + nClueHeight;
	rectClue.right = nRemWidth / 2;
#endif
	pClueWnd->MoveWindow(rectClue);
	DUMP_RECT(TL_VERBOSE, "clue rect: ", rectClue, "\n");

	//	Resize the suggestions frame and its embedded word box
	CRect rectSuggestions (0, rectClue.bottom,
						   rectGrid.left / 2, rectWindow.bottom);
	//x// pSuggestWnd->MoveWindow(rectSuggestions);
	//x// DUMP_RECT(TL_VERBOSE, "suggestions rect: ", rectSuggestions, "\n");
	//x// rectSuggestions.OffsetRect(8, 12);
	//x// rectSuggestions.InflateRect(-4, -8);
	pWords->MoveWindow(rectSuggestions);
	DUMP_RECT(TL_VERBOSE, "words rect: ", rectSuggestions, "\n");
}


// TODO: place these before the functions
//
VOID CGridView::DoDataExchange(CDataExchange* pDX)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::DoDataExchange(%x)\n", pDX));
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CGridView)
    //DDX_Control(pDX, IDC_CLUES_ACROSS, m_edCluesAcross);
    //DDX_Control(pDX, IDC_CLUES_DOWN, m_edCluesDown);
    //DDX_Control(pDX, IDC_WORDS, m_edWords);
    //DDX_Control(pDX, IDC_CLUE, m_edClue);
    DDX_Text(pDX, IDC_CWORD_CHAR, m_sChar);
    DDV_MaxChars(pDX, m_sChar, 1);
    //DDX_Text(pDX, IDC_CLUE, m_sClue);
    //DDX_Text(pDX, IDC_CLUES_ACROSS, m_sCluesAcross);
    //DDX_Text(pDX, IDC_CLUES_DOWN, m_sCluesDown);
    DDX_Control(pDX, IDC_CWORD_CHAR, m_cwordChar);
	//}}AFX_DATA_MAP
    //DDX_Control(pDX, IDC_DEF, m_def);

	if (pDX->m_bSaveAndValidate) {
		m_edClue.GetWindowText(m_sClue);
		}
	else if (m_edWords.m_hWnd != NULL) {
		// m_edWord.SetText(m_Word);
		// m_edWord.SetWindowText(m_Word);
		//m_edCluesAcross.SetText(m_sCluesAcross);
		//m_edCluesDown.SetText(m_sCluesDown);
		m_edWords.SetText(ToRtf(m_sWords));
		m_edClue.SetText(ToRtf(m_sClue));
		}
}

                    
BEGIN_MESSAGE_MAP(CGridView, CFormView)
    //{{AFX_MSG_MAP(CGridView)
    ON_EN_CHANGE(IDC_CWORD_CHAR, OnChangeCwordChar)
    ON_COMMAND(ID_HELP_SUGGESTHORIZONTAL, OnSuggestHorizontal)
    ON_COMMAND(ID_HELP_SUGGESTVERTICAL, OnSuggestVertical)
    ON_COMMAND(ID_HELP_SYNONYMS, OnHelpSynonyms)
    ON_COMMAND(ID_HELP_REFERENCES, OnHelpReferences)
    ON_COMMAND(ID_HELP_DEFINITION, OnHelpDefinition)
    ON_COMMAND(ID_MISC_CHANGEFONT, OnMiscChangefont)
    ON_WM_LBUTTONDBLCLK()
    ON_WM_LBUTTONDOWN()
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_WM_CTLCOLOR()
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_COMMAND(ID_CANCEL_LOOKUP, OnCancelLookup)
	ON_UPDATE_COMMAND_UI(ID_CANCEL_LOOKUP, OnUpdateCancelLookup)
	//}}AFX_MSG_MAP
	ON_NOTIFY(WM_KEYDOWN, IDC_CWORD_CHAR, OnNotifyKeydown)
	ON_NOTIFY(EN_SELCHANGE, IDC_CLUE, OnNotifyChange)
	ON_NOTIFY_RANGE(WM_LBUTTONDOWN, IDC_CLUES_ACROSS_BASE, 
					(IDC_CLUES_ACROSS_BASE + nMAX_COLUMNS - 1), OnNotifyButtonDown)
	ON_NOTIFY_RANGE(WM_LBUTTONDOWN, IDC_CLUES_DOWN_BASE, 
					(IDC_CLUES_DOWN_BASE + nMAX_COLUMNS - 1), OnNotifyButtonDown)
    ON_COMMAND(ID_SUGGESTIONS_READY, OnSuggestionsReady)
    ON_COMMAND(ID_SYNONYMS_READY, OnSynonymsReady)
    ON_COMMAND(ID_REFERENCES_READY, OnReferencesReady)
    ON_COMMAND(ID_DEFINITION_READY, OnDefinitionReady)
	ON_UPDATE_COMMAND_UI(ID_HELP_DEFINITION, OnUpdateAnyLookupCmd)
	ON_UPDATE_COMMAND_UI(ID_HELP_REFERENCES, OnUpdateAnyLookupCmd)
	ON_UPDATE_COMMAND_UI(ID_HELP_SYNONYMS, OnUpdateAnyLookupCmd)
	ON_UPDATE_COMMAND_UI(ID_HELP_SUGGESTHORIZONTAL, OnUpdateAnyLookupCmd)
	ON_UPDATE_COMMAND_UI(ID_HELP_SUGGESTVERTICAL, OnUpdateAnyLookupCmd)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGridView message handlers


//	Convert a point in device coordinates to row/col coordinates.
//	Returns TRUE if the point is within the grid. Modifies the
//	reference parameters for the row and column regardless.
//
BOOL CGridView::ConvertPoint (CPoint& point, INT& nRow, INT& nCol)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::ConvertPoint((%d,%d), %x, %x)\n",
							 point.x, point.y, &nRow, &nCol));
    CClientDC cdc(this);
    cdc.SetMapMode(MM_LOENGLISH);
    cdc.DPtoLP(&point);
    
    nRow = -point.y / m_nCellHeight;
    nCol = (point.x - m_rectGrid.left) / m_nCellWidth;
    
    BOOL bOK = (nRow >= 0) && (nRow < m_nNumRows) && (nCol >= 0) && (nCol < m_nNumCols);
	DEBUG_OUT(TL_GARRULOUS, ("%d <= ConvertPoint: (%d,%d)\n", 
			  bOK, point.x, point.y));
    
    return (bOK);
}


VOID CGridView::OnLButtonDown(UINT nFlags, CPoint point)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnLButtonDown(%x, (%d, %d))\n", nFlags, point.x, point.y));
    INT nCol, nRow;
    BOOL bOK = ConvertPoint(point, nRow, nCol);

    //  process the mouse click if within the puzzle   
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnLButtonDown (%d, %d) to (%d, %d)\n", point.x, point.y, nRow, nCol));
    if (bOK) {
        m_cwordChar.SetFocus();
        ChangeCell(nRow, nCol);
        }

    //  otherwise let the framework handle it
    else {
        CFormView::OnLButtonDown(nFlags, point);
        }
}


//	If the user double clicks on a cell, toggle its blackened state. The
//	previous contents of the cell will be lost if it wasn't blank. Also,
//	revise the crossword indices.
//
VOID CGridView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnLButtonDblClk(%x, (%d, %d))\n", nFlags, point.x, point.y));
    INT nCol, nRow;
    BOOL bOK = ConvertPoint(point, nRow, nCol);

    //  process the double click if within the puzzle   
    DEBUG_OUT(TL_VERBOSE, ("CGridView::OnLButtonDblClk (%d, %d)\n", nRow, nCol));
    if (bOK) {
        //  toggle the darkened status of the cell
        if (IsBlackened(nRow, nCol)) {
			SetEmpty(nRow, nCol);
            }
        else {
            SetBlackened(nRow, nCol);
            }
        ChangeCell(nRow, nCol);
		ReviseIndices();
        }

    //  otherwise let the framework handle it
    else {
        CFormView::OnLButtonDblClk(nFlags, point);
        }
}


//	Revise the indices of the crossword grid. When a particular index
//	changes all subsequent indices in the row-order traversal must be
//	revised.
//
//	Unblackend cells get indices if one of the following hold
//		- It is in the topmost row (row 0)
//		- It is in the leftmost column (col 0)
//		- The cell immediately above it is blackened
//		- The cell to its immediate left if blackened
//
//	The following simplified grid illustrates the rules:
//
//		1   *   2   3   *   4
//		5   6   _   _   _   _
//		7   _   *   8   _   *
//		9   *   10  *   11  _
//		*   12  _   13  _   _
//
VOID CGridView::ReviseIndices (VOID)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::ReviseIndices()\n"));

	UINT uNextIndex = 1;
    for (INT i = 0; i < m_nNumRows; i++) {
        for (INT j = 0; j < m_nNumCols; j++) {
			BOOL bGetsIndex = FALSE;
			if (!IsBlackened(i, j)) {
				//	Note that the order of evaluation is important since
				//	the final two conditions assume each index is nonzero
				bGetsIndex = (i == 0) 						// topmost row
							 || (j == 0)					// leftmost column
							 || (IsBlackened(i - 1, j))		// cell above is marked
							 || (IsBlackened(i, j - 1));	// cell to left is marked
				}
#if TRUE
			// bug fix to handle Irish Times format (no singleton cells)
			// TODO: allow way to override this for a particular cell
			if (!bAllowSingletonCells && bGetsIndex) {
				if (((i == 0) || IsBlackened(i - 1, j))
					&& ((i == (m_nNumRows - 1)) || IsBlackened(i + 1, j))
					&& ((j > 0) && !IsBlackened(i, j - 1))) {
					bGetsIndex = FALSE;
					}			 
				if (((j == 0) || IsBlackened(i, j - 1))
					&& ((j == (m_nNumCols - 1)) || IsBlackened(i, j + 1))
					&& ((i > 0) && !IsBlackened(i - 1, j))) {
					bGetsIndex = FALSE;
					}			 
				}
#endif
			UINT uIndex = bGetsIndex ? uNextIndex++ : uNO_INDEX;
			m_puIndex[i].SetAt(j, uIndex);
            }
        }
#if TRUE
	//	Invalidate the crossgrid display (force cell updates)
	//	TODO: just invalidate those cells that change
	if (m_hWnd) {
		InvalidateRect(NULL);
		}
#endif
}


//	Get the logical coordinates for the specified cell.
//
CRect CGridView::GetCellRect (INT nRow, INT nCol)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CGridView::GetCellRect(%d, %d)", nRow, nCol));

    CRect rect (m_rectGrid.left + nCol * m_nCellWidth, -nRow * m_nCellHeight,
                m_rectGrid.left + (nCol + 1) * m_nCellWidth, -(nRow + 1) * m_nCellHeight);
	DUMP_RECT(TL_GARRULOUS, " => ", rect, "\n");

    return (rect);
}


//	Get the coordinates for the current cell.
//
CRect CGridView::GetCurrentCellRect (void)
{
	DEBUG_OUT(TL_GARRULOUS + 1, ("CGridView::GetCurrentCellRect()\n"));
    return (GetCellRect(m_nRow, m_nCol));
}


//	Add the area occupied by the current cell to the update region.
//
VOID CGridView::InvalidateCell (void)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::InvalidateCell()\n"));
    CClientDC dc(this);
    dc.SetMapMode(MM_LOENGLISH);
    CRect rect = GetCurrentCellRect();
    dc.LPtoDP(rect);
    InvalidateRect(rect);
#if defined(_DEBUG)
    if (IsControlPressed()) {
		if (IsAltPressed()) {
			InvalidateRect(NULL);
			}
        UpdateWindow();
        }
#endif
}

//	Set the current cell to the new coordinates, invalidating both the new
//	and old cell so the highlight is updated.
//
VOID CGridView::ChangeCell (INT nRow, INT nCol)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::ChangeCell(%d, %d)\n", nRow, nCol));
	ASSERT(nRow < m_nNumRows);
	ASSERT(nCol < m_nNumCols);
    InvalidateCell();
    m_nRow = nRow;
    m_nCol = nCol;
    InvalidateCell();
}

//	Get the new character entered in the (invisible) character edit box and
//	put it into the cell at the current position. Then advance the cell in
//	the current direction.
//
VOID CGridView::OnChangeCwordChar()
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnChangeCwordChar()\n"));
    UpdateData(bSAVE_AND_VALIDATE);
    if (m_sChar != "") {
        m_pcRow[m_nRow].SetAt(m_nCol, m_sChar[0]);
        m_sChar = "";
        UpdateData(bINITIALIZE);;
        InvalidateCell();
#if TRUE
		INT nNewRow = m_nRow;
		INT nNewCol = min(m_nCol + 1, m_nNumCols - 1);
		ChangeCell(nNewRow, nNewCol);
#endif
        }
}


// Do a shameless brute-force search for the pattern in an unabridged
// dictionary (unabr.dic).
//
VOID ShowBruteForce (CONST CString& sPattern)
{
	// Replace the question marks with grep dot's.
	CString sNewPattern = sPattern;
	for (INT i = 0; i < sNewPattern.GetLength(); i++) {
		if (sNewPattern[i] == '?') {
			sNewPattern.SetAt(i, '.');
			}
		}

	// Format the grep command ( and invoke via helper batch file)
	CONST CHAR szGREP_FMT[] = "grepdict.bat %s";
	CString sCommandLine;
	sCommandLine.Format(szGREP_FMT, sNewPattern);

	// Invoke the batch file
	UINT uResult = WinExec(sCommandLine, SW_MINIMIZE);
	SetActiveWindow(GetMainHwnd());
	if (uResult < (UINT)HINSTANCE_ERROR) {
	    AfxMessageBox("Unable to load grep program (for brute-force search)");
	    }
}


VOID CGridView::OnSuggestHorizontal()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnSuggestHorizontal()\n"));
    GetHorizontalPattern();
	if (IsControlPressed()) {
		ShowBruteForce(m_pLookup->pattern);
		}
    ShowSuggestions();
}


VOID CGridView::OnSuggestVertical()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnSuggestVertical()\n"));
    GetVerticalPattern();
	if (IsControlPressed()) {
		ShowBruteForce(m_pLookup->pattern);
		}
    ShowSuggestions();
}


VOID CGridView::GetHorizontalPattern (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::GetHorizontalPattern()\n"));
    // determine the extent of the current word pattern
    INT nPos;

	ASSERT(m_pLookup);    
    m_pLookup->pattern = "";
    for (nPos = m_nCol - 1; nPos >= 0; nPos--) {
        if (IsBlackened(m_nRow, nPos)) {
            break;
            }
        m_pLookup->pattern += m_pcRow[m_nRow][nPos];
        }
    m_pLookup->pattern.MakeReverse();
    for (nPos = m_nCol; nPos < m_nNumCols; nPos++) {
        if (IsBlackened(m_nRow, nPos)) {
            break;
            }
        m_pLookup->pattern += m_pcRow[m_nRow][nPos];
        }
}


VOID CGridView::GetVerticalPattern (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::GetVerticalPattern()\n"));
    // determine the extent of the current word pattern
    INT nPos;

	ASSERT(m_pLookup);    
    m_pLookup->pattern = "";
    for (nPos = m_nRow - 1; nPos >= 0; nPos--) {
        if (IsBlackened(nPos, m_nCol)) {
            break;
            }
        m_pLookup->pattern += m_pcRow[nPos][m_nCol];
        }
    m_pLookup->pattern.MakeReverse();
    for (nPos = m_nRow; nPos < m_nNumRows; nPos++) {
        if (IsBlackened(nPos, m_nCol)) {
            break;
            }
        m_pLookup->pattern += m_pcRow[nPos][m_nCol];
        }
}


VOID CGridView::StartLookup (VOID)
{
	AfxGetApp()->DoWaitCursor(1);		// start with MFC's wait cursor (overridden in timer proc)
	SetTimer(666, 333, NULL);
	SetDuringLookup(TRUE);
}


VOID CGridView::EndLookup (VOID)
{
	if (m_bDuringLookup) {
		KillTimer(666);
		AfxGetApp()->DoWaitCursor(-1);	// If -1, the wait cursor ends.
		SetDuringLookup(FALSE);
		}
}


VOID CGridView::ShowSuggestions (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::ShowSuggestions()\n"));
    // look up the word pattern
	ASSERT(m_pLookup);    
    m_pLookup->pattern.MakeUpper();
    UpdateData(bSAVE_AND_VALIDATE);
	StartLookup();
    m_pLookup->DeriveSuggestions(ClueText(m_sClue), ID_SUGGESTIONS_READY);
}


VOID CGridView::OnSuggestionsReady (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnSuggestionsReady()\n"));
	ASSERT(m_pLookup);    
	EndLookup();
    //x// m_suggestions.SetWindowText("Suggestions for " + m_pLookup->pattern);
    //x// m_suggestions.ShowWindow(SW_SHOW);
	//CString sSuggestions;
	m_sWords.Format("Suggestions for %s: %s", 
					(PCSZ)m_pLookup->pattern, (PCSZ)m_pLookup->suggestions);
#if FALSE
    m_edWords.SetWindowText(sSuggestions);
    m_edWords.ShowWindow(SW_SHOW);
#endif
    UpdateData(bINITIALIZE);;
}


VOID CGridView::OnHelpSynonyms (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpSynonyms()\n"));
    UpdateData(bSAVE_AND_VALIDATE);
#if FALSE
	ASSERT(m_pLookup);    
	StartLookup();
	//DoLookup(eSYNONYMS, ID_SYNONYMS_READY, m_sClue, 
    m_pLookup->GetSynonyms(m_sClue, ID_SYNONYMS_READY);
#endif
    UpdateData(bSAVE_AND_VALIDATE);

	ConnectToWordz();
	try {
		m_piWordz->LookupSyns(ClueText(m_sClue));
	}
	catch (...) {
		DEBUG_OUT(TL_BASIC, ("Exception looking up synonyms\n"));
	}
}


VOID CGridView::OnSynonymsReady (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpSynonyms()\n"));
	ASSERT(m_pLookup);    
	EndLookup();
#if FALSE // replaced by m_iWordz usage
    m_suggestions.SetWindowText("Synonyms for '" + m_sClue + "' ");
    m_suggestions.ShowWindow(SW_SHOW);
    m_edWords.SetWindowText(m_pLookup->buffer);
    m_edWords.ShowWindow(SW_SHOW);
#endif
}


VOID CGridView::OnHelpDefinition (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpDefinition()\n"));
#if FALSE
	ASSERT(m_pLookup);    
    UpdateData(bSAVE_AND_VALIDATE);
	StartLookup();
    m_pLookup->GetDefinition(m_sClue, ID_DEFINITION_READY);
#endif
    UpdateData(bSAVE_AND_VALIDATE);

	ConnectToWordz();
	try {
		m_piWordz->LookupDef(ClueText(m_sClue));
	}
	catch (...) {
		DEBUG_OUT(TL_BASIC, ("Exception looking up definition\n"));
	}
}


VOID CGridView::OnDefinitionReady (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnDefinitionReady()\n"));
	ASSERT(m_pLookup);    
	EndLookup();
    //m_def.SetWindowText(m_pLookup->buffer);
    //m_def.ShowWindow(SW_SHOW);
}


VOID CGridView::OnHelpReferences (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnHelpReferences()\n"));
    UpdateData(bSAVE_AND_VALIDATE);
#if FALSE
	ASSERT(m_pLookup);    
	StartLookup();
    m_pLookup->GetReferences(m_sClue, ID_REFERENCES_READY);
#endif
    UpdateData(bSAVE_AND_VALIDATE);

	ConnectToWordz();
	try {
		m_piWordz->LookupXrefs(ClueText(m_sClue));
	}
	catch (...) {
		DEBUG_OUT(TL_BASIC, ("Exception looking up cross references\n"));
	}
}


VOID CGridView::OnReferencesReady (void)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnReferencesReady()\n"));
	ASSERT(m_pLookup);    
    UpdateData(bSAVE_AND_VALIDATE);
	EndLookup();
#if FALSE // replaced by m_iWordz usage
    m_suggestions.SetWindowText("Cross References for '" + m_sClue + "' ");
    m_suggestions.ShowWindow(SW_SHOW);
    m_edWords.SetWindowText(m_pLookup->buffer);
    m_edWords.ShowWindow(SW_SHOW);
#endif
}


VOID CGridView::OnMiscChangefont()
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnMiscChangefont()\n"));
    CWnd* pwnd = GetFocus();
    if (!pwnd) {
        MessageBox("No window has the focus");
        return;
        }
    CEditx::InvokeChangeFont(pwnd);
}


VOID CGridView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnKeyDown(%x, %d, %x)\n",
			  nChar, nRepCnt, nFlags));
	INT nNewRow = m_nRow;
	INT nNewCol = m_nCol;

	switch (nChar) {
    	case VK_UP:
			nNewRow = max(0, m_nRow - 1);
			break;
        case VK_DOWN:
			nNewRow = min(m_nRow + 1, m_nNumRows - 1);
           	break;
       	case VK_LEFT:
			nNewCol = max(0, m_nCol - 1);
           	break;
       	case VK_RIGHT:
			nNewCol = min(m_nCol + 1, m_nNumCols - 1);
           	break;
       	}
	if ((nNewRow != m_nRow) || (nNewCol != m_nCol)) {
		ChangeCell(nNewRow, nNewCol);
		}
	
	CFormView::OnKeyDown(nChar, nRepCnt, nFlags);
}


VOID CGridView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnChar(%x, %d, %x)\n",
			  nChar, nRepCnt, nFlags));
	if (IsAphabetic(nChar)) {
		m_sChar = (CHAR)nChar;
		OnChangeCwordChar();
		}
	
	CFormView::OnChar(nChar, nRepCnt, nFlags);
}


HBRUSH CGridView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnCtlColor(%x, %x, %x)\n", 
			  pDC, pWnd, nCtlColor));
	//	Set the background for the edit box to gray (instead of white)
	if (nCtlColor == CTLCOLOR_EDIT) {
		pDC->SetTextColor(m_colorClueText);
		pDC->SetBkColor(m_colorClueBackground);
		HBRUSH hBrush = (HBRUSH)m_brushClueBackground.GetSafeHandle();
		return (hBrush);
		}
	
	return CFormView::OnCtlColor(pDC, pWnd, nCtlColor);
}


VOID CGridView::OnNotifyKeydown (NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnNotifyKeydown(%x, %x)\n", pNMHDR, pResult));
	NMHDREX* pNMHDREX = (NMHDREX*)pNMHDR;
	ASSERT(IsValidNMHDREX(pNMHDREX));
	OnKeyDown(pNMHDREX->wParam, 1, 0);
	*pResult = 1;						// standard MFC notification processed indicator
}


VOID CGridView::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);
	
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnSize(%x, %d, %d)\n", nType, cx, cy));
	if (nType != SIZE_MINIMIZED) {
		ResizeControls();
		}
}


VOID CGridView::OnTimer (UINT nIDEvent) 
{
	if (nIDEvent == 666) {
		::SetCursor(AfxGetApp()->LoadCursor(IDC_COFFEE));
		}
	
	CFormView::OnTimer(nIDEvent);
}

#if FALSE
VOID CGridView::DeriveSuggestions (CStringx& clue, UINT uCommandID /* = 0 */)
{
	DEBUG_OUT(TL_VERBOSE, ("CLookup::DeriveSuggestions('%s')\n", F(clue)));
    CStringx phrase = clue.Trim();

    CStringArray words;
    TokenizeText(words, phrase, ", ;.");
    suggestions = "";
    for (INT i = 0; i < words.GetSize(); i++) {
        CStringx word (words[i]);
        GetSynonyms(word, FALSE);
        suggestions += MatchPhrases(buffer, pattern, ';');
        GetDefinition(word, FALSE);
        suggestions += MatchPhrases(buffer, pattern, " ;");
        GetReferences(word, FALSE);
        suggestions += MatchPhrases(buffer, pattern, ';');
        }
}
#endif

void CGridView::OnUpdateAnyLookupCmd (CCmdUI* pCmdUI) 
{
	BOOL bEnable = !DuringLookup();
	pCmdUI->Enable(bEnable);
}

void CGridView::OnCancelLookup() 
{
	ASSERT(m_pLookup)
	ASSERT(DuringLookup());

	m_pLookup->AbortLookup();
	EndLookup();
}

void CGridView::OnUpdateCancelLookup(CCmdUI* pCmdUI) 
{
	BOOL bEnable = DuringLookup();
	pCmdUI->Enable(bEnable);
}

void CGridView::OnNotifyChange  (NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_VERBOSE, ("CGridView::OnNotifyChange(%x, %x)\n",
			  pNMHDR, pResult));
	// dont't allow the change (in protected field)
	*pResult = 1;						// standard MFC notification processed indicator
}


VOID CGridView::OnNotifyButtonDown (UINT nId, NMHDR* pNMHDR, LRESULT* pResult)
{
	DEBUG_OUT(TL_GARRULOUS, ("CGridView::OnNotifyButtonDown(%d, %x, %x)\n", 
			  nId, pNMHDR, pResult));
	NMHDREX* pNMHDREX = (NMHDREX*)pNMHDR;
	*pResult = 0;
	ASSERT(IsValidNMHDREX(pNMHDREX));
	CRtfEdit* pEdit = ToCRtfEdit(FromHandlePermanent(pNMHDR->hwndFrom));
	if (pEdit && (*pEdit != m_edWords) && (*pEdit != m_edClue)) {
		ASSERT(((nId >= IDC_CLUES_ACROSS_BASE) && (nId < IDC_CLUES_ACROSS_BASE + nMAX_COLUMNS))
			|| ((nId >= IDC_CLUES_DOWN_BASE) && (nId < IDC_CLUES_DOWN_BASE + nMAX_COLUMNS)));

		// The edit buffer must be one of the clue columns
		// So copy over the current line to the clue window
		INT nCurrent = pEdit->LineFromChar();
		CONST nLINE_MAX = 512;
		CStringx sLine;
		PSZ pszLine = sLine.GetBuffer(nLINE_MAX + 1);
		INT nNumBytes = pEdit->GetLine(nCurrent, pszLine, nLINE_MAX);
		ASSERT((nNumBytes >= 0) && (nNumBytes <= nLINE_MAX));
		pszLine[nNumBytes] = cNULL;
		sLine.ReleaseBuffer();
		DEBUG_OUT(TL_VERBOSE, ("Clue line %d (control %d): %s\n", nCurrent, nId, (PCSZ)sLine));

		// Skip over the number and update the clue buffer with the line
		INT nPos = 0;
		INT nLineNum = StringToInt(sLine, nPos);
		SkipSpaces(sLine, nPos);
		m_sClue = szCLUE;
		m_sClue += sLine.Mid(nPos);
		UpdateData(bINITIALIZE);
		*pResult = 1;						// standard MFC notification processed indicator
		}
}
