//	Copyleft 1993-1996 Tom O'Hara		CWord source
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// gridview.h : header file
//


// Declarations of embedded classes
//
class IWordz;


/////////////////////////////////////////////////////////////////////////////
// CGridView form view

CONST UINT uNO_INDEX = 0;
CONST INT nMAX_COLUMNS = 5;

//	Special characters for cell entries
//	TODO: use a blank char for an empty grip (revise pattern display)
CONST CHAR cBLACKENED = 	'*';
CONST CHAR cEMPTY = 		'?';

class CGridView : public CFormView
{
    DECLARE_DYNCREATE(CGridView)
protected:
    CGridView();            // protected constructor used by dynamic creation

public:
	typedef CArray<CRtfEdit*, CRtfEdit*> CRtfEditArray;

	// Form Data
public:
    //{{AFX_DATA(CGridView)
	enum { IDD = IDD_CWORD_PLUS };
    CRtfEdit m_edCluesAcross;
    CRtfEdit m_edCluesDown;
    CRtfEdit m_edWords;
    CRtfEdit m_edClue;
    CStringx m_sChar;
    CStringx m_sClue;
    CStringx m_sWords;
    CStringx m_sCluesAcross;
    CStringx m_sCluesDown;
    CEditx m_cwordChar;
	//}}AFX_DATA
    //CRtfEdit m_def;

// Attributes
public:
    const INT DEF_NUM_ROWS;				// default row dimension (15)
    const INT DEF_NUM_COLS;				// default col dimension (15)
	INT m_nNumRows;						// current row dimension
	INT m_nNumCols;						// current col dimension
    CByteArray* m_pcRow;				// character at each cell
	CUIntArray* m_puIndex;				// numeric index for cell (or 0)
    INT m_nCellHeight;					// cell height in logical units (MM)
    INT m_nCellWidth;					// cell width in logical units (MM)
    INT m_nCol;							// current column position (0-based)
    INT m_nRow;							// current row position (0-based)
	INT m_nGridLeft;					// x offset of grid from original (logical)
	INT m_nGridTop;						// y offset of grid from original (logical)
	CRect m_rectGrid;					// rectange position in logical units
    //CLookup lu;
	CLookup* m_pLookup;					// class for looking up clues
	BOOL m_bDuringLookup;				// if a lookup operation is in progress
	IWordz* m_piWordz;					// interface to CWordz automation
	// TODO: create a subclass for RTF column info
	CRtfEditArray m_aedCluesAcross;		// columns for Across clues
	CRtfEditArray m_aedCluesDown;		// columns for Down clues
	INT m_nCluesAcrossLen;				// length of text for Across clues
	INT m_nCluesDownLen;				// length of text for Down clues
	CStringArray m_asCluesAcross;		// list of Across clues
	CStringArray m_asCluesDown;			// list of Down clues
	INT m_nAcrossColumnWidth;			// logical width of colums for Across clue
	INT m_nDownColumnWidth;				// logical width of colums for Down clue
	COLORREF m_colorGridText;			// RGB color for crossword grid text
	COLORREF m_colorGridBorder;			// RGB color for crossword grid borders
	COLORREF m_colorGridBackground;		// RGB color for crossword grid background
	COLORREF m_colorActiveCell;			// RGB color for active crossword cell
	COLORREF m_colorIndexText;			// RGB color for crossword grid indices
	COLORREF m_colorBlackened;			// RGB color for "blackened" crossword cell
	COLORREF m_colorClueText;			// RGB color for clue text
	COLORREF m_colorClueBackground;		// RGB color for clue background text
	CBrush m_brushClueBackground;		// brush for clue text background
	CBrush m_brushOtherBackground;		// brush for other child control backgrounds
	INT m_nScaleFactor;					// factor by which to scale grid (eg, 125%)

	inline BOOL IsBlackened (INT nRow, INT nCol)
			{ return (m_pcRow[nRow][nCol] == cBLACKENED); }
	inline VOID SetBlackened (INT nRow, INT nCol)
			{ m_pcRow[nRow].SetAt(nCol, cBLACKENED); }
	inline BOOL IsEmpty  (INT nRow, INT nCol)
			{ return (m_pcRow[nRow][nCol] == cEMPTY); }
	inline VOID SetEmpty (INT nRow, INT nCol)
			{ m_pcRow[nRow].SetAt(nCol, cEMPTY); }
	inline VOID SetDuringLookup (BOOL bActive)	{ m_bDuringLookup = bActive; }
	inline BOOL DuringLookup (VOID)				{ return (m_bDuringLookup); }

// Operations
public:
	VOID GetOptions (VOID);
	VOID CreateRtfColumns (CRtfEditArray& aedRtfColumn, INT nResourceIdBase);
	VOID DeleteRtfColumns (CRtfEditArray& aedRtfColumn);
	VOID ColorRtfColumns (CRtfEditArray& aedRtfColumn);
	VOID FillColumnsWithClues (CRtfEditArray& aedRtfColumn, CStringArray& asCluesAcross,
							   INT nSectionTextLen, PCSZ pszSection);
	VOID MoveRtfColumns (CRtfEditArray& aedRtfColumn, CRect& rectClues, CDC* pDC);
    BOOL ConvertPoint (CPoint& point, INT& nRow, INT& nCol);
    CRect GetCellRect (INT nRow, INT nCol);
    CRect GetCurrentCellRect (void);
    void InvalidateCell (void);
    void ChangeCell (INT nRow, INT nCol);

    void GetHorizontalPattern (void);
    void GetVerticalPattern (void);
    void ShowSuggestions (void);
	void ReviseIndices (VOID);
	void DrawIndices (CDC* pDC);
	VOID SetWindowColors (VOID);
	void ResizeControls (VOID);
	VOID StartLookup (VOID);
	VOID EndLookup (VOID);
	VOID ConnectToWordz (VOID);
	VOID DisconnectWordz (VOID);
	VOID InitGrid (INT nNumRows, INT nNumCols);
	BOOL ReadClues (CStdioFile& fileDesc);
	BOOL ReadSectionHeader (CStdioFile& fileDesc, PCSZ pszHeader);
	BOOL ReadSectionClues (CStdioFile& fileDesc, PCSZ pszHeader, CStringArray& asClues, INT& nCluesLen);
	BOOL ReadGrid (CStdioFile& fileDesc);
	BOOL ReadState (CStdioFile& fileDesc);
	BOOL ReadPuzzle (CStdioFile& fileDesc);
	BOOL ReadPuzzle (CONST CString& sFile);
	BOOL WriteClues(CStdioFile& fileDesc, PCSZ pszHeader, CStringArray& asClues);
	BOOL WriteGrid (CStdioFile& fileDesc);
	BOOL WriteState (CStdioFile& fileDesc);
	BOOL WritePuzzle (CONST CString& sFile);

// Implementation
protected:
	virtual VOID CleanupGridView (VOID);
    virtual ~CGridView();
    virtual void OnDraw (CDC* pDC);
    virtual void OnInitialUpdate();
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
    // Generated message map functions
    //{{AFX_MSG(CGridView)
    afx_msg void OnChangeCwordChar();
    afx_msg void OnSuggestHorizontal();
    afx_msg void OnSuggestVertical();
    afx_msg void OnHelpSynonyms();
    afx_msg void OnHelpReferences();
    afx_msg void OnHelpDefinition();
    afx_msg void OnMiscChangefont();
    afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnCancelLookup();
	afx_msg void OnUpdateCancelLookup(CCmdUI* pCmdUI);
	//}}AFX_MSG
	afx_msg VOID OnNotifyChange (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg VOID OnNotifyDblClick (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg VOID OnNotifyKeydown (NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg VOID OnNotifyButtonDown (UINT nID, NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg VOID OnSuggestionsReady (void);
	afx_msg VOID OnSynonymsReady (void);
	afx_msg VOID OnDefinitionReady (void);
	afx_msg VOID OnReferencesReady (void);
	afx_msg void OnUpdateAnyLookupCmd(CCmdUI* pCmdUI);
    DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
