# Default to the debug build
#
!if !defined(DEBUG)
DEBUG=1
!endif

# Defaults for the component directories
#
# NOTE: All the components are assumed to be on the same drive
# 
!if !defined(COMMON_ROOT)
COMMON_ROOT=\cword\common
!endif
!if !defined(WORDNET_ROOT)
WORDNET_ROOT=\cword\wordnet
!endif
!if !defined(CWORD_ROOT)
CWORD_ROOT=\cword
!endif
!if !defined(MASTER_ROOT)
MASTER_ROOT=$(CWORD_ROOT)
!endif

# Defaults for the optional SDK directories (currently just MediaView)
#
# NOTE: These can be on other drives
#       To do builds from within the VC IDE, these must be added to the
#		the include and lib directories there (Tools/Options/Directories)
# 
!if !defined(MEDIAVIEW_ROOT)
MEDIAVIEW_ROOT=\Mv14kit
!endif
INCLUDE=$(INCLUDE);$(MEDIAVIEW_ROOT)\Mv\Inc
LIB=$(LIB);$(MEDIAVIEW_ROOT)\Mv\SLIB\WIN32

# Setup locations for the binaries and other build-specific settings
#
LIB_ROOT=$(CWORD_ROOT)\Lib
EXE_ROOT=$(CWORD_ROOT)\Bin
!if "$(DEBUG)"=="1"

OBJ_SUBDIR=WinDebug
NEW_OBJ_SUBDIR=Debug
LIB_DIR=$(LIB_ROOT)\Debug
CFG=Win32 Debug
EXE_DIR=$(EXE_ROOT)\Debug

!else

OBJ_SUBDIR=WinRel
NEW_OBJ_SUBDIR=Release
LIB_DIR=$(LIB_ROOT)\Release
CFG=Win32 Release
EXE_DIR=$(EXE_ROOT)\Release

!endif

COMMON_CFG = "Common - $(CFG)"


COMMON_OPTS =
!if "$(DEBUG_REL)"=="1"
COMMON_OPTS = $(COMMON_OPTS) _DEBUG_REL=1
!endif


# Projects to be built (note use of pseudotargets)
#
most:	_dirs _includes _common _commonxx _TrlLookup _MvLookup _WordNet _GnuGPL usual
all:    most usual others
usual:  _Lookup _wordz _cword 
others: _GridInfer _ReadDict

clean:
realclean:

what:
	@echo most:     _dirs _includes _common _commonxx _TrlLookup _MvLookup _WordNet _GnuGPL usual
	@echo all:      most usual others
	@echo usual:    _Lookup _wordz _cword
	@echo others: 	_GridInfer _ReadDict
	@echo common: 	_common
	@echo commonxx: _commonxx
	@echo TrlLookup: _TrlLookup
	@echo MvLookup: _MvLookup
	@echo wordnet: 	_wordnet
	@echo WnLookup: _wordnet
	@echo wordz: 	_wordz
	@echo cword: 	_cword
	@echo GridInfer: _GridInfer
	@echo includes: _includes

# Map project names to pseudotargets. This avoids conflicts
# between pseudotarget names and project directories, which
# might cause the pseudotarget not to be evaluated.
#
common: _common
commonxx: _commonxx
includes: _includes
Lookup: _Lookup
TrlLookup: _TrlLookup
MvLookup: _MvLookup
wordnet: _wordnet
WnLookup: _WordNet
wordz: _wordz
cword: _cword
GridInfer: _GridInfer
ReadDict: _ReadDict
GnuGPL: _GnuGPL

_dirs:
	-if not exist $(MASTER_ROOT)\Include mkdir $(MASTER_ROOT)\Include
	-if not exist $(LIB_ROOT) mkdir $(LIB_ROOT)
	-if not exist $(EXE_ROOT) mkdir $(EXE_ROOT)
	-if not exist $(LIB_DIR) mkdir $(LIB_DIR)
	-if not exist $(EXE_DIR) mkdir $(EXE_DIR)

_common:
	cd $(COMMON_ROOT)
	nmake -f common.mak /$(MAKEFLAGS) CFG="common - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\common.lib $(LIB_DIR)
	-if exist copy $(NEW_OBJ_SUBDIR)\vc40.pdb $(NEW_OBJ_SUBDIR)\vc40.pdb $(EXE_DIR)\common.pdb
	cd $(MASTER_ROOT)

_commonxx:
	cd $(COMMON_ROOT)
	nmake -f commonxx.mak /$(MAKEFLAGS) CFG="commonxx - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)xx\commonxx.lib $(LIB_DIR)
	-if exist $(NEW_OBJ_SUBDIR)xx\vc40.pdb copy $(NEW_OBJ_SUBDIR)xx\vc40.pdb $(EXE_DIR)\commonxx.pdb
	cd $(MASTER_ROOT)

_Lookup:
	cd $(CWORD_ROOT)\Lookup
	nmake -f Lookup.mak /$(MAKEFLAGS) CFG="Lookup - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\Lookup.lib $(LIB_DIR)
	-copy $(NEW_OBJ_SUBDIR)\Lookup.dll $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\Lookup.pdb copy $(NEW_OBJ_SUBDIR)\Lookup.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_TrlLookup:
	cd $(CWORD_ROOT)\TrlLookup
	nmake -f TrlLookup.mak /$(MAKEFLAGS) CFG="TrlLookup - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\TrlLookup.lib $(LIB_DIR)
	-copy $(NEW_OBJ_SUBDIR)\TrlLookup.dll $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\TrlLookup.pdb copy $(NEW_OBJ_SUBDIR)\TrlLookup.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_MvLookup:
	echo %INCLUDE%
	cd $(CWORD_ROOT)\MvLookup
	nmake -f MvLookup.mak /$(MAKEFLAGS) CFG="MvLookup - $(CFG)" MEDIAVIEW_ROOT=$(MEDIAVIEW_ROOT) $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\MvLookup.lib $(LIB_DIR)
	-copy $(NEW_OBJ_SUBDIR)\MvLookup.dll $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\MvLookup.pdb copy $(NEW_OBJ_SUBDIR)\MvLookup.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_GnuGPL:
	cd $(CWORD_ROOT)\GnuGPL
	nmake -f GnuGPL.mak /$(MAKEFLAGS) CFG="GnuGPL - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\GnuGPL.lib $(LIB_DIR)
	-copy $(NEW_OBJ_SUBDIR)\GnuGPL.dll $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\GnuGPL.pdb copy $(NEW_OBJ_SUBDIR)\GnuGPL.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_wordnet:
	cd $(WORDNET_ROOT)\src\lib
	nmake -f wnlib.mak /$(MAKEFLAGS) CFG="wnlib - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\WnLookup.lib $(LIB_DIR)
	-copy $(NEW_OBJ_SUBDIR)\WnLookup.dll $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\WnLookup.pdb copy $(NEW_OBJ_SUBDIR)\WnLookup.pdb $(EXE_DIR)\wnlib.pdb
	cd $(MASTER_ROOT)

_wordz:
	cd $(CWORD_ROOT)\wordz
	nmake -f wordz.mak /$(MAKEFLAGS) CFG="wordz - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\wordz.tlb $(LIB_DIR)
	-copy $(NEW_OBJ_SUBDIR)\wordz.exe $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\wordz.pdb copy $(NEW_OBJ_SUBDIR)\wordz.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_GridInfer:
	cd $(CWORD_ROOT)\GridInfer
	nmake -f GridInfer.mak /$(MAKEFLAGS) CFG="GridInfer - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\GridInfer.exe $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\GridInfer.pdb copy $(NEW_OBJ_SUBDIR)\GridInfer.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_cword:
	cd $(CWORD_ROOT)
	nmake -f cword.mak /$(MAKEFLAGS) CFG="cword - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\cword.exe $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\cword.pdb copy $(NEW_OBJ_SUBDIR)\cword.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)

_includes:
	-attrib -r $(CWORD_ROOT)\include\*.h
	-attrib -r $(CWORD_ROOT)\include\*.inl
	-copy $(CWORD_ROOT)\Lookup\Lookup*.h $(CWORD_ROOT)\include
	-copy $(CWORD_ROOT)\TrlLookup\Trl*.h $(CWORD_ROOT)\include
	-copy $(CWORD_ROOT)\MvLookup\Mv*.h $(CWORD_ROOT)\include
	-copy $(COMMON_ROOT)\*.h $(CWORD_ROOT)\include
	-copy $(COMMON_ROOT)\*.inl $(CWORD_ROOT)\include
	-attrib +r $(CWORD_ROOT)\include\*.h
	-attrib +r $(CWORD_ROOT)\include\*.inl

_test:
	cd $(CWORD_ROOT)\testmfc
	nmake -f testmfc.mak /$(MAKEFLAGS) CFG=$(CFG) $(COMMON_OPTS)
	-copy $(OBJ_SUBDIR)\testmfc.exe $(EXE_DIR)
	cd $(MASTER_ROOT)

_ReadDict:
	cd $(CWORD_ROOT)\ReadDict
	nmake -f ReadDict.mak /$(MAKEFLAGS) CFG="ReadDict - $(CFG)" $(COMMON_OPTS)
	-copy $(NEW_OBJ_SUBDIR)\ReadDict.exe $(EXE_DIR)
	-if exist $(NEW_OBJ_SUBDIR)\ReadDict.pdb copy $(NEW_OBJ_SUBDIR)\ReadDict.pdb $(EXE_DIR)
	cd $(MASTER_ROOT)
